/* Generated automatically by the program `genenums'
   from the machine description file.  */

#include "config.h"

#include "system.h"

#include "insn-constants.h"


const char *const unspec_strings[] = {
  "UNSPEC_GOT",
  "UNSPEC_GOTOFF",
  "UNSPEC_GOTPCREL",
  "UNSPEC_GOTTPOFF",
  "UNSPEC_TPOFF",
  "UNSPEC_NTPOFF",
  "UNSPEC_DTPOFF",
  "UNSPEC_GOTNTPOFF",
  "UNSPEC_INDNTPOFF",
  "UNSPEC_PLTOFF",
  "UNSPEC_MACHOPIC_OFFSET",
  "UNSPEC_PCREL",
  "UNSPEC_STACK_ALLOC",
  "UNSPEC_SET_GOT",
  "UNSPEC_SET_RIP",
  "UNSPEC_SET_GOT_OFFSET",
  "UNSPEC_MEMORY_BLOCKAGE",
  "UNSPEC_STACK_CHECK",
  "UNSPEC_TP",
  "UNSPEC_TLS_GD",
  "UNSPEC_TLS_LD_BASE",
  "UNSPEC_TLSDESC",
  "UNSPEC_TLS_IE_SUN",
  "UNSPEC_SCAS",
  "UNSPEC_FNSTSW",
  "UNSPEC_SAHF",
  "UNSPEC_PARITY",
  "UNSPEC_FSTCW",
  "UNSPEC_ADD_CARRY",
  "UNSPEC_FLDCW",
  "UNSPEC_REP",
  "UNSPEC_LD_MPIC",
  "UNSPEC_TRUNC_NOOP",
  "UNSPEC_DIV_ALREADY_SPLIT",
  "UNSPEC_MS_TO_SYSV_CALL",
  "UNSPEC_CALL_NEEDS_VZEROUPPER",
  "UNSPEC_PAUSE",
  "UNSPEC_LEA_ADDR",
  "UNSPEC_STOS",
  "UNSPEC_FIX_NOTRUNC",
  "UNSPEC_MASKMOV",
  "UNSPEC_MOVMSK",
  "UNSPEC_RCP",
  "UNSPEC_RSQRT",
  "UNSPEC_PSADBW",
  "UNSPEC_COPYSIGN",
  "UNSPEC_IEEE_MIN",
  "UNSPEC_IEEE_MAX",
  "UNSPEC_SIN",
  "UNSPEC_COS",
  "UNSPEC_FPATAN",
  "UNSPEC_FYL2X",
  "UNSPEC_FYL2XP1",
  "UNSPEC_FRNDINT",
  "UNSPEC_FIST",
  "UNSPEC_F2XM1",
  "UNSPEC_TAN",
  "UNSPEC_FXAM",
  "UNSPEC_FRNDINT_FLOOR",
  "UNSPEC_FRNDINT_CEIL",
  "UNSPEC_FRNDINT_TRUNC",
  "UNSPEC_FRNDINT_MASK_PM",
  "UNSPEC_FIST_FLOOR",
  "UNSPEC_FIST_CEIL",
  "UNSPEC_SINCOS_COS",
  "UNSPEC_SINCOS_SIN",
  "UNSPEC_XTRACT_FRACT",
  "UNSPEC_XTRACT_EXP",
  "UNSPEC_FSCALE_FRACT",
  "UNSPEC_FSCALE_EXP",
  "UNSPEC_FPREM_F",
  "UNSPEC_FPREM_U",
  "UNSPEC_FPREM1_F",
  "UNSPEC_FPREM1_U",
  "UNSPEC_C2_FLAG",
  "UNSPEC_FXAM_MEM",
  "UNSPEC_SP_SET",
  "UNSPEC_SP_TEST",
  "UNSPEC_SP_TLS_SET",
  "UNSPEC_SP_TLS_TEST",
  "UNSPEC_ROUND",
  "UNSPEC_CRC32",
  "UNSPEC_BEXTR",
  "UNSPEC_PDEP",
  "UNSPEC_PEXT",
  "UNSPEC_MOVNTQ",
  "UNSPEC_PFRCP",
  "UNSPEC_PFRCPIT1",
  "UNSPEC_PFRCPIT2",
  "UNSPEC_PFRSQRT",
  "UNSPEC_PFRSQIT1",
  "UNSPEC_MOVNT",
  "UNSPEC_LOADU",
  "UNSPEC_STOREU",
  "UNSPEC_LDDQU",
  "UNSPEC_PSHUFB",
  "UNSPEC_PSIGN",
  "UNSPEC_PALIGNR",
  "UNSPEC_EXTRQI",
  "UNSPEC_EXTRQ",
  "UNSPEC_INSERTQI",
  "UNSPEC_INSERTQ",
  "UNSPEC_BLENDV",
  "UNSPEC_INSERTPS",
  "UNSPEC_DP",
  "UNSPEC_MOVNTDQA",
  "UNSPEC_MPSADBW",
  "UNSPEC_PHMINPOSUW",
  "UNSPEC_PTEST",
  "UNSPEC_PCMPESTR",
  "UNSPEC_PCMPISTR",
  "UNSPEC_FMADDSUB",
  "UNSPEC_XOP_UNSIGNED_CMP",
  "UNSPEC_XOP_TRUEFALSE",
  "UNSPEC_XOP_PERMUTE",
  "UNSPEC_FRCZ",
  "UNSPEC_AESENC",
  "UNSPEC_AESENCLAST",
  "UNSPEC_AESDEC",
  "UNSPEC_AESDECLAST",
  "UNSPEC_AESIMC",
  "UNSPEC_AESKEYGENASSIST",
  "UNSPEC_PCLMUL",
  "UNSPEC_PCMP",
  "UNSPEC_VPERMIL",
  "UNSPEC_VPERMIL2",
  "UNSPEC_VPERMIL2F128",
  "UNSPEC_CAST",
  "UNSPEC_VTESTP",
  "UNSPEC_VCVTPH2PS",
  "UNSPEC_VCVTPS2PH",
  "UNSPEC_VPERMSI",
  "UNSPEC_VPERMDF",
  "UNSPEC_VPERMSF",
  "UNSPEC_VPERMTI",
  "UNSPEC_GATHER",
  "UNSPEC_VSIBADDR",
  "UNSPEC_LFENCE",
  "UNSPEC_SFENCE",
  "UNSPEC_MFENCE",
  "UNSPEC_MOVA",
  "UNSPEC_LDA",
  "UNSPEC_STA"
};

const char *const unspecv_strings[] = {
  "UNSPECV_BLOCKAGE",
  "UNSPECV_STACK_PROBE",
  "UNSPECV_PROBE_STACK_RANGE",
  "UNSPECV_ALIGN",
  "UNSPECV_PROLOGUE_USE",
  "UNSPECV_SPLIT_STACK_RETURN",
  "UNSPECV_CLD",
  "UNSPECV_NOPS",
  "UNSPECV_RDTSC",
  "UNSPECV_RDTSCP",
  "UNSPECV_RDPMC",
  "UNSPECV_LLWP_INTRINSIC",
  "UNSPECV_SLWP_INTRINSIC",
  "UNSPECV_LWPVAL_INTRINSIC",
  "UNSPECV_LWPINS_INTRINSIC",
  "UNSPECV_RDFSBASE",
  "UNSPECV_RDGSBASE",
  "UNSPECV_WRFSBASE",
  "UNSPECV_WRGSBASE",
  "UNSPECV_RDRAND",
  "UNSPECV_NLGR",
  "UNSPECV_EMMS",
  "UNSPECV_FEMMS",
  "UNSPECV_LDMXCSR",
  "UNSPECV_STMXCSR",
  "UNSPECV_CLFLUSH",
  "UNSPECV_MONITOR",
  "UNSPECV_MWAIT",
  "UNSPECV_VZEROALL",
  "UNSPECV_VZEROUPPER",
  "UNSPECV_CMPXCHG",
  "UNSPECV_XCHG",
  "UNSPECV_LOCK"
};
