/* Generated automatically. */
static const char configuration_arguments[] = "../configure --build=x86_64-pc-linux-gnu --disable-build-poststage1-with-cxx --disable-build-with-cxx --disable-cloog-version-check --disable-libitm --disable-libmudflap --disable-libquadmath --disable-libssp --disable-libstdcxx-pch --disable-libunwind-exceptions --disable-multilib --disable-ppl-version-check --enable-__cxa_atexit --enable-checking=release --enable-clocale=gnu --enable-cloog-backend=isl --enable-gnu-unique-object --enable-gold --enable-languages=c,c++ --enable-ld=default --enable-libstdcxx-time --enable-linker-build-id --enable-lto --enable-plugin --enable-shared --enable-threads=posix --enable-version-specific-runtime-libs --infodir=/usr/share/info --libdir=/usr/lib --libexecdir=/usr/lib --mandir=/usr/share/man --program-suffix=-4.7 --with-bugurl=https://bugs.archlinux.org/ --with-linker-hash-style=gnu --with-plugin-ld=ld.gold --with-ppl --with-system-zlib --prefix=/usr CXX='g++ -Wno-implicit-function-declaration -Wno-incompatible-pointer-types' CC='gcc -Wno-implicit-function-declaration -Wno-incompatible-pointer-types' MAKEINFO=:";
static const char thread_model[] = "posix";

static const struct {
  const char *name, *value;
} configure_default_options[] = { { "cpu", "generic" }, { "arch", "x86-64" } };
