/* Generated automatically by the program `genattrtab'
from the machine description file `md'.  */

#include "config.h"
#include "system.h"
#include "coretypes.h"
#include "tm.h"
#include "rtl.h"
#include "insn-attr.h"
#include "tm_p.h"
#include "insn-config.h"
#include "recog.h"
#include "regs.h"
#include "output.h"
#include "diagnostic-core.h"
#include "flags.h"
#include "function.h"

#define operands recog_data.operand

static int internal_dfa_insn_code_btver1 (rtx);
static int insn_default_latency_btver1 (rtx);
static int internal_dfa_insn_code_bdver2 (rtx);
static int insn_default_latency_bdver2 (rtx);
static int internal_dfa_insn_code_bdver1 (rtx);
static int insn_default_latency_bdver1 (rtx);
static int internal_dfa_insn_code_amdfam10 (rtx);
static int insn_default_latency_amdfam10 (rtx);
static int internal_dfa_insn_code_generic64 (rtx);
static int insn_default_latency_generic64 (rtx);
static int internal_dfa_insn_code_atom (rtx);
static int insn_default_latency_atom (rtx);
static int internal_dfa_insn_code_corei7 (rtx);
static int insn_default_latency_corei7 (rtx);
static int internal_dfa_insn_code_core2 (rtx);
static int insn_default_latency_core2 (rtx);
static int internal_dfa_insn_code_k8 (rtx);
static int insn_default_latency_k8 (rtx);
static int internal_dfa_insn_code_athlon (rtx);
static int insn_default_latency_athlon (rtx);
static int internal_dfa_insn_code_k6 (rtx);
static int insn_default_latency_k6 (rtx);
static int internal_dfa_insn_code_geode (rtx);
static int insn_default_latency_geode (rtx);
static int internal_dfa_insn_code_pentiumpro (rtx);
static int insn_default_latency_pentiumpro (rtx);
static int internal_dfa_insn_code_pentium (rtx);
static int insn_default_latency_pentium (rtx);
static int internal_dfa_insn_code_none (rtx);
static int insn_default_latency_none (rtx);

int (*internal_dfa_insn_code) (rtx);
int (*insn_default_latency) (rtx);

void
init_sched_attrs (void)
{
  if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_BTVER1)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_btver1;
      insn_default_latency
        = insn_default_latency_btver1;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_BDVER2)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_bdver2;
      insn_default_latency
        = insn_default_latency_bdver2;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_BDVER1)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_bdver1;
      insn_default_latency
        = insn_default_latency_bdver1;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_AMDFAM10)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_amdfam10;
      insn_default_latency
        = insn_default_latency_amdfam10;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_GENERIC64)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_generic64;
      insn_default_latency
        = insn_default_latency_generic64;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_atom;
      insn_default_latency
        = insn_default_latency_atom;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_COREI7)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_corei7;
      insn_default_latency
        = insn_default_latency_corei7;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_CORE2)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_core2;
      insn_default_latency
        = insn_default_latency_core2;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_K8)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_k8;
      insn_default_latency
        = insn_default_latency_k8;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_athlon;
      insn_default_latency
        = insn_default_latency_athlon;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_K6)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_k6;
      insn_default_latency
        = insn_default_latency_k6;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_GEODE)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_geode;
      insn_default_latency
        = insn_default_latency_geode;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_PENTIUMPRO)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_pentiumpro;
      insn_default_latency
        = insn_default_latency_pentiumpro;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_PENTIUM)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_pentium;
      insn_default_latency
        = insn_default_latency_pentium;
    }
  else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_NONE)))))
    {
      internal_dfa_insn_code
        = internal_dfa_insn_code_none;
      insn_default_latency
        = insn_default_latency_none;
    }
  else
    gcc_unreachable ();
}

int
insn_current_length (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 602:  /* jump */
      extract_insn_cached (insn);
      if ((((INSN_ADDRESSES_SET_P () ? INSN_ADDRESSES (INSN_UID (GET_CODE (operands[0]) == LABEL_REF ? XEXP (operands[0], 0) : operands[0])) : 0) - (insn_current_reference_address (insn))) >= (-126)) && (((INSN_ADDRESSES_SET_P () ? INSN_ADDRESSES (INSN_UID (GET_CODE (operands[0]) == LABEL_REF ? XEXP (operands[0], 0) : operands[0])) : 0) - (insn_current_reference_address (insn))) < (128)))
        {
	  return 2;
        }
      else
        {
	  return 5;
        }

    case 588:  /* *jcc_2 */
      extract_insn_cached (insn);
      if ((((INSN_ADDRESSES_SET_P () ? INSN_ADDRESSES (INSN_UID (GET_CODE (operands[0]) == LABEL_REF ? XEXP (operands[0], 0) : operands[0])) : 0) - (insn_current_reference_address (insn))) >= (-126)) && (((INSN_ADDRESSES_SET_P () ? INSN_ADDRESSES (INSN_UID (GET_CODE (operands[0]) == LABEL_REF ? XEXP (operands[0], 0) : operands[0])) : 0) - (insn_current_reference_address (insn))) < (128)))
        {
	  return 2;
        }
      else
        {
	  return 6;
        }

    case 587:  /* *jcc_1 */
      extract_insn_cached (insn);
      if ((((INSN_ADDRESSES_SET_P () ? INSN_ADDRESSES (INSN_UID (GET_CODE (operands[0]) == LABEL_REF ? XEXP (operands[0], 0) : operands[0])) : 0) - (insn_current_reference_address (insn))) >= (-126)) && (((INSN_ADDRESSES_SET_P () ? INSN_ADDRESSES (INSN_UID (GET_CODE (operands[0]) == LABEL_REF ? XEXP (operands[0], 0) : operands[0])) : 0) - (insn_current_reference_address (insn))) < (128)))
        {
	  return 2;
        }
      else
        {
	  return 6;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return 0;

    }
}

int
insn_variable_length_p (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      return 1;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return 0;

    }
}

int
insn_min_length (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_prefix cached_prefix ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x37)) || ((which_alternative == 0) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xbd)) || ((((1 << which_alternative) & 0xc)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xd))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) || ((which_alternative == 2) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1583:  /* *vec_dupv4si */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) || ((which_alternative == 0) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1582:  /* *vec_extractv2di_1 */
    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) || ((((1 << which_alternative) & 0x9)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 16 /* 0x10 */;
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xf)) || ((((1 << which_alternative) & 0x3)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x12)) || ((((1 << which_alternative) & 0x24)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1390:  /* vec_dupv2df */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x8a)) || ((!((1 << which_alternative) & 0xef)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 9)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((((1 << which_alternative) & 0x94)) || ((which_alternative == 0) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (((1 << which_alternative) & 0xa))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 4)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((which_alternative == 2) || ((which_alternative == 0) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x12)) || ((!((1 << which_alternative) & 0x1b)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 16 /* 0x10 */;
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 10)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((((1 << which_alternative) & 0xa0)) || ((((1 << which_alternative) & 0x7)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1352:  /* *vec_concatv4sf */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
    case 1345:  /* sse_loadlps */
    case 1343:  /* sse_loadhps */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xa)) || ((which_alternative == 4) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 4)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 16 /* 0x10 */;
        }
      else if ((((1 << which_alternative) & 0x3c0)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3f00)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 16 /* 0x10 */;
        }
      else if ((((1 << which_alternative) & 0x1c0)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x1f00)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 16 /* 0x10 */;
        }
      else
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 756:  /* *fop_sf_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 749:  /* *fop_df_1_i387 */
    case 748:  /* *fop_sf_1_i387 */
      if (get_attr_unit (insn) == UNIT_I387)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 744:  /* *fop_df_1_mixed */
    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (get_attr_unit (insn) == UNIT_I387)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 738:  /* *fop_df_comm_mixed */
    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1833:  /* pclmulqdq */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1727:  /* sse4_1_roundsd */
    case 1726:  /* sse4_1_roundss */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 1684:  /* sse4_1_packusdw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1674:  /* sse4_1_blendvpd */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1670:  /* sse4_1_blendpd */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1648:  /* ssse3_palignrti */
    case 1647:  /* avx2_palignrv2ti */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1556:  /* vec_interleave_lowv4si */
    case 1554:  /* vec_interleave_highv4si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1510:  /* *andnotv2di3 */
    case 1509:  /* *andnotv4di3 */
    case 1508:  /* *andnotv4si3 */
    case 1507:  /* *andnotv8si3 */
    case 1506:  /* *andnotv8hi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1503:  /* *andnotv32qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1446:  /* ashrv4si3 */
    case 1445:  /* ashrv8si3 */
    case 1444:  /* ashrv8hi3 */
    case 1443:  /* ashrv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1378:  /* vec_interleave_highv2di */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1346:  /* sse_movss */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1332:  /* vec_interleave_highv4sf */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 747:  /* *fop_df_1_sse */
    case 746:  /* *fop_sf_1_sse */
    case 740:  /* *fop_df_comm_sse */
    case 739:  /* *fop_sf_comm_sse */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 16 /* 0x10 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 16 /* 0x10 */;
        }
      else
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 16 /* 0x10 */;
        }
      else if (which_alternative != 1)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((which_alternative == 1) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((which_alternative == 2) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 16 /* 0x10 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xf)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 894:  /* *movxfcc_1 */
    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 761:  /* *fop_xf_comm_i387 */
    case 742:  /* *fop_df_comm_i387 */
    case 741:  /* *fop_sf_comm_i387 */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
      return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((((1 << which_alternative) & 0x31e0)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 16 /* 0x10 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((!((1 << which_alternative) & 0x7f)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 16 /* 0x10 */;
        }
      else
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 16 /* 0x10 */;
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 16 /* 0x10 */;
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3f)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 16 /* 0x10 */;
        }
      else if ((((1 << which_alternative) & 0x1e0)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xf800)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 16 /* 0x10 */;
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1935:  /* vcvtps2ph256 */
    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1932:  /* vcvtph2ps256 */
    case 1931:  /* *vcvtph2ps_load */
    case 1930:  /* vcvtph2ps */
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
    case 1923:  /* avx2_lshrvv2di */
    case 1922:  /* avx2_ashlvv2di */
    case 1921:  /* avx2_lshrvv4di */
    case 1920:  /* avx2_ashlvv4di */
    case 1919:  /* avx2_lshrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1917:  /* avx2_lshrvv8si */
    case 1916:  /* avx2_ashlvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1914:  /* avx2_ashrvv8si */
    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1890:  /* vec_set_hi_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1849:  /* avx2_vec_dupv4df */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1835:  /* avx_vzeroupper */
    case 1834:  /* *avx_vzeroall */
    case 1825:  /* xop_vpermil2v4df3 */
    case 1823:  /* xop_vpermil2v8sf3 */
    case 1804:  /* xop_frczv4df2 */
    case 1803:  /* xop_frczv8sf2 */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1683:  /* avx2_packusdw */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1601:  /* avx2_pmovmskb */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1436:  /* *avx2_pmaddwd */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 1339:  /* avx_shufps256_1 */
    case 1337:  /* avx_movsldup256 */
    case 1335:  /* avx_movshdup256 */
    case 1333:  /* avx_unpcklps256 */
    case 1331:  /* avx_unpckhps256 */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1326:  /* avx_cvtps2pd256 */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1313:  /* floatv4siv4df2 */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1298:  /* avx_cvtps2dq256 */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1181:  /* avx_addsubv4df3 */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 59:  /* *movoi_internal_avx */
      return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 16 /* 0x10 */;
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1731:  /* sse4_2_pcmpestr_cconly */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1725:  /* sse4_1_roundpd */
    case 1724:  /* avx_roundpd256 */
    case 1723:  /* sse4_1_roundps */
    case 1722:  /* avx_roundps256 */
    case 1721:  /* sse4_1_ptest */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1691:  /* sse4_1_phminposuw */
    case 1680:  /* sse4_1_movntdqa */
    case 1679:  /* avx2_movntdqa */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1606:  /* sse_stmxcsr */
    case 1605:  /* sse_ldmxcsr */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1602:  /* sse2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1599:  /* avx_movmskpd256 */
    case 1598:  /* sse_movmskps */
    case 1597:  /* avx_movmskps256 */
    case 1575:  /* sse2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1571:  /* sse2_pshufd_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
    case 1338:  /* sse3_movsldup */
    case 1336:  /* sse3_movshdup */
    case 1328:  /* sse2_cvtps2pd */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1315:  /* sse2_cvtdq2pd */
    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1297:  /* floatv4siv4sf2 */
    case 1296:  /* floatv8siv8sf2 */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1150:  /* avx_rsqrtv8sf2 */
    case 1147:  /* sse2_sqrtv2df2 */
    case 1146:  /* avx_sqrtv4df2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1144:  /* avx_sqrtv8sf2 */
    case 1142:  /* sse_rcpv4sf2 */
    case 1141:  /* avx_rcpv8sf2 */
    case 1112:  /* sse2_movntv2di */
    case 1111:  /* avx_movntv4di */
    case 1110:  /* sse2_movntv2df */
    case 1109:  /* avx_movntv4df */
    case 1108:  /* sse_movntv4sf */
    case 1107:  /* avx_movntv8sf */
    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
    case 1102:  /* sse2_storedqu */
    case 1101:  /* avx_storedqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1099:  /* avx_loaddqu256 */
    case 1098:  /* sse2_storeupd */
    case 1097:  /* avx_storeupd256 */
    case 1096:  /* sse_storeups */
    case 1095:  /* avx_storeups256 */
    case 1094:  /* sse2_loadupd */
    case 1093:  /* avx_loadupd256 */
    case 1092:  /* sse_loadups */
    case 1091:  /* avx_loadups256 */
    case 1089:  /* sse2_movq128 */
    case 1088:  /* *movv2df_internal */
    case 1087:  /* *movv4df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1085:  /* *movv8sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1083:  /* *movv2ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1081:  /* *movv4di_internal */
    case 1080:  /* *movv4si_internal */
    case 1079:  /* *movv8si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1075:  /* *movv32qi_internal */
    case 852:  /* movmsk_df */
    case 810:  /* sse4_1_rounddf2 */
    case 809:  /* sse4_1_roundsf2 */
    case 780:  /* *sqrtdf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 745:  /* *rcpsf2_sse */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
    case 141:  /* *truncdfsf_fast_sse */
    case 136:  /* *extendsfdf2_sse */
    case 105:  /* *movtf_internal_sse2 */
    case 61:  /* *movti_internal_sse */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      return 128 /* 0x80 */;

    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 640:  /* simple_return_pop_internal */
      return 3;

    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
      extract_constrain_insn_cached (insn);
      return 
#line 18133 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn) + 9);

    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
      return 5;

    case 853:  /* cld */
    case 642:  /* nop */
    case 638:  /* simple_return_internal */
    case 33:  /* x86_sahf_1 */
      return 1;

    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 714:  /* bswaphi_lowpart */
      return 4;

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
      return 7;

    case 731:  /* *tls_dynamic_gnu2_lea_32 */
      return 6;

    case 721:  /* *tls_local_dynamic_base_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
      return 12 /* 0xc */;

    case 720:  /* *tls_local_dynamic_base_32_gnu */
      return 11 /* 0xb */;

    case 719:  /* *tls_global_dynamic_64 */
      extract_constrain_insn_cached (insn);
      return 
#line 12636 "../../gcc/config/i386/i386.md"
(TARGET_X32 ? 15 : 16);

    case 713:  /* *bswaphi_lowpart_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 4;
        }

    case 653:  /* split_stack_return */
      extract_insn_cached (insn);
      if (const0_operand (operands[0], SImode))
        {
	  return 1;
        }
      else
        {
	  return 3;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 654:  /* ffssi2_no_cmove */
    case 650:  /* eh_return_internal */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
    case 644:  /* pad */
      return 16 /* 0x10 */;

    case 643:  /* nops */
      extract_constrain_insn_cached (insn);
      return 
#line 11779 "../../gcc/config/i386/i386.md"
(INTVAL (operands[0]));

    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
      return 0;

    case 958:  /* *pause */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 918:  /* trap */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 717:  /* *parityhi2_cmp */
    case 639:  /* simple_return_internal_long */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      return 2;

    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 32:  /* x86_fnstsw_1 */
      extract_constrain_insn_cached (insn);
      return 
#line 4701 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn) + 2);

    default:
      return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);

    }
}

int
insn_default_length (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_prefix cached_prefix ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x37)) || ((which_alternative == 0) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xbd)) || ((((1 << which_alternative) & 0xc)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xd))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) || ((which_alternative == 2) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1583:  /* *vec_dupv4si */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) || ((which_alternative == 0) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1582:  /* *vec_extractv2di_1 */
    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) || ((((1 << which_alternative) & 0x9)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 16 /* 0x10 */;
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xf)) || ((((1 << which_alternative) & 0x3)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x12)) || ((((1 << which_alternative) & 0x24)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1390:  /* vec_dupv2df */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x8a)) || ((!((1 << which_alternative) & 0xef)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 9)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((((1 << which_alternative) & 0x94)) || ((which_alternative == 0) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (((1 << which_alternative) & 0xa))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 4)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((which_alternative == 2) || ((which_alternative == 0) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x12)) || ((!((1 << which_alternative) & 0x1b)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 16 /* 0x10 */;
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 10)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((((1 << which_alternative) & 0xa0)) || ((((1 << which_alternative) & 0x7)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1352:  /* *vec_concatv4sf */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
    case 1345:  /* sse_loadlps */
    case 1343:  /* sse_loadhps */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xa)) || ((which_alternative == 4) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 4)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 16 /* 0x10 */;
        }
      else if ((((1 << which_alternative) & 0x3c0)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3f00)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 16 /* 0x10 */;
        }
      else if ((((1 << which_alternative) & 0x1c0)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x1f00)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 16 /* 0x10 */;
        }
      else
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 756:  /* *fop_sf_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 749:  /* *fop_df_1_i387 */
    case 748:  /* *fop_sf_1_i387 */
      if (get_attr_unit (insn) == UNIT_I387)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 744:  /* *fop_df_1_mixed */
    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (get_attr_unit (insn) == UNIT_I387)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 738:  /* *fop_df_comm_mixed */
    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1833:  /* pclmulqdq */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1727:  /* sse4_1_roundsd */
    case 1726:  /* sse4_1_roundss */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 1684:  /* sse4_1_packusdw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1674:  /* sse4_1_blendvpd */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1670:  /* sse4_1_blendpd */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1648:  /* ssse3_palignrti */
    case 1647:  /* avx2_palignrv2ti */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1556:  /* vec_interleave_lowv4si */
    case 1554:  /* vec_interleave_highv4si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1510:  /* *andnotv2di3 */
    case 1509:  /* *andnotv4di3 */
    case 1508:  /* *andnotv4si3 */
    case 1507:  /* *andnotv8si3 */
    case 1506:  /* *andnotv8hi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1503:  /* *andnotv32qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1446:  /* ashrv4si3 */
    case 1445:  /* ashrv8si3 */
    case 1444:  /* ashrv8hi3 */
    case 1443:  /* ashrv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1378:  /* vec_interleave_highv2di */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1346:  /* sse_movss */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1332:  /* vec_interleave_highv4sf */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 747:  /* *fop_df_1_sse */
    case 746:  /* *fop_sf_1_sse */
    case 740:  /* *fop_df_comm_sse */
    case 739:  /* *fop_sf_comm_sse */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 16 /* 0x10 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 16 /* 0x10 */;
        }
      else
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 16 /* 0x10 */;
        }
      else if (which_alternative != 1)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((which_alternative == 1) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((which_alternative == 2) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 16 /* 0x10 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xf)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 894:  /* *movxfcc_1 */
    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 761:  /* *fop_xf_comm_i387 */
    case 742:  /* *fop_df_comm_i387 */
    case 741:  /* *fop_sf_comm_i387 */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
      return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((((1 << which_alternative) & 0x31e0)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 16 /* 0x10 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }
      else if ((!((1 << which_alternative) & 0x7f)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 16 /* 0x10 */;
        }
      else
        {
	  return 2 + get_attr_prefix_data16 (insn) + get_attr_length_address (insn);
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 16 /* 0x10 */;
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 16 /* 0x10 */;
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3f)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 16 /* 0x10 */;
        }
      else if ((((1 << which_alternative) & 0x1e0)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xf800)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 16 /* 0x10 */;
        }
      else if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1935:  /* vcvtps2ph256 */
    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1932:  /* vcvtph2ps256 */
    case 1931:  /* *vcvtph2ps_load */
    case 1930:  /* vcvtph2ps */
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
    case 1923:  /* avx2_lshrvv2di */
    case 1922:  /* avx2_ashlvv2di */
    case 1921:  /* avx2_lshrvv4di */
    case 1920:  /* avx2_ashlvv4di */
    case 1919:  /* avx2_lshrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1917:  /* avx2_lshrvv8si */
    case 1916:  /* avx2_ashlvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1914:  /* avx2_ashrvv8si */
    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1890:  /* vec_set_hi_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1849:  /* avx2_vec_dupv4df */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1835:  /* avx_vzeroupper */
    case 1834:  /* *avx_vzeroall */
    case 1825:  /* xop_vpermil2v4df3 */
    case 1823:  /* xop_vpermil2v8sf3 */
    case 1804:  /* xop_frczv4df2 */
    case 1803:  /* xop_frczv8sf2 */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1683:  /* avx2_packusdw */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1601:  /* avx2_pmovmskb */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1436:  /* *avx2_pmaddwd */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 1339:  /* avx_shufps256_1 */
    case 1337:  /* avx_movsldup256 */
    case 1335:  /* avx_movshdup256 */
    case 1333:  /* avx_unpcklps256 */
    case 1331:  /* avx_unpckhps256 */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1326:  /* avx_cvtps2pd256 */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1313:  /* floatv4siv4df2 */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1298:  /* avx_cvtps2dq256 */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1181:  /* avx_addsubv4df3 */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 59:  /* *movoi_internal_avx */
      return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 16 /* 0x10 */;
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1731:  /* sse4_2_pcmpestr_cconly */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1725:  /* sse4_1_roundpd */
    case 1724:  /* avx_roundpd256 */
    case 1723:  /* sse4_1_roundps */
    case 1722:  /* avx_roundps256 */
    case 1721:  /* sse4_1_ptest */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1691:  /* sse4_1_phminposuw */
    case 1680:  /* sse4_1_movntdqa */
    case 1679:  /* avx2_movntdqa */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1606:  /* sse_stmxcsr */
    case 1605:  /* sse_ldmxcsr */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1602:  /* sse2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1599:  /* avx_movmskpd256 */
    case 1598:  /* sse_movmskps */
    case 1597:  /* avx_movmskps256 */
    case 1575:  /* sse2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1571:  /* sse2_pshufd_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
    case 1338:  /* sse3_movsldup */
    case 1336:  /* sse3_movshdup */
    case 1328:  /* sse2_cvtps2pd */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1315:  /* sse2_cvtdq2pd */
    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1297:  /* floatv4siv4sf2 */
    case 1296:  /* floatv8siv8sf2 */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1150:  /* avx_rsqrtv8sf2 */
    case 1147:  /* sse2_sqrtv2df2 */
    case 1146:  /* avx_sqrtv4df2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1144:  /* avx_sqrtv8sf2 */
    case 1142:  /* sse_rcpv4sf2 */
    case 1141:  /* avx_rcpv8sf2 */
    case 1112:  /* sse2_movntv2di */
    case 1111:  /* avx_movntv4di */
    case 1110:  /* sse2_movntv2df */
    case 1109:  /* avx_movntv4df */
    case 1108:  /* sse_movntv4sf */
    case 1107:  /* avx_movntv8sf */
    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
    case 1102:  /* sse2_storedqu */
    case 1101:  /* avx_storedqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1099:  /* avx_loaddqu256 */
    case 1098:  /* sse2_storeupd */
    case 1097:  /* avx_storeupd256 */
    case 1096:  /* sse_storeups */
    case 1095:  /* avx_storeups256 */
    case 1094:  /* sse2_loadupd */
    case 1093:  /* avx_loadupd256 */
    case 1092:  /* sse_loadups */
    case 1091:  /* avx_loadups256 */
    case 1089:  /* sse2_movq128 */
    case 1088:  /* *movv2df_internal */
    case 1087:  /* *movv4df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1085:  /* *movv8sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1083:  /* *movv2ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1081:  /* *movv4di_internal */
    case 1080:  /* *movv4si_internal */
    case 1079:  /* *movv8si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1075:  /* *movv32qi_internal */
    case 852:  /* movmsk_df */
    case 810:  /* sse4_1_rounddf2 */
    case 809:  /* sse4_1_roundsf2 */
    case 780:  /* *sqrtdf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 745:  /* *rcpsf2_sse */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
    case 141:  /* *truncdfsf_fast_sse */
    case 136:  /* *extendsfdf2_sse */
    case 105:  /* *movtf_internal_sse2 */
    case 61:  /* *movti_internal_sse */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return get_attr_length_vex (insn) + get_attr_length_immediate (insn) + get_attr_modrm (insn) + get_attr_length_address (insn);
        }
      else
        {
	  return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      return 128 /* 0x80 */;

    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 640:  /* simple_return_pop_internal */
      return 3;

    case 958:  /* *pause */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 918:  /* trap */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 717:  /* *parityhi2_cmp */
    case 639:  /* simple_return_internal_long */
      return 2;

    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
      extract_constrain_insn_cached (insn);
      return 
#line 18133 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn) + 9);

    case 853:  /* cld */
    case 642:  /* nop */
    case 638:  /* simple_return_internal */
    case 33:  /* x86_sahf_1 */
      return 1;

    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 714:  /* bswaphi_lowpart */
      return 4;

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
      return 7;

    case 721:  /* *tls_local_dynamic_base_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
      return 12 /* 0xc */;

    case 720:  /* *tls_local_dynamic_base_32_gnu */
      return 11 /* 0xb */;

    case 719:  /* *tls_global_dynamic_64 */
      extract_constrain_insn_cached (insn);
      return 
#line 12636 "../../gcc/config/i386/i386.md"
(TARGET_X32 ? 15 : 16);

    case 713:  /* *bswaphi_lowpart_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 4;
        }

    case 653:  /* split_stack_return */
      extract_insn_cached (insn);
      if (const0_operand (operands[0], SImode))
        {
	  return 1;
        }
      else
        {
	  return 3;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 654:  /* ffssi2_no_cmove */
    case 650:  /* eh_return_internal */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
    case 644:  /* pad */
      return 16 /* 0x10 */;

    case 643:  /* nops */
      extract_constrain_insn_cached (insn);
      return 
#line 11779 "../../gcc/config/i386/i386.md"
(INTVAL (operands[0]));

    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
      return 0;

    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 602:  /* jump */
      return 5;

    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      return 6;

    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 32:  /* x86_fnstsw_1 */
      extract_constrain_insn_cached (insn);
      return 
#line 4701 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn) + 2);

    default:
      return get_attr_modrm (insn) + get_attr_prefix_0f (insn) + get_attr_prefix_rex (insn) + get_attr_prefix_extra (insn) + 1 + get_attr_prefix_rep (insn) + get_attr_prefix_data16 (insn) + get_attr_length_immediate (insn) + get_attr_length_address (insn);

    }
}

int
bypass_p (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_use_carry cached_use_carry ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (which_alternative == 4))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (which_alternative == 2))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((which_alternative == 10) || ((which_alternative == 9) && (! (get_attr_memory (insn) == MEMORY_NONE)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((which_alternative == 6) || ((which_alternative == 5) && (! (get_attr_memory (insn) == MEMORY_NONE)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((which_alternative == 4) || ((which_alternative == 3) && (! (get_attr_memory (insn) == MEMORY_NONE)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((which_alternative == 9) || ((which_alternative == 10) && (! (get_attr_memory (insn) == MEMORY_NONE)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (which_alternative == 6))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1383:  /* sse2_storehpd */
    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((which_alternative == 5) || ((which_alternative == 4) && (! (get_attr_memory (insn) == MEMORY_NONE)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (((((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (((((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (!((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (which_alternative != 0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (! (memory_operand (operands[0], VOIDmode))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((which_alternative == 0) && (get_attr_memory (insn) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (((which_alternative == 0) && (((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))) || ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))) || (((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))) && (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || ((cached_type == TYPE_ISHIFT) && (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 493:  /* *ashlqi3_1_slp */
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || ((cached_type == TYPE_ISHIFT1) && (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 2)) || ((cached_type == TYPE_ISHIFT) && (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (((which_alternative != 0) || ((cached_type = get_attr_type (insn)) == TYPE_ALU)) || ((cached_type == TYPE_ISHIFT) && (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || (which_alternative == 1)) || (((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))) && (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 1)) || ((cached_type == TYPE_ISHIFT) && (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((which_alternative != 2) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))) || ((which_alternative == 2) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || ((which_alternative == 2) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((which_alternative != 3) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))) || ((which_alternative == 3) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || ((which_alternative == 3) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (which_alternative == 0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 273:  /* *adddi_5 */
    case 266:  /* *adddi_4 */
    case 264:  /* *adddi_3 */
    case 259:  /* *adddi_2 */
      extract_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((! (incdec_operand (operands[2], DImode))) || (get_attr_memory (insn) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 272:  /* *addsi_5 */
    case 269:  /* *addsi_4 */
    case 265:  /* *addsi_3_zext */
    case 263:  /* *addsi_3 */
    case 260:  /* *addsi_2_zext */
    case 258:  /* *addsi_2 */
      extract_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((! (incdec_operand (operands[2], SImode))) || (get_attr_memory (insn) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 271:  /* *addhi_5 */
    case 268:  /* *addhi_4 */
    case 262:  /* *addhi_3 */
    case 257:  /* *addhi_2 */
      extract_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((! (incdec_operand (operands[2], HImode))) || (get_attr_memory (insn) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 270:  /* *addqi_5 */
    case 267:  /* *addqi_4 */
    case 261:  /* *addqi_3 */
    case 256:  /* *addqi_2 */
      extract_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((! (incdec_operand (operands[2], QImode))) || (get_attr_memory (insn) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((! (incdec_operand (operands[1], QImode))) || (! (memory_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((which_alternative != 5) && (! (incdec_operand (operands[2], QImode)))) || (which_alternative == 5)) || (((which_alternative != 5) && (incdec_operand (operands[2], QImode))) && (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((which_alternative != 3) && (! (incdec_operand (operands[2], HImode)))) || (which_alternative == 3)) || (((which_alternative != 3) && (incdec_operand (operands[2], HImode))) && (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((which_alternative != 2) && (! (incdec_operand (operands[2], SImode)))) || (which_alternative == 2)) || (((which_alternative != 2) && (incdec_operand (operands[2], SImode))) && (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 251:  /* *adddi_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((which_alternative != 3) && (! (incdec_operand (operands[2], DImode)))) || (which_alternative == 3)) || (((which_alternative != 3) && (incdec_operand (operands[2], DImode))) && (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((which_alternative != 3) && (! (incdec_operand (operands[2], SImode)))) || (which_alternative == 3)) || (((which_alternative != 3) && (incdec_operand (operands[2], SImode))) && (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((which_alternative == 0) && (! (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (! (get_attr_memory (insn) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((1 << which_alternative) & 0x3)) && (! (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((which_alternative != 0) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))) || ((which_alternative == 0) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || ((which_alternative == 0) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((which_alternative == 1) || ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || ((which_alternative == 0) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || ((which_alternative == 0) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((1 << which_alternative) & 0x18)) || ((((1 << which_alternative) & 0x7)) && (! (get_attr_memory (insn) == MEMORY_NONE)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((1 << which_alternative) & 0x78)) || ((((1 << which_alternative) & 0x7)) && (! (get_attr_memory (insn) == MEMORY_NONE)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((1 << which_alternative) & 0x7)) && (! (get_attr_memory (insn) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_PENTIUM)))) && ((which_alternative == 1) && (! (memory_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      extract_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (((((cached_type = get_attr_type (insn)) == TYPE_IMOV) || ((cached_type == TYPE_IMOVX) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || ((cached_type == TYPE_IMOVX) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || ((cached_type == TYPE_IMOVX) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (((((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) || (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      extract_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) || (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((get_attr_type (insn) == TYPE_IMOV) || ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (!((1 << which_alternative) & 0xffc)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (!((1 << which_alternative) & 0x1fff0)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 649:  /* set_got_offset_rex64 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 379:  /* *andqi_2_slp */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 372:  /* *andqi_1_slp */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 328:  /* *mulsidi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 289:  /* *subqi_1_slp */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 276:  /* *addqi_ext_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      if (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 54:  /* *popdi1_epilogue */
    case 52:  /* *popdi1 */
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_PENTIUM)))) || (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 53:  /* *popsi1_epilogue */
    case 51:  /* *popsi1 */
      if (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_PENTIUM))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      extract_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_PENTIUM)))) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_PENTIUM)))) && ((which_alternative == 0) && (! (memory_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (get_attr_memory (insn) == MEMORY_NONE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && (((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_PENTIUM)))) && ((((cached_type = get_attr_type (insn)) == TYPE_PUSH) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || (((cached_type = get_attr_type (insn)) == TYPE_POP) || (cached_type == TYPE_LEAVE)))) || ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATOM)))) && ((((((((((((((((((((((((((cached_type = get_attr_type (insn)) == TYPE_ALU) && ((((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_use_carry = get_attr_use_carry (insn)) == USE_CARRY_0)) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((cached_use_carry = get_attr_use_carry (insn)) == USE_CARRY_0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_use_carry = get_attr_use_carry (insn)) == USE_CARRY_1))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((cached_use_carry = get_attr_use_carry (insn)) == USE_CARRY_1)))) || (((cached_type = get_attr_type (insn)) == TYPE_ALU1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || (((cached_type = get_attr_type (insn)) == TYPE_ALU1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))) || (((cached_type = get_attr_type (insn)) == TYPE_NEGNOT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))) || (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))) || (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))) || (((cached_type = get_attr_type (insn)) == TYPE_LEA) && (! ((cached_mode = get_attr_mode (insn)) == MODE_HI)))) || (((cached_type = get_attr_type (insn)) == TYPE_INCDEC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || (((cached_type = get_attr_type (insn)) == TYPE_ISHIFT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (get_attr_prefix_0f (insn) == 0)))) || (((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || (((cached_type = get_attr_type (insn)) == TYPE_ROTATE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || (((cached_type = get_attr_type (insn)) == TYPE_ROTATE1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || (((cached_type = get_attr_type (insn)) == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_mode = get_attr_mode (insn)) == MODE_SI)))) || (((cached_type = get_attr_type (insn)) == TYPE_IMUL) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((cached_mode = get_attr_mode (insn)) == MODE_SI)))) || (((cached_type = get_attr_type (insn)) == TYPE_ICMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || (((cached_type = get_attr_type (insn)) == TYPE_TEST) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || (((cached_type = get_attr_type (insn)) == TYPE_SETCC) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))) || (((cached_type = get_attr_type (insn)) == TYPE_ICMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || (((cached_type = get_attr_type (insn)) == TYPE_ICMOV) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))) || (((cached_type = get_attr_type (insn)) == TYPE_POP) && ((cached_mode = get_attr_mode (insn)) == MODE_DI))) || (((cached_type = get_attr_type (insn)) == TYPE_FMOV) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    default:
      return 0;

    }
}

int
insn_default_latency_none (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return 0;

    }
}

int
internal_dfa_insn_code_none (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return 618 /* 0x26a */;

    }
}

int
insn_default_latency_pentium (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_imm_disp cached_imm_disp ATTRIBUTE_UNUSED;
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_pent_pair cached_pent_pair ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 2;
        }
      else
        {
	  return 1;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 2;
        }
      else
        {
	  return 1;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xf))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if (which_alternative == 10)
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if (!((1 << which_alternative) & 0x3f))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((which_alternative == 3) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if (!((1 << which_alternative) & 0xf))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if (which_alternative == 9)
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3f))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1383:  /* sse2_storehpd */
    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode)))
        {
	  return 1;
        }
      else if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_imm_disp == IMM_DISP_TRUE) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_imm_disp == IMM_DISP_TRUE) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode)))
        {
	  return 1;
        }
      else if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_imm_disp == IMM_DISP_TRUE) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_imm_disp == IMM_DISP_TRUE) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 12 /* 0xc */;

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 70 /* 0x46 */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || (mult_operator (operands[3], XFmode)))
        {
	  return 3;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 39 /* 0x27 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || (mult_operator (operands[3], DFmode)))
        {
	  return 3;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 39 /* 0x27 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || (mult_operator (operands[3], SFmode)))
        {
	  return 3;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 39 /* 0x27 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 3;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 39 /* 0x27 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 3;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 39 /* 0x27 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 738:  /* *fop_df_comm_mixed */
    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) || ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1605:  /* sse_ldmxcsr */
    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 2;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      return 10 /* 0xa */;

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
      extract_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (const1_operand (operands[1], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) || (! (const1_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (const1_operand (operands[1], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_imm_disp == IMM_DISP_TRUE) || (! (const1_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (const1_operand (operands[1], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_imm_disp == IMM_DISP_TRUE) || (! (const1_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((which_alternative == 0) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) || ((which_alternative != 0) || (! (const1_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && ((which_alternative == 0) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 0) || (! (const1_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && ((which_alternative == 0) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 0) || (! (const1_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
      extract_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (const1_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) || (! (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (const1_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_imm_disp == IMM_DISP_TRUE) || (! (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (const1_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_imm_disp == IMM_DISP_TRUE) || (! (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
      extract_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (const_int_operand (operands[1], VOIDmode))) && (memory_operand (operands[1], VOIDmode))) || ((((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) || (! (const_int_operand (operands[1], VOIDmode)))) && (memory_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (const_int_operand (operands[1], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_imm_disp == IMM_DISP_TRUE) || (! (const_int_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((which_alternative == 0) && (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) || ((which_alternative != 0) || (! (const_int_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && ((which_alternative == 0) && (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 0) || (! (const_int_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && ((which_alternative == 0) && (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 0) || (! (const_int_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((which_alternative == 1) && (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) || ((which_alternative != 1) || (! (const_int_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && ((which_alternative == 1) && (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 1) || (! (const_int_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && ((which_alternative == 1) && (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 1) || (! (const_int_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 500:  /* *ashlhi3_cconly */
    case 495:  /* *ashlhi3_cmp */
      if ((((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_type = get_attr_type (insn)) == TYPE_ALU)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_type = get_attr_type (insn)) == TYPE_ALU)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_type = get_attr_type (insn)) == TYPE_ALU)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 2))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 2))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 2))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || (which_alternative == 1))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || (which_alternative == 1))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || (which_alternative == 1))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 1))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 1))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 1))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 481:  /* x86_64_shld */
      extract_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (const_int_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) || (! (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (const_int_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_imm_disp == IMM_DISP_TRUE) || (! (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (const_int_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_imm_disp == IMM_DISP_TRUE) || (! (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], VOIDmode))
        {
	  return 3;
        }
      else if (get_attr_memory (insn) == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x3))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) || (which_alternative == 2)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x3))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_imm_disp == IMM_DISP_TRUE) || (which_alternative == 2)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x3))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_imm_disp == IMM_DISP_TRUE) || (which_alternative == 2)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x7))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) || (which_alternative == 3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x7))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_imm_disp == IMM_DISP_TRUE) || (which_alternative == 3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x7))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_imm_disp == IMM_DISP_TRUE) || (which_alternative == 3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if (((which_alternative != 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 357:  /* *testqi_1_maybe_si */
      extract_constrain_insn_cached (insn);
      if (((((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if (((((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 11 /* 0xb */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 323:  /* *mulqi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
      return 11 /* 0xb */;

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (memory_operand (operands[1], VOIDmode))) || (((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) && (memory_operand (operands[1], VOIDmode))))
        {
	  return 2;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (! (memory_operand (operands[1], VOIDmode)))) || ((cached_imm_disp == IMM_DISP_TRUE) && (! (memory_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (which_alternative == 3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x7))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (which_alternative == 3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x7))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (which_alternative == 3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x7))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 254:  /* *addqi_1 */
    case 252:  /* addsi_1_zext */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      if ((((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (((cached_pent_pair == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if (((cached_pent_pair == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || (((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 3;
        }
      else if (immediate_operand (operands[1], VOIDmode))
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 761:  /* *fop_xf_comm_i387 */
    case 742:  /* *fop_df_comm_i387 */
    case 741:  /* *fop_sf_comm_i387 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
      return 3;

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 1;
        }
      else if ((immediate_operand (operands[1], VOIDmode)) || (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((which_alternative == 0) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 3;
        }
      else if ((immediate_operand (operands[1], VOIDmode)) || (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)) && (which_alternative == 1))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((immediate_operand (operands[1], VOIDmode)) || (cached_memory == MEMORY_STORE)))
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || (((which_alternative != 1) || ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((which_alternative != 1) || ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((which_alternative != 1) || ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if (((1 << which_alternative) & 0x78))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      if ((cached_type = get_attr_type (insn)) == TYPE_IMOV)
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2)))))))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 65:  /* *movhi_internal */
      if ((cached_type = get_attr_type (insn)) == TYPE_IMOV)
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (!((1 << which_alternative) & 0xffc))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) || (((1 << which_alternative) & 0xffc))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (!((1 << which_alternative) & 0xffc))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_imm_disp == IMM_DISP_TRUE) || (((1 << which_alternative) & 0xffc))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (!((1 << which_alternative) & 0xffc))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_imm_disp == IMM_DISP_TRUE) || (((1 << which_alternative) & 0xffc))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (!((1 << which_alternative) & 0x1fff0))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) || (((1 << which_alternative) & 0x1fff0))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (!((1 << which_alternative) & 0x1fff0))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_imm_disp == IMM_DISP_TRUE) || (((1 << which_alternative) & 0x1fff0))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((! (cached_imm_disp == IMM_DISP_TRUE)) && (!((1 << which_alternative) & 0x1fff0))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_imm_disp == IMM_DISP_TRUE) || (((1 << which_alternative) & 0x1fff0))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (memory_operand (operands[1], VOIDmode))) || (((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) && (memory_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1835:  /* avx_vzeroupper */
    case 1834:  /* *avx_vzeroall */
    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 920:  /* *prefetch_3dnow */
    case 919:  /* *prefetch_sse */
    case 894:  /* *movxfcc_1 */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 649:  /* set_got_offset_rex64 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 1;

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 3;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((which_alternative == 0) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 361:  /* *testqi_ext_0 */
    case 39:  /* *cmpfp_iu_387 */
    case 38:  /* *cmpfp_iu_sse */
    case 37:  /* *cmpfp_iu_mixed */
    case 36:  /* *cmpfp_i_i387 */
    case 35:  /* *cmpfp_i_sse */
    case 34:  /* *cmpfp_i_mixed */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 2010:  /* mfence_sse2 */
    case 2009:  /* *sse_sfence */
    case 2008:  /* *sse2_lfence */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1607:  /* sse2_clflush */
    case 1606:  /* sse_stmxcsr */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return 0;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_IMUL)
        {
	  return 11 /* 0xb */;
        }
      else if (cached_type == TYPE_STR)
        {
	  return 12 /* 0xc */;
        }
      else if ((cached_type == TYPE_IDIV) || ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)) && (get_attr_mode (insn) == MODE_XF)))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_FMOV) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IMOV) || (((cached_type == TYPE_PUSH) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((cached_type == TYPE_POP) || (cached_type == TYPE_LEAVE))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_IBR)
        {
	  return 1;
        }
      else if ((cached_type == TYPE_FOP) || ((cached_type == TYPE_FISTP) || (cached_type == TYPE_FMUL)))
        {
	  return 3;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 39 /* 0x27 */;
        }
      else if (cached_type == TYPE_FPSPC)
        {
	  return 70 /* 0x46 */;
        }
      else if ((((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || ((((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)) || (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))))
        {
	  return 3;
        }
      else if (((cached_pent_pair == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_pent_pair == PENT_PAIR_PV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))))
        {
	  return 2;
        }
      else if (((cached_pent_pair == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || (((cached_pent_pair == PENT_PAIR_PV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    default:
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    }
}

int
internal_dfa_insn_code_pentium (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_imm_disp cached_imm_disp ATTRIBUTE_UNUSED;
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_pent_pair cached_pent_pair ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 22 /* 0x16 */;
        }
      else
        {
	  return 26 /* 0x1a */;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 22 /* 0x16 */;
        }
      else
        {
	  return 26 /* 0x1a */;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xf))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if ((which_alternative == 9) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if (which_alternative == 10)
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if ((which_alternative == 5) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if (!((1 << which_alternative) & 0x3f))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if ((which_alternative == 3) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if (!((1 << which_alternative) & 0xf))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if ((which_alternative == 10) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if (which_alternative == 9)
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3f))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1383:  /* sse2_storehpd */
    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if ((which_alternative == 4) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 967:  /* *movv2sf_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode)))
        {
	  return 6;
        }
      else if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode)))
        {
	  return 6;
        }
      else if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1835:  /* avx_vzeroupper */
    case 1834:  /* *avx_vzeroall */
    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 920:  /* *prefetch_3dnow */
    case 919:  /* *prefetch_sse */
    case 894:  /* *movxfcc_1 */
      return 26 /* 0x1a */;

    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
      return 24 /* 0x18 */;

    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 1;

    case 1605:  /* sse_ldmxcsr */
    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
      return 22 /* 0x16 */;

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 14 /* 0xe */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 11 /* 0xb */;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 12 /* 0xc */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if (! (mult_operator (operands[3], XFmode)))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 12 /* 0xc */;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 11 /* 0xb */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 12 /* 0xc */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 11 /* 0xb */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 12 /* 0xc */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 11 /* 0xb */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 12 /* 0xc */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 11 /* 0xb */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 12 /* 0xc */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if (! (mult_operator (operands[3], DFmode)))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 12 /* 0xc */;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if (! (mult_operator (operands[3], SFmode)))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 12 /* 0xc */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 11 /* 0xb */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 12 /* 0xc */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 11 /* 0xb */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 12 /* 0xc */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 19 /* 0x13 */;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      return 9;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      return 10 /* 0xa */;

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
      extract_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (const1_operand (operands[1], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (! (const1_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (const1_operand (operands[1], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (! (const1_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (const1_operand (operands[1], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (! (const1_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((which_alternative == 0) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 0) || (! (const1_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((which_alternative == 0) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 0) || (! (const1_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((which_alternative == 0) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 0) || (! (const1_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
      extract_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (const1_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (! (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (const1_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (! (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (const1_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (! (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
      extract_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (const_int_operand (operands[1], VOIDmode))) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 16 /* 0x10 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (! (const_int_operand (operands[1], VOIDmode)))) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (const_int_operand (operands[1], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (! (const_int_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((which_alternative == 0) && (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 0) || (! (const_int_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((which_alternative == 0) && (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 0) || (! (const_int_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((which_alternative == 0) && (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 0) || (! (const_int_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((which_alternative == 1) && (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 1) || (! (const_int_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((which_alternative == 1) && (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 1) || (! (const_int_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((which_alternative == 1) && (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || ((which_alternative != 1) || (! (const_int_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 500:  /* *ashlhi3_cconly */
    case 495:  /* *ashlhi3_cmp */
      if (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_type = get_attr_type (insn)) == TYPE_ALU)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_type = get_attr_type (insn)) == TYPE_ALU)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_type = get_attr_type (insn)) == TYPE_ALU)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 2))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 2))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 2))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || (which_alternative == 1))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || (which_alternative == 1))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || (which_alternative == 1))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 1))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 1))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 1))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 481:  /* x86_64_shld */
      extract_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (const_int_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (! (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (const_int_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (! (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (const_int_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (! (const_int_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], VOIDmode))
        {
	  return 18 /* 0x12 */;
        }
      else if (get_attr_memory (insn) == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 370:  /* *andhi_1 */
      extract_constrain_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x3))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (which_alternative == 2)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x3))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (which_alternative == 2)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x3))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (which_alternative == 2)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x3))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (which_alternative == 2)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x3))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (which_alternative == 2)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x3))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (which_alternative == 2)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x7))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (which_alternative == 3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x7))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (which_alternative == 3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x7))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (which_alternative == 3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if ((which_alternative != 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 357:  /* *testqi_1_maybe_si */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if ((((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      return 2;

    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 323:  /* *mulqi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
      return 0;

    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 16 /* 0x10 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 20 /* 0x14 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 24 /* 0x18 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 376:  /* *andhi_2 */
    case 315:  /* *addhi3_cc_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 296:  /* *subhi_3 */
    case 291:  /* *subhi_2 */
    case 285:  /* *subhi_1 */
    case 271:  /* *addhi_5 */
    case 268:  /* *addhi_4 */
    case 262:  /* *addhi_3 */
    case 257:  /* *addhi_2 */
      if ((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if ((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if ((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if ((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 22 /* 0x16 */;
        }
      else if ((! (cached_imm_disp == IMM_DISP_TRUE)) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (which_alternative == 3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x7))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (which_alternative == 3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x7))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (which_alternative == 3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (((1 << which_alternative) & 0x7))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 254:  /* *addqi_1 */
    case 252:  /* addsi_1_zext */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      if (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
      if ((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if ((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if ((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
      return 23 /* 0x17 */;

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 4;
        }
      else if (immediate_operand (operands[1], VOIDmode))
        {
	  return 5;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((which_alternative == 0) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
      return 11 /* 0xb */;

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((immediate_operand (operands[1], VOIDmode)) || (cached_memory == MEMORY_STORE))
        {
	  return 5;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else if ((immediate_operand (operands[1], VOIDmode)) || (cached_memory == MEMORY_STORE))
        {
	  return 5;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)) && (which_alternative == 1))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((immediate_operand (operands[1], VOIDmode)) || (cached_memory == MEMORY_STORE)))
        {
	  return 5;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((which_alternative != 1) || ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if (((which_alternative != 1) || ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if (((which_alternative != 1) || ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if (((which_alternative != 1) || ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if (((which_alternative != 1) || ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((which_alternative == 1) && (! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if (((which_alternative != 1) || ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0x78))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return 7;
        }
      else if ((which_alternative == 1) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      if ((cached_type = get_attr_type (insn)) == TYPE_IMOV)
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))
        {
	  return 6;
        }
      else if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))) && (cached_memory == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))) && (cached_memory == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 475:  /* *one_cmplhi2_2 */
    case 120:  /* *zero_extendqihi2_and */
    case 84:  /* *movstricthi_xor */
      extract_insn_cached (insn);
      if ((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 16 /* 0x10 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 18 /* 0x12 */;
        }
      else if ((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 6;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2)))))))
        {
	  return 6;
        }
      else if (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_UV) && (cached_memory == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && (cached_memory == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_UV) && (cached_memory == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && (cached_memory == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 65:  /* *movhi_internal */
      if ((cached_type = get_attr_type (insn)) == TYPE_IMOV)
        {
	  return 6;
        }
      else if (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_UV) && (cached_memory == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && (cached_memory == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_UV) && (cached_memory == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && (cached_memory == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode))))
        {
	  return 6;
        }
      else if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (!((1 << which_alternative) & 0xffc))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (((1 << which_alternative) & 0xffc))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (!((1 << which_alternative) & 0xffc))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (((1 << which_alternative) & 0xffc))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (!((1 << which_alternative) & 0xffc))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (((1 << which_alternative) & 0xffc))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode))))
        {
	  return 6;
        }
      else if (((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (!((1 << which_alternative) & 0x1fff0))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (((1 << which_alternative) & 0x1fff0))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (!((1 << which_alternative) & 0x1fff0))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (((1 << which_alternative) & 0x1fff0))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (((! (cached_imm_disp == IMM_DISP_TRUE)) && (!((1 << which_alternative) & 0x1fff0))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if (((cached_imm_disp == IMM_DISP_TRUE) || (((1 << which_alternative) & 0x1fff0))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 15 /* 0xf */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 18 /* 0x12 */;
        }
      else if ((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 8;

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 7;
        }
      else
        {
	  return 18 /* 0x12 */;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return 7;
        }
      else if ((which_alternative == 0) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 361:  /* *testqi_ext_0 */
    case 39:  /* *cmpfp_iu_387 */
    case 38:  /* *cmpfp_iu_sse */
    case 37:  /* *cmpfp_iu_mixed */
    case 36:  /* *cmpfp_i_i387 */
    case 35:  /* *cmpfp_i_sse */
    case 34:  /* *cmpfp_i_mixed */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 2010:  /* mfence_sse2 */
    case 2009:  /* *sse_sfence */
    case 2008:  /* *sse2_lfence */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1607:  /* sse2_clflush */
    case 1606:  /* sse_stmxcsr */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return 618 /* 0x26a */;

    case 9:  /* *cmphi_minus_1 */
    case 5:  /* *cmphi_1 */
    case 1:  /* *cmphi_ccno_1 */
      if ((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if ((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((! ((cached_imm_disp = get_attr_imm_disp (insn)) == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if ((! (cached_imm_disp == IMM_DISP_TRUE)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_imm_disp == IMM_DISP_TRUE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_IMUL)
        {
	  return 0;
        }
      else if (cached_type == TYPE_STR)
        {
	  return 1;
        }
      else if (cached_type == TYPE_IDIV)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)) && (get_attr_mode (insn) == MODE_XF)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FMOV) && ((immediate_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if (cached_type == TYPE_IMOV)
        {
	  return 6;
        }
      else if ((cached_type == TYPE_PUSH) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 7;
        }
      else if ((cached_type == TYPE_POP) || (cached_type == TYPE_LEAVE))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 9;
        }
      else if (cached_type == TYPE_IBR)
        {
	  return 10 /* 0xa */;
        }
      else if ((cached_type == TYPE_FOP) || (cached_type == TYPE_FISTP))
        {
	  return 11 /* 0xb */;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 12 /* 0xc */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FPSPC)
        {
	  return 14 /* 0xe */;
        }
      else if (((cached_pent_pair = get_attr_pent_pair (insn)) == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 15 /* 0xf */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 16 /* 0x10 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 17 /* 0x11 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 21 /* 0x15 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_UV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 23 /* 0x17 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_PV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 25 /* 0x19 */;
        }
      else if ((cached_pent_pair == PENT_PAIR_NP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    default:
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)
        {
	  return 18 /* 0x12 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 26 /* 0x1a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    }
}

int
insn_default_latency_pentiumpro (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1864:  /* *avx_vperm_broadcast_v4sf */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3))) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((1 << which_alternative) & 0x3))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))))
        {
	  return 3;
        }
      else if ((which_alternative == 2) && ((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1587:  /* *vec_concatv2si_sse */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 4;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1583:  /* *vec_dupv4si */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x6)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x6)))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x6)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x6)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x6))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x6)))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x6)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x6)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x7))) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (!((1 << which_alternative) & 0x7))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (!((1 << which_alternative) & 0x7)))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1510:  /* *andnotv2di3 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1509:  /* *andnotv4di3 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1508:  /* *andnotv4si3 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1507:  /* *andnotv8si3 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1506:  /* *andnotv8hi3 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1505:  /* *andnotv16hi3 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1504:  /* *andnotv16qi3 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1503:  /* *andnotv32qi3 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 6))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 6))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 6))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 10))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 10))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (which_alternative == 10)) || (((cached_memory == MEMORY_NONE) && (which_alternative == 9)) || (((cached_memory == MEMORY_LOAD) && (which_alternative == 9)) || ((cached_memory == MEMORY_STORE) && (which_alternative == 9)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 5)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 5)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 5)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 3)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 3)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 3)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 4)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 4)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (which_alternative == 9)) || (((cached_memory == MEMORY_NONE) && (which_alternative == 10)) || (((cached_memory == MEMORY_LOAD) && (which_alternative == 10)) || ((cached_memory == MEMORY_STORE) && (which_alternative == 10)))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x3a))) || (((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3a))) || ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3a)))))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (which_alternative == 9)) || (((cached_memory == MEMORY_NONE) && (which_alternative == 10)) || (((cached_memory == MEMORY_LOAD) && (which_alternative == 10)) || ((cached_memory == MEMORY_STORE) && (which_alternative == 10)))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x38))) || (((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x38))) || ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x38)))))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 4;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 1)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 1))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (which_alternative == 0)) || ((cached_memory == MEMORY_LOAD) && (which_alternative == 0)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 4;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 4)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 4))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0xf))) || ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0xf))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 1)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative != 1)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1903:  /* avx_maskstoreps */
    case 1895:  /* avx_maskloadps */
    case 1348:  /* avx2_vec_dupv4sf */
      if (get_attr_memory (insn) == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1661:  /* sse4a_vmmovntv4sf */
    case 1659:  /* sse4a_movntsf */
    case 1346:  /* sse_movss */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((1 << which_alternative) & 0x3))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0xc)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0xc)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0xc)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1352:  /* *vec_concatv4sf */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1332:  /* vec_interleave_highv4sf */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1229:  /* *andnotsf3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1214:  /* sse_andnotv4sf3 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1211:  /* sse_ucomi */
    case 1209:  /* sse_comi */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1204:  /* sse_maskcmpv4sf3 */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1194:  /* avx_cmpv4sf3 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else
        {
	  return 4;
        }

    case 1139:  /* sse_vmdivv4sf3 */
      if (get_attr_memory (insn) == MEMORY_NONE)
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 0;
        }

    case 1136:  /* sse_divv4sf3 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 48 /* 0x30 */;
        }
      else
        {
	  return 0;
        }

    case 1133:  /* sse_vmmulv4sf3 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1676:  /* sse4_1_dpps */
    case 1130:  /* *mulv4sf3 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1672:  /* sse4_1_blendvps */
    case 1668:  /* sse4_1_blendps */
    case 1598:  /* sse_movmskps */
    case 1108:  /* sse_movntv4sf */
    case 1096:  /* sse_storeups */
    case 1092:  /* sse_loadups */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1088:  /* *movv2df_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V2DFmode) == (V4SFmode)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V2DFmode) == (V4SFmode)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1087:  /* *movv4df_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V4DFmode) == (V4SFmode)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V4DFmode) == (V4SFmode)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1086:  /* *movv4sf_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((V4SFmode) == (V4SFmode))))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((V4SFmode) == (V4SFmode))))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V8SFmode) == (V4SFmode)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V8SFmode) == (V4SFmode)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1084:  /* *movv1ti_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V1TImode) == (V4SFmode)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V1TImode) == (V4SFmode)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1083:  /* *movv2ti_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V2TImode) == (V4SFmode)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V2TImode) == (V4SFmode)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1082:  /* *movv2di_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V2DImode) == (V4SFmode)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V2DImode) == (V4SFmode)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1081:  /* *movv4di_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V4DImode) == (V4SFmode)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V4DImode) == (V4SFmode)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1080:  /* *movv4si_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V4SImode) == (V4SFmode)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V4SImode) == (V4SFmode)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1079:  /* *movv8si_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V8SImode) == (V4SFmode)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V8SImode) == (V4SFmode)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1078:  /* *movv8hi_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V8HImode) == (V4SFmode)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V8HImode) == (V4SFmode)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1077:  /* *movv16hi_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V16HImode) == (V4SFmode)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V16HImode) == (V4SFmode)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1076:  /* *movv16qi_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V16QImode) == (V4SFmode)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V16QImode) == (V4SFmode)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V32QImode) == (V4SFmode)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V32QImode) == (V4SFmode)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (which_alternative == 0)
        {
	  return 4;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 3)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 3)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 4)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 4)))))
        {
	  return 1;
        }
      else if (which_alternative == 0)
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && (which_alternative == 3)) || (((cached_memory == MEMORY_LOAD) && (which_alternative == 3)) || ((cached_memory == MEMORY_STORE) && (which_alternative == 3))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (which_alternative == 1)) || ((cached_memory == MEMORY_LOAD) && (which_alternative == 1)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1930:  /* vcvtph2ps */
    case 1726:  /* sse4_1_roundss */
    case 1723:  /* sse4_1_roundps */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1297:  /* floatv4siv4sf2 */
    case 1285:  /* sse_cvtpi2ps */
    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 972:  /* *mmx_mulv2sf3 */
      if (get_attr_memory (insn) == MEMORY_NONE)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 6)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 6)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 7))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 7))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 7))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && (cached_memory == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 0))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && (which_alternative == 9)) || ((cached_memory == MEMORY_LOAD) && (which_alternative == 9)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 8))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 8))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 8))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 9)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 9)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 10))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 10))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 10))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && (cached_memory == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 0))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3))) || ((cached_memory == MEMORY_NONE) && (which_alternative == 2)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode)))) || ((cached_memory == MEMORY_NONE) && ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode)))) || ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode)))) || ((cached_memory == MEMORY_NONE) && ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode)))) || ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 894:  /* *movxfcc_1 */
      return 2;

    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 38 /* 0x26 */;

    case 772:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 32 /* 0x20 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 33 /* 0x21 */;
        }
      else
        {
	  return 0;
        }

    case 771:  /* *fop_xf_6_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 767:  /* *fop_xf_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 19 /* 0x13 */;
        }
      else
        {
	  return 0;
        }

    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 38 /* 0x26 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 39 /* 0x27 */;
        }
      else
        {
	  return 0;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], XFmode)))) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 19 /* 0x13 */;
        }
      else
        {
	  return 0;
        }

    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 32 /* 0x20 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 33 /* 0x21 */;
        }
      else
        {
	  return 0;
        }

    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 19 /* 0x13 */;
        }
      else
        {
	  return 0;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))) || ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 0;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 32 /* 0x20 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 33 /* 0x21 */;
        }
      else
        {
	  return 0;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 19 /* 0x13 */;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) || ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 0;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], DFmode)))) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], SFmode)))) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], SFmode)))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))) || ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))) || ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (mult_operator (operands[3], DFmode))))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))) || ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (mult_operator (operands[3], SFmode))))
        {
	  return 6;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 2010:  /* mfence_sse2 */
    case 2009:  /* *sse_sfence */
    case 2008:  /* *sse2_lfence */
    case 1607:  /* sse2_clflush */
    case 1311:  /* sse2_cvttsd2si */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1294:  /* sse_cvttss2si */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 779:  /* *sqrtsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 745:  /* *rcpsf2_sse */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 3;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 0))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 6;
        }

    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 904:  /* *ieee_sminsf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 585:  /* setcc_sf_sse */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative == 0))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (! (cached_memory == MEMORY_NONE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 493:  /* *ashlqi3_1_slp */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 2) || ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT)))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative != 0) || ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT)))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 1) || ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 1) || ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT)))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x3)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x7)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 355:  /* udivmodhiqi3 */
    case 346:  /* divmodhiqi3 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else
        {
	  return 0;
        }

    case 353:  /* *udivmodsi4_noext */
    case 344:  /* *divmodsi4_noext */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 39 /* 0x27 */;
        }
      else
        {
	  return 0;
        }

    case 352:  /* *udivmodhi4_noext */
    case 343:  /* *divmodhi4_noext */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 23 /* 0x17 */;
        }
      else
        {
	  return 0;
        }

    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (which_alternative == 0)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 3;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 5))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 5))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 5))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 2))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 3))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 3))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 3))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else if (which_alternative != 0)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 199:  /* *floatdisf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
      return 5;

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || ((cached_memory == MEMORY_LOAD) && (which_alternative == 0)))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (which_alternative == 1)) || ((cached_memory == MEMORY_NONE) && (which_alternative == 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1923:  /* avx2_lshrvv2di */
    case 1922:  /* avx2_ashlvv2di */
    case 1921:  /* avx2_lshrvv4di */
    case 1920:  /* avx2_ashlvv4di */
    case 1919:  /* avx2_lshrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1917:  /* avx2_lshrvv8si */
    case 1916:  /* avx2_ashlvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1914:  /* avx2_ashrvv8si */
    case 1798:  /* xop_shlv2di3 */
    case 1797:  /* xop_shlv4si3 */
    case 1796:  /* xop_shlv8hi3 */
    case 1795:  /* xop_shlv16qi3 */
    case 1794:  /* xop_shav2di3 */
    case 1793:  /* xop_shav4si3 */
    case 1792:  /* xop_shav8hi3 */
    case 1791:  /* xop_shav16qi3 */
    case 1790:  /* xop_vrotlv2di3 */
    case 1789:  /* xop_vrotlv4si3 */
    case 1788:  /* xop_vrotlv8hi3 */
    case 1787:  /* xop_vrotlv16qi3 */
    case 1786:  /* xop_rotrv2di3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1779:  /* xop_rotlv16qi3 */
    case 1649:  /* ssse3_palignrdi */
    case 1648:  /* ssse3_palignrti */
    case 1647:  /* avx2_palignrv2ti */
    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1631:  /* ssse3_pmaddubsw */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1446:  /* ashrv4si3 */
    case 1445:  /* ashrv8si3 */
    case 1444:  /* ashrv8hi3 */
    case 1443:  /* ashrv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1436:  /* *avx2_pmaddwd */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x18)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x18)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x18))) || (((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7))) || (((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7))) || ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7))))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && (!((1 << which_alternative) & 0xce3f)))) || (((cached_memory == MEMORY_LOAD) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && (!((1 << which_alternative) & 0xce3f)))) || ((cached_memory == MEMORY_STORE) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && (!((1 << which_alternative) & 0xce3f))))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((! (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (which_alternative == 5))) || ((cached_memory == MEMORY_LOAD) && ((((! (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (which_alternative == 5))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7))) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((1 << which_alternative) & 0x7))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((1 << which_alternative) & 0x220)))) || ((cached_memory == MEMORY_LOAD) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((1 << which_alternative) & 0x220)))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (!((1 << which_alternative) & 0x23f))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (!((1 << which_alternative) & 0x23f))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (!((1 << which_alternative) & 0x23f))))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x78)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x78)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x78))) || (((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7))) || (((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7))) || ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7))))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) && (which_alternative == 7))) || ((cached_memory == MEMORY_LOAD) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) && (which_alternative == 7))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 8) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 8) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 8) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 2) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((which_alternative == 2) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (((1 << which_alternative) & 0x3))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (((1 << which_alternative) & 0x3))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (((1 << which_alternative) & 0x3))))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) && (which_alternative == 2))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) && (which_alternative == 2))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (((1 << which_alternative) & 0x3))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (((1 << which_alternative) & 0x3))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (((1 << which_alternative) & 0x3))))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 6;
        }
      else if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV)) || ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOVX)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_IMOVX))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))) || ((cached_memory == MEMORY_NONE) && ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1338:  /* sse3_movsldup */
    case 1336:  /* sse3_movshdup */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1142:  /* sse_rcpv4sf2 */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 993:  /* *vec_dupv2sf */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 323:  /* *mulqi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 211:  /* *floatsisf2_vector_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 154:  /* fix_truncsfdi_sse */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 4;

    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 920:  /* *prefetch_3dnow */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 1;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2)))))))) || ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOVX)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_IMOVX))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV)) || ((cached_memory == MEMORY_NONE) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode))))) || ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))) || (((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))) || ((cached_memory == MEMORY_STORE) && ((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (which_alternative == 6))) || ((cached_memory == MEMORY_LOAD) && ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (which_alternative == 6))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((((1 << which_alternative) & 0xc0)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && (((1 << which_alternative) & 0xf80))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((((1 << which_alternative) & 0xc0)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && (((1 << which_alternative) & 0xf80))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((((1 << which_alternative) & 0xc0)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && (((1 << which_alternative) & 0xf80))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 9)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 9)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 11))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 11))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 11))
        {
	  return 3;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode))))) || ((cached_memory == MEMORY_NONE) && (((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode))) || (which_alternative == 4))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 4))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (which_alternative == 0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (which_alternative == 0))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) && (which_alternative != 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) && (which_alternative != 0)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) && (which_alternative != 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (which_alternative == 2))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (which_alternative == 2))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) && (!((1 << which_alternative) & 0x7))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) && (!((1 << which_alternative) & 0x7))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) && (!((1 << which_alternative) & 0x7))))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 3;
        }
      else
        {
	  return 4;
        }

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      extract_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (GET_MODE (operands[1]) == SFmode)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (GET_MODE (operands[1]) == SFmode)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && ((which_alternative == 1) && (GET_MODE (operands[1]) == SFmode))) || ((cached_memory == MEMORY_LOAD) && ((which_alternative == 1) && (GET_MODE (operands[1]) == SFmode))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return 6;

    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_OTHER) || ((cached_type == TYPE_MULTI) || ((cached_type == TYPE_CALL) || ((cached_type == TYPE_CALLV) || (cached_type == TYPE_STR)))))
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (cached_type == TYPE_IMOV))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_IMOV))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (cached_type == TYPE_IMOV)) || ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_IMOVX)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_IMOVX))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_LEA) || ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1))))))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_IBR))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (cached_type == TYPE_IBR))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_LEAVE) || (((cached_memory == MEMORY_NONE) && (cached_type == TYPE_IMUL)) || ((! (cached_memory == MEMORY_NONE)) && (cached_type == TYPE_IMUL))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_QI) && (cached_type == TYPE_IDIV))) || ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_QI) && (cached_type == TYPE_IDIV))))
        {
	  return 19 /* 0x13 */;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (cached_type == TYPE_IDIV))) || ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (cached_type == TYPE_IDIV))))
        {
	  return 23 /* 0x17 */;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_SI) && (cached_type == TYPE_IDIV))) || ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_SI) && (cached_type == TYPE_IDIV))))
        {
	  return 39 /* 0x27 */;
        }
      else if (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && (cached_type == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_BOTH) && (cached_type == TYPE_FOP))
        {
	  return 5;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 1;
        }
      else if (cached_type == TYPE_FISTP)
        {
	  return 5;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FCMP))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FCMP))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FMOV)) || ((cached_memory == MEMORY_LOAD) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_XF)) && (cached_type == TYPE_FMOV))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && (cached_type == TYPE_FMOV)))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_XF)) && (cached_type == TYPE_FMOV)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_STORE) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && (cached_type == TYPE_FMOV)))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FMUL))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FMUL))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 32 /* 0x20 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 38 /* 0x26 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 39 /* 0x27 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_MMXSHFT))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_MMXMUL))
        {
	  return 3;
        }
      else if (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_MMXCVT))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_UNKNOWN) && (cached_type == TYPE_SSE)) || (((cached_mode == MODE_SF) && (cached_type == TYPE_SSE)) || (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEADD))) || (((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEADD))) || (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSECMP))) || ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSECMP))))))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSECOMI))) || ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSECOMI))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEMUL))) || ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEMUL))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEDIV)))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEICVT))
        {
	  return 4;
        }
      else if (((cached_mode == MODE_SI) && (cached_type == TYPE_SSEICVT)) || (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEMOV))) || (((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEMOV))) || ((cached_memory == MEMORY_STORE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEMOV))))))
        {
	  return 3;
        }
      else if ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSE))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEADD))) || (((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEADD))) || (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSECMP))) || (((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSECMP))) || (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSECVT)))))))
        {
	  return 3;
        }
      else if (((! (cached_memory == MEMORY_NONE)) && (! (cached_memory == MEMORY_UNKNOWN))) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSECMP)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEMUL))) || ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEMUL))))
        {
	  return 5;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEDIV))) || ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEDIV))))
        {
	  return 48 /* 0x30 */;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_V4SF) && ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)))) || ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_V4SF) && ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEMOV)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEMOV)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEMOV)))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_ICMOV) || ((cached_type == TYPE_PUSH) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_FXCH) || ((cached_type == TYPE_SSEIADD) || ((cached_type == TYPE_SSEISHFT) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_SSEIMUL) || ((cached_type == TYPE_MMX) || ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_MMXCMP)))))))))))))))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_ICMOV) || ((cached_type == TYPE_PUSH) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_FXCH) || ((cached_type == TYPE_SSEIADD) || ((cached_type == TYPE_SSEISHFT) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_SSEIMUL) || ((cached_type == TYPE_MMX) || ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_MMXCMP)))))))))))))))))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_ICMOV) || ((cached_type == TYPE_PUSH) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_FXCH) || ((cached_type == TYPE_SSEIADD) || ((cached_type == TYPE_SSEISHFT) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_SSEIMUL) || ((cached_type == TYPE_MMX) || ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_MMXCMP)))))))))))))))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_ICMOV) || ((cached_type == TYPE_PUSH) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_FXCH) || ((cached_type == TYPE_SSEIADD) || ((cached_type == TYPE_SSEISHFT) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_SSEIMUL) || ((cached_type == TYPE_MMX) || ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_MMXCMP)))))))))))))))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    default:
      return 0;

    }
}

int
internal_dfa_insn_code_pentiumpro (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1864:  /* *avx_vperm_broadcast_v4sf */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 88 /* 0x58 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 89 /* 0x59 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 90 /* 0x5a */;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_NONE))
        {
	  return 102 /* 0x66 */;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_LOAD))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2010:  /* mfence_sse2 */
    case 2009:  /* *sse_sfence */
    case 2008:  /* *sse2_lfence */
    case 1607:  /* sse2_clflush */
      return 74 /* 0x4a */;

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1587:  /* *vec_concatv2si_sse */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 73 /* 0x49 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 73 /* 0x49 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 73 /* 0x49 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 104 /* 0x68 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 105 /* 0x69 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1583:  /* *vec_dupv4si */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 102 /* 0x66 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 103 /* 0x67 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 106 /* 0x6a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x6)))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x6)))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x6)))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x6)))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 28 /* 0x1c */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 29 /* 0x1d */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 30 /* 0x1e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x6)))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x6)))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x6)))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x6)))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 27 /* 0x1b */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 29 /* 0x1d */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 30 /* 0x1e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 88 /* 0x58 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x7)))
        {
	  return 89 /* 0x59 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 90 /* 0x5a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1510:  /* *andnotv2di3 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1509:  /* *andnotv4di3 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1508:  /* *andnotv4si3 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1507:  /* *andnotv8si3 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1506:  /* *andnotv8hi3 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1505:  /* *andnotv16hi3 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1504:  /* *andnotv16qi3 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1503:  /* *andnotv32qi3 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 6))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 6))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 6))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 10))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 10))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 10))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 9))
        {
	  return 59 /* 0x3b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 28 /* 0x1c */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 29 /* 0x1d */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 30 /* 0x1e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 5))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 5))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 5))
        {
	  return 59 /* 0x3b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 28 /* 0x1c */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 29 /* 0x1d */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 30 /* 0x1e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 3))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 3))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 3))
        {
	  return 59 /* 0x3b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 28 /* 0x1c */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 29 /* 0x1d */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 30 /* 0x1e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 59 /* 0x3b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 27 /* 0x1b */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 102 /* 0x66 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 9))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 10))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 10))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 10))
        {
	  return 59 /* 0x3b */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x3a)))
        {
	  return 88 /* 0x58 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3a)))
        {
	  return 89 /* 0x59 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3a)))
        {
	  return 90 /* 0x5a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 9))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 10))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 10))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 10))
        {
	  return 59 /* 0x3b */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x38)))
        {
	  return 88 /* 0x58 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x38)))
        {
	  return 89 /* 0x59 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x38)))
        {
	  return 90 /* 0x5a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 73 /* 0x49 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 88 /* 0x58 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 89 /* 0x59 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 90 /* 0x5a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 73 /* 0x49 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 88 /* 0x58 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 89 /* 0x59 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 90 /* 0x5a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0xf)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0xf)))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 1))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 1))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1903:  /* avx_maskstoreps */
    case 1895:  /* avx_maskloadps */
    case 1348:  /* avx2_vec_dupv4sf */
      if (get_attr_memory (insn) == MEMORY_NONE)
        {
	  return 102 /* 0x66 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1661:  /* sse4a_vmmovntv4sf */
    case 1659:  /* sse4a_movntsf */
    case 1346:  /* sse_movss */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 88 /* 0x58 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 89 /* 0x59 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 90 /* 0x5a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0xc)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0xc)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0xc)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1352:  /* *vec_concatv4sf */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1311:  /* sse2_cvttsd2si */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1294:  /* sse_cvttss2si */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
      return 87 /* 0x57 */;

    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1930:  /* vcvtph2ps */
    case 1726:  /* sse4_1_roundss */
    case 1723:  /* sse4_1_roundps */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1297:  /* floatv4siv4sf2 */
    case 1285:  /* sse_cvtpi2ps */
      if (get_attr_memory (insn) == MEMORY_NONE)
        {
	  return 96 /* 0x60 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1332:  /* vec_interleave_highv4sf */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1229:  /* *andnotsf3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1214:  /* sse_andnotv4sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 102 /* 0x66 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1211:  /* sse_ucomi */
    case 1209:  /* sse_comi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 80 /* 0x50 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 81 /* 0x51 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1204:  /* sse_maskcmpv4sf3 */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1194:  /* avx_cmpv4sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 94 /* 0x5e */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 95 /* 0x5f */;
        }
      else
        {
	  return 97 /* 0x61 */;
        }

    case 1338:  /* sse3_movsldup */
    case 1336:  /* sse3_movshdup */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1142:  /* sse_rcpv4sf2 */
      return 91 /* 0x5b */;

    case 1139:  /* sse_vmdivv4sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 84 /* 0x54 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1136:  /* sse_divv4sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 100 /* 0x64 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 101 /* 0x65 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1133:  /* sse_vmmulv4sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 82 /* 0x52 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 83 /* 0x53 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1676:  /* sse4_1_dpps */
    case 1130:  /* *mulv4sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 98 /* 0x62 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 99 /* 0x63 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 92 /* 0x5c */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 93 /* 0x5d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1672:  /* sse4_1_blendvps */
    case 1668:  /* sse4_1_blendps */
    case 1598:  /* sse_movmskps */
    case 1108:  /* sse_movntv4sf */
    case 1096:  /* sse_storeups */
    case 1092:  /* sse_loadups */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 104 /* 0x68 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 105 /* 0x69 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1088:  /* *movv2df_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V2DFmode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V2DFmode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1087:  /* *movv4df_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V4DFmode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V4DFmode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1086:  /* *movv4sf_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((V4SFmode) == (V4SFmode))))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((V4SFmode) == (V4SFmode))))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V8SFmode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V8SFmode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1084:  /* *movv1ti_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V1TImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V1TImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1083:  /* *movv2ti_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V2TImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V2TImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1082:  /* *movv2di_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V2DImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V2DImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1081:  /* *movv4di_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V4DImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V4DImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1080:  /* *movv4si_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V4SImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V4SImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1079:  /* *movv8si_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V8SImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V8SImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1078:  /* *movv8hi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V8HImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V8HImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1077:  /* *movv16hi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V16HImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V16HImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1076:  /* *movv16qi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V16QImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V16QImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V32QImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((V32QImode) == (V4SFmode)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 28 /* 0x1c */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 29 /* 0x1d */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 30 /* 0x1e */;
        }
      else if (which_alternative == 0)
        {
	  return 73 /* 0x49 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 3))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 3))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 69 /* 0x45 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 28 /* 0x1c */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 29 /* 0x1d */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 30 /* 0x1e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 59 /* 0x3b */;
        }
      else if (which_alternative == 0)
        {
	  return 73 /* 0x49 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 3))
        {
	  return 88 /* 0x58 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 3))
        {
	  return 89 /* 0x59 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 3))
        {
	  return 90 /* 0x5a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 103 /* 0x67 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 73 /* 0x49 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 993:  /* *vec_dupv2sf */
      return 73 /* 0x49 */;

    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 972:  /* *mmx_mulv2sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 71 /* 0x47 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 27 /* 0x1b */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 6))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 6))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 7))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 7))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 7))
        {
	  return 106 /* 0x6a */;
        }
      else if ((which_alternative == 0) && (cached_memory == MEMORY_NONE))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 0))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 8))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 8))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 8))
        {
	  return 106 /* 0x6a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 27 /* 0x1b */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 10))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 10))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 10))
        {
	  return 106 /* 0x6a */;
        }
      else if ((which_alternative == 0) && (cached_memory == MEMORY_NONE))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 0))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode)))))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode)))))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 904:  /* *ieee_sminsf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 76 /* 0x4c */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 77 /* 0x4d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 27 /* 0x1b */;
        }
      else
        {
	  return 53 /* 0x35 */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 53 /* 0x35 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 107 /* 0x6b */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 108 /* 0x6c */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 109 /* 0x6d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 894:  /* *movxfcc_1 */
      return 53 /* 0x35 */;

    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 920:  /* *prefetch_3dnow */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
      return 107 /* 0x6b */;

    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 67 /* 0x43 */;

    case 772:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 62 /* 0x3e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 65 /* 0x41 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 66 /* 0x42 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 771:  /* *fop_xf_6_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 767:  /* *fop_xf_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 62 /* 0x3e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 63 /* 0x3f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 64 /* 0x40 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 62 /* 0x3e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 62 /* 0x3e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 67 /* 0x43 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 68 /* 0x44 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 62 /* 0x3e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 62 /* 0x3e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 63 /* 0x3f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 64 /* 0x40 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 62 /* 0x3e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 62 /* 0x3e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 62 /* 0x3e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 65 /* 0x41 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 66 /* 0x42 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 62 /* 0x3e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 63 /* 0x3f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 64 /* 0x40 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 76 /* 0x4c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 77 /* 0x4d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 82 /* 0x52 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 83 /* 0x53 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 84 /* 0x54 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 779:  /* *sqrtsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 745:  /* *rcpsf2_sse */
      return 75 /* 0x4b */;

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 62 /* 0x3e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 65 /* 0x41 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 66 /* 0x42 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 62 /* 0x3e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 63 /* 0x3f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 64 /* 0x40 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 76 /* 0x4c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 77 /* 0x4d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 82 /* 0x52 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 83 /* 0x53 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 84 /* 0x54 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 62 /* 0x3e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 62 /* 0x3e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 76 /* 0x4c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 77 /* 0x4d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 82 /* 0x52 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 83 /* 0x53 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (mult_operator (operands[3], DFmode))))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (mult_operator (operands[3], DFmode))))
        {
	  return 62 /* 0x3e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 50 /* 0x32 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (mult_operator (operands[3], SFmode))))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (mult_operator (operands[3], SFmode))))
        {
	  return 62 /* 0x3e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 76 /* 0x4c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 77 /* 0x4d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode))))
        {
	  return 82 /* 0x52 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode))))
        {
	  return 83 /* 0x53 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 108 /* 0x6c */;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 0))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 0))
        {
	  return 30 /* 0x1e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 38 /* 0x26 */;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 36 /* 0x24 */;
        }
      else
        {
	  return 37 /* 0x25 */;
        }

    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 585:  /* setcc_sf_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 78 /* 0x4e */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 79 /* 0x4f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 107 /* 0x6b */;
        }
      else
        {
	  return 109 /* 0x6d */;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 34 /* 0x22 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative == 0))
        {
	  return 35 /* 0x23 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 32 /* 0x20 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 34 /* 0x22 */;
        }
      else if ((which_alternative == 1) && (! (cached_memory == MEMORY_NONE)))
        {
	  return 35 /* 0x23 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 34 /* 0x22 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 35 /* 0x23 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 34 /* 0x22 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 35 /* 0x23 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 493:  /* *ashlqi3_1_slp */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1))
        {
	  return 34 /* 0x22 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1))
        {
	  return 35 /* 0x23 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 34 /* 0x22 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 35 /* 0x23 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))))
        {
	  return 34 /* 0x22 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))))
        {
	  return 35 /* 0x23 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 491:  /* *ashlhi3_1 */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 34 /* 0x22 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 35 /* 0x23 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 34 /* 0x22 */;
        }
      else
        {
	  return 35 /* 0x23 */;
        }

    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
      return 51 /* 0x33 */;

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 31 /* 0x1f */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 32 /* 0x20 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x3)))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 31 /* 0x1f */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 32 /* 0x20 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x7)))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 355:  /* udivmodhiqi3 */
    case 346:  /* divmodhiqi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 41 /* 0x29 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 42 /* 0x2a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 353:  /* *udivmodsi4_noext */
    case 344:  /* *divmodsi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 45 /* 0x2d */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 46 /* 0x2e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 352:  /* *udivmodhi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 43 /* 0x2b */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 44 /* 0x2c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 39 /* 0x27 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative == 0))
        {
	  return 40 /* 0x28 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 323:  /* *mulqi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 39 /* 0x27 */;
        }
      else
        {
	  return 40 /* 0x28 */;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 107 /* 0x6b */;
        }
      else
        {
	  return 108 /* 0x6c */;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 5))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative != 5))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 5))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 5))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 5))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative != 2))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 2))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 2))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 2))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 3))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative != 3))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 3))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 3))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 3))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
      return 33 /* 0x21 */;

    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 86 /* 0x56 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 59 /* 0x3b */;
        }
      else if (which_alternative != 0)
        {
	  return 86 /* 0x56 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 199:  /* *floatdisf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 27 /* 0x1b */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 59 /* 0x3b */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 86 /* 0x56 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 27 /* 0x1b */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 59 /* 0x3b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 27 /* 0x1b */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 59 /* 0x3b */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 86 /* 0x56 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 56 /* 0x38 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 58 /* 0x3a */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 60 /* 0x3c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 27 /* 0x1b */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 56 /* 0x38 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 58 /* 0x3a */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 60 /* 0x3c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
      return 52 /* 0x34 */;

    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 211:  /* *floatsisf2_vector_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 154:  /* fix_truncsfdi_sse */
      return 86 /* 0x56 */;

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 27 /* 0x1b */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 56 /* 0x38 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 57 /* 0x39 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 59 /* 0x3b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 27 /* 0x1b */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 59 /* 0x3b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 56 /* 0x38 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 57 /* 0x39 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 59 /* 0x3b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 59 /* 0x3b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 57 /* 0x39 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 58 /* 0x3a */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 59 /* 0x3b */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 60 /* 0x3c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 58 /* 0x3a */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 59 /* 0x3b */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 60 /* 0x3c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 32 /* 0x20 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 107 /* 0x6b */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 108 /* 0x6c */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 109 /* 0x6d */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_BOTH))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 27 /* 0x1b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 32 /* 0x20 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1923:  /* avx2_lshrvv2di */
    case 1922:  /* avx2_ashlvv2di */
    case 1921:  /* avx2_lshrvv4di */
    case 1920:  /* avx2_ashlvv4di */
    case 1919:  /* avx2_lshrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1917:  /* avx2_lshrvv8si */
    case 1916:  /* avx2_ashlvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1914:  /* avx2_ashrvv8si */
    case 1798:  /* xop_shlv2di3 */
    case 1797:  /* xop_shlv4si3 */
    case 1796:  /* xop_shlv8hi3 */
    case 1795:  /* xop_shlv16qi3 */
    case 1794:  /* xop_shav2di3 */
    case 1793:  /* xop_shav4si3 */
    case 1792:  /* xop_shav8hi3 */
    case 1791:  /* xop_shav16qi3 */
    case 1790:  /* xop_vrotlv2di3 */
    case 1789:  /* xop_vrotlv4si3 */
    case 1788:  /* xop_vrotlv8hi3 */
    case 1787:  /* xop_vrotlv16qi3 */
    case 1786:  /* xop_rotrv2di3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1779:  /* xop_rotlv16qi3 */
    case 1649:  /* ssse3_palignrdi */
    case 1648:  /* ssse3_palignrti */
    case 1647:  /* avx2_palignrv2ti */
    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1631:  /* ssse3_pmaddubsw */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1446:  /* ashrv4si3 */
    case 1445:  /* ashrv8si3 */
    case 1444:  /* ashrv8hi3 */
    case 1443:  /* ashrv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1436:  /* *avx2_pmaddwd */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 107 /* 0x6b */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 108 /* 0x6c */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 109 /* 0x6d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x18)))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x18)))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x18)))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 59 /* 0x3b */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && (!((1 << which_alternative) & 0xce3f))))
        {
	  return 88 /* 0x58 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && (!((1 << which_alternative) & 0xce3f))))
        {
	  return 89 /* 0x59 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && (!((1 << which_alternative) & 0xce3f))))
        {
	  return 90 /* 0x5a */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((! (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (which_alternative == 5)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((! (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (which_alternative == 5)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 27 /* 0x1b */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 59 /* 0x3b */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((1 << which_alternative) & 0x220))))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((1 << which_alternative) & 0x220))))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (!((1 << which_alternative) & 0x23f))))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (!((1 << which_alternative) & 0x23f))))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (!((1 << which_alternative) & 0x23f))))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x78)))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x78)))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x78)))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 59 /* 0x3b */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) && (which_alternative == 7)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) && (which_alternative == 7)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 8) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 8) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 8) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 27 /* 0x1b */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 56 /* 0x38 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 58 /* 0x3a */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 60 /* 0x3c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 2) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 2) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (((1 << which_alternative) & 0x3))))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (((1 << which_alternative) & 0x3))))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (((1 << which_alternative) & 0x3))))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 27 /* 0x1b */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) && (which_alternative == 2)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) && (which_alternative == 2)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (((1 << which_alternative) & 0x3))))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (((1 << which_alternative) & 0x3))))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) && (((1 << which_alternative) & 0x3))))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 27 /* 0x1b */;
        }
      else if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 109 /* 0x6d */;
        }
      else
        {
	  return 110 /* 0x6e */;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOVX))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_IMOVX))
        {
	  return 32 /* 0x20 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))
        {
	  return 32 /* 0x20 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 31 /* 0x1f */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 32 /* 0x20 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 28 /* 0x1c */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 29 /* 0x1d */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 30 /* 0x1e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 29 /* 0x1d */;

    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 30 /* 0x1e */;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOVX))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_IMOVX))
        {
	  return 32 /* 0x20 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))))
        {
	  return 32 /* 0x20 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode))))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 88 /* 0x58 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 89 /* 0x59 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 90 /* 0x5a */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (which_alternative == 6)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (which_alternative == 6)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((((1 << which_alternative) & 0xc0)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && (((1 << which_alternative) & 0xf80))))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((((1 << which_alternative) & 0xc0)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && (((1 << which_alternative) & 0xf80))))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((((1 << which_alternative) & 0xc0)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && (((1 << which_alternative) & 0xf80))))
        {
	  return 106 /* 0x6a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 27 /* 0x1b */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 11))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 11))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 11))
        {
	  return 106 /* 0x6a */;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_NONE))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode))))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 4))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (which_alternative == 0)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (which_alternative == 0)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) && (which_alternative != 0)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) && (which_alternative != 0)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) && (which_alternative != 0)))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 27 /* 0x1b */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (which_alternative == 2)))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (which_alternative == 2)))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) && (!((1 << which_alternative) & 0x7))))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) && (!((1 << which_alternative) & 0x7))))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) && (!((1 << which_alternative) & 0x7))))
        {
	  return 106 /* 0x6a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 107 /* 0x6b */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 109 /* 0x6d */;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 108 /* 0x6c */;
        }
      else
        {
	  return 110 /* 0x6e */;
        }

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 109 /* 0x6d */;
        }
      else
        {
	  return 110 /* 0x6e */;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 27 /* 0x1b */;
        }
      else if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 109 /* 0x6d */;
        }
      else
        {
	  return 110 /* 0x6e */;
        }

    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 54 /* 0x36 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 55 /* 0x37 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (GET_MODE (operands[1]) == SFmode))
        {
	  return 80 /* 0x50 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (GET_MODE (operands[1]) == SFmode))
        {
	  return 81 /* 0x51 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 54 /* 0x36 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 55 /* 0x37 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 1) && (GET_MODE (operands[1]) == SFmode)))
        {
	  return 80 /* 0x50 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 1) && (GET_MODE (operands[1]) == SFmode)))
        {
	  return 81 /* 0x51 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return 27 /* 0x1b */;

    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 107 /* 0x6b */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 108 /* 0x6c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_OTHER) || ((cached_type == TYPE_MULTI) || ((cached_type == TYPE_CALL) || ((cached_type == TYPE_CALLV) || (cached_type == TYPE_STR)))))
        {
	  return 27 /* 0x1b */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (cached_type == TYPE_IMOV))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_IMOV))
        {
	  return 29 /* 0x1d */;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_IMOV))
        {
	  return 30 /* 0x1e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_IMOVX))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_IMOVX))
        {
	  return 32 /* 0x20 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_LEA))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1)))))
        {
	  return 34 /* 0x22 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1)))))
        {
	  return 35 /* 0x23 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_IBR))
        {
	  return 36 /* 0x24 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (cached_type == TYPE_IBR))
        {
	  return 37 /* 0x25 */;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 38 /* 0x26 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_IMUL))
        {
	  return 39 /* 0x27 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (cached_type == TYPE_IMUL))
        {
	  return 40 /* 0x28 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_QI) && (cached_type == TYPE_IDIV)))
        {
	  return 41 /* 0x29 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_QI) && (cached_type == TYPE_IDIV)))
        {
	  return 42 /* 0x2a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (cached_type == TYPE_IDIV)))
        {
	  return 43 /* 0x2b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (cached_type == TYPE_IDIV)))
        {
	  return 44 /* 0x2c */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_SI) && (cached_type == TYPE_IDIV)))
        {
	  return 45 /* 0x2d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_SI) && (cached_type == TYPE_IDIV)))
        {
	  return 46 /* 0x2e */;
        }
      else if (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && (cached_type == TYPE_FOP))
        {
	  return 47 /* 0x2f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FOP))
        {
	  return 48 /* 0x30 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_FOP))
        {
	  return 49 /* 0x31 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (cached_type == TYPE_FOP))
        {
	  return 50 /* 0x32 */;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 51 /* 0x33 */;
        }
      else if (cached_type == TYPE_FISTP)
        {
	  return 52 /* 0x34 */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 53 /* 0x35 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FCMP))
        {
	  return 54 /* 0x36 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FCMP))
        {
	  return 55 /* 0x37 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FMOV))
        {
	  return 56 /* 0x38 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_XF)) && (cached_type == TYPE_FMOV)))
        {
	  return 57 /* 0x39 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && (cached_type == TYPE_FMOV)))
        {
	  return 58 /* 0x3a */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_XF)) && (cached_type == TYPE_FMOV)))
        {
	  return 59 /* 0x3b */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && (cached_type == TYPE_FMOV)))
        {
	  return 60 /* 0x3c */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FMUL))
        {
	  return 61 /* 0x3d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FMUL))
        {
	  return 62 /* 0x3e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 63 /* 0x3f */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 64 /* 0x40 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 65 /* 0x41 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 66 /* 0x42 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 67 /* 0x43 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 68 /* 0x44 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_MMXSHFT))
        {
	  return 69 /* 0x45 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_MMXMUL))
        {
	  return 71 /* 0x47 */;
        }
      else if (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_MMXCVT))
        {
	  return 73 /* 0x49 */;
        }
      else if ((cached_memory == MEMORY_UNKNOWN) && (cached_type == TYPE_SSE))
        {
	  return 74 /* 0x4a */;
        }
      else if ((cached_mode == MODE_SF) && (cached_type == TYPE_SSE))
        {
	  return 75 /* 0x4b */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEADD)))
        {
	  return 76 /* 0x4c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEADD)))
        {
	  return 77 /* 0x4d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSECMP)))
        {
	  return 78 /* 0x4e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSECMP)))
        {
	  return 79 /* 0x4f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSECOMI)))
        {
	  return 80 /* 0x50 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSECOMI)))
        {
	  return 81 /* 0x51 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEMUL)))
        {
	  return 82 /* 0x52 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEMUL)))
        {
	  return 83 /* 0x53 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEDIV)))
        {
	  return 84 /* 0x54 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEDIV)))
        {
	  return 85 /* 0x55 */;
        }
      else if ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEICVT))
        {
	  return 86 /* 0x56 */;
        }
      else if ((cached_mode == MODE_SI) && (cached_type == TYPE_SSEICVT))
        {
	  return 87 /* 0x57 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEMOV)))
        {
	  return 88 /* 0x58 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEMOV)))
        {
	  return 89 /* 0x59 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEMOV)))
        {
	  return 90 /* 0x5a */;
        }
      else if ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSE))
        {
	  return 91 /* 0x5b */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEADD)))
        {
	  return 92 /* 0x5c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEADD)))
        {
	  return 93 /* 0x5d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSECMP)))
        {
	  return 94 /* 0x5e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSECMP)))
        {
	  return 95 /* 0x5f */;
        }
      else if (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSECVT)))
        {
	  return 96 /* 0x60 */;
        }
      else if (((! (cached_memory == MEMORY_NONE)) && (! (cached_memory == MEMORY_UNKNOWN))) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSECMP)))
        {
	  return 97 /* 0x61 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEMUL)))
        {
	  return 98 /* 0x62 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEMUL)))
        {
	  return 99 /* 0x63 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEDIV)))
        {
	  return 100 /* 0x64 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEDIV)))
        {
	  return 101 /* 0x65 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_V4SF) && ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1))))
        {
	  return 102 /* 0x66 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_V4SF) && ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1))))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEMOV)))
        {
	  return 104 /* 0x68 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEMOV)))
        {
	  return 105 /* 0x69 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSEMOV)))
        {
	  return 106 /* 0x6a */;
        }
      else if (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_ICMOV) || ((cached_type == TYPE_PUSH) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_FXCH) || ((cached_type == TYPE_SSEIADD) || ((cached_type == TYPE_SSEISHFT) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_SSEIMUL) || ((cached_type == TYPE_MMX) || ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_MMXCMP)))))))))))))))))))
        {
	  return 107 /* 0x6b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_ICMOV) || ((cached_type == TYPE_PUSH) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_FXCH) || ((cached_type == TYPE_SSEIADD) || ((cached_type == TYPE_SSEISHFT) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_SSEIMUL) || ((cached_type == TYPE_MMX) || ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_MMXCMP)))))))))))))))))))
        {
	  return 108 /* 0x6c */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_ICMOV) || ((cached_type == TYPE_PUSH) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_FXCH) || ((cached_type == TYPE_SSEIADD) || ((cached_type == TYPE_SSEISHFT) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_SSEIMUL) || ((cached_type == TYPE_MMX) || ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_MMXCMP)))))))))))))))))))
        {
	  return 109 /* 0x6d */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_ICMOV) || ((cached_type == TYPE_PUSH) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_FXCH) || ((cached_type == TYPE_SSEIADD) || ((cached_type == TYPE_SSEISHFT) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_SSEIMUL) || ((cached_type == TYPE_MMX) || ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_MMXCMP)))))))))))))))))))
        {
	  return 110 /* 0x6e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    default:
      return 618 /* 0x26a */;

    }
}

int
insn_default_latency_geode (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x1f))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xf))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x600))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else if (which_alternative == 2)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3f))
        {
	  return 1;
        }
      else if (which_alternative == 5)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x2f))
        {
	  return 1;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xb))
        {
	  return 1;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else
        {
	  return 2;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc0e))
        {
	  return 1;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x600e))
        {
	  return 1;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3b))
        {
	  return 1;
        }
      else if (which_alternative == 2)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
      return 12 /* 0xc */;

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 54 /* 0x36 */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 6;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 47 /* 0x2f */;
        }
      else
        {
	  return 0;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if (! (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else
        {
	  return 10 /* 0xa */;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 6;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 47 /* 0x2f */;
        }
      else
        {
	  return 0;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 6;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 47 /* 0x2f */;
        }
      else
        {
	  return 0;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 6;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 47 /* 0x2f */;
        }
      else
        {
	  return 0;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 6;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 47 /* 0x2f */;
        }
      else
        {
	  return 0;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if (! (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else
        {
	  return 10 /* 0xa */;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if (! (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else
        {
	  return 10 /* 0xa */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 10 /* 0xa */;
        }
      else
        {
	  return 0;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 10 /* 0xa */;
        }
      else
        {
	  return 0;
        }

    case 1358:  /* *sse4_1_extractps */
    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 2;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 2;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      if ((cached_type = get_attr_type (insn)) == TYPE_ALU)
        {
	  return 1;
        }
      else if (cached_type == TYPE_ISHIFT)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 493:  /* *ashlqi3_1_slp */
      if ((cached_type = get_attr_type (insn)) == TYPE_ALU)
        {
	  return 1;
        }
      else if (cached_type == TYPE_ISHIFT1)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 2))
        {
	  return 1;
        }
      else if (cached_type == TYPE_ISHIFT)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) || ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if (cached_type == TYPE_ISHIFT)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || (which_alternative == 1))
        {
	  return 1;
        }
      else if ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 1))
        {
	  return 1;
        }
      else if (cached_type == TYPE_ISHIFT)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 993:  /* *vec_dupv2sf */
    case 992:  /* mmx_pswapdv2sf2 */
    case 991:  /* mmx_floatv2si2 */
    case 990:  /* mmx_pi2fw */
    case 989:  /* mmx_pf2iw */
    case 988:  /* mmx_pf2id */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 972:  /* *mmx_mulv2sf3 */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
    case 920:  /* *prefetch_3dnow */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      return 2;

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      return 40 /* 0x28 */;

    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 323:  /* *mulqi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
      return 7;

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 5) && (! (incdec_operand (operands[2], QImode)))) || ((which_alternative == 5) || ((which_alternative != 5) && (incdec_operand (operands[2], QImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], HImode)))) || ((which_alternative == 3) || ((which_alternative != 3) && (incdec_operand (operands[2], HImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 2) && (! (incdec_operand (operands[2], SImode)))) || ((which_alternative == 2) || ((which_alternative != 2) && (incdec_operand (operands[2], SImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 251:  /* *adddi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], DImode)))) || ((which_alternative == 3) || ((which_alternative != 3) && (incdec_operand (operands[2], DImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], SImode)))) || ((which_alternative == 3) || ((which_alternative != 3) && (incdec_operand (operands[2], SImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
      return 4;

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xce1f))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 115:  /* zero_extendsidi2_1 */
    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7f))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (cached_type == TYPE_IMOVX))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) || ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) || (get_attr_type (insn) == TYPE_IMOVX))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if ((get_attr_type (insn) == TYPE_IMOV) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xfc4))
        {
	  return 1;
        }
      else if (which_alternative == 2)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1b))
        {
	  return 1;
        }
      else if (which_alternative == 2)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x1fe10))
        {
	  return 1;
        }
      else if (which_alternative == 4)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
    case 135:  /* *extendsfdf2_mixed */
    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      return 6;

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1579:  /* *sse2_storeq_rex64 */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 969:  /* sse_movntq */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 897:  /* *movsfcc_1_387 */
    case 896:  /* *movdfcc_1 */
    case 895:  /* *movdfcc_1_rex64 */
    case 894:  /* *movxfcc_1 */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 649:  /* set_got_offset_rex64 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 641:  /* simple_return_indirect_internal */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 379:  /* *andqi_2_slp */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 372:  /* *andqi_1_slp */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 289:  /* *subqi_1_slp */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 255:  /* *addqi_1_slp */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 145:  /* *truncdfsf2_i387_1 */
    case 144:  /* *truncdfsf_i387 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 127:  /* extendsidi2_1 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 118:  /* zero_extendhisi2_and */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
    case 42:  /* *pushdi2_rex64 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      return 1;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_LEA) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_IMOV) || ((cached_type == TYPE_IMOVX) || ((cached_type == TYPE_ICMOV) || ((cached_type == TYPE_INCDEC) || (cached_type == TYPE_SETCC)))))))))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1))))
        {
	  return 2;
        }
      else if (cached_type == TYPE_IMUL)
        {
	  return 7;
        }
      else if (cached_type == TYPE_IDIV)
        {
	  return 40 /* 0x28 */;
        }
      else if ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IBR) || ((cached_type == TYPE_POP) || (cached_type == TYPE_PUSH)))
        {
	  return 1;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_STR) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_MULTI) || (cached_type == TYPE_OTHER))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_FOP) || (cached_type == TYPE_FCMP))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FMOV) || ((cached_type == TYPE_FCMOV) || ((cached_type == TYPE_FSGN) || (cached_type == TYPE_FXCH))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_FISTP) || (cached_type == TYPE_FISTTP))
        {
	  return 4;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 47 /* 0x2f */;
        }
      else if (cached_type == TYPE_FPSPC)
        {
	  return 54 /* 0x36 */;
        }
      else if (cached_type == TYPE_FRNDINT)
        {
	  return 12 /* 0xc */;
        }
      else if (cached_type == TYPE_MMXMOV)
        {
	  return 1;
        }
      else if ((cached_type == TYPE_MMX) || ((cached_type == TYPE_MMXADD) || ((cached_type == TYPE_MMXMUL) || ((cached_type == TYPE_MMXCMP) || ((cached_type == TYPE_MMXCVT) || (cached_type == TYPE_MMXSHFT))))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    default:
      return 0;

    }
}

int
internal_dfa_insn_code_geode (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xf))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 433 /* 0x1b1 */;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 10)
        {
	  return 423 /* 0x1a7 */;
        }
      else if (which_alternative == 9)
        {
	  return 435 /* 0x1b3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3f))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (which_alternative == 5)
        {
	  return 435 /* 0x1b3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xf))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (which_alternative == 3)
        {
	  return 435 /* 0x1b3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x1f))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (which_alternative == 4)
        {
	  return 435 /* 0x1b3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 433 /* 0x1b1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 9)
        {
	  return 423 /* 0x1a7 */;
        }
      else if (which_alternative == 10)
        {
	  return 435 /* 0x1b3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 441 /* 0x1b9 */;
        }
      else if (which_alternative == 2)
        {
	  return 442 /* 0x1ba */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3f))
        {
	  return 441 /* 0x1b9 */;
        }
      else if (which_alternative == 5)
        {
	  return 442 /* 0x1ba */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3f))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (which_alternative == 4)
        {
	  return 441 /* 0x1b9 */;
        }
      else if (which_alternative == 0)
        {
	  return 442 /* 0x1ba */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x1f))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (which_alternative == 4)
        {
	  return 435 /* 0x1b3 */;
        }
      else if (which_alternative == 2)
        {
	  return 441 /* 0x1b9 */;
        }
      else if (which_alternative == 0)
        {
	  return 442 /* 0x1ba */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 441 /* 0x1b9 */;
        }
      else
        {
	  return 442 /* 0x1ba */;
        }

    case 969:  /* sse_movntq */
      return 441 /* 0x1b9 */;

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 433 /* 0x1b1 */;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 441 /* 0x1b9 */;
        }
      else if (which_alternative == 0)
        {
	  return 442 /* 0x1ba */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 433 /* 0x1b1 */;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 441 /* 0x1b9 */;
        }
      else if (which_alternative == 0)
        {
	  return 442 /* 0x1ba */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 967:  /* *movv2sf_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (((1 << which_alternative) & 0x38))
        {
	  return 441 /* 0x1b9 */;
        }
      else if (which_alternative == 2)
        {
	  return 442 /* 0x1ba */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 993:  /* *vec_dupv2sf */
    case 992:  /* mmx_pswapdv2sf2 */
    case 991:  /* mmx_floatv2si2 */
    case 990:  /* mmx_pi2fw */
    case 989:  /* mmx_pf2iw */
    case 988:  /* mmx_pf2id */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 972:  /* *mmx_mulv2sf3 */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
    case 920:  /* *prefetch_3dnow */
      return 442 /* 0x1ba */;

    case 909:  /* pro_epilogue_adjust_stack_di_add */
    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 433 /* 0x1b1 */;
        }
      else
        {
	  return 435 /* 0x1b3 */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 435 /* 0x1b3 */;
        }

    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
      return 432 /* 0x1b0 */;

    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 431 /* 0x1af */;

    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
      return 440 /* 0x1b8 */;

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 439 /* 0x1b7 */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 434 /* 0x1b2 */;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 437 /* 0x1b5 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 438 /* 0x1b6 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if (! (mult_operator (operands[3], XFmode)))
        {
	  return 434 /* 0x1b2 */;
        }
      else
        {
	  return 437 /* 0x1b5 */;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 434 /* 0x1b2 */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 437 /* 0x1b5 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 438 /* 0x1b6 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 434 /* 0x1b2 */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 437 /* 0x1b5 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 438 /* 0x1b6 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 434 /* 0x1b2 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 437 /* 0x1b5 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 438 /* 0x1b6 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_FOP)
        {
	  return 434 /* 0x1b2 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 437 /* 0x1b5 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 438 /* 0x1b6 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if (! (mult_operator (operands[3], DFmode)))
        {
	  return 434 /* 0x1b2 */;
        }
      else
        {
	  return 437 /* 0x1b5 */;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if (! (mult_operator (operands[3], SFmode)))
        {
	  return 434 /* 0x1b2 */;
        }
      else
        {
	  return 437 /* 0x1b5 */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 434 /* 0x1b2 */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 437 /* 0x1b5 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 434 /* 0x1b2 */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 437 /* 0x1b5 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 430 /* 0x1ae */;

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      return 427 /* 0x1ab */;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      return 428 /* 0x1ac */;

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 424 /* 0x1a8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 424 /* 0x1a8 */;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 424 /* 0x1a8 */;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      if ((cached_type = get_attr_type (insn)) == TYPE_ALU)
        {
	  return 423 /* 0x1a7 */;
        }
      else if (cached_type == TYPE_ISHIFT)
        {
	  return 424 /* 0x1a8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 493:  /* *ashlqi3_1_slp */
      if ((cached_type = get_attr_type (insn)) == TYPE_ALU)
        {
	  return 423 /* 0x1a7 */;
        }
      else if (cached_type == TYPE_ISHIFT1)
        {
	  return 424 /* 0x1a8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 2))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (cached_type == TYPE_ISHIFT)
        {
	  return 424 /* 0x1a8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) || ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (cached_type == TYPE_ISHIFT)
        {
	  return 424 /* 0x1a8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || (which_alternative == 1))
        {
	  return 423 /* 0x1a7 */;
        }
      else if ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 424 /* 0x1a8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 1))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (cached_type == TYPE_ISHIFT)
        {
	  return 424 /* 0x1a8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      return 424 /* 0x1a8 */;

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      return 426 /* 0x1aa */;

    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 425 /* 0x1a9 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 323:  /* *mulqi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
      return 425 /* 0x1a9 */;

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 5) && (! (incdec_operand (operands[2], QImode)))) || ((which_alternative == 5) || ((which_alternative != 5) && (incdec_operand (operands[2], QImode)))))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], HImode)))) || ((which_alternative == 3) || ((which_alternative != 3) && (incdec_operand (operands[2], HImode)))))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 2) && (! (incdec_operand (operands[2], SImode)))) || ((which_alternative == 2) || ((which_alternative != 2) && (incdec_operand (operands[2], SImode)))))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 251:  /* *adddi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], DImode)))) || ((which_alternative == 3) || ((which_alternative != 3) && (incdec_operand (operands[2], DImode)))))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], SImode)))) || ((which_alternative == 3) || ((which_alternative != 3) && (incdec_operand (operands[2], SImode)))))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 433 /* 0x1b1 */;
        }
      else if (which_alternative == 0)
        {
	  return 435 /* 0x1b3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
      return 436 /* 0x1b4 */;

    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 433 /* 0x1b1 */;
        }
      else
        {
	  return 435 /* 0x1b3 */;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 433 /* 0x1b1 */;
        }
      else if (which_alternative == 0)
        {
	  return 435 /* 0x1b3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 435 /* 0x1b3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 435 /* 0x1b3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 433 /* 0x1b1 */;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return 441 /* 0x1b9 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 441 /* 0x1b9 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 894:  /* *movxfcc_1 */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
      return 435 /* 0x1b3 */;

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 435 /* 0x1b3 */;
        }
      else if (((1 << which_alternative) & 0xce00))
        {
	  return 441 /* 0x1b9 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 433 /* 0x1b1 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 435 /* 0x1b3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x78))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 435 /* 0x1b3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 433 /* 0x1b1 */;
        }
      else
        {
	  return 435 /* 0x1b3 */;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 433 /* 0x1b1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 429 /* 0x1ad */;
        }
      else
        {
	  return 433 /* 0x1b1 */;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (cached_type == TYPE_IMOVX))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) || ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) || (get_attr_type (insn) == TYPE_IMOVX))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if ((get_attr_type (insn) == TYPE_IMOV) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))))
        {
	  return 423 /* 0x1a7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xffc))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (((1 << which_alternative) & 0x38))
        {
	  return 441 /* 0x1b9 */;
        }
      else if (which_alternative == 2)
        {
	  return 442 /* 0x1ba */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 433 /* 0x1b1 */;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return 441 /* 0x1b9 */;
        }
      else if (which_alternative == 2)
        {
	  return 442 /* 0x1ba */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x1fff0))
        {
	  return 423 /* 0x1a7 */;
        }
      else if (((1 << which_alternative) & 0x1e0))
        {
	  return 441 /* 0x1b9 */;
        }
      else if (which_alternative == 4)
        {
	  return 442 /* 0x1ba */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 433 /* 0x1b1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      return 429 /* 0x1ad */;

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 429 /* 0x1ad */;
        }
      else
        {
	  return 433 /* 0x1b1 */;
        }

    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      return 434 /* 0x1b2 */;

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 434 /* 0x1b2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return 433 /* 0x1b1 */;

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 649:  /* set_got_offset_rex64 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 379:  /* *andqi_2_slp */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 372:  /* *andqi_1_slp */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 289:  /* *subqi_1_slp */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 255:  /* *addqi_1_slp */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 118:  /* zero_extendhisi2_and */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      return 423 /* 0x1a7 */;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_LEA) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_IMOV) || ((cached_type == TYPE_IMOVX) || ((cached_type == TYPE_ICMOV) || ((cached_type == TYPE_INCDEC) || (cached_type == TYPE_SETCC)))))))))))
        {
	  return 423 /* 0x1a7 */;
        }
      else if ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1))))
        {
	  return 424 /* 0x1a8 */;
        }
      else if (cached_type == TYPE_IMUL)
        {
	  return 425 /* 0x1a9 */;
        }
      else if (cached_type == TYPE_IDIV)
        {
	  return 426 /* 0x1aa */;
        }
      else if ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 427 /* 0x1ab */;
        }
      else if (cached_type == TYPE_IBR)
        {
	  return 428 /* 0x1ac */;
        }
      else if ((cached_type == TYPE_POP) || (cached_type == TYPE_PUSH))
        {
	  return 429 /* 0x1ad */;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 430 /* 0x1ae */;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 431 /* 0x1af */;
        }
      else if ((cached_type == TYPE_STR) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 432 /* 0x1b0 */;
        }
      else if ((cached_type == TYPE_MULTI) || (cached_type == TYPE_OTHER))
        {
	  return 433 /* 0x1b1 */;
        }
      else if ((cached_type == TYPE_FOP) || (cached_type == TYPE_FCMP))
        {
	  return 434 /* 0x1b2 */;
        }
      else if ((cached_type == TYPE_FMOV) || ((cached_type == TYPE_FCMOV) || ((cached_type == TYPE_FSGN) || (cached_type == TYPE_FXCH))))
        {
	  return 435 /* 0x1b3 */;
        }
      else if ((cached_type == TYPE_FISTP) || (cached_type == TYPE_FISTTP))
        {
	  return 436 /* 0x1b4 */;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 437 /* 0x1b5 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 438 /* 0x1b6 */;
        }
      else if (cached_type == TYPE_FPSPC)
        {
	  return 439 /* 0x1b7 */;
        }
      else if (cached_type == TYPE_FRNDINT)
        {
	  return 440 /* 0x1b8 */;
        }
      else if (cached_type == TYPE_MMXMOV)
        {
	  return 441 /* 0x1b9 */;
        }
      else if ((cached_type == TYPE_MMX) || ((cached_type == TYPE_MMXADD) || ((cached_type == TYPE_MMXMUL) || ((cached_type == TYPE_MMXCMP) || ((cached_type == TYPE_MMXCVT) || (cached_type == TYPE_MMXSHFT))))))
        {
	  return 442 /* 0x1ba */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    default:
      return 618 /* 0x26a */;

    }
}

int
insn_default_latency_k6 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((which_alternative == 2) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || ((which_alternative == 9) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || ((which_alternative == 5) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || ((which_alternative == 3) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || ((which_alternative == 10) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1383:  /* sse2_storehpd */
    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || ((which_alternative == 4) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], VOIDmode))
        {
	  return 3;
        }
      else if (get_attr_memory (insn) == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode)))) || (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode)))) || (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
      return 10 /* 0xa */;

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 56 /* 0x38 */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || (((cached_type = get_attr_type (insn)) == TYPE_FOP) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((mult_operator (operands[3], XFmode)) && (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 56 /* 0x38 */;
        }
      else
        {
	  return 0;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || ((! (mult_operator (operands[3], XFmode))) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((mult_operator (operands[3], XFmode)) && (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || (((cached_type = get_attr_type (insn)) == TYPE_FOP) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((mult_operator (operands[3], DFmode)) && (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 56 /* 0x38 */;
        }
      else
        {
	  return 0;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || (((cached_type = get_attr_type (insn)) == TYPE_FOP) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((mult_operator (operands[3], SFmode)) && (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 56 /* 0x38 */;
        }
      else
        {
	  return 0;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || (((cached_type = get_attr_type (insn)) == TYPE_FOP) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 56 /* 0x38 */;
        }
      else
        {
	  return 0;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || (((cached_type = get_attr_type (insn)) == TYPE_FOP) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 56 /* 0x38 */;
        }
      else
        {
	  return 0;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || ((! (mult_operator (operands[3], DFmode))) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((mult_operator (operands[3], DFmode)) && (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || ((! (mult_operator (operands[3], SFmode))) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((mult_operator (operands[3], SFmode)) && (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || (((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], DFmode))) && (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || (((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], SFmode))) && (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 3;
        }

    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_STORE) || (memory_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      if (((cached_type = get_attr_type (insn)) == TYPE_ISHIFT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_ISHIFT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((cached_type == TYPE_ALU) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 493:  /* *ashlqi3_1_slp */
      if (((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_ISHIFT1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((cached_type == TYPE_ALU) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ISHIFT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_ISHIFT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((cached_type == TYPE_ALU) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((which_alternative == 2) || (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ISHIFT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_ISHIFT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((cached_type == TYPE_ALU) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((which_alternative != 0) || (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((which_alternative == 1) || (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ISHIFT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_ISHIFT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((cached_type == TYPE_ALU) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((which_alternative == 1) || (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 17 /* 0x11 */;
        }
      else
        {
	  return 19 /* 0x13 */;
        }

    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if ((! (incdec_operand (operands[1], QImode))) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((! (incdec_operand (operands[1], QImode))) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 3;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((which_alternative != 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((which_alternative == 5) || (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((which_alternative != 2) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((which_alternative == 2) || (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((which_alternative != 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if ((which_alternative == 3) || (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || ((which_alternative == 0) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x18)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || ((((1 << which_alternative) & 0x7)) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x78)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || ((((1 << which_alternative) & 0x7)) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) || ((((1 << which_alternative) & 0x7)) && (cached_memory == MEMORY_NONE)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (which_alternative == 1)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMOVX) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode)))
        {
	  return 0;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1606:  /* sse_stmxcsr */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 323:  /* *mulqi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 2;

    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 641:  /* simple_return_indirect_internal */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 1;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMOVX) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode))) || (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 2;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode))) || (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_STORE) || (memory_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1605:  /* sse_ldmxcsr */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 3;

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], VOIDmode))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 3;
        }
      else if ((which_alternative == 0) && (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 2010:  /* mfence_sse2 */
    case 2009:  /* *sse_sfence */
    case 2008:  /* *sse2_lfence */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1835:  /* avx_vzeroupper */
    case 1834:  /* *avx_vzeroall */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1607:  /* sse2_clflush */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 920:  /* *prefetch_3dnow */
    case 919:  /* *prefetch_sse */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 896:  /* *movdfcc_1 */
    case 894:  /* *movxfcc_1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return 0;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if ((((cached_type = get_attr_type (insn)) == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || ((cached_type == TYPE_ROTATE1) || ((cached_type == TYPE_ALU1) || (cached_type == TYPE_NEGNOT)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || ((cached_type == TYPE_ROTATE1) || ((cached_type == TYPE_ALU1) || (cached_type == TYPE_NEGNOT)))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_UNKNOWN)))))
        {
	  return 3;
        }
      else if (cached_type == TYPE_IMUL)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 17 /* 0x11 */;
        }
      else if ((cached_type == TYPE_IDIV) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 19 /* 0x13 */;
        }
      else if (((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_IMOVX) || ((cached_type == TYPE_INCDEC) || (cached_type == TYPE_SETCC)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_IMOVX) || ((cached_type == TYPE_INCDEC) || (cached_type == TYPE_SETCC)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_UNKNOWN)))))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_CALL) || ((cached_type == TYPE_CALLV) || (cached_type == TYPE_IBR)))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_POP) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 5;
        }
      else if ((cached_type == TYPE_LEA) || ((cached_type == TYPE_PUSH) || (cached_memory == MEMORY_STORE)))
        {
	  return 2;
        }
      else if (cached_type == TYPE_STR)
        {
	  return 10 /* 0xa */;
        }
      else if (((cached_type == TYPE_FOP) || ((cached_type == TYPE_FMOV) || ((cached_type == TYPE_FCMP) || (cached_type == TYPE_FISTP)))) && (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FMUL) && (cached_memory == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))
        {
	  return 56 /* 0x38 */;
        }
      else
        {
	  return 0;
        }

    default:
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    }
}

int
internal_dfa_insn_code_k6 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 129 /* 0x81 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 129 /* 0x81 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((which_alternative == 2) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if ((which_alternative == 9) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((which_alternative == 6) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if ((which_alternative == 5) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if ((which_alternative == 3) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if ((which_alternative == 10) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((which_alternative == 6) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1383:  /* sse2_storehpd */
    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if ((which_alternative == 4) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 967:  /* *movv2sf_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], VOIDmode))
        {
	  return 129 /* 0x81 */;
        }
      else if (get_attr_memory (insn) == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))
        {
	  return 132 /* 0x84 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))
        {
	  return 132 /* 0x84 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
      return 119 /* 0x77 */;

    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
      return 134 /* 0x86 */;

    case 1606:  /* sse_stmxcsr */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
      return 133 /* 0x85 */;

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 140 /* 0x8c */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else if ((mult_operator (operands[3], XFmode)) && (cached_memory == MEMORY_NONE))
        {
	  return 138 /* 0x8a */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 140 /* 0x8c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if ((! (mult_operator (operands[3], XFmode))) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else if ((mult_operator (operands[3], XFmode)) && (cached_memory == MEMORY_NONE))
        {
	  return 138 /* 0x8a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else if ((mult_operator (operands[3], DFmode)) && (cached_memory == MEMORY_NONE))
        {
	  return 138 /* 0x8a */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 140 /* 0x8c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else if ((mult_operator (operands[3], SFmode)) && (cached_memory == MEMORY_NONE))
        {
	  return 138 /* 0x8a */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 140 /* 0x8c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && (cached_memory == MEMORY_NONE))
        {
	  return 138 /* 0x8a */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 140 /* 0x8c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && (cached_memory == MEMORY_NONE))
        {
	  return 138 /* 0x8a */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 140 /* 0x8c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if ((! (mult_operator (operands[3], DFmode))) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else if ((mult_operator (operands[3], DFmode)) && (cached_memory == MEMORY_NONE))
        {
	  return 138 /* 0x8a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if ((! (mult_operator (operands[3], SFmode))) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else if ((mult_operator (operands[3], SFmode)) && (cached_memory == MEMORY_NONE))
        {
	  return 138 /* 0x8a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if (((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], DFmode))) && (cached_memory == MEMORY_NONE))
        {
	  return 138 /* 0x8a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if (((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], SFmode))) && (cached_memory == MEMORY_NONE))
        {
	  return 138 /* 0x8a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 120 /* 0x78 */;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      return 127 /* 0x7f */;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      return 128 /* 0x80 */;

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 119 /* 0x77 */;
        }
      else
        {
	  return 121 /* 0x79 */;
        }

    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 111 /* 0x6f */;
        }
      else if ((cached_memory == MEMORY_STORE) || (memory_operand (operands[1], VOIDmode)))
        {
	  return 113 /* 0x71 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 111 /* 0x6f */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 112 /* 0x70 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 113 /* 0x71 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 111 /* 0x6f */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 112 /* 0x70 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 113 /* 0x71 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 111 /* 0x6f */;
        }
      else if (((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 112 /* 0x70 */;
        }
      else if (((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 113 /* 0x71 */;
        }
      else if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      if (((cached_type = get_attr_type (insn)) == TYPE_ISHIFT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 111 /* 0x6f */;
        }
      else if ((cached_type == TYPE_ISHIFT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 112 /* 0x70 */;
        }
      else if ((cached_type == TYPE_ISHIFT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 113 /* 0x71 */;
        }
      else if ((cached_type == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if ((cached_type == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if ((cached_type == TYPE_ALU) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 493:  /* *ashlqi3_1_slp */
      if (((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 111 /* 0x6f */;
        }
      else if ((cached_type == TYPE_ISHIFT1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 112 /* 0x70 */;
        }
      else if ((cached_type == TYPE_ISHIFT1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 113 /* 0x71 */;
        }
      else if ((cached_type == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if ((cached_type == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if ((cached_type == TYPE_ALU) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ISHIFT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 111 /* 0x6f */;
        }
      else if ((cached_type == TYPE_ISHIFT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 112 /* 0x70 */;
        }
      else if ((cached_type == TYPE_ISHIFT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 113 /* 0x71 */;
        }
      else if ((cached_type == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if ((cached_type == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if ((cached_type == TYPE_ALU) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (which_alternative == 2)
        {
	  return 132 /* 0x84 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 111 /* 0x6f */;
        }
      else if (((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 112 /* 0x70 */;
        }
      else if (((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 113 /* 0x71 */;
        }
      else if (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (which_alternative == 1)
        {
	  return 132 /* 0x84 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 491:  /* *ashlhi3_1 */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ISHIFT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 111 /* 0x6f */;
        }
      else if ((cached_type == TYPE_ISHIFT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 112 /* 0x70 */;
        }
      else if ((cached_type == TYPE_ISHIFT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 113 /* 0x71 */;
        }
      else if ((cached_type == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if ((cached_type == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if ((cached_type == TYPE_ALU) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (which_alternative == 1)
        {
	  return 132 /* 0x84 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 111 /* 0x6f */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 112 /* 0x70 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 113 /* 0x71 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 117 /* 0x75 */;
        }
      else
        {
	  return 118 /* 0x76 */;
        }

    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 114 /* 0x72 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 323:  /* *mulqi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
      return 114 /* 0x72 */;

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if ((! (incdec_operand (operands[1], QImode))) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return 111 /* 0x6f */;
        }
      else if ((! (incdec_operand (operands[1], QImode))) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 112 /* 0x70 */;
        }
      else if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 119 /* 0x77 */;
        }
      else
        {
	  return 120 /* 0x78 */;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if ((which_alternative != 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if ((which_alternative != 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (which_alternative == 5)
        {
	  return 132 /* 0x84 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if ((which_alternative != 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if ((which_alternative != 2) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (which_alternative == 2)
        {
	  return 132 /* 0x84 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if ((which_alternative != 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if ((which_alternative != 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (which_alternative == 3)
        {
	  return 132 /* 0x84 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
      return 132 /* 0x84 */;

    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if ((which_alternative == 0) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 135 /* 0x87 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 111 /* 0x6f */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 112 /* 0x70 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 113 /* 0x71 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 119 /* 0x77 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 120 /* 0x78 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 121 /* 0x79 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((((1 << which_alternative) & 0x18)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((((1 << which_alternative) & 0x78)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 109:  /* *movdf_internal */
    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 129 /* 0x81 */;
        }
      else if (which_alternative == 1)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if ((cached_type == TYPE_IMOVX) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if ((cached_type == TYPE_IMOVX) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if ((cached_type == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((cached_type == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 119 /* 0x77 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 120 /* 0x78 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 121 /* 0x79 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode)))
        {
	  return 122 /* 0x7a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode)))
        {
	  return 123 /* 0x7b */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 124 /* 0x7c */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 125 /* 0x7d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 126 /* 0x7e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 124 /* 0x7c */;

    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 125 /* 0x7d */;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if ((cached_type == TYPE_IMOVX) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if ((cached_type == TYPE_IMOVX) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 121 /* 0x79 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((cached_type == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 132 /* 0x84 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 126 /* 0x7e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 132 /* 0x84 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 111 /* 0x6f */;
        }
      else if ((cached_memory == MEMORY_STORE) || (memory_operand (operands[1], VOIDmode)))
        {
	  return 113 /* 0x71 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1605:  /* sse_ldmxcsr */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 129 /* 0x81 */;

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], VOIDmode))
        {
	  return 129 /* 0x81 */;
        }
      else
        {
	  return 133 /* 0x85 */;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 129 /* 0x81 */;
        }
      else if (which_alternative == 0)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 135 /* 0x87 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 129 /* 0x81 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 129 /* 0x81 */;
        }
      else if ((which_alternative == 0) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 2010:  /* mfence_sse2 */
    case 2009:  /* *sse_sfence */
    case 2008:  /* *sse2_lfence */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1835:  /* avx_vzeroupper */
    case 1834:  /* *avx_vzeroall */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1607:  /* sse2_clflush */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 920:  /* *prefetch_3dnow */
    case 919:  /* *prefetch_sse */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 896:  /* *movdfcc_1 */
    case 894:  /* *movxfcc_1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return 618 /* 0x26a */;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 119 /* 0x77 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 120 /* 0x78 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if ((((cached_type = get_attr_type (insn)) == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || ((cached_type == TYPE_ROTATE1) || ((cached_type == TYPE_ALU1) || (cached_type == TYPE_NEGNOT)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 111 /* 0x6f */;
        }
      else if (((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || ((cached_type == TYPE_ROTATE1) || ((cached_type == TYPE_ALU1) || (cached_type == TYPE_NEGNOT)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 112 /* 0x70 */;
        }
      else if (((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || ((cached_type == TYPE_ROTATE1) || ((cached_type == TYPE_ALU1) || (cached_type == TYPE_NEGNOT)))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 113 /* 0x71 */;
        }
      else if (cached_type == TYPE_IMUL)
        {
	  return 114 /* 0x72 */;
        }
      else if ((cached_type == TYPE_IDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 117 /* 0x75 */;
        }
      else if ((cached_type == TYPE_IDIV) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 118 /* 0x76 */;
        }
      else if (((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_IMOVX) || ((cached_type == TYPE_INCDEC) || (cached_type == TYPE_SETCC)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 119 /* 0x77 */;
        }
      else if (((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_IMOVX) || ((cached_type == TYPE_INCDEC) || (cached_type == TYPE_SETCC)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 120 /* 0x78 */;
        }
      else if (((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_IMOVX) || ((cached_type == TYPE_INCDEC) || (cached_type == TYPE_SETCC)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 121 /* 0x79 */;
        }
      else if ((cached_type == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (nonimmediate_operand (operands[1], VOIDmode))))
        {
	  return 122 /* 0x7a */;
        }
      else if ((cached_type == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return 123 /* 0x7b */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 124 /* 0x7c */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 125 /* 0x7d */;
        }
      else if ((cached_type == TYPE_IMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 126 /* 0x7e */;
        }
      else if ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 127 /* 0x7f */;
        }
      else if (cached_type == TYPE_IBR)
        {
	  return 128 /* 0x80 */;
        }
      else if ((cached_type == TYPE_POP) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 130 /* 0x82 */;
        }
      else if (cached_type == TYPE_LEA)
        {
	  return 132 /* 0x84 */;
        }
      else if ((cached_type == TYPE_PUSH) || (cached_memory == MEMORY_STORE))
        {
	  return 133 /* 0x85 */;
        }
      else if (cached_type == TYPE_STR)
        {
	  return 134 /* 0x86 */;
        }
      else if (((cached_type == TYPE_FOP) || ((cached_type == TYPE_FMOV) || ((cached_type == TYPE_FCMP) || (cached_type == TYPE_FISTP)))) && (cached_memory == MEMORY_NONE))
        {
	  return 135 /* 0x87 */;
        }
      else if ((cached_type == TYPE_FMUL) && (cached_memory == MEMORY_NONE))
        {
	  return 138 /* 0x8a */;
        }
      else if ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))
        {
	  return 140 /* 0x8c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    default:
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 129 /* 0x81 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 133 /* 0x85 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    }
}

int
insn_default_latency_athlon (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_athlon_decode cached_athlon_decode ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (which_alternative != 0))
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 0;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((memory_operand (operands[1], DFmode)) || (get_attr_memory (insn) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xc)) && ((memory_operand (operands[1], DFmode)) || (get_attr_memory (insn) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0xc))
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x63)) && ((memory_operand (operands[1], DFmode)) || (get_attr_memory (insn) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x63))
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x5))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if (which_alternative == 0)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 4) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x2f))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 2;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (!((1 << which_alternative) & 0x7)))
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x6)) && ((memory_operand (operands[1], DFmode)) || (get_attr_memory (insn) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x6))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0x9))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 0;
        }
      else
        {
	  return 2;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 5) && ((memory_operand (operands[1], DFmode)) || (get_attr_memory (insn) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (which_alternative != 5)
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (which_alternative == 9)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x620))
        {
	  return 2;
        }
      else if (which_alternative == 5)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (which_alternative == 5)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0x13))
        {
	  return 2;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (which_alternative == 3)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (which_alternative == 4)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0x9))
        {
	  return 2;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1391:  /* *vec_concatv2df */
    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && ((memory_operand (operands[1], DFmode)) || (get_attr_memory (insn) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (which_alternative == 10)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (memory_operand (operands[1], DFmode)))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x3b)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x6c1))
        {
	  return 2;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (which_alternative == 10)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (memory_operand (operands[1], DFmode)))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x39)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x6c1))
        {
	  return 2;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 0;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x5))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if (which_alternative == 0)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (memory_operand (operands[1], DFmode)))
        {
	  return 0;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x2f))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 2;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((memory_operand (operands[1], DFmode)) || (get_attr_memory (insn) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (which_alternative == 1)
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 1588:  /* *vec_concatv4si */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((memory_operand (operands[1], DFmode)) || (get_attr_memory (insn) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 1328:  /* sse2_cvtps2pd */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1322:  /* sse2_cvtsd2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 12 /* 0xc */;
        }
      else
        {
	  return 0;
        }

    case 1325:  /* *sse2_cvtpd2ps */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1318:  /* *sse2_cvtpd2dq */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_NONE))
        {
	  return 8;
        }
      else
        {
	  return 0;
        }

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 1302:  /* sse2_cvtpi2pd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (get_attr_memory (insn) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 9;
        }
      else
        {
	  return 0;
        }

    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 14 /* 0xe */;
        }
      else
        {
	  return 0;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
      return 20 /* 0x14 */;

    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
      return 39 /* 0x27 */;

    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
      return 5;

    case 1661:  /* sse4a_vmmovntv4sf */
    case 1659:  /* sse4a_movntsf */
    case 1601:  /* avx2_pmovmskb */
    case 1346:  /* sse_movss */
    case 1106:  /* sse2_movntidi */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 0;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 1;
        }
      else
        {
	  return 2;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((which_alternative == 5) && (memory_operand (operands[1], DFmode))) || ((((1 << which_alternative) & 0x30)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0x30))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x11)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x11))
        {
	  return 2;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (which_alternative == 4)
        {
	  return 2;
        }
      else if ((which_alternative == 3) && (memory_operand (operands[1], DFmode)))
        {
	  return 0;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x5))
        {
	  return 2;
        }
      else if (which_alternative == 1)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (which_alternative != 0))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 972:  /* *mmx_mulv2sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 3;
        }

    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1287:  /* sse_cvttps2pi */
    case 1286:  /* sse_cvtps2pi */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 993:  /* *vec_dupv2sf */
    case 992:  /* mmx_pswapdv2sf2 */
    case 991:  /* mmx_floatv2si2 */
    case 990:  /* mmx_pi2fw */
    case 989:  /* mmx_pf2iw */
    case 988:  /* mmx_pf2id */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 969:  /* sse_movntq */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 0;
        }
      else
        {
	  return 2;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 2;
        }
      else if (((!((1 << which_alternative) & 0xc7f)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || ((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0xc71))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x3f))
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0x300)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((!((1 << which_alternative) & 0xf00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x2c7))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xfc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0xfc))
        {
	  return 2;
        }
      else if (which_alternative == 9)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0x640)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((!((1 << which_alternative) & 0x1e40)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x6271))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x3f))
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && (memory_operand (operands[1], DFmode)))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x1c7))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xfc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0xfc))
        {
	  return 2;
        }
      else if (which_alternative == 8)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 7;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 894:  /* *movxfcc_1 */
      return 7;

    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 6;

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1679:  /* avx2_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1670:  /* sse4_1_blendpd */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1660:  /* sse4a_movntdf */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1602:  /* sse2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1599:  /* avx_movmskpd256 */
    case 1598:  /* sse_movmskps */
    case 1597:  /* avx_movmskps256 */
    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1352:  /* *vec_concatv4sf */
    case 1344:  /* sse_storelps */
    case 1343:  /* sse_loadhps */
    case 1342:  /* sse_storehps */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
    case 1110:  /* sse2_movntv2df */
    case 1109:  /* avx_movntv4df */
    case 1108:  /* sse_movntv4sf */
    case 1107:  /* avx_movntv8sf */
    case 1105:  /* sse2_movntisi */
    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
    case 1102:  /* sse2_storedqu */
    case 1101:  /* avx_storedqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1099:  /* avx_loaddqu256 */
    case 1098:  /* sse2_storeupd */
    case 1097:  /* avx_storeupd256 */
    case 1096:  /* sse_storeups */
    case 1095:  /* avx_storeups256 */
    case 1094:  /* sse2_loadupd */
    case 1093:  /* avx_loadupd256 */
    case 1092:  /* sse_loadups */
    case 1091:  /* avx_loadups256 */
    case 1089:  /* sse2_movq128 */
    case 852:  /* movmsk_df */
      extract_insn_cached (insn);
      if ((memory_operand (operands[1], DFmode)) || (get_attr_memory (insn) == MEMORY_LOAD))
        {
	  return 0;
        }
      else
        {
	  return 2;
        }

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 100 /* 0x64 */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || (mult_operator (operands[3], XFmode)))
        {
	  return 4;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 24 /* 0x18 */;
        }
      else
        {
	  return 0;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 24 /* 0x18 */;
        }
      else
        {
	  return 0;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 24 /* 0x18 */;
        }
      else
        {
	  return 0;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)
        {
	  return 4;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 4;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)
        {
	  return 4;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 4;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 4;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 24 /* 0x18 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 4;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 24 /* 0x18 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if (! (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else
        {
	  return 4;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if (! (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else
        {
	  return 4;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) || ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) || ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 491:  /* *ashlhi3_1 */
    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 9;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 5;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 323:  /* *mulqi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else if ((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 322:  /* *mulhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else if (((which_alternative == 0) && (! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((which_alternative != 0) || (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else if (((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((which_alternative != 1) && ((which_alternative != 2) || (! (memory_operand (operands[1], VOIDmode)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) || ((which_alternative == 1) || ((which_alternative == 2) && (memory_operand (operands[1], VOIDmode))))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative != 1) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 9;
        }
      else if ((which_alternative != 1) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((!((1 << which_alternative) & 0xb)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 9;
        }
      else if ((!((1 << which_alternative) & 0xb)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 12 /* 0xc */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 10 /* 0xa */;
        }
      else
        {
	  return 2;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 12 /* 0xc */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 10 /* 0xa */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 14 /* 0xe */;
        }
      else
        {
	  return 0;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 0;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else
        {
	  return 2;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 12 /* 0xc */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 0;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 10 /* 0xa */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else
        {
	  return 2;
        }

    case 1323:  /* sse2_cvtss2sd */
    case 810:  /* sse4_1_rounddf2 */
    case 136:  /* *extendsfdf2_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 2;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 12 /* 0xc */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && (which_alternative == 1))
        {
	  return 10 /* 0xa */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else
        {
	  return 2;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if (((!((1 << which_alternative) & 0x1f)) && (memory_operand (operands[1], DFmode))) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else
        {
	  return 2;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0xf)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if (((!((1 << which_alternative) & 0xce3f)) && (memory_operand (operands[1], DFmode))) || (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (((!((1 << which_alternative) & 0x3f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x3f))))
        {
	  return 2;
        }
      else if (which_alternative == 5)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x23f))
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if (((!((1 << which_alternative) & 0xff)) && (memory_operand (operands[1], DFmode))) || (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((((1 << which_alternative) & 0x1858)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (((!((1 << which_alternative) & 0xff)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) || (!((1 << which_alternative) & 0xff))))
        {
	  return 2;
        }
      else if (which_alternative == 7)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 12 /* 0xc */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 10 /* 0xa */;
        }
      else
        {
	  return 2;
        }

    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 1;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((cached_type == TYPE_IMOV) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 0;
        }
      else if (((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0xfb8))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3c)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x3c))
        {
	  return 2;
        }
      else if (which_alternative == 6)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0xaa0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((!((1 << which_alternative) & 0x1ea0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x6227))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x1c)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x1c))
        {
	  return 2;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0xa00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 0;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((!((1 << which_alternative) & 0xa01)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0x7de0))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x1f0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x1f0))
        {
	  return 2;
        }
      else if (which_alternative == 9)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 1088:  /* *movv2df_internal */
    case 1087:  /* *movv4df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1085:  /* *movv8sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1083:  /* *movv2ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1081:  /* *movv4di_internal */
    case 1080:  /* *movv4si_internal */
    case 1079:  /* *movv8si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1075:  /* *movv32qi_internal */
    case 61:  /* *movti_internal_sse */
    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || (get_attr_memory (insn) == MEMORY_LOAD)))
        {
	  return 0;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else
        {
	  return 3;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1890:  /* vec_set_hi_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1849:  /* avx2_vec_dupv4df */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1731:  /* sse4_2_pcmpestr_cconly */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
    case 1691:  /* sse4_1_phminposuw */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1510:  /* *andnotv2di3 */
    case 1509:  /* *andnotv4di3 */
    case 1508:  /* *andnotv4si3 */
    case 1507:  /* *andnotv8si3 */
    case 1506:  /* *andnotv8hi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1503:  /* *andnotv32qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1390:  /* vec_dupv2df */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1339:  /* avx_shufps256_1 */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1333:  /* avx_unpcklps256 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1331:  /* avx_unpckhps256 */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      return 3;

    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 761:  /* *fop_xf_comm_i387 */
    case 742:  /* *fop_df_comm_i387 */
    case 741:  /* *fop_sf_comm_i387 */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      return 4;

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 3;
        }
      else
        {
	  return 4;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 920:  /* *prefetch_3dnow */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 853:  /* cld */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
    case 42:  /* *pushdi2_rex64 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
      return 2;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_IBR) || ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV)))
        {
	  return 0;
        }
      else if (cached_type == TYPE_PUSH)
        {
	  return 2;
        }
      else if (cached_type == TYPE_POP)
        {
	  return 4;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 3;
        }
      else if (cached_type == TYPE_LEA)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))))
        {
	  return 6;
        }
      else if (((cached_athlon_decode = get_attr_athlon_decode (insn)) == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 1;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 4;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 6;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 1;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 12 /* 0xc */;
        }
      else if ((cached_type == TYPE_FMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 10 /* 0xa */;
        }
      else if (((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((cached_type == TYPE_FISTP) || (cached_type == TYPE_FISTTP)))
        {
	  return 4;
        }
      else if (cached_type == TYPE_FMOV)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FOP) || (cached_type == TYPE_FMUL))
        {
	  return 4;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 2;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_type == TYPE_FPSPC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 103 /* 0x67 */;
        }
      else if (cached_type == TYPE_FPSPC)
        {
	  return 100 /* 0x64 */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 7;
        }
      else if (((cached_type == TYPE_FCMP) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && (cached_type == TYPE_FCMP)))
        {
	  return 3;
        }
      else if (cached_type == TYPE_FCMP)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((memory_operand (operands[1], DFmode)) || ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 0;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 0;
        }
      else if ((((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_type == TYPE_SSEMOV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI)))) || ((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_MMXMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_MMXMUL) || (((cached_unit = get_attr_unit (insn)) == UNIT_MMX) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if (cached_unit == UNIT_MMX)
        {
	  return 2;
        }
      else if (((cached_type == TYPE_SSELOG1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_SSECMP) && (((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI))))))
        {
	  return 2;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 3;
        }
      else if ((cached_type == TYPE_SSECOMI) || ((cached_type == TYPE_SSEADD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI)))))
        {
	  return 4;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((cached_mode = get_attr_mode (insn)) == MODE_DF)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI)))))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 11 /* 0xb */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 14 /* 0xe */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 12 /* 0xc */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_NONE)))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSEICVT) && (((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_SSEMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 4;
        }
      else if (cached_type == TYPE_SSEMUL)
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 20 /* 0x14 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 39 /* 0x27 */;
        }
      else
        {
	  return 0;
        }

    default:
      return 0;

    }
}

int
internal_dfa_insn_code_athlon (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_athlon_decode cached_athlon_decode ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 218 /* 0xda */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xc)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x13))
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0xc))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x63)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x1c))
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0x63))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 2)
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 1)
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0x5))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (which_alternative == 0)
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 4)
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0x2f))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 235 /* 0xeb */;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 230 /* 0xe6 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x6)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0x6))
        {
	  return 231 /* 0xe7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 3)
        {
	  return 230 /* 0xe6 */;
        }
      else if (((1 << which_alternative) & 0x9))
        {
	  return 231 /* 0xe7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 218 /* 0xda */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 220 /* 0xdc */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 230 /* 0xe6 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
      return 244 /* 0xf4 */;

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 6)
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 5) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative != 5)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (which_alternative == 9)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x620))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (which_alternative == 5)
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (which_alternative == 5)
        {
	  return 189 /* 0xbd */;
        }
      else if ((((1 << which_alternative) & 0x13)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((((1 << which_alternative) & 0x13)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x13))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (which_alternative == 3)
        {
	  return 189 /* 0xbd */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 231 /* 0xe7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (which_alternative == 4)
        {
	  return 189 /* 0xbd */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x9))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 164 /* 0xa4 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (which_alternative == 10)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x3b)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 218 /* 0xda */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x6c1))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (which_alternative == 10)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x39)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 218 /* 0xda */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x6c1))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 218 /* 0xda */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x5))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (which_alternative == 0)
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 218 /* 0xda */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x2f))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 235 /* 0xeb */;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 1)
        {
	  return 230 /* 0xe6 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 220 /* 0xdc */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 230 /* 0xe6 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 220 /* 0xdc */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 1)
        {
	  return 230 /* 0xe6 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1352:  /* *vec_concatv4sf */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 220 /* 0xdc */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 230 /* 0xe6 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1328:  /* sse2_cvtps2pd */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 269 /* 0x10d */;
        }
      else
        {
	  return 271 /* 0x10f */;
        }

    case 1322:  /* sse2_cvtsd2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 282 /* 0x11a */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 284 /* 0x11c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1325:  /* *sse2_cvtpd2ps */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1318:  /* *sse2_cvtpd2dq */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 286 /* 0x11e */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 288 /* 0x120 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 275 /* 0x113 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1302:  /* sse2_cvtpi2pd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (get_attr_memory (insn) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (which_alternative == 0)
        {
	  return 235 /* 0xeb */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 290 /* 0x122 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 292 /* 0x124 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 275 /* 0x113 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 280 /* 0x118 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1890:  /* vec_set_hi_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1510:  /* *andnotv2di3 */
    case 1509:  /* *andnotv4di3 */
    case 1508:  /* *andnotv4si3 */
    case 1507:  /* *andnotv8si3 */
    case 1506:  /* *andnotv8hi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1503:  /* *andnotv32qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1339:  /* avx_shufps256_1 */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1333:  /* avx_unpcklps256 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1331:  /* avx_unpckhps256 */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 245 /* 0xf5 */;
        }
      else
        {
	  return 248 /* 0xf8 */;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 306 /* 0x132 */;
        }
      else
        {
	  return 308 /* 0x134 */;
        }

    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 309 /* 0x135 */;
        }
      else
        {
	  return 312 /* 0x138 */;
        }

    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 297 /* 0x129 */;
        }
      else
        {
	  return 299 /* 0x12b */;
        }

    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 300 /* 0x12c */;
        }
      else
        {
	  return 303 /* 0x12f */;
        }

    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 259 /* 0x103 */;
        }
      else
        {
	  return 262 /* 0x106 */;
        }

    case 1661:  /* sse4a_vmmovntv4sf */
    case 1659:  /* sse4a_movntsf */
    case 1601:  /* avx2_pmovmskb */
    case 1346:  /* sse_movss */
    case 1106:  /* sse2_movntidi */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 218 /* 0xda */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1672:  /* sse4_1_blendvps */
    case 1670:  /* sse4_1_blendpd */
    case 1668:  /* sse4_1_blendps */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1600:  /* sse2_movmskpd */
    case 1598:  /* sse_movmskps */
    case 1110:  /* sse2_movntv2df */
    case 1108:  /* sse_movntv4sf */
    case 1104:  /* sse3_lddqu */
    case 1102:  /* sse2_storedqu */
    case 1100:  /* sse2_loaddqu */
    case 1098:  /* sse2_storeupd */
    case 1096:  /* sse_storeups */
    case 1094:  /* sse2_loadupd */
    case 1092:  /* sse_loadups */
    case 1089:  /* sse2_movq128 */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 217 /* 0xd9 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else
        {
	  return 230 /* 0xe6 */;
        }

    case 1088:  /* *movv2df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1087:  /* *movv4df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1086:  /* *movv4sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1084:  /* *movv1ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1083:  /* *movv2ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1082:  /* *movv2di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1081:  /* *movv4di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1080:  /* *movv4si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1079:  /* *movv8si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1078:  /* *movv8hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1077:  /* *movv16hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1076:  /* *movv16qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 5) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x30))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x11)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x11))
        {
	  return 235 /* 0xeb */;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (which_alternative == 4)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 3) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 218 /* 0xda */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x5))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (which_alternative == 1)
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 234 /* 0xea */;
        }
      else
        {
	  return 235 /* 0xeb */;
        }

    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 972:  /* *mmx_mulv2sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 232 /* 0xe8 */;
        }
      else
        {
	  return 233 /* 0xe9 */;
        }

    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1287:  /* sse_cvttps2pi */
    case 1286:  /* sse_cvtps2pi */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 993:  /* *vec_dupv2sf */
    case 992:  /* mmx_pswapdv2sf2 */
    case 991:  /* mmx_floatv2si2 */
    case 990:  /* mmx_pi2fw */
    case 989:  /* mmx_pf2iw */
    case 988:  /* mmx_pf2id */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 234 /* 0xea */;
        }
      else
        {
	  return 235 /* 0xeb */;
        }

    case 969:  /* sse_movntq */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 220 /* 0xdc */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 7)
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0xc71))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x3f))
        {
	  return 235 /* 0xeb */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((((1 << which_alternative) & 0x300)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((!((1 << which_alternative) & 0xf00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 218 /* 0xda */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 8)
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0x2c7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xfc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0xfc))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (which_alternative == 9)
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((((1 << which_alternative) & 0x640)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((!((1 << which_alternative) & 0x1e40)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 218 /* 0xda */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 10)
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0x6271))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x3f))
        {
	  return 235 /* 0xeb */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 218 /* 0xda */;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x1c7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xfc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0xfc))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 8) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (which_alternative == 8)
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
      return 239 /* 0xef */;

    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 920:  /* *prefetch_3dnow */
      return 235 /* 0xeb */;

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 256 /* 0x100 */;
        }
      else
        {
	  return 258 /* 0x102 */;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else
        {
	  return 204 /* 0xcc */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 203 /* 0xcb */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 204 /* 0xcc */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 894:  /* *movxfcc_1 */
      return 204 /* 0xcc */;

    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
      return 162 /* 0xa2 */;

    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 161 /* 0xa1 */;

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1685:  /* avx2_pblendvb */
    case 1679:  /* avx2_movntdqa */
    case 1673:  /* avx_blendvpd256 */
    case 1671:  /* avx_blendvps256 */
    case 1669:  /* avx_blendpd256 */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1660:  /* sse4a_movntdf */
    case 1602:  /* sse2_pmovmskb */
    case 1599:  /* avx_movmskpd256 */
    case 1597:  /* avx_movmskps256 */
    case 1109:  /* avx_movntv4df */
    case 1107:  /* avx_movntv8sf */
    case 1105:  /* sse2_movntisi */
    case 1103:  /* avx_lddqu256 */
    case 1101:  /* avx_storedqu256 */
    case 1099:  /* avx_loaddqu256 */
    case 1097:  /* avx_storeupd256 */
    case 1095:  /* avx_storeups256 */
    case 1093:  /* avx_loadupd256 */
    case 1091:  /* avx_loadups256 */
    case 852:  /* movmsk_df */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 220 /* 0xdc */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 202 /* 0xca */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 190 /* 0xbe */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((mult_operator (operands[3], XFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 193 /* 0xc1 */;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 197 /* 0xc5 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 199 /* 0xc7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], XFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 190 /* 0xbe */;
        }
      else if (! (mult_operator (operands[3], XFmode)))
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 193 /* 0xc1 */;
        }
      else
        {
	  return 195 /* 0xc3 */;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 190 /* 0xbe */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 193 /* 0xc1 */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 197 /* 0xc5 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 199 /* 0xc7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 190 /* 0xbe */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 193 /* 0xc1 */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 197 /* 0xc5 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 199 /* 0xc7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 256 /* 0x100 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 297 /* 0x129 */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 306 /* 0x132 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 256 /* 0x100 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 297 /* 0x129 */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 306 /* 0x132 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 190 /* 0xbe */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 193 /* 0xc1 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 197 /* 0xc5 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 199 /* 0xc7 */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 256 /* 0x100 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 297 /* 0x129 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 306 /* 0x132 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 190 /* 0xbe */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 193 /* 0xc1 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 197 /* 0xc5 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 199 /* 0xc7 */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 256 /* 0x100 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 297 /* 0x129 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 306 /* 0x132 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 190 /* 0xbe */;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 193 /* 0xc1 */;
        }
      else
        {
	  return 195 /* 0xc3 */;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 190 /* 0xbe */;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 193 /* 0xc1 */;
        }
      else
        {
	  return 195 /* 0xc3 */;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 256 /* 0x100 */;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 258 /* 0x102 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 297 /* 0x129 */;
        }
      else
        {
	  return 299 /* 0x12b */;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 256 /* 0x100 */;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 258 /* 0x102 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 297 /* 0x129 */;
        }
      else
        {
	  return 299 /* 0x12b */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 190 /* 0xbe */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 192 /* 0xc0 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 193 /* 0xc1 */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 256 /* 0x100 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 297 /* 0x129 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 190 /* 0xbe */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 192 /* 0xc0 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 193 /* 0xc1 */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 256 /* 0x100 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 297 /* 0x129 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 167 /* 0xa7 */;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 148 /* 0x94 */;

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      return 142 /* 0x8e */;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      return 141 /* 0x8d */;

    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 242 /* 0xf2 */;
        }
      else
        {
	  return 244 /* 0xf4 */;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 162 /* 0xa2 */;
        }
      else
        {
	  return 176 /* 0xb0 */;
        }

    case 491:  /* *ashlhi3_1 */
    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 164 /* 0xa4 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 169 /* 0xa9 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 174 /* 0xae */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
      return 196 /* 0xc4 */;

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 159 /* 0x9f */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 160 /* 0xa0 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 152 /* 0x98 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 156 /* 0x9c */;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 152 /* 0x98 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 156 /* 0x9c */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 169 /* 0xa9 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 174 /* 0xae */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 323:  /* *mulqi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 152 /* 0x98 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 156 /* 0x9c */;
        }
      else if ((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && (cached_memory == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 322:  /* *mulhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 152 /* 0x98 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 156 /* 0x9c */;
        }
      else if (((which_alternative == 0) && (! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((which_alternative != 0) || (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 152 /* 0x98 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 156 /* 0x9c */;
        }
      else if (((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((which_alternative != 1) && ((which_alternative != 2) || (! (memory_operand (operands[1], VOIDmode)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) || ((which_alternative == 1) || ((which_alternative == 2) && (memory_operand (operands[1], VOIDmode))))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 162 /* 0xa2 */;
        }
      else
        {
	  return 167 /* 0xa7 */;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 492:  /* *ashlqi3_1 */
    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
      return 150 /* 0x96 */;

    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 275 /* 0x113 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 273 /* 0x111 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else if ((which_alternative != 1) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 275 /* 0x113 */;
        }
      else if ((which_alternative != 1) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 275 /* 0x113 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 275 /* 0x113 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 273 /* 0x111 */;
        }
      else if ((!((1 << which_alternative) & 0xb)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 275 /* 0x113 */;
        }
      else if ((!((1 << which_alternative) & 0xb)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 180 /* 0xb4 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 184 /* 0xb8 */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 180 /* 0xb4 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 184 /* 0xb8 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
      return 188 /* 0xbc */;

    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 275 /* 0x113 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 280 /* 0x118 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 182 /* 0xb6 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 186 /* 0xba */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 180 /* 0xb4 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 182 /* 0xb6 */;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 184 /* 0xb8 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 186 /* 0xba */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 1323:  /* sse2_cvtss2sd */
    case 810:  /* sse4_1_rounddf2 */
    case 136:  /* *extendsfdf2_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 265 /* 0x109 */;
        }
      else
        {
	  return 267 /* 0x10b */;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 180 /* 0xb4 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if (((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && (which_alternative == 1))
        {
	  return 184 /* 0xb8 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_LOAD))
        {
	  return 265 /* 0x109 */;
        }
      else
        {
	  return 267 /* 0x10b */;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 220 /* 0xdc */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 230 /* 0xe6 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0xf))
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 218 /* 0xda */;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0x3f))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (which_alternative == 5)
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0x23f))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((((1 << which_alternative) & 0x1858)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 218 /* 0xda */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0xff))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 7) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (which_alternative == 7)
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 180 /* 0xb4 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 184 /* 0xb8 */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 144 /* 0x90 */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 164 /* 0xa4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 164 /* 0xa4 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 174 /* 0xae */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 166 /* 0xa6 */;

    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 171 /* 0xab */;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_type == TYPE_IMOV) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 150 /* 0x96 */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if (((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 218 /* 0xda */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 7)
        {
	  return 230 /* 0xe6 */;
        }
      else if (((1 << which_alternative) & 0xfb8))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x3c)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x3c))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (which_alternative == 6)
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((((1 << which_alternative) & 0xaa0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((!((1 << which_alternative) & 0x1ea0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 218 /* 0xda */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x880))
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0x6227))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x1c)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x1c))
        {
	  return 235 /* 0xeb */;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 150 /* 0x96 */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((((1 << which_alternative) & 0xa00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((!((1 << which_alternative) & 0xa01)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 218 /* 0xda */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 11)
        {
	  return 230 /* 0xe6 */;
        }
      else if (((1 << which_alternative) & 0x7de0))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x1f0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x1f0))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if (which_alternative == 9)
        {
	  return 239 /* 0xef */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))))
        {
	  return 230 /* 0xe6 */;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 236 /* 0xec */;
        }
      else
        {
	  return 239 /* 0xef */;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 145 /* 0x91 */;

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      return 144 /* 0x90 */;

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 144 /* 0x90 */;
        }
      else
        {
	  return 164 /* 0xa4 */;
        }

    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 207 /* 0xcf */;
        }
      else
        {
	  return 209 /* 0xd1 */;
        }

    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 251 /* 0xfb */;
        }
      else
        {
	  return 254 /* 0xfe */;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 207 /* 0xcf */;
        }
      else if (which_alternative == 0)
        {
	  return 209 /* 0xd1 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 251 /* 0xfb */;
        }
      else
        {
	  return 254 /* 0xfe */;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 853:  /* cld */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
      return 164 /* 0xa4 */;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_IBR)
        {
	  return 141 /* 0x8d */;
        }
      else if ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 142 /* 0x8e */;
        }
      else if (cached_type == TYPE_PUSH)
        {
	  return 144 /* 0x90 */;
        }
      else if (cached_type == TYPE_POP)
        {
	  return 145 /* 0x91 */;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 148 /* 0x94 */;
        }
      else if (cached_type == TYPE_LEA)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 152 /* 0x98 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 156 /* 0x9c */;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 159 /* 0x9f */;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 160 /* 0xa0 */;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))))
        {
	  return 161 /* 0xa1 */;
        }
      else if (((cached_athlon_decode = get_attr_athlon_decode (insn)) == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 169 /* 0xa9 */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 174 /* 0xae */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 178 /* 0xb2 */;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 180 /* 0xb4 */;
        }
      else if ((cached_type == TYPE_FMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 182 /* 0xb6 */;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 184 /* 0xb8 */;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 186 /* 0xba */;
        }
      else if ((cached_type == TYPE_FISTP) || (cached_type == TYPE_FISTTP))
        {
	  return 188 /* 0xbc */;
        }
      else if (cached_type == TYPE_FMOV)
        {
	  return 189 /* 0xbd */;
        }
      else if ((cached_type == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 190 /* 0xbe */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_type == TYPE_FMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 193 /* 0xc1 */;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 195 /* 0xc3 */;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 196 /* 0xc4 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 197 /* 0xc5 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 199 /* 0xc7 */;
        }
      else if ((cached_type == TYPE_FPSPC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 201 /* 0xc9 */;
        }
      else if (cached_type == TYPE_FPSPC)
        {
	  return 202 /* 0xca */;
        }
      else if ((cached_type == TYPE_FCMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 203 /* 0xcb */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 204 /* 0xcc */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 207 /* 0xcf */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && (cached_type == TYPE_FCMP))
        {
	  return 209 /* 0xd1 */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 210 /* 0xd2 */;
        }
      else if (cached_type == TYPE_FCMP)
        {
	  return 212 /* 0xd4 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (memory_operand (operands[1], DFmode)))
        {
	  return 213 /* 0xd5 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 217 /* 0xd9 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 218 /* 0xda */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 220 /* 0xdc */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))))
        {
	  return 230 /* 0xe6 */;
        }
      else if ((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_type == TYPE_MMXMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 232 /* 0xe8 */;
        }
      else if (cached_type == TYPE_MMXMUL)
        {
	  return 233 /* 0xe9 */;
        }
      else if (((cached_unit = get_attr_unit (insn)) == UNIT_MMX) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (cached_unit == UNIT_MMX)
        {
	  return 235 /* 0xeb */;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 236 /* 0xec */;
        }
      else if ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1))
        {
	  return 239 /* 0xef */;
        }
      else if ((cached_type == TYPE_SSECMP) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 242 /* 0xf2 */;
        }
      else if ((cached_type == TYPE_SSECMP) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI)))))
        {
	  return 244 /* 0xf4 */;
        }
      else if ((cached_type == TYPE_SSECMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 245 /* 0xf5 */;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 248 /* 0xf8 */;
        }
      else if ((cached_type == TYPE_SSECOMI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 251 /* 0xfb */;
        }
      else if (cached_type == TYPE_SSECOMI)
        {
	  return 254 /* 0xfe */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 256 /* 0x100 */;
        }
      else if ((cached_type == TYPE_SSEADD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))))
        {
	  return 258 /* 0x102 */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 259 /* 0x103 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 262 /* 0x106 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 265 /* 0x109 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((cached_mode = get_attr_mode (insn)) == MODE_DF)))
        {
	  return 267 /* 0x10b */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 269 /* 0x10d */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI)))))
        {
	  return 271 /* 0x10f */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 273 /* 0x111 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 275 /* 0x113 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 278 /* 0x116 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 280 /* 0x118 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 282 /* 0x11a */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 284 /* 0x11c */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 286 /* 0x11e */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 288 /* 0x120 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 290 /* 0x122 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 292 /* 0x124 */;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 297 /* 0x129 */;
        }
      else if ((cached_type == TYPE_SSEMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 300 /* 0x12c */;
        }
      else if (cached_type == TYPE_SSEMUL)
        {
	  return 303 /* 0x12f */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 306 /* 0x132 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 308 /* 0x134 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 309 /* 0x135 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 312 /* 0x138 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    default:
      return 618 /* 0x26a */;

    }
}

int
insn_default_latency_k8 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  enum attr_athlon_decode cached_athlon_decode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && (((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || (which_alternative != 0))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xc)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0xc))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x63)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x63))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x5))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 4) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x2f))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (!((1 << which_alternative) & 0x7)))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x6)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x6))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x9))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 5)
        {
	  return 2;
        }
      else if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (!((1 << which_alternative) & 0x420))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 5)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0x33))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0x19))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) || ((!((1 << which_alternative) & 0x6c1)) && (memory_operand (operands[1], DFmode))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x3b)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if (!((1 << which_alternative) & 0x6c1))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) || ((!((1 << which_alternative) & 0x6c1)) && (memory_operand (operands[1], DFmode))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x39)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if (!((1 << which_alternative) & 0x6c1))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if (!((1 << which_alternative) & 0x5))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (memory_operand (operands[1], DFmode)))
        {
	  return 2;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if (!((1 << which_alternative) & 0x2f))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (which_alternative == 1)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1352:  /* *vec_concatv4sf */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1322:  /* sse2_cvtsd2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 12 /* 0xc */;
        }
      else
        {
	  return 0;
        }

    case 1325:  /* *sse2_cvtpd2ps */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1318:  /* *sse2_cvtpd2dq */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_NONE))
        {
	  return 8;
        }
      else
        {
	  return 0;
        }

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 1302:  /* sse2_cvtpi2pd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (get_attr_memory (insn) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 9;
        }
      else
        {
	  return 0;
        }

    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 14 /* 0xe */;
        }
      else
        {
	  return 0;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else
        {
	  return 20 /* 0x14 */;
        }

    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 35 /* 0x23 */;
        }
      else
        {
	  return 39 /* 0x27 */;
        }

    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 7;
        }
      else
        {
	  return 5;
        }

    case 1661:  /* sse4a_vmmovntv4sf */
    case 1659:  /* sse4a_movntsf */
    case 1601:  /* avx2_pmovmskb */
    case 1346:  /* sse_movss */
    case 1106:  /* sse2_movntidi */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 2;
        }
      else if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 1;
        }
      else
        {
	  return 2;
        }

    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1672:  /* sse4_1_blendvps */
    case 1670:  /* sse4_1_blendpd */
    case 1668:  /* sse4_1_blendps */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1600:  /* sse2_movmskpd */
    case 1598:  /* sse_movmskps */
    case 1110:  /* sse2_movntv2df */
    case 1108:  /* sse_movntv4sf */
    case 1104:  /* sse3_lddqu */
    case 1102:  /* sse2_storedqu */
    case 1100:  /* sse2_loaddqu */
    case 1098:  /* sse2_storeupd */
    case 1096:  /* sse_storeups */
    case 1094:  /* sse2_loadupd */
    case 1092:  /* sse_loadups */
    case 1089:  /* sse2_movq128 */
      extract_insn_cached (insn);
      if ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1088:  /* *movv2df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1087:  /* *movv4df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1086:  /* *movv4sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1084:  /* *movv1ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1083:  /* *movv2ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1082:  /* *movv2di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1081:  /* *movv4di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1080:  /* *movv4si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1079:  /* *movv8si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1078:  /* *movv8hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1077:  /* *movv16hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1076:  /* *movv16qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((which_alternative == 5) && (memory_operand (operands[1], DFmode))) || (((1 << which_alternative) & 0x30)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x11)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x11))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) || ((which_alternative == 3) && (memory_operand (operands[1], DFmode))))
        {
	  return 2;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x5))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 1)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || (which_alternative != 0))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 972:  /* *mmx_mulv2sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 3;
        }

    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1287:  /* sse_cvttps2pi */
    case 1286:  /* sse_cvtps2pi */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 993:  /* *vec_dupv2sf */
    case 992:  /* mmx_pswapdv2sf2 */
    case 991:  /* mmx_floatv2si2 */
    case 990:  /* mmx_pi2fw */
    case 989:  /* mmx_pf2iw */
    case 988:  /* mmx_pf2id */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 2;
        }
      else if (((!((1 << which_alternative) & 0xc7f)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || ((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0xc71))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x3f))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0x300)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((!((1 << which_alternative) & 0xf00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((((1 << which_alternative) & 0x300)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x2c7))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xfc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0xfc))
        {
	  return 2;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 9)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0x640)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((!((1 << which_alternative) & 0x1e40)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((((1 << which_alternative) & 0x640)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x6271))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x3f))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && (memory_operand (operands[1], DFmode)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if (!((1 << which_alternative) & 0x1c7))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xfc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0xfc))
        {
	  return 2;
        }
      else if ((which_alternative == 8) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 8)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 15 /* 0xf */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 17 /* 0x11 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 15 /* 0xf */;
        }
      else
        {
	  return 0;
        }

    case 894:  /* *movxfcc_1 */
      return 15 /* 0xf */;

    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 6;

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 100 /* 0x64 */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], XFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], XFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], XFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 491:  /* *ashlhi3_1 */
    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 9;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 7;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 328:  /* *mulsidi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 7;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 323:  /* *mulqi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if ((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 322:  /* *mulhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if (((which_alternative == 0) && (! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((which_alternative != 0) || (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 320:  /* *muldi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 7;
        }
      else if (((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((which_alternative != 1) && ((which_alternative != 2) || (! (memory_operand (operands[1], VOIDmode)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) || ((which_alternative == 1) || ((which_alternative == 2) && (memory_operand (operands[1], VOIDmode))))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if (((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((which_alternative != 1) && ((which_alternative != 2) || (! (memory_operand (operands[1], VOIDmode)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) || ((which_alternative == 1) || ((which_alternative == 2) && (memory_operand (operands[1], VOIDmode))))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative != 1) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 9;
        }
      else if ((which_alternative != 1) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 1) && (get_attr_memory (insn) == MEMORY_LOAD))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((!((1 << which_alternative) & 0xb)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 9;
        }
      else if ((!((1 << which_alternative) & 0xb)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 14 /* 0xe */;
        }
      else
        {
	  return 0;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1323:  /* sse2_cvtss2sd */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 810:  /* sse4_1_rounddf2 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 136:  /* *extendsfdf2_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 2;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 13 /* 0xd */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_LOAD))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && (which_alternative == 1))
        {
	  return 8;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else
        {
	  return 2;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if (((!((1 << which_alternative) & 0x1f)) && (memory_operand (operands[1], DFmode))) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0xf)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) || (((!((1 << which_alternative) & 0xce3f)) && (memory_operand (operands[1], DFmode))) || (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (((!((1 << which_alternative) & 0x3f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x3f))))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 5)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) || ((!((1 << which_alternative) & 0x23f)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x23f))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) || (((!((1 << which_alternative) & 0xff)) && (memory_operand (operands[1], DFmode))) || (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((((1 << which_alternative) & 0x1858)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (((!((1 << which_alternative) & 0xff)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) || (!((1 << which_alternative) & 0xff))))
        {
	  return 2;
        }
      else if ((which_alternative == 7) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 7)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 3;

    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 1;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((cached_type == TYPE_IMOV) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if (((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0xfb8))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3c)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x3c))
        {
	  return 2;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 6)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0xaa0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((!((1 << which_alternative) & 0x1ea0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((((1 << which_alternative) & 0xaa0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x6227))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x1c)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x1c))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0xa00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((!((1 << which_alternative) & 0xa01)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((((1 << which_alternative) & 0xa00)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x7de0))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x1f0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x1f0))
        {
	  return 2;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 9)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 2;
        }
      else if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 4;

    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1890:  /* vec_set_hi_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1510:  /* *andnotv2di3 */
    case 1509:  /* *andnotv4di3 */
    case 1508:  /* *andnotv4si3 */
    case 1507:  /* *andnotv8si3 */
    case 1506:  /* *andnotv8hi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1503:  /* *andnotv32qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1339:  /* avx_shufps256_1 */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1333:  /* avx_unpcklps256 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1331:  /* avx_unpckhps256 */
    case 1328:  /* sse2_cvtps2pd */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 3;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1685:  /* avx2_pblendvb */
    case 1679:  /* avx2_movntdqa */
    case 1673:  /* avx_blendvpd256 */
    case 1671:  /* avx_blendvps256 */
    case 1669:  /* avx_blendpd256 */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1660:  /* sse4a_movntdf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1602:  /* sse2_pmovmskb */
    case 1599:  /* avx_movmskpd256 */
    case 1597:  /* avx_movmskps256 */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1109:  /* avx_movntv4df */
    case 1107:  /* avx_movntv8sf */
    case 1105:  /* sse2_movntisi */
    case 1103:  /* avx_lddqu256 */
    case 1101:  /* avx_storedqu256 */
    case 1099:  /* avx_loaddqu256 */
    case 1097:  /* avx_storeupd256 */
    case 1095:  /* avx_storeups256 */
    case 1093:  /* avx_loadupd256 */
    case 1091:  /* avx_loadups256 */
    case 1090:  /* movdi_to_sse */
    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 969:  /* sse_movntq */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 920:  /* *prefetch_3dnow */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 853:  /* cld */
    case 852:  /* movmsk_df */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 127:  /* extendsidi2_1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
    case 42:  /* *pushdi2_rex64 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
      return 2;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_IBR) || ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV)))
        {
	  return 0;
        }
      else if (cached_type == TYPE_PUSH)
        {
	  return 2;
        }
      else if (cached_type == TYPE_POP)
        {
	  return 4;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 3;
        }
      else if (cached_type == TYPE_LEA)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))))
        {
	  return 7;
        }
      else if (((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))) || ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))))
        {
	  return 6;
        }
      else if (((cached_athlon_decode = get_attr_athlon_decode (insn)) == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 1;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 4;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 6;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 1;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_type == TYPE_FMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FISTP) || (cached_type == TYPE_FISTTP))
        {
	  return 4;
        }
      else if (cached_type == TYPE_FMOV)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 4;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else if ((cached_type == TYPE_FPSPC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 103 /* 0x67 */;
        }
      else if (cached_type == TYPE_FPSPC)
        {
	  return 100 /* 0x64 */;
        }
      else if ((cached_type == TYPE_FCMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 17 /* 0x11 */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 15 /* 0xf */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 5;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && (cached_type == TYPE_FCMP))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FCMP) || ((cached_type == TYPE_SSEMOV) && ((memory_operand (operands[1], DFmode)) || ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_type == TYPE_SSEMOV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI)))) || ((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_MMXMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_MMXMUL) || (((cached_unit = get_attr_unit (insn)) == UNIT_MMX) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if (cached_unit == UNIT_MMX)
        {
	  return 2;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_SSECMP) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECMP) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI)))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 3;
        }
      else if ((cached_type == TYPE_SSECOMI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSECOMI)
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEADD) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEADD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((cached_mode = get_attr_mode (insn)) == MODE_DF)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI)))))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 11 /* 0xb */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 14 /* 0xe */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 12 /* 0xc */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_NONE)))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSEICVT) && (((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if (cached_type == TYPE_SSEMUL)
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 22 /* 0x16 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 35 /* 0x23 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 39 /* 0x27 */;
        }
      else
        {
	  return 0;
        }

    default:
      return 0;

    }
}

int
internal_dfa_insn_code_k8 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  enum attr_athlon_decode cached_athlon_decode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xc)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x13))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0xc))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x63)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x1c))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x63))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 2)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 1)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x5))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 0)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 4)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x2f))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 235 /* 0xeb */;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 229 /* 0xe5 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x6)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x6))
        {
	  return 231 /* 0xe7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 3)
        {
	  return 229 /* 0xe5 */;
        }
      else if (((1 << which_alternative) & 0x9))
        {
	  return 231 /* 0xe7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 229 /* 0xe5 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 6)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 5) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative != 5)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 9)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x620))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 5)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 5)
        {
	  return 189 /* 0xbd */;
        }
      else if ((((1 << which_alternative) & 0x13)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x13)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x13))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 3)
        {
	  return 189 /* 0xbd */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 231 /* 0xe7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 4)
        {
	  return 189 /* 0xbd */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x9))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 164 /* 0xa4 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 10)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x3b)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x6c1))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 10)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x39)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x6c1))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x5))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 0)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x2f))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 235 /* 0xeb */;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 1)
        {
	  return 229 /* 0xe5 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 229 /* 0xe5 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 1)
        {
	  return 229 /* 0xe5 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1352:  /* *vec_concatv4sf */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 229 /* 0xe5 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1328:  /* sse2_cvtps2pd */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 269 /* 0x10d */;
        }
      else
        {
	  return 271 /* 0x10f */;
        }

    case 1322:  /* sse2_cvtsd2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 282 /* 0x11a */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 284 /* 0x11c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1325:  /* *sse2_cvtpd2ps */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1318:  /* *sse2_cvtpd2dq */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 286 /* 0x11e */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 288 /* 0x120 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 276 /* 0x114 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1302:  /* sse2_cvtpi2pd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (get_attr_memory (insn) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (which_alternative == 0)
        {
	  return 235 /* 0xeb */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 290 /* 0x122 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 293 /* 0x125 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 276 /* 0x114 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 280 /* 0x118 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1890:  /* vec_set_hi_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1510:  /* *andnotv2di3 */
    case 1509:  /* *andnotv4di3 */
    case 1508:  /* *andnotv4si3 */
    case 1507:  /* *andnotv8si3 */
    case 1506:  /* *andnotv8hi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1503:  /* *andnotv32qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1339:  /* avx_shufps256_1 */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1333:  /* avx_unpcklps256 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1331:  /* avx_unpckhps256 */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 246 /* 0xf6 */;
        }
      else
        {
	  return 249 /* 0xf9 */;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 307 /* 0x133 */;
        }
      else
        {
	  return 308 /* 0x134 */;
        }

    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 310 /* 0x136 */;
        }
      else
        {
	  return 313 /* 0x139 */;
        }

    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 298 /* 0x12a */;
        }
      else
        {
	  return 299 /* 0x12b */;
        }

    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 301 /* 0x12d */;
        }
      else
        {
	  return 304 /* 0x130 */;
        }

    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 260 /* 0x104 */;
        }
      else
        {
	  return 263 /* 0x107 */;
        }

    case 1661:  /* sse4a_vmmovntv4sf */
    case 1659:  /* sse4a_movntsf */
    case 1601:  /* avx2_pmovmskb */
    case 1346:  /* sse_movss */
    case 1106:  /* sse2_movntidi */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 219 /* 0xdb */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1672:  /* sse4_1_blendvps */
    case 1670:  /* sse4_1_blendpd */
    case 1668:  /* sse4_1_blendps */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1600:  /* sse2_movmskpd */
    case 1598:  /* sse_movmskps */
    case 1110:  /* sse2_movntv2df */
    case 1108:  /* sse_movntv4sf */
    case 1104:  /* sse3_lddqu */
    case 1102:  /* sse2_storedqu */
    case 1100:  /* sse2_loaddqu */
    case 1098:  /* sse2_storeupd */
    case 1096:  /* sse_storeups */
    case 1094:  /* sse2_loadupd */
    case 1092:  /* sse_loadups */
    case 1089:  /* sse2_movq128 */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 216 /* 0xd8 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 224 /* 0xe0 */;
        }
      else
        {
	  return 229 /* 0xe5 */;
        }

    case 1088:  /* *movv2df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1087:  /* *movv4df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1086:  /* *movv4sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1084:  /* *movv1ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1083:  /* *movv2ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1082:  /* *movv2di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1081:  /* *movv4di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1080:  /* *movv4si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1079:  /* *movv8si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1078:  /* *movv8hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1077:  /* *movv16hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1076:  /* *movv16qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 5) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x30))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x11)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x11))
        {
	  return 235 /* 0xeb */;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 4)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 3) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x5))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 1)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 234 /* 0xea */;
        }
      else
        {
	  return 235 /* 0xeb */;
        }

    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 972:  /* *mmx_mulv2sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 232 /* 0xe8 */;
        }
      else
        {
	  return 233 /* 0xe9 */;
        }

    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1287:  /* sse_cvttps2pi */
    case 1286:  /* sse_cvtps2pi */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 993:  /* *vec_dupv2sf */
    case 992:  /* mmx_pswapdv2sf2 */
    case 991:  /* mmx_floatv2si2 */
    case 990:  /* mmx_pi2fw */
    case 989:  /* mmx_pf2iw */
    case 988:  /* mmx_pf2id */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 234 /* 0xea */;
        }
      else
        {
	  return 235 /* 0xeb */;
        }

    case 969:  /* sse_movntq */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 7)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0xc71))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x3f))
        {
	  return 235 /* 0xeb */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((((1 << which_alternative) & 0x300)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((!((1 << which_alternative) & 0xf00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((((1 << which_alternative) & 0x300)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((((1 << which_alternative) & 0x300)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 8)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x2c7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xfc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0xfc))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 9)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((((1 << which_alternative) & 0x640)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((!((1 << which_alternative) & 0x1e40)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((((1 << which_alternative) & 0x640)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((((1 << which_alternative) & 0x640)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 10)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x6271))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x3f))
        {
	  return 235 /* 0xeb */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x1c7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xfc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0xfc))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 8) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 8)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
      return 240 /* 0xf0 */;

    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 920:  /* *prefetch_3dnow */
      return 235 /* 0xeb */;

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 257 /* 0x101 */;
        }
      else
        {
	  return 258 /* 0x102 */;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else
        {
	  return 206 /* 0xce */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 205 /* 0xcd */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 206 /* 0xce */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 894:  /* *movxfcc_1 */
      return 206 /* 0xce */;

    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
      return 162 /* 0xa2 */;

    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 161 /* 0xa1 */;

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1685:  /* avx2_pblendvb */
    case 1679:  /* avx2_movntdqa */
    case 1673:  /* avx_blendvpd256 */
    case 1671:  /* avx_blendvps256 */
    case 1669:  /* avx_blendpd256 */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1660:  /* sse4a_movntdf */
    case 1602:  /* sse2_pmovmskb */
    case 1599:  /* avx_movmskpd256 */
    case 1597:  /* avx_movmskps256 */
    case 1109:  /* avx_movntv4df */
    case 1107:  /* avx_movntv8sf */
    case 1105:  /* sse2_movntisi */
    case 1103:  /* avx_lddqu256 */
    case 1101:  /* avx_storedqu256 */
    case 1099:  /* avx_loaddqu256 */
    case 1097:  /* avx_storeupd256 */
    case 1095:  /* avx_storeups256 */
    case 1093:  /* avx_loadupd256 */
    case 1091:  /* avx_loadups256 */
    case 852:  /* movmsk_df */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 202 /* 0xca */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((mult_operator (operands[3], XFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], XFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (! (mult_operator (operands[3], XFmode)))
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 194 /* 0xc2 */;
        }
      else
        {
	  return 195 /* 0xc3 */;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 307 /* 0x133 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 307 /* 0x133 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 307 /* 0x133 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 307 /* 0x133 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 194 /* 0xc2 */;
        }
      else
        {
	  return 195 /* 0xc3 */;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 194 /* 0xc2 */;
        }
      else
        {
	  return 195 /* 0xc3 */;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 258 /* 0x102 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 298 /* 0x12a */;
        }
      else
        {
	  return 299 /* 0x12b */;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 258 /* 0x102 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 298 /* 0x12a */;
        }
      else
        {
	  return 299 /* 0x12b */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 192 /* 0xc0 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 192 /* 0xc0 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 167 /* 0xa7 */;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 149 /* 0x95 */;

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      return 142 /* 0x8e */;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      return 141 /* 0x8d */;

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 243 /* 0xf3 */;
        }
      else
        {
	  return 244 /* 0xf4 */;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 162 /* 0xa2 */;
        }
      else
        {
	  return 176 /* 0xb0 */;
        }

    case 491:  /* *ashlhi3_1 */
    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 164 /* 0xa4 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 169 /* 0xa9 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 174 /* 0xae */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
      return 196 /* 0xc4 */;

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 159 /* 0x9f */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 160 /* 0xa0 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 153 /* 0x99 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 157 /* 0x9d */;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 328:  /* *mulsidi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 158 /* 0x9e */;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 153 /* 0x99 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 157 /* 0x9d */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 158 /* 0x9e */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 169 /* 0xa9 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 174 /* 0xae */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 154 /* 0x9a */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 158 /* 0x9e */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 169 /* 0xa9 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 174 /* 0xae */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 323:  /* *mulqi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 158 /* 0x9e */;
        }
      else if ((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && (cached_memory == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 322:  /* *mulhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 158 /* 0x9e */;
        }
      else if (((which_alternative == 0) && (! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((which_alternative != 0) || (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 320:  /* *muldi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 153 /* 0x99 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 157 /* 0x9d */;
        }
      else if (((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((which_alternative != 1) && ((which_alternative != 2) || (! (memory_operand (operands[1], VOIDmode)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) || ((which_alternative == 1) || ((which_alternative == 2) && (memory_operand (operands[1], VOIDmode))))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 158 /* 0x9e */;
        }
      else if (((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((which_alternative != 1) && ((which_alternative != 2) || (! (memory_operand (operands[1], VOIDmode)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) || ((which_alternative == 1) || ((which_alternative == 2) && (memory_operand (operands[1], VOIDmode))))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 162 /* 0xa2 */;
        }
      else
        {
	  return 167 /* 0xa7 */;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 492:  /* *ashlqi3_1 */
    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
      return 150 /* 0x96 */;

    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 276 /* 0x114 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 273 /* 0x111 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else if ((which_alternative != 1) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 276 /* 0x114 */;
        }
      else if ((which_alternative != 1) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 276 /* 0x114 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 273 /* 0x111 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 276 /* 0x114 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 273 /* 0x111 */;
        }
      else if ((!((1 << which_alternative) & 0xb)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 276 /* 0x114 */;
        }
      else if ((!((1 << which_alternative) & 0xb)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 181 /* 0xb5 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 185 /* 0xb9 */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 181 /* 0xb5 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 185 /* 0xb9 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
      return 188 /* 0xbc */;

    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 276 /* 0x114 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 280 /* 0x118 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 183 /* 0xb7 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 187 /* 0xbb */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 181 /* 0xb5 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 185 /* 0xb9 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 187 /* 0xbb */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 1323:  /* sse2_cvtss2sd */
    case 810:  /* sse4_1_rounddf2 */
    case 136:  /* *extendsfdf2_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 265 /* 0x109 */;
        }
      else
        {
	  return 267 /* 0x10b */;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 181 /* 0xb5 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if (((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && (which_alternative == 1))
        {
	  return 185 /* 0xb9 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_LOAD))
        {
	  return 265 /* 0x109 */;
        }
      else
        {
	  return 267 /* 0x10b */;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 229 /* 0xe5 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0xf))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x3f))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 5)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x23f))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((((1 << which_alternative) & 0x1858)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0xff))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 7) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 7)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 181 /* 0xb5 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 185 /* 0xb9 */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 144 /* 0x90 */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 164 /* 0xa4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 164 /* 0xa4 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 174 /* 0xae */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 166 /* 0xa6 */;

    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 171 /* 0xab */;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_type == TYPE_IMOV) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 150 /* 0x96 */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if (((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 7)
        {
	  return 229 /* 0xe5 */;
        }
      else if (((1 << which_alternative) & 0xfb8))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x3c)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x3c))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 6)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((((1 << which_alternative) & 0xaa0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((!((1 << which_alternative) & 0x1ea0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((((1 << which_alternative) & 0xaa0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((((1 << which_alternative) & 0xaa0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x880))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x6227))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x1c)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x1c))
        {
	  return 235 /* 0xeb */;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 150 /* 0x96 */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((((1 << which_alternative) & 0xa00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((!((1 << which_alternative) & 0xa01)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((((1 << which_alternative) & 0xa00)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((((1 << which_alternative) & 0xa00)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 11)
        {
	  return 229 /* 0xe5 */;
        }
      else if (((1 << which_alternative) & 0x7de0))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x1f0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x1f0))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 9)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 145 /* 0x91 */;

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      return 144 /* 0x90 */;

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 144 /* 0x90 */;
        }
      else
        {
	  return 164 /* 0xa4 */;
        }

    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 208 /* 0xd0 */;
        }
      else
        {
	  return 209 /* 0xd1 */;
        }

    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 252 /* 0xfc */;
        }
      else
        {
	  return 254 /* 0xfe */;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 208 /* 0xd0 */;
        }
      else if (which_alternative == 0)
        {
	  return 209 /* 0xd1 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 252 /* 0xfc */;
        }
      else
        {
	  return 254 /* 0xfe */;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 853:  /* cld */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
      return 164 /* 0xa4 */;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_IBR)
        {
	  return 141 /* 0x8d */;
        }
      else if ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 142 /* 0x8e */;
        }
      else if (cached_type == TYPE_PUSH)
        {
	  return 144 /* 0x90 */;
        }
      else if (cached_type == TYPE_POP)
        {
	  return 145 /* 0x91 */;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 149 /* 0x95 */;
        }
      else if (cached_type == TYPE_LEA)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 153 /* 0x99 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))))
        {
	  return 157 /* 0x9d */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 158 /* 0x9e */;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 159 /* 0x9f */;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 160 /* 0xa0 */;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))))
        {
	  return 161 /* 0xa1 */;
        }
      else if (((cached_athlon_decode = get_attr_athlon_decode (insn)) == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 169 /* 0xa9 */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 174 /* 0xae */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 178 /* 0xb2 */;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 181 /* 0xb5 */;
        }
      else if ((cached_type == TYPE_FMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 185 /* 0xb9 */;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if ((cached_type == TYPE_FISTP) || (cached_type == TYPE_FISTTP))
        {
	  return 188 /* 0xbc */;
        }
      else if (cached_type == TYPE_FMOV)
        {
	  return 189 /* 0xbd */;
        }
      else if ((cached_type == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_type == TYPE_FMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 195 /* 0xc3 */;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 196 /* 0xc4 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else if ((cached_type == TYPE_FPSPC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 201 /* 0xc9 */;
        }
      else if (cached_type == TYPE_FPSPC)
        {
	  return 202 /* 0xca */;
        }
      else if ((cached_type == TYPE_FCMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 205 /* 0xcd */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 206 /* 0xce */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 208 /* 0xd0 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && (cached_type == TYPE_FCMP))
        {
	  return 209 /* 0xd1 */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 211 /* 0xd3 */;
        }
      else if (cached_type == TYPE_FCMP)
        {
	  return 212 /* 0xd4 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (memory_operand (operands[1], DFmode)))
        {
	  return 214 /* 0xd6 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))))
        {
	  return 229 /* 0xe5 */;
        }
      else if ((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_type == TYPE_MMXMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 232 /* 0xe8 */;
        }
      else if (cached_type == TYPE_MMXMUL)
        {
	  return 233 /* 0xe9 */;
        }
      else if (((cached_unit = get_attr_unit (insn)) == UNIT_MMX) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (cached_unit == UNIT_MMX)
        {
	  return 235 /* 0xeb */;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1))
        {
	  return 240 /* 0xf0 */;
        }
      else if ((cached_type == TYPE_SSECMP) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 243 /* 0xf3 */;
        }
      else if ((cached_type == TYPE_SSECMP) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI)))))
        {
	  return 244 /* 0xf4 */;
        }
      else if ((cached_type == TYPE_SSECMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 246 /* 0xf6 */;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 249 /* 0xf9 */;
        }
      else if ((cached_type == TYPE_SSECOMI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 252 /* 0xfc */;
        }
      else if (cached_type == TYPE_SSECOMI)
        {
	  return 254 /* 0xfe */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 257 /* 0x101 */;
        }
      else if ((cached_type == TYPE_SSEADD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))))
        {
	  return 258 /* 0x102 */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 260 /* 0x104 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 263 /* 0x107 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 265 /* 0x109 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((cached_mode = get_attr_mode (insn)) == MODE_DF)))
        {
	  return 267 /* 0x10b */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 269 /* 0x10d */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI)))))
        {
	  return 271 /* 0x10f */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 273 /* 0x111 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 276 /* 0x114 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 278 /* 0x116 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 280 /* 0x118 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 282 /* 0x11a */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 284 /* 0x11c */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 286 /* 0x11e */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 288 /* 0x120 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 290 /* 0x122 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 293 /* 0x125 */;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 298 /* 0x12a */;
        }
      else if ((cached_type == TYPE_SSEMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 301 /* 0x12d */;
        }
      else if (cached_type == TYPE_SSEMUL)
        {
	  return 304 /* 0x130 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 307 /* 0x133 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 308 /* 0x134 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 310 /* 0x136 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 313 /* 0x139 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    default:
      return 618 /* 0x26a */;

    }
}

int
insn_default_latency_core2 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  int cached_length_immediate ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1923:  /* avx2_lshrvv2di */
    case 1922:  /* avx2_ashlvv2di */
    case 1921:  /* avx2_lshrvv4di */
    case 1920:  /* avx2_ashlvv4di */
    case 1919:  /* avx2_lshrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1917:  /* avx2_lshrvv8si */
    case 1916:  /* avx2_ashlvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1914:  /* avx2_ashrvv8si */
    case 1798:  /* xop_shlv2di3 */
    case 1797:  /* xop_shlv4si3 */
    case 1796:  /* xop_shlv8hi3 */
    case 1795:  /* xop_shlv16qi3 */
    case 1794:  /* xop_shav2di3 */
    case 1793:  /* xop_shav4si3 */
    case 1792:  /* xop_shav8hi3 */
    case 1791:  /* xop_shav16qi3 */
    case 1790:  /* xop_vrotlv2di3 */
    case 1789:  /* xop_vrotlv4si3 */
    case 1788:  /* xop_vrotlv8hi3 */
    case 1787:  /* xop_vrotlv16qi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0xc)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xc)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xc)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x63)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x63)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x63)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x6)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x6)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x6))) || ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x6))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x6)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x6)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x6)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x9))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x9)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x9))) || ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x6))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x6)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x6)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x6)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1446:  /* ashrv4si3 */
    case 1445:  /* ashrv8si3 */
    case 1444:  /* ashrv8hi3 */
    case 1443:  /* ashrv16hi3 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (const_int_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_LOAD) && (const_int_operand (operands[2], VOIDmode))) || ((cached_memory == MEMORY_NONE) && (! (const_int_operand (operands[2], VOIDmode)))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (const_int_operand (operands[2], VOIDmode))))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 5))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 5))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 5))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 10))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 10))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (which_alternative == 10)) || (((cached_memory == MEMORY_NONE) && (which_alternative == 9)) || (((cached_memory == MEMORY_LOAD) && (which_alternative == 9)) || (((cached_memory == MEMORY_STORE) && (which_alternative == 9)) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x620)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x620)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x620)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 5)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 5)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 5)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x13)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x13)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x13)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 3)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 3)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 3)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 4)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 4)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x9)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x9)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x9)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1391:  /* *vec_concatv2df */
    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x7)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (which_alternative == 9)) || (((cached_memory == MEMORY_NONE) && (which_alternative == 10)) || (((cached_memory == MEMORY_LOAD) && (which_alternative == 10)) || (((cached_memory == MEMORY_STORE) && (which_alternative == 10)) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x6c1)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x6c1)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x6c1)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 4;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 4;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1588:  /* *vec_concatv4si */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x3)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1138:  /* sse2_divv2df3 */
      if (get_attr_memory (insn) == MEMORY_NONE)
        {
	  return 32 /* 0x20 */;
        }
      else
        {
	  return 0;
        }

    case 1139:  /* sse_vmdivv4sf3 */
    case 1136:  /* sse_divv4sf3 */
      if (get_attr_memory (insn) == MEMORY_NONE)
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 0;
        }

    case 1678:  /* sse4_1_dppd */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1132:  /* *mulv2df3 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case 1676:  /* sse4_1_dpps */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1130:  /* *mulv4sf3 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (which_alternative == 0)
        {
	  return 4;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 5))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 5))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 5))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 4)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 4)))))
        {
	  return 1;
        }
      else if (which_alternative == 0)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 3))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 3))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 972:  /* *mmx_mulv2sf3 */
      if (get_attr_memory (insn) == MEMORY_NONE)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1786:  /* xop_rotrv2di3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1779:  /* xop_rotlv16qi3 */
    case 1649:  /* ssse3_palignrdi */
    case 1648:  /* ssse3_palignrti */
    case 1647:  /* avx2_palignrv2ti */
    case 1631:  /* ssse3_pmaddubsw */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1436:  /* *avx2_pmaddwd */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0xc7f)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xc7f)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xc7f))) || ((which_alternative == 0) && (cached_memory == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 0))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3))) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x2ff))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x2ff)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x2ff))) || ((cached_memory == MEMORY_NONE) && (which_alternative == 2)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x627f)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x627f)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x627f))) || ((which_alternative == 0) && (cached_memory == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 0))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3))) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x1ff))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x1ff)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x1ff))) || ((cached_memory == MEMORY_NONE) && (which_alternative == 2)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode)))) || ((cached_memory == MEMORY_NONE) && ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode)))) || ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode)))) || ((cached_memory == MEMORY_NONE) && ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode)))) || ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x3))) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (!((1 << which_alternative) & 0x3))) || (((1 << which_alternative) & 0x3))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 894:  /* *movxfcc_1 */
      return 2;

    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1679:  /* avx2_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1670:  /* sse4_1_blendpd */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1602:  /* sse2_pmovmskb */
    case 1601:  /* avx2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1599:  /* avx_movmskpd256 */
    case 1598:  /* sse_movmskps */
    case 1597:  /* avx_movmskps256 */
    case 1576:  /* sse2_loadld */
    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1352:  /* *vec_concatv4sf */
    case 1346:  /* sse_movss */
    case 1344:  /* sse_storelps */
    case 1343:  /* sse_loadhps */
    case 1342:  /* sse_storehps */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
    case 1110:  /* sse2_movntv2df */
    case 1109:  /* avx_movntv4df */
    case 1108:  /* sse_movntv4sf */
    case 1107:  /* avx_movntv8sf */
    case 1106:  /* sse2_movntidi */
    case 1105:  /* sse2_movntisi */
    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
    case 1102:  /* sse2_storedqu */
    case 1101:  /* avx_storedqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1099:  /* avx_loaddqu256 */
    case 1098:  /* sse2_storeupd */
    case 1097:  /* avx_storeupd256 */
    case 1096:  /* sse_storeups */
    case 1095:  /* avx_storeups256 */
    case 1094:  /* sse2_loadupd */
    case 1093:  /* avx_loadupd256 */
    case 1092:  /* sse_loadups */
    case 1091:  /* avx_loadups256 */
    case 1089:  /* sse2_movq128 */
    case 852:  /* movmsk_df */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 38 /* 0x26 */;

    case 772:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 32 /* 0x20 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 33 /* 0x21 */;
        }
      else
        {
	  return 0;
        }

    case 771:  /* *fop_xf_6_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 767:  /* *fop_xf_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 19 /* 0x13 */;
        }
      else
        {
	  return 0;
        }

    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 38 /* 0x26 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 39 /* 0x27 */;
        }
      else
        {
	  return 0;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], XFmode)))) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 19 /* 0x13 */;
        }
      else
        {
	  return 0;
        }

    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 32 /* 0x20 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 33 /* 0x21 */;
        }
      else
        {
	  return 0;
        }

    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 19 /* 0x13 */;
        }
      else
        {
	  return 0;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))) || ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 32 /* 0x20 */;
        }
      else
        {
	  return 0;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))) || ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 0;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 32 /* 0x20 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 33 /* 0x21 */;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) || ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 32 /* 0x20 */;
        }
      else
        {
	  return 0;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 19 /* 0x13 */;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) || ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 0;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], DFmode)))) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], SFmode)))) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], DFmode)))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))) || ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], SFmode)))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))) || ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))) || ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (mult_operator (operands[3], DFmode))))
        {
	  return 6;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode))))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode))))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode)))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode)))))
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))) || ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (mult_operator (operands[3], SFmode))))
        {
	  return 6;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 0))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 6;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative == 0))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (! (cached_memory == MEMORY_NONE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 493:  /* *ashlqi3_1_slp */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 2) || ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT)))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative != 0) || ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT)))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 1) || ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 1) || ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT)))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x3)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x7)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 355:  /* udivmodhiqi3 */
    case 346:  /* divmodhiqi3 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else
        {
	  return 0;
        }

    case 353:  /* *udivmodsi4_noext */
    case 344:  /* *divmodsi4_noext */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 39 /* 0x27 */;
        }
      else
        {
	  return 0;
        }

    case 352:  /* *udivmodhi4_noext */
    case 343:  /* *divmodhi4_noext */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 23 /* 0x17 */;
        }
      else
        {
	  return 0;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (which_alternative == 0)))
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (which_alternative == 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 2010:  /* mfence_sse2 */
    case 2009:  /* *sse_sfence */
    case 2008:  /* *sse2_lfence */
    case 1607:  /* sse2_clflush */
    case 1311:  /* sse2_cvttsd2si */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1294:  /* sse_cvttss2si */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 780:  /* *sqrtdf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 745:  /* *rcpsf2_sse */
    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 328:  /* *mulsidi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      return 3;

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 5))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 5))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 5))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 2))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 3))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 3))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 3))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((1 << which_alternative) & 0x3))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else if (which_alternative == 1)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && (which_alternative != 0)) || ((! (cached_memory == MEMORY_NONE)) && (which_alternative != 0)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x3))) || ((! (cached_memory == MEMORY_NONE)) && (!((1 << which_alternative) & 0x3))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0xc))) || ((! (cached_memory == MEMORY_NONE)) && (((1 << which_alternative) & 0xc))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
    case 320:  /* *muldi3_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
      return 5;

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || ((cached_memory == MEMORY_LOAD) && (which_alternative == 0)))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x1f)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x1f)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x1f)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (which_alternative == 1)) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0xf))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xf)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xf)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x18)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x18)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x18))) || (((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7))) || (((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7))) || (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7))) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0xce3f)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xce3f)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xce3f)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7))) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((1 << which_alternative) & 0x7))) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (((1 << which_alternative) & 0x7))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x23f))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x23f)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x23f)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x78)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x78)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x78))) || (((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7))) || (((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7))) || (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7))) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0xff)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xff)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xff)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 6;
        }
      else
        {
	  return 1;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) || ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) || ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) || ((cached_type = get_attr_type (insn)) == TYPE_IMOVX)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) || ((cached_type = get_attr_type (insn)) == TYPE_IMOVX)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5))))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode))))) || ((cached_memory == MEMORY_NONE) && (((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode))) || (((1 << which_alternative) & 0xf80)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0xf80)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0xf80))) || ((cached_memory == MEMORY_NONE) && (which_alternative == 2)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x623f)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x623f)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x623f))) || ((which_alternative == 2) && (cached_memory == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode))))) || ((cached_memory == MEMORY_NONE) && (((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode))) || (((1 << which_alternative) & 0x7c00)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7c00)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7c00))) || ((cached_memory == MEMORY_NONE) && (which_alternative == 4)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 4))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x7)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1088:  /* *movv2df_internal */
    case 1087:  /* *movv4df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1085:  /* *movv8sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1083:  /* *movv2ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1081:  /* *movv4di_internal */
    case 1080:  /* *movv4si_internal */
    case 1079:  /* *movv8si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1075:  /* *movv32qi_internal */
    case 61:  /* *movti_internal_sse */
    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 0))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1338:  /* sse3_movsldup */
    case 1336:  /* sse3_movshdup */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1142:  /* sse_rcpv4sf2 */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 993:  /* *vec_dupv2sf */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 4;

    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 920:  /* *prefetch_3dnow */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      return 1;

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else
        {
	  return 1;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return 6;

    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_OTHER) || ((cached_type == TYPE_MULTI) || (cached_type == TYPE_STR)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_CALL) || ((cached_type == TYPE_CALLV) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_IMOV))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && (cached_type == TYPE_ICMOV)) || ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_ICMOV)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (cached_type == TYPE_PUSH)) || (((cached_memory == MEMORY_BOTH) && (cached_type == TYPE_PUSH)) || ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_LEA) || ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1))))))))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_IBR))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (cached_type == TYPE_IBR))
        {
	  return 6;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((cached_mode = get_attr_mode (insn)) == MODE_HI) || (cached_mode == MODE_SI)) && (cached_type == TYPE_IMUL))) || ((! (cached_memory == MEMORY_NONE)) && ((((cached_mode = get_attr_mode (insn)) == MODE_HI) || (cached_mode == MODE_SI)) && (cached_type == TYPE_IMUL))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_IMUL))) || ((! (cached_memory == MEMORY_NONE)) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_IMUL))))
        {
	  return 5;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_QI) && (cached_type == TYPE_IDIV))) || ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_QI) && (cached_type == TYPE_IDIV))))
        {
	  return 19 /* 0x13 */;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (cached_type == TYPE_IDIV))) || ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (cached_type == TYPE_IDIV))))
        {
	  return 23 /* 0x17 */;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_SI) && (cached_type == TYPE_IDIV))) || ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_SI) && (cached_type == TYPE_IDIV))))
        {
	  return 39 /* 0x27 */;
        }
      else if (cached_type == TYPE_FXCH)
        {
	  return 0;
        }
      else if (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && (cached_type == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_BOTH) && (cached_type == TYPE_FOP))
        {
	  return 5;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 1;
        }
      else if (cached_type == TYPE_FISTP)
        {
	  return 5;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FCMP))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FCMP))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FMOV)) || ((cached_memory == MEMORY_LOAD) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_XF)) && (cached_type == TYPE_FMOV))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && (cached_type == TYPE_FMOV)))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_XF)) && (cached_type == TYPE_FMOV)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_STORE) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && (cached_type == TYPE_FMOV)))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FMUL))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FMUL))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 32 /* 0x20 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 38 /* 0x26 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 39 /* 0x27 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_SSEIADD)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_SSEIADD)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_MMXSHFT))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_MMXSHFT))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_SSEISHFT) && (! ((cached_length_immediate = get_attr_length_immediate (insn)) == 0))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_SSEISHFT) && (! ((cached_length_immediate = get_attr_length_immediate (insn)) == 0)))) || ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_SSEISHFT) && ((cached_length_immediate = get_attr_length_immediate (insn)) == 0))))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_SSEISHFT) && ((cached_length_immediate = get_attr_length_immediate (insn)) == 0))) || ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_MMXMUL) || (cached_type == TYPE_SSEIMUL))))
        {
	  return 3;
        }
      else if (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_MMXCVT))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_UNKNOWN) && (cached_type == TYPE_SSE)) || (((cached_mode == MODE_SF) || (cached_mode == MODE_DF)) && (cached_type == TYPE_SSE)))
        {
	  return 3;
        }
      else if ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSE))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_SSEADD) || ((cached_type == TYPE_SSECMP) || (cached_type == TYPE_SSECOMI)))) || ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_SSEADD) || ((cached_type == TYPE_SSECMP) || (cached_type == TYPE_SSECOMI)))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEMUL))) || ((cached_memory == MEMORY_LOAD) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEMUL))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEMUL))) || ((cached_memory == MEMORY_LOAD) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEMUL))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEDIV)))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEDIV)))
        {
	  return 32 /* 0x20 */;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEICVT))) || (((! (cached_memory == MEMORY_NONE)) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEICVT))) || (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_DF) && (cached_type == TYPE_SSEICVT))) || ((! (cached_memory == MEMORY_NONE)) && ((cached_mode == MODE_DF) && (cached_type == TYPE_SSEICVT))))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SI) && (cached_type == TYPE_SSEICVT))) || ((! (cached_memory == MEMORY_NONE)) && ((cached_mode == MODE_SI) && (cached_type == TYPE_SSEICVT))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_SSEMOV))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_SSEMOV))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (cached_type == TYPE_SSEMOV)) || (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP))))))))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP))))))))))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP)))))))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP))))))))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    default:
      return 0;

    }
}

int
internal_dfa_insn_code_core2 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  int cached_length_immediate ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 610 /* 0x262 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 611 /* 0x263 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1923:  /* avx2_lshrvv2di */
    case 1922:  /* avx2_ashlvv2di */
    case 1921:  /* avx2_lshrvv4di */
    case 1920:  /* avx2_ashlvv4di */
    case 1919:  /* avx2_lshrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1917:  /* avx2_lshrvv8si */
    case 1916:  /* avx2_ashlvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1914:  /* avx2_ashrvv8si */
    case 1798:  /* xop_shlv2di3 */
    case 1797:  /* xop_shlv4si3 */
    case 1796:  /* xop_shlv8hi3 */
    case 1795:  /* xop_shlv16qi3 */
    case 1794:  /* xop_shav2di3 */
    case 1793:  /* xop_shav4si3 */
    case 1792:  /* xop_shav8hi3 */
    case 1791:  /* xop_shav16qi3 */
    case 1790:  /* xop_vrotlv2di3 */
    case 1789:  /* xop_vrotlv4si3 */
    case 1788:  /* xop_vrotlv8hi3 */
    case 1787:  /* xop_vrotlv16qi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 586 /* 0x24a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 587 /* 0x24b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2010:  /* mfence_sse2 */
    case 2009:  /* *sse_sfence */
    case 2008:  /* *sse2_lfence */
    case 1607:  /* sse2_clflush */
      return 591 /* 0x24f */;

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0xc)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xc)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xc)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x63)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x63)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x63)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 610 /* 0x262 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 611 /* 0x263 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x6)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x6)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x6)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x6)))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x6)))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x6)))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x6)))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 536 /* 0x218 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x9)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x9)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x9)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x6)))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x6)))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x6)))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x6)))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 532 /* 0x214 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 535 /* 0x217 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1786:  /* xop_rotrv2di3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1779:  /* xop_rotlv16qi3 */
    case 1649:  /* ssse3_palignrdi */
    case 1648:  /* ssse3_palignrti */
    case 1647:  /* avx2_palignrv2ti */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 584 /* 0x248 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 585 /* 0x249 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1446:  /* ashrv4si3 */
    case 1445:  /* ashrv8si3 */
    case 1444:  /* ashrv8hi3 */
    case 1443:  /* ashrv16hi3 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (const_int_operand (operands[2], VOIDmode)))
        {
	  return 584 /* 0x248 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (const_int_operand (operands[2], VOIDmode)))
        {
	  return 585 /* 0x249 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (! (const_int_operand (operands[2], VOIDmode))))
        {
	  return 586 /* 0x24a */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (const_int_operand (operands[2], VOIDmode))))
        {
	  return 587 /* 0x24b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 5))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 5))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 5))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 10))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 10))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 10))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 9))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x620)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x620)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x620)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 536 /* 0x218 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 5))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 5))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 5))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x13)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x13)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x13)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 536 /* 0x218 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 3))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 3))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 3))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 536 /* 0x218 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x9)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x9)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x9)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1391:  /* *vec_concatv2df */
    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x7)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 532 /* 0x214 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 9))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 10))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 10))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 10))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x6c1)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x6c1)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x6c1)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 590 /* 0x24e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 590 /* 0x24e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1588:  /* *vec_concatv4si */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x3)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1311:  /* sse2_cvttsd2si */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1294:  /* sse_cvttss2si */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 608 /* 0x260 */;
        }
      else
        {
	  return 609 /* 0x261 */;
        }

    case 1338:  /* sse3_movsldup */
    case 1336:  /* sse3_movshdup */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1142:  /* sse_rcpv4sf2 */
      return 593 /* 0x251 */;

    case 1140:  /* sse2_vmdivv2df3 */
    case 1138:  /* sse2_divv2df3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 602 /* 0x25a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1139:  /* sse_vmdivv4sf3 */
    case 1136:  /* sse_divv4sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 600 /* 0x258 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1678:  /* sse4_1_dppd */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1132:  /* *mulv2df3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 598 /* 0x256 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 599 /* 0x257 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1676:  /* sse4_1_dpps */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1130:  /* *mulv4sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 596 /* 0x254 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 597 /* 0x255 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 536 /* 0x218 */;
        }
      else if (which_alternative == 0)
        {
	  return 590 /* 0x24e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 5))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 5))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 5))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 582 /* 0x246 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 583 /* 0x247 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 536 /* 0x218 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 570 /* 0x23a */;
        }
      else if (which_alternative == 0)
        {
	  return 590 /* 0x24e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 3))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 3))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 3))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 590 /* 0x24e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 993:  /* *vec_dupv2sf */
      return 590 /* 0x24e */;

    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 972:  /* *mmx_mulv2sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 588 /* 0x24c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1631:  /* ssse3_pmaddubsw */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1436:  /* *avx2_pmaddwd */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 580 /* 0x244 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 581 /* 0x245 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0xc7f)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xc7f)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xc7f)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((which_alternative == 0) && (cached_memory == MEMORY_NONE))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 0))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x2ff)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x2ff)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x2ff)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x627f)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x627f)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x627f)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((which_alternative == 0) && (cached_memory == MEMORY_NONE))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 0))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x1ff)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x1ff)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x1ff)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode)))))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode)))))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 532 /* 0x214 */;
        }
      else
        {
	  return 564 /* 0x234 */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 537 /* 0x219 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x3)))
        {
	  return 538 /* 0x21a */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 564 /* 0x234 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 894:  /* *movxfcc_1 */
      return 564 /* 0x234 */;

    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 537 /* 0x219 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 538 /* 0x21a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 920:  /* *prefetch_3dnow */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
      return 613 /* 0x265 */;

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1679:  /* avx2_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1670:  /* sse4_1_blendpd */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1602:  /* sse2_pmovmskb */
    case 1601:  /* avx2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1599:  /* avx_movmskpd256 */
    case 1598:  /* sse_movmskps */
    case 1597:  /* avx_movmskps256 */
    case 1576:  /* sse2_loadld */
    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1352:  /* *vec_concatv4sf */
    case 1346:  /* sse_movss */
    case 1344:  /* sse_storelps */
    case 1343:  /* sse_loadhps */
    case 1342:  /* sse_storehps */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
    case 1110:  /* sse2_movntv2df */
    case 1109:  /* avx_movntv4df */
    case 1108:  /* sse_movntv4sf */
    case 1107:  /* avx_movntv8sf */
    case 1106:  /* sse2_movntidi */
    case 1105:  /* sse2_movntisi */
    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
    case 1102:  /* sse2_storedqu */
    case 1101:  /* avx_storedqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1099:  /* avx_loaddqu256 */
    case 1098:  /* sse2_storeupd */
    case 1097:  /* avx_storeupd256 */
    case 1096:  /* sse_storeups */
    case 1095:  /* avx_storeups256 */
    case 1094:  /* sse2_loadupd */
    case 1093:  /* avx_loadupd256 */
    case 1092:  /* sse_loadups */
    case 1091:  /* avx_loadups256 */
    case 1089:  /* sse2_movq128 */
    case 852:  /* movmsk_df */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 610 /* 0x262 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 611 /* 0x263 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 578 /* 0x242 */;

    case 772:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 576 /* 0x240 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 577 /* 0x241 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 771:  /* *fop_xf_6_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 767:  /* *fop_xf_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 574 /* 0x23e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 575 /* 0x23f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 578 /* 0x242 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 579 /* 0x243 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 574 /* 0x23e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 575 /* 0x23f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 576 /* 0x240 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 577 /* 0x241 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 574 /* 0x23e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 575 /* 0x23f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 598 /* 0x256 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 599 /* 0x257 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 602 /* 0x25a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 596 /* 0x254 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 597 /* 0x255 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 600 /* 0x258 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1172:  /* sse2_vmsminv2df3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 780:  /* *sqrtdf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 745:  /* *rcpsf2_sse */
      return 592 /* 0x250 */;

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 576 /* 0x240 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 577 /* 0x241 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 598 /* 0x256 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 599 /* 0x257 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 602 /* 0x25a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 574 /* 0x23e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 575 /* 0x23f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 596 /* 0x254 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 597 /* 0x255 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 600 /* 0x258 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 598 /* 0x256 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 599 /* 0x257 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 596 /* 0x254 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 597 /* 0x255 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (mult_operator (operands[3], DFmode))))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (mult_operator (operands[3], DFmode))))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode))))
        {
	  return 598 /* 0x256 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode))))
        {
	  return 599 /* 0x257 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (mult_operator (operands[3], SFmode))))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (mult_operator (operands[3], SFmode))))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode))))
        {
	  return 596 /* 0x254 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode))))
        {
	  return 597 /* 0x255 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 614 /* 0x266 */;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 0))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 0))
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 546 /* 0x222 */;

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      return 533 /* 0x215 */;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 544 /* 0x220 */;
        }
      else
        {
	  return 545 /* 0x221 */;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 613 /* 0x265 */;
        }
      else
        {
	  return 615 /* 0x267 */;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative == 0))
        {
	  return 543 /* 0x21f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 542 /* 0x21e */;
        }
      else if ((which_alternative == 1) && (! (cached_memory == MEMORY_NONE)))
        {
	  return 543 /* 0x21f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 493:  /* *ashlqi3_1_slp */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 491:  /* *ashlhi3_1 */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 542 /* 0x21e */;
        }
      else
        {
	  return 543 /* 0x21f */;
        }

    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
      return 562 /* 0x232 */;

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x3)))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x7)))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 355:  /* udivmodhiqi3 */
    case 346:  /* divmodhiqi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 551 /* 0x227 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 552 /* 0x228 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 353:  /* *udivmodsi4_noext */
    case 344:  /* *divmodsi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 555 /* 0x22b */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 556 /* 0x22c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 352:  /* *udivmodhi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 553 /* 0x229 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 554 /* 0x22a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 549 /* 0x225 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative == 0))
        {
	  return 550 /* 0x226 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 547 /* 0x223 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative == 0))
        {
	  return 548 /* 0x224 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
    case 320:  /* *muldi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 549 /* 0x225 */;
        }
      else
        {
	  return 550 /* 0x226 */;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 328:  /* *mulsidi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 547 /* 0x223 */;
        }
      else
        {
	  return 548 /* 0x224 */;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 613 /* 0x265 */;
        }
      else
        {
	  return 614 /* 0x266 */;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 5))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative != 5))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 5))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 5))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 5))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative != 2))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 2))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 2))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 2))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 3))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative != 3))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 3))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 3))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 3))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 613 /* 0x265 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 614 /* 0x266 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 615 /* 0x267 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
      return 541 /* 0x21d */;

    case 210:  /* *floatsidf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 606 /* 0x25e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (((1 << which_alternative) & 0x3)))
        {
	  return 607 /* 0x25f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 604 /* 0x25c */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (((1 << which_alternative) & 0x3)))
        {
	  return 605 /* 0x25d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 606 /* 0x25e */;
        }
      else if ((which_alternative == 1) && (! (cached_memory == MEMORY_NONE)))
        {
	  return 607 /* 0x25f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 604 /* 0x25c */;
        }
      else if ((which_alternative == 1) && (! (cached_memory == MEMORY_NONE)))
        {
	  return 605 /* 0x25d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 606 /* 0x25e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative != 0))
        {
	  return 607 /* 0x25f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 203:  /* *floatdisf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 604 /* 0x25c */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative != 0))
        {
	  return 605 /* 0x25d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 606 /* 0x25e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (!((1 << which_alternative) & 0x3)))
        {
	  return 607 /* 0x25f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 199:  /* *floatdisf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 604 /* 0x25c */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (!((1 << which_alternative) & 0x3)))
        {
	  return 605 /* 0x25d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0xc)))
        {
	  return 606 /* 0x25e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (((1 << which_alternative) & 0xc)))
        {
	  return 607 /* 0x25f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0xc)))
        {
	  return 604 /* 0x25c */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (((1 << which_alternative) & 0xc)))
        {
	  return 605 /* 0x25d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 567 /* 0x237 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 569 /* 0x239 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 571 /* 0x23b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 532 /* 0x214 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 567 /* 0x237 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 569 /* 0x239 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 571 /* 0x23b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
      return 563 /* 0x233 */;

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 606 /* 0x25e */;
        }
      else
        {
	  return 607 /* 0x25f */;
        }

    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 211:  /* *floatsisf2_vector_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 154:  /* fix_truncsfdi_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 604 /* 0x25c */;
        }
      else
        {
	  return 605 /* 0x25d */;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 532 /* 0x214 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 567 /* 0x237 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 568 /* 0x238 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 570 /* 0x23a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 567 /* 0x237 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 568 /* 0x238 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 570 /* 0x23a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 569 /* 0x239 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 571 /* 0x23b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 569 /* 0x239 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 571 /* 0x23b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 613 /* 0x265 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 614 /* 0x266 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 615 /* 0x267 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_BOTH))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x1f)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x1f)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x1f)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0xf)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xf)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xf)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
      return 557 /* 0x22d */;

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x18)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x18)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x18)))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0xce3f)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xce3f)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xce3f)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x23f)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x23f)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x23f)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x78)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x78)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x78)))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0xff)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xff)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xff)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 532 /* 0x214 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 567 /* 0x237 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 569 /* 0x239 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 571 /* 0x23b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 532 /* 0x214 */;
        }
      else if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 539 /* 0x21b */;
        }
      else
        {
	  return 540 /* 0x21c */;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) || ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) || ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 534 /* 0x216 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 535 /* 0x217 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 534 /* 0x216 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 535 /* 0x217 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 535 /* 0x217 */;

    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 536 /* 0x218 */;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) || ((cached_type = get_attr_type (insn)) == TYPE_IMOVX)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) || ((cached_type = get_attr_type (insn)) == TYPE_IMOVX)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))))
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5))))))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5))))))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode))))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0xf80)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0xf80)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0xf80)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x623f)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x623f)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x623f)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_NONE))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode))))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7c00)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7c00)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7c00)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 4))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x7)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1088:  /* *movv2df_internal */
    case 1087:  /* *movv4df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1085:  /* *movv8sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1083:  /* *movv2ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1081:  /* *movv4di_internal */
    case 1080:  /* *movv4si_internal */
    case 1079:  /* *movv8si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1075:  /* *movv32qi_internal */
    case 61:  /* *movti_internal_sse */
    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 0))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 0))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 613 /* 0x265 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 615 /* 0x267 */;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 614 /* 0x266 */;
        }
      else
        {
	  return 616 /* 0x268 */;
        }

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 539 /* 0x21b */;
        }
      else
        {
	  return 540 /* 0x21c */;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 532 /* 0x214 */;
        }
      else if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 539 /* 0x21b */;
        }
      else
        {
	  return 540 /* 0x21c */;
        }

    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 565 /* 0x235 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 566 /* 0x236 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 594 /* 0x252 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 595 /* 0x253 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 565 /* 0x235 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 566 /* 0x236 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 594 /* 0x252 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 595 /* 0x253 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return 532 /* 0x214 */;

    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 613 /* 0x265 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 614 /* 0x266 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_OTHER) || ((cached_type == TYPE_MULTI) || (cached_type == TYPE_STR)))
        {
	  return 532 /* 0x214 */;
        }
      else if ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 533 /* 0x215 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_IMOV))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_ICMOV))
        {
	  return 537 /* 0x219 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_ICMOV))
        {
	  return 538 /* 0x21a */;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_PUSH))
        {
	  return 539 /* 0x21b */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (cached_type == TYPE_PUSH))
        {
	  return 540 /* 0x21c */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_LEA))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1)))))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1)))))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_IBR))
        {
	  return 544 /* 0x220 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (cached_type == TYPE_IBR))
        {
	  return 545 /* 0x221 */;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 546 /* 0x222 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((cached_mode = get_attr_mode (insn)) == MODE_HI) || (cached_mode == MODE_SI)) && (cached_type == TYPE_IMUL)))
        {
	  return 547 /* 0x223 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((((cached_mode = get_attr_mode (insn)) == MODE_HI) || (cached_mode == MODE_SI)) && (cached_type == TYPE_IMUL)))
        {
	  return 548 /* 0x224 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_IMUL)))
        {
	  return 549 /* 0x225 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_IMUL)))
        {
	  return 550 /* 0x226 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_QI) && (cached_type == TYPE_IDIV)))
        {
	  return 551 /* 0x227 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_QI) && (cached_type == TYPE_IDIV)))
        {
	  return 552 /* 0x228 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (cached_type == TYPE_IDIV)))
        {
	  return 553 /* 0x229 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (cached_type == TYPE_IDIV)))
        {
	  return 554 /* 0x22a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_SI) && (cached_type == TYPE_IDIV)))
        {
	  return 555 /* 0x22b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_SI) && (cached_type == TYPE_IDIV)))
        {
	  return 556 /* 0x22c */;
        }
      else if (cached_type == TYPE_FXCH)
        {
	  return 557 /* 0x22d */;
        }
      else if (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && (cached_type == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (cached_type == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 562 /* 0x232 */;
        }
      else if (cached_type == TYPE_FISTP)
        {
	  return 563 /* 0x233 */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 564 /* 0x234 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FCMP))
        {
	  return 565 /* 0x235 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FCMP))
        {
	  return 566 /* 0x236 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FMOV))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_XF)) && (cached_type == TYPE_FMOV)))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && (cached_type == TYPE_FMOV)))
        {
	  return 569 /* 0x239 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_XF)) && (cached_type == TYPE_FMOV)))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && (cached_type == TYPE_FMOV)))
        {
	  return 571 /* 0x23b */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FMUL))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FMUL))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 574 /* 0x23e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 575 /* 0x23f */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 576 /* 0x240 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 577 /* 0x241 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 578 /* 0x242 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 579 /* 0x243 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_SSEIADD)))
        {
	  return 580 /* 0x244 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_SSEIADD)))
        {
	  return 581 /* 0x245 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_MMXSHFT))
        {
	  return 582 /* 0x246 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_MMXSHFT))
        {
	  return 583 /* 0x247 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_SSEISHFT) && (! ((cached_length_immediate = get_attr_length_immediate (insn)) == 0))))
        {
	  return 584 /* 0x248 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_SSEISHFT) && (! ((cached_length_immediate = get_attr_length_immediate (insn)) == 0))))
        {
	  return 585 /* 0x249 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_SSEISHFT) && ((cached_length_immediate = get_attr_length_immediate (insn)) == 0)))
        {
	  return 586 /* 0x24a */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_SSEISHFT) && ((cached_length_immediate = get_attr_length_immediate (insn)) == 0)))
        {
	  return 587 /* 0x24b */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_MMXMUL) || (cached_type == TYPE_SSEIMUL)))
        {
	  return 588 /* 0x24c */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_MMXMUL) || (cached_type == TYPE_SSEIMUL)))
        {
	  return 589 /* 0x24d */;
        }
      else if (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_MMXCVT))
        {
	  return 590 /* 0x24e */;
        }
      else if ((cached_memory == MEMORY_UNKNOWN) && (cached_type == TYPE_SSE))
        {
	  return 591 /* 0x24f */;
        }
      else if (((cached_mode == MODE_SF) || (cached_mode == MODE_DF)) && (cached_type == TYPE_SSE))
        {
	  return 592 /* 0x250 */;
        }
      else if ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSE))
        {
	  return 593 /* 0x251 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_SSEADD) || ((cached_type == TYPE_SSECMP) || (cached_type == TYPE_SSECOMI))))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_SSEADD) || ((cached_type == TYPE_SSECMP) || (cached_type == TYPE_SSECOMI))))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEMUL)))
        {
	  return 596 /* 0x254 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEMUL)))
        {
	  return 597 /* 0x255 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEMUL)))
        {
	  return 598 /* 0x256 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEMUL)))
        {
	  return 599 /* 0x257 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEDIV)))
        {
	  return 600 /* 0x258 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEDIV)))
        {
	  return 601 /* 0x259 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEDIV)))
        {
	  return 602 /* 0x25a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEDIV)))
        {
	  return 603 /* 0x25b */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEICVT)))
        {
	  return 604 /* 0x25c */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEICVT)))
        {
	  return 605 /* 0x25d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_DF) && (cached_type == TYPE_SSEICVT)))
        {
	  return 606 /* 0x25e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_mode == MODE_DF) && (cached_type == TYPE_SSEICVT)))
        {
	  return 607 /* 0x25f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SI) && (cached_type == TYPE_SSEICVT)))
        {
	  return 608 /* 0x260 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_mode == MODE_SI) && (cached_type == TYPE_SSEICVT)))
        {
	  return 609 /* 0x261 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_SSEMOV))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_SSEMOV))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_SSEMOV))
        {
	  return 612 /* 0x264 */;
        }
      else if (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP)))))))))))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP))))))))))))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP)))))))))))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP))))))))))))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    default:
      return 618 /* 0x26a */;

    }
}

int
insn_default_latency_corei7 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  int cached_length_immediate ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1923:  /* avx2_lshrvv2di */
    case 1922:  /* avx2_ashlvv2di */
    case 1921:  /* avx2_lshrvv4di */
    case 1920:  /* avx2_ashlvv4di */
    case 1919:  /* avx2_lshrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1917:  /* avx2_lshrvv8si */
    case 1916:  /* avx2_ashlvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1914:  /* avx2_ashrvv8si */
    case 1798:  /* xop_shlv2di3 */
    case 1797:  /* xop_shlv4si3 */
    case 1796:  /* xop_shlv8hi3 */
    case 1795:  /* xop_shlv16qi3 */
    case 1794:  /* xop_shav2di3 */
    case 1793:  /* xop_shav4si3 */
    case 1792:  /* xop_shav8hi3 */
    case 1791:  /* xop_shav16qi3 */
    case 1790:  /* xop_vrotlv2di3 */
    case 1789:  /* xop_vrotlv4si3 */
    case 1788:  /* xop_vrotlv8hi3 */
    case 1787:  /* xop_vrotlv16qi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0xc)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xc)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xc)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x63)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x63)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x63)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x6)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x6)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x6))) || ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x6))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x6)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x6)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x6)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x9))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x9)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x9))) || ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x6))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x6)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x6)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x6)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1446:  /* ashrv4si3 */
    case 1445:  /* ashrv8si3 */
    case 1444:  /* ashrv8hi3 */
    case 1443:  /* ashrv16hi3 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (const_int_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_LOAD) && (const_int_operand (operands[2], VOIDmode))) || ((cached_memory == MEMORY_NONE) && (! (const_int_operand (operands[2], VOIDmode)))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (const_int_operand (operands[2], VOIDmode))))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 5))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 5))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 5))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 10))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 10))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (which_alternative == 10)) || (((cached_memory == MEMORY_NONE) && (which_alternative == 9)) || (((cached_memory == MEMORY_LOAD) && (which_alternative == 9)) || (((cached_memory == MEMORY_STORE) && (which_alternative == 9)) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x620)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x620)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x620)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 5)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 5)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 5)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x13)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x13)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x13)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 3)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 3)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 3)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 4)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 4)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x9)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x9)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x9)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1391:  /* *vec_concatv2df */
    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x7)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (which_alternative == 9)) || (((cached_memory == MEMORY_NONE) && (which_alternative == 10)) || (((cached_memory == MEMORY_LOAD) && (which_alternative == 10)) || (((cached_memory == MEMORY_STORE) && (which_alternative == 10)) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x6c1)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x6c1)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x6c1)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 4;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 4;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1588:  /* *vec_concatv4si */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x3)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1138:  /* sse2_divv2df3 */
      if (get_attr_memory (insn) == MEMORY_NONE)
        {
	  return 32 /* 0x20 */;
        }
      else
        {
	  return 0;
        }

    case 1139:  /* sse_vmdivv4sf3 */
    case 1136:  /* sse_divv4sf3 */
      if (get_attr_memory (insn) == MEMORY_NONE)
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 0;
        }

    case 1678:  /* sse4_1_dppd */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1132:  /* *mulv2df3 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case 1676:  /* sse4_1_dpps */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1130:  /* *mulv4sf3 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (which_alternative == 0)
        {
	  return 4;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 5))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 5))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 5))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 4)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 4)))))
        {
	  return 1;
        }
      else if (which_alternative == 0)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 3))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 3))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 972:  /* *mmx_mulv2sf3 */
      if (get_attr_memory (insn) == MEMORY_NONE)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1786:  /* xop_rotrv2di3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1779:  /* xop_rotlv16qi3 */
    case 1649:  /* ssse3_palignrdi */
    case 1648:  /* ssse3_palignrti */
    case 1647:  /* avx2_palignrv2ti */
    case 1631:  /* ssse3_pmaddubsw */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1436:  /* *avx2_pmaddwd */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0xc7f)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xc7f)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xc7f))) || ((which_alternative == 0) && (cached_memory == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 0))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3))) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x2ff))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x2ff)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x2ff))) || ((cached_memory == MEMORY_NONE) && (which_alternative == 2)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x627f)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x627f)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x627f))) || ((which_alternative == 0) && (cached_memory == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 0))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3))) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x1ff))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x1ff)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x1ff))) || ((cached_memory == MEMORY_NONE) && (which_alternative == 2)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode)))) || ((cached_memory == MEMORY_NONE) && ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode)))) || ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode)))) || ((cached_memory == MEMORY_NONE) && ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode)))) || ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x3))) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (!((1 << which_alternative) & 0x3))) || (((1 << which_alternative) & 0x3))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 894:  /* *movxfcc_1 */
      return 2;

    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1679:  /* avx2_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1670:  /* sse4_1_blendpd */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1602:  /* sse2_pmovmskb */
    case 1601:  /* avx2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1599:  /* avx_movmskpd256 */
    case 1598:  /* sse_movmskps */
    case 1597:  /* avx_movmskps256 */
    case 1576:  /* sse2_loadld */
    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1352:  /* *vec_concatv4sf */
    case 1346:  /* sse_movss */
    case 1344:  /* sse_storelps */
    case 1343:  /* sse_loadhps */
    case 1342:  /* sse_storehps */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
    case 1110:  /* sse2_movntv2df */
    case 1109:  /* avx_movntv4df */
    case 1108:  /* sse_movntv4sf */
    case 1107:  /* avx_movntv8sf */
    case 1106:  /* sse2_movntidi */
    case 1105:  /* sse2_movntisi */
    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
    case 1102:  /* sse2_storedqu */
    case 1101:  /* avx_storedqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1099:  /* avx_loaddqu256 */
    case 1098:  /* sse2_storeupd */
    case 1097:  /* avx_storeupd256 */
    case 1096:  /* sse_storeups */
    case 1095:  /* avx_storeups256 */
    case 1094:  /* sse2_loadupd */
    case 1093:  /* avx_loadupd256 */
    case 1092:  /* sse_loadups */
    case 1091:  /* avx_loadups256 */
    case 1089:  /* sse2_movq128 */
    case 852:  /* movmsk_df */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 38 /* 0x26 */;

    case 772:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 32 /* 0x20 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 33 /* 0x21 */;
        }
      else
        {
	  return 0;
        }

    case 771:  /* *fop_xf_6_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 767:  /* *fop_xf_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 19 /* 0x13 */;
        }
      else
        {
	  return 0;
        }

    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 38 /* 0x26 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 39 /* 0x27 */;
        }
      else
        {
	  return 0;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], XFmode)))) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 19 /* 0x13 */;
        }
      else
        {
	  return 0;
        }

    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 32 /* 0x20 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 33 /* 0x21 */;
        }
      else
        {
	  return 0;
        }

    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 19 /* 0x13 */;
        }
      else
        {
	  return 0;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))) || ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 32 /* 0x20 */;
        }
      else
        {
	  return 0;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))) || ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 0;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 32 /* 0x20 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 33 /* 0x21 */;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) || ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 32 /* 0x20 */;
        }
      else
        {
	  return 0;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP)) || ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 19 /* 0x13 */;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) || ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 18 /* 0x12 */;
        }
      else
        {
	  return 0;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], DFmode)))) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], SFmode)))) || ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], DFmode)))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode))) || ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], SFmode)))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode))) || ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))) || ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (mult_operator (operands[3], DFmode))))
        {
	  return 6;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode))))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode))))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode)))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode)))))
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))) || ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (mult_operator (operands[3], SFmode))))
        {
	  return 6;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))) || ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 0))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 6;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative == 0))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (! (cached_memory == MEMORY_NONE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 493:  /* *ashlqi3_1_slp */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 2) || ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT)))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative != 0) || ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT)))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 1) || ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 1) || ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT)))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x3)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x7)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 355:  /* udivmodhiqi3 */
    case 346:  /* divmodhiqi3 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 19 /* 0x13 */;
        }
      else
        {
	  return 0;
        }

    case 353:  /* *udivmodsi4_noext */
    case 344:  /* *divmodsi4_noext */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 39 /* 0x27 */;
        }
      else
        {
	  return 0;
        }

    case 352:  /* *udivmodhi4_noext */
    case 343:  /* *divmodhi4_noext */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 23 /* 0x17 */;
        }
      else
        {
	  return 0;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (which_alternative == 0)))
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (which_alternative == 0)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 2010:  /* mfence_sse2 */
    case 2009:  /* *sse_sfence */
    case 2008:  /* *sse2_lfence */
    case 1607:  /* sse2_clflush */
    case 1311:  /* sse2_cvttsd2si */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1294:  /* sse_cvttss2si */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 780:  /* *sqrtdf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 745:  /* *rcpsf2_sse */
    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 328:  /* *mulsidi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      return 3;

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 5))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 5))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 5))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 2))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 3))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 3))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 3))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((1 << which_alternative) & 0x3))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else if (which_alternative == 1)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && (which_alternative != 0)) || ((! (cached_memory == MEMORY_NONE)) && (which_alternative != 0)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x3))) || ((! (cached_memory == MEMORY_NONE)) && (!((1 << which_alternative) & 0x3))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0xc))) || ((! (cached_memory == MEMORY_NONE)) && (((1 << which_alternative) & 0xc))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
    case 320:  /* *muldi3_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
      return 5;

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0)) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (which_alternative == 0))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || ((cached_memory == MEMORY_LOAD) && (which_alternative == 0)))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 0)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x1f)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x1f)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x1f)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (which_alternative == 1)) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0xf))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xf)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xf)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x18)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x18)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x18))) || (((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7))) || (((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7))) || (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7))) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0xce3f)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xce3f)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xce3f)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 6;
        }
      else if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7))) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((1 << which_alternative) & 0x7))) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) && (((1 << which_alternative) & 0x7))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x23f))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x23f)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x23f)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x78)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x78)))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x78))) || (((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7))) || (((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7))) || (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7))) || ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0xff)))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xff)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xff)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 6;
        }
      else
        {
	  return 1;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) || ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) || ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) || ((cached_type = get_attr_type (insn)) == TYPE_IMOVX)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) || ((cached_type = get_attr_type (insn)) == TYPE_IMOVX)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5))))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode))))) || ((cached_memory == MEMORY_NONE) && (((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode))) || (((1 << which_alternative) & 0xf80)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0xf80)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0xf80))) || ((cached_memory == MEMORY_NONE) && (which_alternative == 2)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x623f)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x623f)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x623f))) || ((which_alternative == 2) && (cached_memory == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_STORE) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode))))) || ((cached_memory == MEMORY_NONE) && (((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode))) || (((1 << which_alternative) & 0x7c00)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7c00)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7c00))) || ((cached_memory == MEMORY_NONE) && (which_alternative == 4)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 4))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 6;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x7)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1088:  /* *movv2df_internal */
    case 1087:  /* *movv4df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1085:  /* *movv8sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1083:  /* *movv2ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1081:  /* *movv4di_internal */
    case 1080:  /* *movv4si_internal */
    case 1079:  /* *movv8si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1075:  /* *movv32qi_internal */
    case 61:  /* *movti_internal_sse */
    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 0))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1338:  /* sse3_movsldup */
    case 1336:  /* sse3_movshdup */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1142:  /* sse_rcpv4sf2 */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 993:  /* *vec_dupv2sf */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 4;

    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 920:  /* *prefetch_3dnow */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      return 1;

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else
        {
	  return 1;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_LOAD)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return 6;

    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_OTHER) || ((cached_type == TYPE_MULTI) || (cached_type == TYPE_STR)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_CALL) || ((cached_type == TYPE_CALLV) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_IMOV))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_NONE) && (cached_type == TYPE_ICMOV)) || ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_ICMOV)))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (cached_type == TYPE_PUSH)) || (((cached_memory == MEMORY_BOTH) && (cached_type == TYPE_PUSH)) || ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_LEA) || ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1))))))))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_IBR))
        {
	  return 1;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (cached_type == TYPE_IBR))
        {
	  return 6;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && ((((cached_mode = get_attr_mode (insn)) == MODE_HI) || (cached_mode == MODE_SI)) && (cached_type == TYPE_IMUL))) || ((! (cached_memory == MEMORY_NONE)) && ((((cached_mode = get_attr_mode (insn)) == MODE_HI) || (cached_mode == MODE_SI)) && (cached_type == TYPE_IMUL))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_IMUL))) || ((! (cached_memory == MEMORY_NONE)) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_IMUL))))
        {
	  return 5;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_QI) && (cached_type == TYPE_IDIV))) || ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_QI) && (cached_type == TYPE_IDIV))))
        {
	  return 19 /* 0x13 */;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (cached_type == TYPE_IDIV))) || ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (cached_type == TYPE_IDIV))))
        {
	  return 23 /* 0x17 */;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_SI) && (cached_type == TYPE_IDIV))) || ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_SI) && (cached_type == TYPE_IDIV))))
        {
	  return 39 /* 0x27 */;
        }
      else if (cached_type == TYPE_FXCH)
        {
	  return 0;
        }
      else if (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && (cached_type == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FOP))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_FOP))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_BOTH) && (cached_type == TYPE_FOP))
        {
	  return 5;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 1;
        }
      else if (cached_type == TYPE_FISTP)
        {
	  return 5;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FCMP))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FCMP))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FMOV)) || ((cached_memory == MEMORY_LOAD) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_XF)) && (cached_type == TYPE_FMOV))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && (cached_type == TYPE_FMOV)))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_STORE) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_XF)) && (cached_type == TYPE_FMOV)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_STORE) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && (cached_type == TYPE_FMOV)))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FMUL))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FMUL))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 19 /* 0x13 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 32 /* 0x20 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 33 /* 0x21 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 38 /* 0x26 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 39 /* 0x27 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_SSEIADD)))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_SSEIADD)))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_MMXSHFT))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_MMXSHFT))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_SSEISHFT) && (! ((cached_length_immediate = get_attr_length_immediate (insn)) == 0))))
        {
	  return 1;
        }
      else if (((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_SSEISHFT) && (! ((cached_length_immediate = get_attr_length_immediate (insn)) == 0)))) || ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_SSEISHFT) && ((cached_length_immediate = get_attr_length_immediate (insn)) == 0))))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_SSEISHFT) && ((cached_length_immediate = get_attr_length_immediate (insn)) == 0))) || ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_MMXMUL) || (cached_type == TYPE_SSEIMUL))))
        {
	  return 3;
        }
      else if (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_MMXCVT))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_UNKNOWN) && (cached_type == TYPE_SSE)) || (((cached_mode == MODE_SF) || (cached_mode == MODE_DF)) && (cached_type == TYPE_SSE)))
        {
	  return 3;
        }
      else if ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSE))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_SSEADD) || ((cached_type == TYPE_SSECMP) || (cached_type == TYPE_SSECOMI)))) || ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_SSEADD) || ((cached_type == TYPE_SSECMP) || (cached_type == TYPE_SSECOMI)))))
        {
	  return 3;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEMUL))) || ((cached_memory == MEMORY_LOAD) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEMUL))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEMUL))) || ((cached_memory == MEMORY_LOAD) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEMUL))))
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEDIV)))
        {
	  return 18 /* 0x12 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEDIV)))
        {
	  return 32 /* 0x20 */;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEICVT))) || (((! (cached_memory == MEMORY_NONE)) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEICVT))) || (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_DF) && (cached_type == TYPE_SSEICVT))) || ((! (cached_memory == MEMORY_NONE)) && ((cached_mode == MODE_DF) && (cached_type == TYPE_SSEICVT))))))
        {
	  return 4;
        }
      else if (((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SI) && (cached_type == TYPE_SSEICVT))) || ((! (cached_memory == MEMORY_NONE)) && ((cached_mode == MODE_SI) && (cached_type == TYPE_SSEICVT))))
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_SSEMOV))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_SSEMOV))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) && (cached_type == TYPE_SSEMOV)) || (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP))))))))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP))))))))))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP)))))))))))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP))))))))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    default:
      return 0;

    }
}

int
internal_dfa_insn_code_corei7 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  int cached_length_immediate ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 610 /* 0x262 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 611 /* 0x263 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1923:  /* avx2_lshrvv2di */
    case 1922:  /* avx2_ashlvv2di */
    case 1921:  /* avx2_lshrvv4di */
    case 1920:  /* avx2_ashlvv4di */
    case 1919:  /* avx2_lshrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1917:  /* avx2_lshrvv8si */
    case 1916:  /* avx2_ashlvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1914:  /* avx2_ashrvv8si */
    case 1798:  /* xop_shlv2di3 */
    case 1797:  /* xop_shlv4si3 */
    case 1796:  /* xop_shlv8hi3 */
    case 1795:  /* xop_shlv16qi3 */
    case 1794:  /* xop_shav2di3 */
    case 1793:  /* xop_shav4si3 */
    case 1792:  /* xop_shav8hi3 */
    case 1791:  /* xop_shav16qi3 */
    case 1790:  /* xop_vrotlv2di3 */
    case 1789:  /* xop_vrotlv4si3 */
    case 1788:  /* xop_vrotlv8hi3 */
    case 1787:  /* xop_vrotlv16qi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 586 /* 0x24a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 587 /* 0x24b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2010:  /* mfence_sse2 */
    case 2009:  /* *sse_sfence */
    case 2008:  /* *sse2_lfence */
    case 1607:  /* sse2_clflush */
      return 591 /* 0x24f */;

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0xc)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xc)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xc)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x63)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x63)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x63)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 610 /* 0x262 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 611 /* 0x263 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x6)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x6)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x6)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x6)))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x6)))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x6)))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x6)))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 536 /* 0x218 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x9)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x9)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x9)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x6)))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x6)))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x6)))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x6)))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 532 /* 0x214 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 535 /* 0x217 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1786:  /* xop_rotrv2di3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1779:  /* xop_rotlv16qi3 */
    case 1649:  /* ssse3_palignrdi */
    case 1648:  /* ssse3_palignrti */
    case 1647:  /* avx2_palignrv2ti */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 584 /* 0x248 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 585 /* 0x249 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1446:  /* ashrv4si3 */
    case 1445:  /* ashrv8si3 */
    case 1444:  /* ashrv8hi3 */
    case 1443:  /* ashrv16hi3 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (const_int_operand (operands[2], VOIDmode)))
        {
	  return 584 /* 0x248 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (const_int_operand (operands[2], VOIDmode)))
        {
	  return 585 /* 0x249 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (! (const_int_operand (operands[2], VOIDmode))))
        {
	  return 586 /* 0x24a */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (const_int_operand (operands[2], VOIDmode))))
        {
	  return 587 /* 0x24b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 5))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 5))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 5))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 10))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 10))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 10))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 9))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x620)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x620)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x620)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 536 /* 0x218 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 5))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 5))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 5))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x13)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x13)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x13)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 536 /* 0x218 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 3))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 3))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 3))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 536 /* 0x218 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x9)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x9)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x9)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1391:  /* *vec_concatv2df */
    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x7)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 532 /* 0x214 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 9))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 9))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 9))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 10))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 10))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 10))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x6c1)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x6c1)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x6c1)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 590 /* 0x24e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 590 /* 0x24e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1588:  /* *vec_concatv4si */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x3)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1311:  /* sse2_cvttsd2si */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1294:  /* sse_cvttss2si */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 608 /* 0x260 */;
        }
      else
        {
	  return 609 /* 0x261 */;
        }

    case 1338:  /* sse3_movsldup */
    case 1336:  /* sse3_movshdup */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1142:  /* sse_rcpv4sf2 */
      return 593 /* 0x251 */;

    case 1140:  /* sse2_vmdivv2df3 */
    case 1138:  /* sse2_divv2df3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 602 /* 0x25a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1139:  /* sse_vmdivv4sf3 */
    case 1136:  /* sse_divv4sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 600 /* 0x258 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1678:  /* sse4_1_dppd */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1132:  /* *mulv2df3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 598 /* 0x256 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 599 /* 0x257 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1676:  /* sse4_1_dpps */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1130:  /* *mulv4sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 596 /* 0x254 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 597 /* 0x255 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 536 /* 0x218 */;
        }
      else if (which_alternative == 0)
        {
	  return 590 /* 0x24e */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 5))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 5))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 5))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 582 /* 0x246 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 583 /* 0x247 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 536 /* 0x218 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 570 /* 0x23a */;
        }
      else if (which_alternative == 0)
        {
	  return 590 /* 0x24e */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 3))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 3))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 3))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 590 /* 0x24e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 993:  /* *vec_dupv2sf */
      return 590 /* 0x24e */;

    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 972:  /* *mmx_mulv2sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 588 /* 0x24c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1631:  /* ssse3_pmaddubsw */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1436:  /* *avx2_pmaddwd */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 580 /* 0x244 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 581 /* 0x245 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0xc7f)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xc7f)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xc7f)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((which_alternative == 0) && (cached_memory == MEMORY_NONE))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 0))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x2ff)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x2ff)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x2ff)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x627f)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x627f)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x627f)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((which_alternative == 0) && (cached_memory == MEMORY_NONE))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 0))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x1ff)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x1ff)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x1ff)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode)))))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode)))))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 532 /* 0x214 */;
        }
      else
        {
	  return 564 /* 0x234 */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 537 /* 0x219 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x3)))
        {
	  return 538 /* 0x21a */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 564 /* 0x234 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 894:  /* *movxfcc_1 */
      return 564 /* 0x234 */;

    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 537 /* 0x219 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 538 /* 0x21a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 920:  /* *prefetch_3dnow */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
      return 613 /* 0x265 */;

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1679:  /* avx2_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1670:  /* sse4_1_blendpd */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1602:  /* sse2_pmovmskb */
    case 1601:  /* avx2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1599:  /* avx_movmskpd256 */
    case 1598:  /* sse_movmskps */
    case 1597:  /* avx_movmskps256 */
    case 1576:  /* sse2_loadld */
    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1352:  /* *vec_concatv4sf */
    case 1346:  /* sse_movss */
    case 1344:  /* sse_storelps */
    case 1343:  /* sse_loadhps */
    case 1342:  /* sse_storehps */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
    case 1110:  /* sse2_movntv2df */
    case 1109:  /* avx_movntv4df */
    case 1108:  /* sse_movntv4sf */
    case 1107:  /* avx_movntv8sf */
    case 1106:  /* sse2_movntidi */
    case 1105:  /* sse2_movntisi */
    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
    case 1102:  /* sse2_storedqu */
    case 1101:  /* avx_storedqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1099:  /* avx_loaddqu256 */
    case 1098:  /* sse2_storeupd */
    case 1097:  /* avx_storeupd256 */
    case 1096:  /* sse_storeups */
    case 1095:  /* avx_storeups256 */
    case 1094:  /* sse2_loadupd */
    case 1093:  /* avx_loadupd256 */
    case 1092:  /* sse_loadups */
    case 1091:  /* avx_loadups256 */
    case 1089:  /* sse2_movq128 */
    case 852:  /* movmsk_df */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 610 /* 0x262 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 611 /* 0x263 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 578 /* 0x242 */;

    case 772:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 576 /* 0x240 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 577 /* 0x241 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 771:  /* *fop_xf_6_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 767:  /* *fop_xf_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 574 /* 0x23e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 575 /* 0x23f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 578 /* 0x242 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 579 /* 0x243 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], XFmode))))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], XFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], XFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 574 /* 0x23e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 575 /* 0x23f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 576 /* 0x240 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 577 /* 0x241 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 574 /* 0x23e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 575 /* 0x23f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 598 /* 0x256 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 599 /* 0x257 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 602 /* 0x25a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 596 /* 0x254 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 597 /* 0x255 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 600 /* 0x258 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1172:  /* sse2_vmsminv2df3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 780:  /* *sqrtdf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 745:  /* *rcpsf2_sse */
      return 592 /* 0x250 */;

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 576 /* 0x240 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 577 /* 0x241 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 598 /* 0x256 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 599 /* 0x257 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 602 /* 0x25a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 574 /* 0x23e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_FDIV))
        {
	  return 575 /* 0x23f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_SSEADD))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 596 /* 0x254 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))))
        {
	  return 597 /* 0x255 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_SSEDIV))
        {
	  return 600 /* 0x258 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 573 /* 0x23d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], DFmode)))
        {
	  return 598 /* 0x256 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], DFmode)))
        {
	  return 599 /* 0x257 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (mult_operator (operands[3], SFmode)))
        {
	  return 596 /* 0x254 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (mult_operator (operands[3], SFmode)))
        {
	  return 597 /* 0x255 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (mult_operator (operands[3], DFmode))))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (mult_operator (operands[3], DFmode))))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode))))
        {
	  return 598 /* 0x256 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode))))
        {
	  return 599 /* 0x257 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 561 /* 0x231 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && (mult_operator (operands[3], SFmode))))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && (mult_operator (operands[3], SFmode))))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode))))
        {
	  return 596 /* 0x254 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode))))
        {
	  return 597 /* 0x255 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 614 /* 0x266 */;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 0))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 0))
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 546 /* 0x222 */;

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      return 533 /* 0x215 */;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 544 /* 0x220 */;
        }
      else
        {
	  return 545 /* 0x221 */;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 613 /* 0x265 */;
        }
      else
        {
	  return 615 /* 0x267 */;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative == 0))
        {
	  return 543 /* 0x21f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 542 /* 0x21e */;
        }
      else if ((which_alternative == 1) && (! (cached_memory == MEMORY_NONE)))
        {
	  return 543 /* 0x21f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 493:  /* *ashlqi3_1_slp */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 491:  /* *ashlhi3_1 */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 1))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type = get_attr_type (insn)) == TYPE_ISHIFT))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 542 /* 0x21e */;
        }
      else
        {
	  return 543 /* 0x21f */;
        }

    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
      return 562 /* 0x232 */;

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x3)))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 534 /* 0x216 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 535 /* 0x217 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (((1 << which_alternative) & 0x7)))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 355:  /* udivmodhiqi3 */
    case 346:  /* divmodhiqi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 551 /* 0x227 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 552 /* 0x228 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 353:  /* *udivmodsi4_noext */
    case 344:  /* *divmodsi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 555 /* 0x22b */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 556 /* 0x22c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 352:  /* *udivmodhi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 553 /* 0x229 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 554 /* 0x22a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 549 /* 0x225 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative == 0))
        {
	  return 550 /* 0x226 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 547 /* 0x223 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative == 0))
        {
	  return 548 /* 0x224 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
    case 320:  /* *muldi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 549 /* 0x225 */;
        }
      else
        {
	  return 550 /* 0x226 */;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 328:  /* *mulsidi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 547 /* 0x223 */;
        }
      else
        {
	  return 548 /* 0x224 */;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 613 /* 0x265 */;
        }
      else
        {
	  return 614 /* 0x266 */;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 5))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative != 5))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 5))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 5))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 5))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative != 2))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 2))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 2))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 2))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 3))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative != 3))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 3))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 3))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative != 3))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 613 /* 0x265 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 614 /* 0x266 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 615 /* 0x267 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
      return 541 /* 0x21d */;

    case 210:  /* *floatsidf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 606 /* 0x25e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (((1 << which_alternative) & 0x3)))
        {
	  return 607 /* 0x25f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 604 /* 0x25c */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (((1 << which_alternative) & 0x3)))
        {
	  return 605 /* 0x25d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 606 /* 0x25e */;
        }
      else if ((which_alternative == 1) && (! (cached_memory == MEMORY_NONE)))
        {
	  return 607 /* 0x25f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 604 /* 0x25c */;
        }
      else if ((which_alternative == 1) && (! (cached_memory == MEMORY_NONE)))
        {
	  return 605 /* 0x25d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 606 /* 0x25e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative != 0))
        {
	  return 607 /* 0x25f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 203:  /* *floatdisf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 604 /* 0x25c */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (which_alternative != 0))
        {
	  return 605 /* 0x25d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 606 /* 0x25e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (!((1 << which_alternative) & 0x3)))
        {
	  return 607 /* 0x25f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 199:  /* *floatdisf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x3)))
        {
	  return 604 /* 0x25c */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (!((1 << which_alternative) & 0x3)))
        {
	  return 605 /* 0x25d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0xc)))
        {
	  return 606 /* 0x25e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (((1 << which_alternative) & 0xc)))
        {
	  return 607 /* 0x25f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0xc)))
        {
	  return 604 /* 0x25c */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (((1 << which_alternative) & 0xc)))
        {
	  return 605 /* 0x25d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 567 /* 0x237 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 569 /* 0x239 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 571 /* 0x23b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 532 /* 0x214 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 567 /* 0x237 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 569 /* 0x239 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 571 /* 0x23b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
      return 563 /* 0x233 */;

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 606 /* 0x25e */;
        }
      else
        {
	  return 607 /* 0x25f */;
        }

    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 211:  /* *floatsisf2_vector_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 154:  /* fix_truncsfdi_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 604 /* 0x25c */;
        }
      else
        {
	  return 605 /* 0x25d */;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 532 /* 0x214 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 567 /* 0x237 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 568 /* 0x238 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 570 /* 0x23a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 567 /* 0x237 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 568 /* 0x238 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 570 /* 0x23a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 569 /* 0x239 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 571 /* 0x23b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 569 /* 0x239 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 0))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 571 /* 0x23b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 535 /* 0x217 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 613 /* 0x265 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 614 /* 0x266 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_STORE))
        {
	  return 615 /* 0x267 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_BOTH))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x1f)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x1f)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x1f)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 1))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0xf)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xf)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xf)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
      return 557 /* 0x22d */;

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x18)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x18)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x18)))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0xce3f)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xce3f)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xce3f)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0x23f)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x23f)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x23f)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x78)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x78)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x78)))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7)))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7)))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7)))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (!((1 << which_alternative) & 0xff)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0xff)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0xff)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 532 /* 0x214 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 567 /* 0x237 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 569 /* 0x239 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 571 /* 0x23b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((1 << which_alternative) & 0x3)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x3)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x3)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 532 /* 0x214 */;
        }
      else if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 539 /* 0x21b */;
        }
      else
        {
	  return 540 /* 0x21c */;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) || ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) || ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))))
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 534 /* 0x216 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 535 /* 0x217 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 534 /* 0x216 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 535 /* 0x217 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 535 /* 0x217 */;

    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 536 /* 0x218 */;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) || ((cached_type = get_attr_type (insn)) == TYPE_IMOVX)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) || ((cached_type = get_attr_type (insn)) == TYPE_IMOVX)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))))
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5))))))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5))))))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type = get_attr_type (insn)) == TYPE_IMOV))
        {
	  return 536 /* 0x218 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode))))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0xf80)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0xf80)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0xf80)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 2))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x623f)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x623f)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x623f)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_NONE))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 2))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 2))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 2))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode))))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((1 << which_alternative) & 0x7c00)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((1 << which_alternative) & 0x7c00)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((1 << which_alternative) & 0x7c00)))
        {
	  return 612 /* 0x264 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (which_alternative == 4))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 4))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative == 4))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (which_alternative == 4))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 532 /* 0x214 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (!((1 << which_alternative) & 0x7)))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (!((1 << which_alternative) & 0x7)))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1088:  /* *movv2df_internal */
    case 1087:  /* *movv4df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1085:  /* *movv8sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1083:  /* *movv2ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1081:  /* *movv4di_internal */
    case 1080:  /* *movv4si_internal */
    case 1079:  /* *movv8si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1075:  /* *movv32qi_internal */
    case 61:  /* *movti_internal_sse */
    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative != 0))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative != 0))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (which_alternative != 0))
        {
	  return 612 /* 0x264 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 613 /* 0x265 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 615 /* 0x267 */;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 614 /* 0x266 */;
        }
      else
        {
	  return 616 /* 0x268 */;
        }

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 539 /* 0x21b */;
        }
      else
        {
	  return 540 /* 0x21c */;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 532 /* 0x214 */;
        }
      else if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 539 /* 0x21b */;
        }
      else
        {
	  return 540 /* 0x21c */;
        }

    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 565 /* 0x235 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 566 /* 0x236 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 594 /* 0x252 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 595 /* 0x253 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (which_alternative == 0))
        {
	  return 565 /* 0x235 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (which_alternative == 0))
        {
	  return 566 /* 0x236 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_NONE))
        {
	  return 594 /* 0x252 */;
        }
      else if ((which_alternative == 1) && (cached_memory == MEMORY_LOAD))
        {
	  return 595 /* 0x253 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return 532 /* 0x214 */;

    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 613 /* 0x265 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 614 /* 0x266 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_OTHER) || ((cached_type == TYPE_MULTI) || (cached_type == TYPE_STR)))
        {
	  return 532 /* 0x214 */;
        }
      else if ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 533 /* 0x215 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_type == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 534 /* 0x216 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_IMOV) || (cached_type == TYPE_IMOVX)))
        {
	  return 535 /* 0x217 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_IMOV))
        {
	  return 536 /* 0x218 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_ICMOV))
        {
	  return 537 /* 0x219 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_ICMOV))
        {
	  return 538 /* 0x21a */;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_PUSH))
        {
	  return 539 /* 0x21b */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (cached_type == TYPE_PUSH))
        {
	  return 540 /* 0x21c */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_LEA))
        {
	  return 541 /* 0x21d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1)))))
        {
	  return 542 /* 0x21e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1)))))
        {
	  return 543 /* 0x21f */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_IBR))
        {
	  return 544 /* 0x220 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (cached_type == TYPE_IBR))
        {
	  return 545 /* 0x221 */;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 546 /* 0x222 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((((cached_mode = get_attr_mode (insn)) == MODE_HI) || (cached_mode == MODE_SI)) && (cached_type == TYPE_IMUL)))
        {
	  return 547 /* 0x223 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((((cached_mode = get_attr_mode (insn)) == MODE_HI) || (cached_mode == MODE_SI)) && (cached_type == TYPE_IMUL)))
        {
	  return 548 /* 0x224 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_IMUL)))
        {
	  return 549 /* 0x225 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_IMUL)))
        {
	  return 550 /* 0x226 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_QI) && (cached_type == TYPE_IDIV)))
        {
	  return 551 /* 0x227 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_QI) && (cached_type == TYPE_IDIV)))
        {
	  return 552 /* 0x228 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (cached_type == TYPE_IDIV)))
        {
	  return 553 /* 0x229 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (cached_type == TYPE_IDIV)))
        {
	  return 554 /* 0x22a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_SI) && (cached_type == TYPE_IDIV)))
        {
	  return 555 /* 0x22b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_SI) && (cached_type == TYPE_IDIV)))
        {
	  return 556 /* 0x22c */;
        }
      else if (cached_type == TYPE_FXCH)
        {
	  return 557 /* 0x22d */;
        }
      else if (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && (cached_type == TYPE_FOP))
        {
	  return 558 /* 0x22e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FOP))
        {
	  return 559 /* 0x22f */;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_FOP))
        {
	  return 560 /* 0x230 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && (cached_type == TYPE_FOP))
        {
	  return 561 /* 0x231 */;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 562 /* 0x232 */;
        }
      else if (cached_type == TYPE_FISTP)
        {
	  return 563 /* 0x233 */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 564 /* 0x234 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FCMP))
        {
	  return 565 /* 0x235 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FCMP))
        {
	  return 566 /* 0x236 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FMOV))
        {
	  return 567 /* 0x237 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_XF)) && (cached_type == TYPE_FMOV)))
        {
	  return 568 /* 0x238 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && (cached_type == TYPE_FMOV)))
        {
	  return 569 /* 0x239 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_XF)) && (cached_type == TYPE_FMOV)))
        {
	  return 570 /* 0x23a */;
        }
      else if ((cached_memory == MEMORY_STORE) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && (cached_type == TYPE_FMOV)))
        {
	  return 571 /* 0x23b */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_FMUL))
        {
	  return 572 /* 0x23c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_FMUL))
        {
	  return 573 /* 0x23d */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 574 /* 0x23e */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 575 /* 0x23f */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 576 /* 0x240 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 577 /* 0x241 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 578 /* 0x242 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode = get_attr_mode (insn)) == MODE_XF) && ((cached_type == TYPE_FDIV) || (cached_type == TYPE_FPSPC))))
        {
	  return 579 /* 0x243 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_SSEIADD)))
        {
	  return 580 /* 0x244 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_MMXADD) || (cached_type == TYPE_SSEIADD)))
        {
	  return 581 /* 0x245 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_MMXSHFT))
        {
	  return 582 /* 0x246 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_MMXSHFT))
        {
	  return 583 /* 0x247 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_SSEISHFT) && (! ((cached_length_immediate = get_attr_length_immediate (insn)) == 0))))
        {
	  return 584 /* 0x248 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_SSEISHFT) && (! ((cached_length_immediate = get_attr_length_immediate (insn)) == 0))))
        {
	  return 585 /* 0x249 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_SSEISHFT) && ((cached_length_immediate = get_attr_length_immediate (insn)) == 0)))
        {
	  return 586 /* 0x24a */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_SSEISHFT) && ((cached_length_immediate = get_attr_length_immediate (insn)) == 0)))
        {
	  return 587 /* 0x24b */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_MMXMUL) || (cached_type == TYPE_SSEIMUL)))
        {
	  return 588 /* 0x24c */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_MMXMUL) || (cached_type == TYPE_SSEIMUL)))
        {
	  return 589 /* 0x24d */;
        }
      else if (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (cached_type == TYPE_MMXCVT))
        {
	  return 590 /* 0x24e */;
        }
      else if ((cached_memory == MEMORY_UNKNOWN) && (cached_type == TYPE_SSE))
        {
	  return 591 /* 0x24f */;
        }
      else if (((cached_mode == MODE_SF) || (cached_mode == MODE_DF)) && (cached_type == TYPE_SSE))
        {
	  return 592 /* 0x250 */;
        }
      else if ((cached_mode == MODE_V4SF) && (cached_type == TYPE_SSE))
        {
	  return 593 /* 0x251 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_type == TYPE_SSEADD) || ((cached_type == TYPE_SSECMP) || (cached_type == TYPE_SSECOMI))))
        {
	  return 594 /* 0x252 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_SSEADD) || ((cached_type == TYPE_SSECMP) || (cached_type == TYPE_SSECOMI))))
        {
	  return 595 /* 0x253 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEMUL)))
        {
	  return 596 /* 0x254 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEMUL)))
        {
	  return 597 /* 0x255 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEMUL)))
        {
	  return 598 /* 0x256 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEMUL)))
        {
	  return 599 /* 0x257 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEDIV)))
        {
	  return 600 /* 0x258 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_SF) || (cached_mode == MODE_V4SF)) && (cached_type == TYPE_SSEDIV)))
        {
	  return 601 /* 0x259 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEDIV)))
        {
	  return 602 /* 0x25a */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((cached_mode == MODE_DF) || (cached_mode == MODE_V2DF)) && (cached_type == TYPE_SSEDIV)))
        {
	  return 603 /* 0x25b */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEICVT)))
        {
	  return 604 /* 0x25c */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_mode == MODE_SF) && (cached_type == TYPE_SSEICVT)))
        {
	  return 605 /* 0x25d */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_DF) && (cached_type == TYPE_SSEICVT)))
        {
	  return 606 /* 0x25e */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_mode == MODE_DF) && (cached_type == TYPE_SSEICVT)))
        {
	  return 607 /* 0x25f */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((cached_mode == MODE_SI) && (cached_type == TYPE_SSEICVT)))
        {
	  return 608 /* 0x260 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((cached_mode == MODE_SI) && (cached_type == TYPE_SSEICVT)))
        {
	  return 609 /* 0x261 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (cached_type == TYPE_SSEMOV))
        {
	  return 610 /* 0x262 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && (cached_type == TYPE_SSEMOV))
        {
	  return 611 /* 0x263 */;
        }
      else if ((cached_memory == MEMORY_STORE) && (cached_type == TYPE_SSEMOV))
        {
	  return 612 /* 0x264 */;
        }
      else if (((cached_memory == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP)))))))))))
        {
	  return 613 /* 0x265 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP))))))))))))
        {
	  return 614 /* 0x266 */;
        }
      else if ((cached_memory == MEMORY_STORE) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP)))))))))))
        {
	  return 615 /* 0x267 */;
        }
      else if ((cached_memory == MEMORY_BOTH) && ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_MMX) || (cached_type == TYPE_MMXCMP))))))))))))
        {
	  return 616 /* 0x268 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    default:
      return 618 /* 0x26a */;

    }
}

int
insn_default_latency_atom (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_atom_unit cached_atom_unit ATTRIBUTE_UNUSED;
  enum attr_use_carry cached_use_carry ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  int cached_prefix_0f ATTRIBUTE_UNUSED;
  enum attr_atom_sse_attr cached_atom_sse_attr ATTRIBUTE_UNUSED;
  enum attr_movu cached_movu ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 1) || (((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))) || ((which_alternative == 1) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) || ((which_alternative == 0) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1631:  /* ssse3_pmaddubsw */
      extract_insn_cached (insn);
      if (! (register_operand (operands[0], V2DImode)))
        {
	  return 4;
        }
      else
        {
	  return 6;
        }

    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
      return 6;

    case 1595:  /* avx2_psadbw */
      extract_insn_cached (insn);
      if (register_operand (operands[0], V2DImode))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xc)) || ((!((1 << which_alternative) & 0xc)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x63)) || ((!((1 << which_alternative) & 0x63)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 3) || (((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))) || ((which_alternative == 3) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x6)) || ((!((1 << which_alternative) & 0x6)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x16)) || ((((1 << which_alternative) & 0x9)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x9)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 9;
        }
      else if (which_alternative == 2)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1923:  /* avx2_lshrvv2di */
    case 1922:  /* avx2_ashlvv2di */
    case 1921:  /* avx2_lshrvv4di */
    case 1920:  /* avx2_ashlvv4di */
    case 1919:  /* avx2_lshrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1917:  /* avx2_lshrvv8si */
    case 1916:  /* avx2_ashlvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1914:  /* avx2_ashrvv8si */
    case 1798:  /* xop_shlv2di3 */
    case 1797:  /* xop_shlv4si3 */
    case 1796:  /* xop_shlv8hi3 */
    case 1795:  /* xop_shlv16qi3 */
    case 1794:  /* xop_shav2di3 */
    case 1793:  /* xop_shav4si3 */
    case 1792:  /* xop_shav8hi3 */
    case 1791:  /* xop_shav16qi3 */
    case 1790:  /* xop_vrotlv2di3 */
    case 1789:  /* xop_vrotlv4si3 */
    case 1788:  /* xop_vrotlv8hi3 */
    case 1787:  /* xop_vrotlv16qi3 */
    case 1786:  /* xop_rotrv2di3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1779:  /* xop_rotlv16qi3 */
    case 1649:  /* ssse3_palignrdi */
    case 1648:  /* ssse3_palignrti */
    case 1647:  /* avx2_palignrv2ti */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1446:  /* ashrv4si3 */
    case 1445:  /* ashrv8si3 */
    case 1444:  /* ashrv8hi3 */
    case 1443:  /* ashrv16hi3 */
      extract_insn_cached (insn);
      if (immediate_operand (operands[2], VOIDmode))
        {
	  return 1;
        }
      else
        {
	  return 2;
        }

    case 1630:  /* ssse3_pmaddubsw128 */
    case 1596:  /* sse2_psadbw */
    case 1437:  /* *sse2_pmaddwd */
      extract_insn_cached (insn);
      if (! (register_operand (operands[0], V2DImode)))
        {
	  return 5;
        }
      else
        {
	  return 6;
        }

    case 1629:  /* avx2_pmaddubsw256 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1436:  /* *avx2_pmaddwd */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
      extract_insn_cached (insn);
      if (! (register_operand (operands[0], V2DImode)))
        {
	  return 1;
        }
      else
        {
	  return 6;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) || ((which_alternative != 5) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x420)) || ((!((1 << which_alternative) & 0x620)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if (which_alternative == 9)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x4c)) || ((((1 << which_alternative) & 0x13)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x13)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if (which_alternative == 5)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) || ((((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if (which_alternative == 3)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x26)) || ((((1 << which_alternative) & 0x9)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x9)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if (which_alternative == 4)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1391:  /* *vec_concatv2df */
    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) || ((!((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x2c1)) || ((!((1 << which_alternative) & 0x6c1)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if (which_alternative == 10)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) || ((which_alternative == 1) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((which_alternative == 1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xf)) || ((which_alternative == 4) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((which_alternative == 4) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 1) || ((which_alternative == 1) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1588:  /* *vec_concatv4si */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) || ((!((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
      return 62 /* 0x3e */;

    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
      return 8;

    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
    case 1102:  /* sse2_storedqu */
    case 1101:  /* avx_storedqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1099:  /* avx_loaddqu256 */
    case 1098:  /* sse2_storeupd */
    case 1097:  /* avx_storeupd256 */
    case 1096:  /* sse_storeups */
    case 1095:  /* avx_storeups256 */
    case 1094:  /* sse2_loadupd */
    case 1093:  /* avx_loadupd256 */
    case 1092:  /* sse_loadups */
    case 1091:  /* avx_loadups256 */
      extract_insn_cached (insn);
      if ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x4e)) || ((which_alternative == 5) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((which_alternative == 5) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x22)) || ((which_alternative == 3) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((which_alternative == 3) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((which_alternative == 3) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if (which_alternative == 4)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 9;
        }
      else if ((which_alternative == 6) || ((!((1 << which_alternative) & 0xc7f)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && ((((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))) || (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode))))))
        {
	  return 5;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x203)) || ((!((1 << which_alternative) & 0x2ff)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && ((((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))) || (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode))))))
        {
	  return 5;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 9;
        }
      else if ((((1 << which_alternative) & 0x240)) || ((!((1 << which_alternative) & 0x627f)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && ((((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))) || (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode))))))
        {
	  return 5;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x103)) || ((!((1 << which_alternative) & 0x1ff)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && ((((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))) || (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode))))))
        {
	  return 5;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 9;
        }
      else
        {
	  return 0;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1679:  /* avx2_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1670:  /* sse4_1_blendpd */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1602:  /* sse2_pmovmskb */
    case 1601:  /* avx2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1599:  /* avx_movmskpd256 */
    case 1598:  /* sse_movmskps */
    case 1597:  /* avx_movmskps256 */
    case 1576:  /* sse2_loadld */
    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1352:  /* *vec_concatv4sf */
    case 1346:  /* sse_movss */
    case 1344:  /* sse_storelps */
    case 1343:  /* sse_loadhps */
    case 1342:  /* sse_storehps */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
    case 1110:  /* sse2_movntv2df */
    case 1109:  /* avx_movntv4df */
    case 1108:  /* sse_movntv4sf */
    case 1107:  /* avx_movntv8sf */
    case 1106:  /* sse2_movntidi */
    case 1105:  /* sse2_movntisi */
    case 1089:  /* sse2_movq128 */
    case 852:  /* movmsk_df */
      extract_insn_cached (insn);
      if ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) || (mult_operator (operands[3], DFmode)))
        {
	  return 5;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 62 /* 0x3e */;
        }
      else
        {
	  return 0;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)
        {
	  return 5;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 4;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 62 /* 0x3e */;
        }
      else
        {
	  return 0;
        }

    case 1143:  /* sse_vmrcpv4sf2 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 778:  /* *rsqrtsf2_sse */
    case 745:  /* *rcpsf2_sse */
      return 4;

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) || ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))))
        {
	  return 5;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 62 /* 0x3e */;
        }
      else
        {
	  return 0;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)
        {
	  return 5;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 62 /* 0x3e */;
        }
      else
        {
	  return 0;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if (! (mult_operator (operands[3], SFmode)))
        {
	  return 5;
        }
      else
        {
	  return 4;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6))
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 5;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 3;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 2;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))))
        {
	  return 1;
        }
      else if ((which_alternative == 0) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 3;
        }
      else if (which_alternative == 1)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))) || ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (cached_type == TYPE_ISHIFT))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 493:  /* *ashlqi3_1_slp */
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (cached_type == TYPE_ISHIFT1))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || ((which_alternative == 2) || (cached_type == TYPE_ISHIFT)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) || (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (cached_type == TYPE_ISHIFT)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || ((which_alternative == 1) || ((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || ((which_alternative == 1) || (cached_type == TYPE_ISHIFT)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1606:  /* sse_stmxcsr */
    case 1605:  /* sse_ldmxcsr */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1147:  /* sse2_sqrtv2df2 */
    case 1146:  /* avx_sqrtv4df2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1144:  /* avx_sqrtv8sf2 */
    case 1142:  /* sse_rcpv4sf2 */
    case 780:  /* *sqrtdf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      return 7;

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 2) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || ((which_alternative == 2) && ((((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 3) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || ((which_alternative == 3) && ((((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      return 65 /* 0x41 */;

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 10 /* 0xa */;
        }
      else
        {
	  return 0;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 740:  /* *fop_df_comm_sse */
    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 328:  /* *mulsidi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      return 5;

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 5) && (! (incdec_operand (operands[2], QImode)))) || ((which_alternative == 5) || ((which_alternative != 5) && (incdec_operand (operands[2], QImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], HImode)))) || ((which_alternative == 3) || ((which_alternative != 3) && (incdec_operand (operands[2], HImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 2) && (! (incdec_operand (operands[2], SImode)))) || ((which_alternative == 2) || ((which_alternative != 2) && (incdec_operand (operands[2], SImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 251:  /* *adddi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], DImode)))) || ((which_alternative == 3) || ((which_alternative != 3) && (incdec_operand (operands[2], DImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], SImode)))) || ((which_alternative == 3) || ((which_alternative != 3) && (incdec_operand (operands[2], SImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((register_operand (operands[0], V2DFmode)) && (memory_operand (operands[1], SImode))))
        {
	  return 5;
        }
      else if ((which_alternative != 0) && ((! (register_operand (operands[0], V2DFmode))) || (! (memory_operand (operands[1], SImode)))))
        {
	  return 8;
        }
      else if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((register_operand (operands[0], V2DFmode)) && (memory_operand (operands[1], SImode))))
        {
	  return 5;
        }
      else if ((which_alternative == 1) && ((! (register_operand (operands[0], V2DFmode))) || (! (memory_operand (operands[1], SImode)))))
        {
	  return 8;
        }
      else if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 9;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], V2DFmode)) && (memory_operand (operands[1], SImode))))
        {
	  return 5;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((! (register_operand (operands[0], V2DFmode))) || (! (memory_operand (operands[1], SImode)))))
        {
	  return 8;
        }
      else if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      extract_insn_cached (insn);
      if ((register_operand (operands[0], V2DFmode)) && (memory_operand (operands[1], SImode)))
        {
	  return 5;
        }
      else
        {
	  return 8;
        }

    case 1358:  /* *sse4_1_extractps */
    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 9;
        }
      else
        {
	  return 1;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 9;
        }
      else if ((which_alternative == 1) && ((((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))) || (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode))))))
        {
	  return 5;
        }
      else if ((which_alternative == 1) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 7;
        }
      else if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))) || (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode))))))
        {
	  return 5;
        }
      else if ((which_alternative == 1) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 7;
        }
      else if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1935:  /* vcvtps2ph256 */
    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1932:  /* vcvtph2ps256 */
    case 1931:  /* *vcvtph2ps_load */
    case 1930:  /* vcvtph2ps */
    case 1727:  /* sse4_1_roundsd */
    case 1726:  /* sse4_1_roundss */
    case 1725:  /* sse4_1_roundpd */
    case 1724:  /* avx_roundpd256 */
    case 1723:  /* sse4_1_roundps */
    case 1722:  /* avx_roundps256 */
    case 1328:  /* sse2_cvtps2pd */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1326:  /* avx_cvtps2pd256 */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1315:  /* sse2_cvtdq2pd */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1313:  /* floatv4siv4df2 */
    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1302:  /* sse2_cvtpi2pd */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1298:  /* avx_cvtps2dq256 */
    case 1297:  /* floatv4siv4sf2 */
    case 1296:  /* floatv8siv8sf2 */
    case 1287:  /* sse_cvttps2pi */
    case 1286:  /* sse_cvtps2pi */
    case 1285:  /* sse_cvtpi2ps */
    case 1112:  /* sse2_movntv2di */
    case 1111:  /* avx_movntv4di */
    case 810:  /* sse4_1_rounddf2 */
    case 809:  /* sse4_1_roundsf2 */
    case 141:  /* *truncdfsf_fast_sse */
    case 136:  /* *extendsfdf2_sse */
      extract_insn_cached (insn);
      if (((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || (((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode))) || (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode))))))
        {
	  return 5;
        }
      else if (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode)))))
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))) || (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode))))))
        {
	  return 5;
        }
      else if ((which_alternative == 2) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 7;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || ((which_alternative == 0) && ((((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 9;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) || ((which_alternative == 0) && ((((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else if ((which_alternative == 0) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0xf)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xf)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x38)) || ((!((1 << which_alternative) & 0xce3f)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 9;
        }
      else if ((((1 << which_alternative) & 0x220)) || ((!((1 << which_alternative) & 0x23f)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xf8)) || ((!((1 << which_alternative) & 0xff)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0xff)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xff)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 9;
        }
      else
        {
	  return 1;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) || ((((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 9;
        }
      else if ((which_alternative == 2) || ((((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 9;
        }
      else
        {
	  return 2;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) || ((cached_type == TYPE_IMOVX) && ((((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMOVX) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) || (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && ((((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      extract_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || (((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode)))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) || (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && ((((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMOVX) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if ((get_attr_type (insn) == TYPE_IMOV) || ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && ((((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))))
        {
	  return 1;
        }
      else if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xfbc)) || ((((1 << which_alternative) & 0xf80)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0xf80)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 9;
        }
      else if ((((1 << which_alternative) & 0x220)) || ((!((1 << which_alternative) & 0x623f)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x6000)) && ((((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))) || (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode))))))
        {
	  return 5;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fdf0)) || ((((1 << which_alternative) & 0x7c00)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x18000)) && ((((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))) || (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode))))))
        {
	  return 5;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 9;
        }
      else if ((which_alternative == 2) || ((!((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1088:  /* *movv2df_internal */
    case 1087:  /* *movv4df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1085:  /* *movv8sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1083:  /* *movv2ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1081:  /* *movv4di_internal */
    case 1080:  /* *movv4si_internal */
    case 1079:  /* *movv8si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1075:  /* *movv32qi_internal */
    case 61:  /* *movti_internal_sse */
    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) || ((which_alternative != 0) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode))))))
        {
	  return 1;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 3;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 53:  /* *popsi1_epilogue */
    case 51:  /* *popsi1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      return 2;

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 9;
        }
      else
        {
	  return 2;
        }

    case 1835:  /* avx_vzeroupper */
    case 1834:  /* *avx_vzeroall */
    case 1826:  /* xop_vpermil2v2df3 */
    case 1825:  /* xop_vpermil2v4df3 */
    case 1824:  /* xop_vpermil2v4sf3 */
    case 1823:  /* xop_vpermil2v8sf3 */
    case 1810:  /* xop_maskcmpv2di3 */
    case 1809:  /* xop_maskcmpv4si3 */
    case 1808:  /* xop_maskcmpv8hi3 */
    case 1807:  /* xop_maskcmpv16qi3 */
    case 1806:  /* *xop_vmfrczv2df2 */
    case 1805:  /* *xop_vmfrczv4sf2 */
    case 1804:  /* xop_frczv4df2 */
    case 1803:  /* xop_frczv8sf2 */
    case 1802:  /* xop_frczv2df2 */
    case 1801:  /* xop_frczv4sf2 */
    case 1800:  /* xop_frczdf2 */
    case 1799:  /* xop_frczsf2 */
    case 1778:  /* xop_pperm_pack_v8hi_v16qi */
    case 1777:  /* xop_pperm_pack_v4si_v8hi */
    case 1776:  /* xop_pperm_pack_v2di_v4si */
    case 1775:  /* xop_pperm */
    case 1774:  /* xop_phsubdq */
    case 1773:  /* xop_phsubwd */
    case 1772:  /* xop_phsubbw */
    case 1771:  /* xop_phaddudq */
    case 1770:  /* xop_phadduwq */
    case 1769:  /* xop_phadduwd */
    case 1768:  /* xop_phaddubq */
    case 1767:  /* xop_phaddubd */
    case 1766:  /* xop_phaddubw */
    case 1765:  /* xop_phadddq */
    case 1764:  /* xop_phaddwq */
    case 1763:  /* xop_phaddwd */
    case 1762:  /* xop_phaddbq */
    case 1761:  /* xop_phaddbd */
    case 1760:  /* xop_phaddbw */
    case 1759:  /* xop_pcmov_v2df */
    case 1758:  /* xop_pcmov_v4df256 */
    case 1757:  /* xop_pcmov_v4sf */
    case 1756:  /* xop_pcmov_v8sf256 */
    case 1755:  /* xop_pcmov_v2di */
    case 1754:  /* xop_pcmov_v4di256 */
    case 1753:  /* xop_pcmov_v4si */
    case 1752:  /* xop_pcmov_v8si256 */
    case 1751:  /* xop_pcmov_v8hi */
    case 1750:  /* xop_pcmov_v16hi256 */
    case 1749:  /* xop_pcmov_v16qi */
    case 1748:  /* xop_pcmov_v32qi256 */
    case 1747:  /* xop_pmadcswd */
    case 1746:  /* xop_pmadcsswd */
    case 1745:  /* xop_pmacswd */
    case 1744:  /* xop_pmacsswd */
    case 1743:  /* xop_pmacsdqh */
    case 1742:  /* xop_pmacsdql */
    case 1741:  /* xop_pmacssdqh */
    case 1740:  /* xop_pmacssdql */
    case 1739:  /* xop_pmacssdd */
    case 1738:  /* xop_pmacsdd */
    case 1737:  /* xop_pmacssww */
    case 1736:  /* xop_pmacsww */
    case 1666:  /* sse4a_insertq */
    case 1665:  /* sse4a_insertqi */
    case 1664:  /* sse4a_extrq */
    case 1663:  /* sse4a_extrqi */
    case 1338:  /* sse3_movsldup */
    case 1337:  /* avx_movsldup256 */
    case 1336:  /* sse3_movshdup */
    case 1335:  /* avx_movshdup256 */
    case 1284:  /* *fma4i_vmfnmsub_v2df */
    case 1283:  /* *fma4i_vmfnmsub_v4sf */
    case 1282:  /* *fma4i_vmfnmadd_v2df */
    case 1281:  /* *fma4i_vmfnmadd_v4sf */
    case 1280:  /* *fma4i_vmfmsub_v2df */
    case 1279:  /* *fma4i_vmfmsub_v4sf */
    case 1278:  /* *fma4i_vmfmadd_v2df */
    case 1277:  /* *fma4i_vmfmadd_v4sf */
    case 1276:  /* *fmai_fnmsub_v2df */
    case 1275:  /* *fmai_fnmsub_v4sf */
    case 1274:  /* *fmai_fnmadd_v2df */
    case 1273:  /* *fmai_fnmadd_v4sf */
    case 1272:  /* *fmai_fmsub_v2df */
    case 1271:  /* *fmai_fmsub_v4sf */
    case 1270:  /* *fmai_fmadd_v2df */
    case 1269:  /* *fmai_fmadd_v4sf */
    case 1268:  /* *fma_fmsubadd_v2df */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1266:  /* *fma_fmsubadd_v4sf */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1264:  /* *fma_fmaddsub_v2df */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1262:  /* *fma_fmaddsub_v4sf */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1258:  /* *fma_fnmsub_v2df */
    case 1257:  /* *fma_fnmsub_v4sf */
    case 1256:  /* *fma_fnmsub_df */
    case 1255:  /* *fma_fnmsub_sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1252:  /* *fma_fnmadd_v2df */
    case 1251:  /* *fma_fnmadd_v4sf */
    case 1250:  /* *fma_fnmadd_df */
    case 1249:  /* *fma_fnmadd_sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1246:  /* *fma_fmsub_v2df */
    case 1245:  /* *fma_fmsub_v4sf */
    case 1244:  /* *fma_fmsub_df */
    case 1243:  /* *fma_fmsub_sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1240:  /* *fma_fmadd_v2df */
    case 1239:  /* *fma_fmadd_v4sf */
    case 1238:  /* *fma_fmadd_df */
    case 1237:  /* *fma_fmadd_sf */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1150:  /* avx_rsqrtv8sf2 */
    case 1141:  /* avx_rcpv8sf2 */
    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1064:  /* *mmx_concatv2si */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 994:  /* *mmx_concatv2sf */
    case 993:  /* *vec_dupv2sf */
    case 992:  /* mmx_pswapdv2sf2 */
    case 991:  /* mmx_floatv2si2 */
    case 990:  /* mmx_pi2fw */
    case 989:  /* mmx_pf2iw */
    case 988:  /* mmx_pf2id */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 972:  /* *mmx_mulv2sf3 */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
    case 969:  /* sse_movntq */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 920:  /* *prefetch_3dnow */
    case 899:  /* *xop_pcmov_df */
    case 898:  /* *xop_pcmov_sf */
    case 894:  /* *movxfcc_1 */
    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
    case 761:  /* *fop_xf_comm_i387 */
    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 756:  /* *fop_sf_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 749:  /* *fop_df_1_i387 */
    case 748:  /* *fop_sf_1_i387 */
    case 742:  /* *fop_df_comm_i387 */
    case 741:  /* *fop_sf_comm_i387 */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      return 0;

    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 323:  /* *mulqi3_1 */
    case 322:  /* *mulhi3_1 */
    case 320:  /* *muldi3_1 */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      return 10 /* 0xa */;

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 10 /* 0xa */;
        }
      else
        {
	  return 0;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return 9;

    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) || ((which_alternative != 1) || (get_attr_memory (insn) == MEMORY_NONE)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_OTHER) && (! ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_JEU)))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_OTHER) && ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_JEU))
        {
	  return 1;
        }
      else if (cached_type == TYPE_MULTI)
        {
	  return 9;
        }
      else if (((cached_type == TYPE_ALU) && ((((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_use_carry = get_attr_use_carry (insn)) == USE_CARRY_0)) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((cached_use_carry = get_attr_use_carry (insn)) == USE_CARRY_0))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_use_carry = get_attr_use_carry (insn)) == USE_CARRY_1))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((cached_use_carry = get_attr_use_carry (insn)) == USE_CARRY_1)))) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_IMOV) || ((cached_type == TYPE_IMOVX) && ((((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))) || (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))))))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMOVX) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_LEA) && (! ((cached_mode = get_attr_mode (insn)) == MODE_HI)))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_LEA) && ((cached_mode = get_attr_mode (insn)) == MODE_HI))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_INCDEC) || ((cached_type == TYPE_ISHIFT) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_prefix_0f = get_attr_prefix_0f (insn)) == 0)) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((cached_prefix_0f = get_attr_prefix_0f (insn)) == 0)))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_ISHIFT) && ((cached_prefix_0f = get_attr_prefix_0f (insn)) == 1))
        {
	  return 7;
        }
      else if ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || (cached_type == TYPE_ROTATE1)))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMUL) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_mode = get_attr_mode (insn)) == MODE_SI)) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((cached_mode = get_attr_mode (insn)) == MODE_SI))))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_IMUL) && (! ((cached_mode = get_attr_mode (insn)) == MODE_SI)))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_IDIV)
        {
	  return 65 /* 0x41 */;
        }
      else if ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_IBR) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IBR) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SETCC) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_SETCC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 2;
        }
      else if (cached_type == TYPE_ICMOV)
        {
	  return 1;
        }
      else if (cached_type == TYPE_PUSH)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_POP) && ((cached_mode = get_attr_mode (insn)) == MODE_DI))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_POP) && (! ((cached_mode = get_attr_mode (insn)) == MODE_DI)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_LEAVE) || (cached_type == TYPE_STR))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_SSELOG) || ((cached_type == TYPE_SSELOG1) || ((cached_type == TYPE_SSEIADD) && ((! (register_operand (operands[0], V2DImode))) && ((! ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_SIMUL)) && (! (cached_atom_unit == ATOM_UNIT_COMPLEX)))))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((! (register_operand (operands[0], V2DImode))) && (((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_SIMUL) && ((cached_mode = get_attr_mode (insn)) == MODE_DI))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((! (register_operand (operands[0], V2DImode))) && (((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_SIMUL) && ((cached_mode = get_attr_mode (insn)) == MODE_TI))))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((register_operand (operands[0], V2DImode)) || ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_COMPLEX)))
        {
	  return 6;
        }
      else if (((cached_type == TYPE_SSEISHFT) && ((! ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_SISHUF)) && (immediate_operand (operands[2], VOIDmode)))) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_SSEISHFT) && (((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_SISHUF) && (immediate_operand (operands[2], VOIDmode))))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_SSEISHFT) && (! (immediate_operand (operands[2], VOIDmode))))
        {
	  return 2;
        }
      else if (cached_type == TYPE_SSEIMUL)
        {
	  return 1;
        }
      else if ((cached_type == TYPE_SSE) && (((cached_atom_sse_attr = get_attr_atom_sse_attr (insn)) == ATOM_SSE_ATTR_RCP) && ((cached_mode = get_attr_mode (insn)) == MODE_SF)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSE) && ((((cached_atom_sse_attr = get_attr_atom_sse_attr (insn)) == ATOM_SSE_ATTR_MOVDUP) || (cached_atom_sse_attr == ATOM_SSE_ATTR_LFENCE)) || ((cached_atom_sse_attr == ATOM_SSE_ATTR_FENCE) || (cached_atom_sse_attr == ATOM_SSE_ATTR_PREFETCH))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_SSE) && ((((cached_atom_sse_attr = get_attr_atom_sse_attr (insn)) == ATOM_SSE_ATTR_SQRT) || (cached_atom_sse_attr == ATOM_SSE_ATTR_MXCSR)) || ((cached_atom_sse_attr == ATOM_SSE_ATTR_RCP) && ((cached_mode = get_attr_mode (insn)) == MODE_V4SF))))
        {
	  return 7;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((register_operand (operands[0], VOIDmode)) && ((register_operand (operands[1], VOIDmode)) || (register_operand (operands[1], VOIDmode)))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_movu = get_attr_movu (insn)) == MOVU_0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 1;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_movu = get_attr_movu (insn)) == MOVU_1) || (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSEADD) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_V2DF)) && (! ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_COMPLEX)))) || ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_V2DF)) && (! ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_COMPLEX))))))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSEADD) && (((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_COMPLEX)))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSEMUL) && (! ((cached_mode = get_attr_mode (insn)) == MODE_SF)))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((cached_mode = get_attr_mode (insn)) == MODE_SF))
        {
	  return 4;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 1;
        }
      else if (cached_type == TYPE_SSECOMI)
        {
	  return 10 /* 0xa */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))) || (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode))))))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSECVT) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 7;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((register_operand (operands[0], V2DFmode)) && (memory_operand (operands[1], SImode))))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((! (register_operand (operands[0], V2DFmode))) || (! (memory_operand (operands[1], SImode)))))
        {
	  return 8;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 62 /* 0x3e */;
        }
      else if (cached_type == TYPE_FMOV)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    default:
      return 1;

    }
}

int
internal_dfa_insn_code_atom (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_atom_unit cached_atom_unit ATTRIBUTE_UNUSED;
  enum attr_use_carry cached_use_carry ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  int cached_prefix_0f ATTRIBUTE_UNUSED;
  enum attr_atom_sse_attr cached_atom_sse_attr ATTRIBUTE_UNUSED;
  enum attr_movu cached_movu ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 2008:  /* *sse2_lfence */
      return 509 /* 0x1fd */;

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative != 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 512 /* 0x200 */;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 513 /* 0x201 */;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 514 /* 0x202 */;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 515 /* 0x203 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x5))
        {
	  return 495 /* 0x1ef */;
        }
      else
        {
	  return 496 /* 0x1f0 */;
        }

    case 1631:  /* ssse3_pmaddubsw */
      extract_insn_cached (insn);
      if (! (register_operand (operands[0], V2DImode)))
        {
	  return 500 /* 0x1f4 */;
        }
      else
        {
	  return 502 /* 0x1f6 */;
        }

    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
      return 502 /* 0x1f6 */;

    case 1595:  /* avx2_psadbw */
      extract_insn_cached (insn);
      if (register_operand (operands[0], V2DImode))
        {
	  return 502 /* 0x1f6 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 495 /* 0x1ef */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 496 /* 0x1f0 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 495 /* 0x1ef */;
        }
      else if ((((1 << which_alternative) & 0x63)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 496 /* 0x1f0 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((which_alternative == 3) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((which_alternative == 3) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((which_alternative == 3) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((which_alternative == 3) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6))
        {
	  return 504 /* 0x1f8 */;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 512 /* 0x200 */;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 513 /* 0x201 */;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 514 /* 0x202 */;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 515 /* 0x203 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((which_alternative == 4) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 504 /* 0x1f8 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 443 /* 0x1bb */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else
        {
	  return 455 /* 0x1c7 */;
        }

    case 1649:  /* ssse3_palignrdi */
    case 1648:  /* ssse3_palignrti */
    case 1647:  /* avx2_palignrv2ti */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
      extract_insn_cached (insn);
      if (immediate_operand (operands[2], VOIDmode))
        {
	  return 504 /* 0x1f8 */;
        }
      else
        {
	  return 505 /* 0x1f9 */;
        }

    case 1923:  /* avx2_lshrvv2di */
    case 1922:  /* avx2_ashlvv2di */
    case 1921:  /* avx2_lshrvv4di */
    case 1920:  /* avx2_ashlvv4di */
    case 1919:  /* avx2_lshrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1917:  /* avx2_lshrvv8si */
    case 1916:  /* avx2_ashlvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1914:  /* avx2_ashrvv8si */
    case 1798:  /* xop_shlv2di3 */
    case 1797:  /* xop_shlv4si3 */
    case 1796:  /* xop_shlv8hi3 */
    case 1795:  /* xop_shlv16qi3 */
    case 1794:  /* xop_shav2di3 */
    case 1793:  /* xop_shav4si3 */
    case 1792:  /* xop_shav8hi3 */
    case 1791:  /* xop_shav16qi3 */
    case 1790:  /* xop_vrotlv2di3 */
    case 1789:  /* xop_vrotlv4si3 */
    case 1788:  /* xop_vrotlv8hi3 */
    case 1787:  /* xop_vrotlv16qi3 */
    case 1786:  /* xop_rotrv2di3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1779:  /* xop_rotlv16qi3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1446:  /* ashrv4si3 */
    case 1445:  /* ashrv8si3 */
    case 1444:  /* ashrv8hi3 */
    case 1443:  /* ashrv16hi3 */
      extract_insn_cached (insn);
      if (immediate_operand (operands[2], VOIDmode))
        {
	  return 503 /* 0x1f7 */;
        }
      else
        {
	  return 505 /* 0x1f9 */;
        }

    case 1630:  /* ssse3_pmaddubsw128 */
    case 1596:  /* sse2_psadbw */
    case 1437:  /* *sse2_pmaddwd */
      extract_insn_cached (insn);
      if (! (register_operand (operands[0], V2DImode)))
        {
	  return 501 /* 0x1f5 */;
        }
      else
        {
	  return 502 /* 0x1f6 */;
        }

    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
      return 506 /* 0x1fa */;

    case 1629:  /* avx2_pmaddubsw256 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1436:  /* *avx2_pmaddwd */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
      extract_insn_cached (insn);
      if (! (register_operand (operands[0], V2DImode)))
        {
	  return 499 /* 0x1f3 */;
        }
      else
        {
	  return 502 /* 0x1f6 */;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 495 /* 0x1ef */;
        }
      else if ((which_alternative == 5) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 496 /* 0x1f0 */;
        }
      else if ((which_alternative != 5) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((which_alternative != 5) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((which_alternative != 5) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((which_alternative != 5) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((which_alternative != 5) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((which_alternative == 10) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 495 /* 0x1ef */;
        }
      else if ((which_alternative == 5) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 496 /* 0x1f0 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((which_alternative == 9) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((which_alternative == 6) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 495 /* 0x1ef */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 496 /* 0x1f0 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((which_alternative == 5) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((which_alternative == 4) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((which_alternative == 3) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((which_alternative == 5) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((which_alternative == 4) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1391:  /* *vec_concatv2df */
    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 495 /* 0x1ef */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 496 /* 0x1f0 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 495 /* 0x1ef */;
        }
      else
        {
	  return 496 /* 0x1f0 */;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 443 /* 0x1bb */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 495 /* 0x1ef */;
        }
      else
        {
	  return 496 /* 0x1f0 */;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((which_alternative == 9) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 495 /* 0x1ef */;
        }
      else if ((((1 << which_alternative) & 0xc1)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 496 /* 0x1f0 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((which_alternative == 10) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 495 /* 0x1ef */;
        }
      else if ((which_alternative == 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 496 /* 0x1f0 */;
        }
      else if ((which_alternative == 1) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((which_alternative == 1) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((which_alternative == 1) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((which_alternative == 1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 495 /* 0x1ef */;
        }
      else if ((((1 << which_alternative) & 0xf)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 496 /* 0x1f0 */;
        }
      else if ((which_alternative == 4) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((which_alternative == 4) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((which_alternative == 4) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((which_alternative == 4) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative != 1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 512 /* 0x200 */;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 513 /* 0x201 */;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 514 /* 0x202 */;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 515 /* 0x203 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1588:  /* *vec_concatv4si */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 495 /* 0x1ef */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 496 /* 0x1f0 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1890:  /* vec_set_hi_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1510:  /* *andnotv2di3 */
    case 1509:  /* *andnotv4di3 */
    case 1508:  /* *andnotv4si3 */
    case 1507:  /* *andnotv8si3 */
    case 1506:  /* *andnotv8hi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1503:  /* *andnotv32qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1339:  /* avx_shufps256_1 */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1333:  /* avx_unpcklps256 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1331:  /* avx_unpckhps256 */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 495 /* 0x1ef */;
        }
      else
        {
	  return 496 /* 0x1f0 */;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
      return 529 /* 0x211 */;

    case 1133:  /* sse_vmmulv4sf3 */
      return 521 /* 0x209 */;

    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
      return 520 /* 0x208 */;

    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
      return 519 /* 0x207 */;

    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
    case 1102:  /* sse2_storedqu */
    case 1101:  /* avx_storedqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1099:  /* avx_loaddqu256 */
    case 1098:  /* sse2_storeupd */
    case 1097:  /* avx_storeupd256 */
    case 1096:  /* sse_storeups */
    case 1095:  /* avx_storeups256 */
    case 1094:  /* sse2_loadupd */
    case 1093:  /* avx_loadupd256 */
    case 1092:  /* sse_loadups */
    case 1091:  /* avx_loadups256 */
      extract_insn_cached (insn);
      if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 512 /* 0x200 */;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 513 /* 0x201 */;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 514 /* 0x202 */;
        }
      else
        {
	  return 516 /* 0x204 */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((which_alternative == 6) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((((1 << which_alternative) & 0xe)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((which_alternative == 5) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((which_alternative == 5) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((which_alternative == 5) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((which_alternative == 5) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((which_alternative == 5) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative == 1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((which_alternative == 3) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((which_alternative == 3) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((which_alternative == 3) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((which_alternative == 3) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((which_alternative == 4) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 445 /* 0x1bd */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative == 6) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))))
        {
	  return 524 /* 0x20c */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode)))))
        {
	  return 525 /* 0x20d */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 526 /* 0x20e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative == 9) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))))
        {
	  return 524 /* 0x20c */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode)))))
        {
	  return 525 /* 0x20d */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 526 /* 0x20e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 445 /* 0x1bd */;
        }
      else if ((((1 << which_alternative) & 0x240)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((((1 << which_alternative) & 0x240)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))))
        {
	  return 524 /* 0x20c */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode)))))
        {
	  return 525 /* 0x20d */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 526 /* 0x20e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((which_alternative == 8) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative == 8) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))))
        {
	  return 524 /* 0x20c */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode)))))
        {
	  return 525 /* 0x20d */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 526 /* 0x20e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 497 /* 0x1f1 */;
        }
      else
        {
	  return 498 /* 0x1f2 */;
        }

    case 2010:  /* mfence_sse2 */
    case 2009:  /* *sse_sfence */
    case 1607:  /* sse2_clflush */
    case 919:  /* *prefetch_sse */
      return 510 /* 0x1fe */;

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))
        {
	  return 461 /* 0x1cd */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))
        {
	  return 461 /* 0x1cd */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 517 /* 0x205 */;
        }
      else
        {
	  return 518 /* 0x206 */;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 445 /* 0x1bd */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 486 /* 0x1e6 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 487 /* 0x1e7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 486 /* 0x1e6 */;
        }
      else
        {
	  return 487 /* 0x1e7 */;
        }

    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
      return 448 /* 0x1c0 */;

    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 494 /* 0x1ee */;

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1679:  /* avx2_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1670:  /* sse4_1_blendpd */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1602:  /* sse2_pmovmskb */
    case 1601:  /* avx2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1599:  /* avx_movmskpd256 */
    case 1598:  /* sse_movmskps */
    case 1597:  /* avx_movmskps256 */
    case 1576:  /* sse2_loadld */
    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1352:  /* *vec_concatv4sf */
    case 1346:  /* sse_movss */
    case 1344:  /* sse_storelps */
    case 1343:  /* sse_loadhps */
    case 1342:  /* sse_storehps */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
    case 1110:  /* sse2_movntv2df */
    case 1109:  /* avx_movntv4df */
    case 1108:  /* sse_movntv4sf */
    case 1107:  /* avx_movntv8sf */
    case 1106:  /* sse2_movntidi */
    case 1105:  /* sse2_movntisi */
    case 1089:  /* sse2_movq128 */
    case 852:  /* movmsk_df */
      extract_insn_cached (insn);
      if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 512 /* 0x200 */;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 513 /* 0x201 */;
        }
      else if ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode)))
        {
	  return 514 /* 0x202 */;
        }
      else if (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 515 /* 0x203 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1606:  /* sse_stmxcsr */
    case 1605:  /* sse_ldmxcsr */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1147:  /* sse2_sqrtv2df2 */
    case 1146:  /* avx_sqrtv4df2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1144:  /* avx_sqrtv8sf2 */
    case 1142:  /* sse_rcpv4sf2 */
    case 780:  /* *sqrtdf2_sse */
    case 779:  /* *sqrtsf2_sse */
      return 511 /* 0x1ff */;

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 517 /* 0x205 */;
        }
      else if ((cached_type == TYPE_SSEADD) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 518 /* 0x206 */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 520 /* 0x208 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 529 /* 0x211 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 517 /* 0x205 */;
        }
      else if ((cached_type == TYPE_SSEADD) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 518 /* 0x206 */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 521 /* 0x209 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 529 /* 0x211 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1143:  /* sse_vmrcpv4sf2 */
    case 778:  /* *rsqrtsf2_sse */
    case 745:  /* *rcpsf2_sse */
      return 507 /* 0x1fb */;

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 517 /* 0x205 */;
        }
      else if ((cached_type == TYPE_SSEADD) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 518 /* 0x206 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 520 /* 0x208 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 529 /* 0x211 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 517 /* 0x205 */;
        }
      else if ((cached_type == TYPE_SSEADD) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 518 /* 0x206 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 521 /* 0x209 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 529 /* 0x211 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 517 /* 0x205 */;
        }
      else if ((! (mult_operator (operands[3], DFmode))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 518 /* 0x206 */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 520 /* 0x208 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 517 /* 0x205 */;
        }
      else if ((! (mult_operator (operands[3], SFmode))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 518 /* 0x206 */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 521 /* 0x209 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 517 /* 0x205 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 518 /* 0x206 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode)))
        {
	  return 520 /* 0x208 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 517 /* 0x205 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 518 /* 0x206 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))
        {
	  return 521 /* 0x209 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 447 /* 0x1bf */;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((which_alternative != 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 493 /* 0x1ed */;

    case 653:  /* split_stack_return */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
      return 444 /* 0x1bc */;

    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
      return 492 /* 0x1ec */;

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      return 491 /* 0x1eb */;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 482 /* 0x1e2 */;
        }
      else
        {
	  return 483 /* 0x1e3 */;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
      return 522 /* 0x20a */;

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 484 /* 0x1e4 */;
        }
      else
        {
	  return 485 /* 0x1e5 */;
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 472 /* 0x1d8 */;
        }
      else
        {
	  return 473 /* 0x1d9 */;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 470 /* 0x1d6 */;
        }
      else if ((which_alternative == 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 471 /* 0x1d7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 470 /* 0x1d6 */;
        }
      else
        {
	  return 471 /* 0x1d7 */;
        }

    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 468 /* 0x1d4 */;
        }
      else
        {
	  return 469 /* 0x1d5 */;
        }

    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 465 /* 0x1d1 */;
        }
      else if ((which_alternative == 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 466 /* 0x1d2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 456 /* 0x1c8 */;
        }
      else if ((which_alternative == 0) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 457 /* 0x1c9 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 458 /* 0x1ca */;
        }
      else if ((which_alternative == 0) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 459 /* 0x1cb */;
        }
      else if ((which_alternative == 0) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 460 /* 0x1cc */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 465 /* 0x1d1 */;
        }
      else if ((which_alternative == 1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 466 /* 0x1d2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if (((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 465 /* 0x1d1 */;
        }
      else if (((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 466 /* 0x1d2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if ((cached_type == TYPE_ALU) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if ((cached_type == TYPE_ISHIFT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 465 /* 0x1d1 */;
        }
      else if ((cached_type == TYPE_ISHIFT) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 466 /* 0x1d2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 493:  /* *ashlqi3_1_slp */
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if ((cached_type == TYPE_ALU) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if ((cached_type == TYPE_ISHIFT1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 468 /* 0x1d4 */;
        }
      else if ((cached_type == TYPE_ISHIFT1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 469 /* 0x1d5 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if ((cached_type == TYPE_ALU) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if (which_alternative == 2)
        {
	  return 461 /* 0x1cd */;
        }
      else if ((cached_type == TYPE_ISHIFT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 465 /* 0x1d1 */;
        }
      else if ((cached_type == TYPE_ISHIFT) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 466 /* 0x1d2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if (which_alternative == 1)
        {
	  return 461 /* 0x1cd */;
        }
      else if (((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 465 /* 0x1d1 */;
        }
      else if (((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 466 /* 0x1d2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 491:  /* *ashlhi3_1 */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if ((cached_type == TYPE_ALU) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if (which_alternative == 1)
        {
	  return 461 /* 0x1cd */;
        }
      else if ((cached_type == TYPE_ISHIFT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 465 /* 0x1d1 */;
        }
      else if ((cached_type == TYPE_ISHIFT) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 466 /* 0x1d2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 465 /* 0x1d1 */;
        }
      else
        {
	  return 466 /* 0x1d2 */;
        }

    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      return 467 /* 0x1d3 */;

    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 452 /* 0x1c4 */;
        }
      else
        {
	  return 453 /* 0x1c5 */;
        }

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if ((which_alternative == 2) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 456 /* 0x1c8 */;
        }
      else if ((which_alternative == 2) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 457 /* 0x1c9 */;
        }
      else if ((which_alternative == 2) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 458 /* 0x1ca */;
        }
      else if ((which_alternative == 2) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 459 /* 0x1cb */;
        }
      else if ((which_alternative == 2) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 460 /* 0x1cc */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 456 /* 0x1c8 */;
        }
      else if ((which_alternative == 3) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 457 /* 0x1c9 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 458 /* 0x1ca */;
        }
      else if ((which_alternative == 3) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 459 /* 0x1cb */;
        }
      else if ((which_alternative == 3) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 460 /* 0x1cc */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      return 477 /* 0x1dd */;

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 476 /* 0x1dc */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 474 /* 0x1da */;
        }
      else if ((which_alternative == 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 475 /* 0x1db */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 323:  /* *mulqi3_1 */
    case 322:  /* *mulhi3_1 */
    case 320:  /* *muldi3_1 */
      return 476 /* 0x1dc */;

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 328:  /* *mulsidi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 474 /* 0x1da */;
        }
      else
        {
	  return 475 /* 0x1db */;
        }

    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 448 /* 0x1c0 */;
        }
      else
        {
	  return 449 /* 0x1c1 */;
        }

    case 273:  /* *adddi_5 */
    case 266:  /* *adddi_4 */
    case 264:  /* *adddi_3 */
    case 259:  /* *adddi_2 */
      extract_insn_cached (insn);
      if ((! (incdec_operand (operands[2], DImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if ((! (incdec_operand (operands[2], DImode))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if ((incdec_operand (operands[2], DImode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 463 /* 0x1cf */;
        }
      else if ((incdec_operand (operands[2], DImode)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 464 /* 0x1d0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 272:  /* *addsi_5 */
    case 269:  /* *addsi_4 */
    case 265:  /* *addsi_3_zext */
    case 263:  /* *addsi_3 */
    case 260:  /* *addsi_2_zext */
    case 258:  /* *addsi_2 */
      extract_insn_cached (insn);
      if ((! (incdec_operand (operands[2], SImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if ((! (incdec_operand (operands[2], SImode))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if ((incdec_operand (operands[2], SImode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 463 /* 0x1cf */;
        }
      else if ((incdec_operand (operands[2], SImode)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 464 /* 0x1d0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 271:  /* *addhi_5 */
    case 268:  /* *addhi_4 */
    case 262:  /* *addhi_3 */
    case 257:  /* *addhi_2 */
      extract_insn_cached (insn);
      if ((! (incdec_operand (operands[2], HImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if ((! (incdec_operand (operands[2], HImode))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if ((incdec_operand (operands[2], HImode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 463 /* 0x1cf */;
        }
      else if ((incdec_operand (operands[2], HImode)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 464 /* 0x1d0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 270:  /* *addqi_5 */
    case 267:  /* *addqi_4 */
    case 261:  /* *addqi_3 */
    case 256:  /* *addqi_2 */
      extract_insn_cached (insn);
      if ((! (incdec_operand (operands[2], QImode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if ((! (incdec_operand (operands[2], QImode))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if ((incdec_operand (operands[2], QImode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 463 /* 0x1cf */;
        }
      else if ((incdec_operand (operands[2], QImode)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 464 /* 0x1d0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if ((! (incdec_operand (operands[1], QImode))) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return 450 /* 0x1c2 */;
        }
      else if ((! (incdec_operand (operands[1], QImode))) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 451 /* 0x1c3 */;
        }
      else if ((incdec_operand (operands[1], QImode)) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return 463 /* 0x1cf */;
        }
      else if ((incdec_operand (operands[1], QImode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return 464 /* 0x1d0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 5) && (! (incdec_operand (operands[2], QImode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if (((which_alternative != 5) && (! (incdec_operand (operands[2], QImode)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if (which_alternative == 5)
        {
	  return 461 /* 0x1cd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 463 /* 0x1cf */;
        }
      else
        {
	  return 464 /* 0x1d0 */;
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], HImode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if (((which_alternative != 3) && (! (incdec_operand (operands[2], HImode)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if (which_alternative == 3)
        {
	  return 461 /* 0x1cd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 463 /* 0x1cf */;
        }
      else
        {
	  return 464 /* 0x1d0 */;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 2) && (! (incdec_operand (operands[2], SImode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if (((which_alternative != 2) && (! (incdec_operand (operands[2], SImode)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if (which_alternative == 2)
        {
	  return 461 /* 0x1cd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 463 /* 0x1cf */;
        }
      else
        {
	  return 464 /* 0x1d0 */;
        }

    case 251:  /* *adddi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], DImode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if (((which_alternative != 3) && (! (incdec_operand (operands[2], DImode)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if (which_alternative == 3)
        {
	  return 461 /* 0x1cd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 463 /* 0x1cf */;
        }
      else
        {
	  return 464 /* 0x1d0 */;
        }

    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], SImode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 446 /* 0x1be */;
        }
      else if (((which_alternative != 3) && (! (incdec_operand (operands[2], SImode)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 447 /* 0x1bf */;
        }
      else if (which_alternative == 3)
        {
	  return 461 /* 0x1cd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 463 /* 0x1cf */;
        }
      else
        {
	  return 464 /* 0x1d0 */;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 446 /* 0x1be */;
        }
      else
        {
	  return 447 /* 0x1bf */;
        }

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
      return 461 /* 0x1cd */;

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((register_operand (operands[0], V2DFmode)) && (memory_operand (operands[1], SImode))))
        {
	  return 527 /* 0x20f */;
        }
      else if ((which_alternative != 0) && ((! (register_operand (operands[0], V2DFmode))) || (! (memory_operand (operands[1], SImode)))))
        {
	  return 528 /* 0x210 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((which_alternative == 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((register_operand (operands[0], V2DFmode)) && (memory_operand (operands[1], SImode))))
        {
	  return 527 /* 0x20f */;
        }
      else if ((which_alternative == 1) && ((! (register_operand (operands[0], V2DFmode))) || (! (memory_operand (operands[1], SImode)))))
        {
	  return 528 /* 0x210 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((which_alternative == 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 445 /* 0x1bd */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], V2DFmode)) && (memory_operand (operands[1], SImode))))
        {
	  return 527 /* 0x20f */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((! (register_operand (operands[0], V2DFmode))) || (! (memory_operand (operands[1], SImode)))))
        {
	  return 528 /* 0x210 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((which_alternative == 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      extract_insn_cached (insn);
      if ((register_operand (operands[0], V2DFmode)) && (memory_operand (operands[1], SImode)))
        {
	  return 527 /* 0x20f */;
        }
      else
        {
	  return 528 /* 0x210 */;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 445 /* 0x1bd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 530 /* 0x212 */;
        }
      else
        {
	  return 531 /* 0x213 */;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 445 /* 0x1bd */;
        }
      else if ((which_alternative == 1) && (((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))))
        {
	  return 524 /* 0x20c */;
        }
      else if ((which_alternative == 1) && (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode)))))
        {
	  return 525 /* 0x20d */;
        }
      else if ((which_alternative == 1) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 526 /* 0x20e */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((which_alternative == 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))))
        {
	  return 524 /* 0x20c */;
        }
      else if ((which_alternative == 1) && (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode)))))
        {
	  return 525 /* 0x20d */;
        }
      else if ((which_alternative == 1) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 526 /* 0x20e */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((which_alternative == 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 530 /* 0x212 */;
        }
      else
        {
	  return 531 /* 0x213 */;
        }

    case 1935:  /* vcvtps2ph256 */
    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1932:  /* vcvtph2ps256 */
    case 1931:  /* *vcvtph2ps_load */
    case 1930:  /* vcvtph2ps */
    case 1727:  /* sse4_1_roundsd */
    case 1726:  /* sse4_1_roundss */
    case 1725:  /* sse4_1_roundpd */
    case 1724:  /* avx_roundpd256 */
    case 1723:  /* sse4_1_roundps */
    case 1722:  /* avx_roundps256 */
    case 1328:  /* sse2_cvtps2pd */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1326:  /* avx_cvtps2pd256 */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1315:  /* sse2_cvtdq2pd */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1313:  /* floatv4siv4df2 */
    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1302:  /* sse2_cvtpi2pd */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1298:  /* avx_cvtps2dq256 */
    case 1297:  /* floatv4siv4sf2 */
    case 1296:  /* floatv8siv8sf2 */
    case 1287:  /* sse_cvttps2pi */
    case 1286:  /* sse_cvtps2pi */
    case 1285:  /* sse_cvtpi2ps */
    case 1112:  /* sse2_movntv2di */
    case 1111:  /* avx_movntv4di */
    case 810:  /* sse4_1_rounddf2 */
    case 809:  /* sse4_1_roundsf2 */
    case 141:  /* *truncdfsf_fast_sse */
    case 136:  /* *extendsfdf2_sse */
      extract_insn_cached (insn);
      if (((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode))))
        {
	  return 524 /* 0x20c */;
        }
      else if (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode))))
        {
	  return 525 /* 0x20d */;
        }
      else if (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode)))))
        {
	  return 526 /* 0x20e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && (((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))))
        {
	  return 524 /* 0x20c */;
        }
      else if ((which_alternative == 2) && (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode)))))
        {
	  return 525 /* 0x20d */;
        }
      else if ((which_alternative == 2) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 526 /* 0x20e */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 450 /* 0x1c2 */;
        }
      else if ((which_alternative == 1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 451 /* 0x1c3 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 456 /* 0x1c8 */;
        }
      else if ((which_alternative == 0) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 457 /* 0x1c9 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 458 /* 0x1ca */;
        }
      else if ((which_alternative == 0) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 459 /* 0x1cb */;
        }
      else if ((which_alternative == 0) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 460 /* 0x1cc */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 445 /* 0x1bd */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((which_alternative == 1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 456 /* 0x1c8 */;
        }
      else if ((which_alternative == 0) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 457 /* 0x1c9 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 458 /* 0x1ca */;
        }
      else if ((which_alternative == 0) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 459 /* 0x1cb */;
        }
      else if ((which_alternative == 0) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 460 /* 0x1cc */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((((1 << which_alternative) & 0x18)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative == 5) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 445 /* 0x1bd */;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((((1 << which_alternative) & 0x220)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((((1 << which_alternative) & 0x78)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((which_alternative == 7) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative == 7) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 445 /* 0x1bd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 530 /* 0x212 */;
        }
      else
        {
	  return 531 /* 0x213 */;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative == 2) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 443 /* 0x1bb */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative == 2) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 445 /* 0x1bd */;
        }
      else
        {
	  return 488 /* 0x1e8 */;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((cached_type == TYPE_IMOV) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((cached_type == TYPE_IMOVX) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 456 /* 0x1c8 */;
        }
      else if ((cached_type == TYPE_IMOVX) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 457 /* 0x1c9 */;
        }
      else if ((cached_type == TYPE_IMOVX) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 458 /* 0x1ca */;
        }
      else if ((cached_type == TYPE_IMOVX) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 459 /* 0x1cb */;
        }
      else if ((cached_type == TYPE_IMOVX) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 460 /* 0x1cc */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 456 /* 0x1c8 */;
        }
      else if (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 457 /* 0x1c9 */;
        }
      else if (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 458 /* 0x1ca */;
        }
      else if (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 459 /* 0x1cb */;
        }
      else if (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 460 /* 0x1cc */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))
        {
	  return 456 /* 0x1c8 */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode)))))
        {
	  return 457 /* 0x1c9 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))
        {
	  return 458 /* 0x1ca */;
        }
      else if ((! (cached_memory == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode)))))
        {
	  return 459 /* 0x1cb */;
        }
      else if ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode)))
        {
	  return 460 /* 0x1cc */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 454 /* 0x1c6 */;
        }
      else
        {
	  return 455 /* 0x1c7 */;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 455 /* 0x1c7 */;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 456 /* 0x1c8 */;
        }
      else if ((cached_type == TYPE_IMOVX) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 457 /* 0x1c9 */;
        }
      else if ((cached_type == TYPE_IMOVX) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 458 /* 0x1ca */;
        }
      else if ((cached_type == TYPE_IMOVX) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 459 /* 0x1cb */;
        }
      else if ((cached_type == TYPE_IMOVX) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 460 /* 0x1cc */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((cached_type == TYPE_IMOV) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 456 /* 0x1c8 */;
        }
      else if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 457 /* 0x1c9 */;
        }
      else if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 458 /* 0x1ca */;
        }
      else if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 459 /* 0x1cb */;
        }
      else if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 460 /* 0x1cc */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 461 /* 0x1cd */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative == 6) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 445 /* 0x1bd */;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((((1 << which_alternative) & 0x220)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))))
        {
	  return 524 /* 0x20c */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode)))))
        {
	  return 525 /* 0x20d */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 526 /* 0x20e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 461 /* 0x1cd */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative == 9) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))))
        {
	  return 524 /* 0x20c */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode)))))
        {
	  return 525 /* 0x20d */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 526 /* 0x20e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 443 /* 0x1bb */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative == 2) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1088:  /* *movv2df_internal */
    case 1087:  /* *movv4df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1085:  /* *movv8sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1083:  /* *movv2ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1081:  /* *movv4di_internal */
    case 1080:  /* *movv4si_internal */
    case 1079:  /* *movv8si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1075:  /* *movv32qi_internal */
    case 61:  /* *movti_internal_sse */
    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((which_alternative == 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((which_alternative != 0) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((which_alternative != 0) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((which_alternative != 0) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((which_alternative != 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 515 /* 0x203 */;
        }
      else if ((which_alternative != 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 516 /* 0x204 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 450 /* 0x1c2 */;
        }
      else
        {
	  return 451 /* 0x1c3 */;
        }

    case 54:  /* *popdi1_epilogue */
    case 52:  /* *popdi1 */
      return 489 /* 0x1e9 */;

    case 53:  /* *popsi1_epilogue */
    case 51:  /* *popsi1 */
      return 490 /* 0x1ea */;

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      return 488 /* 0x1e8 */;

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 445 /* 0x1bd */;
        }
      else
        {
	  return 488 /* 0x1e8 */;
        }

    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      return 523 /* 0x20b */;

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 523 /* 0x20b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 918:  /* trap */
    case 853:  /* cld */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 650:  /* eh_return_internal */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
      return 443 /* 0x1bb */;

    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return 445 /* 0x1bd */;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 14:  /* *cmpqi_ext_2 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 480 /* 0x1e0 */;
        }
      else
        {
	  return 481 /* 0x1e1 */;
        }

    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 478 /* 0x1de */;
        }
      else
        {
	  return 479 /* 0x1df */;
        }

    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 478 /* 0x1de */;
        }
      else if ((which_alternative == 1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 479 /* 0x1df */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 480 /* 0x1e0 */;
        }
      else if ((which_alternative == 0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 481 /* 0x1e1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_OTHER) && (! ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_JEU)))
        {
	  return 443 /* 0x1bb */;
        }
      else if ((cached_type == TYPE_OTHER) && ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_JEU))
        {
	  return 444 /* 0x1bc */;
        }
      else if (cached_type == TYPE_MULTI)
        {
	  return 445 /* 0x1bd */;
        }
      else if ((cached_type == TYPE_ALU) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_use_carry = get_attr_use_carry (insn)) == USE_CARRY_0)))
        {
	  return 446 /* 0x1be */;
        }
      else if ((cached_type == TYPE_ALU) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((cached_use_carry = get_attr_use_carry (insn)) == USE_CARRY_0)))
        {
	  return 447 /* 0x1bf */;
        }
      else if ((cached_type == TYPE_ALU) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_use_carry = get_attr_use_carry (insn)) == USE_CARRY_1)))
        {
	  return 448 /* 0x1c0 */;
        }
      else if ((cached_type == TYPE_ALU) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((cached_use_carry = get_attr_use_carry (insn)) == USE_CARRY_1)))
        {
	  return 449 /* 0x1c1 */;
        }
      else if ((cached_type == TYPE_ALU1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 450 /* 0x1c2 */;
        }
      else if ((cached_type == TYPE_ALU1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 451 /* 0x1c3 */;
        }
      else if ((cached_type == TYPE_NEGNOT) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 452 /* 0x1c4 */;
        }
      else if ((cached_type == TYPE_NEGNOT) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 453 /* 0x1c5 */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 454 /* 0x1c6 */;
        }
      else if ((cached_type == TYPE_IMOV) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 455 /* 0x1c7 */;
        }
      else if ((cached_type == TYPE_IMOVX) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 456 /* 0x1c8 */;
        }
      else if ((cached_type == TYPE_IMOVX) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && (((register_operand (operands[0], HImode)) && (general_operand (operands[1], HImode))) || ((register_operand (operands[0], SImode)) && (general_operand (operands[1], SImode))))))
        {
	  return 457 /* 0x1c9 */;
        }
      else if ((cached_type == TYPE_IMOVX) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 458 /* 0x1ca */;
        }
      else if ((cached_type == TYPE_IMOVX) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((register_operand (operands[0], QImode)) || (((register_operand (operands[0], SImode)) && (! (general_operand (operands[1], SImode)))) || (register_operand (operands[0], DImode))))))
        {
	  return 459 /* 0x1cb */;
        }
      else if ((cached_type == TYPE_IMOVX) && ((register_operand (operands[0], HImode)) && (general_operand (operands[1], QImode))))
        {
	  return 460 /* 0x1cc */;
        }
      else if ((cached_type == TYPE_LEA) && (! ((cached_mode = get_attr_mode (insn)) == MODE_HI)))
        {
	  return 461 /* 0x1cd */;
        }
      else if ((cached_type == TYPE_LEA) && ((cached_mode = get_attr_mode (insn)) == MODE_HI))
        {
	  return 462 /* 0x1ce */;
        }
      else if ((cached_type == TYPE_INCDEC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 463 /* 0x1cf */;
        }
      else if ((cached_type == TYPE_INCDEC) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 464 /* 0x1d0 */;
        }
      else if ((cached_type == TYPE_ISHIFT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_prefix_0f = get_attr_prefix_0f (insn)) == 0)))
        {
	  return 465 /* 0x1d1 */;
        }
      else if ((cached_type == TYPE_ISHIFT) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((cached_prefix_0f = get_attr_prefix_0f (insn)) == 0)))
        {
	  return 466 /* 0x1d2 */;
        }
      else if ((cached_type == TYPE_ISHIFT) && ((cached_prefix_0f = get_attr_prefix_0f (insn)) == 1))
        {
	  return 467 /* 0x1d3 */;
        }
      else if ((cached_type == TYPE_ISHIFT1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 468 /* 0x1d4 */;
        }
      else if ((cached_type == TYPE_ISHIFT1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 469 /* 0x1d5 */;
        }
      else if ((cached_type == TYPE_ROTATE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 470 /* 0x1d6 */;
        }
      else if ((cached_type == TYPE_ROTATE) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 471 /* 0x1d7 */;
        }
      else if ((cached_type == TYPE_ROTATE1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 472 /* 0x1d8 */;
        }
      else if ((cached_type == TYPE_ROTATE1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 473 /* 0x1d9 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((cached_mode = get_attr_mode (insn)) == MODE_SI)))
        {
	  return 474 /* 0x1da */;
        }
      else if ((cached_type == TYPE_IMUL) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((cached_mode = get_attr_mode (insn)) == MODE_SI)))
        {
	  return 475 /* 0x1db */;
        }
      else if ((cached_type == TYPE_IMUL) && (! ((cached_mode = get_attr_mode (insn)) == MODE_SI)))
        {
	  return 476 /* 0x1dc */;
        }
      else if (cached_type == TYPE_IDIV)
        {
	  return 477 /* 0x1dd */;
        }
      else if ((cached_type == TYPE_ICMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 478 /* 0x1de */;
        }
      else if ((cached_type == TYPE_ICMP) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 479 /* 0x1df */;
        }
      else if ((cached_type == TYPE_TEST) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 480 /* 0x1e0 */;
        }
      else if ((cached_type == TYPE_TEST) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 481 /* 0x1e1 */;
        }
      else if ((cached_type == TYPE_IBR) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 482 /* 0x1e2 */;
        }
      else if ((cached_type == TYPE_IBR) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 483 /* 0x1e3 */;
        }
      else if ((cached_type == TYPE_SETCC) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 484 /* 0x1e4 */;
        }
      else if ((cached_type == TYPE_SETCC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 485 /* 0x1e5 */;
        }
      else if ((cached_type == TYPE_ICMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 486 /* 0x1e6 */;
        }
      else if ((cached_type == TYPE_ICMOV) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 487 /* 0x1e7 */;
        }
      else if (cached_type == TYPE_PUSH)
        {
	  return 488 /* 0x1e8 */;
        }
      else if ((cached_type == TYPE_POP) && ((cached_mode = get_attr_mode (insn)) == MODE_DI))
        {
	  return 489 /* 0x1e9 */;
        }
      else if ((cached_type == TYPE_POP) && (! ((cached_mode = get_attr_mode (insn)) == MODE_DI)))
        {
	  return 490 /* 0x1ea */;
        }
      else if (cached_type == TYPE_CALL)
        {
	  return 491 /* 0x1eb */;
        }
      else if (cached_type == TYPE_CALLV)
        {
	  return 492 /* 0x1ec */;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 493 /* 0x1ed */;
        }
      else if (cached_type == TYPE_STR)
        {
	  return 494 /* 0x1ee */;
        }
      else if ((cached_type == TYPE_SSELOG) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 495 /* 0x1ef */;
        }
      else if ((cached_type == TYPE_SSELOG) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 496 /* 0x1f0 */;
        }
      else if ((cached_type == TYPE_SSELOG1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 497 /* 0x1f1 */;
        }
      else if ((cached_type == TYPE_SSELOG1) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 498 /* 0x1f2 */;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((! (register_operand (operands[0], V2DImode))) && ((! ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_SIMUL)) && (! (cached_atom_unit == ATOM_UNIT_COMPLEX)))))
        {
	  return 499 /* 0x1f3 */;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((! (register_operand (operands[0], V2DImode))) && (((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_SIMUL) && ((cached_mode = get_attr_mode (insn)) == MODE_DI))))
        {
	  return 500 /* 0x1f4 */;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((! (register_operand (operands[0], V2DImode))) && (((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_SIMUL) && ((cached_mode = get_attr_mode (insn)) == MODE_TI))))
        {
	  return 501 /* 0x1f5 */;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((register_operand (operands[0], V2DImode)) || ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_COMPLEX)))
        {
	  return 502 /* 0x1f6 */;
        }
      else if ((cached_type == TYPE_SSEISHFT) && ((! ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_SISHUF)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return 503 /* 0x1f7 */;
        }
      else if ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_SSEISHFT) && (((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_SISHUF) && (immediate_operand (operands[2], VOIDmode)))))
        {
	  return 504 /* 0x1f8 */;
        }
      else if ((cached_type == TYPE_SSEISHFT) && (! (immediate_operand (operands[2], VOIDmode))))
        {
	  return 505 /* 0x1f9 */;
        }
      else if (cached_type == TYPE_SSEIMUL)
        {
	  return 506 /* 0x1fa */;
        }
      else if ((cached_type == TYPE_SSE) && (((cached_atom_sse_attr = get_attr_atom_sse_attr (insn)) == ATOM_SSE_ATTR_RCP) && ((cached_mode = get_attr_mode (insn)) == MODE_SF)))
        {
	  return 507 /* 0x1fb */;
        }
      else if ((cached_type == TYPE_SSE) && ((cached_atom_sse_attr = get_attr_atom_sse_attr (insn)) == ATOM_SSE_ATTR_MOVDUP))
        {
	  return 508 /* 0x1fc */;
        }
      else if ((cached_type == TYPE_SSE) && ((cached_atom_sse_attr = get_attr_atom_sse_attr (insn)) == ATOM_SSE_ATTR_LFENCE))
        {
	  return 509 /* 0x1fd */;
        }
      else if ((cached_type == TYPE_SSE) && (((cached_atom_sse_attr = get_attr_atom_sse_attr (insn)) == ATOM_SSE_ATTR_FENCE) || (cached_atom_sse_attr == ATOM_SSE_ATTR_PREFETCH)))
        {
	  return 510 /* 0x1fe */;
        }
      else if ((cached_type == TYPE_SSE) && ((((cached_atom_sse_attr = get_attr_atom_sse_attr (insn)) == ATOM_SSE_ATTR_SQRT) || (cached_atom_sse_attr == ATOM_SSE_ATTR_MXCSR)) || ((cached_atom_sse_attr == ATOM_SSE_ATTR_RCP) && ((cached_mode = get_attr_mode (insn)) == MODE_V4SF))))
        {
	  return 511 /* 0x1ff */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 512 /* 0x200 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 513 /* 0x201 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((register_operand (operands[0], VOIDmode)) && (register_operand (operands[1], VOIDmode))))
        {
	  return 514 /* 0x202 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_movu = get_attr_movu (insn)) == MOVU_0) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 515 /* 0x203 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_movu = get_attr_movu (insn)) == MOVU_1) || (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 516 /* 0x204 */;
        }
      else if ((cached_type == TYPE_SSEADD) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_V2DF)) && (! ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_COMPLEX)))))
        {
	  return 517 /* 0x205 */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) && ((! ((cached_mode = get_attr_mode (insn)) == MODE_V2DF)) && (! ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_COMPLEX)))))
        {
	  return 518 /* 0x206 */;
        }
      else if ((cached_type == TYPE_SSEADD) && (((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_atom_unit = get_attr_atom_unit (insn)) == ATOM_UNIT_COMPLEX)))
        {
	  return 519 /* 0x207 */;
        }
      else if ((cached_type == TYPE_SSEMUL) && (! ((cached_mode = get_attr_mode (insn)) == MODE_SF)))
        {
	  return 520 /* 0x208 */;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((cached_mode = get_attr_mode (insn)) == MODE_SF))
        {
	  return 521 /* 0x209 */;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 522 /* 0x20a */;
        }
      else if (cached_type == TYPE_SSECOMI)
        {
	  return 523 /* 0x20b */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((register_operand (operands[0], V2SImode)) && (register_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (register_operand (operands[1], V2SImode)))))
        {
	  return 524 /* 0x20c */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((register_operand (operands[0], V2SImode)) && (memory_operand (operands[1], V4SFmode))) || ((register_operand (operands[0], V4SFmode)) && (memory_operand (operands[1], V2SImode)))))
        {
	  return 525 /* 0x20d */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((! (register_operand (operands[0], V2SImode))) || (! (nonimmediate_operand (operands[1], V4SFmode)))) && ((! (register_operand (operands[0], V4SFmode))) || (! (nonimmediate_operand (operands[1], V2SImode))))))
        {
	  return 526 /* 0x20e */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((register_operand (operands[0], V2DFmode)) && (memory_operand (operands[1], SImode))))
        {
	  return 527 /* 0x20f */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((! (register_operand (operands[0], V2DFmode))) || (! (memory_operand (operands[1], SImode)))))
        {
	  return 528 /* 0x210 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 529 /* 0x211 */;
        }
      else if ((cached_type == TYPE_FMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 530 /* 0x212 */;
        }
      else if ((cached_type == TYPE_FMOV) && (! ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 531 /* 0x213 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    default:
      return 618 /* 0x26a */;

    }
}

int
insn_default_latency_generic64 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  enum attr_athlon_decode cached_athlon_decode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && (((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || (which_alternative != 0))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xc)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0xc))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x63)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x63))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x5))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 4) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x2f))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (!((1 << which_alternative) & 0x7)))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x6)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x6))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x9))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 5)
        {
	  return 2;
        }
      else if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (!((1 << which_alternative) & 0x420))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 5)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0x33))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0x19))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) || ((!((1 << which_alternative) & 0x6c1)) && (memory_operand (operands[1], DFmode))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x3b)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if (!((1 << which_alternative) & 0x6c1))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) || ((!((1 << which_alternative) & 0x6c1)) && (memory_operand (operands[1], DFmode))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x39)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if (!((1 << which_alternative) & 0x6c1))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if (!((1 << which_alternative) & 0x5))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (memory_operand (operands[1], DFmode)))
        {
	  return 2;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if (!((1 << which_alternative) & 0x2f))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (which_alternative == 1)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1352:  /* *vec_concatv4sf */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1322:  /* sse2_cvtsd2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 12 /* 0xc */;
        }
      else
        {
	  return 0;
        }

    case 1325:  /* *sse2_cvtpd2ps */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1318:  /* *sse2_cvtpd2dq */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_NONE))
        {
	  return 8;
        }
      else
        {
	  return 0;
        }

    case 1302:  /* sse2_cvtpi2pd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (get_attr_memory (insn) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 9;
        }
      else
        {
	  return 0;
        }

    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 14 /* 0xe */;
        }
      else
        {
	  return 0;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else
        {
	  return 20 /* 0x14 */;
        }

    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 35 /* 0x23 */;
        }
      else
        {
	  return 39 /* 0x27 */;
        }

    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 7;
        }
      else
        {
	  return 5;
        }

    case 1661:  /* sse4a_vmmovntv4sf */
    case 1659:  /* sse4a_movntsf */
    case 1601:  /* avx2_pmovmskb */
    case 1346:  /* sse_movss */
    case 1106:  /* sse2_movntidi */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 2;
        }
      else if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 1;
        }
      else
        {
	  return 2;
        }

    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1672:  /* sse4_1_blendvps */
    case 1670:  /* sse4_1_blendpd */
    case 1668:  /* sse4_1_blendps */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1600:  /* sse2_movmskpd */
    case 1598:  /* sse_movmskps */
    case 1110:  /* sse2_movntv2df */
    case 1108:  /* sse_movntv4sf */
    case 1104:  /* sse3_lddqu */
    case 1102:  /* sse2_storedqu */
    case 1100:  /* sse2_loaddqu */
    case 1098:  /* sse2_storeupd */
    case 1096:  /* sse_storeups */
    case 1094:  /* sse2_loadupd */
    case 1092:  /* sse_loadups */
    case 1089:  /* sse2_movq128 */
      extract_insn_cached (insn);
      if ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1088:  /* *movv2df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1087:  /* *movv4df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1086:  /* *movv4sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1084:  /* *movv1ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1083:  /* *movv2ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1082:  /* *movv2di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1081:  /* *movv4di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1080:  /* *movv4si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1079:  /* *movv8si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1078:  /* *movv8hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1077:  /* *movv16hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1076:  /* *movv16qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((which_alternative == 5) && (memory_operand (operands[1], DFmode))) || (((1 << which_alternative) & 0x30)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x11)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x11))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) || ((which_alternative == 3) && (memory_operand (operands[1], DFmode))))
        {
	  return 2;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x5))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 1)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || (which_alternative != 0))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 972:  /* *mmx_mulv2sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 3;
        }

    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1287:  /* sse_cvttps2pi */
    case 1286:  /* sse_cvtps2pi */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 993:  /* *vec_dupv2sf */
    case 992:  /* mmx_pswapdv2sf2 */
    case 991:  /* mmx_floatv2si2 */
    case 990:  /* mmx_pi2fw */
    case 989:  /* mmx_pf2iw */
    case 988:  /* mmx_pf2id */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 2;
        }
      else if (((!((1 << which_alternative) & 0xc7f)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || ((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0xc71))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x3f))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0x300)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((!((1 << which_alternative) & 0xf00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((((1 << which_alternative) & 0x300)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x2c7))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xfc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0xfc))
        {
	  return 2;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 9)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0x640)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((!((1 << which_alternative) & 0x1e40)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((((1 << which_alternative) & 0x640)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x6271))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x3f))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && (memory_operand (operands[1], DFmode)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if (!((1 << which_alternative) & 0x1c7))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xfc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0xfc))
        {
	  return 2;
        }
      else if ((which_alternative == 8) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 8)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 15 /* 0xf */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 17 /* 0x11 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 15 /* 0xf */;
        }
      else
        {
	  return 0;
        }

    case 894:  /* *movxfcc_1 */
      return 15 /* 0xf */;

    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 6;

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 100 /* 0x64 */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], XFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], XFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], XFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 491:  /* *ashlhi3_1 */
    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 9;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 7;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 328:  /* *mulsidi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 7;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 323:  /* *mulqi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if ((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 322:  /* *mulhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if (((which_alternative == 0) && (! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((which_alternative != 0) || (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 320:  /* *muldi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 7;
        }
      else if (((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((which_alternative != 1) && ((which_alternative != 2) || (! (memory_operand (operands[1], VOIDmode)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) || ((which_alternative == 1) || ((which_alternative == 2) && (memory_operand (operands[1], VOIDmode))))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if (((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((which_alternative != 1) && ((which_alternative != 2) || (! (memory_operand (operands[1], VOIDmode)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) || ((which_alternative == 1) || ((which_alternative == 2) && (memory_operand (operands[1], VOIDmode))))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 1) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 9;
        }
      else if ((which_alternative != 1) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xb)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 9;
        }
      else if ((!((1 << which_alternative) & 0xb)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 11 /* 0xb */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 14 /* 0xe */;
        }
      else
        {
	  return 0;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1323:  /* sse2_cvtss2sd */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 810:  /* sse4_1_rounddf2 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 136:  /* *extendsfdf2_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 2;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 13 /* 0xd */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_LOAD))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && (which_alternative == 1))
        {
	  return 8;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else
        {
	  return 2;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if (((!((1 << which_alternative) & 0x1f)) && (memory_operand (operands[1], DFmode))) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else
        {
	  return 2;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0xf)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) || (((!((1 << which_alternative) & 0xce3f)) && (memory_operand (operands[1], DFmode))) || (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (((!((1 << which_alternative) & 0x3f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x3f))))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 5)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) || ((!((1 << which_alternative) & 0x23f)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x23f))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) || (((!((1 << which_alternative) & 0xff)) && (memory_operand (operands[1], DFmode))) || (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((((1 << which_alternative) & 0x1858)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 3;
        }
      else if (((!((1 << which_alternative) & 0xff)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) || (!((1 << which_alternative) & 0xff))))
        {
	  return 2;
        }
      else if ((which_alternative == 7) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 7)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 3;

    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 1;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((cached_type == TYPE_IMOV) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if (((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0xfb8))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3c)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x3c))
        {
	  return 2;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 6)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0xaa0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((!((1 << which_alternative) & 0x1ea0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((((1 << which_alternative) & 0xaa0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x6227))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x1c)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x1c))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((memory_operand (operands[1], DFmode)) || ((((1 << which_alternative) & 0xa00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((!((1 << which_alternative) & 0xa01)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((((1 << which_alternative) & 0xa00)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x7de0))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x1f0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (((1 << which_alternative) & 0x1f0))
        {
	  return 2;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 9)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((memory_operand (operands[1], DFmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 2;
        }
      else if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 4;

    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1890:  /* vec_set_hi_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1510:  /* *andnotv2di3 */
    case 1509:  /* *andnotv4di3 */
    case 1508:  /* *andnotv4si3 */
    case 1507:  /* *andnotv8si3 */
    case 1506:  /* *andnotv8hi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1503:  /* *andnotv32qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1339:  /* avx_shufps256_1 */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1333:  /* avx_unpcklps256 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1331:  /* avx_unpckhps256 */
    case 1328:  /* sse2_cvtps2pd */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 3;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1685:  /* avx2_pblendvb */
    case 1679:  /* avx2_movntdqa */
    case 1673:  /* avx_blendvpd256 */
    case 1671:  /* avx_blendvps256 */
    case 1669:  /* avx_blendpd256 */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1660:  /* sse4a_movntdf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1602:  /* sse2_pmovmskb */
    case 1599:  /* avx_movmskpd256 */
    case 1597:  /* avx_movmskps256 */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1109:  /* avx_movntv4df */
    case 1107:  /* avx_movntv8sf */
    case 1105:  /* sse2_movntisi */
    case 1103:  /* avx_lddqu256 */
    case 1101:  /* avx_storedqu256 */
    case 1099:  /* avx_loaddqu256 */
    case 1097:  /* avx_storeupd256 */
    case 1095:  /* avx_storeups256 */
    case 1093:  /* avx_loadupd256 */
    case 1091:  /* avx_loadups256 */
    case 1090:  /* movdi_to_sse */
    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 969:  /* sse_movntq */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 920:  /* *prefetch_3dnow */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 853:  /* cld */
    case 852:  /* movmsk_df */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 127:  /* extendsidi2_1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
    case 42:  /* *pushdi2_rex64 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
      return 2;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_IBR) || ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV)))
        {
	  return 0;
        }
      else if (cached_type == TYPE_PUSH)
        {
	  return 2;
        }
      else if (cached_type == TYPE_POP)
        {
	  return 4;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 3;
        }
      else if (cached_type == TYPE_LEA)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))))
        {
	  return 7;
        }
      else if (((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))) || ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))))
        {
	  return 6;
        }
      else if (((cached_athlon_decode = get_attr_athlon_decode (insn)) == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 1;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 4;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 6;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 1;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_type == TYPE_FMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FISTP) || (cached_type == TYPE_FISTTP))
        {
	  return 4;
        }
      else if (cached_type == TYPE_FMOV)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 4;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else if ((cached_type == TYPE_FPSPC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 103 /* 0x67 */;
        }
      else if (cached_type == TYPE_FPSPC)
        {
	  return 100 /* 0x64 */;
        }
      else if ((cached_type == TYPE_FCMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 17 /* 0x11 */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 15 /* 0xf */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 5;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && (cached_type == TYPE_FCMP))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FCMP) || ((cached_type == TYPE_SSEMOV) && ((memory_operand (operands[1], DFmode)) || ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 1;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 3;
        }
      else if ((((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || (((cached_type == TYPE_SSEMOV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI)))) || ((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_MMXMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_MMXMUL) || (((cached_unit = get_attr_unit (insn)) == UNIT_MMX) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 3;
        }
      else if (cached_unit == UNIT_MMX)
        {
	  return 2;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_SSECMP) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECMP) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI)))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 3;
        }
      else if ((cached_type == TYPE_SSECOMI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSECOMI)
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEADD) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEADD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((cached_mode = get_attr_mode (insn)) == MODE_DF)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI)))))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 11 /* 0xb */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 14 /* 0xe */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 12 /* 0xc */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_NONE)))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSEICVT) && (((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if (cached_type == TYPE_SSEMUL)
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 22 /* 0x16 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 35 /* 0x23 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 39 /* 0x27 */;
        }
      else
        {
	  return 0;
        }

    default:
      return 0;

    }
}

int
internal_dfa_insn_code_generic64 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  enum attr_athlon_decode cached_athlon_decode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xc)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x13))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0xc))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x63)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x1c))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x63))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 2)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 1)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x5))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 0)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 4)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x2f))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 235 /* 0xeb */;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 229 /* 0xe5 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x6)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x6))
        {
	  return 231 /* 0xe7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 3)
        {
	  return 229 /* 0xe5 */;
        }
      else if (((1 << which_alternative) & 0x9))
        {
	  return 231 /* 0xe7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 229 /* 0xe5 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 6)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 5) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative != 5)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 9)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x620))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 5)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 5)
        {
	  return 189 /* 0xbd */;
        }
      else if ((((1 << which_alternative) & 0x13)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x13)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x13))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 3)
        {
	  return 189 /* 0xbd */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 231 /* 0xe7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 4)
        {
	  return 189 /* 0xbd */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x9))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 164 /* 0xa4 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 10)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x3b)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x6c1))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 10)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x39)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x6c1))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x5))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 0)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x2f))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 235 /* 0xeb */;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 1)
        {
	  return 229 /* 0xe5 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 229 /* 0xe5 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 1)
        {
	  return 229 /* 0xe5 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1352:  /* *vec_concatv4sf */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 229 /* 0xe5 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1328:  /* sse2_cvtps2pd */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 269 /* 0x10d */;
        }
      else
        {
	  return 271 /* 0x10f */;
        }

    case 1322:  /* sse2_cvtsd2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 282 /* 0x11a */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 284 /* 0x11c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1325:  /* *sse2_cvtpd2ps */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1318:  /* *sse2_cvtpd2dq */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 286 /* 0x11e */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 288 /* 0x120 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1302:  /* sse2_cvtpi2pd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (get_attr_memory (insn) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (which_alternative == 0)
        {
	  return 235 /* 0xeb */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 290 /* 0x122 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 293 /* 0x125 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 276 /* 0x114 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 280 /* 0x118 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1890:  /* vec_set_hi_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1510:  /* *andnotv2di3 */
    case 1509:  /* *andnotv4di3 */
    case 1508:  /* *andnotv4si3 */
    case 1507:  /* *andnotv8si3 */
    case 1506:  /* *andnotv8hi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1503:  /* *andnotv32qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1339:  /* avx_shufps256_1 */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1333:  /* avx_unpcklps256 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1331:  /* avx_unpckhps256 */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 246 /* 0xf6 */;
        }
      else
        {
	  return 249 /* 0xf9 */;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 307 /* 0x133 */;
        }
      else
        {
	  return 308 /* 0x134 */;
        }

    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 310 /* 0x136 */;
        }
      else
        {
	  return 313 /* 0x139 */;
        }

    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 298 /* 0x12a */;
        }
      else
        {
	  return 299 /* 0x12b */;
        }

    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 301 /* 0x12d */;
        }
      else
        {
	  return 304 /* 0x130 */;
        }

    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 260 /* 0x104 */;
        }
      else
        {
	  return 263 /* 0x107 */;
        }

    case 1661:  /* sse4a_vmmovntv4sf */
    case 1659:  /* sse4a_movntsf */
    case 1601:  /* avx2_pmovmskb */
    case 1346:  /* sse_movss */
    case 1106:  /* sse2_movntidi */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 219 /* 0xdb */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1672:  /* sse4_1_blendvps */
    case 1670:  /* sse4_1_blendpd */
    case 1668:  /* sse4_1_blendps */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1600:  /* sse2_movmskpd */
    case 1598:  /* sse_movmskps */
    case 1110:  /* sse2_movntv2df */
    case 1108:  /* sse_movntv4sf */
    case 1104:  /* sse3_lddqu */
    case 1102:  /* sse2_storedqu */
    case 1100:  /* sse2_loaddqu */
    case 1098:  /* sse2_storeupd */
    case 1096:  /* sse_storeups */
    case 1094:  /* sse2_loadupd */
    case 1092:  /* sse_loadups */
    case 1089:  /* sse2_movq128 */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 216 /* 0xd8 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 224 /* 0xe0 */;
        }
      else
        {
	  return 229 /* 0xe5 */;
        }

    case 1088:  /* *movv2df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1087:  /* *movv4df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1086:  /* *movv4sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1084:  /* *movv1ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1083:  /* *movv2ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1082:  /* *movv2di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1081:  /* *movv4di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1080:  /* *movv4si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1079:  /* *movv8si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1078:  /* *movv8hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1077:  /* *movv16hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1076:  /* *movv16qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 5) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x30))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x11)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x11))
        {
	  return 235 /* 0xeb */;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 4)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 3) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x5))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 1)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 234 /* 0xea */;
        }
      else
        {
	  return 235 /* 0xeb */;
        }

    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 972:  /* *mmx_mulv2sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 232 /* 0xe8 */;
        }
      else
        {
	  return 233 /* 0xe9 */;
        }

    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1287:  /* sse_cvttps2pi */
    case 1286:  /* sse_cvtps2pi */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 993:  /* *vec_dupv2sf */
    case 992:  /* mmx_pswapdv2sf2 */
    case 991:  /* mmx_floatv2si2 */
    case 990:  /* mmx_pi2fw */
    case 989:  /* mmx_pf2iw */
    case 988:  /* mmx_pf2id */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 234 /* 0xea */;
        }
      else
        {
	  return 235 /* 0xeb */;
        }

    case 969:  /* sse_movntq */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 7)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0xc71))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x3f))
        {
	  return 235 /* 0xeb */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((((1 << which_alternative) & 0x300)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((!((1 << which_alternative) & 0xf00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((((1 << which_alternative) & 0x300)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((((1 << which_alternative) & 0x300)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 8)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x2c7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xfc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0xfc))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 9)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((((1 << which_alternative) & 0x640)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((!((1 << which_alternative) & 0x1e40)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((((1 << which_alternative) & 0x640)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((((1 << which_alternative) & 0x640)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 10)
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x6271))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x3f))
        {
	  return 235 /* 0xeb */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x1c7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0xfc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0xfc))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 8) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 8)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
      return 240 /* 0xf0 */;

    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 920:  /* *prefetch_3dnow */
      return 235 /* 0xeb */;

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 257 /* 0x101 */;
        }
      else
        {
	  return 258 /* 0x102 */;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else
        {
	  return 206 /* 0xce */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 205 /* 0xcd */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 206 /* 0xce */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 894:  /* *movxfcc_1 */
      return 206 /* 0xce */;

    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
      return 162 /* 0xa2 */;

    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 161 /* 0xa1 */;

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1685:  /* avx2_pblendvb */
    case 1679:  /* avx2_movntdqa */
    case 1673:  /* avx_blendvpd256 */
    case 1671:  /* avx_blendvps256 */
    case 1669:  /* avx_blendpd256 */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1660:  /* sse4a_movntdf */
    case 1602:  /* sse2_pmovmskb */
    case 1599:  /* avx_movmskpd256 */
    case 1597:  /* avx_movmskps256 */
    case 1109:  /* avx_movntv4df */
    case 1107:  /* avx_movntv8sf */
    case 1105:  /* sse2_movntisi */
    case 1103:  /* avx_lddqu256 */
    case 1101:  /* avx_storedqu256 */
    case 1099:  /* avx_loaddqu256 */
    case 1097:  /* avx_storeupd256 */
    case 1095:  /* avx_storeups256 */
    case 1093:  /* avx_loadupd256 */
    case 1091:  /* avx_loadups256 */
    case 852:  /* movmsk_df */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], DFmode))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 202 /* 0xca */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((mult_operator (operands[3], XFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], XFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (! (mult_operator (operands[3], XFmode)))
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 194 /* 0xc2 */;
        }
      else
        {
	  return 195 /* 0xc3 */;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 307 /* 0x133 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 307 /* 0x133 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 307 /* 0x133 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 307 /* 0x133 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 194 /* 0xc2 */;
        }
      else
        {
	  return 195 /* 0xc3 */;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 194 /* 0xc2 */;
        }
      else
        {
	  return 195 /* 0xc3 */;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 258 /* 0x102 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 298 /* 0x12a */;
        }
      else
        {
	  return 299 /* 0x12b */;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 258 /* 0x102 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 298 /* 0x12a */;
        }
      else
        {
	  return 299 /* 0x12b */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 192 /* 0xc0 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 192 /* 0xc0 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 167 /* 0xa7 */;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 149 /* 0x95 */;

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      return 142 /* 0x8e */;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      return 141 /* 0x8d */;

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 243 /* 0xf3 */;
        }
      else
        {
	  return 244 /* 0xf4 */;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 162 /* 0xa2 */;
        }
      else
        {
	  return 176 /* 0xb0 */;
        }

    case 491:  /* *ashlhi3_1 */
    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 164 /* 0xa4 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 169 /* 0xa9 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 174 /* 0xae */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
      return 196 /* 0xc4 */;

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 159 /* 0x9f */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 160 /* 0xa0 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 153 /* 0x99 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 157 /* 0x9d */;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 328:  /* *mulsidi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 158 /* 0x9e */;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 153 /* 0x99 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 157 /* 0x9d */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 158 /* 0x9e */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 169 /* 0xa9 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 174 /* 0xae */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 154 /* 0x9a */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 158 /* 0x9e */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 169 /* 0xa9 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 174 /* 0xae */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((which_alternative == 0) && (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 323:  /* *mulqi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 158 /* 0x9e */;
        }
      else if ((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && (cached_memory == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 322:  /* *mulhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 158 /* 0x9e */;
        }
      else if (((which_alternative == 0) && (! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((which_alternative != 0) || (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 320:  /* *muldi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 153 /* 0x99 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 157 /* 0x9d */;
        }
      else if (((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((which_alternative != 1) && ((which_alternative != 2) || (! (memory_operand (operands[1], VOIDmode)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) || ((which_alternative == 1) || ((which_alternative == 2) && (memory_operand (operands[1], VOIDmode))))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 158 /* 0x9e */;
        }
      else if (((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))) && ((which_alternative != 1) && ((which_alternative != 2) || (! (memory_operand (operands[1], VOIDmode)))))) && (cached_memory == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if (((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) || ((which_alternative == 1) || ((which_alternative == 2) && (memory_operand (operands[1], VOIDmode))))) && (cached_memory == MEMORY_STORE))
        {
	  return 178 /* 0xb2 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 162 /* 0xa2 */;
        }
      else
        {
	  return 167 /* 0xa7 */;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 492:  /* *ashlqi3_1 */
    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
      return 150 /* 0x96 */;

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 276 /* 0x114 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 1) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 276 /* 0x114 */;
        }
      else if ((which_alternative != 1) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 276 /* 0x114 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 276 /* 0x114 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0xb)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 276 /* 0x114 */;
        }
      else if ((!((1 << which_alternative) & 0xb)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 278 /* 0x116 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 181 /* 0xb5 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 185 /* 0xb9 */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 181 /* 0xb5 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 185 /* 0xb9 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
      return 188 /* 0xbc */;

    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 276 /* 0x114 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 278 /* 0x116 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 280 /* 0x118 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 183 /* 0xb7 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 187 /* 0xbb */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 181 /* 0xb5 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 185 /* 0xb9 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 187 /* 0xbb */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 1323:  /* sse2_cvtss2sd */
    case 810:  /* sse4_1_rounddf2 */
    case 136:  /* *extendsfdf2_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 265 /* 0x109 */;
        }
      else
        {
	  return 267 /* 0x10b */;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 181 /* 0xb5 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if (((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && (which_alternative == 1))
        {
	  return 185 /* 0xb9 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_LOAD))
        {
	  return 265 /* 0x109 */;
        }
      else
        {
	  return 267 /* 0x10b */;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return 229 /* 0xe5 */;
        }
      else
        {
	  return 231 /* 0xe7 */;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (!((1 << which_alternative) & 0xf))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x3f))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 5)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x23f))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((((1 << which_alternative) & 0x1858)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 224 /* 0xe0 */;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0xff))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 7) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 7)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 181 /* 0xb5 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 185 /* 0xb9 */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 144 /* 0x90 */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 164 /* 0xa4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 164 /* 0xa4 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 174 /* 0xae */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 166 /* 0xa6 */;

    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 171 /* 0xab */;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_type == TYPE_IMOV) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 150 /* 0x96 */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if (((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 219 /* 0xdb */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 7)
        {
	  return 229 /* 0xe5 */;
        }
      else if (((1 << which_alternative) & 0xfb8))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x3c)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x3c))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 6)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((((1 << which_alternative) & 0xaa0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((!((1 << which_alternative) & 0x1ea0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((((1 << which_alternative) & 0xaa0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((((1 << which_alternative) & 0xaa0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (((1 << which_alternative) & 0x880))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x6227))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x1c)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x1c))
        {
	  return 235 /* 0xeb */;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 150 /* 0x96 */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 162 /* 0xa2 */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 167 /* 0xa7 */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 172 /* 0xac */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((((1 << which_alternative) & 0xa00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((!((1 << which_alternative) & 0xa01)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((((1 << which_alternative) & 0xa00)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((((1 << which_alternative) & 0xa00)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative == 11)
        {
	  return 229 /* 0xe5 */;
        }
      else if (((1 << which_alternative) & 0x7de0))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((((1 << which_alternative) & 0x1f0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (((1 << which_alternative) & 0x1f0))
        {
	  return 235 /* 0xeb */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if (which_alternative == 9)
        {
	  return 240 /* 0xf0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))))
        {
	  return 229 /* 0xe5 */;
        }
      else if (!((1 << which_alternative) & 0x7))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if (which_alternative != 0)
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 237 /* 0xed */;
        }
      else
        {
	  return 240 /* 0xf0 */;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 172 /* 0xac */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 176 /* 0xb0 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 145 /* 0x91 */;

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      return 144 /* 0x90 */;

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 144 /* 0x90 */;
        }
      else
        {
	  return 164 /* 0xa4 */;
        }

    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 208 /* 0xd0 */;
        }
      else
        {
	  return 209 /* 0xd1 */;
        }

    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 252 /* 0xfc */;
        }
      else
        {
	  return 254 /* 0xfe */;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 208 /* 0xd0 */;
        }
      else if (which_alternative == 0)
        {
	  return 209 /* 0xd1 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 252 /* 0xfc */;
        }
      else
        {
	  return 254 /* 0xfe */;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 853:  /* cld */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
      return 164 /* 0xa4 */;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 162 /* 0xa2 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 167 /* 0xa7 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_IBR)
        {
	  return 141 /* 0x8d */;
        }
      else if ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 142 /* 0x8e */;
        }
      else if (cached_type == TYPE_PUSH)
        {
	  return 144 /* 0x90 */;
        }
      else if (cached_type == TYPE_POP)
        {
	  return 145 /* 0x91 */;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 149 /* 0x95 */;
        }
      else if (cached_type == TYPE_LEA)
        {
	  return 150 /* 0x96 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 153 /* 0x99 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))))
        {
	  return 157 /* 0x9d */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 158 /* 0x9e */;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 159 /* 0x9f */;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 160 /* 0xa0 */;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))))
        {
	  return 161 /* 0xa1 */;
        }
      else if (((cached_athlon_decode = get_attr_athlon_decode (insn)) == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 162 /* 0xa2 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 164 /* 0xa4 */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 167 /* 0xa7 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 169 /* 0xa9 */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 172 /* 0xac */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 174 /* 0xae */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 176 /* 0xb0 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 178 /* 0xb2 */;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 181 /* 0xb5 */;
        }
      else if ((cached_type == TYPE_FMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 185 /* 0xb9 */;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if ((cached_type == TYPE_FISTP) || (cached_type == TYPE_FISTTP))
        {
	  return 188 /* 0xbc */;
        }
      else if (cached_type == TYPE_FMOV)
        {
	  return 189 /* 0xbd */;
        }
      else if ((cached_type == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_type == TYPE_FMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 195 /* 0xc3 */;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 196 /* 0xc4 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else if ((cached_type == TYPE_FPSPC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 201 /* 0xc9 */;
        }
      else if (cached_type == TYPE_FPSPC)
        {
	  return 202 /* 0xca */;
        }
      else if ((cached_type == TYPE_FCMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 205 /* 0xcd */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 206 /* 0xce */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 208 /* 0xd0 */;
        }
      else if ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && (cached_type == TYPE_FCMP))
        {
	  return 209 /* 0xd1 */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 211 /* 0xd3 */;
        }
      else if (cached_type == TYPE_FCMP)
        {
	  return 212 /* 0xd4 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (memory_operand (operands[1], DFmode)))
        {
	  return 215 /* 0xd7 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 216 /* 0xd8 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 219 /* 0xdb */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 221 /* 0xdd */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 224 /* 0xe0 */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 225 /* 0xe1 */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 226 /* 0xe2 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))))
        {
	  return 229 /* 0xe5 */;
        }
      else if ((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV))
        {
	  return 231 /* 0xe7 */;
        }
      else if ((cached_type == TYPE_MMXMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 232 /* 0xe8 */;
        }
      else if (cached_type == TYPE_MMXMUL)
        {
	  return 233 /* 0xe9 */;
        }
      else if (((cached_unit = get_attr_unit (insn)) == UNIT_MMX) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 234 /* 0xea */;
        }
      else if (cached_unit == UNIT_MMX)
        {
	  return 235 /* 0xeb */;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 237 /* 0xed */;
        }
      else if ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1))
        {
	  return 240 /* 0xf0 */;
        }
      else if ((cached_type == TYPE_SSECMP) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 243 /* 0xf3 */;
        }
      else if ((cached_type == TYPE_SSECMP) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI)))))
        {
	  return 244 /* 0xf4 */;
        }
      else if ((cached_type == TYPE_SSECMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 246 /* 0xf6 */;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 249 /* 0xf9 */;
        }
      else if ((cached_type == TYPE_SSECOMI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 252 /* 0xfc */;
        }
      else if (cached_type == TYPE_SSECOMI)
        {
	  return 254 /* 0xfe */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 257 /* 0x101 */;
        }
      else if ((cached_type == TYPE_SSEADD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))))
        {
	  return 258 /* 0x102 */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 260 /* 0x104 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 263 /* 0x107 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 265 /* 0x109 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DIRECT) && ((cached_mode = get_attr_mode (insn)) == MODE_DF)))
        {
	  return 267 /* 0x10b */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 269 /* 0x10d */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI)))))
        {
	  return 271 /* 0x10f */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 276 /* 0x114 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 278 /* 0x116 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 280 /* 0x118 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 282 /* 0x11a */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 284 /* 0x11c */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 286 /* 0x11e */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 288 /* 0x120 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 290 /* 0x122 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_athlon_decode == ATHLON_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 293 /* 0x125 */;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 298 /* 0x12a */;
        }
      else if ((cached_type == TYPE_SSEMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 301 /* 0x12d */;
        }
      else if (cached_type == TYPE_SSEMUL)
        {
	  return 304 /* 0x130 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 307 /* 0x133 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 308 /* 0x134 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 310 /* 0x136 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 313 /* 0x139 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    default:
      return 618 /* 0x26a */;

    }
}

int
insn_default_latency_amdfam10 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  enum attr_amdfam10_decode cached_amdfam10_decode ATTRIBUTE_UNUSED;
  enum attr_athlon_decode cached_athlon_decode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 4;
        }
      else
        {
	  return 2;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 4;
        }
      else
        {
	  return 2;
        }

    case 1666:  /* sse4a_insertq */
    case 1665:  /* sse4a_insertqi */
      return 5;

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x63)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0x63))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x6)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 2) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (((((1 << which_alternative) & 0x7)) && (cached_memory == MEMORY_BOTH)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 5)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) || ((!((1 << which_alternative) & 0x620)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 5)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) || ((((1 << which_alternative) & 0x13)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 3) || ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) || ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1391:  /* *vec_concatv2df */
    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 2;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) || ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative != 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative != 1)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1588:  /* *vec_concatv4si */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (((((1 << which_alternative) & 0xc)) && (cached_memory == MEMORY_BOTH)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (((which_alternative == 1) && (cached_memory == MEMORY_BOTH)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1352:  /* *vec_concatv4sf */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_BOTH)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1323:  /* sse2_cvtss2sd */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && (get_attr_memory (insn) == MEMORY_LOAD)) || (which_alternative == 0))
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 1322:  /* sse2_cvtsd2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 8;
        }
      else
        {
	  return 0;
        }

    case 1325:  /* *sse2_cvtpd2ps */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1318:  /* *sse2_cvtpd2dq */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 9;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1291:  /* sse_cvtss2si_2 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 10 /* 0xa */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 8;
        }
      else
        {
	  return 0;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 22 /* 0x16 */;
        }
      else
        {
	  return 20 /* 0x14 */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) || ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 1)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 969:  /* sse_movntq */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xc7f)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((!((1 << which_alternative) & 0xc71)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 6)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x38)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0x2ff)) && ((((1 << which_alternative) & 0x300)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((!((1 << which_alternative) & 0x2c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 9)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0x627f)) && ((((1 << which_alternative) & 0x640)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((!((1 << which_alternative) & 0x6271)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x240)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0x240))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x38)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 2;
        }
      else if ((which_alternative == 8) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 8)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode)))) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode)))) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 15 /* 0xf */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 17 /* 0x11 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 15 /* 0xf */;
        }
      else
        {
	  return 0;
        }

    case 894:  /* *movxfcc_1 */
      return 15 /* 0xf */;

    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 6;

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1679:  /* avx2_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1670:  /* sse4_1_blendpd */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1602:  /* sse2_pmovmskb */
    case 1601:  /* avx2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1599:  /* avx_movmskpd256 */
    case 1598:  /* sse_movmskps */
    case 1597:  /* avx_movmskps256 */
    case 1346:  /* sse_movss */
    case 1110:  /* sse2_movntv2df */
    case 1109:  /* avx_movntv4df */
    case 1108:  /* sse_movntv4sf */
    case 1107:  /* avx_movntv8sf */
    case 1106:  /* sse2_movntidi */
    case 1105:  /* sse2_movntisi */
    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
    case 1102:  /* sse2_storedqu */
    case 1101:  /* avx_storedqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1099:  /* avx_loaddqu256 */
    case 1098:  /* sse2_storeupd */
    case 1097:  /* avx_storeupd256 */
    case 1096:  /* sse_storeups */
    case 1095:  /* avx_storeups256 */
    case 1094:  /* sse2_loadupd */
    case 1093:  /* avx_loadupd256 */
    case 1092:  /* sse_loadups */
    case 1091:  /* avx_loadups256 */
    case 1089:  /* sse2_movq128 */
    case 852:  /* movmsk_df */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 100 /* 0x64 */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], XFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], XFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], XFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else
        {
	  return 0;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else
        {
	  return 0;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 4;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 4;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1930:  /* vcvtph2ps */
    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1890:  /* vec_set_hi_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1727:  /* sse4_1_roundsd */
    case 1726:  /* sse4_1_roundss */
    case 1725:  /* sse4_1_roundpd */
    case 1723:  /* sse4_1_roundps */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1510:  /* *andnotv2di3 */
    case 1509:  /* *andnotv4di3 */
    case 1508:  /* *andnotv4si3 */
    case 1507:  /* *andnotv8si3 */
    case 1506:  /* *andnotv8hi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1503:  /* *andnotv32qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1339:  /* avx_shufps256_1 */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1333:  /* avx_unpcklps256 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1331:  /* avx_unpckhps256 */
    case 1328:  /* sse2_cvtps2pd */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1315:  /* sse2_cvtdq2pd */
    case 1304:  /* sse2_cvttpd2pi */
    case 1302:  /* sse2_cvtpi2pd */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1297:  /* floatv4siv4sf2 */
    case 1285:  /* sse_cvtpi2ps */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1112:  /* sse2_movntv2di */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 2;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 9;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 328:  /* *mulsidi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 7;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 323:  /* *mulqi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 322:  /* *mulhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 320:  /* *muldi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 7;
        }
      else if (((!((1 << which_alternative) & 0x3)) || (! (memory_operand (operands[1], VOIDmode)))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], VOIDmode))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 3;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 6;
        }
      else if (((!((1 << which_alternative) & 0x3)) || (! (memory_operand (operands[1], VOIDmode)))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], VOIDmode))) && (cached_memory == MEMORY_STORE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 14 /* 0xe */;
        }
      else
        {
	  return 0;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 9;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 14 /* 0xe */;
        }
      else
        {
	  return 0;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 1) && (get_attr_memory (insn) == MEMORY_LOAD))
        {
	  return 9;
        }
      else
        {
	  return 0;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 14 /* 0xe */;
        }
      else
        {
	  return 0;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 9;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 14 /* 0xe */;
        }
      else
        {
	  return 0;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 9;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 14 /* 0xe */;
        }
      else
        {
	  return 0;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
      return 4;

    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 9;
        }
      else
        {
	  return 0;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 13 /* 0xd */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_LOAD))
        {
	  return 2;
        }
      else if (((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && (which_alternative == 1))
        {
	  return 8;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xf)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) || ((!((1 << which_alternative) & 0xce3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xce00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0x3f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 5)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) || ((!((1 << which_alternative) & 0x23f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) || (((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0xff)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))
        {
	  return 2;
        }
      else if ((which_alternative == 7) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 7)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 2)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 2;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (cached_memory == MEMORY_LOAD))
        {
	  return 3;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((cached_type == TYPE_IMOV) && (cached_memory == MEMORY_STORE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && ((pic_32bit_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x38)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0xf80)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((((1 << which_alternative) & 0xfb8)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 6)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0x623f)) && ((((1 << which_alternative) & 0xaa0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((!((1 << which_alternative) & 0x6227)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && ((pic_32bit_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x1e0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x7c00)) && ((((1 << which_alternative) & 0xa00)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((((1 << which_alternative) & 0x7de0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 9)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 2)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1088:  /* *movv2df_internal */
    case 1087:  /* *movv4df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1085:  /* *movv8sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1083:  /* *movv2ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1081:  /* *movv4di_internal */
    case 1080:  /* *movv4si_internal */
    case 1079:  /* *movv8si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1075:  /* *movv32qi_internal */
    case 61:  /* *movti_internal_sse */
    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 3;

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      return 2;

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 1;
        }

    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 39:  /* *cmpfp_iu_387 */
    case 38:  /* *cmpfp_iu_sse */
    case 36:  /* *cmpfp_i_i387 */
    case 35:  /* *cmpfp_i_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 3;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else
        {
	  return 3;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 853:  /* cld */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
      return 1;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_IBR) || ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV)))
        {
	  return 0;
        }
      else if (cached_type == TYPE_PUSH)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_POP) || (cached_type == TYPE_LEAVE))
        {
	  return 3;
        }
      else if (cached_type == TYPE_LEA)
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))))
        {
	  return 7;
        }
      else if (((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))) || ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))))
        {
	  return 6;
        }
      else if (((cached_amdfam10_decode = get_attr_amdfam10_decode (insn)) == AMDFAM10_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 1;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 3;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 4;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 1;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 6;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 1;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_type == TYPE_FMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FISTP) || (cached_type == TYPE_FISTTP))
        {
	  return 4;
        }
      else if (cached_type == TYPE_FMOV)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 4;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 11 /* 0xb */;
        }
      else if ((cached_type == TYPE_FPSPC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 103 /* 0x67 */;
        }
      else if (cached_type == TYPE_FPSPC)
        {
	  return 100 /* 0x64 */;
        }
      else if ((cached_type == TYPE_FCMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 17 /* 0x11 */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 15 /* 0xf */;
        }
      else if ((cached_type == TYPE_FCMP) && (((cached_athlon_decode = get_attr_athlon_decode (insn)) == ATHLON_DECODE_VECTOR) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 5;
        }
      else if (((cached_athlon_decode = get_attr_athlon_decode (insn)) == ATHLON_DECODE_VECTOR) && (cached_type == TYPE_FCMP))
        {
	  return 3;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_FCMP) || ((cached_type == TYPE_SSEMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_MMXMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 2;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECMP) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECMP) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI)))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECOMI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (cached_type == TYPE_SSECOMI)
        {
	  return 3;
        }
      else if ((cached_type == TYPE_SSEADD) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEADD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && ((cached_mode = get_attr_mode (insn)) == MODE_DF))))
        {
	  return 7;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DIRECT) && ((((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DIRECT) && (((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI)))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 14 /* 0xe */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 7;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 10 /* 0xa */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_TI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 9;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_TI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 7;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSEMUL)
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 22 /* 0x16 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 22 /* 0x16 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 20 /* 0x14 */;
        }
      else if ((cached_type == TYPE_SSEINS) && ((cached_mode = get_attr_mode (insn)) == MODE_TI))
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    default:
      return 0;

    }
}

int
internal_dfa_insn_code_amdfam10 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  enum attr_amdfam10_decode cached_amdfam10_decode ATTRIBUTE_UNUSED;
  enum attr_athlon_decode cached_athlon_decode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative != 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 238 /* 0xee */;
        }
      else
        {
	  return 241 /* 0xf1 */;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 238 /* 0xee */;
        }
      else
        {
	  return 241 /* 0xf1 */;
        }

    case 1666:  /* sse4a_insertq */
    case 1665:  /* sse4a_insertqi */
      return 315 /* 0x13b */;

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0x63))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 2) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 173 /* 0xad */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 228 /* 0xe4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 5)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 9)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 5)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 5)
        {
	  return 189 /* 0xbd */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x13)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 3)
        {
	  return 189 /* 0xbd */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 4)
        {
	  return 189 /* 0xbd */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 163 /* 0xa3 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 238 /* 0xee */;
        }
      else
        {
	  return 241 /* 0xf1 */;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 10)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative != 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative != 1)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 228 /* 0xe4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 228 /* 0xe4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1352:  /* *vec_concatv4sf */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 228 /* 0xe4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1323:  /* sse2_cvtss2sd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (get_attr_memory (insn) == MEMORY_LOAD))
        {
	  return 266 /* 0x10a */;
        }
      else if (which_alternative == 0)
        {
	  return 268 /* 0x10c */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1322:  /* sse2_cvtsd2ss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 283 /* 0x11b */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 285 /* 0x11d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1325:  /* *sse2_cvtpd2ps */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1318:  /* *sse2_cvtpd2dq */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 287 /* 0x11f */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 289 /* 0x121 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1291:  /* sse_cvtss2si_2 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 291 /* 0x123 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 294 /* 0x126 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1890:  /* vec_set_hi_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1510:  /* *andnotv2di3 */
    case 1509:  /* *andnotv4di3 */
    case 1508:  /* *andnotv4si3 */
    case 1507:  /* *andnotv8si3 */
    case 1506:  /* *andnotv8hi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1503:  /* *andnotv32qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1339:  /* avx_shufps256_1 */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1333:  /* avx_unpcklps256 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1331:  /* avx_unpckhps256 */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 238 /* 0xee */;
        }
      else
        {
	  return 241 /* 0xf1 */;
        }

    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 247 /* 0xf7 */;
        }
      else
        {
	  return 250 /* 0xfa */;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 307 /* 0x133 */;
        }
      else
        {
	  return 308 /* 0x134 */;
        }

    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 311 /* 0x137 */;
        }
      else
        {
	  return 314 /* 0x13a */;
        }

    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 298 /* 0x12a */;
        }
      else
        {
	  return 299 /* 0x12b */;
        }

    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 302 /* 0x12e */;
        }
      else
        {
	  return 305 /* 0x131 */;
        }

    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 261 /* 0x105 */;
        }
      else
        {
	  return 264 /* 0x108 */;
        }

    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1930:  /* vcvtph2ps */
    case 1727:  /* sse4_1_roundsd */
    case 1726:  /* sse4_1_roundss */
    case 1725:  /* sse4_1_roundpd */
    case 1723:  /* sse4_1_roundps */
    case 1328:  /* sse2_cvtps2pd */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1315:  /* sse2_cvtdq2pd */
    case 1304:  /* sse2_cvttpd2pi */
    case 1302:  /* sse2_cvtpi2pd */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1297:  /* floatv4siv4sf2 */
    case 1285:  /* sse_cvtpi2ps */
    case 1112:  /* sse2_movntv2di */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 270 /* 0x10e */;
        }
      else
        {
	  return 272 /* 0x110 */;
        }

    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1672:  /* sse4_1_blendvps */
    case 1670:  /* sse4_1_blendpd */
    case 1668:  /* sse4_1_blendps */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1600:  /* sse2_movmskpd */
    case 1598:  /* sse_movmskps */
    case 1110:  /* sse2_movntv2df */
    case 1108:  /* sse_movntv4sf */
    case 1104:  /* sse3_lddqu */
    case 1102:  /* sse2_storedqu */
    case 1100:  /* sse2_loaddqu */
    case 1098:  /* sse2_storeupd */
    case 1096:  /* sse_storeups */
    case 1094:  /* sse2_loadupd */
    case 1092:  /* sse_loadups */
    case 1089:  /* sse2_movq128 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 222 /* 0xde */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 227 /* 0xe3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1088:  /* *movv2df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1087:  /* *movv4df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1086:  /* *movv4sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1084:  /* *movv1ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1083:  /* *movv2ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1082:  /* *movv2di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1081:  /* *movv4di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1080:  /* *movv4si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1079:  /* *movv8si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1078:  /* *movv8hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1077:  /* *movv16hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1076:  /* *movv16qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 4)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 1)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 969:  /* sse_movntq */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 223 /* 0xdf */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 228 /* 0xe4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 6)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x38)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((((1 << which_alternative) & 0x300)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 9)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((((1 << which_alternative) & 0x640)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0x240)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0x240))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x38)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 8) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 8)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
      return 241 /* 0xf1 */;

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))
        {
	  return 151 /* 0x97 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 168 /* 0xa8 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 173 /* 0xad */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))
        {
	  return 151 /* 0x97 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 168 /* 0xa8 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 173 /* 0xad */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 257 /* 0x101 */;
        }
      else
        {
	  return 258 /* 0x102 */;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 163 /* 0xa3 */;
        }
      else
        {
	  return 206 /* 0xce */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 168 /* 0xa8 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 205 /* 0xcd */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 206 /* 0xce */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 894:  /* *movxfcc_1 */
      return 206 /* 0xce */;

    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 161 /* 0xa1 */;

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1685:  /* avx2_pblendvb */
    case 1679:  /* avx2_movntdqa */
    case 1673:  /* avx_blendvpd256 */
    case 1671:  /* avx_blendvps256 */
    case 1669:  /* avx_blendpd256 */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1602:  /* sse2_pmovmskb */
    case 1601:  /* avx2_pmovmskb */
    case 1599:  /* avx_movmskpd256 */
    case 1597:  /* avx_movmskps256 */
    case 1346:  /* sse_movss */
    case 1109:  /* avx_movntv4df */
    case 1107:  /* avx_movntv8sf */
    case 1106:  /* sse2_movntidi */
    case 1105:  /* sse2_movntisi */
    case 1103:  /* avx_lddqu256 */
    case 1101:  /* avx_storedqu256 */
    case 1099:  /* avx_loaddqu256 */
    case 1097:  /* avx_storeupd256 */
    case 1095:  /* avx_storeups256 */
    case 1093:  /* avx_loadupd256 */
    case 1091:  /* avx_loadups256 */
    case 852:  /* movmsk_df */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 222 /* 0xde */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 228 /* 0xe4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
      return 202 /* 0xca */;

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 168 /* 0xa8 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((mult_operator (operands[3], XFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], XFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (! (mult_operator (operands[3], XFmode)))
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 194 /* 0xc2 */;
        }
      else
        {
	  return 195 /* 0xc3 */;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 168 /* 0xa8 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 168 /* 0xa8 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 168 /* 0xa8 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 307 /* 0x133 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 168 /* 0xa8 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 307 /* 0x133 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 168 /* 0xa8 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 307 /* 0x133 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 168 /* 0xa8 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 307 /* 0x133 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 308 /* 0x134 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 194 /* 0xc2 */;
        }
      else
        {
	  return 195 /* 0xc3 */;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 194 /* 0xc2 */;
        }
      else
        {
	  return 195 /* 0xc3 */;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 258 /* 0x102 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 298 /* 0x12a */;
        }
      else
        {
	  return 299 /* 0x12b */;
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 258 /* 0x102 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 298 /* 0x12a */;
        }
      else
        {
	  return 299 /* 0x12b */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 192 /* 0xc0 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], DFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 192 /* 0xc0 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 195 /* 0xc3 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 257 /* 0x101 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 258 /* 0x102 */;
        }
      else if (((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 298 /* 0x12a */;
        }
      else if ((((1 << which_alternative) & 0x6)) && (mult_operator (operands[3], SFmode)))
        {
	  return 299 /* 0x12b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 168 /* 0xa8 */;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 168 /* 0xa8 */;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 173 /* 0xad */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 149 /* 0x95 */;

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      return 143 /* 0x8f */;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      return 141 /* 0x8d */;

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 243 /* 0xf3 */;
        }
      else
        {
	  return 244 /* 0xf4 */;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 163 /* 0xa3 */;
        }
      else
        {
	  return 177 /* 0xb1 */;
        }

    case 491:  /* *ashlhi3_1 */
    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 151 /* 0x97 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 168 /* 0xa8 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 173 /* 0xad */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 165 /* 0xa5 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 170 /* 0xaa */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 175 /* 0xaf */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 179 /* 0xb3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
      return 196 /* 0xc4 */;

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 159 /* 0x9f */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 160 /* 0xa0 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 153 /* 0x99 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 157 /* 0x9d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 328:  /* *mulsidi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 158 /* 0x9e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 153 /* 0x99 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 157 /* 0x9d */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 158 /* 0x9e */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 168 /* 0xa8 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 154 /* 0x9a */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 158 /* 0x9e */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 168 /* 0xa8 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 323:  /* *mulqi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 158 /* 0x9e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 322:  /* *mulhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 158 /* 0x9e */;
        }
      else if ((which_alternative == 2) && (cached_memory == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_STORE))
        {
	  return 179 /* 0xb3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 320:  /* *muldi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 153 /* 0x99 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 157 /* 0x9d */;
        }
      else if (((!((1 << which_alternative) & 0x3)) || (! (memory_operand (operands[1], VOIDmode)))) && (cached_memory == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if (((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], VOIDmode))) && (cached_memory == MEMORY_STORE))
        {
	  return 179 /* 0xb3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 158 /* 0x9e */;
        }
      else if (((!((1 << which_alternative) & 0x3)) || (! (memory_operand (operands[1], VOIDmode)))) && (cached_memory == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if (((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], VOIDmode))) && (cached_memory == MEMORY_STORE))
        {
	  return 179 /* 0xb3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 163 /* 0xa3 */;
        }
      else
        {
	  return 168 /* 0xa8 */;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 151 /* 0x97 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 168 /* 0xa8 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 173 /* 0xad */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 492:  /* *ashlqi3_1 */
    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 151 /* 0x97 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 168 /* 0xa8 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 173 /* 0xad */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return 151 /* 0x97 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 168 /* 0xa8 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 173 /* 0xad */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
      return 151 /* 0x97 */;

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 274 /* 0x112 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 279 /* 0x117 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 274 /* 0x112 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 279 /* 0x117 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 274 /* 0x112 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 274 /* 0x112 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 279 /* 0x117 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 274 /* 0x112 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 279 /* 0x117 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 274 /* 0x112 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 279 /* 0x117 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 181 /* 0xb5 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 185 /* 0xb9 */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 181 /* 0xb5 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 185 /* 0xb9 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
      return 188 /* 0xbc */;

    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 274 /* 0x112 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 183 /* 0xb7 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 187 /* 0xbb */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (which_alternative == 0)
        {
	  return 189 /* 0xbd */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 181 /* 0xb5 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 183 /* 0xb7 */;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 185 /* 0xb9 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 187 /* 0xbb */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 181 /* 0xb5 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if (((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && (which_alternative == 1))
        {
	  return 185 /* 0xb9 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 189 /* 0xbd */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 227 /* 0xe3 */;
        }
      else if (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 228 /* 0xe4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 168 /* 0xa8 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0xce00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 5)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 189 /* 0xbd */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 7) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 7)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 181 /* 0xb5 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 185 /* 0xb9 */;
        }
      else
        {
	  return 189 /* 0xbd */;
        }

    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 2)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 144 /* 0x90 */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 163 /* 0xa3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 163 /* 0xa3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 168 /* 0xa8 */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 173 /* 0xad */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 168 /* 0xa8 */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 173 /* 0xad */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 173 /* 0xad */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 171 /* 0xab */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 166 /* 0xa6 */;

    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 171 /* 0xab */;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 168 /* 0xa8 */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 173 /* 0xad */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (cached_memory == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 168 /* 0xa8 */;
        }
      else if ((cached_type == TYPE_IMOV) && (cached_memory == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 173 /* 0xad */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 151 /* 0x97 */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 168 /* 0xa8 */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x38)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 6)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((((1 << which_alternative) & 0xaa0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 151 /* 0x97 */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 163 /* 0xa3 */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 168 /* 0xa8 */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 173 /* 0xad */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((((1 << which_alternative) & 0x1e0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((((1 << which_alternative) & 0xa00)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 9)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 2)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if (which_alternative == 0)
        {
	  return 241 /* 0xf1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 173 /* 0xad */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 177 /* 0xb1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 147 /* 0x93 */;

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      return 144 /* 0x90 */;

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 144 /* 0x90 */;
        }
      else
        {
	  return 163 /* 0xa3 */;
        }

    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 208 /* 0xd0 */;
        }
      else
        {
	  return 209 /* 0xd1 */;
        }

    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 253 /* 0xfd */;
        }
      else
        {
	  return 255 /* 0xff */;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 208 /* 0xd0 */;
        }
      else if (which_alternative == 0)
        {
	  return 209 /* 0xd1 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 253 /* 0xfd */;
        }
      else
        {
	  return 255 /* 0xff */;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 853:  /* cld */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
      return 163 /* 0xa3 */;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 163 /* 0xa3 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 168 /* 0xa8 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_IBR)
        {
	  return 141 /* 0x8d */;
        }
      else if ((cached_type == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 143 /* 0x8f */;
        }
      else if (cached_type == TYPE_PUSH)
        {
	  return 144 /* 0x90 */;
        }
      else if (cached_type == TYPE_POP)
        {
	  return 147 /* 0x93 */;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 149 /* 0x95 */;
        }
      else if (cached_type == TYPE_LEA)
        {
	  return 151 /* 0x97 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 153 /* 0x99 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 154 /* 0x9a */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_HI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 155 /* 0x9b */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))))
        {
	  return 157 /* 0x9d */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 158 /* 0x9e */;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 159 /* 0x9f */;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 160 /* 0xa0 */;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))))
        {
	  return 161 /* 0xa1 */;
        }
      else if (((cached_amdfam10_decode = get_attr_amdfam10_decode (insn)) == AMDFAM10_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 163 /* 0xa3 */;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 165 /* 0xa5 */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 166 /* 0xa6 */;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 168 /* 0xa8 */;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 170 /* 0xaa */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 171 /* 0xab */;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 173 /* 0xad */;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 175 /* 0xaf */;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 177 /* 0xb1 */;
        }
      else if ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 179 /* 0xb3 */;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 181 /* 0xb5 */;
        }
      else if ((cached_type == TYPE_FMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 183 /* 0xb7 */;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 185 /* 0xb9 */;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 187 /* 0xbb */;
        }
      else if ((cached_type == TYPE_FISTP) || (cached_type == TYPE_FISTTP))
        {
	  return 188 /* 0xbc */;
        }
      else if (cached_type == TYPE_FMOV)
        {
	  return 189 /* 0xbd */;
        }
      else if ((cached_type == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 191 /* 0xbf */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 192 /* 0xc0 */;
        }
      else if ((cached_type == TYPE_FMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 194 /* 0xc2 */;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 195 /* 0xc3 */;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 196 /* 0xc4 */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 198 /* 0xc6 */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 200 /* 0xc8 */;
        }
      else if ((cached_type == TYPE_FPSPC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 201 /* 0xc9 */;
        }
      else if (cached_type == TYPE_FPSPC)
        {
	  return 202 /* 0xca */;
        }
      else if ((cached_type == TYPE_FCMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 205 /* 0xcd */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 206 /* 0xce */;
        }
      else if ((cached_type == TYPE_FCMP) && (((cached_athlon_decode = get_attr_athlon_decode (insn)) == ATHLON_DECODE_VECTOR) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 208 /* 0xd0 */;
        }
      else if (((cached_athlon_decode = get_attr_athlon_decode (insn)) == ATHLON_DECODE_VECTOR) && (cached_type == TYPE_FCMP))
        {
	  return 209 /* 0xd1 */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 211 /* 0xd3 */;
        }
      else if (cached_type == TYPE_FCMP)
        {
	  return 212 /* 0xd4 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 222 /* 0xde */;
        }
      else if ((cached_type == TYPE_MMXMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 223 /* 0xdf */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 227 /* 0xe3 */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 228 /* 0xe4 */;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 238 /* 0xee */;
        }
      else if ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1))
        {
	  return 241 /* 0xf1 */;
        }
      else if ((cached_type == TYPE_SSECMP) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 243 /* 0xf3 */;
        }
      else if ((cached_type == TYPE_SSECMP) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || ((cached_mode == MODE_DI) || (cached_mode == MODE_TI)))))
        {
	  return 244 /* 0xf4 */;
        }
      else if ((cached_type == TYPE_SSECMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 247 /* 0xf7 */;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 250 /* 0xfa */;
        }
      else if ((cached_type == TYPE_SSECOMI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 253 /* 0xfd */;
        }
      else if (cached_type == TYPE_SSECOMI)
        {
	  return 255 /* 0xff */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 257 /* 0x101 */;
        }
      else if ((cached_type == TYPE_SSEADD) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || ((cached_mode == MODE_DF) || (cached_mode == MODE_DI))))
        {
	  return 258 /* 0x102 */;
        }
      else if ((cached_type == TYPE_SSEADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 261 /* 0x105 */;
        }
      else if (cached_type == TYPE_SSEADD)
        {
	  return 264 /* 0x108 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 266 /* 0x10a */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && ((cached_mode = get_attr_mode (insn)) == MODE_DF)))
        {
	  return 268 /* 0x10c */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DIRECT) && ((((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 270 /* 0x10e */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DIRECT) && (((cached_mode = get_attr_mode (insn)) == MODE_V2DF) || ((cached_mode == MODE_V4SF) || (cached_mode == MODE_TI)))))
        {
	  return 272 /* 0x110 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 274 /* 0x112 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 277 /* 0x115 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 279 /* 0x117 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 281 /* 0x119 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 283 /* 0x11b */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_VECTOR) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 285 /* 0x11d */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 287 /* 0x11f */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 289 /* 0x121 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 291 /* 0x123 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 294 /* 0x126 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_TI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 295 /* 0x127 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((cached_amdfam10_decode == AMDFAM10_DECODE_DOUBLE) && (((cached_mode = get_attr_mode (insn)) == MODE_TI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))))
        {
	  return 296 /* 0x128 */;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 298 /* 0x12a */;
        }
      else if ((cached_type == TYPE_SSEMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 299 /* 0x12b */;
        }
      else if ((cached_type == TYPE_SSEMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 302 /* 0x12e */;
        }
      else if (cached_type == TYPE_SSEMUL)
        {
	  return 305 /* 0x131 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 307 /* 0x133 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 308 /* 0x134 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 311 /* 0x137 */;
        }
      else if (cached_type == TYPE_SSEDIV)
        {
	  return 314 /* 0x13a */;
        }
      else if ((cached_type == TYPE_SSEINS) && ((cached_mode = get_attr_mode (insn)) == MODE_TI))
        {
	  return 315 /* 0x13b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    default:
      return 618 /* 0x26a */;

    }
}

int
insn_default_latency_bdver1 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  enum attr_bdver1_decode cached_bdver1_decode ATTRIBUTE_UNUSED;
  enum attr_prefix cached_prefix ATTRIBUTE_UNUSED;
  enum attr_movu cached_movu ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1928:  /* avx_vec_concatv8sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 5;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if (which_alternative == 0)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1929:  /* avx_vec_concatv4df */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 5;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1862:  /* avx_vbroadcastf128_v8sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 5;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 3;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if (which_alternative != 0)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 5;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 3;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x63)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x63))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x6)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 2) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (((((1 << which_alternative) & 0x7)) && (cached_memory == MEMORY_BOTH)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1510:  /* *andnotv2di3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1509:  /* *andnotv4di3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1508:  /* *andnotv4si3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1507:  /* *andnotv8si3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1506:  /* *andnotv8hi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1505:  /* *andnotv16hi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1504:  /* *andnotv16qi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1503:  /* *andnotv32qi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1631:  /* ssse3_pmaddubsw */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1436:  /* *avx2_pmaddwd */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((which_alternative != 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((which_alternative != 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 5)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 9)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 5)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 6) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 5)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x13)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 3)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 4)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1391:  /* *vec_concatv2df */
    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 10)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 4;
        }
      else if (((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 4;
        }
      else if (((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative != 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative != 1)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1588:  /* *vec_concatv4si */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (((((1 << which_alternative) & 0xc)) && (cached_memory == MEMORY_BOTH)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (((which_alternative == 1) && (cached_memory == MEMORY_BOTH)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1352:  /* *vec_concatv4sf */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_BOTH)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1890:  /* vec_set_hi_v8sf */
    case 1888:  /* vec_set_lo_v8sf */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1339:  /* avx_shufps256_1 */
    case 1333:  /* avx_unpcklps256 */
    case 1331:  /* avx_unpckhps256 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1213:  /* avx_andnotv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 7;
        }
      else
        {
	  return 3;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 31 /* 0x1f */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 27 /* 0x1b */;
        }
      else
        {
	  return 0;
        }

    case 1139:  /* sse_vmdivv4sf3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 28 /* 0x1c */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 24 /* 0x18 */;
        }
      else
        {
	  return 0;
        }

    case 1677:  /* avx_dppd256 */
    case 1675:  /* avx_dpps256 */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1131:  /* *mulv4df3 */
    case 1129:  /* *mulv8sf3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 11 /* 0xb */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 1935:  /* vcvtps2ph256 */
    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1932:  /* vcvtph2ps256 */
    case 1931:  /* *vcvtph2ps_load */
    case 1930:  /* vcvtph2ps */
    case 1727:  /* sse4_1_roundsd */
    case 1726:  /* sse4_1_roundss */
    case 1725:  /* sse4_1_roundpd */
    case 1724:  /* avx_roundpd256 */
    case 1723:  /* sse4_1_roundps */
    case 1722:  /* avx_roundps256 */
    case 1328:  /* sse2_cvtps2pd */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1326:  /* avx_cvtps2pd256 */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1315:  /* sse2_cvtdq2pd */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1313:  /* floatv4siv4df2 */
    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1302:  /* sse2_cvtpi2pd */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1298:  /* avx_cvtps2dq256 */
    case 1297:  /* floatv4siv4sf2 */
    case 1296:  /* floatv8siv8sf2 */
    case 1286:  /* sse_cvtps2pi */
    case 1285:  /* sse_cvtpi2ps */
    case 1112:  /* sse2_movntv2di */
    case 1111:  /* avx_movntv4di */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode)))))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode)))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode))))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode)))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1685:  /* avx2_pblendvb */
    case 1679:  /* avx2_movntdqa */
    case 1673:  /* avx_blendvpd256 */
    case 1671:  /* avx_blendvps256 */
    case 1669:  /* avx_blendpd256 */
    case 1667:  /* avx_blendps256 */
    case 1599:  /* avx_movmskpd256 */
    case 1597:  /* avx_movmskps256 */
    case 1109:  /* avx_movntv4df */
    case 1107:  /* avx_movntv8sf */
    case 1103:  /* avx_lddqu256 */
    case 1101:  /* avx_storedqu256 */
    case 1099:  /* avx_loaddqu256 */
    case 1097:  /* avx_storeupd256 */
    case 1095:  /* avx_storeups256 */
    case 1093:  /* avx_loadupd256 */
    case 1091:  /* avx_loadups256 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 5;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 5;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 5;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 3;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 7;
        }
      else if ((which_alternative == 0) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1087:  /* *movv4df_internal */
    case 1083:  /* *movv2ti_internal */
    case 1081:  /* *movv4di_internal */
    case 1079:  /* *movv8si_internal */
    case 1077:  /* *movv16hi_internal */
    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 5;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 5;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 3;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 6) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 4)
        {
	  return 2;
        }
      else if (((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 4;
        }
      else if (((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 1)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0xc7f)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || (((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0xc7f)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((!((1 << which_alternative) & 0xc71)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 6)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || (((!((1 << which_alternative) & 0x2ff)) && (((((1 << which_alternative) & 0x300)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0xf00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || (((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0x2ff)) && ((((1 << which_alternative) & 0x300)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((!((1 << which_alternative) & 0x2c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 9)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x627f)) && (((((1 << which_alternative) & 0x640)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0x1e40)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || (((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0x627f)) && ((((1 << which_alternative) & 0x640)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((!((1 << which_alternative) & 0x6271)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x240)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x240))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || (((!((1 << which_alternative) & 0x1ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0x1c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 8) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 8)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode)))) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if (((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_STORE)) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode)))) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if (((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_STORE)) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 15 /* 0xf */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 17 /* 0x11 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 15 /* 0xf */;
        }
      else
        {
	  return 0;
        }

    case 894:  /* *movxfcc_1 */
      return 15 /* 0xf */;

    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 6;

    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1672:  /* sse4_1_blendvps */
    case 1670:  /* sse4_1_blendpd */
    case 1668:  /* sse4_1_blendps */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1602:  /* sse2_pmovmskb */
    case 1601:  /* avx2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1598:  /* sse_movmskps */
    case 1346:  /* sse_movss */
    case 1110:  /* sse2_movntv2df */
    case 1108:  /* sse_movntv4sf */
    case 1106:  /* sse2_movntidi */
    case 1105:  /* sse2_movntisi */
    case 1104:  /* sse3_lddqu */
    case 1102:  /* sse2_storedqu */
    case 1100:  /* sse2_loaddqu */
    case 1098:  /* sse2_storeupd */
    case 1096:  /* sse_storeups */
    case 1094:  /* sse2_loadupd */
    case 1092:  /* sse_loadups */
    case 1089:  /* sse2_movq128 */
    case 969:  /* sse_movntq */
    case 852:  /* movmsk_df */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 6;
        }
      else if ((mult_operator (operands[3], XFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 46 /* 0x2e */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 42 /* 0x2a */;
        }
      else
        {
	  return 0;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], XFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (! (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 10 /* 0xa */;
        }
      else
        {
	  return 6;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 6;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 46 /* 0x2e */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 42 /* 0x2a */;
        }
      else
        {
	  return 0;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 6;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 46 /* 0x2e */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 42 /* 0x2a */;
        }
      else
        {
	  return 0;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((mult_operator (operands[3], DFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (((mult_operator (operands[3], DFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 27 /* 0x1b */;
        }
      else
        {
	  return 0;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((mult_operator (operands[3], SFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (((mult_operator (operands[3], SFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else
        {
	  return 0;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 6;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 46 /* 0x2e */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 42 /* 0x2a */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 27 /* 0x1b */;
        }
      else
        {
	  return 0;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 6;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 46 /* 0x2e */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 42 /* 0x2a */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else
        {
	  return 0;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 10 /* 0xa */;
        }
      else
        {
	  return 6;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 10 /* 0xa */;
        }
      else
        {
	  return 6;
        }

    case 1747:  /* xop_pmadcswd */
    case 1746:  /* xop_pmadcsswd */
    case 1745:  /* xop_pmacswd */
    case 1744:  /* xop_pmacsswd */
    case 1743:  /* xop_pmacsdqh */
    case 1742:  /* xop_pmacsdql */
    case 1741:  /* xop_pmacssdqh */
    case 1740:  /* xop_pmacssdql */
    case 1739:  /* xop_pmacssdd */
    case 1738:  /* xop_pmacsdd */
    case 1737:  /* xop_pmacssww */
    case 1736:  /* xop_pmacsww */
    case 1678:  /* sse4_1_dppd */
    case 1676:  /* sse4_1_dpps */
    case 1284:  /* *fma4i_vmfnmsub_v2df */
    case 1283:  /* *fma4i_vmfnmsub_v4sf */
    case 1282:  /* *fma4i_vmfnmadd_v2df */
    case 1281:  /* *fma4i_vmfnmadd_v4sf */
    case 1280:  /* *fma4i_vmfmsub_v2df */
    case 1279:  /* *fma4i_vmfmsub_v4sf */
    case 1278:  /* *fma4i_vmfmadd_v2df */
    case 1277:  /* *fma4i_vmfmadd_v4sf */
    case 1276:  /* *fmai_fnmsub_v2df */
    case 1275:  /* *fmai_fnmsub_v4sf */
    case 1274:  /* *fmai_fnmadd_v2df */
    case 1273:  /* *fmai_fnmadd_v4sf */
    case 1272:  /* *fmai_fmsub_v2df */
    case 1271:  /* *fmai_fmsub_v4sf */
    case 1270:  /* *fmai_fmadd_v2df */
    case 1269:  /* *fmai_fmadd_v4sf */
    case 1268:  /* *fma_fmsubadd_v2df */
    case 1266:  /* *fma_fmsubadd_v4sf */
    case 1264:  /* *fma_fmaddsub_v2df */
    case 1262:  /* *fma_fmaddsub_v4sf */
    case 1258:  /* *fma_fnmsub_v2df */
    case 1257:  /* *fma_fnmsub_v4sf */
    case 1256:  /* *fma_fnmsub_df */
    case 1255:  /* *fma_fnmsub_sf */
    case 1252:  /* *fma_fnmadd_v2df */
    case 1251:  /* *fma_fnmadd_v4sf */
    case 1250:  /* *fma_fnmadd_df */
    case 1249:  /* *fma_fnmadd_sf */
    case 1246:  /* *fma_fmsub_v2df */
    case 1245:  /* *fma_fmsub_v4sf */
    case 1244:  /* *fma_fmsub_df */
    case 1243:  /* *fma_fmsub_sf */
    case 1240:  /* *fma_fmadd_v2df */
    case 1239:  /* *fma_fmadd_v4sf */
    case 1238:  /* *fma_fmadd_df */
    case 1237:  /* *fma_fmadd_sf */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1132:  /* *mulv2df3 */
    case 1130:  /* *mulv4sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 740:  /* *fop_df_comm_sse */
    case 739:  /* *fop_sf_comm_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 10 /* 0xa */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 6;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 6;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 5;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if (((which_alternative != 0) && (cached_memory == MEMORY_STORE)) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1905:  /* avx_maskstoreps256 */
    case 1897:  /* avx_maskloadps256 */
    case 1666:  /* sse4a_insertq */
    case 1665:  /* sse4a_insertqi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 3;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 5;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 10 /* 0xa */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 322:  /* *mulhi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else
        {
	  return 0;
        }

    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
    case 320:  /* *muldi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 323:  /* *mulqi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 5;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 8;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 8;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 8;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 8;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (get_attr_memory (insn) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 2;
        }

    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1287:  /* sse_cvttps2pi */
    case 810:  /* sse4_1_rounddf2 */
    case 809:  /* sse4_1_roundsf2 */
    case 141:  /* *truncdfsf_fast_sse */
    case 136:  /* *extendsfdf2_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 8;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 13 /* 0xd */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && (which_alternative == 1))
        {
	  return 8;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 8;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || (((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || (((!((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0xf)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1348:  /* avx2_vec_dupv4sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
      return 2;

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || (((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0x3f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || ((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 5)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x78)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0xff)) && ((((1 << which_alternative) & 0x1858)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || (((((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || (((which_alternative == 9) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x1bff)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0xff)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))))
        {
	  return 4;
        }
      else if (((((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || (((which_alternative == 9) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x1bff)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((which_alternative == 7) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 7)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 2)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_STORE)) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 4;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if (((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_STORE)) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if (((cached_type == TYPE_IMOV) && (cached_memory == MEMORY_STORE)) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && ((pic_32bit_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || (((!((1 << which_alternative) & 0xffc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || (((((1 << which_alternative) & 0xf80)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((((1 << which_alternative) & 0xf80)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((((1 << which_alternative) & 0xfb8)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))))))
        {
	  return 4;
        }
      else if ((((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 6)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x623f)) && (((((1 << which_alternative) & 0xaa0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0x1ea0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || (((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0x623f)) && ((((1 << which_alternative) & 0xaa0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((!((1 << which_alternative) & 0x6227)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && ((pic_32bit_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || (((!((1 << which_alternative) & 0x1fff0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || (((((1 << which_alternative) & 0x7c00)) && (((((1 << which_alternative) & 0xa00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0xa01)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || (((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((((1 << which_alternative) & 0x7c00)) && ((((1 << which_alternative) & 0xa00)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((((1 << which_alternative) & 0x7de0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 9)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1088:  /* *movv2df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1080:  /* *movv4si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 2)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 5;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 3;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1889:  /* vec_set_hi_v8si */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1332:  /* vec_interleave_highv4sf */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 39:  /* *cmpfp_iu_387 */
    case 38:  /* *cmpfp_iu_sse */
    case 36:  /* *cmpfp_i_i387 */
    case 35:  /* *cmpfp_i_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 853:  /* cld */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
    case 42:  /* *pushdi2_rex64 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
      return 1;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 0;
        }
      else if ((cached_type == TYPE_PUSH) || (cached_type == TYPE_POP))
        {
	  return 1;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 3;
        }
      else if (cached_type == TYPE_LEA)
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))))
        {
	  return 10 /* 0xa */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 10 /* 0xa */;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))))
        {
	  return 6;
        }
      else if (((cached_bdver1_decode = get_attr_bdver1_decode (insn)) == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 1;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 5;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if (((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((cached_bdver1_decode == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 5;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_type == TYPE_FMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 8;
        }
      else if (((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((cached_type == TYPE_FISTP) || ((cached_type == TYPE_FISTTP) || (cached_type == TYPE_FMOV))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 6;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 46 /* 0x2e */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 42 /* 0x2a */;
        }
      else if ((cached_type == TYPE_FPSPC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_type == TYPE_FCMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 17 /* 0x11 */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 15 /* 0xf */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_bdver1_decode == BDVER1_DECODE_DOUBLE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DOUBLE) && (cached_type == TYPE_FCMP))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FCMP) || (cached_type == TYPE_FXCH))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_prefix = get_attr_prefix (insn)) == PREFIX_VEX) && (((cached_movu = get_attr_movu (insn)) == MOVU_1) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_movu = get_attr_movu (insn)) == MOVU_1) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || (cached_mode == MODE_V4DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_movu = get_attr_movu (insn)) == MOVU_1) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || (((cached_prefix = get_attr_prefix (insn)) == PREFIX_VEX) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || ((cached_mode == MODE_V4DF) || (cached_mode == MODE_OI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 5;
        }
      else if (((cached_type == TYPE_SSEMOV) && (((((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_mode = get_attr_mode (insn)) == MODE_DI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || (((cached_prefix = get_attr_prefix (insn)) == PREFIX_VEX) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || ((cached_mode == MODE_V4DF) || (cached_mode == MODE_OI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 5;
        }
      else if (((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || ((cached_mode == MODE_V4DF) || (cached_mode == MODE_OI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 3;
        }
      else if (((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && (((cached_mode = get_attr_mode (insn)) == MODE_V8SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 7;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_mode = get_attr_mode (insn)) == MODE_V8SF))
        {
	  return 3;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECOMI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSECOMI)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || (cached_mode == MODE_V4DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 11 /* 0xb */;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || (cached_mode == MODE_V4DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 7;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEIMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSEIMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 27 /* 0x1b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V8SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V8SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_DF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_DF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 27 /* 0x1b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_V4SF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_V4SF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_type == TYPE_SSEINS) && ((cached_mode = get_attr_mode (insn)) == MODE_TI))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    default:
      return 0;

    }
}

int
internal_dfa_insn_code_bdver1 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  enum attr_bdver1_decode cached_bdver1_decode ATTRIBUTE_UNUSED;
  enum attr_prefix cached_prefix ATTRIBUTE_UNUSED;
  enum attr_movu cached_movu ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1928:  /* avx_vec_concatv8sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if (which_alternative == 0)
        {
	  return 377 /* 0x179 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1929:  /* avx_vec_concatv4df */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 367 /* 0x16f */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1862:  /* avx_vbroadcastf128_v8sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if (which_alternative != 0)
        {
	  return 377 /* 0x179 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative != 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1690:  /* avx2_pblenddv4si */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 363 /* 0x16b */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 371 /* 0x173 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1674:  /* sse4_1_blendvpd */
    case 1672:  /* sse4_1_blendvps */
    case 1670:  /* sse4_1_blendpd */
    case 1668:  /* sse4_1_blendps */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 363 /* 0x16b */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 371 /* 0x173 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1666:  /* sse4a_insertq */
    case 1665:  /* sse4a_insertqi */
      return 422 /* 0x1a6 */;

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x63))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 2) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 367 /* 0x16f */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (cached_memory == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1510:  /* *andnotv2di3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1509:  /* *andnotv4di3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1508:  /* *andnotv4si3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1507:  /* *andnotv8si3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1506:  /* *andnotv8hi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1505:  /* *andnotv16hi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1504:  /* *andnotv16qi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1503:  /* *andnotv32qi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 410 /* 0x19a */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 411 /* 0x19b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1631:  /* ssse3_pmaddubsw */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1436:  /* *avx2_pmaddwd */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 412 /* 0x19c */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 413 /* 0x19d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x12)) && ((((1 << which_alternative) & 0x43)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 363 /* 0x16b */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x24)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x24)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((which_alternative != 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((which_alternative != 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 5)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 9)
        {
	  return 343 /* 0x157 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((!((1 << which_alternative) & 0xfe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((!((1 << which_alternative) & 0xfe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 5)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 5)
        {
	  return 343 /* 0x157 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((!((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((!((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 3)
        {
	  return 343 /* 0x157 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 4)
        {
	  return 343 /* 0x157 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 10)
        {
	  return 343 /* 0x157 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0xa0)) && ((((1 << which_alternative) & 0x3b)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 367 /* 0x16f */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x3b)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x3b)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 10)
        {
	  return 343 /* 0x157 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0xa0)) && ((((1 << which_alternative) & 0x39)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 367 /* 0x16f */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x39)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x39)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1352:  /* *vec_concatv4sf */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x5)) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 363 /* 0x16b */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 363 /* 0x16b */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative != 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative != 1)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1905:  /* avx_maskstoreps256 */
    case 1897:  /* avx_maskloadps256 */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1347:  /* avx2_vec_dupv8sf */
      return 377 /* 0x179 */;

    case 1346:  /* sse_movss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 367 /* 0x16f */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 368 /* 0x170 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 374 /* 0x176 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xa)) && ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 363 /* 0x16b */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xa)) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 363 /* 0x16b */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 398 /* 0x18e */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 399 /* 0x18f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1889:  /* vec_set_hi_v8si */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1332:  /* vec_interleave_highv4sf */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1890:  /* vec_set_hi_v8sf */
    case 1888:  /* vec_set_lo_v8sf */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1339:  /* avx_shufps256_1 */
    case 1333:  /* avx_unpcklps256 */
    case 1331:  /* avx_unpckhps256 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1213:  /* avx_andnotv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 376 /* 0x178 */;
        }
      else
        {
	  return 377 /* 0x179 */;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1138:  /* sse2_divv2df3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 418 /* 0x1a2 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 419 /* 0x1a3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1137:  /* avx_divv4df3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 414 /* 0x19e */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 415 /* 0x19f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1139:  /* sse_vmdivv4sf3 */
    case 1136:  /* sse_divv4sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 420 /* 0x1a4 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 421 /* 0x1a5 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1135:  /* avx_divv8sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 416 /* 0x1a0 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 417 /* 0x1a1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1677:  /* avx_dppd256 */
    case 1675:  /* avx_dpps256 */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1131:  /* *mulv4df3 */
    case 1129:  /* *mulv8sf3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 406 /* 0x196 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 407 /* 0x197 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1935:  /* vcvtps2ph256 */
    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1932:  /* vcvtph2ps256 */
    case 1931:  /* *vcvtph2ps_load */
    case 1930:  /* vcvtph2ps */
    case 1727:  /* sse4_1_roundsd */
    case 1726:  /* sse4_1_roundss */
    case 1725:  /* sse4_1_roundpd */
    case 1724:  /* avx_roundpd256 */
    case 1723:  /* sse4_1_roundps */
    case 1722:  /* avx_roundps256 */
    case 1328:  /* sse2_cvtps2pd */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1326:  /* avx_cvtps2pd256 */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1315:  /* sse2_cvtdq2pd */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1313:  /* floatv4siv4df2 */
    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1302:  /* sse2_cvtpi2pd */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1298:  /* avx_cvtps2dq256 */
    case 1297:  /* floatv4siv4sf2 */
    case 1296:  /* floatv8siv8sf2 */
    case 1286:  /* sse_cvtps2pi */
    case 1285:  /* sse_cvtpi2ps */
    case 1112:  /* sse2_movntv2di */
    case 1111:  /* avx_movntv4di */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode)))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode)))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode)))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode)))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1601:  /* avx2_pmovmskb */
    case 1106:  /* sse2_movntidi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 366 /* 0x16e */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1685:  /* avx2_pblendvb */
    case 1679:  /* avx2_movntdqa */
    case 1673:  /* avx_blendvpd256 */
    case 1671:  /* avx_blendvps256 */
    case 1669:  /* avx_blendpd256 */
    case 1667:  /* avx_blendps256 */
    case 1599:  /* avx_movmskpd256 */
    case 1597:  /* avx_movmskps256 */
    case 1109:  /* avx_movntv4df */
    case 1107:  /* avx_movntv8sf */
    case 1103:  /* avx_lddqu256 */
    case 1101:  /* avx_storedqu256 */
    case 1099:  /* avx_loaddqu256 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 364 /* 0x16c */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 370 /* 0x172 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 373 /* 0x175 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1098:  /* sse2_storeupd */
    case 1096:  /* sse_storeups */
    case 1094:  /* sse2_loadupd */
    case 1092:  /* sse_loadups */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 362 /* 0x16a */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 371 /* 0x173 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1097:  /* avx_storeupd256 */
    case 1095:  /* avx_storeups256 */
    case 1093:  /* avx_loadupd256 */
    case 1091:  /* avx_loadups256 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 361 /* 0x169 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 370 /* 0x172 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 373 /* 0x175 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1680:  /* sse4_1_movntdqa */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1600:  /* sse2_movmskpd */
    case 1598:  /* sse_movmskps */
    case 1110:  /* sse2_movntv2df */
    case 1108:  /* sse_movntv4sf */
    case 1104:  /* sse3_lddqu */
    case 1102:  /* sse2_storedqu */
    case 1100:  /* sse2_loaddqu */
    case 1089:  /* sse2_movq128 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 371 /* 0x173 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1088:  /* *movv2df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1087:  /* *movv4df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1086:  /* *movv4sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 376 /* 0x178 */;
        }
      else if ((which_alternative == 0) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1084:  /* *movv1ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1083:  /* *movv2ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1082:  /* *movv2di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1081:  /* *movv4di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1080:  /* *movv4si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1079:  /* *movv8si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1078:  /* *movv8hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1077:  /* *movv16hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1076:  /* *movv16qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 4)
        {
	  return 343 /* 0x157 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 1)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1602:  /* sse2_pmovmskb */
    case 1105:  /* sse2_movntisi */
    case 969:  /* sse_movntq */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 328 /* 0x148 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 6)
        {
	  return 379 /* 0x17b */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((((1 << which_alternative) & 0x300)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((!((1 << which_alternative) & 0xf00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 366 /* 0x16e */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((((1 << which_alternative) & 0x300)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 9)
        {
	  return 379 /* 0x17b */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 328 /* 0x148 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((((1 << which_alternative) & 0x640)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((!((1 << which_alternative) & 0x1e40)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 366 /* 0x16e */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((((1 << which_alternative) & 0x640)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x240)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x240))
        {
	  return 379 /* 0x17b */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 366 /* 0x16e */;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 8) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 8)
        {
	  return 379 /* 0x17b */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1348:  /* avx2_vec_dupv4sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
      return 379 /* 0x17b */;

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 328 /* 0x148 */;
        }
      else
        {
	  return 354 /* 0x162 */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 353 /* 0x161 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 354 /* 0x162 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 894:  /* *movxfcc_1 */
      return 354 /* 0x162 */;

    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 327 /* 0x147 */;

    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 852:  /* movmsk_df */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 368 /* 0x170 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 374 /* 0x176 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 345 /* 0x159 */;
        }
      else if ((mult_operator (operands[3], XFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 347 /* 0x15b */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 349 /* 0x15d */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 350 /* 0x15e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], XFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (! (mult_operator (operands[3], XFmode)))
        {
	  return 345 /* 0x159 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 346 /* 0x15a */;
        }
      else
        {
	  return 347 /* 0x15b */;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 345 /* 0x159 */;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 347 /* 0x15b */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 349 /* 0x15d */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 350 /* 0x15e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 345 /* 0x159 */;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 347 /* 0x15b */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 349 /* 0x15d */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 350 /* 0x15e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((mult_operator (operands[3], DFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if (((mult_operator (operands[3], DFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 418 /* 0x1a2 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 419 /* 0x1a3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((mult_operator (operands[3], SFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if (((mult_operator (operands[3], SFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 420 /* 0x1a4 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 421 /* 0x1a5 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 345 /* 0x159 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 347 /* 0x15b */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 349 /* 0x15d */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 350 /* 0x15e */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 418 /* 0x1a2 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 419 /* 0x1a3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 345 /* 0x159 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 347 /* 0x15b */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 349 /* 0x15d */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 350 /* 0x15e */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 420 /* 0x1a4 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 421 /* 0x1a5 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 345 /* 0x159 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 346 /* 0x15a */;
        }
      else
        {
	  return 347 /* 0x15b */;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 345 /* 0x159 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 346 /* 0x15a */;
        }
      else
        {
	  return 347 /* 0x15b */;
        }

    case 1747:  /* xop_pmadcswd */
    case 1746:  /* xop_pmadcsswd */
    case 1745:  /* xop_pmacswd */
    case 1744:  /* xop_pmacsswd */
    case 1743:  /* xop_pmacsdqh */
    case 1742:  /* xop_pmacsdql */
    case 1741:  /* xop_pmacssdqh */
    case 1740:  /* xop_pmacssdql */
    case 1739:  /* xop_pmacssdd */
    case 1738:  /* xop_pmacsdd */
    case 1737:  /* xop_pmacssww */
    case 1736:  /* xop_pmacsww */
    case 1678:  /* sse4_1_dppd */
    case 1676:  /* sse4_1_dpps */
    case 1284:  /* *fma4i_vmfnmsub_v2df */
    case 1283:  /* *fma4i_vmfnmsub_v4sf */
    case 1282:  /* *fma4i_vmfnmadd_v2df */
    case 1281:  /* *fma4i_vmfnmadd_v4sf */
    case 1280:  /* *fma4i_vmfmsub_v2df */
    case 1279:  /* *fma4i_vmfmsub_v4sf */
    case 1278:  /* *fma4i_vmfmadd_v2df */
    case 1277:  /* *fma4i_vmfmadd_v4sf */
    case 1276:  /* *fmai_fnmsub_v2df */
    case 1275:  /* *fmai_fnmsub_v4sf */
    case 1274:  /* *fmai_fnmadd_v2df */
    case 1273:  /* *fmai_fnmadd_v4sf */
    case 1272:  /* *fmai_fmsub_v2df */
    case 1271:  /* *fmai_fmsub_v4sf */
    case 1270:  /* *fmai_fmadd_v2df */
    case 1269:  /* *fmai_fmadd_v4sf */
    case 1268:  /* *fma_fmsubadd_v2df */
    case 1266:  /* *fma_fmsubadd_v4sf */
    case 1264:  /* *fma_fmaddsub_v2df */
    case 1262:  /* *fma_fmaddsub_v4sf */
    case 1258:  /* *fma_fnmsub_v2df */
    case 1257:  /* *fma_fnmsub_v4sf */
    case 1256:  /* *fma_fnmsub_df */
    case 1255:  /* *fma_fnmsub_sf */
    case 1252:  /* *fma_fnmadd_v2df */
    case 1251:  /* *fma_fnmadd_v4sf */
    case 1250:  /* *fma_fnmadd_df */
    case 1249:  /* *fma_fnmadd_sf */
    case 1246:  /* *fma_fmsub_v2df */
    case 1245:  /* *fma_fmsub_v4sf */
    case 1244:  /* *fma_fmsub_df */
    case 1243:  /* *fma_fmsub_sf */
    case 1240:  /* *fma_fmadd_v2df */
    case 1239:  /* *fma_fmadd_v4sf */
    case 1238:  /* *fma_fmadd_df */
    case 1237:  /* *fma_fmadd_sf */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1132:  /* *mulv2df3 */
    case 1130:  /* *mulv4sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 740:  /* *fop_df_comm_sse */
    case 739:  /* *fop_sf_comm_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 408 /* 0x198 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 409 /* 0x199 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 345 /* 0x159 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 347 /* 0x15b */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 345 /* 0x159 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 347 /* 0x15b */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 331 /* 0x14b */;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 319 /* 0x13f */;

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      return 316 /* 0x13c */;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 328 /* 0x148 */;
        }
      else
        {
	  return 331 /* 0x14b */;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 380 /* 0x17c */;
        }
      else
        {
	  return 381 /* 0x17d */;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 328 /* 0x148 */;
        }
      else
        {
	  return 336 /* 0x150 */;
        }

    case 491:  /* *ashlhi3_1 */
    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 329 /* 0x149 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 332 /* 0x14c */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 335 /* 0x14f */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 337 /* 0x151 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
      return 348 /* 0x15c */;

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 325 /* 0x145 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 326 /* 0x146 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 321 /* 0x141 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 323 /* 0x143 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 324 /* 0x144 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 322 /* 0x142 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 324 /* 0x144 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 322:  /* *mulhi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 322 /* 0x142 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 324 /* 0x144 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
    case 320:  /* *muldi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 321 /* 0x141 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 323 /* 0x143 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 323:  /* *mulqi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 322 /* 0x142 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 324 /* 0x144 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 328 /* 0x148 */;
        }
      else
        {
	  return 331 /* 0x14b */;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 492:  /* *ashlqi3_1 */
    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
      return 320 /* 0x140 */;

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 388 /* 0x184 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 389 /* 0x185 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 388 /* 0x184 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 389 /* 0x185 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 388 /* 0x184 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 389 /* 0x185 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 388 /* 0x184 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 389 /* 0x185 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 388 /* 0x184 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 389 /* 0x185 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 338 /* 0x152 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 340 /* 0x154 */;
        }
      else
        {
	  return 343 /* 0x157 */;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 338 /* 0x152 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 340 /* 0x154 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
      return 342 /* 0x156 */;

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 388 /* 0x184 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 389 /* 0x185 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 339 /* 0x153 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 341 /* 0x155 */;
        }
      else
        {
	  return 343 /* 0x157 */;
        }

    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 386 /* 0x182 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 387 /* 0x183 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 338 /* 0x152 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 340 /* 0x154 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 341 /* 0x155 */;
        }
      else
        {
	  return 343 /* 0x157 */;
        }

    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1287:  /* sse_cvttps2pi */
    case 810:  /* sse4_1_rounddf2 */
    case 809:  /* sse4_1_roundsf2 */
    case 141:  /* *truncdfsf_fast_sse */
    case 136:  /* *extendsfdf2_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 385 /* 0x181 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 386 /* 0x182 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 387 /* 0x183 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 338 /* 0x152 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if (((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && (which_alternative == 1))
        {
	  return 340 /* 0x154 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 343 /* 0x157 */;
        }
      else if ((which_alternative == 2) && ((cached_memory == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 385 /* 0x181 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 386 /* 0x182 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 387 /* 0x183 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 328 /* 0x148 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
      return 359 /* 0x167 */;

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 343 /* 0x157 */;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 5)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 328 /* 0x148 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 343 /* 0x157 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || ((((1 << which_alternative) & 0x880)) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xee0)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || ((((1 << which_alternative) & 0x880)) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xee0)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 343 /* 0x157 */;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((((1 << which_alternative) & 0x1858)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 366 /* 0x16e */;
        }
      else if ((((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || (((which_alternative == 9) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x1bff)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || (((which_alternative == 9) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x1bff)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 7) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 7)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 338 /* 0x152 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 340 /* 0x154 */;
        }
      else
        {
	  return 343 /* 0x157 */;
        }

    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 328 /* 0x148 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 2)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 317 /* 0x13d */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 328 /* 0x148 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 328 /* 0x148 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 333 /* 0x14d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 330 /* 0x14a */;

    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 333 /* 0x14d */;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (cached_memory == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_type == TYPE_IMOV) && (cached_memory == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 320 /* 0x140 */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if (((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if (((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 6)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 328 /* 0x148 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((((1 << which_alternative) & 0xaa0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((!((1 << which_alternative) & 0x1ea0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 366 /* 0x16e */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((((1 << which_alternative) & 0xaa0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 379 /* 0x17b */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 320 /* 0x140 */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((((1 << which_alternative) & 0xa00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((!((1 << which_alternative) & 0xa01)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 366 /* 0x16e */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((((1 << which_alternative) & 0xa00)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 9)
        {
	  return 379 /* 0x17b */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 328 /* 0x148 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 2)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 318 /* 0x13e */;

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      return 317 /* 0x13d */;

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 317 /* 0x13d */;
        }
      else
        {
	  return 328 /* 0x148 */;
        }

    case 39:  /* *cmpfp_iu_387 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 357 /* 0x165 */;
        }
      else
        {
	  return 358 /* 0x166 */;
        }

    case 36:  /* *cmpfp_i_i387 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 355 /* 0x163 */;
        }
      else
        {
	  return 356 /* 0x164 */;
        }

    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 382 /* 0x17e */;
        }
      else
        {
	  return 383 /* 0x17f */;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 355 /* 0x163 */;
        }
      else if (which_alternative == 0)
        {
	  return 356 /* 0x164 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 382 /* 0x17e */;
        }
      else
        {
	  return 383 /* 0x17f */;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 853:  /* cld */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
      return 328 /* 0x148 */;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 316 /* 0x13c */;
        }
      else if (cached_type == TYPE_PUSH)
        {
	  return 317 /* 0x13d */;
        }
      else if (cached_type == TYPE_POP)
        {
	  return 318 /* 0x13e */;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 319 /* 0x13f */;
        }
      else if (cached_type == TYPE_LEA)
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 321 /* 0x141 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 322 /* 0x142 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))))
        {
	  return 323 /* 0x143 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 324 /* 0x144 */;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 325 /* 0x145 */;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 326 /* 0x146 */;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))))
        {
	  return 327 /* 0x147 */;
        }
      else if (((cached_bdver1_decode = get_attr_bdver1_decode (insn)) == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 329 /* 0x149 */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 332 /* 0x14c */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 335 /* 0x14f */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 336 /* 0x150 */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 337 /* 0x151 */;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 338 /* 0x152 */;
        }
      else if ((cached_type == TYPE_FMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 340 /* 0x154 */;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if ((cached_type == TYPE_FISTP) || (cached_type == TYPE_FISTTP))
        {
	  return 342 /* 0x156 */;
        }
      else if (cached_type == TYPE_FMOV)
        {
	  return 343 /* 0x157 */;
        }
      else if ((cached_type == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 345 /* 0x159 */;
        }
      else if ((cached_type == TYPE_FMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 347 /* 0x15b */;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 348 /* 0x15c */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 349 /* 0x15d */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 350 /* 0x15e */;
        }
      else if ((cached_type == TYPE_FPSPC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 351 /* 0x15f */;
        }
      else if ((cached_type == TYPE_FCMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 353 /* 0x161 */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 354 /* 0x162 */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_bdver1_decode == BDVER1_DECODE_DOUBLE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 355 /* 0x163 */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DOUBLE) && (cached_type == TYPE_FCMP))
        {
	  return 356 /* 0x164 */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 357 /* 0x165 */;
        }
      else if (cached_type == TYPE_FCMP)
        {
	  return 358 /* 0x166 */;
        }
      else if (cached_type == TYPE_FXCH)
        {
	  return 359 /* 0x167 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_prefix = get_attr_prefix (insn)) == PREFIX_VEX) && (((cached_movu = get_attr_movu (insn)) == MOVU_1) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))))
        {
	  return 360 /* 0x168 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_movu = get_attr_movu (insn)) == MOVU_1) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || (cached_mode == MODE_V4DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 361 /* 0x169 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_movu = get_attr_movu (insn)) == MOVU_1) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 362 /* 0x16a */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_prefix = get_attr_prefix (insn)) == PREFIX_VEX) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 363 /* 0x16b */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || ((cached_mode == MODE_V4DF) || (cached_mode == MODE_OI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 366 /* 0x16e */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_prefix = get_attr_prefix (insn)) == PREFIX_VEX) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 367 /* 0x16f */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || ((cached_mode == MODE_V4DF) || (cached_mode == MODE_OI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || ((cached_mode == MODE_V4DF) || (cached_mode == MODE_OI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && (((cached_mode = get_attr_mode (insn)) == MODE_V8SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 376 /* 0x178 */;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_mode = get_attr_mode (insn)) == MODE_V8SF))
        {
	  return 377 /* 0x179 */;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1))
        {
	  return 379 /* 0x17b */;
        }
      else if ((cached_type == TYPE_SSECMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 380 /* 0x17c */;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 381 /* 0x17d */;
        }
      else if ((cached_type == TYPE_SSECOMI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 382 /* 0x17e */;
        }
      else if (cached_type == TYPE_SSECOMI)
        {
	  return 383 /* 0x17f */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 386 /* 0x182 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 387 /* 0x183 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 388 /* 0x184 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 389 /* 0x185 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 398 /* 0x18e */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 399 /* 0x18f */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || (cached_mode == MODE_V4DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 406 /* 0x196 */;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || (cached_mode == MODE_V4DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 407 /* 0x197 */;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else if ((cached_type == TYPE_SSEIMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 410 /* 0x19a */;
        }
      else if ((cached_type == TYPE_SSEIMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 411 /* 0x19b */;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 412 /* 0x19c */;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 413 /* 0x19d */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 414 /* 0x19e */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 415 /* 0x19f */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V8SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 416 /* 0x1a0 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V8SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 417 /* 0x1a1 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_DF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 418 /* 0x1a2 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_DF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 419 /* 0x1a3 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_V4SF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 420 /* 0x1a4 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_V4SF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 421 /* 0x1a5 */;
        }
      else if ((cached_type == TYPE_SSEINS) && ((cached_mode = get_attr_mode (insn)) == MODE_TI))
        {
	  return 422 /* 0x1a6 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    default:
      return 618 /* 0x26a */;

    }
}

int
insn_default_latency_bdver2 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  enum attr_bdver1_decode cached_bdver1_decode ATTRIBUTE_UNUSED;
  enum attr_prefix cached_prefix ATTRIBUTE_UNUSED;
  enum attr_movu cached_movu ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1928:  /* avx_vec_concatv8sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 5;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if (which_alternative == 0)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1929:  /* avx_vec_concatv4df */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 5;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1862:  /* avx_vbroadcastf128_v8sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 5;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 3;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if (which_alternative != 0)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 5;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 3;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative != 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x63)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x63))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x6)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 2) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (((((1 << which_alternative) & 0x7)) && (cached_memory == MEMORY_BOTH)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1510:  /* *andnotv2di3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1509:  /* *andnotv4di3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1508:  /* *andnotv4si3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1507:  /* *andnotv8si3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1506:  /* *andnotv8hi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1505:  /* *andnotv16hi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1504:  /* *andnotv16qi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1503:  /* *andnotv32qi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 7;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16)))
        {
	  return 3;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1631:  /* ssse3_pmaddubsw */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1436:  /* *avx2_pmaddwd */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((which_alternative != 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((which_alternative != 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 5)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 9)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 5)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 6) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 5)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x13)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 3)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 4)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1391:  /* *vec_concatv2df */
    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 10)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 4;
        }
      else if (((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 4;
        }
      else if (((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative != 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative != 1)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1588:  /* *vec_concatv4si */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (((((1 << which_alternative) & 0xc)) && (cached_memory == MEMORY_BOTH)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (((which_alternative == 1) && (cached_memory == MEMORY_BOTH)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1352:  /* *vec_concatv4sf */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_BOTH)) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1890:  /* vec_set_hi_v8sf */
    case 1888:  /* vec_set_lo_v8sf */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1339:  /* avx_shufps256_1 */
    case 1333:  /* avx_unpcklps256 */
    case 1331:  /* avx_unpckhps256 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1213:  /* avx_andnotv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 7;
        }
      else
        {
	  return 3;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 31 /* 0x1f */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 27 /* 0x1b */;
        }
      else
        {
	  return 0;
        }

    case 1139:  /* sse_vmdivv4sf3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 28 /* 0x1c */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 24 /* 0x18 */;
        }
      else
        {
	  return 0;
        }

    case 1677:  /* avx_dppd256 */
    case 1675:  /* avx_dpps256 */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1131:  /* *mulv4df3 */
    case 1129:  /* *mulv8sf3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 11 /* 0xb */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 7;
        }
      else
        {
	  return 0;
        }

    case 1935:  /* vcvtps2ph256 */
    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1932:  /* vcvtph2ps256 */
    case 1931:  /* *vcvtph2ps_load */
    case 1930:  /* vcvtph2ps */
    case 1727:  /* sse4_1_roundsd */
    case 1726:  /* sse4_1_roundss */
    case 1725:  /* sse4_1_roundpd */
    case 1724:  /* avx_roundpd256 */
    case 1723:  /* sse4_1_roundps */
    case 1722:  /* avx_roundps256 */
    case 1328:  /* sse2_cvtps2pd */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1326:  /* avx_cvtps2pd256 */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1315:  /* sse2_cvtdq2pd */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1313:  /* floatv4siv4df2 */
    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1302:  /* sse2_cvtpi2pd */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1298:  /* avx_cvtps2dq256 */
    case 1297:  /* floatv4siv4sf2 */
    case 1296:  /* floatv8siv8sf2 */
    case 1286:  /* sse_cvtps2pi */
    case 1285:  /* sse_cvtpi2ps */
    case 1112:  /* sse2_movntv2di */
    case 1111:  /* avx_movntv4di */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode)))))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode)))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode))))
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode))))
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode))))
        {
	  return 2;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode)))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode)))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1685:  /* avx2_pblendvb */
    case 1679:  /* avx2_movntdqa */
    case 1673:  /* avx_blendvpd256 */
    case 1671:  /* avx_blendvps256 */
    case 1669:  /* avx_blendpd256 */
    case 1667:  /* avx_blendps256 */
    case 1599:  /* avx_movmskpd256 */
    case 1597:  /* avx_movmskps256 */
    case 1109:  /* avx_movntv4df */
    case 1107:  /* avx_movntv8sf */
    case 1103:  /* avx_lddqu256 */
    case 1101:  /* avx_storedqu256 */
    case 1099:  /* avx_loaddqu256 */
    case 1097:  /* avx_storeupd256 */
    case 1095:  /* avx_storeups256 */
    case 1093:  /* avx_loadupd256 */
    case 1091:  /* avx_loadups256 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 5;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 5;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 5;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 3;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 7;
        }
      else if ((which_alternative == 0) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return 3;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1087:  /* *movv4df_internal */
    case 1083:  /* *movv2ti_internal */
    case 1081:  /* *movv4di_internal */
    case 1079:  /* *movv8si_internal */
    case 1077:  /* *movv16hi_internal */
    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 5;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 5;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 3;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((which_alternative == 6) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 4)
        {
	  return 2;
        }
      else if (((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))
        {
	  return 4;
        }
      else if (((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 1)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0xc7f)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || (((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0xc7f)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((!((1 << which_alternative) & 0xc71)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 6)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || (((!((1 << which_alternative) & 0x2ff)) && (((((1 << which_alternative) & 0x300)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0xf00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || (((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0x2ff)) && ((((1 << which_alternative) & 0x300)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((!((1 << which_alternative) & 0x2c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 9)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x627f)) && (((((1 << which_alternative) & 0x640)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0x1e40)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || (((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0x627f)) && ((((1 << which_alternative) & 0x640)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((!((1 << which_alternative) & 0x6271)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x240)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x240))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || (((!((1 << which_alternative) & 0x1ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0x1c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 8) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 8)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode)))) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if (((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_STORE)) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode)))) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if (((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_STORE)) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 15 /* 0xf */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 17 /* 0x11 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 15 /* 0xf */;
        }
      else
        {
	  return 0;
        }

    case 894:  /* *movxfcc_1 */
      return 15 /* 0xf */;

    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 6;

    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1672:  /* sse4_1_blendvps */
    case 1670:  /* sse4_1_blendpd */
    case 1668:  /* sse4_1_blendps */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1602:  /* sse2_pmovmskb */
    case 1601:  /* avx2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1598:  /* sse_movmskps */
    case 1346:  /* sse_movss */
    case 1110:  /* sse2_movntv2df */
    case 1108:  /* sse_movntv4sf */
    case 1106:  /* sse2_movntidi */
    case 1105:  /* sse2_movntisi */
    case 1104:  /* sse3_lddqu */
    case 1102:  /* sse2_storedqu */
    case 1100:  /* sse2_loaddqu */
    case 1098:  /* sse2_storeupd */
    case 1096:  /* sse_storeups */
    case 1094:  /* sse2_loadupd */
    case 1092:  /* sse_loadups */
    case 1089:  /* sse2_movq128 */
    case 969:  /* sse_movntq */
    case 852:  /* movmsk_df */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 6;
        }
      else if ((mult_operator (operands[3], XFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 46 /* 0x2e */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 42 /* 0x2a */;
        }
      else
        {
	  return 0;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], XFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (! (mult_operator (operands[3], XFmode)))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 10 /* 0xa */;
        }
      else
        {
	  return 6;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 6;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 46 /* 0x2e */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 42 /* 0x2a */;
        }
      else
        {
	  return 0;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 6;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 46 /* 0x2e */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 42 /* 0x2a */;
        }
      else
        {
	  return 0;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((mult_operator (operands[3], DFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (((mult_operator (operands[3], DFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 27 /* 0x1b */;
        }
      else
        {
	  return 0;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((mult_operator (operands[3], SFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (((mult_operator (operands[3], SFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else
        {
	  return 0;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 6;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 46 /* 0x2e */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 42 /* 0x2a */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 27 /* 0x1b */;
        }
      else
        {
	  return 0;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_unit == UNIT_INTEGER) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 6;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 46 /* 0x2e */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 42 /* 0x2a */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 24 /* 0x18 */;
        }
      else
        {
	  return 0;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 10 /* 0xa */;
        }
      else
        {
	  return 6;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 10 /* 0xa */;
        }
      else
        {
	  return 6;
        }

    case 1747:  /* xop_pmadcswd */
    case 1746:  /* xop_pmadcsswd */
    case 1745:  /* xop_pmacswd */
    case 1744:  /* xop_pmacsswd */
    case 1743:  /* xop_pmacsdqh */
    case 1742:  /* xop_pmacsdql */
    case 1741:  /* xop_pmacssdqh */
    case 1740:  /* xop_pmacssdql */
    case 1739:  /* xop_pmacssdd */
    case 1738:  /* xop_pmacsdd */
    case 1737:  /* xop_pmacssww */
    case 1736:  /* xop_pmacsww */
    case 1678:  /* sse4_1_dppd */
    case 1676:  /* sse4_1_dpps */
    case 1284:  /* *fma4i_vmfnmsub_v2df */
    case 1283:  /* *fma4i_vmfnmsub_v4sf */
    case 1282:  /* *fma4i_vmfnmadd_v2df */
    case 1281:  /* *fma4i_vmfnmadd_v4sf */
    case 1280:  /* *fma4i_vmfmsub_v2df */
    case 1279:  /* *fma4i_vmfmsub_v4sf */
    case 1278:  /* *fma4i_vmfmadd_v2df */
    case 1277:  /* *fma4i_vmfmadd_v4sf */
    case 1276:  /* *fmai_fnmsub_v2df */
    case 1275:  /* *fmai_fnmsub_v4sf */
    case 1274:  /* *fmai_fnmadd_v2df */
    case 1273:  /* *fmai_fnmadd_v4sf */
    case 1272:  /* *fmai_fmsub_v2df */
    case 1271:  /* *fmai_fmsub_v4sf */
    case 1270:  /* *fmai_fmadd_v2df */
    case 1269:  /* *fmai_fmadd_v4sf */
    case 1268:  /* *fma_fmsubadd_v2df */
    case 1266:  /* *fma_fmsubadd_v4sf */
    case 1264:  /* *fma_fmaddsub_v2df */
    case 1262:  /* *fma_fmaddsub_v4sf */
    case 1258:  /* *fma_fnmsub_v2df */
    case 1257:  /* *fma_fnmsub_v4sf */
    case 1256:  /* *fma_fnmsub_df */
    case 1255:  /* *fma_fnmsub_sf */
    case 1252:  /* *fma_fnmadd_v2df */
    case 1251:  /* *fma_fnmadd_v4sf */
    case 1250:  /* *fma_fnmadd_df */
    case 1249:  /* *fma_fnmadd_sf */
    case 1246:  /* *fma_fmsub_v2df */
    case 1245:  /* *fma_fmsub_v4sf */
    case 1244:  /* *fma_fmsub_df */
    case 1243:  /* *fma_fmsub_sf */
    case 1240:  /* *fma_fmadd_v2df */
    case 1239:  /* *fma_fmadd_v4sf */
    case 1238:  /* *fma_fmadd_df */
    case 1237:  /* *fma_fmadd_sf */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1132:  /* *mulv2df3 */
    case 1130:  /* *mulv4sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 740:  /* *fop_df_comm_sse */
    case 739:  /* *fop_sf_comm_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 10 /* 0xa */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 6;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 6;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else
        {
	  return 0;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 5;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if (((which_alternative != 0) && (cached_memory == MEMORY_STORE)) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1905:  /* avx_maskstoreps256 */
    case 1897:  /* avx_maskloadps256 */
    case 1666:  /* sse4a_insertq */
    case 1665:  /* sse4a_insertqi */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 3;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 5;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 4;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 2;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 10 /* 0xa */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 322:  /* *mulhi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else
        {
	  return 0;
        }

    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
    case 320:  /* *muldi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 6;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 323:  /* *mulqi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  return 5;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 492:  /* *ashlqi3_1 */
    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 8;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 8;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 8;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 8;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (get_attr_memory (insn) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 2;
        }

    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1287:  /* sse_cvttps2pi */
    case 810:  /* sse4_1_rounddf2 */
    case 809:  /* sse4_1_roundsf2 */
    case 141:  /* *truncdfsf_fast_sse */
    case 136:  /* *extendsfdf2_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 8;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 13 /* 0xd */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && (which_alternative == 1))
        {
	  return 8;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((cached_memory == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 8;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || (((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || (((!((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0xf)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1348:  /* avx2_vec_dupv4sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
      return 2;

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || (((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0x3f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))))
        {
	  return 4;
        }
      else if (((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || ((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 5)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x78)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 2;
        }
      else if ((((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0xff)) && ((((1 << which_alternative) & 0x1858)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || (((((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || (((which_alternative == 9) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x1bff)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((!((1 << which_alternative) & 0xff)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))))
        {
	  return 4;
        }
      else if (((((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || (((which_alternative == 9) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x1bff)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((which_alternative == 7) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 7)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 8;
        }
      else
        {
	  return 2;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 2)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_STORE)) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 4;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 4;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if (((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_STORE)) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (cached_memory == MEMORY_LOAD))
        {
	  return 4;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else if (((cached_type == TYPE_IMOV) && (cached_memory == MEMORY_STORE)) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE)))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && ((pic_32bit_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || (((!((1 << which_alternative) & 0xffc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || (((((1 << which_alternative) & 0xf80)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || ((((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((((1 << which_alternative) & 0xf80)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((((1 << which_alternative) & 0xfb8)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))))))))
        {
	  return 4;
        }
      else if ((((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)) || ((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 6)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x623f)) && (((((1 << which_alternative) & 0xaa0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0x1ea0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || (((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((!((1 << which_alternative) & 0x623f)) && ((((1 << which_alternative) & 0xaa0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((!((1 << which_alternative) & 0x6227)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && ((pic_32bit_operand (operands[1], VOIDmode)) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 1;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || (((!((1 << which_alternative) & 0x1fff0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))) || (((((1 << which_alternative) & 0x7c00)) && (((((1 << which_alternative) & 0xa00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || ((!((1 << which_alternative) & 0xa01)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || (((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((((1 << which_alternative) & 0x7c00)) && ((((1 << which_alternative) & 0xa00)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || ((((1 << which_alternative) & 0x7de0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 9)
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1088:  /* *movv2df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1080:  /* *movv4si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 2)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 5;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 4;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 3;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else
        {
	  return 0;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || (cached_memory == MEMORY_STORE))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1889:  /* vec_set_hi_v8si */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1332:  /* vec_interleave_highv4sf */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 39:  /* *cmpfp_iu_387 */
    case 38:  /* *cmpfp_iu_sse */
    case 36:  /* *cmpfp_i_i387 */
    case 35:  /* *cmpfp_i_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (which_alternative == 0)
        {
	  return 2;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else
        {
	  return 2;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 853:  /* cld */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
    case 42:  /* *pushdi2_rex64 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
      return 1;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 1;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 5;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 0;
        }
      else if ((cached_type == TYPE_PUSH) || (cached_type == TYPE_POP))
        {
	  return 1;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 3;
        }
      else if (cached_type == TYPE_LEA)
        {
	  return 1;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))))
        {
	  return 10 /* 0xa */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 10 /* 0xa */;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))))
        {
	  return 6;
        }
      else if (((cached_bdver1_decode = get_attr_bdver1_decode (insn)) == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 1;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 4;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 5;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if (((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)) || ((cached_bdver1_decode == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 5;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 4;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 13 /* 0xd */;
        }
      else if ((cached_type == TYPE_FMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 8;
        }
      else if (((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))) || ((cached_type == TYPE_FISTP) || ((cached_type == TYPE_FISTTP) || (cached_type == TYPE_FMOV))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 6;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 46 /* 0x2e */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 42 /* 0x2a */;
        }
      else if ((cached_type == TYPE_FPSPC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 103 /* 0x67 */;
        }
      else if ((cached_type == TYPE_FCMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 17 /* 0x11 */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 15 /* 0xf */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_bdver1_decode == BDVER1_DECODE_DOUBLE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 6;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DOUBLE) && (cached_type == TYPE_FCMP))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_FCMP) || (cached_type == TYPE_FXCH))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_prefix = get_attr_prefix (insn)) == PREFIX_VEX) && (((cached_movu = get_attr_movu (insn)) == MOVU_1) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_movu = get_attr_movu (insn)) == MOVU_1) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || (cached_mode == MODE_V4DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 5;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_movu = get_attr_movu (insn)) == MOVU_1) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || (((cached_prefix = get_attr_prefix (insn)) == PREFIX_VEX) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || ((cached_mode == MODE_V4DF) || (cached_mode == MODE_OI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 5;
        }
      else if (((cached_type == TYPE_SSEMOV) && (((((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)) || (((cached_mode = get_attr_mode (insn)) == MODE_DI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))) || (((cached_prefix = get_attr_prefix (insn)) == PREFIX_VEX) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))) || (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || ((cached_mode == MODE_V4DF) || (cached_mode == MODE_OI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 5;
        }
      else if (((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))) || (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || ((cached_mode == MODE_V4DF) || (cached_mode == MODE_OI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 3;
        }
      else if (((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))) || (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 2;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && (((cached_mode = get_attr_mode (insn)) == MODE_V8SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 7;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_mode = get_attr_mode (insn)) == MODE_V8SF))
        {
	  return 3;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECOMI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if (cached_type == TYPE_SSECOMI)
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 4;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || (cached_mode == MODE_V4DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 11 /* 0xb */;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || (cached_mode == MODE_V4DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 7;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 10 /* 0xa */;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEIMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 8;
        }
      else if ((cached_type == TYPE_SSEIMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 4;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 6;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 27 /* 0x1b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V8SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V8SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_DF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 31 /* 0x1f */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_DF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 27 /* 0x1b */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_V4SF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 28 /* 0x1c */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_V4SF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 24 /* 0x18 */;
        }
      else if ((cached_type == TYPE_SSEINS) && ((cached_mode = get_attr_mode (insn)) == MODE_TI))
        {
	  return 3;
        }
      else
        {
	  return 0;
        }

    default:
      return 0;

    }
}

int
internal_dfa_insn_code_bdver2 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  enum attr_bdver1_decode cached_bdver1_decode ATTRIBUTE_UNUSED;
  enum attr_prefix cached_prefix ATTRIBUTE_UNUSED;
  enum attr_movu cached_movu ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1928:  /* avx_vec_concatv8sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if (which_alternative == 0)
        {
	  return 377 /* 0x179 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1929:  /* avx_vec_concatv4df */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 367 /* 0x16f */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1862:  /* avx_vbroadcastf128_v8sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if (which_alternative != 0)
        {
	  return 377 /* 0x179 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative != 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1731:  /* sse4_2_pcmpestr_cconly */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1690:  /* avx2_pblenddv4si */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 363 /* 0x16b */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 371 /* 0x173 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1674:  /* sse4_1_blendvpd */
    case 1672:  /* sse4_1_blendvps */
    case 1670:  /* sse4_1_blendpd */
    case 1668:  /* sse4_1_blendps */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 363 /* 0x16b */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 371 /* 0x173 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1666:  /* sse4a_insertq */
    case 1665:  /* sse4a_insertqi */
      return 422 /* 0x1a6 */;

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((((1 << which_alternative) & 0x1f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((((1 << which_alternative) & 0x7f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x63)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x63))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((((1 << which_alternative) & 0x1e)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((((1 << which_alternative) & 0xe)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 2) || ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 367 /* 0x16f */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (cached_memory == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1510:  /* *andnotv2di3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1509:  /* *andnotv4di3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1508:  /* *andnotv4si3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1507:  /* *andnotv8si3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1506:  /* *andnotv8hi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1505:  /* *andnotv16hi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1504:  /* *andnotv16qi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1503:  /* *andnotv32qi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 376 /* 0x178 */;
        }
      else if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 410 /* 0x19a */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 411 /* 0x19b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1631:  /* ssse3_pmaddubsw */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1436:  /* *avx2_pmaddwd */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 412 /* 0x19c */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 413 /* 0x19d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x12)) && ((((1 << which_alternative) & 0x43)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 363 /* 0x16b */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x24)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x43)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((1 << which_alternative) & 0x24)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((which_alternative != 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((which_alternative != 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 5)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 9) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 9)
        {
	  return 343 /* 0x157 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((!((1 << which_alternative) & 0xfe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((!((1 << which_alternative) & 0xfe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x620)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 5)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 5) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 5)
        {
	  return 343 /* 0x157 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((!((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((!((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((((1 << which_alternative) & 0x13)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 3) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 3)
        {
	  return 343 /* 0x157 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 4)
        {
	  return 343 /* 0x157 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((((1 << which_alternative) & 0x9)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 10)
        {
	  return 343 /* 0x157 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0xa0)) && ((((1 << which_alternative) & 0x3b)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 367 /* 0x16f */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x3b)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x3b)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 10) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 10) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 10)
        {
	  return 343 /* 0x157 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0xa0)) && ((((1 << which_alternative) & 0x39)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 367 /* 0x16f */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x39)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((((1 << which_alternative) & 0x39)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x6c1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xc1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xc1))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1352:  /* *vec_concatv4sf */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x5)) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 363 /* 0x16b */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x5)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x2f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 363 /* 0x16b */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative != 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative != 1)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1905:  /* avx_maskstoreps256 */
    case 1897:  /* avx_maskloadps256 */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1347:  /* avx2_vec_dupv8sf */
      return 377 /* 0x179 */;

    case 1346:  /* sse_movss */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 367 /* 0x16f */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 368 /* 0x170 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 374 /* 0x176 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xa)) && ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 363 /* 0x16b */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xa)) && ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 363 /* 0x16b */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 398 /* 0x18e */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 399 /* 0x18f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1889:  /* vec_set_hi_v8si */
    case 1887:  /* vec_set_lo_v8si */
    case 1886:  /* vec_set_hi_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1332:  /* vec_interleave_highv4sf */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 378 /* 0x17a */;
        }
      else
        {
	  return 379 /* 0x17b */;
        }

    case 1890:  /* vec_set_hi_v8sf */
    case 1888:  /* vec_set_lo_v8sf */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1339:  /* avx_shufps256_1 */
    case 1333:  /* avx_unpcklps256 */
    case 1331:  /* avx_unpckhps256 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1213:  /* avx_andnotv8sf3 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 376 /* 0x178 */;
        }
      else
        {
	  return 377 /* 0x179 */;
        }

    case 1140:  /* sse2_vmdivv2df3 */
    case 1138:  /* sse2_divv2df3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 418 /* 0x1a2 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 419 /* 0x1a3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1137:  /* avx_divv4df3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 414 /* 0x19e */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 415 /* 0x19f */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1139:  /* sse_vmdivv4sf3 */
    case 1136:  /* sse_divv4sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 420 /* 0x1a4 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 421 /* 0x1a5 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1135:  /* avx_divv8sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 416 /* 0x1a0 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 417 /* 0x1a1 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1677:  /* avx_dppd256 */
    case 1675:  /* avx_dpps256 */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1131:  /* *mulv4df3 */
    case 1129:  /* *mulv8sf3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 406 /* 0x196 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 407 /* 0x197 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1935:  /* vcvtps2ph256 */
    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1932:  /* vcvtph2ps256 */
    case 1931:  /* *vcvtph2ps_load */
    case 1930:  /* vcvtph2ps */
    case 1727:  /* sse4_1_roundsd */
    case 1726:  /* sse4_1_roundss */
    case 1725:  /* sse4_1_roundpd */
    case 1724:  /* avx_roundpd256 */
    case 1723:  /* sse4_1_roundps */
    case 1722:  /* avx_roundps256 */
    case 1328:  /* sse2_cvtps2pd */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1326:  /* avx_cvtps2pd256 */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1315:  /* sse2_cvtdq2pd */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1313:  /* floatv4siv4df2 */
    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1302:  /* sse2_cvtpi2pd */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1298:  /* avx_cvtps2dq256 */
    case 1297:  /* floatv4siv4sf2 */
    case 1296:  /* floatv8siv8sf2 */
    case 1286:  /* sse_cvtps2pi */
    case 1285:  /* sse_cvtpi2ps */
    case 1112:  /* sse2_movntv2di */
    case 1111:  /* avx_movntv4di */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode)))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode)))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((cached_memory == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode)))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((cached_memory == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode)))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1601:  /* avx2_pmovmskb */
    case 1106:  /* sse2_movntidi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 366 /* 0x16e */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1685:  /* avx2_pblendvb */
    case 1679:  /* avx2_movntdqa */
    case 1673:  /* avx_blendvpd256 */
    case 1671:  /* avx_blendvps256 */
    case 1669:  /* avx_blendpd256 */
    case 1667:  /* avx_blendps256 */
    case 1599:  /* avx_movmskpd256 */
    case 1597:  /* avx_movmskps256 */
    case 1109:  /* avx_movntv4df */
    case 1107:  /* avx_movntv8sf */
    case 1103:  /* avx_lddqu256 */
    case 1101:  /* avx_storedqu256 */
    case 1099:  /* avx_loaddqu256 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 364 /* 0x16c */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 370 /* 0x172 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 373 /* 0x175 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1098:  /* sse2_storeupd */
    case 1096:  /* sse_storeups */
    case 1094:  /* sse2_loadupd */
    case 1092:  /* sse_loadups */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 362 /* 0x16a */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 371 /* 0x173 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1097:  /* avx_storeupd256 */
    case 1095:  /* avx_storeups256 */
    case 1093:  /* avx_loadupd256 */
    case 1091:  /* avx_loadups256 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 361 /* 0x169 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 370 /* 0x172 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 373 /* 0x175 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1680:  /* sse4_1_movntdqa */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1600:  /* sse2_movmskpd */
    case 1598:  /* sse_movmskps */
    case 1110:  /* sse2_movntv2df */
    case 1108:  /* sse_movntv4sf */
    case 1104:  /* sse3_lddqu */
    case 1102:  /* sse2_storedqu */
    case 1100:  /* sse2_loaddqu */
    case 1089:  /* sse2_movq128 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 371 /* 0x173 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1088:  /* *movv2df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))) || (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1087:  /* *movv4df_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1086:  /* *movv4sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 376 /* 0x178 */;
        }
      else if ((which_alternative == 0) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return 377 /* 0x179 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1084:  /* *movv1ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1083:  /* *movv2ti_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1082:  /* *movv2di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1081:  /* *movv4di_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1080:  /* *movv4si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1079:  /* *movv8si_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1078:  /* *movv8hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1077:  /* *movv16hi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1076:  /* *movv16qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))) || (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0xe)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((which_alternative == 4) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 4) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 4)
        {
	  return 343 /* 0x157 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 3) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((((1 << which_alternative) & 0xc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 1)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1064:  /* *mmx_concatv2si */
    case 994:  /* *mmx_concatv2sf */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1602:  /* sse2_pmovmskb */
    case 1105:  /* sse2_movntisi */
    case 969:  /* sse_movntq */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 328 /* 0x148 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0xc7f)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0xc71)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 6)
        {
	  return 379 /* 0x17b */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((((1 << which_alternative) & 0x300)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((!((1 << which_alternative) & 0xf00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 366 /* 0x16e */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x2ff)) && ((((1 << which_alternative) & 0x300)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x2c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 9)
        {
	  return 379 /* 0x17b */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 328 /* 0x148 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((((1 << which_alternative) & 0x640)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((!((1 << which_alternative) & 0x1e40)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 366 /* 0x16e */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x627f)) && ((((1 << which_alternative) & 0x640)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x6271)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x240)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x240))
        {
	  return 379 /* 0x17b */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((((1 << which_alternative) & 0x30)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((!((1 << which_alternative) & 0x1ff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 366 /* 0x16e */;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x1c7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 8) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 8)
        {
	  return 379 /* 0x17b */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1366:  /* vec_extract_hi_v8si */
    case 1363:  /* vec_extract_hi_v4df */
    case 1362:  /* vec_extract_hi_v4di */
    case 1348:  /* avx2_vec_dupv4sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
      return 379 /* 0x17b */;

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (cached_memory == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 328 /* 0x148 */;
        }
      else
        {
	  return 354 /* 0x162 */;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 353 /* 0x161 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 354 /* 0x162 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 894:  /* *movxfcc_1 */
      return 354 /* 0x162 */;

    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
      return 327 /* 0x147 */;

    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 852:  /* movmsk_df */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 368 /* 0x170 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 374 /* 0x176 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 345 /* 0x159 */;
        }
      else if ((mult_operator (operands[3], XFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if (mult_operator (operands[3], XFmode))
        {
	  return 347 /* 0x15b */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 349 /* 0x15d */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 350 /* 0x15e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], XFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (! (mult_operator (operands[3], XFmode)))
        {
	  return 345 /* 0x159 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 346 /* 0x15a */;
        }
      else
        {
	  return 347 /* 0x15b */;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 345 /* 0x159 */;
        }
      else if ((mult_operator (operands[3], DFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return 347 /* 0x15b */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 349 /* 0x15d */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 350 /* 0x15e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 345 /* 0x159 */;
        }
      else if ((mult_operator (operands[3], SFmode)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return 347 /* 0x15b */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 349 /* 0x15d */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 350 /* 0x15e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((mult_operator (operands[3], DFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if (((mult_operator (operands[3], DFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 418 /* 0x1a2 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 419 /* 0x1a3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((mult_operator (operands[3], SFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if (((mult_operator (operands[3], SFmode)) || ((cached_type = get_attr_type (insn)) == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 420 /* 0x1a4 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 421 /* 0x1a5 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 345 /* 0x159 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return 347 /* 0x15b */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 349 /* 0x15d */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 350 /* 0x15e */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 418 /* 0x1a2 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 419 /* 0x1a3 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_unit == UNIT_INTEGER) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 345 /* 0x159 */;
        }
      else if (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if ((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return 347 /* 0x15b */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 349 /* 0x15d */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 350 /* 0x15e */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if ((((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) || (cached_type == TYPE_SSEADD)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 420 /* 0x1a4 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 421 /* 0x1a5 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (! (mult_operator (operands[3], DFmode)))
        {
	  return 345 /* 0x159 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 346 /* 0x15a */;
        }
      else
        {
	  return 347 /* 0x15b */;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if ((! (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (! (mult_operator (operands[3], SFmode)))
        {
	  return 345 /* 0x159 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 346 /* 0x15a */;
        }
      else
        {
	  return 347 /* 0x15b */;
        }

    case 1747:  /* xop_pmadcswd */
    case 1746:  /* xop_pmadcsswd */
    case 1745:  /* xop_pmacswd */
    case 1744:  /* xop_pmacsswd */
    case 1743:  /* xop_pmacsdqh */
    case 1742:  /* xop_pmacsdql */
    case 1741:  /* xop_pmacssdqh */
    case 1740:  /* xop_pmacssdql */
    case 1739:  /* xop_pmacssdd */
    case 1738:  /* xop_pmacsdd */
    case 1737:  /* xop_pmacssww */
    case 1736:  /* xop_pmacsww */
    case 1678:  /* sse4_1_dppd */
    case 1676:  /* sse4_1_dpps */
    case 1284:  /* *fma4i_vmfnmsub_v2df */
    case 1283:  /* *fma4i_vmfnmsub_v4sf */
    case 1282:  /* *fma4i_vmfnmadd_v2df */
    case 1281:  /* *fma4i_vmfnmadd_v4sf */
    case 1280:  /* *fma4i_vmfmsub_v2df */
    case 1279:  /* *fma4i_vmfmsub_v4sf */
    case 1278:  /* *fma4i_vmfmadd_v2df */
    case 1277:  /* *fma4i_vmfmadd_v4sf */
    case 1276:  /* *fmai_fnmsub_v2df */
    case 1275:  /* *fmai_fnmsub_v4sf */
    case 1274:  /* *fmai_fnmadd_v2df */
    case 1273:  /* *fmai_fnmadd_v4sf */
    case 1272:  /* *fmai_fmsub_v2df */
    case 1271:  /* *fmai_fmsub_v4sf */
    case 1270:  /* *fmai_fmadd_v2df */
    case 1269:  /* *fmai_fmadd_v4sf */
    case 1268:  /* *fma_fmsubadd_v2df */
    case 1266:  /* *fma_fmsubadd_v4sf */
    case 1264:  /* *fma_fmaddsub_v2df */
    case 1262:  /* *fma_fmaddsub_v4sf */
    case 1258:  /* *fma_fnmsub_v2df */
    case 1257:  /* *fma_fnmsub_v4sf */
    case 1256:  /* *fma_fnmsub_df */
    case 1255:  /* *fma_fnmsub_sf */
    case 1252:  /* *fma_fnmadd_v2df */
    case 1251:  /* *fma_fnmadd_v4sf */
    case 1250:  /* *fma_fnmadd_df */
    case 1249:  /* *fma_fnmadd_sf */
    case 1246:  /* *fma_fmsub_v2df */
    case 1245:  /* *fma_fmsub_v4sf */
    case 1244:  /* *fma_fmsub_df */
    case 1243:  /* *fma_fmsub_sf */
    case 1240:  /* *fma_fmadd_v2df */
    case 1239:  /* *fma_fmadd_v4sf */
    case 1238:  /* *fma_fmadd_df */
    case 1237:  /* *fma_fmadd_sf */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1132:  /* *mulv2df3 */
    case 1130:  /* *mulv4sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 740:  /* *fop_df_comm_sse */
    case 739:  /* *fop_sf_comm_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 408 /* 0x198 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 409 /* 0x199 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], DFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], DFmode))))
        {
	  return 345 /* 0x159 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], DFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], DFmode)))
        {
	  return 347 /* 0x15b */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (mult_operator (operands[3], SFmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if ((which_alternative == 0) && (! (mult_operator (operands[3], SFmode))))
        {
	  return 345 /* 0x159 */;
        }
      else if (((which_alternative == 0) && (mult_operator (operands[3], SFmode))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if ((which_alternative == 0) && (mult_operator (operands[3], SFmode)))
        {
	  return 347 /* 0x15b */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if ((((1 << which_alternative) & 0x6)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
      return 331 /* 0x14b */;

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if ((which_alternative != 0) && (cached_memory == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 652:  /* leave_rex64 */
    case 651:  /* leave */
      return 319 /* 0x13f */;

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      return 316 /* 0x13c */;

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 328 /* 0x148 */;
        }
      else
        {
	  return 331 /* 0x14b */;
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 380 /* 0x17c */;
        }
      else
        {
	  return 381 /* 0x17d */;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 328 /* 0x148 */;
        }
      else
        {
	  return 336 /* 0x150 */;
        }

    case 491:  /* *ashlhi3_1 */
    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 329 /* 0x149 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 332 /* 0x14c */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 335 /* 0x14f */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 337 /* 0x151 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
      return 348 /* 0x15c */;

    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 325 /* 0x145 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 326 /* 0x146 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 321 /* 0x141 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 323 /* 0x143 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 324 /* 0x144 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 322 /* 0x142 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 324 /* 0x144 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 322:  /* *mulhi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 322 /* 0x142 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 324 /* 0x144 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
    case 320:  /* *muldi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 321 /* 0x141 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 323 /* 0x143 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 323:  /* *mulqi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 319:  /* *mulsi3_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 322 /* 0x142 */;
        }
      else if ((cached_memory == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))
        {
	  return 324 /* 0x144 */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 328 /* 0x148 */;
        }
      else
        {
	  return 331 /* 0x14b */;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 492:  /* *ashlqi3_1 */
    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
      return 320 /* 0x140 */;

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 388 /* 0x184 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 389 /* 0x185 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 388 /* 0x184 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 389 /* 0x185 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 388 /* 0x184 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 389 /* 0x185 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 388 /* 0x184 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 389 /* 0x185 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 388 /* 0x184 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 389 /* 0x185 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 338 /* 0x152 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 340 /* 0x154 */;
        }
      else
        {
	  return 343 /* 0x157 */;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 338 /* 0x152 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 340 /* 0x154 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
      return 342 /* 0x156 */;

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 388 /* 0x184 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 389 /* 0x185 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 339 /* 0x153 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 341 /* 0x155 */;
        }
      else
        {
	  return 343 /* 0x157 */;
        }

    case 143:  /* *truncdfsf_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (which_alternative == 0)
        {
	  return 343 /* 0x157 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 386 /* 0x182 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 387 /* 0x183 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 338 /* 0x152 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 339 /* 0x153 */;
        }
      else if ((which_alternative == 1) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 340 /* 0x154 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 341 /* 0x155 */;
        }
      else
        {
	  return 343 /* 0x157 */;
        }

    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1287:  /* sse_cvttps2pi */
    case 810:  /* sse4_1_rounddf2 */
    case 809:  /* sse4_1_roundsf2 */
    case 141:  /* *truncdfsf_fast_sse */
    case 136:  /* *extendsfdf2_sse */
      extract_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 385 /* 0x181 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 386 /* 0x182 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 387 /* 0x183 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (which_alternative == 1))
        {
	  return 338 /* 0x152 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (cached_memory == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if (((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && (which_alternative == 1))
        {
	  return 340 /* 0x154 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return 343 /* 0x157 */;
        }
      else if ((which_alternative == 2) && ((cached_memory == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((cached_memory == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode))))))
        {
	  return 385 /* 0x181 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 386 /* 0x182 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 387 /* 0x183 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 328 /* 0x148 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x1f)) && ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 372 /* 0x174 */;
        }
      else if (cached_memory == MEMORY_NONE)
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0xf)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
      return 359 /* 0x167 */;

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((((1 << which_alternative) & 0x18)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 343 /* 0x157 */;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if (((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0xce3f)) && ((((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x3f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 5) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 5)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 328 /* 0x148 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 343 /* 0x157 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || ((((1 << which_alternative) & 0x880)) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xee0)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && (((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || ((((1 << which_alternative) & 0x880)) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xee0)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0x23f)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((((1 << which_alternative) & 0x78)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return 343 /* 0x157 */;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((((1 << which_alternative) & 0x1858)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 366 /* 0x16e */;
        }
      else if ((((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || (((which_alternative == 9) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x1bff)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if (((which_alternative == 8) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || (((which_alternative == 9) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x1bff)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((!((1 << which_alternative) & 0xff)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 7) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 7)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 338 /* 0x152 */;
        }
      else if ((cached_memory == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))
        {
	  return 340 /* 0x154 */;
        }
      else
        {
	  return 343 /* 0x157 */;
        }

    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 328 /* 0x148 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x3)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 2)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 317 /* 0x13d */;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return 328 /* 0x148 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 328 /* 0x148 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (cached_memory == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 649:  /* set_got_offset_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 333 /* 0x14d */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
      return 330 /* 0x14a */;

    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      return 333 /* 0x14d */;

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (cached_memory == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_IMOV) && (cached_memory == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_type == TYPE_IMOV) && (cached_memory == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if (cached_memory == MEMORY_BOTH)
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 320 /* 0x140 */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((!((1 << which_alternative) & 0xffc)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((((1 << which_alternative) & 0xc0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if (((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 368 /* 0x170 */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0xf80)) && ((((1 << which_alternative) & 0xc0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if (((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 374 /* 0x176 */;
        }
      else if ((((1 << which_alternative) & 0xfb8)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 6) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 6)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 328 /* 0x148 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((((1 << which_alternative) & 0xaa0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((!((1 << which_alternative) & 0x1ea0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 366 /* 0x16e */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x623f)) && ((((1 << which_alternative) & 0xaa0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x6227)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((((1 << which_alternative) & 0x220)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return 379 /* 0x17b */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((((1 << which_alternative) & 0x6000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return 320 /* 0x140 */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 328 /* 0x148 */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 331 /* 0x14b */;
        }
      else if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH))
        {
	  return 334 /* 0x14e */;
        }
      else if ((!((1 << which_alternative) & 0x1fff0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 336 /* 0x150 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((((1 << which_alternative) & 0xa00)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((!((1 << which_alternative) & 0xa01)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 366 /* 0x16e */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((((1 << which_alternative) & 0x7c00)) && ((((1 << which_alternative) & 0xa00)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((((1 << which_alternative) & 0x7de0)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 9) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 9)
        {
	  return 379 /* 0x17b */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((((1 << which_alternative) & 0x18000)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((which_alternative != 0) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 328 /* 0x148 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))))))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((!((1 << which_alternative) & 0x7)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 2) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 2)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 364 /* 0x16c */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 370 /* 0x172 */;
        }
      else if ((which_alternative != 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 373 /* 0x175 */;
        }
      else if ((which_alternative != 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if (which_alternative == 0)
        {
	  return 379 /* 0x17b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return 334 /* 0x14e */;
        }
      else if (cached_memory == MEMORY_STORE)
        {
	  return 336 /* 0x150 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      return 318 /* 0x13e */;

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      return 317 /* 0x13d */;

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 317 /* 0x13d */;
        }
      else
        {
	  return 328 /* 0x148 */;
        }

    case 39:  /* *cmpfp_iu_387 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 357 /* 0x165 */;
        }
      else
        {
	  return 358 /* 0x166 */;
        }

    case 36:  /* *cmpfp_i_i387 */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 355 /* 0x163 */;
        }
      else
        {
	  return 356 /* 0x164 */;
        }

    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      if (get_attr_memory (insn) == MEMORY_LOAD)
        {
	  return 382 /* 0x17e */;
        }
      else
        {
	  return 383 /* 0x17f */;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 355 /* 0x163 */;
        }
      else if (which_alternative == 0)
        {
	  return 356 /* 0x164 */;
        }
      else if ((which_alternative == 1) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 382 /* 0x17e */;
        }
      else
        {
	  return 383 /* 0x17f */;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 853:  /* cld */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
      return 328 /* 0x148 */;

    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)
        {
	  return 328 /* 0x148 */;
        }
      else if (cached_memory == MEMORY_LOAD)
        {
	  return 331 /* 0x14b */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_CALL) || (cached_type == TYPE_CALLV))
        {
	  return 316 /* 0x13c */;
        }
      else if (cached_type == TYPE_PUSH)
        {
	  return 317 /* 0x13d */;
        }
      else if (cached_type == TYPE_POP)
        {
	  return 318 /* 0x13e */;
        }
      else if (cached_type == TYPE_LEAVE)
        {
	  return 319 /* 0x13f */;
        }
      else if (cached_type == TYPE_LEA)
        {
	  return 320 /* 0x140 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 321 /* 0x141 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 322 /* 0x142 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH))))
        {
	  return 323 /* 0x143 */;
        }
      else if ((cached_type == TYPE_IMUL) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 324 /* 0x144 */;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN)))
        {
	  return 325 /* 0x145 */;
        }
      else if ((cached_type == TYPE_IDIV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_BOTH)))
        {
	  return 326 /* 0x146 */;
        }
      else if ((cached_type == TYPE_STR) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || ((cached_memory == MEMORY_BOTH) || (cached_memory == MEMORY_STORE))))
        {
	  return 327 /* 0x147 */;
        }
      else if (((cached_bdver1_decode = get_attr_bdver1_decode (insn)) == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 328 /* 0x148 */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) || (cached_memory == MEMORY_UNKNOWN))))
        {
	  return 329 /* 0x149 */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 330 /* 0x14a */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 331 /* 0x14b */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 332 /* 0x14c */;
        }
      else if ((cached_type == TYPE_IMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE))
        {
	  return 333 /* 0x14d */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 334 /* 0x14e */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_BOTH)))
        {
	  return 335 /* 0x14f */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DIRECT) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 336 /* 0x150 */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_VECTOR) && ((((cached_unit = get_attr_unit (insn)) == UNIT_INTEGER) || (cached_unit == UNIT_UNKNOWN)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_STORE)))
        {
	  return 337 /* 0x151 */;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 338 /* 0x152 */;
        }
      else if ((cached_type == TYPE_FMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 339 /* 0x153 */;
        }
      else if ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)) && ((cached_mode = get_attr_mode (insn)) == MODE_XF)))
        {
	  return 340 /* 0x154 */;
        }
      else if ((cached_type == TYPE_FMOV) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 341 /* 0x155 */;
        }
      else if ((cached_type == TYPE_FISTP) || (cached_type == TYPE_FISTTP))
        {
	  return 342 /* 0x156 */;
        }
      else if (cached_type == TYPE_FMOV)
        {
	  return 343 /* 0x157 */;
        }
      else if ((cached_type == TYPE_FOP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 344 /* 0x158 */;
        }
      else if (cached_type == TYPE_FOP)
        {
	  return 345 /* 0x159 */;
        }
      else if ((cached_type == TYPE_FMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 346 /* 0x15a */;
        }
      else if (cached_type == TYPE_FMUL)
        {
	  return 347 /* 0x15b */;
        }
      else if (cached_type == TYPE_FSGN)
        {
	  return 348 /* 0x15c */;
        }
      else if ((cached_type == TYPE_FDIV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 349 /* 0x15d */;
        }
      else if (cached_type == TYPE_FDIV)
        {
	  return 350 /* 0x15e */;
        }
      else if ((cached_type == TYPE_FPSPC) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 351 /* 0x15f */;
        }
      else if ((cached_type == TYPE_FCMOV) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 353 /* 0x161 */;
        }
      else if (cached_type == TYPE_FCMOV)
        {
	  return 354 /* 0x162 */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_bdver1_decode == BDVER1_DECODE_DOUBLE) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 355 /* 0x163 */;
        }
      else if ((cached_bdver1_decode == BDVER1_DECODE_DOUBLE) && (cached_type == TYPE_FCMP))
        {
	  return 356 /* 0x164 */;
        }
      else if ((cached_type == TYPE_FCMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 357 /* 0x165 */;
        }
      else if (cached_type == TYPE_FCMP)
        {
	  return 358 /* 0x166 */;
        }
      else if (cached_type == TYPE_FXCH)
        {
	  return 359 /* 0x167 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_prefix = get_attr_prefix (insn)) == PREFIX_VEX) && (((cached_movu = get_attr_movu (insn)) == MOVU_1) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))))
        {
	  return 360 /* 0x168 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_movu = get_attr_movu (insn)) == MOVU_1) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || (cached_mode == MODE_V4DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 361 /* 0x169 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_movu = get_attr_movu (insn)) == MOVU_1) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 362 /* 0x16a */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_prefix = get_attr_prefix (insn)) == PREFIX_VEX) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 363 /* 0x16b */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || ((cached_mode == MODE_V4DF) || (cached_mode == MODE_OI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 364 /* 0x16c */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 365 /* 0x16d */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_mode = get_attr_mode (insn)) == MODE_DI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 366 /* 0x16e */;
        }
      else if ((cached_type == TYPE_SSEMOV) && (((cached_prefix = get_attr_prefix (insn)) == PREFIX_VEX) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))))
        {
	  return 367 /* 0x16f */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 368 /* 0x170 */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 369 /* 0x171 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || ((cached_mode == MODE_V4DF) || (cached_mode == MODE_OI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 370 /* 0x172 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V4SF) || ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH))))
        {
	  return 371 /* 0x173 */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_STORE) || (cached_memory == MEMORY_BOTH)))
        {
	  return 372 /* 0x174 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || ((cached_mode == MODE_V4DF) || (cached_mode == MODE_OI))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 373 /* 0x175 */;
        }
      else if ((cached_type == TYPE_SSEMOV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 374 /* 0x176 */;
        }
      else if (((cached_type == TYPE_MMXMOV) || (cached_type == TYPE_SSEMOV)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 375 /* 0x177 */;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && (((cached_mode = get_attr_mode (insn)) == MODE_V8SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 376 /* 0x178 */;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_mode = get_attr_mode (insn)) == MODE_V8SF))
        {
	  return 377 /* 0x179 */;
        }
      else if (((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 378 /* 0x17a */;
        }
      else if ((cached_type == TYPE_SSELOG) || (cached_type == TYPE_SSELOG1))
        {
	  return 379 /* 0x17b */;
        }
      else if ((cached_type == TYPE_SSECMP) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 380 /* 0x17c */;
        }
      else if (cached_type == TYPE_SSECMP)
        {
	  return 381 /* 0x17d */;
        }
      else if ((cached_type == TYPE_SSECOMI) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 382 /* 0x17e */;
        }
      else if (cached_type == TYPE_SSECOMI)
        {
	  return 383 /* 0x17f */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 384 /* 0x180 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && (((register_operand (operands[0], V4DFmode)) || ((register_operand (operands[0], V8SFmode)) || (register_operand (operands[0], V8SImode)))) || ((nonimmediate_operand (operands[1], V4DFmode)) || ((nonimmediate_operand (operands[1], V8SFmode)) || (nonimmediate_operand (operands[1], V8SImode)))))))
        {
	  return 385 /* 0x181 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 386 /* 0x182 */;
        }
      else if ((cached_type == TYPE_SSECVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 387 /* 0x183 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 388 /* 0x184 */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 389 /* 0x185 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 390 /* 0x186 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && (nonimmediate_operand (operands[1], V2DFmode)))))
        {
	  return 391 /* 0x187 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 392 /* 0x188 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V4SFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SImode))))))
        {
	  return 393 /* 0x189 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 394 /* 0x18a */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((register_operand (operands[0], V2DFmode)) && (nonimmediate_operand (operands[1], V4SImode)))))
        {
	  return 395 /* 0x18b */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 396 /* 0x18c */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((register_operand (operands[0], V2DFmode)) && ((nonimmediate_operand (operands[1], V2SImode)) || (nonimmediate_operand (operands[1], V4SFmode))))))
        {
	  return 397 /* 0x18d */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 398 /* 0x18e */;
        }
      else if ((cached_type == TYPE_SSEICVT) && ((((cached_mode = get_attr_mode (insn)) == MODE_SI) || (cached_mode == MODE_DI)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 399 /* 0x18f */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 400 /* 0x190 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V2SImode)))))
        {
	  return 401 /* 0x191 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 402 /* 0x192 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V2DFmode)) && (register_operand (operands[0], V4SImode)))))
        {
	  return 403 /* 0x193 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 404 /* 0x194 */;
        }
      else if ((cached_type == TYPE_SSECVT) && (((cached_memory = get_attr_memory (insn)) == MEMORY_NONE) && ((nonimmediate_operand (operands[1], V4SFmode)) && ((register_operand (operands[0], V2SImode)) || (register_operand (operands[0], V4SImode))))))
        {
	  return 405 /* 0x195 */;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || (cached_mode == MODE_V4DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 406 /* 0x196 */;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((((cached_mode = get_attr_mode (insn)) == MODE_V8SF) || (cached_mode == MODE_V4DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 407 /* 0x197 */;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 408 /* 0x198 */;
        }
      else if (((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSEADD) || (cached_type == TYPE_SSEMULADD))) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 409 /* 0x199 */;
        }
      else if ((cached_type == TYPE_SSEIMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 410 /* 0x19a */;
        }
      else if ((cached_type == TYPE_SSEIMUL) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 411 /* 0x19b */;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD))
        {
	  return 412 /* 0x19c */;
        }
      else if ((cached_type == TYPE_SSEIADD) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE))
        {
	  return 413 /* 0x19d */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 414 /* 0x19e */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V4DF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 415 /* 0x19f */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V8SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 416 /* 0x1a0 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && (((cached_mode = get_attr_mode (insn)) == MODE_V8SF) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 417 /* 0x1a1 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_DF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 418 /* 0x1a2 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_DF) || (cached_mode == MODE_V2DF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 419 /* 0x1a3 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_V4SF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD)))
        {
	  return 420 /* 0x1a4 */;
        }
      else if ((cached_type == TYPE_SSEDIV) && ((((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_V4SF)) && ((cached_memory = get_attr_memory (insn)) == MEMORY_NONE)))
        {
	  return 421 /* 0x1a5 */;
        }
      else if ((cached_type == TYPE_SSEINS) && ((cached_mode = get_attr_mode (insn)) == MODE_TI))
        {
	  return 422 /* 0x1a6 */;
        }
      else
        {
	  return 618 /* 0x26a */;
        }

    default:
      return 618 /* 0x26a */;

    }
}

int
insn_default_latency_btver1 (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return 0;

    }
}

int
internal_dfa_insn_code_btver1 (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return 618 /* 0x26a */;

    }
}

enum attr_atom_sse_attr
get_attr_atom_sse_attr (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 1605:  /* sse_ldmxcsr */
    case 1606:  /* sse_stmxcsr */
      return ATOM_SSE_ATTR_MXCSR;

    case 779:  /* *sqrtsf2_sse */
    case 780:  /* *sqrtdf2_sse */
    case 1144:  /* avx_sqrtv8sf2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1146:  /* avx_sqrtv4df2 */
    case 1147:  /* sse2_sqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
      return ATOM_SSE_ATTR_SQRT;

    case 919:  /* *prefetch_sse */
      return ATOM_SSE_ATTR_PREFETCH;

    case 1607:  /* sse2_clflush */
    case 2009:  /* *sse_sfence */
    case 2010:  /* mfence_sse2 */
      return ATOM_SSE_ATTR_FENCE;

    case 2008:  /* *sse2_lfence */
      return ATOM_SSE_ATTR_LFENCE;

    case 745:  /* *rcpsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 1141:  /* avx_rcpv8sf2 */
    case 1142:  /* sse_rcpv4sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
      return ATOM_SSE_ATTR_RCP;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return ATOM_SSE_ATTR_OTHER;

    }
}

enum attr_atom_unit
get_attr_atom_unit (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 1182:  /* sse3_addsubv2df3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1628:  /* ssse3_phsubswv4hi3 */
      return ATOM_UNIT_COMPLEX;

    case 638:  /* simple_return_internal */
    case 639:  /* simple_return_internal_long */
    case 640:  /* simple_return_pop_internal */
    case 653:  /* split_stack_return */
      return ATOM_UNIT_JEU;

    case 1437:  /* *sse2_pmaddwd */
    case 1595:  /* avx2_psadbw */
    case 1596:  /* sse2_psadbw */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1631:  /* ssse3_pmaddubsw */
      return ATOM_UNIT_SIMUL;

    case 1461:  /* avx2_lshrv2ti3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1647:  /* avx2_palignrv2ti */
    case 1648:  /* ssse3_palignrti */
    case 1649:  /* ssse3_palignrdi */
      return ATOM_UNIT_SISHUF;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return ATOM_UNIT_OTHER;

    }
}

enum attr_amdfam10_decode
get_attr_amdfam10_decode (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return AMDFAM10_DECODE_DIRECT;
        }
      else if (which_alternative == 2)
        {
	  return AMDFAM10_DECODE_VECTOR;
        }
      else
        {
	  return AMDFAM10_DECODE_DOUBLE;
        }

    case 201:  /* *floatsisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 204:  /* *floatdidf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return AMDFAM10_DECODE_DIRECT;
        }
      else if (which_alternative == 1)
        {
	  return AMDFAM10_DECODE_VECTOR;
        }
      else
        {
	  return AMDFAM10_DECODE_DOUBLE;
        }

    case 195:  /* *floatsisf2_vector_mixed */
    case 196:  /* *floatsidf2_vector_mixed */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 208:  /* *floatdidf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return AMDFAM10_DECODE_DIRECT;
        }
      else
        {
	  return AMDFAM10_DECODE_DOUBLE;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 217:  /* *floatsisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 220:  /* *floatdidf2_sse_interunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return AMDFAM10_DECODE_VECTOR;
        }
      else
        {
	  return AMDFAM10_DECODE_DOUBLE;
        }

    case 319:  /* *mulsi3_1 */
    case 320:  /* *muldi3_1 */
    case 321:  /* *mulsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return AMDFAM10_DECODE_VECTOR;
        }
      else
        {
	  return AMDFAM10_DECODE_DIRECT;
        }

    case 322:  /* *mulhi3_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return AMDFAM10_DECODE_VECTOR;
        }
      else
        {
	  return AMDFAM10_DECODE_DIRECT;
        }

    case 326:  /* *umulsidi3_1 */
    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return AMDFAM10_DECODE_DOUBLE;
        }
      else
        {
	  return AMDFAM10_DECODE_DIRECT;
        }

    case 1288:  /* sse_cvtsi2ss */
    case 1289:  /* sse_cvtsi2ssq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1322:  /* sse2_cvtsd2ss */
    case 1323:  /* sse2_cvtss2sd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return AMDFAM10_DECODE_VECTOR;
        }
      else if (which_alternative == 1)
        {
	  return AMDFAM10_DECODE_DOUBLE;
        }
      else
        {
	  return AMDFAM10_DECODE_DIRECT;
        }

    case 75:  /* *swapsi */
    case 76:  /* *swapdi */
    case 77:  /* *swapqi_1 */
    case 78:  /* *swaphi_1 */
    case 154:  /* fix_truncsfdi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 212:  /* *floatsidf2_vector_sse */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 328:  /* *mulsidi3_1 */
    case 329:  /* *mulditi3_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 335:  /* *umulsi3_highpart_1 */
    case 336:  /* *smulsi3_highpart_zext */
    case 337:  /* *umulsi3_highpart_zext */
    case 1291:  /* sse_cvtss2si_2 */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1294:  /* sse_cvttss2si */
    case 1295:  /* sse_cvttss2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1311:  /* sse2_cvttsd2si */
    case 1312:  /* sse2_cvttsd2siq */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1325:  /* *sse2_cvtpd2ps */
      return AMDFAM10_DECODE_DOUBLE;

    case 177:  /* x86_fldcw_1 */
    case 481:  /* x86_64_shld */
    case 482:  /* x86_shld */
    case 511:  /* x86_64_shrd */
    case 512:  /* x86_shrd */
      return AMDFAM10_DECODE_VECTOR;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return AMDFAM10_DECODE_DIRECT;

    }
}

enum attr_athlon_decode
get_attr_athlon_decode (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_memory cached_memory ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1322:  /* sse2_cvtsd2ss */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else if (which_alternative == 1)
        {
	  return ATHLON_DECODE_DOUBLE;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  if (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))
	    {
	      return ATHLON_DECODE_VECTOR;
	    }
	  else
	    {
	      return ATHLON_DECODE_DOUBLE;
	    }
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 322:  /* *mulhi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) || (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON)))) || ((which_alternative == 1) || ((which_alternative == 2) && (memory_operand (operands[1], VOIDmode)))))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ATHLON_DECODE_DIRECT;
        }
      else if (which_alternative == 1)
        {
	  return ATHLON_DECODE_DOUBLE;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  if (which_alternative == 1)
	    {
	      return ATHLON_DECODE_VECTOR;
	    }
	  else
	    {
	      return ATHLON_DECODE_DIRECT;
	    }
        }
      else if (which_alternative == 2)
        {
	  return ATHLON_DECODE_DOUBLE;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  if (which_alternative == 1)
	    {
	      return ATHLON_DECODE_VECTOR;
	    }
	  else
	    {
	      return ATHLON_DECODE_DIRECT;
	    }
        }
      else if (which_alternative == 2)
        {
	  return ATHLON_DECODE_DOUBLE;
        }
      else if (which_alternative == 3)
        {
	  return ATHLON_DECODE_DIRECT;
        }
      else
        {
	  return ATHLON_DECODE_DOUBLE;
        }

    case 236:  /* *floatdixf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 186:  /* *floathixf2_i387 */
      if (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) || ((which_alternative == 0) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 1358:  /* *sse4_1_extractps */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)) && (which_alternative == 1))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7)) || ((((1 << which_alternative) & 0x7)) && (((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE))))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 1) || ((which_alternative == 1) && (memory_operand (operands[1], VOIDmode))))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 1579:  /* *sse2_storeq_rex64 */
    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
    case 63:  /* *movdi_internal */
    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], VOIDmode))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) || ((which_alternative == 0) && (memory_operand (operands[1], VOIDmode))))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_CALL) || ((cached_type == TYPE_IMUL) || ((cached_type == TYPE_IDIV) || ((cached_type == TYPE_OTHER) || ((cached_type == TYPE_MULTI) || ((cached_type == TYPE_FCMOV) || ((cached_type == TYPE_FPSPC) || ((cached_type == TYPE_STR) || ((cached_type == TYPE_POP) || ((cached_type == TYPE_LEAVE) || (((cached_type == TYPE_PUSH) && (memory_operand (operands[1], VOIDmode))) || ((cached_type == TYPE_FMOV) && ((((cached_memory = get_attr_memory (insn)) == MEMORY_LOAD) || (cached_memory == MEMORY_STORE)) && (get_attr_mode (insn) == MODE_XF))))))))))))))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 209:  /* *floatsisf2_vector_sse_with_temp */
    case 210:  /* *floatsidf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ATHLON_DECODE_DOUBLE;
        }
      else if (which_alternative == 1)
        {
	  return ATHLON_DECODE_DIRECT;
        }
      else
        {
	  return ATHLON_DECODE_DOUBLE;
        }

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 217:  /* *floatsisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 220:  /* *floatdidf2_sse_interunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ATHLON_DECODE_DOUBLE;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 323:  /* *mulqi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 331:  /* *umulqihi3_1 */
      if (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DIRECT;
        }

    case 328:  /* *mulsidi3_1 */
    case 329:  /* *mulditi3_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 335:  /* *umulsi3_highpart_1 */
    case 336:  /* *smulsi3_highpart_zext */
    case 337:  /* *umulsi3_highpart_zext */
      if (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_ATHLON))))
        {
	  return ATHLON_DECODE_VECTOR;
        }
      else
        {
	  return ATHLON_DECODE_DOUBLE;
        }

    case 154:  /* fix_truncsfdi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 1290:  /* sse_cvtss2si */
    case 1291:  /* sse_cvtss2si_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1294:  /* sse_cvttss2si */
    case 1295:  /* sse_cvttss2siq */
    case 1307:  /* sse2_cvtsd2si */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1311:  /* sse2_cvttsd2si */
    case 1312:  /* sse2_cvttsd2siq */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ATHLON_DECODE_DOUBLE;
        }
      else
        {
	  return ATHLON_DECODE_VECTOR;
        }

    case 1328:  /* sse2_cvtps2pd */
      return ATHLON_DECODE_DOUBLE;

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 896:  /* *movdfcc_1 */
    case 894:  /* *movxfcc_1 */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
    case 33:  /* x86_sahf_1 */
    case 34:  /* *cmpfp_i_mixed */
    case 35:  /* *cmpfp_i_sse */
    case 36:  /* *cmpfp_i_i387 */
    case 37:  /* *cmpfp_iu_mixed */
    case 38:  /* *cmpfp_iu_sse */
    case 39:  /* *cmpfp_iu_387 */
    case 75:  /* *swapsi */
    case 76:  /* *swapdi */
    case 77:  /* *swapqi_1 */
    case 78:  /* *swaphi_1 */
    case 79:  /* *swapqi_2 */
    case 80:  /* *swaphi_2 */
    case 177:  /* x86_fldcw_1 */
    case 481:  /* x86_64_shld */
    case 482:  /* x86_shld */
    case 511:  /* x86_64_shrd */
    case 512:  /* x86_shrd */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1325:  /* *sse2_cvtpd2ps */
      return ATHLON_DECODE_VECTOR;

    default:
      return ATHLON_DECODE_DIRECT;

    }
}

enum attr_bdver1_decode
get_attr_bdver1_decode (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return BDVER1_DECODE_DIRECT;
        }
      else if (which_alternative == 2)
        {
	  return BDVER1_DECODE_DOUBLE;
        }
      else
        {
	  return BDVER1_DECODE_DIRECT;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return BDVER1_DECODE_DIRECT;
        }
      else if (which_alternative == 2)
        {
	  return BDVER1_DECODE_DOUBLE;
        }
      else if (which_alternative == 3)
        {
	  return BDVER1_DECODE_DIRECT;
        }
      else
        {
	  return BDVER1_DECODE_DOUBLE;
        }

    case 201:  /* *floatsisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 204:  /* *floatdidf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return BDVER1_DECODE_DIRECT;
        }
      else if (which_alternative == 1)
        {
	  return BDVER1_DECODE_DOUBLE;
        }
      else
        {
	  return BDVER1_DECODE_DIRECT;
        }

    case 209:  /* *floatsisf2_vector_sse_with_temp */
    case 210:  /* *floatsidf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return BDVER1_DECODE_DOUBLE;
        }
      else if (which_alternative == 1)
        {
	  return BDVER1_DECODE_DIRECT;
        }
      else
        {
	  return BDVER1_DECODE_DOUBLE;
        }

    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 217:  /* *floatsisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 220:  /* *floatdidf2_sse_interunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return BDVER1_DECODE_DOUBLE;
        }
      else
        {
	  return BDVER1_DECODE_DIRECT;
        }

    case 34:  /* *cmpfp_i_mixed */
    case 35:  /* *cmpfp_i_sse */
    case 36:  /* *cmpfp_i_i387 */
    case 37:  /* *cmpfp_iu_mixed */
    case 38:  /* *cmpfp_iu_sse */
    case 75:  /* *swapsi */
    case 76:  /* *swapdi */
    case 77:  /* *swapqi_1 */
    case 78:  /* *swaphi_1 */
    case 154:  /* fix_truncsfdi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 322:  /* *mulhi3_1 */
    case 1290:  /* sse_cvtss2si */
    case 1291:  /* sse_cvtss2si_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1294:  /* sse_cvttss2si */
    case 1295:  /* sse_cvttss2siq */
    case 1303:  /* sse2_cvtpd2pi */
    case 1304:  /* sse2_cvttpd2pi */
    case 1307:  /* sse2_cvtsd2si */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1311:  /* sse2_cvttsd2si */
    case 1312:  /* sse2_cvttsd2siq */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1328:  /* sse2_cvtps2pd */
      return BDVER1_DECODE_DOUBLE;

    case 176:  /* x86_fnstcw_1 */
    case 177:  /* x86_fldcw_1 */
    case 481:  /* x86_64_shld */
    case 482:  /* x86_shld */
    case 511:  /* x86_64_shrd */
    case 512:  /* x86_shrd */
      return BDVER1_DECODE_VECTOR;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return BDVER1_DECODE_DIRECT;

    }
}

int
get_attr_enabled (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_isa cached_isa ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else if (which_alternative == 2)
        {
	  return 
#line 642 "../../gcc/config/i386/i386.md"
(TARGET_SSE2 && !TARGET_AVX);
        }
      else if (!((1 << which_alternative) & 0x37))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 
#line 646 "../../gcc/config/i386/i386.md"
(TARGET_SSE4_1 && !TARGET_AVX);
        }
      else if (!((1 << which_alternative) & 0xbd))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else if (((1 << which_alternative) & 0xa0))
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else
        {
	  return 1;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 
#line 642 "../../gcc/config/i386/i386.md"
(TARGET_SSE2 && !TARGET_AVX);
        }
      else if (!((1 << which_alternative) & 0xd))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 
#line 642 "../../gcc/config/i386/i386.md"
(TARGET_SSE2 && !TARGET_AVX);
        }
      else if (which_alternative == 2)
        {
	  return 
#line 643 "../../gcc/config/i386/i386.md"
(TARGET_SSE3);
        }
      else if (which_alternative == 1)
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }

    case 1583:  /* *vec_dupv4si */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else if (which_alternative == 1)
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else if (which_alternative == 1)
        {
	  return 
#line 642 "../../gcc/config/i386/i386.md"
(TARGET_SSE2 && !TARGET_AVX);
        }
      else if (which_alternative == 2)
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else if (!((1 << which_alternative) & 0xf))
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else
        {
	  return 1;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else if (!((1 << which_alternative) & 0xf))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else
        {
	  return 1;
        }

    case 1434:  /* *avx2_mulv4siv4di3 */
      extract_constrain_insn_cached (insn);
      return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else if (((1 << which_alternative) & 0x9))
        {
	  return 
#line 642 "../../gcc/config/i386/i386.md"
(TARGET_SSE2 && !TARGET_AVX);
        }
      else if (which_alternative == 2)
        {
	  return 
#line 643 "../../gcc/config/i386/i386.md"
(TARGET_SSE3);
        }
      else if (((1 << which_alternative) & 0x12))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }

    case 1390:  /* vec_dupv2df */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 
#line 643 "../../gcc/config/i386/i386.md"
(TARGET_SSE3);
        }
      else
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x8a))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else if (((1 << which_alternative) & 0x65))
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else
        {
	  return 1;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x94))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else if (((1 << which_alternative) & 0x6a))
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else
        {
	  return 1;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else if (which_alternative == 1)
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else
        {
	  return 1;
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 
#line 643 "../../gcc/config/i386/i386.md"
(TARGET_SSE3);
        }
      else if (((1 << which_alternative) & 0x12))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else if (((1 << which_alternative) & 0x9))
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else
        {
	  return 1;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6))
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else if (which_alternative == 0)
        {
	  return 
#line 644 "../../gcc/config/i386/i386.md"
(TARGET_SSE4_1);
        }
      else if (which_alternative == 6)
        {
	  return 
#line 646 "../../gcc/config/i386/i386.md"
(TARGET_SSE4_1 && !TARGET_AVX);
        }
      else if (((1 << which_alternative) & 0xa0))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else
        {
	  return 1;
        }

    case 1352:  /* *vec_concatv4sf */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x5))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }

    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1387:  /* sse2_loadhpd */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
    case 1345:  /* sse_loadlps */
    case 1343:  /* sse_loadhps */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xa))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else if (((1 << which_alternative) & 0x5))
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else
        {
	  return 1;
        }

    case 1268:  /* *fma_fmsubadd_v2df */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1266:  /* *fma_fmsubadd_v4sf */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1264:  /* *fma_fmaddsub_v2df */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1262:  /* *fma_fmaddsub_v4sf */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1258:  /* *fma_fnmsub_v2df */
    case 1257:  /* *fma_fnmsub_v4sf */
    case 1256:  /* *fma_fnmsub_df */
    case 1255:  /* *fma_fnmsub_sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1252:  /* *fma_fnmadd_v2df */
    case 1251:  /* *fma_fnmadd_v4sf */
    case 1250:  /* *fma_fnmadd_df */
    case 1249:  /* *fma_fnmadd_sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1246:  /* *fma_fmsub_v2df */
    case 1245:  /* *fma_fmsub_v4sf */
    case 1244:  /* *fma_fmsub_df */
    case 1243:  /* *fma_fmsub_sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1240:  /* *fma_fmadd_v2df */
    case 1239:  /* *fma_fmadd_v4sf */
    case 1238:  /* *fma_fmadd_df */
    case 1237:  /* *fma_fmadd_sf */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 
#line 650 "../../gcc/config/i386/i386.md"
(TARGET_FMA4);
        }
      else
        {
	  return 
#line 651 "../../gcc/config/i386/i386.md"
(TARGET_FMA);
        }

    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6))
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else
        {
	  return 1;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x30))
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else
        {
	  return 1;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f0))
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else if (((1 << which_alternative) & 0x1e00))
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else
        {
	  return 1;
        }

    case 744:  /* *fop_df_1_mixed */
    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else if (which_alternative == 2)
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else
        {
	  return 1;
        }

    case 1358:  /* *sse4_1_extractps */
    case 738:  /* *fop_df_comm_mixed */
    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else if (which_alternative == 1)
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else
        {
	  return 1;
        }

    case 1833:  /* pclmulqdq */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1727:  /* sse4_1_roundsd */
    case 1726:  /* sse4_1_roundss */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 1684:  /* sse4_1_packusdw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1674:  /* sse4_1_blendvpd */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1670:  /* sse4_1_blendpd */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1648:  /* ssse3_palignrti */
    case 1647:  /* avx2_palignrv2ti */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1556:  /* vec_interleave_lowv4si */
    case 1554:  /* vec_interleave_highv4si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1510:  /* *andnotv2di3 */
    case 1509:  /* *andnotv4di3 */
    case 1508:  /* *andnotv4si3 */
    case 1507:  /* *andnotv8si3 */
    case 1506:  /* *andnotv8hi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1503:  /* *andnotv32qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1446:  /* ashrv4si3 */
    case 1445:  /* ashrv8si3 */
    case 1444:  /* ashrv8hi3 */
    case 1443:  /* ashrv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1378:  /* vec_interleave_highv2di */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1346:  /* sse_movss */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1332:  /* vec_interleave_highv4sf */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 747:  /* *fop_df_1_sse */
    case 746:  /* *fop_sf_1_sse */
    case 740:  /* *fop_df_comm_sse */
    case 739:  /* *fop_sf_comm_sse */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }

    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 
#line 649 "../../gcc/config/i386/i386.md"
(TARGET_BMI2);
        }
      else
        {
	  return 1;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 
#line 649 "../../gcc/config/i386/i386.md"
(TARGET_BMI2);
        }
      else
        {
	  return 1;
        }

    case 147:  /* *truncxfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else
        {
	  return 1;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else
        {
	  return 1;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3f))
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else
        {
	  return 1;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1e0))
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else
        {
	  return 1;
        }

    case 101:  /* *pushdf */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else
        {
	  return 1;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x61e0))
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else if (((1 << which_alternative) & 0x1e00))
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else
        {
	  return 1;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_constrain_insn_cached (insn);
      if ((cached_isa = get_attr_isa (insn)) == ISA_SSE2)
        {
	  return 
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2);
        }
      else if (cached_isa == ISA_SSE2_NOAVX)
        {
	  return 
#line 642 "../../gcc/config/i386/i386.md"
(TARGET_SSE2 && !TARGET_AVX);
        }
      else if (cached_isa == ISA_SSE3)
        {
	  return 
#line 643 "../../gcc/config/i386/i386.md"
(TARGET_SSE3);
        }
      else if (cached_isa == ISA_SSE4)
        {
	  return 
#line 644 "../../gcc/config/i386/i386.md"
(TARGET_SSE4_1);
        }
      else if (cached_isa == ISA_SSE4_NOAVX)
        {
	  return 
#line 646 "../../gcc/config/i386/i386.md"
(TARGET_SSE4_1 && !TARGET_AVX);
        }
      else if (cached_isa == ISA_AVX)
        {
	  return 
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX);
        }
      else if (cached_isa == ISA_NOAVX)
        {
	  return 
#line 648 "../../gcc/config/i386/i386.md"
(!TARGET_AVX);
        }
      else if (cached_isa == ISA_BMI2)
        {
	  return 
#line 649 "../../gcc/config/i386/i386.md"
(TARGET_BMI2);
        }
      else if (cached_isa == ISA_FMA4)
        {
	  return 
#line 650 "../../gcc/config/i386/i386.md"
(TARGET_FMA4);
        }
      else if (cached_isa == ISA_FMA)
        {
	  return 
#line 651 "../../gcc/config/i386/i386.md"
(TARGET_FMA);
        }
      else
        {
	  return 1;
        }

    default:
      return 1;

    }
}

enum attr_fp_int_src
get_attr_fp_int_src (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 28:  /* *cmpfp_hi */
    case 29:  /* *cmpfp_si */
    case 30:  /* *cmpfp_hi_cc */
    case 31:  /* *cmpfp_si_cc */
    case 181:  /* *floathisf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 184:  /* *floathisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 186:  /* *floathixf2_i387 */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 195:  /* *floatsisf2_vector_mixed */
    case 196:  /* *floatsidf2_vector_mixed */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 200:  /* *floatdidf2_mixed_with_temp */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 211:  /* *floatsisf2_vector_sse */
    case 212:  /* *floatsidf2_vector_sse */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 217:  /* *floatsisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 230:  /* *floatdixf2_i387_with_temp */
    case 231:  /* *floatsisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 236:  /* *floatdixf2_i387 */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 750:  /* *fop_sf_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 756:  /* *fop_sf_3_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 2018:  /* loaddi_via_fpu */
      return FP_INT_SRC_TRUE;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return FP_INT_SRC_FALSE;

    }
}

enum attr_i7_domain
get_attr_i7_domain (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1583:  /* *vec_dupv4si */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1590:  /* vec_concatv2di */
    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 4)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 1584:  /* *vec_dupv2di */
    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1510:  /* *andnotv2di3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16))) || (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1509:  /* *andnotv4di3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16))) || (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1508:  /* *andnotv4si3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16))) || (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1507:  /* *andnotv8si3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16))) || (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1506:  /* *andnotv8hi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16))) || (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1505:  /* *andnotv16hi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16))) || (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1504:  /* *andnotv16qi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16))) || (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1503:  /* *andnotv32qi3 */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16))) || (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16)))) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x67))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x23))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 9)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else if (!((1 << which_alternative) & 0x600))
        {
	  if (!((1 << which_alternative) & 0xde))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else if (((1 << which_alternative) & 0x1f))
        {
	  if (!((1 << which_alternative) & 0x3))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  if (which_alternative != 0)
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 4)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  if (!((1 << which_alternative) & 0x3))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 10)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else if (!((1 << which_alternative) & 0x600))
        {
	  if (((1 << which_alternative) & 0x3b))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else if (which_alternative == 2)
	    {
	      return I7_DOMAIN_INT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 10)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else if (!((1 << which_alternative) & 0x600))
        {
	  if (((1 << which_alternative) & 0x39))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else if (((1 << which_alternative) & 0x6))
	    {
	      return I7_DOMAIN_INT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1344:  /* sse_storelps */
    case 1342:  /* sse_storehps */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1352:  /* *vec_concatv4sf */
    case 1345:  /* sse_loadlps */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1088:  /* *movv2df_internal */
      extract_constrain_insn_cached (insn);
      if (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1087:  /* *movv4df_internal */
      extract_constrain_insn_cached (insn);
      if ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4DFmode) == (V4SFmode))) && ((V4DFmode) == (V2DFmode))))) || (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1086:  /* *movv4sf_internal */
      extract_constrain_insn_cached (insn);
      if ((((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode))))))) && ((V4SFmode) == (V2DFmode))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode)))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8SFmode) == (V4SFmode))) && ((V8SFmode) == (V2DFmode))))) || (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1084:  /* *movv1ti_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode))))) || (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1083:  /* *movv2ti_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2TImode) == (V4SFmode))) && ((V2TImode) == (V2DFmode))))) || (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1082:  /* *movv2di_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode))))) || (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1081:  /* *movv4di_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4DImode) == (V4SFmode))) && ((V4DImode) == (V2DFmode))))) || (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1080:  /* *movv4si_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode))))) || (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1079:  /* *movv8si_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8SImode) == (V4SFmode))) && ((V8SImode) == (V2DFmode))))) || (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1078:  /* *movv8hi_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode))))) || (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1077:  /* *movv16hi_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16HImode) == (V4SFmode))) && ((V16HImode) == (V2DFmode))))) || (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1076:  /* *movv16qi_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode))))) || (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V32QImode) == (V4SFmode))) && ((V32QImode) == (V2DFmode))))) || (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x2e))
        {
	  if (which_alternative == 3)
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else if (!((1 << which_alternative) & 0xf))
	    {
	      return I7_DOMAIN_INT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else if (((1 << which_alternative) & 0x11))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 4)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else if (((1 << which_alternative) & 0xa))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else if (((1 << which_alternative) & 0x5))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xc0f))
        {
	  if (((1 << which_alternative) & 0xc0))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3f))
        {
	  if (((1 << which_alternative) & 0x300))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else if (((1 << which_alternative) & 0x3c))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x600f))
        {
	  if (((1 << which_alternative) & 0x600))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || ((mult_operator (operands[3], XFmode)) || (cached_type == TYPE_FDIV)))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || ((mult_operator (operands[3], DFmode)) || (cached_type == TYPE_FDIV)))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || ((mult_operator (operands[3], SFmode)) || (cached_type == TYPE_FDIV)))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) || ((mult_operator (operands[3], DFmode)) || (cached_type == TYPE_SSEDIV)))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) || ((mult_operator (operands[3], SFmode)) || (cached_type == TYPE_SSEDIV)))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) || ((cached_type == TYPE_FDIV) || ((cached_type == TYPE_SSEADD) || (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) || (cached_type == TYPE_SSEDIV))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) || ((cached_type == TYPE_FDIV) || ((cached_type == TYPE_SSEADD) || (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) || (cached_type == TYPE_SSEDIV))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 897:  /* *movsfcc_1_387 */
    case 896:  /* *movdfcc_1 */
    case 895:  /* *movdfcc_1_rex64 */
    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_FLOAT;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x1f))
        {
	  return I7_DOMAIN_SIMD;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xf))
        {
	  return I7_DOMAIN_SIMD;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else if (!((1 << which_alternative) & 0xce1f))
        {
	  if ((((which_alternative == 5) && (((! (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 6) && ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else if (((1 << which_alternative) & 0xce00))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  if (((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((((1 << which_alternative) & 0x220)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))))) || ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)))))) || ((((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((1 << which_alternative) & 0x660))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (((((1 << which_alternative) & 0x220)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((((1 << which_alternative) & 0x440)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))))) || ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || ((((1 << which_alternative) & 0x880)) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xee0))))))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else if (!((1 << which_alternative) & 0x7f))
        {
	  if ((((which_alternative == 7) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))))) || ((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || ((((which_alternative == 7) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 8) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || (((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || (((which_alternative == 9) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x1bff))))))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 738:  /* *fop_df_comm_mixed */
    case 737:  /* *fop_sf_comm_mixed */
    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if (((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  if (((((1 << which_alternative) & 0x5)) && (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) || ((which_alternative == 1) && ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xfc0))
        {
	  if (((((1 << which_alternative) & 0xc0)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else if (!((1 << which_alternative) & 0xcc))
	    {
	      return I7_DOMAIN_INT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else if (((1 << which_alternative) & 0x3c))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x1f))
        {
	  if (((1 << which_alternative) & 0xa00))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else if (((1 << which_alternative) & 0x1c))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1fe00))
        {
	  return I7_DOMAIN_SIMD;
        }
      else if (((1 << which_alternative) & 0x1f0))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else
        {
	  return I7_DOMAIN_SIMD;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  if ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FMOV) || ((cached_type == TYPE_FOP) || ((cached_type == TYPE_FSGN) || ((cached_type == TYPE_FMUL) || ((cached_type == TYPE_FDIV) || ((cached_type == TYPE_FPSPC) || ((cached_type == TYPE_FCMOV) || ((cached_type == TYPE_FCMP) || ((cached_type == TYPE_FXCH) || ((cached_type == TYPE_FISTP) || ((cached_type == TYPE_FISTTP) || (cached_type == TYPE_FRNDINT))))))))))))
        {
	  return I7_DOMAIN_FLOAT;
        }
      else if ((cached_type == TYPE_SSELOG) || ((cached_type == TYPE_SSELOG1) || ((cached_type == TYPE_SSEIADD) || ((cached_type == TYPE_SSEIADD1) || ((cached_type == TYPE_SSEISHFT) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_SSEIMUL) || ((cached_type == TYPE_SSE) || ((cached_type == TYPE_SSEMOV) || ((cached_type == TYPE_SSEADD) || ((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSECMP) || ((cached_type == TYPE_SSECOMI) || ((cached_type == TYPE_SSECVT) || ((cached_type == TYPE_SSECVT1) || ((cached_type == TYPE_SSEICVT) || ((cached_type == TYPE_SSEDIV) || ((cached_type == TYPE_SSEINS) || ((cached_type == TYPE_SSEMULADD) || (cached_type == TYPE_SSE4ARG))))))))))))))))))))
        {
	  if ((get_attr_mode (insn) == MODE_V4DF) || ((get_attr_mode (insn) == MODE_V8SF) || ((get_attr_mode (insn) == MODE_V2DF) || ((get_attr_mode (insn) == MODE_V4SF) || ((get_attr_mode (insn) == MODE_SF) || (get_attr_mode (insn) == MODE_DF))))))
	    {
	      return I7_DOMAIN_FLOAT;
	    }
	  else if (get_attr_mode (insn) == MODE_SI)
	    {
	      return I7_DOMAIN_INT;
	    }
	  else
	    {
	      return I7_DOMAIN_SIMD;
	    }
        }
      else if ((cached_type == TYPE_MMX) || ((cached_type == TYPE_MMXMOV) || ((cached_type == TYPE_MMXADD) || ((cached_type == TYPE_MMXMUL) || ((cached_type == TYPE_MMXCMP) || ((cached_type == TYPE_MMXCVT) || (cached_type == TYPE_MMXSHFT)))))))
        {
	  return I7_DOMAIN_SIMD;
        }
      else
        {
	  return I7_DOMAIN_INT;
        }

    case 2010:  /* mfence_sse2 */
    case 2009:  /* *sse_sfence */
    case 2008:  /* *sse2_lfence */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
    case 1923:  /* avx2_lshrvv2di */
    case 1922:  /* avx2_ashlvv2di */
    case 1921:  /* avx2_lshrvv4di */
    case 1920:  /* avx2_ashlvv4di */
    case 1919:  /* avx2_lshrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1917:  /* avx2_lshrvv8si */
    case 1916:  /* avx2_ashlvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1914:  /* avx2_ashrvv8si */
    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1894:  /* vec_set_hi_v32qi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1891:  /* vec_set_lo_v16hi */
    case 1889:  /* vec_set_hi_v8si */
    case 1887:  /* vec_set_lo_v8si */
    case 1885:  /* vec_set_hi_v4di */
    case 1883:  /* vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1845:  /* avx2_permv4df */
    case 1844:  /* avx2_permvarv8si */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1835:  /* avx_vzeroupper */
    case 1834:  /* *avx_vzeroall */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1810:  /* xop_maskcmpv2di3 */
    case 1809:  /* xop_maskcmpv4si3 */
    case 1808:  /* xop_maskcmpv8hi3 */
    case 1807:  /* xop_maskcmpv16qi3 */
    case 1798:  /* xop_shlv2di3 */
    case 1797:  /* xop_shlv4si3 */
    case 1796:  /* xop_shlv8hi3 */
    case 1795:  /* xop_shlv16qi3 */
    case 1794:  /* xop_shav2di3 */
    case 1793:  /* xop_shav4si3 */
    case 1792:  /* xop_shav8hi3 */
    case 1791:  /* xop_shav16qi3 */
    case 1790:  /* xop_vrotlv2di3 */
    case 1789:  /* xop_vrotlv4si3 */
    case 1788:  /* xop_vrotlv8hi3 */
    case 1787:  /* xop_vrotlv16qi3 */
    case 1786:  /* xop_rotrv2di3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1779:  /* xop_rotlv16qi3 */
    case 1778:  /* xop_pperm_pack_v8hi_v16qi */
    case 1777:  /* xop_pperm_pack_v4si_v8hi */
    case 1776:  /* xop_pperm_pack_v2di_v4si */
    case 1775:  /* xop_pperm */
    case 1774:  /* xop_phsubdq */
    case 1773:  /* xop_phsubwd */
    case 1772:  /* xop_phsubbw */
    case 1771:  /* xop_phaddudq */
    case 1770:  /* xop_phadduwq */
    case 1769:  /* xop_phadduwd */
    case 1768:  /* xop_phaddubq */
    case 1767:  /* xop_phaddubd */
    case 1766:  /* xop_phaddubw */
    case 1765:  /* xop_phadddq */
    case 1764:  /* xop_phaddwq */
    case 1763:  /* xop_phaddwd */
    case 1762:  /* xop_phaddbq */
    case 1761:  /* xop_phaddbd */
    case 1760:  /* xop_phaddbw */
    case 1759:  /* xop_pcmov_v2df */
    case 1758:  /* xop_pcmov_v4df256 */
    case 1757:  /* xop_pcmov_v4sf */
    case 1756:  /* xop_pcmov_v8sf256 */
    case 1755:  /* xop_pcmov_v2di */
    case 1754:  /* xop_pcmov_v4di256 */
    case 1753:  /* xop_pcmov_v4si */
    case 1752:  /* xop_pcmov_v8si256 */
    case 1751:  /* xop_pcmov_v8hi */
    case 1750:  /* xop_pcmov_v16hi256 */
    case 1749:  /* xop_pcmov_v16qi */
    case 1748:  /* xop_pcmov_v32qi256 */
    case 1747:  /* xop_pmadcswd */
    case 1746:  /* xop_pmadcsswd */
    case 1745:  /* xop_pmacswd */
    case 1744:  /* xop_pmacsswd */
    case 1743:  /* xop_pmacsdqh */
    case 1742:  /* xop_pmacsdql */
    case 1741:  /* xop_pmacssdqh */
    case 1740:  /* xop_pmacssdql */
    case 1739:  /* xop_pmacssdd */
    case 1738:  /* xop_pmacsdd */
    case 1737:  /* xop_pmacssww */
    case 1736:  /* xop_pmacsww */
    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1734:  /* sse4_2_pcmpistrm */
    case 1733:  /* sse4_2_pcmpistri */
    case 1732:  /* sse4_2_pcmpistr */
    case 1731:  /* sse4_2_pcmpestr_cconly */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1729:  /* sse4_2_pcmpestri */
    case 1728:  /* sse4_2_pcmpestr */
    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1691:  /* sse4_1_phminposuw */
    case 1690:  /* avx2_pblenddv4si */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 1684:  /* sse4_1_packusdw */
    case 1683:  /* avx2_packusdw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1680:  /* sse4_1_movntdqa */
    case 1679:  /* avx2_movntdqa */
    case 1666:  /* sse4a_insertq */
    case 1665:  /* sse4a_insertqi */
    case 1664:  /* sse4a_extrq */
    case 1663:  /* sse4a_extrqi */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1649:  /* ssse3_palignrdi */
    case 1648:  /* ssse3_palignrti */
    case 1647:  /* avx2_palignrv2ti */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1631:  /* ssse3_pmaddubsw */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1607:  /* sse2_clflush */
    case 1606:  /* sse_stmxcsr */
    case 1605:  /* sse_ldmxcsr */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1601:  /* avx2_pmovmskb */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1589:  /* *vec_concatv2di_rex64 */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1446:  /* ashrv4si3 */
    case 1445:  /* ashrv8si3 */
    case 1444:  /* ashrv8hi3 */
    case 1443:  /* ashrv16hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1436:  /* *avx2_pmaddwd */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1366:  /* vec_extract_hi_v8si */
    case 1362:  /* vec_extract_hi_v4di */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1312:  /* sse2_cvttsd2siq */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1298:  /* avx_cvtps2dq256 */
    case 1295:  /* sse_cvttss2siq */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1286:  /* sse_cvtps2pi */
    case 1112:  /* sse2_movntv2di */
    case 1111:  /* avx_movntv4di */
    case 1106:  /* sse2_movntidi */
    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
    case 1102:  /* sse2_storedqu */
    case 1101:  /* avx_storedqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1099:  /* avx_loaddqu256 */
    case 1089:  /* sse2_movq128 */
    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1064:  /* *mmx_concatv2si */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 994:  /* *mmx_concatv2sf */
    case 993:  /* *vec_dupv2sf */
    case 992:  /* mmx_pswapdv2sf2 */
    case 991:  /* mmx_floatv2si2 */
    case 990:  /* mmx_pi2fw */
    case 989:  /* mmx_pf2iw */
    case 988:  /* mmx_pf2id */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 972:  /* *mmx_mulv2sf3 */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
    case 969:  /* sse_movntq */
    case 932:  /* sse4_2_crc32di */
    case 920:  /* *prefetch_3dnow */
    case 919:  /* *prefetch_sse */
    case 899:  /* *xop_pcmov_df */
    case 898:  /* *xop_pcmov_sf */
    case 59:  /* *movoi_internal_avx */
      return I7_DOMAIN_SIMD;

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1935:  /* vcvtps2ph256 */
    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1932:  /* vcvtph2ps256 */
    case 1931:  /* *vcvtph2ps_load */
    case 1930:  /* vcvtph2ps */
    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1890:  /* vec_set_hi_v8sf */
    case 1888:  /* vec_set_lo_v8sf */
    case 1886:  /* vec_set_hi_v4df */
    case 1884:  /* vec_set_lo_v4df */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1849:  /* avx2_vec_dupv4df */
    case 1826:  /* xop_vpermil2v2df3 */
    case 1825:  /* xop_vpermil2v4df3 */
    case 1824:  /* xop_vpermil2v4sf3 */
    case 1823:  /* xop_vpermil2v8sf3 */
    case 1806:  /* *xop_vmfrczv2df2 */
    case 1805:  /* *xop_vmfrczv4sf2 */
    case 1804:  /* xop_frczv4df2 */
    case 1803:  /* xop_frczv8sf2 */
    case 1802:  /* xop_frczv2df2 */
    case 1801:  /* xop_frczv4sf2 */
    case 1800:  /* xop_frczdf2 */
    case 1799:  /* xop_frczsf2 */
    case 1727:  /* sse4_1_roundsd */
    case 1726:  /* sse4_1_roundss */
    case 1725:  /* sse4_1_roundpd */
    case 1724:  /* avx_roundpd256 */
    case 1723:  /* sse4_1_roundps */
    case 1722:  /* avx_roundps256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1678:  /* sse4_1_dppd */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1674:  /* sse4_1_blendvpd */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1670:  /* sse4_1_blendpd */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1600:  /* sse2_movmskpd */
    case 1599:  /* avx_movmskpd256 */
    case 1598:  /* sse_movmskps */
    case 1597:  /* avx_movmskps256 */
    case 1390:  /* vec_dupv2df */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1376:  /* avx_shufpd256_1 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1372:  /* avx_unpckhpd256 */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1363:  /* vec_extract_hi_v4df */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1349:  /* vec_dupv4sf */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 1346:  /* sse_movss */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1339:  /* avx_shufps256_1 */
    case 1338:  /* sse3_movsldup */
    case 1337:  /* avx_movsldup256 */
    case 1336:  /* sse3_movshdup */
    case 1335:  /* avx_movshdup256 */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1333:  /* avx_unpcklps256 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1331:  /* avx_unpckhps256 */
    case 1328:  /* sse2_cvtps2pd */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1326:  /* avx_cvtps2pd256 */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1315:  /* sse2_cvtdq2pd */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1313:  /* floatv4siv4df2 */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1302:  /* sse2_cvtpi2pd */
    case 1297:  /* floatv4siv4sf2 */
    case 1296:  /* floatv8siv8sf2 */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 1287:  /* sse_cvttps2pi */
    case 1285:  /* sse_cvtpi2ps */
    case 1284:  /* *fma4i_vmfnmsub_v2df */
    case 1283:  /* *fma4i_vmfnmsub_v4sf */
    case 1282:  /* *fma4i_vmfnmadd_v2df */
    case 1281:  /* *fma4i_vmfnmadd_v4sf */
    case 1280:  /* *fma4i_vmfmsub_v2df */
    case 1279:  /* *fma4i_vmfmsub_v4sf */
    case 1278:  /* *fma4i_vmfmadd_v2df */
    case 1277:  /* *fma4i_vmfmadd_v4sf */
    case 1276:  /* *fmai_fnmsub_v2df */
    case 1275:  /* *fmai_fnmsub_v4sf */
    case 1274:  /* *fmai_fnmadd_v2df */
    case 1273:  /* *fmai_fnmadd_v4sf */
    case 1272:  /* *fmai_fmsub_v2df */
    case 1271:  /* *fmai_fmsub_v4sf */
    case 1270:  /* *fmai_fmadd_v2df */
    case 1269:  /* *fmai_fmadd_v4sf */
    case 1268:  /* *fma_fmsubadd_v2df */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1266:  /* *fma_fmsubadd_v4sf */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1264:  /* *fma_fmaddsub_v2df */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1262:  /* *fma_fmaddsub_v4sf */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1258:  /* *fma_fnmsub_v2df */
    case 1257:  /* *fma_fnmsub_v4sf */
    case 1256:  /* *fma_fnmsub_df */
    case 1255:  /* *fma_fnmsub_sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1252:  /* *fma_fnmadd_v2df */
    case 1251:  /* *fma_fnmadd_v4sf */
    case 1250:  /* *fma_fnmadd_df */
    case 1249:  /* *fma_fnmadd_sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1246:  /* *fma_fmsub_v2df */
    case 1245:  /* *fma_fmsub_v4sf */
    case 1244:  /* *fma_fmsub_df */
    case 1243:  /* *fma_fmsub_sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1240:  /* *fma_fmadd_v2df */
    case 1239:  /* *fma_fmadd_v4sf */
    case 1238:  /* *fma_fmadd_df */
    case 1237:  /* *fma_fmadd_sf */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1150:  /* avx_rsqrtv8sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1147:  /* sse2_sqrtv2df2 */
    case 1146:  /* avx_sqrtv4df2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1144:  /* avx_sqrtv8sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 1142:  /* sse_rcpv4sf2 */
    case 1141:  /* avx_rcpv8sf2 */
    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
    case 1110:  /* sse2_movntv2df */
    case 1109:  /* avx_movntv4df */
    case 1108:  /* sse_movntv4sf */
    case 1107:  /* avx_movntv8sf */
    case 1098:  /* sse2_storeupd */
    case 1097:  /* avx_storeupd256 */
    case 1096:  /* sse_storeups */
    case 1095:  /* avx_storeups256 */
    case 1094:  /* sse2_loadupd */
    case 1093:  /* avx_loadupd256 */
    case 1092:  /* sse_loadups */
    case 1091:  /* avx_loadups256 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 894:  /* *movxfcc_1 */
    case 852:  /* movmsk_df */
    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 810:  /* sse4_1_rounddf2 */
    case 809:  /* sse4_1_roundsf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 780:  /* *sqrtdf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 761:  /* *fop_xf_comm_i387 */
    case 745:  /* *rcpsf2_sse */
    case 742:  /* *fop_df_comm_i387 */
    case 741:  /* *fop_sf_comm_i387 */
    case 740:  /* *fop_df_comm_sse */
    case 739:  /* *fop_sf_comm_sse */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 141:  /* *truncdfsf_fast_sse */
    case 140:  /* *truncdfsf_fast_mixed */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
    case 136:  /* *extendsfdf2_sse */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
    case 39:  /* *cmpfp_iu_387 */
    case 38:  /* *cmpfp_iu_sse */
    case 37:  /* *cmpfp_iu_mixed */
    case 36:  /* *cmpfp_i_i387 */
    case 35:  /* *cmpfp_i_sse */
    case 34:  /* *cmpfp_i_mixed */
      return I7_DOMAIN_FLOAT;

    default:
      return I7_DOMAIN_INT;

    }
}

enum attr_isa
get_attr_isa (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return ISA_SSE2;
        }
      else if (which_alternative == 2)
        {
	  return ISA_SSE2_NOAVX;
        }
      else if (which_alternative == 3)
        {
	  return ISA_AVX;
        }
      else if (((1 << which_alternative) & 0x30))
        {
	  return ISA_NOAVX;
        }
      else
        {
	  return ISA_AVX;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_SSE4_NOAVX;
        }
      else if (which_alternative == 1)
        {
	  return ISA_AVX;
        }
      else if (((1 << which_alternative) & 0x1c))
        {
	  return ISA_BASE;
        }
      else if (which_alternative == 5)
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 6)
        {
	  return ISA_AVX;
        }
      else if (which_alternative == 7)
        {
	  return ISA_NOAVX;
        }
      else
        {
	  return ISA_AVX;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_SSE2_NOAVX;
        }
      else if (which_alternative == 1)
        {
	  return ISA_AVX;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return ISA_NOAVX;
        }
      else
        {
	  return ISA_AVX;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_BASE;
        }
      else if (which_alternative == 1)
        {
	  return ISA_SSE2_NOAVX;
        }
      else if (which_alternative == 2)
        {
	  return ISA_AVX;
        }
      else if (which_alternative == 3)
        {
	  return ISA_SSE2;
        }
      else
        {
	  return ISA_NOAVX;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_SSE2;
        }
      else if (which_alternative == 1)
        {
	  return ISA_BASE;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return ISA_NOAVX;
        }
      else
        {
	  return ISA_AVX;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_SSE2_NOAVX;
        }
      else if (which_alternative == 1)
        {
	  return ISA_AVX;
        }
      else if (which_alternative == 2)
        {
	  return ISA_SSE3;
        }
      else if (which_alternative == 3)
        {
	  return ISA_SSE2_NOAVX;
        }
      else if (which_alternative == 4)
        {
	  return ISA_AVX;
        }
      else if (which_alternative == 5)
        {
	  return ISA_SSE2;
        }
      else
        {
	  return ISA_NOAVX;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 1)
        {
	  return ISA_AVX;
        }
      else if (which_alternative == 2)
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 3)
        {
	  return ISA_AVX;
        }
      else if (which_alternative == 4)
        {
	  return ISA_BASE;
        }
      else if (((1 << which_alternative) & 0x60))
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 7)
        {
	  return ISA_AVX;
        }
      else
        {
	  return ISA_BASE;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_BASE;
        }
      else if (which_alternative == 1)
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 2)
        {
	  return ISA_AVX;
        }
      else if (which_alternative == 3)
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 4)
        {
	  return ISA_AVX;
        }
      else if (((1 << which_alternative) & 0x60))
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 7)
        {
	  return ISA_AVX;
        }
      else
        {
	  return ISA_BASE;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_BASE;
        }
      else if (which_alternative == 1)
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 2)
        {
	  return ISA_AVX;
        }
      else
        {
	  return ISA_BASE;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_SSE4;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return ISA_SSE2;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 5)
        {
	  return ISA_AVX;
        }
      else if (which_alternative == 6)
        {
	  return ISA_SSE4_NOAVX;
        }
      else if (which_alternative == 7)
        {
	  return ISA_AVX;
        }
      else
        {
	  return ISA_BASE;
        }

    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return ISA_AVX;
        }
      else
        {
	  return ISA_NOAVX;
        }

    case 1268:  /* *fma_fmsubadd_v2df */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1266:  /* *fma_fmsubadd_v4sf */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1264:  /* *fma_fmaddsub_v2df */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1262:  /* *fma_fmaddsub_v4sf */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1258:  /* *fma_fnmsub_v2df */
    case 1257:  /* *fma_fnmsub_v4sf */
    case 1256:  /* *fma_fnmsub_df */
    case 1255:  /* *fma_fnmsub_sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1252:  /* *fma_fnmadd_v2df */
    case 1251:  /* *fma_fnmadd_v4sf */
    case 1250:  /* *fma_fnmadd_df */
    case 1249:  /* *fma_fnmadd_sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1246:  /* *fma_fmsub_v2df */
    case 1245:  /* *fma_fmsub_v4sf */
    case 1244:  /* *fma_fmsub_df */
    case 1243:  /* *fma_fmsub_sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1240:  /* *fma_fmadd_v2df */
    case 1239:  /* *fma_fmadd_v4sf */
    case 1238:  /* *fma_fmadd_df */
    case 1237:  /* *fma_fmadd_sf */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return ISA_FMA;
        }
      else
        {
	  return ISA_FMA4;
        }

    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return ISA_NOAVX;
        }
      else
        {
	  return ISA_AVX;
        }

    case 744:  /* *fop_df_1_mixed */
    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return ISA_BASE;
        }
      else if (which_alternative == 2)
        {
	  return ISA_NOAVX;
        }
      else
        {
	  return ISA_AVX;
        }

    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return ISA_BASE;
        }
      else
        {
	  return ISA_BMI2;
        }

    case 147:  /* *truncxfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return ISA_BASE;
        }
      else if (which_alternative == 2)
        {
	  return ISA_SSE2;
        }
      else
        {
	  return ISA_BASE;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_BASE;
        }
      else if (which_alternative == 1)
        {
	  return ISA_SSE2;
        }
      else
        {
	  return ISA_BASE;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3f))
        {
	  return ISA_BASE;
        }
      else
        {
	  return ISA_SSE2;
        }

    case 101:  /* *pushdf */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return ISA_BASE;
        }
      else
        {
	  return ISA_SSE2;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x61e0))
        {
	  return ISA_SSE2;
        }
      else if (((1 << which_alternative) & 0x1e00))
        {
	  return ISA_NOAVX;
        }
      else
        {
	  return ISA_BASE;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1e0))
        {
	  return ISA_SSE2;
        }
      else
        {
	  return ISA_BASE;
        }

    case 326:  /* *umulsidi3_1 */
    case 327:  /* *umulditi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 523:  /* *ashrdi3_1 */
    case 526:  /* *lshrsi3_1_zext */
    case 527:  /* *ashrsi3_1_zext */
    case 562:  /* *rotlsi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 564:  /* *rotldi3_1 */
    case 565:  /* *rotrdi3_1 */
    case 567:  /* *rotlsi3_1_zext */
    case 568:  /* *rotrsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_BASE;
        }
      else
        {
	  return ISA_BMI2;
        }

    case 963:  /* *movv8qi_internal */
    case 964:  /* *movv4hi_internal */
    case 965:  /* *movv2si_internal */
    case 966:  /* *movv1di_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f0))
        {
	  return ISA_SSE2;
        }
      else if (((1 << which_alternative) & 0x1e00))
        {
	  return ISA_NOAVX;
        }
      else
        {
	  return ISA_BASE;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x30))
        {
	  return ISA_SSE2;
        }
      else
        {
	  return ISA_BASE;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6))
        {
	  return ISA_SSE2;
        }
      else
        {
	  return ISA_BASE;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1387:  /* sse2_loadhpd */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
    case 1329:  /* sse_movhlps */
    case 1330:  /* sse_movlhps */
    case 1343:  /* sse_loadhps */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 1)
        {
	  return ISA_AVX;
        }
      else if (which_alternative == 2)
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 3)
        {
	  return ISA_AVX;
        }
      else
        {
	  return ISA_BASE;
        }

    case 1352:  /* *vec_concatv4sf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 1)
        {
	  return ISA_AVX;
        }
      else if (which_alternative == 2)
        {
	  return ISA_NOAVX;
        }
      else
        {
	  return ISA_AVX;
        }

    case 737:  /* *fop_sf_comm_mixed */
    case 738:  /* *fop_df_comm_mixed */
    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_BASE;
        }
      else if (which_alternative == 1)
        {
	  return ISA_NOAVX;
        }
      else
        {
	  return ISA_AVX;
        }

    case 1373:  /* *vec_interleave_highv2df */
    case 1375:  /* *vec_interleave_lowv2df */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 1)
        {
	  return ISA_AVX;
        }
      else if (which_alternative == 2)
        {
	  return ISA_SSE3;
        }
      else if (which_alternative == 3)
        {
	  return ISA_NOAVX;
        }
      else if (which_alternative == 4)
        {
	  return ISA_AVX;
        }
      else
        {
	  return ISA_BASE;
        }

    case 1390:  /* vec_dupv2df */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_NOAVX;
        }
      else
        {
	  return ISA_SSE3;
        }

    case 1583:  /* *vec_dupv4si */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_SSE2;
        }
      else if (which_alternative == 1)
        {
	  return ISA_AVX;
        }
      else
        {
	  return ISA_NOAVX;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_SSE2_NOAVX;
        }
      else if (which_alternative == 1)
        {
	  return ISA_AVX;
        }
      else if (which_alternative == 2)
        {
	  return ISA_SSE3;
        }
      else
        {
	  return ISA_NOAVX;
        }

    case 585:  /* setcc_sf_sse */
    case 586:  /* setcc_df_sse */
    case 739:  /* *fop_sf_comm_sse */
    case 740:  /* *fop_df_comm_sse */
    case 746:  /* *fop_sf_1_sse */
    case 747:  /* *fop_df_1_sse */
    case 900:  /* smaxsf3 */
    case 901:  /* sminsf3 */
    case 902:  /* smaxdf3 */
    case 903:  /* smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 905:  /* *ieee_smindf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 1117:  /* *addv8sf3 */
    case 1118:  /* *subv8sf3 */
    case 1119:  /* *addv4sf3 */
    case 1120:  /* *subv4sf3 */
    case 1121:  /* *addv4df3 */
    case 1122:  /* *subv4df3 */
    case 1123:  /* *addv2df3 */
    case 1124:  /* *subv2df3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1129:  /* *mulv8sf3 */
    case 1130:  /* *mulv4sf3 */
    case 1131:  /* *mulv4df3 */
    case 1132:  /* *mulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1135:  /* avx_divv8sf3 */
    case 1136:  /* sse_divv4sf3 */
    case 1137:  /* avx_divv4df3 */
    case 1138:  /* sse2_divv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
    case 1140:  /* sse2_vmdivv2df3 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1153:  /* *smaxv8sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1160:  /* *sminv2df3_finite */
    case 1161:  /* *smaxv8sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1164:  /* *sminv4sf3 */
    case 1165:  /* *smaxv4df3 */
    case 1166:  /* *sminv4df3 */
    case 1167:  /* *smaxv2df3 */
    case 1168:  /* *sminv2df3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1213:  /* avx_andnotv8sf3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1217:  /* *andv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1220:  /* *andv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1222:  /* *xorv4sf3 */
    case 1223:  /* *andv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1225:  /* *xorv4df3 */
    case 1226:  /* *andv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1228:  /* *xorv2df3 */
    case 1229:  /* *andnotsf3 */
    case 1230:  /* *andnotdf3 */
    case 1231:  /* *andsf3 */
    case 1232:  /* *iorsf3 */
    case 1233:  /* *xorsf3 */
    case 1234:  /* *anddf3 */
    case 1235:  /* *iordf3 */
    case 1236:  /* *xordf3 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1341:  /* sse_shufps_v4sf */
    case 1346:  /* sse_movss */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1356:  /* sse4_1_insertps */
    case 1378:  /* vec_interleave_highv2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1381:  /* sse2_shufpd_v2di */
    case 1382:  /* sse2_shufpd_v2df */
    case 1392:  /* *addv32qi3 */
    case 1393:  /* *subv32qi3 */
    case 1394:  /* *addv16qi3 */
    case 1395:  /* *subv16qi3 */
    case 1396:  /* *addv16hi3 */
    case 1397:  /* *subv16hi3 */
    case 1398:  /* *addv8hi3 */
    case 1399:  /* *subv8hi3 */
    case 1400:  /* *addv8si3 */
    case 1401:  /* *subv8si3 */
    case 1402:  /* *addv4si3 */
    case 1403:  /* *subv4si3 */
    case 1404:  /* *addv4di3 */
    case 1405:  /* *subv4di3 */
    case 1406:  /* *addv2di3 */
    case 1407:  /* *subv2di3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1427:  /* *mulv8hi3 */
    case 1428:  /* *smulv16hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1431:  /* *umulv8hi3_highpart */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1438:  /* *avx2_mulv8si3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1443:  /* ashrv16hi3 */
    case 1444:  /* ashrv8hi3 */
    case 1445:  /* ashrv8si3 */
    case 1446:  /* ashrv4si3 */
    case 1447:  /* ashlv16hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1450:  /* lshrv8hi3 */
    case 1451:  /* ashlv8si3 */
    case 1452:  /* lshrv8si3 */
    case 1453:  /* ashlv4si3 */
    case 1454:  /* lshrv4si3 */
    case 1455:  /* ashlv4di3 */
    case 1456:  /* lshrv4di3 */
    case 1457:  /* ashlv2di3 */
    case 1458:  /* lshrv2di3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1479:  /* *smaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1485:  /* *umaxv16qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1503:  /* *andnotv32qi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1506:  /* *andnotv8hi3 */
    case 1507:  /* *andnotv8si3 */
    case 1508:  /* *andnotv4si3 */
    case 1509:  /* *andnotv4di3 */
    case 1510:  /* *andnotv2di3 */
    case 1511:  /* *andv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1514:  /* *andv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1516:  /* *xorv16qi3 */
    case 1517:  /* *andv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1520:  /* *andv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1522:  /* *xorv8hi3 */
    case 1523:  /* *andv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1525:  /* *xorv8si3 */
    case 1526:  /* *andv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1528:  /* *xorv4si3 */
    case 1529:  /* *andv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1531:  /* *xorv4di3 */
    case 1532:  /* *andv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1534:  /* *xorv2di3 */
    case 1535:  /* *andnottf3 */
    case 1536:  /* *andtf3 */
    case 1537:  /* *iortf3 */
    case 1538:  /* *xortf3 */
    case 1539:  /* avx2_packsswb */
    case 1540:  /* sse2_packsswb */
    case 1541:  /* avx2_packssdw */
    case 1542:  /* sse2_packssdw */
    case 1543:  /* avx2_packuswb */
    case 1544:  /* sse2_packuswb */
    case 1546:  /* vec_interleave_highv16qi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1554:  /* vec_interleave_highv4si */
    case 1556:  /* vec_interleave_lowv4si */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1595:  /* avx2_psadbw */
    case 1596:  /* sse2_psadbw */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1647:  /* avx2_palignrv2ti */
    case 1648:  /* ssse3_palignrti */
    case 1667:  /* avx_blendps256 */
    case 1668:  /* sse4_1_blendps */
    case 1669:  /* avx_blendpd256 */
    case 1670:  /* sse4_1_blendpd */
    case 1671:  /* avx_blendvps256 */
    case 1672:  /* sse4_1_blendvps */
    case 1673:  /* avx_blendvpd256 */
    case 1674:  /* sse4_1_blendvpd */
    case 1675:  /* avx_dpps256 */
    case 1676:  /* sse4_1_dpps */
    case 1677:  /* avx_dppd256 */
    case 1678:  /* sse4_1_dppd */
    case 1681:  /* avx2_mpsadbw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1684:  /* sse4_1_packusdw */
    case 1685:  /* avx2_pblendvb */
    case 1686:  /* sse4_1_pblendvb */
    case 1687:  /* sse4_1_pblendw */
    case 1726:  /* sse4_1_roundss */
    case 1727:  /* sse4_1_roundsd */
    case 1827:  /* aesenc */
    case 1828:  /* aesenclast */
    case 1829:  /* aesdec */
    case 1830:  /* aesdeclast */
    case 1833:  /* pclmulqdq */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return ISA_NOAVX;
        }
      else
        {
	  return ISA_AVX;
        }

    case 1434:  /* *avx2_mulv4siv4di3 */
      return ISA_AVX;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return ISA_BASE;

    }
}

enum attr_i387_cw
get_attr_i387_cw (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 845:  /* frndintxf2_mask_pm */
    case 846:  /* frndintxf2_mask_pm_i387 */
      return I387_CW_MASK_PM;

    case 832:  /* frndintxf2_ceil */
    case 833:  /* frndintxf2_ceil_i387 */
    case 834:  /* *fisthi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 836:  /* *fistdi2_ceil_1 */
    case 837:  /* fistdi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 839:  /* fisthi2_ceil */
    case 840:  /* fistsi2_ceil */
    case 841:  /* fisthi2_ceil_with_temp */
    case 842:  /* fistsi2_ceil_with_temp */
      return I387_CW_CEIL;

    case 821:  /* frndintxf2_floor */
    case 822:  /* frndintxf2_floor_i387 */
    case 823:  /* *fisthi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 825:  /* *fistdi2_floor_1 */
    case 826:  /* fistdi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 828:  /* fisthi2_floor */
    case 829:  /* fistsi2_floor */
    case 830:  /* fisthi2_floor_with_temp */
    case 831:  /* fistsi2_floor_with_temp */
      return I387_CW_FLOOR;

    case 167:  /* *fix_trunchi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 170:  /* fix_truncdi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 172:  /* fix_trunchi_i387 */
    case 173:  /* fix_truncsi_i387 */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 843:  /* frndintxf2_trunc */
    case 844:  /* frndintxf2_trunc_i387 */
      return I387_CW_TRUNC;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return I387_CW_ANY;

    }
}

enum attr_imm_disp
get_attr_imm_disp (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return IMM_DISP_UNKNOWN;
        }
      else if ((which_alternative == 2) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 1387:  /* sse2_loadhpd */
    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 1383:  /* sse2_storehpd */
    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return IMM_DISP_UNKNOWN;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return IMM_DISP_UNKNOWN;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 967:  /* *movv2sf_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode)))) || (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode)))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode)))) || (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode)))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if ((((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))) || ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 493:  /* *ashlqi3_1_slp */
      extract_insn_cached (insn);
      if ((((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode)))) || (((cached_type = get_attr_type (insn)) == TYPE_ALU) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode)))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 492:  /* *ashlqi3_1 */
    case 491:  /* *ashlhi3_1 */
      extract_insn_cached (insn);
      if ((((cached_type = get_attr_type (insn)) == TYPE_ALU) || (cached_type == TYPE_ISHIFT)) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || (((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))) || (which_alternative == 2))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((((cached_type = get_attr_type (insn)) == TYPE_ALU) || ((cached_type == TYPE_ISHIFT) || (which_alternative == 2))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 273:  /* *adddi_5 */
    case 266:  /* *adddi_4 */
    case 264:  /* *adddi_3 */
    case 259:  /* *adddi_2 */
      extract_insn_cached (insn);
      if ((! (incdec_operand (operands[2], DImode))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 272:  /* *addsi_5 */
    case 269:  /* *addsi_4 */
    case 265:  /* *addsi_3_zext */
    case 263:  /* *addsi_3 */
    case 260:  /* *addsi_2_zext */
    case 258:  /* *addsi_2 */
      extract_insn_cached (insn);
      if ((! (incdec_operand (operands[2], SImode))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 271:  /* *addhi_5 */
    case 268:  /* *addhi_4 */
    case 262:  /* *addhi_3 */
    case 257:  /* *addhi_2 */
      extract_insn_cached (insn);
      if ((! (incdec_operand (operands[2], HImode))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 270:  /* *addqi_5 */
    case 267:  /* *addqi_4 */
    case 261:  /* *addqi_3 */
    case 256:  /* *addqi_2 */
      extract_insn_cached (insn);
      if ((! (incdec_operand (operands[2], QImode))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if ((! (incdec_operand (operands[1], QImode))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 5) && (! (incdec_operand (operands[2], QImode)))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], HImode)))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 2) && (! (incdec_operand (operands[2], SImode)))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 251:  /* *adddi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], DImode)))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], SImode)))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 323:  /* *mulqi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
      extract_insn_cached (insn);
      if ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode)))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return IMM_DISP_UNKNOWN;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 896:  /* *movdfcc_1 */
    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return IMM_DISP_UNKNOWN;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return IMM_DISP_UNKNOWN;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x18)) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return IMM_DISP_UNKNOWN;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x78)) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return IMM_DISP_UNKNOWN;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return IMM_DISP_UNKNOWN;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      extract_insn_cached (insn);
      if ((get_attr_type (insn) == TYPE_IMOV) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 63:  /* *movdi_internal */
    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return IMM_DISP_UNKNOWN;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 1358:  /* *sse4_1_extractps */
    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return IMM_DISP_UNKNOWN;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 649:  /* set_got_offset_rex64 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      extract_insn_cached (insn);
      if ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode)))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_OTHER) || (cached_type == TYPE_MULTI))
        {
	  return IMM_DISP_UNKNOWN;
        }
      else if ((((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_IMOV) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_ISHIFT1) || (cached_type == TYPE_ROTATE1)))))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode)))) || (((cached_type == TYPE_ALU) || ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFTX) || ((cached_type == TYPE_ROTATE) || ((cached_type == TYPE_ROTATEX) || ((cached_type == TYPE_IMUL) || (cached_type == TYPE_IDIV))))))) && ((memory_displacement_operand (operands[0], VOIDmode)) && (immediate_operand (operands[2], VOIDmode)))))
        {
	  return IMM_DISP_TRUE;
        }
      else
        {
	  return IMM_DISP_FALSE;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
      return IMM_DISP_UNKNOWN;

    default:
      return IMM_DISP_FALSE;

    }
}

int
get_attr_length_vex (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 1560:  /* sse4_1_pinsrq */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((V2DImode) == (V8HImode)))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1559:  /* sse4_1_pinsrd */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((V4SImode) == (V8HImode)))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1558:  /* sse2_pinsrw */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((V8HImode) == (V8HImode)))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1557:  /* sse4_1_pinsrb */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((V16QImode) == (V8HImode)))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x600))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x6c0))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xc))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1306:  /* sse2_cvtsi2sdq */
    case 1289:  /* sse_cvtsi2ssq */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 4;
        }

    case 1067:  /* *mmx_uavgv8qi3 */
      extract_constrain_insn_cached (insn);
      if ((
#line 1590 "../../gcc/config/i386/mmx.md"
(TARGET_SSE)) || (
#line 1591 "../../gcc/config/i386/mmx.md"
(TARGET_3DNOW_A)))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3f))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
    case 1383:  /* sse2_storehpd */
    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xc00))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3000)) && (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))
        {
	  return 4;
        }
      else
        {
	  if (!((1 << which_alternative) & 0x3))
	    {
	      return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
	    }
	  else
	    {
	      return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
	    }
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x6000))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 747:  /* *fop_df_1_sse */
    case 746:  /* *fop_sf_1_sse */
    case 744:  /* *fop_df_1_mixed */
    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (get_attr_unit (insn) == UNIT_SSE)
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 738:  /* *fop_df_comm_mixed */
    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 134:  /* extendqihi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_K6)))) || (which_alternative != 0))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 367:  /* *anddi_1 */
    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 110:  /* *movsf_internal */
    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x1f))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7f))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1385:  /* sse2_storelpd */
    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (get_attr_type (insn) == TYPE_IMOVX)
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5))))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xffc))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1fff0))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1589:  /* *vec_concatv2di_rex64 */
    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
    case 135:  /* *extendsfdf2_mixed */
    case 114:  /* *zero_extendsidi2_rex64 */
    case 63:  /* *movdi_internal */
    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 2010:  /* mfence_sse2 */
    case 2009:  /* *sse_sfence */
    case 2008:  /* *sse2_lfence */
    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1935:  /* vcvtps2ph256 */
    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1932:  /* vcvtph2ps256 */
    case 1931:  /* *vcvtph2ps_load */
    case 1930:  /* vcvtph2ps */
    case 1923:  /* avx2_lshrvv2di */
    case 1922:  /* avx2_ashlvv2di */
    case 1921:  /* avx2_lshrvv4di */
    case 1920:  /* avx2_ashlvv4di */
    case 1919:  /* avx2_lshrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1917:  /* avx2_lshrvv8si */
    case 1916:  /* avx2_ashlvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1914:  /* avx2_ashrvv8si */
    case 1849:  /* avx2_vec_dupv4df */
    case 1848:  /* avx2_permv2ti */
    case 1847:  /* avx2_permv4di_1 */
    case 1846:  /* avx2_permvarv8sf */
    case 1844:  /* avx2_permvarv8si */
    case 1835:  /* avx_vzeroupper */
    case 1834:  /* *avx_vzeroall */
    case 1786:  /* xop_rotrv2di3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1779:  /* xop_rotlv16qi3 */
    case 1666:  /* sse4a_insertq */
    case 1665:  /* sse4a_insertqi */
    case 1664:  /* sse4a_extrq */
    case 1663:  /* sse4a_extrqi */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1607:  /* sse2_clflush */
    case 1606:  /* sse_stmxcsr */
    case 1605:  /* sse_ldmxcsr */
    case 1602:  /* sse2_pmovmskb */
    case 1601:  /* avx2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1599:  /* avx_movmskpd256 */
    case 1598:  /* sse_movmskps */
    case 1597:  /* avx_movmskps256 */
    case 1596:  /* sse2_psadbw */
    case 1595:  /* avx2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1590:  /* vec_concatv2di */
    case 1588:  /* *vec_concatv4si */
    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1584:  /* *vec_dupv2di */
    case 1582:  /* *vec_extractv2di_1 */
    case 1576:  /* sse2_loadld */
    case 1575:  /* sse2_pshufhw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1565:  /* *sse2_pextrw_di */
    case 1564:  /* *sse2_pextrw_si */
    case 1556:  /* vec_interleave_lowv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1553:  /* avx2_interleave_highv8si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1544:  /* sse2_packuswb */
    case 1543:  /* avx2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1541:  /* avx2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1539:  /* avx2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1531:  /* *xorv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1529:  /* *andv4di3 */
    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1525:  /* *xorv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1523:  /* *andv8si3 */
    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1510:  /* *andnotv2di3 */
    case 1509:  /* *andnotv4di3 */
    case 1508:  /* *andnotv4si3 */
    case 1507:  /* *andnotv8si3 */
    case 1506:  /* *andnotv8hi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1503:  /* *andnotv32qi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1446:  /* ashrv4si3 */
    case 1445:  /* ashrv8si3 */
    case 1444:  /* ashrv8hi3 */
    case 1443:  /* ashrv16hi3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1436:  /* *avx2_pmaddwd */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
    case 1391:  /* *vec_concatv2df */
    case 1390:  /* vec_dupv2df */
    case 1389:  /* sse2_movsd */
    case 1386:  /* *vec_extractv2df_0_sse */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1377:  /* avx2_interleave_highv4di */
    case 1376:  /* avx_shufpd256_1 */
    case 1375:  /* *vec_interleave_lowv2df */
    case 1374:  /* *avx_unpcklpd256 */
    case 1373:  /* *vec_interleave_highv2df */
    case 1372:  /* avx_unpckhpd256 */
    case 1352:  /* *vec_concatv4sf */
    case 1351:  /* *vec_concatv2sf_sse */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 1346:  /* sse_movss */
    case 1345:  /* sse_loadlps */
    case 1344:  /* sse_storelps */
    case 1343:  /* sse_loadhps */
    case 1342:  /* sse_storehps */
    case 1341:  /* sse_shufps_v4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1339:  /* avx_shufps256_1 */
    case 1338:  /* sse3_movsldup */
    case 1337:  /* avx_movsldup256 */
    case 1336:  /* sse3_movshdup */
    case 1335:  /* avx_movshdup256 */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1333:  /* avx_unpcklps256 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1331:  /* avx_unpckhps256 */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
    case 1328:  /* sse2_cvtps2pd */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1326:  /* avx_cvtps2pd256 */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1315:  /* sse2_cvtdq2pd */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1313:  /* floatv4siv4df2 */
    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1305:  /* sse2_cvtsi2sd */
    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1302:  /* sse2_cvtpi2pd */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1298:  /* avx_cvtps2dq256 */
    case 1297:  /* floatv4siv4sf2 */
    case 1296:  /* floatv8siv8sf2 */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
    case 1288:  /* sse_cvtsi2ss */
    case 1287:  /* sse_cvttps2pi */
    case 1286:  /* sse_cvtps2pi */
    case 1285:  /* sse_cvtpi2ps */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1233:  /* *xorsf3 */
    case 1232:  /* *iorsf3 */
    case 1231:  /* *andsf3 */
    case 1230:  /* *andnotdf3 */
    case 1229:  /* *andnotsf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1225:  /* *xorv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1223:  /* *andv4df3 */
    case 1222:  /* *xorv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1213:  /* avx_andnotv8sf3 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1166:  /* *sminv4df3 */
    case 1165:  /* *smaxv4df3 */
    case 1164:  /* *sminv4sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1161:  /* *smaxv8sf3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1153:  /* *smaxv8sf3_finite */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1150:  /* avx_rsqrtv8sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1147:  /* sse2_sqrtv2df2 */
    case 1146:  /* avx_sqrtv4df2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1144:  /* avx_sqrtv8sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 1142:  /* sse_rcpv4sf2 */
    case 1141:  /* avx_rcpv8sf2 */
    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
    case 1138:  /* sse2_divv2df3 */
    case 1137:  /* avx_divv4df3 */
    case 1136:  /* sse_divv4sf3 */
    case 1135:  /* avx_divv8sf3 */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1132:  /* *mulv2df3 */
    case 1131:  /* *mulv4df3 */
    case 1130:  /* *mulv4sf3 */
    case 1129:  /* *mulv8sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1122:  /* *subv4df3 */
    case 1121:  /* *addv4df3 */
    case 1120:  /* *subv4sf3 */
    case 1119:  /* *addv4sf3 */
    case 1118:  /* *subv8sf3 */
    case 1117:  /* *addv8sf3 */
    case 1112:  /* sse2_movntv2di */
    case 1111:  /* avx_movntv4di */
    case 1110:  /* sse2_movntv2df */
    case 1109:  /* avx_movntv4df */
    case 1108:  /* sse_movntv4sf */
    case 1107:  /* avx_movntv8sf */
    case 1106:  /* sse2_movntidi */
    case 1105:  /* sse2_movntisi */
    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
    case 1102:  /* sse2_storedqu */
    case 1101:  /* avx_storedqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1099:  /* avx_loaddqu256 */
    case 1098:  /* sse2_storeupd */
    case 1097:  /* avx_storeupd256 */
    case 1096:  /* sse_storeups */
    case 1095:  /* avx_storeups256 */
    case 1094:  /* sse2_loadupd */
    case 1093:  /* avx_loadupd256 */
    case 1092:  /* sse_loadups */
    case 1091:  /* avx_loadups256 */
    case 1089:  /* sse2_movq128 */
    case 1088:  /* *movv2df_internal */
    case 1087:  /* *movv4df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1085:  /* *movv8sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1083:  /* *movv2ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1081:  /* *movv4di_internal */
    case 1080:  /* *movv4si_internal */
    case 1079:  /* *movv8si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1075:  /* *movv32qi_internal */
    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1064:  /* *mmx_concatv2si */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 994:  /* *mmx_concatv2sf */
    case 993:  /* *vec_dupv2sf */
    case 969:  /* sse_movntq */
    case 920:  /* *prefetch_3dnow */
    case 919:  /* *prefetch_sse */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 852:  /* movmsk_df */
    case 780:  /* *sqrtdf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 745:  /* *rcpsf2_sse */
    case 740:  /* *fop_df_comm_sse */
    case 739:  /* *fop_sf_comm_sse */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
    case 141:  /* *truncdfsf_fast_sse */
    case 136:  /* *extendsfdf2_sse */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 105:  /* *movtf_internal_sse2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
    case 61:  /* *movti_internal_sse */
    case 59:  /* *movoi_internal_avx */
    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      extract_constrain_insn_cached (insn);
      return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));

    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
        }
      else
        {
	  return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
        }

    case 1603:  /* *sse2_maskmovdqu */
    case 1604:  /* *sse2_maskmovdqu */
      extract_constrain_insn_cached (insn);
      return 
#line 8032 "../../gcc/config/i386/sse.md"
(3 + REX_SSE_REGNO_P (REGNO (operands[2])));

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_constrain_insn_cached (insn);
      if ((get_attr_prefix_0f (insn) == 1) && (get_attr_prefix_extra (insn) == 0))
        {
	  if (get_attr_prefix_vex_w (insn) == 1)
	    {
	      return 
#line 462 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, true));
	    }
	  else
	    {
	      return 
#line 463 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, true, false));
	    }
        }
      else
        {
	  if (get_attr_prefix_vex_w (insn) == 1)
	    {
	      return 
#line 465 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, true));
	    }
	  else
	    {
	      return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));
	    }
        }

    default:
      extract_constrain_insn_cached (insn);
      return 
#line 466 "../../gcc/config/i386/i386.md"
(ix86_attr_length_vex_default (insn, false, false));

    }
}

int
get_attr_length_address (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return 0;
        }
      else
        {
	  return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return 0;
        }
      else
        {
	  return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));
        }

    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
      extract_constrain_insn_cached (insn);
      if (constant_call_address_operand (operands[1], VOIDmode))
        {
	  return 0;
        }
      else
        {
	  return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));
        }

    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      extract_constrain_insn_cached (insn);
      if (constant_call_address_operand (operands[0], VOIDmode))
        {
	  return 0;
        }
      else
        {
	  return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 0;
        }
      else
        {
	  return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));
        }

    case 896:  /* *movdfcc_1 */
    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 0;
        }
      else
        {
	  return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 0;
        }
      else
        {
	  return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return 0;
        }
      else
        {
	  return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 0;
        }
      else
        {
	  return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 0;
        }
      else
        {
	  return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));
        }

    case 1579:  /* *sse2_storeq_rex64 */
    case 63:  /* *movdi_internal */
    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 0;
        }
      else
        {
	  return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));
        }

    case 1358:  /* *sse4_1_extractps */
    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 0;
        }
      else
        {
	  return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_STR) || ((cached_type == TYPE_OTHER) || ((cached_type == TYPE_MULTI) || ((cached_type == TYPE_FXCH) || (((cached_type == TYPE_CALL) && (constant_call_address_operand (operands[0], VOIDmode))) || ((cached_type == TYPE_CALLV) && (constant_call_address_operand (operands[1], VOIDmode))))))))
        {
	  return 0;
        }
      else
        {
	  return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));
        }

    case 67:  /* *movabsqi_1 */
    case 68:  /* *movabshi_1 */
    case 69:  /* *movabssi_1 */
    case 70:  /* *movabsdi_1 */
    case 71:  /* *movabsqi_2 */
    case 72:  /* *movabshi_2 */
    case 73:  /* *movabssi_2 */
    case 74:  /* *movabsdi_2 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 8;
        }
      else
        {
	  return 0;
        }

    case 649:  /* set_got_offset_rex64 */
      return 8;

    case 647:  /* set_got_rex64 */
    case 648:  /* set_rip_rex64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
      return 4;

    case 919:  /* *prefetch_sse */
    case 920:  /* *prefetch_3dnow */
      extract_constrain_insn_cached (insn);
      return 
#line 17757 "../../gcc/config/i386/i386.md"
(memory_address_length (operands[0], false));

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 2008:  /* *sse2_lfence */
    case 2009:  /* *sse_sfence */
    case 2010:  /* mfence_sse2 */
      return 0;

    default:
      extract_constrain_insn_cached (insn);
      return 
#line 384 "../../gcc/config/i386/i386.md"
(ix86_attr_length_address_default (insn));

    }
}

int
get_attr_length_immediate (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1864:  /* *avx_vperm_broadcast_v4sf */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return 1;
        }
      else
        {
	  if (((1 << which_alternative) & 0xf))
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
	    }
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 0;
        }
      else
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f))
        {
	  return 0;
        }
      else if (which_alternative == 5)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f))
        {
	  return 0;
        }
      else if (which_alternative == 5)
        {
	  return 1;
        }
      else
        {
	  if (which_alternative != 10)
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
	    }
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3f))
        {
	  return 0;
        }
      else
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return 0;
        }
      else
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3f))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0xc0))
        {
	  return 1;
        }
      else
        {
	  if (which_alternative != 9)
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
	    }
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
    case 1358:  /* *sse4_1_extractps */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 0;
        }
      else if (which_alternative == 2)
        {
	  return 1;
        }
      else
        {
	  if (((1 << which_alternative) & 0x3f))
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
	    }
        }

    case 1383:  /* sse2_storehpd */
    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f))
        {
	  return 0;
        }
      else
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }

    case 967:  /* *movv2sf_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 0;
        }
      else
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }

    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
      extract_constrain_insn_cached (insn);
      return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], DImode)))
        {
	  return 0;
        }
      else if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && (const128_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))
	    {
	      return 0;
	    }
	  else if ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
	  else if (const0_operand (operands[2], DImode))
	    {
	      return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
	    }
	  else
	    {
	      return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
	    }
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode)))
        {
	  return 0;
        }
      else if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && (const128_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))
	    {
	      return 0;
	    }
	  else if ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
	  else if (const0_operand (operands[2], SImode))
	    {
	      return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
	    }
	  else
	    {
	      return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
	    }
        }

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 756:  /* *fop_sf_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 749:  /* *fop_df_1_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_constrain_insn_cached (insn);
      if (get_attr_unit (insn) == UNIT_I387)
        {
	  return 0;
        }
      else
        {
	  return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
        }

    case 747:  /* *fop_df_1_sse */
    case 746:  /* *fop_sf_1_sse */
      extract_constrain_insn_cached (insn);
      if (get_attr_unit (insn) == UNIT_SSE)
        {
	  return 0;
        }
      else
        {
	  return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
        }

    case 744:  /* *fop_df_1_mixed */
    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_unit = get_attr_unit (insn)) == UNIT_I387) || (cached_unit == UNIT_SSE))
        {
	  return 0;
        }
      else
        {
	  return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
        }

    case 738:  /* *fop_df_comm_mixed */
    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 0;
        }
      else
        {
	  return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }

    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
      extract_insn_cached (insn);
      if (constant_call_address_operand (operands[1], VOIDmode))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      extract_insn_cached (insn);
      if (constant_call_address_operand (operands[0], VOIDmode))
        {
	  return 4;
        }
      else
        {
	  return 0;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && ((const1_operand (operands[2], VOIDmode)) && ((
#line 10300 "../../gcc/config/i386/i386.md"
(TARGET_SHIFT1)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))))
        {
	  return 0;
        }
      else
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
      extract_constrain_insn_cached (insn);
      if ((const1_operand (operands[1], VOIDmode)) && ((
#line 10300 "../../gcc/config/i386/i386.md"
(TARGET_SHIFT1)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))
        {
	  return 0;
        }
      else
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }

    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((const1_operand (operands[2], VOIDmode)) && ((
#line 10300 "../../gcc/config/i386/i386.md"
(TARGET_SHIFT1)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))
        {
	  return 0;
        }
      else
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))) || (((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode)))) && ((const1_operand (operands[2], VOIDmode)) && ((
#line 10300 "../../gcc/config/i386/i386.md"
(TARGET_SHIFT1)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))
        {
	  return 0;
        }
      else
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || ((cached_type == TYPE_ISHIFT) && ((const1_operand (operands[2], VOIDmode)) && ((
#line 10300 "../../gcc/config/i386/i386.md"
(TARGET_SHIFT1)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))
        {
	  return 0;
        }
      else
        {
	  if (cached_type == TYPE_ISHIFT)
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
	  else
	    {
	      return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
	    }
        }

    case 493:  /* *ashlqi3_1_slp */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || ((cached_type == TYPE_ISHIFT1) && ((const1_operand (operands[1], VOIDmode)) && ((
#line 10300 "../../gcc/config/i386/i386.md"
(TARGET_SHIFT1)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))
        {
	  return 0;
        }
      else
        {
	  if (cached_type == TYPE_ISHIFT1)
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
	  else
	    {
	      return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
	    }
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || ((cached_type == TYPE_ISHIFT) && ((const1_operand (operands[2], VOIDmode)) && ((
#line 10300 "../../gcc/config/i386/i386.md"
(TARGET_SHIFT1)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))
        {
	  return 0;
        }
      else
        {
	  if (which_alternative == 2)
	    {
	      return 0;
	    }
	  else if (cached_type == TYPE_ISHIFT)
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
	  else
	    {
	      return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
	    }
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || ((cached_type == TYPE_ISHIFT) && ((const1_operand (operands[2], VOIDmode)) && ((
#line 10300 "../../gcc/config/i386/i386.md"
(TARGET_SHIFT1)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))
        {
	  return 0;
        }
      else
        {
	  if (which_alternative == 1)
	    {
	      return 0;
	    }
	  else if (cached_type == TYPE_ISHIFT)
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
	  else
	    {
	      return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
	    }
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || (((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))) && ((const1_operand (operands[2], VOIDmode)) && ((
#line 10300 "../../gcc/config/i386/i386.md"
(TARGET_SHIFT1)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))
        {
	  return 0;
        }
      else
        {
	  if (which_alternative == 1)
	    {
	      return 0;
	    }
	  else if (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || (((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))) || (which_alternative == 2)))
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
	  else
	    {
	      return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
	    }
        }

    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || ((cached_type == TYPE_ISHIFT) && ((const1_operand (operands[2], VOIDmode)) && ((
#line 10300 "../../gcc/config/i386/i386.md"
(TARGET_SHIFT1)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))
        {
	  return 0;
        }
      else
        {
	  if (which_alternative == 1)
	    {
	      return 0;
	    }
	  else if ((cached_type == TYPE_ISHIFT) || (which_alternative == 2))
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
	  else
	    {
	      return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
	    }
        }

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }
      else
        {
	  return 0;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }
      else
        {
	  return 0;
        }

    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else
        {
	  return 0;
        }

    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (incdec_operand (operands[2], QImode))
        {
	  return 0;
        }
      else
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }

    case 273:  /* *adddi_5 */
    case 266:  /* *adddi_4 */
    case 264:  /* *adddi_3 */
    case 259:  /* *adddi_2 */
      extract_constrain_insn_cached (insn);
      if ((! (incdec_operand (operands[2], DImode))) && (const128_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  if (incdec_operand (operands[2], DImode))
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
        }

    case 272:  /* *addsi_5 */
    case 269:  /* *addsi_4 */
    case 265:  /* *addsi_3_zext */
    case 263:  /* *addsi_3 */
    case 260:  /* *addsi_2_zext */
    case 258:  /* *addsi_2 */
      extract_constrain_insn_cached (insn);
      if ((! (incdec_operand (operands[2], SImode))) && (const128_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  if (incdec_operand (operands[2], SImode))
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
        }

    case 271:  /* *addhi_5 */
    case 268:  /* *addhi_4 */
    case 262:  /* *addhi_3 */
    case 257:  /* *addhi_2 */
      extract_constrain_insn_cached (insn);
      if ((! (incdec_operand (operands[2], HImode))) && (const128_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  if (incdec_operand (operands[2], HImode))
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
        }

    case 270:  /* *addqi_5 */
    case 267:  /* *addqi_4 */
    case 261:  /* *addqi_3 */
    case 256:  /* *addqi_2 */
      extract_constrain_insn_cached (insn);
      if ((! (incdec_operand (operands[2], QImode))) && (const128_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  if (incdec_operand (operands[2], QImode))
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
        }

    case 255:  /* *addqi_1_slp */
      extract_constrain_insn_cached (insn);
      if (incdec_operand (operands[1], QImode))
        {
	  return 0;
        }
      else
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 5) && (! (incdec_operand (operands[2], QImode)))) && (const128_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  if (((which_alternative != 5) && (incdec_operand (operands[2], QImode))) || (which_alternative == 5))
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], HImode)))) && (const128_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  if (((which_alternative != 3) && (incdec_operand (operands[2], HImode))) || (which_alternative == 3))
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 2) && (! (incdec_operand (operands[2], SImode)))) && (const128_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  if (((which_alternative != 2) && (incdec_operand (operands[2], SImode))) || (which_alternative == 2))
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
        }

    case 251:  /* *adddi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], DImode)))) && (const128_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  if (((which_alternative != 3) && (incdec_operand (operands[2], DImode))) || (which_alternative == 3))
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
        }

    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (! (incdec_operand (operands[2], SImode)))) && (const128_operand (operands[2], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  if (((which_alternative != 3) && (incdec_operand (operands[2], SImode))) || (which_alternative == 3))
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
	    }
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 0;
        }
      else if (which_alternative == 0)
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }
      else
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x18))
        {
	  return 0;
        }
      else
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 6)
        {
	  return 8;
        }
      else
        {
	  if (!((1 << which_alternative) & 0x78))
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
	    }
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 0;
        }
      else
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      extract_constrain_insn_cached (insn);
      if ((cached_type = get_attr_type (insn)) == TYPE_IMOVX)
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }
      else if (cached_type == TYPE_IMOV)
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }
      else
        {
	  return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }
      else
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }

    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      extract_constrain_insn_cached (insn);
      return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));

    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (get_attr_type (insn) == TYPE_IMOVX)
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }
      else if (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2)))))))
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }
      else
        {
	  return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5))))
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }
      else if (get_attr_type (insn) == TYPE_IMOV)
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }
      else
        {
	  return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode))) || (((1 << which_alternative) & 0xffc)))
        {
	  return 0;
        }
      else
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && (! (pic_32bit_operand (operands[1], VOIDmode))))
        {
	  return 8;
        }
      else
        {
	  if (((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode))) || (((1 << which_alternative) & 0x1fff0)))
	    {
	      return 0;
	    }
	  else
	    {
	      return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
	    }
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 0;
        }
      else
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 379:  /* *andqi_2_slp */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 372:  /* *andqi_1_slp */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 289:  /* *subqi_1_slp */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 118:  /* zero_extendhisi2_and */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
      extract_constrain_insn_cached (insn);
      return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_INCDEC) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_ICMOV) || ((cached_type == TYPE_STR) || ((cached_type == TYPE_LEA) || ((cached_type == TYPE_OTHER) || ((cached_type == TYPE_MULTI) || ((cached_type == TYPE_IDIV) || ((cached_type == TYPE_LEAVE) || ((cached_type == TYPE_BITMANIP) || ((cached_type == TYPE_IMULX) || (((cached_unit = get_attr_unit (insn)) == UNIT_I387) || ((cached_unit == UNIT_SSE) || (cached_unit == UNIT_MMX))))))))))))))
        {
	  return 0;
        }
      else if ((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_IMOVX) || ((cached_type == TYPE_ISHIFT) || ((cached_type == TYPE_ISHIFTX) || ((cached_type == TYPE_ISHIFT1) || ((cached_type == TYPE_ROTATE) || ((cached_type == TYPE_ROTATEX) || ((cached_type == TYPE_ROTATE1) || ((cached_type == TYPE_IMUL) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_PUSH) || (cached_type == TYPE_POP))))))))))))))
        {
	  return 
#line 354 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, true));
        }
      else if ((cached_type == TYPE_IMOV) || (cached_type == TYPE_TEST))
        {
	  return 
#line 356 "../../gcc/config/i386/i386.md"
(ix86_attr_length_immediate_default (insn, false));
        }
      else if (cached_type == TYPE_CALL)
        {
	  if (constant_call_address_operand (operands[0], VOIDmode))
	    {
	      return 4;
	    }
	  else
	    {
	      return 0;
	    }
        }
      else if (cached_type == TYPE_CALLV)
        {
	  if (constant_call_address_operand (operands[1], VOIDmode))
	    {
	      return 4;
	    }
	  else
	    {
	      return 0;
	    }
        }
      else if (cached_type == TYPE_IBR)
        {
	  return 1;
        }
      else
        {
	  return 
#line 370 "../../gcc/config/i386/i386.md"
(/* Update immediate_length and other attributes! */
		      gcc_unreachable (),1);
        }

    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 0:  /* *cmpqi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 3:  /* *cmpdi_ccno_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 653:  /* split_stack_return */
      extract_insn_cached (insn);
      if (const0_operand (operands[0], SImode))
        {
	  return 0;
        }
      else
        {
	  return 2;
        }

    case 1023:  /* mmx_ashrv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1443:  /* ashrv16hi3 */
    case 1444:  /* ashrv8hi3 */
    case 1445:  /* ashrv8si3 */
    case 1446:  /* ashrv4si3 */
    case 1447:  /* ashlv16hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1450:  /* lshrv8hi3 */
    case 1451:  /* ashlv8si3 */
    case 1452:  /* lshrv8si3 */
    case 1453:  /* ashlv4si3 */
    case 1454:  /* lshrv4si3 */
    case 1455:  /* ashlv4di3 */
    case 1456:  /* lshrv4di3 */
    case 1457:  /* ashlv2di3 */
    case 1458:  /* lshrv2di3 */
      extract_insn_cached (insn);
      if (const_int_operand (operands[2], VOIDmode))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1349:  /* vec_dupv4sf */
    case 1583:  /* *vec_dupv4si */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 1;
        }
      else if (which_alternative == 1)
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 640:  /* simple_return_pop_internal */
    case 1663:  /* sse4a_extrqi */
    case 1665:  /* sse4a_insertqi */
      return 2;

    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
    case 57:  /* *movsi_or */
    case 58:  /* *movdi_or */
    case 361:  /* *testqi_ext_0 */
    case 380:  /* andqi_ext_0 */
    case 381:  /* *andqi_ext_0_cc */
    case 421:  /* *iorqi_ext_0 */
    case 422:  /* *xorqi_ext_0 */
    case 585:  /* setcc_sf_sse */
    case 586:  /* setcc_df_sse */
    case 1058:  /* *mmx_pinsrw */
    case 1059:  /* mmx_pextrw */
    case 1060:  /* mmx_pshufw_1 */
    case 1062:  /* *vec_dupv4hi */
    case 1193:  /* avx_cmpv8sf3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1339:  /* avx_shufps256_1 */
    case 1340:  /* sse_shufps_v4si */
    case 1341:  /* sse_shufps_v4sf */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1356:  /* sse4_1_insertps */
    case 1362:  /* vec_extract_hi_v4di */
    case 1363:  /* vec_extract_hi_v4df */
    case 1366:  /* vec_extract_hi_v8si */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1376:  /* avx_shufpd256_1 */
    case 1381:  /* sse2_shufpd_v2di */
    case 1382:  /* sse2_shufpd_v2df */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1557:  /* sse4_1_pinsrb */
    case 1558:  /* sse2_pinsrw */
    case 1559:  /* sse4_1_pinsrd */
    case 1560:  /* sse4_1_pinsrq */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1564:  /* *sse2_pextrw_si */
    case 1565:  /* *sse2_pextrw_di */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1567:  /* *sse4_1_pextrd */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1569:  /* *sse4_1_pextrq */
    case 1570:  /* avx2_pshufd_1 */
    case 1571:  /* sse2_pshufd_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1575:  /* sse2_pshufhw_1 */
    case 1647:  /* avx2_palignrv2ti */
    case 1648:  /* ssse3_palignrti */
    case 1649:  /* ssse3_palignrdi */
    case 1667:  /* avx_blendps256 */
    case 1668:  /* sse4_1_blendps */
    case 1669:  /* avx_blendpd256 */
    case 1670:  /* sse4_1_blendpd */
    case 1671:  /* avx_blendvps256 */
    case 1672:  /* sse4_1_blendvps */
    case 1673:  /* avx_blendvpd256 */
    case 1674:  /* sse4_1_blendvpd */
    case 1675:  /* avx_dpps256 */
    case 1676:  /* sse4_1_dpps */
    case 1677:  /* avx_dppd256 */
    case 1678:  /* sse4_1_dppd */
    case 1681:  /* avx2_mpsadbw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1687:  /* sse4_1_pblendw */
    case 1688:  /* *avx2_pblendw */
    case 1689:  /* avx2_pblenddv8si */
    case 1690:  /* avx2_pblenddv4si */
    case 1722:  /* avx_roundps256 */
    case 1723:  /* sse4_1_roundps */
    case 1724:  /* avx_roundpd256 */
    case 1725:  /* sse4_1_roundpd */
    case 1726:  /* sse4_1_roundss */
    case 1727:  /* sse4_1_roundsd */
    case 1728:  /* sse4_2_pcmpestr */
    case 1729:  /* sse4_2_pcmpestri */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1731:  /* sse4_2_pcmpestr_cconly */
    case 1732:  /* sse4_2_pcmpistr */
    case 1733:  /* sse4_2_pcmpistri */
    case 1734:  /* sse4_2_pcmpistrm */
    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1779:  /* xop_rotlv16qi3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1786:  /* xop_rotrv2di3 */
    case 1807:  /* xop_maskcmpv16qi3 */
    case 1808:  /* xop_maskcmpv8hi3 */
    case 1809:  /* xop_maskcmpv4si3 */
    case 1810:  /* xop_maskcmpv2di3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1822:  /* xop_pcom_tfv2di3 */
    case 1823:  /* xop_vpermil2v8sf3 */
    case 1824:  /* xop_vpermil2v4sf3 */
    case 1825:  /* xop_vpermil2v4df3 */
    case 1826:  /* xop_vpermil2v2df3 */
    case 1832:  /* aeskeygenassist */
    case 1833:  /* pclmulqdq */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1869:  /* *avx_vpermilpv4df */
    case 1870:  /* *avx_vpermilpv2df */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1883:  /* vec_set_lo_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1886:  /* vec_set_hi_v4df */
    case 1887:  /* vec_set_lo_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1890:  /* vec_set_hi_v8sf */
    case 1891:  /* vec_set_lo_v16hi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1894:  /* vec_set_hi_v32qi */
      return 1;

    default:
      return 0;

    }
}

enum attr_movu
get_attr_movu (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 1091:  /* avx_loadups256 */
    case 1092:  /* sse_loadups */
    case 1093:  /* avx_loadupd256 */
    case 1094:  /* sse2_loadupd */
    case 1095:  /* avx_storeups256 */
    case 1096:  /* sse_storeups */
    case 1097:  /* avx_storeupd256 */
    case 1098:  /* sse2_storeupd */
    case 1099:  /* avx_loaddqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1101:  /* avx_storedqu256 */
    case 1102:  /* sse2_storedqu */
    case 1103:  /* avx_lddqu256 */
    case 1104:  /* sse3_lddqu */
      return MOVU_1;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return MOVU_0;

    }
}

enum attr_memory
get_attr_memory (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((((1 << which_alternative) & 0x63)) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if (((((1 << which_alternative) & 0x7)) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1582:  /* *vec_extractv2di_1 */
    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
	    {
	      return MEMORY_BOTH;
	    }
	  else if (memory_operand (operands[0], VOIDmode))
	    {
	      return MEMORY_STORE;
	    }
	  else if (memory_operand (operands[1], VOIDmode))
	    {
	      return MEMORY_LOAD;
	    }
	  else
	    {
	      return MEMORY_NONE;
	    }
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return MEMORY_NONE;
        }
      else
        {
	  if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
	    {
	      return MEMORY_BOTH;
	    }
	  else if (memory_operand (operands[0], VOIDmode))
	    {
	      return MEMORY_STORE;
	    }
	  else if (memory_operand (operands[1], VOIDmode))
	    {
	      return MEMORY_LOAD;
	    }
	  else
	    {
	      return MEMORY_NONE;
	    }
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MEMORY_UNKNOWN;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1389:  /* sse2_movsd */
    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((which_alternative == 5) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1590:  /* vec_concatv2di */
    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((((1 << which_alternative) & 0xc)) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if (((((1 << which_alternative) & 0x6)) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return MEMORY_UNKNOWN;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || (memory_operand (operands[2], VOIDmode)))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((((1 << which_alternative) & 0xc1)) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((which_alternative == 0) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((((1 << which_alternative) & 0xf)) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 1) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (((((1 << which_alternative) & 0xe)) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc00))
        {
	  return MEMORY_UNKNOWN;
        }
      else if (((which_alternative == 6) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 9) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6000))
        {
	  return MEMORY_UNKNOWN;
        }
      else if (((((1 << which_alternative) & 0x240)) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 8) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], DImode))))
        {
	  return MEMORY_NONE;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || (! (const0_operand (operands[2], DImode)))) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (! (const0_operand (operands[2], SImode))))
        {
	  return MEMORY_NONE;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || (! (const0_operand (operands[2], SImode)))) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return MEMORY_UNKNOWN;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MEMORY_NONE;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((memory_operand (operands[2], VOIDmode)) || (memory_operand (operands[3], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1826:  /* xop_vpermil2v2df3 */
    case 1825:  /* xop_vpermil2v4df3 */
    case 1824:  /* xop_vpermil2v4sf3 */
    case 1823:  /* xop_vpermil2v8sf3 */
    case 1810:  /* xop_maskcmpv2di3 */
    case 1809:  /* xop_maskcmpv4si3 */
    case 1808:  /* xop_maskcmpv8hi3 */
    case 1807:  /* xop_maskcmpv16qi3 */
    case 1778:  /* xop_pperm_pack_v8hi_v16qi */
    case 1777:  /* xop_pperm_pack_v4si_v8hi */
    case 1776:  /* xop_pperm_pack_v2di_v4si */
    case 1775:  /* xop_pperm */
    case 1759:  /* xop_pcmov_v2df */
    case 1758:  /* xop_pcmov_v4df256 */
    case 1757:  /* xop_pcmov_v4sf */
    case 1756:  /* xop_pcmov_v8sf256 */
    case 1755:  /* xop_pcmov_v2di */
    case 1754:  /* xop_pcmov_v4di256 */
    case 1753:  /* xop_pcmov_v4si */
    case 1752:  /* xop_pcmov_v8si256 */
    case 1751:  /* xop_pcmov_v8hi */
    case 1750:  /* xop_pcmov_v16hi256 */
    case 1749:  /* xop_pcmov_v16qi */
    case 1748:  /* xop_pcmov_v32qi256 */
    case 1747:  /* xop_pmadcswd */
    case 1746:  /* xop_pmadcsswd */
    case 1745:  /* xop_pmacswd */
    case 1744:  /* xop_pmacsswd */
    case 1743:  /* xop_pmacsdqh */
    case 1742:  /* xop_pmacsdql */
    case 1741:  /* xop_pmacssdqh */
    case 1740:  /* xop_pmacssdql */
    case 1739:  /* xop_pmacssdd */
    case 1738:  /* xop_pmacsdd */
    case 1737:  /* xop_pmacssww */
    case 1736:  /* xop_pmacsww */
    case 1284:  /* *fma4i_vmfnmsub_v2df */
    case 1283:  /* *fma4i_vmfnmsub_v4sf */
    case 1282:  /* *fma4i_vmfnmadd_v2df */
    case 1281:  /* *fma4i_vmfnmadd_v4sf */
    case 1280:  /* *fma4i_vmfmsub_v2df */
    case 1279:  /* *fma4i_vmfmsub_v4sf */
    case 1278:  /* *fma4i_vmfmadd_v2df */
    case 1277:  /* *fma4i_vmfmadd_v4sf */
    case 1276:  /* *fmai_fnmsub_v2df */
    case 1275:  /* *fmai_fnmsub_v4sf */
    case 1274:  /* *fmai_fnmadd_v2df */
    case 1273:  /* *fmai_fnmadd_v4sf */
    case 1272:  /* *fmai_fmsub_v2df */
    case 1271:  /* *fmai_fmsub_v4sf */
    case 1270:  /* *fmai_fmadd_v2df */
    case 1269:  /* *fmai_fmadd_v4sf */
    case 1268:  /* *fma_fmsubadd_v2df */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1266:  /* *fma_fmsubadd_v4sf */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1264:  /* *fma_fmaddsub_v2df */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1262:  /* *fma_fmaddsub_v4sf */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1258:  /* *fma_fnmsub_v2df */
    case 1257:  /* *fma_fnmsub_v4sf */
    case 1256:  /* *fma_fnmsub_df */
    case 1255:  /* *fma_fnmsub_sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1252:  /* *fma_fnmadd_v2df */
    case 1251:  /* *fma_fnmadd_v4sf */
    case 1250:  /* *fma_fnmadd_df */
    case 1249:  /* *fma_fnmadd_sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1246:  /* *fma_fmsub_v2df */
    case 1245:  /* *fma_fmsub_v4sf */
    case 1244:  /* *fma_fmsub_df */
    case 1243:  /* *fma_fmsub_sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1240:  /* *fma_fmadd_v2df */
    case 1239:  /* *fma_fmadd_v4sf */
    case 1238:  /* *fma_fmadd_df */
    case 1237:  /* *fma_fmadd_sf */
    case 899:  /* *xop_pcmov_df */
    case 898:  /* *xop_pcmov_sf */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
      extract_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((memory_operand (operands[2], VOIDmode)) || (memory_operand (operands[3], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
      extract_insn_cached (insn);
      if (constant_call_address_operand (operands[1], VOIDmode))
        {
	  return MEMORY_NONE;
        }
      else
        {
	  return MEMORY_LOAD;
        }

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      extract_insn_cached (insn);
      if (constant_call_address_operand (operands[0], VOIDmode))
        {
	  return MEMORY_NONE;
        }
      else
        {
	  return MEMORY_LOAD;
        }

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      extract_insn_cached (insn);
      if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
      extract_insn_cached (insn);
      if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((which_alternative == 1) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 493:  /* *ashlqi3_1_slp */
      extract_insn_cached (insn);
      if ((((cached_type = get_attr_type (insn)) == TYPE_ISHIFT1) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((! (cached_type == TYPE_ISHIFT1)) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 491:  /* *ashlhi3_1 */
    case 490:  /* *ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return MEMORY_NONE;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || (memory_operand (operands[2], VOIDmode)))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1588:  /* *vec_concatv4si */
    case 1345:  /* sse_loadlps */
    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((((1 << which_alternative) & 0x3)) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1391:  /* *vec_concatv2df */
    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((((1 << which_alternative) & 0x7)) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return MEMORY_NONE;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || (memory_operand (operands[2], VOIDmode)))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 492:  /* *ashlqi3_1 */
    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return MEMORY_NONE;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || (memory_operand (operands[2], VOIDmode)))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 253:  /* *addhi_1 */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return MEMORY_NONE;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || (memory_operand (operands[2], VOIDmode)))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return MEMORY_UNKNOWN;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return MEMORY_UNKNOWN;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return MEMORY_UNKNOWN;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return MEMORY_UNKNOWN;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 996:  /* *vec_extractv2sf_1 */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 1) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return MEMORY_UNKNOWN;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 5) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x18))
        {
	  return MEMORY_UNKNOWN;
        }
      else if (((((1 << which_alternative) & 0x220)) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 7) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return MEMORY_UNKNOWN;
        }
      else if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 2) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return MEMORY_UNKNOWN;
        }
      else if (((which_alternative == 2) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return MEMORY_UNKNOWN;
        }
      else
        {
	  if (memory_operand (operands[1], VOIDmode))
	    {
	      return MEMORY_BOTH;
	    }
	  else
	    {
	      return MEMORY_STORE;
	    }
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      extract_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || (((! ((cached_type = get_attr_type (insn)) == TYPE_IMOV)) && (! (cached_type == TYPE_IMOVX))) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 2007:  /* *avx2_gatherdiv8sf_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1949:  /* *avx2_gathersiv8si */
    case 1948:  /* *avx2_gathersiv8si */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1943:  /* *avx2_gathersiv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1941:  /* *avx2_gathersiv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1935:  /* vcvtps2ph256 */
    case 1934:  /* *vcvtps2ph_store */
    case 1933:  /* *vcvtps2ph */
    case 1932:  /* vcvtph2ps256 */
    case 1931:  /* *vcvtph2ps_load */
    case 1930:  /* vcvtph2ps */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1853:  /* vec_dupv4df */
    case 1852:  /* vec_dupv4di */
    case 1851:  /* vec_dupv8sf */
    case 1850:  /* vec_dupv8si */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1806:  /* *xop_vmfrczv2df2 */
    case 1805:  /* *xop_vmfrczv4sf2 */
    case 1804:  /* xop_frczv4df2 */
    case 1803:  /* xop_frczv8sf2 */
    case 1802:  /* xop_frczv2df2 */
    case 1801:  /* xop_frczv4sf2 */
    case 1800:  /* xop_frczdf2 */
    case 1799:  /* xop_frczsf2 */
    case 1774:  /* xop_phsubdq */
    case 1773:  /* xop_phsubwd */
    case 1772:  /* xop_phsubbw */
    case 1771:  /* xop_phaddudq */
    case 1770:  /* xop_phadduwq */
    case 1769:  /* xop_phadduwd */
    case 1768:  /* xop_phaddubq */
    case 1767:  /* xop_phaddubd */
    case 1766:  /* xop_phaddubw */
    case 1765:  /* xop_phadddq */
    case 1764:  /* xop_phaddwq */
    case 1763:  /* xop_phaddwd */
    case 1762:  /* xop_phaddbq */
    case 1761:  /* xop_phaddbd */
    case 1760:  /* xop_phaddbw */
    case 1727:  /* sse4_1_roundsd */
    case 1726:  /* sse4_1_roundss */
    case 1725:  /* sse4_1_roundpd */
    case 1724:  /* avx_roundpd256 */
    case 1723:  /* sse4_1_roundps */
    case 1722:  /* avx_roundps256 */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1690:  /* avx2_pblenddv4si */
    case 1689:  /* avx2_pblenddv8si */
    case 1688:  /* *avx2_pblendw */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1685:  /* avx2_pblendvb */
    case 1680:  /* sse4_1_movntdqa */
    case 1679:  /* avx2_movntdqa */
    case 1674:  /* sse4_1_blendvpd */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1670:  /* sse4_1_blendpd */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1664:  /* sse4a_extrq */
    case 1663:  /* sse4a_extrqi */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1604:  /* *sse2_maskmovdqu */
    case 1603:  /* *sse2_maskmovdqu */
    case 1602:  /* sse2_pmovmskb */
    case 1601:  /* avx2_pmovmskb */
    case 1600:  /* sse2_movmskpd */
    case 1599:  /* avx_movmskpd256 */
    case 1598:  /* sse_movmskps */
    case 1597:  /* avx_movmskps256 */
    case 1576:  /* sse2_loadld */
    case 1386:  /* *vec_extractv2df_0_sse */
    case 1385:  /* sse2_storelpd */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1352:  /* *vec_concatv4sf */
    case 1346:  /* sse_movss */
    case 1344:  /* sse_storelps */
    case 1343:  /* sse_loadhps */
    case 1342:  /* sse_storehps */
    case 1338:  /* sse3_movsldup */
    case 1337:  /* avx_movsldup256 */
    case 1336:  /* sse3_movshdup */
    case 1335:  /* avx_movshdup256 */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
    case 1328:  /* sse2_cvtps2pd */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1326:  /* avx_cvtps2pd256 */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1315:  /* sse2_cvtdq2pd */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1313:  /* floatv4siv4df2 */
    case 1312:  /* sse2_cvttsd2siq */
    case 1311:  /* sse2_cvttsd2si */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1304:  /* sse2_cvttpd2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1302:  /* sse2_cvtpi2pd */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1298:  /* avx_cvtps2dq256 */
    case 1297:  /* floatv4siv4sf2 */
    case 1296:  /* floatv8siv8sf2 */
    case 1295:  /* sse_cvttss2siq */
    case 1294:  /* sse_cvttss2si */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1291:  /* sse_cvtss2si_2 */
    case 1290:  /* sse_cvtss2si */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 1287:  /* sse_cvttps2pi */
    case 1286:  /* sse_cvtps2pi */
    case 1285:  /* sse_cvtpi2ps */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1150:  /* avx_rsqrtv8sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1147:  /* sse2_sqrtv2df2 */
    case 1146:  /* avx_sqrtv4df2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1144:  /* avx_sqrtv8sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 1142:  /* sse_rcpv4sf2 */
    case 1141:  /* avx_rcpv8sf2 */
    case 1112:  /* sse2_movntv2di */
    case 1111:  /* avx_movntv4di */
    case 1110:  /* sse2_movntv2df */
    case 1109:  /* avx_movntv4df */
    case 1108:  /* sse_movntv4sf */
    case 1107:  /* avx_movntv8sf */
    case 1106:  /* sse2_movntidi */
    case 1105:  /* sse2_movntisi */
    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
    case 1102:  /* sse2_storedqu */
    case 1101:  /* avx_storedqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1099:  /* avx_loaddqu256 */
    case 1098:  /* sse2_storeupd */
    case 1097:  /* avx_storeupd256 */
    case 1096:  /* sse_storeups */
    case 1095:  /* avx_storeups256 */
    case 1094:  /* sse2_loadupd */
    case 1093:  /* avx_loadupd256 */
    case 1092:  /* sse_loadups */
    case 1091:  /* avx_loadups256 */
    case 1089:  /* sse2_movq128 */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1064:  /* *mmx_concatv2si */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 994:  /* *mmx_concatv2sf */
    case 993:  /* *vec_dupv2sf */
    case 992:  /* mmx_pswapdv2sf2 */
    case 991:  /* mmx_floatv2si2 */
    case 990:  /* mmx_pi2fw */
    case 989:  /* mmx_pf2iw */
    case 988:  /* mmx_pf2id */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 969:  /* sse_movntq */
    case 852:  /* movmsk_df */
    case 810:  /* sse4_1_rounddf2 */
    case 809:  /* sse4_1_roundsf2 */
    case 780:  /* *sqrtdf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 745:  /* *rcpsf2_sse */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 649:  /* set_got_offset_rex64 */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 141:  /* *truncdfsf_fast_sse */
    case 140:  /* *truncdfsf_fast_mixed */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
    case 136:  /* *extendsfdf2_sse */
    case 135:  /* *extendsfdf2_mixed */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 114:  /* *zero_extendsidi2_rex64 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
      extract_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || (((((which_alternative == 5) && (! (aligned_operand (operands[1], QImode)))) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH))))) && ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH))))) || ((((1 << which_alternative) & 0x28)) || ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (which_alternative == 2))))))) && (! (get_attr_type (insn) == TYPE_IMOVX))) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || (((! (get_attr_type (insn) == TYPE_IMOV)) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (((which_alternative == 0) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH))))) || ((((1 << which_alternative) & 0x6)) && (aligned_operand (operands[1], HImode))))) || ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (!((1 << which_alternative) & 0x5))))) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xffc)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_NONE;
        }
      else if (((which_alternative == 6) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || (((pic_32bit_operand (operands[1], VOIDmode)) && (!((1 << which_alternative) & 0xffc))) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MEMORY_UNKNOWN;
        }
      else if (((((1 << which_alternative) & 0x220)) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1fff0)) && (pic_32bit_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_NONE;
        }
      else if (((which_alternative == 9) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || (((pic_32bit_operand (operands[1], VOIDmode)) && (!((1 << which_alternative) & 0x1fff0))) && (memory_operand (operands[2], VOIDmode))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MEMORY_UNKNOWN;
        }
      else if (((which_alternative == 2) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1088:  /* *movv2df_internal */
    case 1087:  /* *movv4df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1085:  /* *movv8sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1083:  /* *movv2ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1081:  /* *movv4di_internal */
    case 1080:  /* *movv4si_internal */
    case 1079:  /* *movv8si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1075:  /* *movv32qi_internal */
    case 61:  /* *movti_internal_sse */
    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1910:  /* avx2_maskstoreq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1906:  /* avx_maskstorepd256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1904:  /* avx_maskstorepd */
    case 1903:  /* avx_maskstoreps */
    case 1902:  /* avx2_maskloadq256 */
    case 1901:  /* avx2_maskloadd256 */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1898:  /* avx_maskloadpd256 */
    case 1897:  /* avx_maskloadps256 */
    case 1896:  /* avx_maskloadpd */
    case 1895:  /* avx_maskloadps */
    case 1849:  /* avx2_vec_dupv4df */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1691:  /* sse4_1_phminposuw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1681:  /* avx2_mpsadbw */
    case 1658:  /* absv2si2 */
    case 1657:  /* absv4hi2 */
    case 1656:  /* absv8qi2 */
    case 1655:  /* absv4si2 */
    case 1654:  /* absv8si2 */
    case 1653:  /* absv8hi2 */
    case 1652:  /* absv16hi2 */
    case 1651:  /* absv16qi2 */
    case 1650:  /* absv32qi2 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1571:  /* sse2_pshufd_1 */
    case 1570:  /* avx2_pshufd_1 */
    case 1390:  /* vec_dupv2df */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1347:  /* avx2_vec_dupv8sf */
    case 932:  /* sse4_2_crc32di */
    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 372:  /* *andqi_1_slp */
    case 289:  /* *subqi_1_slp */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      extract_insn_cached (insn);
      if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_BOTH;
        }
      else
        {
	  return MEMORY_LOAD;
        }

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_BOTH;
        }
      else
        {
	  return MEMORY_STORE;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return MEMORY_UNKNOWN;
        }
      else
        {
	  if (memory_operand (operands[1], VOIDmode))
	    {
	      return MEMORY_BOTH;
	    }
	  else
	    {
	      return MEMORY_STORE;
	    }
        }

    case 1822:  /* xop_pcom_tfv2di3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1721:  /* sse4_1_ptest */
    case 1720:  /* avx_ptest256 */
    case 1719:  /* avx_vtestpd */
    case 1718:  /* avx_vtestpd256 */
    case 1717:  /* avx_vtestps */
    case 1716:  /* avx_vtestps256 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1212:  /* sse2_ucomi */
    case 1211:  /* sse_ucomi */
    case 1210:  /* sse2_comi */
    case 1209:  /* sse_comi */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 39:  /* *cmpfp_iu_387 */
    case 38:  /* *cmpfp_iu_sse */
    case 37:  /* *cmpfp_iu_mixed */
    case 36:  /* *cmpfp_i_i387 */
    case 35:  /* *cmpfp_i_sse */
    case 34:  /* *cmpfp_i_mixed */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      extract_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) || (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_OTHER) || ((cached_type == TYPE_MULTI) || ((cached_type == TYPE_STR) || (cached_type == TYPE_LWP))))
        {
	  return MEMORY_UNKNOWN;
        }
      else if ((cached_type == TYPE_LEA) || ((cached_type == TYPE_FCMOV) || (cached_type == TYPE_FPSPC)))
        {
	  return MEMORY_NONE;
        }
      else if ((cached_type == TYPE_FISTP) || (cached_type == TYPE_LEAVE))
        {
	  return MEMORY_BOTH;
        }
      else if (cached_type == TYPE_FRNDINT)
        {
	  return MEMORY_LOAD;
        }
      else if (cached_type == TYPE_PUSH)
        {
	  if (memory_operand (operands[1], VOIDmode))
	    {
	      return MEMORY_BOTH;
	    }
	  else
	    {
	      return MEMORY_STORE;
	    }
        }
      else if (cached_type == TYPE_POP)
        {
	  if (memory_operand (operands[0], VOIDmode))
	    {
	      return MEMORY_BOTH;
	    }
	  else
	    {
	      return MEMORY_LOAD;
	    }
        }
      else if (cached_type == TYPE_SETCC)
        {
	  if (memory_operand (operands[0], VOIDmode))
	    {
	      return MEMORY_STORE;
	    }
	  else
	    {
	      return MEMORY_NONE;
	    }
        }
      else if ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_SSECMP) || ((cached_type == TYPE_SSECOMI) || ((cached_type == TYPE_MMXCMP) || (cached_type == TYPE_FCMP))))))
        {
	  if ((memory_operand (operands[0], VOIDmode)) || (memory_operand (operands[1], VOIDmode)))
	    {
	      return MEMORY_LOAD;
	    }
	  else
	    {
	      return MEMORY_NONE;
	    }
        }
      else if (cached_type == TYPE_IBR)
        {
	  if (memory_operand (operands[0], VOIDmode))
	    {
	      return MEMORY_LOAD;
	    }
	  else
	    {
	      return MEMORY_NONE;
	    }
        }
      else if (cached_type == TYPE_CALL)
        {
	  if (constant_call_address_operand (operands[0], VOIDmode))
	    {
	      return MEMORY_NONE;
	    }
	  else
	    {
	      return MEMORY_LOAD;
	    }
        }
      else if (cached_type == TYPE_CALLV)
        {
	  if (constant_call_address_operand (operands[1], VOIDmode))
	    {
	      return MEMORY_NONE;
	    }
	  else
	    {
	      return MEMORY_LOAD;
	    }
        }
      else if ((((cached_type == TYPE_ALU1) || ((cached_type == TYPE_NEGNOT) || ((cached_type == TYPE_ISHIFT1) || (cached_type == TYPE_SSELOG1)))) && (memory_operand (operands[1], VOIDmode))) || ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode))))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || ((((! (cached_type == TYPE_ALU1)) && ((! (cached_type == TYPE_NEGNOT)) && ((! (cached_type == TYPE_ISHIFT1)) && ((! (cached_type == TYPE_IMOV)) && ((! (cached_type == TYPE_IMOVX)) && ((! (cached_type == TYPE_BITMANIP)) && ((! (cached_type == TYPE_FMOV)) && ((! (cached_type == TYPE_FSGN)) && ((! (cached_type == TYPE_SSE)) && ((! (cached_type == TYPE_SSEMOV)) && ((! (cached_type == TYPE_SSECVT)) && ((! (cached_type == TYPE_SSECVT1)) && ((! (cached_type == TYPE_SSEICVT)) && ((! (cached_type == TYPE_SSELOG1)) && ((! (cached_type == TYPE_SSEIADD1)) && ((! (cached_type == TYPE_MMX)) && ((! (cached_type == TYPE_MMXMOV)) && (! (cached_type == TYPE_MMXCVT))))))))))))))))))) && (memory_operand (operands[2], VOIDmode))) || (((cached_type == TYPE_ICMOV) || ((cached_type == TYPE_SSEMULADD) || (cached_type == TYPE_SSE4ARG))) && (memory_operand (operands[3], VOIDmode)))))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (memory_operand (operands[1], VOIDmode))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    case 1362:  /* vec_extract_hi_v4di */
    case 1363:  /* vec_extract_hi_v4df */
    case 1366:  /* vec_extract_hi_v8si */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1371:  /* vec_extract_hi_v32qi */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MEMORY_NONE;
        }
      else
        {
	  return MEMORY_STORE;
        }

    case 1728:  /* sse4_2_pcmpestr */
    case 1729:  /* sse4_2_pcmpestri */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1732:  /* sse4_2_pcmpistr */
    case 1733:  /* sse4_2_pcmpistri */
    case 1734:  /* sse4_2_pcmpistrm */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MEMORY_NONE;
        }
      else
        {
	  return MEMORY_LOAD;
        }

    case 1731:  /* sse4_2_pcmpestr_cconly */
    case 1735:  /* sse4_2_pcmpistr_cconly */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MEMORY_NONE;
        }
      else if (which_alternative == 1)
        {
	  return MEMORY_LOAD;
        }
      else if (which_alternative == 2)
        {
	  return MEMORY_NONE;
        }
      else
        {
	  return MEMORY_LOAD;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 97:  /* *pushtf */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
    case 958:  /* *pause */
    case 1607:  /* sse2_clflush */
    case 2008:  /* *sse2_lfence */
    case 2009:  /* *sse_sfence */
    case 2010:  /* mfence_sse2 */
    case 2011:  /* mfence_nosse */
      return MEMORY_UNKNOWN;

    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 855:  /* *strmovsi_1 */
    case 856:  /* *strmovsi_1 */
    case 857:  /* *strmovhi_1 */
    case 858:  /* *strmovhi_1 */
    case 859:  /* *strmovqi_1 */
    case 860:  /* *strmovqi_1 */
    case 861:  /* *rep_movdi_rex64 */
    case 862:  /* *rep_movsi */
    case 863:  /* *rep_movsi */
    case 864:  /* *rep_movqi */
    case 865:  /* *rep_movqi */
      return MEMORY_BOTH;

    case 67:  /* *movabsqi_1 */
    case 68:  /* *movabshi_1 */
    case 69:  /* *movabssi_1 */
    case 70:  /* *movabsdi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 867:  /* *strsetsi_1 */
    case 868:  /* *strsetsi_1 */
    case 869:  /* *strsethi_1 */
    case 870:  /* *strsethi_1 */
    case 871:  /* *strsetqi_1 */
    case 872:  /* *strsetqi_1 */
    case 873:  /* *rep_stosdi_rex64 */
    case 874:  /* *rep_stossi */
    case 875:  /* *rep_stossi */
    case 876:  /* *rep_stosqi */
    case 877:  /* *rep_stosqi */
    case 1606:  /* sse_stmxcsr */
      return MEMORY_STORE;

    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
    case 71:  /* *movabsqi_2 */
    case 72:  /* *movabshi_2 */
    case 73:  /* *movabssi_2 */
    case 74:  /* *movabsdi_2 */
    case 723:  /* *load_tp_x32 */
    case 724:  /* *load_tp_x32_zext */
    case 725:  /* *load_tp_si */
    case 726:  /* *load_tp_di */
    case 727:  /* *add_tp_x32 */
    case 728:  /* *add_tp_x32_zext */
    case 729:  /* *add_tp_si */
    case 730:  /* *add_tp_di */
    case 1605:  /* sse_ldmxcsr */
      return MEMORY_LOAD;

    case 894:  /* *movxfcc_1 */
    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 884:  /* *x86_movsicc_0_m1 */
    case 885:  /* *x86_movdicc_0_m1 */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 919:  /* *prefetch_sse */
    case 920:  /* *prefetch_3dnow */
    case 1073:  /* *mmx_emms */
    case 1074:  /* *mmx_femms */
    case 1834:  /* *avx_vzeroall */
    case 1835:  /* avx_vzeroupper */
      return MEMORY_NONE;

    default:
      extract_insn_cached (insn);
      if ((memory_operand (operands[0], VOIDmode)) && (memory_operand (operands[1], VOIDmode)))
        {
	  return MEMORY_BOTH;
        }
      else if (memory_operand (operands[0], VOIDmode))
        {
	  return MEMORY_STORE;
        }
      else if ((memory_operand (operands[1], VOIDmode)) || (memory_operand (operands[2], VOIDmode)))
        {
	  return MEMORY_LOAD;
        }
      else
        {
	  return MEMORY_NONE;
        }

    }
}

int
get_attr_modrm (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) || ((which_alternative == 10) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode)))))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) || ((which_alternative == 6) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode)))))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) || ((which_alternative == 4) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode)))))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 10) || ((which_alternative == 9) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode)))))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 6) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode))))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1383:  /* sse2_storehpd */
    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 4) || ((which_alternative == 5) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode)))))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))) && (const0_operand (operands[2], SImode))) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode))))))
        {
	  return 0;
        }
      else if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 756:  /* *fop_sf_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 749:  /* *fop_df_1_i387 */
    case 748:  /* *fop_sf_1_i387 */
    case 744:  /* *fop_df_1_mixed */
    case 743:  /* *fop_sf_1_mixed */
      if (get_attr_unit (insn) == UNIT_I387)
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
      extract_insn_cached (insn);
      if (constant_call_address_operand (operands[1], VOIDmode))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      extract_insn_cached (insn);
      if (constant_call_address_operand (operands[0], VOIDmode))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 492:  /* *ashlqi3_1 */
    case 491:  /* *ashlhi3_1 */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if ((get_attr_type (insn) == TYPE_ALU) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x3)) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 356:  /* *testdi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else if (which_alternative == 1)
        {
	  return 1;
        }
      else if (which_alternative == 2)
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 273:  /* *adddi_5 */
    case 266:  /* *adddi_4 */
    case 264:  /* *adddi_3 */
    case 259:  /* *adddi_2 */
      extract_constrain_insn_cached (insn);
      if ((incdec_operand (operands[2], DImode)) && ((! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT))) && ((register_operand (operands[1], SImode)) || (register_operand (operands[1], HImode)))))
        {
	  return 0;
        }
      else if ((! (incdec_operand (operands[2], DImode))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 272:  /* *addsi_5 */
    case 269:  /* *addsi_4 */
    case 265:  /* *addsi_3_zext */
    case 263:  /* *addsi_3 */
    case 260:  /* *addsi_2_zext */
    case 258:  /* *addsi_2 */
      extract_constrain_insn_cached (insn);
      if ((incdec_operand (operands[2], SImode)) && ((! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT))) && ((register_operand (operands[1], SImode)) || (register_operand (operands[1], HImode)))))
        {
	  return 0;
        }
      else if ((! (incdec_operand (operands[2], SImode))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 271:  /* *addhi_5 */
    case 268:  /* *addhi_4 */
    case 262:  /* *addhi_3 */
    case 257:  /* *addhi_2 */
      extract_constrain_insn_cached (insn);
      if ((incdec_operand (operands[2], HImode)) && ((! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT))) && ((register_operand (operands[1], SImode)) || (register_operand (operands[1], HImode)))))
        {
	  return 0;
        }
      else if ((! (incdec_operand (operands[2], HImode))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 270:  /* *addqi_5 */
    case 267:  /* *addqi_4 */
    case 261:  /* *addqi_3 */
    case 256:  /* *addqi_2 */
      extract_constrain_insn_cached (insn);
      if ((incdec_operand (operands[2], QImode)) && ((! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT))) && ((register_operand (operands[1], SImode)) || (register_operand (operands[1], HImode)))))
        {
	  return 0;
        }
      else if ((! (incdec_operand (operands[2], QImode))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 255:  /* *addqi_1_slp */
      extract_constrain_insn_cached (insn);
      if ((incdec_operand (operands[1], QImode)) && ((! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT))) && ((register_operand (operands[1], SImode)) || (register_operand (operands[1], HImode)))))
        {
	  return 0;
        }
      else if ((! (incdec_operand (operands[1], QImode))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 5) && (incdec_operand (operands[2], QImode))) && ((! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT))) && ((register_operand (operands[1], SImode)) || (register_operand (operands[1], HImode)))))
        {
	  return 0;
        }
      else if (((which_alternative != 5) && (! (incdec_operand (operands[2], QImode)))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (incdec_operand (operands[2], HImode))) && ((! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT))) && ((register_operand (operands[1], SImode)) || (register_operand (operands[1], HImode)))))
        {
	  return 0;
        }
      else if (((which_alternative != 3) && (! (incdec_operand (operands[2], HImode)))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 2) && (incdec_operand (operands[2], SImode))) && ((! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT))) && ((register_operand (operands[1], SImode)) || (register_operand (operands[1], HImode)))))
        {
	  return 0;
        }
      else if (((which_alternative != 2) && (! (incdec_operand (operands[2], SImode)))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 251:  /* *adddi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (incdec_operand (operands[2], DImode))) && ((! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT))) && ((register_operand (operands[1], SImode)) || (register_operand (operands[1], HImode)))))
        {
	  return 0;
        }
      else if (((which_alternative != 3) && (! (incdec_operand (operands[2], DImode)))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative != 3) && (incdec_operand (operands[2], SImode))) && ((! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT))) && ((register_operand (operands[1], SImode)) || (register_operand (operands[1], HImode)))))
        {
	  return 0;
        }
      else if (((which_alternative != 3) && (! (incdec_operand (operands[2], SImode)))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 1)
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 897:  /* *movsfcc_1_387 */
    case 896:  /* *movdfcc_1 */
    case 895:  /* *movdfcc_1_rex64 */
    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 134:  /* extendqihi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
      extract_constrain_insn_cached (insn);
      if ((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_K6))))) && (which_alternative == 0))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) || ((((1 << which_alternative) & 0x18)) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode)))))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 6)
        {
	  return 0;
        }
      else
        {
	  if ((((1 << which_alternative) & 0x7)) || ((((1 << which_alternative) & 0x78)) && ((!((1 << which_alternative) & 0x1858)) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode))))))))
	    {
	      return 0;
	    }
	  else
	    {
	      return 1;
	    }
        }

    case 109:  /* *movdf_internal */
    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) || ((which_alternative == 1) && (! (memory_operand (operands[1], VOIDmode)))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode))))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 75:  /* *swapsi */
      extract_insn_cached (insn);
      if (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode)))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else
        {
	  if (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode)))))
	    {
	      return 0;
	    }
	  else
	    {
	      return 1;
	    }
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))))))) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode))))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 65:  /* *movhi_internal */
      extract_insn_cached (insn);
      if ((get_attr_type (insn) == TYPE_IMOV) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode))))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((!((1 << which_alternative) & 0xffc)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((!((1 << which_alternative) & 0xc)) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode)))))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && (! (pic_32bit_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else
        {
	  if (((!((1 << which_alternative) & 0x1fff0)) && (! (pic_32bit_operand (operands[1], VOIDmode)))) && ((((1 << which_alternative) & 0xa01)) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode)))))))
	    {
	      return 0;
	    }
	  else
	    {
	      return 1;
	    }
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 379:  /* *andqi_2_slp */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 372:  /* *andqi_1_slp */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 289:  /* *subqi_1_slp */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
    case 17:  /* *cmpqi_ext_4 */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      extract_constrain_insn_cached (insn);
      if (ax_reg_operand (operands[0], VOIDmode))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_STR) || ((cached_type == TYPE_LEAVE) || ((get_attr_unit (insn) == UNIT_I387) || (((cached_type == TYPE_INCDEC) && ((! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT))) && ((register_operand (operands[1], SImode)) || (register_operand (operands[1], HImode))))) || (((cached_type == TYPE_PUSH) && (! (memory_operand (operands[1], VOIDmode)))) || (((cached_type == TYPE_POP) && (! (memory_operand (operands[0], VOIDmode)))) || (((cached_type == TYPE_IMOV) && ((! (get_attr_mode (insn) == MODE_DI)) && (((register_operand (operands[0], VOIDmode)) && (immediate_operand (operands[1], VOIDmode))) || (((ax_reg_operand (operands[0], VOIDmode)) && (memory_displacement_only_operand (operands[1], VOIDmode))) || ((memory_displacement_only_operand (operands[0], VOIDmode)) && (ax_reg_operand (operands[1], VOIDmode))))))) || (((cached_type == TYPE_CALL) && (constant_call_address_operand (operands[0], VOIDmode))) || ((cached_type == TYPE_CALLV) && (constant_call_address_operand (operands[1], VOIDmode)))))))))))
        {
	  return 0;
        }
      else if (((cached_type == TYPE_ALU) || ((cached_type == TYPE_ALU1) || ((cached_type == TYPE_ICMP) || (cached_type == TYPE_TEST)))) && (ax_reg_operand (operands[0], VOIDmode)))
        {
	  return 
#line 502 "../../gcc/config/i386/i386.md"
((get_attr_length_immediate (insn) <= (get_attr_mode (insn) != MODE_QI)));
        }
      else
        {
	  return 1;
        }

    case 738:  /* *fop_df_comm_mixed */
    case 737:  /* *fop_sf_comm_mixed */
    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 74:  /* *movabsdi_2 */
    case 70:  /* *movabsdi_1 */
    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
    case 126:  /* *extendsidi2_rex64 */
    case 513:  /* ashrdi3_cvt */
    case 514:  /* ashrsi3_cvt */
    case 515:  /* *ashrsi3_cvt_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 894:  /* *movxfcc_1 */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 761:  /* *fop_xf_comm_i387 */
    case 742:  /* *fop_df_comm_i387 */
    case 741:  /* *fop_sf_comm_i387 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 145:  /* *truncdfsf2_i387_1 */
    case 144:  /* *truncdfsf_i387 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
    case 587:  /* *jcc_1 */
    case 588:  /* *jcc_2 */
    case 602:  /* jump */
    case 638:  /* simple_return_internal */
    case 639:  /* simple_return_internal_long */
    case 640:  /* simple_return_pop_internal */
    case 642:  /* nop */
    case 643:  /* nops */
    case 653:  /* split_stack_return */
    case 711:  /* *bswapsi2_1 */
    case 712:  /* *bswapdi2_1 */
    case 723:  /* *load_tp_x32 */
    case 724:  /* *load_tp_x32_zext */
    case 725:  /* *load_tp_si */
    case 726:  /* *load_tp_di */
    case 727:  /* *add_tp_x32 */
    case 728:  /* *add_tp_x32_zext */
    case 729:  /* *add_tp_si */
    case 730:  /* *add_tp_di */
    case 853:  /* cld */
    case 1073:  /* *mmx_emms */
    case 1074:  /* *mmx_femms */
    case 1834:  /* *avx_vzeroall */
    case 1835:  /* avx_vzeroupper */
      return 0;

    default:
      return 1;

    }
}

enum attr_mode
get_attr_mode (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return MODE_TI;
        }
      else if (which_alternative == 4)
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_V2SF;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7f))
        {
	  return MODE_TI;
        }
      else
        {
	  return MODE_V2SF;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_TI;
        }
      else if (which_alternative == 2)
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_V2SF;
        }

    case 1587:  /* *vec_concatv2si_sse */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_DI;
        }

    case 1586:  /* *vec_concatv2si_sse2 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_TI;
        }
      else
        {
	  return MODE_DI;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f))
        {
	  return MODE_TI;
        }
      else
        {
	  return MODE_DI;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_TI;
        }
      else if (which_alternative == 2)
        {
	  return MODE_DF;
        }
      else
        {
	  return MODE_V4SF;
        }

    case 1583:  /* *vec_dupv4si */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_TI;
        }
      else
        {
	  return MODE_V4SF;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_V2SF;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return MODE_TI;
        }
      else if (which_alternative == 4)
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_V2SF;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_V2SF;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return MODE_TI;
        }
      else
        {
	  return MODE_DI;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_UNKNOWN;
        }
      else
        {
	  return MODE_DI;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_TI;
        }
      else if (which_alternative == 2)
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_SF;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_V2DF;
        }
      else if (which_alternative == 2)
        {
	  return MODE_DF;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return MODE_V1DF;
        }
      else if (which_alternative == 5)
        {
	  return MODE_DF;
        }
      else if (which_alternative == 6)
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_V2SF;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_DF;
        }
      else if (((1 << which_alternative) & 0x1c))
        {
	  return MODE_V1DF;
        }
      else if (which_alternative == 5)
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_V1DF;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_DF;
        }
      else if (((1 << which_alternative) & 0x1e))
        {
	  return MODE_V1DF;
        }
      else if (which_alternative == 5)
        {
	  return MODE_V2DF;
        }
      else if (((1 << which_alternative) & 0xc0))
        {
	  return MODE_V1DF;
        }
      else
        {
	  return MODE_DF;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_V1DF;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_DF;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_V1DF;
        }
      else
        {
	  return MODE_DF;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_V1DF;
        }
      else if (which_alternative == 2)
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_DF;
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_V2DF;
        }
      else if (which_alternative == 2)
        {
	  return MODE_DF;
        }
      else
        {
	  return MODE_V1DF;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_UNKNOWN;
        }

    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_SF;
        }
      else if (which_alternative == 2)
        {
	  return MODE_SI;
        }
      else if (((1 << which_alternative) & 0x38))
        {
	  return MODE_SF;
        }
      else if (((1 << which_alternative) & 0xc0))
        {
	  return MODE_TI;
        }
      else
        {
	  return MODE_UNKNOWN;
        }

    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_SF;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return MODE_SI;
        }
      else if (((1 << which_alternative) & 0x38))
        {
	  return MODE_SF;
        }
      else if (((1 << which_alternative) & 0xc0))
        {
	  return MODE_TI;
        }
      else
        {
	  return MODE_UNKNOWN;
        }

    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_V4SF;
        }
      else if (which_alternative == 1)
        {
	  return MODE_SF;
        }
      else
        {
	  return MODE_DI;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return MODE_V4SF;
        }
      else if (which_alternative == 4)
        {
	  return MODE_SF;
        }
      else
        {
	  return MODE_DI;
        }

    case 1343:  /* sse_loadhps */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_V2SF;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_V2SF;
        }

    case 1352:  /* *vec_concatv4sf */
    case 1345:  /* sse_loadlps */
    case 1330:  /* sse_movlhps */
    case 1329:  /* sse_movhlps */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_V2SF;
        }

    case 1088:  /* *movv2df_internal */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return MODE_V2DF;
        }
      else if (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DFmode) == (V4SFmode))))
        {
	  return MODE_V4SF;
        }
      else if ((V2DFmode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1087:  /* *movv4df_internal */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return MODE_V4DF;
        }
      else if (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DFmode) == (V4SFmode))))
        {
	  return MODE_V4SF;
        }
      else if ((V4DFmode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1086:  /* *movv4sf_internal */
      extract_constrain_insn_cached (insn);
      if ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SFmode) == (V4SFmode))))))
        {
	  return MODE_V4SF;
        }
      else if ((V4SFmode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return MODE_V8SF;
        }
      else if (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SFmode) == (V4SFmode))))
        {
	  return MODE_V4SF;
        }
      else if ((V8SFmode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1084:  /* *movv1ti_internal */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return MODE_TI;
        }
      else if (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V1TImode) == (V4SFmode))))
        {
	  return MODE_V4SF;
        }
      else if ((V1TImode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1083:  /* *movv2ti_internal */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return MODE_OI;
        }
      else if (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2TImode) == (V4SFmode))))
        {
	  return MODE_V4SF;
        }
      else if ((V2TImode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1082:  /* *movv2di_internal */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return MODE_TI;
        }
      else if (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V2DImode) == (V4SFmode))))
        {
	  return MODE_V4SF;
        }
      else if ((V2DImode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1081:  /* *movv4di_internal */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return MODE_OI;
        }
      else if (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4DImode) == (V4SFmode))))
        {
	  return MODE_V4SF;
        }
      else if ((V4DImode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1080:  /* *movv4si_internal */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return MODE_TI;
        }
      else if (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V4SImode) == (V4SFmode))))
        {
	  return MODE_V4SF;
        }
      else if ((V4SImode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1079:  /* *movv8si_internal */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return MODE_OI;
        }
      else if (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8SImode) == (V4SFmode))))
        {
	  return MODE_V4SF;
        }
      else if ((V8SImode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1078:  /* *movv8hi_internal */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return MODE_TI;
        }
      else if (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V8HImode) == (V4SFmode))))
        {
	  return MODE_V4SF;
        }
      else if ((V8HImode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1077:  /* *movv16hi_internal */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return MODE_OI;
        }
      else if (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16HImode) == (V4SFmode))))
        {
	  return MODE_V4SF;
        }
      else if ((V16HImode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1076:  /* *movv16qi_internal */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return MODE_TI;
        }
      else if (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V16QImode) == (V4SFmode))))
        {
	  return MODE_V4SF;
        }
      else if ((V16QImode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return MODE_OI;
        }
      else if (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || (((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) || ((V32QImode) == (V4SFmode))))
        {
	  return MODE_V4SF;
        }
      else if ((V32QImode) == (V2DFmode))
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_DI;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return MODE_TI;
        }
      else if (which_alternative == 3)
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_SI;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_DI;
        }
      else if (which_alternative == 1)
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_SF;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3f))
        {
	  return MODE_DI;
        }
      else if (((1 << which_alternative) & 0xc0))
        {
	  return MODE_V4SF;
        }
      else if (((1 << which_alternative) & 0x300))
        {
	  return MODE_V2SF;
        }
      else
        {
	  return MODE_DI;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xff))
        {
	  return MODE_DI;
        }
      else if (((1 << which_alternative) & 0x300))
        {
	  return MODE_V4SF;
        }
      else if (((1 << which_alternative) & 0xc00))
        {
	  return MODE_V2SF;
        }
      else
        {
	  return MODE_DI;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3f))
        {
	  return MODE_DI;
        }
      else if (which_alternative == 6)
        {
	  return MODE_TI;
        }
      else if (((1 << which_alternative) & 0x180))
        {
	  return MODE_DI;
        }
      else if (((1 << which_alternative) & 0x600))
        {
	  return MODE_V4SF;
        }
      else if (((1 << which_alternative) & 0x1800))
        {
	  return MODE_V2SF;
        }
      else
        {
	  return MODE_DI;
        }

    case 897:  /* *movsfcc_1_387 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_SF;
        }
      else
        {
	  return MODE_SI;
        }

    case 896:  /* *movdfcc_1 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_DF;
        }
      else
        {
	  return MODE_DI;
        }

    case 373:  /* *anddi_2 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_SI;
        }
      else
        {
	  return MODE_DI;
        }

    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 374:  /* *andqi_2_maybe_si */
    case 371:  /* *andqi_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_QI;
        }
      else
        {
	  return MODE_SI;
        }

    case 370:  /* *andhi_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_HI;
        }
      else
        {
	  return MODE_SI;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_SI;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return MODE_DI;
        }
      else
        {
	  return MODE_SI;
        }

    case 356:  /* *testdi_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_SI;
        }
      else
        {
	  return MODE_DI;
        }

    case 357:  /* *testqi_1_maybe_si */
    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return MODE_QI;
        }
      else
        {
	  return MODE_SI;
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return MODE_HI;
        }
      else
        {
	  return MODE_SI;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_DF;
        }
      else
        {
	  return MODE_V2DF;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_SF;
        }
      else
        {
	  return MODE_V4SF;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return MODE_DF;
        }
      else
        {
	  return MODE_V2DF;
        }

    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return MODE_SF;
        }
      else
        {
	  return MODE_V4SF;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return MODE_SI;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return MODE_DI;
        }
      else
        {
	  return MODE_TI;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_SI;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return MODE_DI;
        }
      else
        {
	  return MODE_TI;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return MODE_XF;
        }
      else
        {
	  return MODE_SI;
        }

    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_TF;
        }
      else
        {
	  return MODE_SI;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))
        {
	  return MODE_SI;
        }
      else
        {
	  return MODE_QI;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x38))
        {
	  return MODE_SI;
        }
      else if (which_alternative == 6)
        {
	  return MODE_QI;
        }
      else if ((get_attr_type (insn) == TYPE_IMOVX) || (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2))) && ((((1 << which_alternative) & 0x3)) && (((
#line 2311 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_DEPENDENCY)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))))) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH))))))))
        {
	  return MODE_SI;
        }
      else
        {
	  return MODE_QI;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) || (((((1 << which_alternative) & 0x6)) && (aligned_operand (operands[1], HImode))) || ((which_alternative == 0) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))))))
        {
	  return MODE_SI;
        }
      else
        {
	  return MODE_HI;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f))
        {
	  return MODE_DI;
        }
      else if (which_alternative == 5)
        {
	  return MODE_TI;
        }
      else if (which_alternative == 6)
        {
	  return MODE_DI;
        }
      else if (which_alternative == 7)
        {
	  return MODE_TI;
        }
      else if (which_alternative == 8)
        {
	  return MODE_DI;
        }
      else if (which_alternative == 9)
        {
	  return MODE_V4SF;
        }
      else if (which_alternative == 10)
        {
	  return MODE_V2SF;
        }
      else if (which_alternative == 11)
        {
	  return MODE_V4SF;
        }
      else if (which_alternative == 12)
        {
	  return MODE_V2SF;
        }
      else
        {
	  return MODE_DI;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_SI;
        }
      else if (((1 << which_alternative) & 0x1fe))
        {
	  return MODE_DI;
        }
      else if (which_alternative == 9)
        {
	  return MODE_TI;
        }
      else if (which_alternative == 10)
        {
	  return MODE_DI;
        }
      else if (which_alternative == 11)
        {
	  return MODE_TI;
        }
      else
        {
	  return MODE_DI;
        }

    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 2) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))))
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_TI;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return MODE_DI;
        }
      else if ((! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 4) && (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))))
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_TI;
        }

    case 34:  /* *cmpfp_i_mixed */
    case 35:  /* *cmpfp_i_sse */
    case 37:  /* *cmpfp_iu_mixed */
    case 38:  /* *cmpfp_iu_sse */
      extract_insn_cached (insn);
      if (GET_MODE (operands[1]) == SFmode)
        {
	  return MODE_SF;
        }
      else
        {
	  return MODE_DF;
        }

    case 18:  /* *cmpfp_0 */
    case 19:  /* *cmpfp_0_cc */
    case 26:  /* *cmpfp_u */
    case 27:  /* *cmpfp_u_cc */
    case 36:  /* *cmpfp_i_i387 */
    case 39:  /* *cmpfp_iu_387 */
      extract_insn_cached (insn);
      if (GET_MODE (operands[1]) == SFmode)
        {
	  return MODE_SF;
        }
      else if (GET_MODE (operands[1]) == DFmode)
        {
	  return MODE_DF;
        }
      else
        {
	  return MODE_XF;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc))
        {
	  return MODE_DI;
        }
      else if (((1 << which_alternative) & 0xc0))
        {
	  if (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
	    {
	      return MODE_V4SF;
	    }
	  else
	    {
	      return MODE_TI;
	    }
        }
      else if ((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))
        {
	  return MODE_SF;
        }
      else
        {
	  return MODE_SI;
        }

    case 88:  /* *movqi_extv_1 */
    case 92:  /* *movqi_extzv_2 */
      if (get_attr_type (insn) == TYPE_IMOVX)
        {
	  return MODE_SI;
        }
      else
        {
	  return MODE_QI;
        }

    case 98:  /* *pushxf */
    case 99:  /* *pushxf_nointeger */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_XF;
        }
      else
        {
	  return MODE_SI;
        }

    case 100:  /* *pushdf_rex64 */
    case 101:  /* *pushdf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_DF;
        }
      else if (which_alternative == 1)
        {
	  return MODE_DI;
        }
      else
        {
	  return MODE_DF;
        }

    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_SF;
        }
      else if (which_alternative == 1)
        {
	  return MODE_DI;
        }
      else
        {
	  return MODE_SF;
        }

    case 103:  /* *pushsf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_SF;
        }
      else if (which_alternative == 1)
        {
	  return MODE_SI;
        }
      else
        {
	  return MODE_SF;
        }

    case 104:  /* *movtf_internal_rex64 */
    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x5))
        {
	  if (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))
	    {
	      return MODE_V4SF;
	    }
	  else
	    {
	      return MODE_TI;
	    }
        }
      else if (which_alternative == 1)
        {
	  if ((
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)) || (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))
	    {
	      return MODE_V4SF;
	    }
	  else
	    {
	      return MODE_TI;
	    }
        }
      else
        {
	  return MODE_DI;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return MODE_DF;
        }
      else if (((1 << which_alternative) & 0x1858))
        {
	  return MODE_DI;
        }
      else if (which_alternative == 5)
        {
	  return MODE_SI;
        }
      else if (which_alternative == 7)
        {
	  if (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))
	    {
	      return MODE_V4SF;
	    }
	  else if (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))
	    {
	      return MODE_TI;
	    }
	  else
	    {
	      return MODE_V2DF;
	    }
        }
      else if (which_alternative == 8)
        {
	  if (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))
	    {
	      return MODE_V4SF;
	    }
	  else if (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))
	    {
	      return MODE_V2DF;
	    }
	  else
	    {
	      return MODE_DF;
	    }
        }
      else if (which_alternative == 9)
        {
	  if (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))
	    {
	      return MODE_V1DF;
	    }
	  else
	    {
	      return MODE_DF;
	    }
        }
      else
        {
	  return MODE_DF;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return MODE_DF;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return MODE_SI;
        }
      else if (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
        {
	  if (((1 << which_alternative) & 0x660))
	    {
	      return MODE_V4SF;
	    }
	  else
	    {
	      return MODE_V2SF;
	    }
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  if (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))
	    {
	      return MODE_V4SF;
	    }
	  else if (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))
	    {
	      return MODE_TI;
	    }
	  else
	    {
	      return MODE_V2DF;
	    }
        }
      else if (((1 << which_alternative) & 0x440))
        {
	  if (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))
	    {
	      return MODE_V4SF;
	    }
	  else if (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))
	    {
	      return MODE_V2DF;
	    }
	  else
	    {
	      return MODE_DF;
	    }
        }
      else if (((1 << which_alternative) & 0x880))
        {
	  if (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))
	    {
	      return MODE_V1DF;
	    }
	  else
	    {
	      return MODE_DF;
	    }
        }
      else
        {
	  return MODE_DF;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x618))
        {
	  return MODE_SI;
        }
      else if (which_alternative == 5)
        {
	  if (((
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR)) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))
	    {
	      return MODE_TI;
	    }
	  else
	    {
	      return MODE_V4SF;
	    }
        }
      else if (which_alternative == 6)
        {
	  if ((
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY)) || (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))
	    {
	      return MODE_V4SF;
	    }
	  else
	    {
	      return MODE_SF;
	    }
        }
      else if (which_alternative == 11)
        {
	  return MODE_DI;
        }
      else
        {
	  return MODE_SF;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_SF;
        }
      else if (which_alternative == 1)
        {
	  return MODE_XF;
        }
      else
        {
	  return MODE_DF;
        }

    case 137:  /* *extendsfdf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_SF;
        }
      else
        {
	  return MODE_XF;
        }

    case 139:  /* *extenddfxf2_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_DF;
        }
      else
        {
	  return MODE_XF;
        }

    case 195:  /* *floatsisf2_vector_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_SF;
        }
      else
        {
	  return MODE_V4SF;
        }

    case 196:  /* *floatsidf2_vector_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_DF;
        }
      else
        {
	  return MODE_V2DF;
        }

    case 244:  /* *leadi */
      extract_insn_cached (insn);
      if (SImode_address_operand (operands[1], VOIDmode))
        {
	  return MODE_SI;
        }
      else
        {
	  return MODE_DI;
        }

    case 280:  /* *lea_general_4 */
    case 281:  /* *lea_general_4 */
      extract_insn_cached (insn);
      if (GET_MODE (operands[0]) == DImode)
        {
	  return MODE_DI;
        }
      else
        {
	  return MODE_SI;
        }

    case 492:  /* *ashlqi3_1 */
    case 472:  /* *one_cmplqi2_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_QI;
        }
      else
        {
	  return MODE_SI;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_HI;
        }
      else
        {
	  return MODE_SI;
        }

    case 713:  /* *bswaphi_lowpart_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_QI;
        }
      else
        {
	  return MODE_HI;
        }

    case 1342:  /* sse_storehps */
    case 1344:  /* sse_storelps */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1386:  /* *vec_extractv2df_0_sse */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_V2SF;
        }
      else if (which_alternative == 1)
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_V2SF;
        }

    case 1390:  /* vec_dupv2df */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return MODE_V2DF;
        }
      else
        {
	  return MODE_DF;
        }

    case 1503:  /* *andnotv32qi3 */
    case 1511:  /* *andv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1513:  /* *xorv32qi3 */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V32QImode) > 16)))
        {
	  return MODE_V8SF;
        }
      else if (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_OI;
        }

    case 1504:  /* *andnotv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1516:  /* *xorv16qi3 */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16)))
        {
	  return MODE_V8SF;
        }
      else if (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1505:  /* *andnotv16hi3 */
    case 1517:  /* *andv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1519:  /* *xorv16hi3 */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16HImode) > 16)))
        {
	  return MODE_V8SF;
        }
      else if (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_OI;
        }

    case 1506:  /* *andnotv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1522:  /* *xorv8hi3 */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16)))
        {
	  return MODE_V8SF;
        }
      else if (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1507:  /* *andnotv8si3 */
    case 1523:  /* *andv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1525:  /* *xorv8si3 */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8SImode) > 16)))
        {
	  return MODE_V8SF;
        }
      else if (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_OI;
        }

    case 1508:  /* *andnotv4si3 */
    case 1526:  /* *andv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1528:  /* *xorv4si3 */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16)))
        {
	  return MODE_V8SF;
        }
      else if (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_TI;
        }

    case 1509:  /* *andnotv4di3 */
    case 1529:  /* *andv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1531:  /* *xorv4di3 */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4DImode) > 16)))
        {
	  return MODE_V8SF;
        }
      else if (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_OI;
        }

    case 1510:  /* *andnotv2di3 */
    case 1532:  /* *andv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1534:  /* *xorv2di3 */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2))) && (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16)))
        {
	  return MODE_V8SF;
        }
      else if (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
        {
	  return MODE_V4SF;
        }
      else
        {
	  return MODE_TI;
        }

    case 970:  /* *mmx_addv2sf3 */
    case 971:  /* *mmx_subv2sf3 */
    case 972:  /* *mmx_mulv2sf3 */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 975:  /* *mmx_smaxv2sf3 */
    case 976:  /* *mmx_sminv2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 987:  /* mmx_gev2sf3 */
    case 988:  /* mmx_pf2id */
    case 989:  /* mmx_pf2iw */
    case 990:  /* mmx_pi2fw */
    case 991:  /* mmx_floatv2si2 */
    case 992:  /* mmx_pswapdv2sf2 */
      return MODE_V2SF;

    case 1094:  /* sse2_loadupd */
    case 1098:  /* sse2_storeupd */
    case 1110:  /* sse2_movntv2df */
    case 1123:  /* *addv2df3 */
    case 1124:  /* *subv2df3 */
    case 1132:  /* *mulv2df3 */
    case 1138:  /* sse2_divv2df3 */
    case 1147:  /* sse2_sqrtv2df2 */
    case 1159:  /* *smaxv2df3_finite */
    case 1160:  /* *sminv2df3_finite */
    case 1167:  /* *smaxv2df3 */
    case 1168:  /* *sminv2df3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1226:  /* *andv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1228:  /* *xorv2df3 */
    case 1230:  /* *andnotdf3 */
    case 1234:  /* *anddf3 */
    case 1235:  /* *iordf3 */
    case 1236:  /* *xordf3 */
    case 1240:  /* *fma_fmadd_v2df */
    case 1246:  /* *fma_fmsub_v2df */
    case 1252:  /* *fma_fnmadd_v2df */
    case 1258:  /* *fma_fnmsub_v2df */
    case 1264:  /* *fma_fmaddsub_v2df */
    case 1268:  /* *fma_fmsubadd_v2df */
    case 1270:  /* *fmai_fmadd_v2df */
    case 1272:  /* *fmai_fmsub_v2df */
    case 1274:  /* *fmai_fnmadd_v2df */
    case 1276:  /* *fmai_fnmsub_v2df */
    case 1278:  /* *fma4i_vmfmadd_v2df */
    case 1280:  /* *fma4i_vmfmsub_v2df */
    case 1282:  /* *fma4i_vmfnmadd_v2df */
    case 1284:  /* *fma4i_vmfnmsub_v2df */
    case 1302:  /* sse2_cvtpi2pd */
    case 1315:  /* sse2_cvtdq2pd */
    case 1328:  /* sse2_cvtps2pd */
    case 1381:  /* sse2_shufpd_v2di */
    case 1382:  /* sse2_shufpd_v2df */
    case 1600:  /* sse2_movmskpd */
    case 1670:  /* sse4_1_blendpd */
    case 1674:  /* sse4_1_blendvpd */
    case 1678:  /* sse4_1_dppd */
    case 1719:  /* avx_vtestpd */
    case 1725:  /* sse4_1_roundpd */
    case 1727:  /* sse4_1_roundsd */
    case 1802:  /* xop_frczv2df2 */
    case 1806:  /* *xop_vmfrczv2df2 */
    case 1826:  /* xop_vpermil2v2df3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1896:  /* avx_maskloadpd */
    case 1904:  /* avx_maskstorepd */
    case 1938:  /* *avx2_gathersiv2df */
    case 1939:  /* *avx2_gathersiv2df */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
      return MODE_V2DF;

    case 1092:  /* sse_loadups */
    case 1096:  /* sse_storeups */
    case 1108:  /* sse_movntv4sf */
    case 1119:  /* *addv4sf3 */
    case 1120:  /* *subv4sf3 */
    case 1130:  /* *mulv4sf3 */
    case 1136:  /* sse_divv4sf3 */
    case 1142:  /* sse_rcpv4sf2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1155:  /* *smaxv4sf3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1163:  /* *smaxv4sf3 */
    case 1164:  /* *sminv4sf3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1220:  /* *andv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1222:  /* *xorv4sf3 */
    case 1229:  /* *andnotsf3 */
    case 1231:  /* *andsf3 */
    case 1232:  /* *iorsf3 */
    case 1233:  /* *xorsf3 */
    case 1239:  /* *fma_fmadd_v4sf */
    case 1245:  /* *fma_fmsub_v4sf */
    case 1251:  /* *fma_fnmadd_v4sf */
    case 1257:  /* *fma_fnmsub_v4sf */
    case 1262:  /* *fma_fmaddsub_v4sf */
    case 1266:  /* *fma_fmsubadd_v4sf */
    case 1269:  /* *fmai_fmadd_v4sf */
    case 1271:  /* *fmai_fmsub_v4sf */
    case 1273:  /* *fmai_fnmadd_v4sf */
    case 1275:  /* *fmai_fnmsub_v4sf */
    case 1277:  /* *fma4i_vmfmadd_v4sf */
    case 1279:  /* *fma4i_vmfmsub_v4sf */
    case 1281:  /* *fma4i_vmfnmadd_v4sf */
    case 1283:  /* *fma4i_vmfnmsub_v4sf */
    case 1285:  /* sse_cvtpi2ps */
    case 1297:  /* floatv4siv4sf2 */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1332:  /* vec_interleave_highv4sf */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1336:  /* sse3_movshdup */
    case 1338:  /* sse3_movsldup */
    case 1340:  /* sse_shufps_v4si */
    case 1341:  /* sse_shufps_v4sf */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1349:  /* vec_dupv4sf */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1356:  /* sse4_1_insertps */
    case 1598:  /* sse_movmskps */
    case 1668:  /* sse4_1_blendps */
    case 1672:  /* sse4_1_blendvps */
    case 1676:  /* sse4_1_dpps */
    case 1717:  /* avx_vtestps */
    case 1723:  /* sse4_1_roundps */
    case 1726:  /* sse4_1_roundss */
    case 1801:  /* xop_frczv4sf2 */
    case 1805:  /* *xop_vmfrczv4sf2 */
    case 1824:  /* xop_vpermil2v4sf3 */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1895:  /* avx_maskloadps */
    case 1903:  /* avx_maskstoreps */
    case 1930:  /* vcvtph2ps */
    case 1933:  /* *vcvtps2ph */
    case 1934:  /* *vcvtps2ph_store */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
      return MODE_V4SF;

    case 1093:  /* avx_loadupd256 */
    case 1097:  /* avx_storeupd256 */
    case 1109:  /* avx_movntv4df */
    case 1121:  /* *addv4df3 */
    case 1122:  /* *subv4df3 */
    case 1131:  /* *mulv4df3 */
    case 1137:  /* avx_divv4df3 */
    case 1146:  /* avx_sqrtv4df2 */
    case 1157:  /* *smaxv4df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1165:  /* *smaxv4df3 */
    case 1166:  /* *sminv4df3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1185:  /* avx_haddv4df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1223:  /* *andv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1225:  /* *xorv4df3 */
    case 1242:  /* *fma_fmadd_v4df */
    case 1248:  /* *fma_fmsub_v4df */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1313:  /* floatv4siv4df2 */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1326:  /* avx_cvtps2pd256 */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1363:  /* vec_extract_hi_v4df */
    case 1372:  /* avx_unpckhpd256 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1376:  /* avx_shufpd256_1 */
    case 1599:  /* avx_movmskpd256 */
    case 1669:  /* avx_blendpd256 */
    case 1673:  /* avx_blendvpd256 */
    case 1677:  /* avx_dppd256 */
    case 1718:  /* avx_vtestpd256 */
    case 1724:  /* avx_roundpd256 */
    case 1804:  /* xop_frczv4df2 */
    case 1825:  /* xop_vpermil2v4df3 */
    case 1849:  /* avx2_vec_dupv4df */
    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1869:  /* *avx_vpermilpv4df */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1884:  /* vec_set_lo_v4df */
    case 1886:  /* vec_set_hi_v4df */
    case 1898:  /* avx_maskloadpd256 */
    case 1906:  /* avx_maskstorepd256 */
    case 1929:  /* avx_vec_concatv4df */
    case 1942:  /* *avx2_gathersiv4df */
    case 1943:  /* *avx2_gathersiv4df */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
      return MODE_V4DF;

    case 1091:  /* avx_loadups256 */
    case 1095:  /* avx_storeups256 */
    case 1107:  /* avx_movntv8sf */
    case 1117:  /* *addv8sf3 */
    case 1118:  /* *subv8sf3 */
    case 1129:  /* *mulv8sf3 */
    case 1135:  /* avx_divv8sf3 */
    case 1141:  /* avx_rcpv8sf2 */
    case 1144:  /* avx_sqrtv8sf2 */
    case 1150:  /* avx_rsqrtv8sf2 */
    case 1153:  /* *smaxv8sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1161:  /* *smaxv8sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1213:  /* avx_andnotv8sf3 */
    case 1217:  /* *andv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1296:  /* floatv8siv8sf2 */
    case 1331:  /* avx_unpckhps256 */
    case 1333:  /* avx_unpcklps256 */
    case 1335:  /* avx_movshdup256 */
    case 1337:  /* avx_movsldup256 */
    case 1339:  /* avx_shufps256_1 */
    case 1347:  /* avx2_vec_dupv8sf */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1597:  /* avx_movmskps256 */
    case 1667:  /* avx_blendps256 */
    case 1671:  /* avx_blendvps256 */
    case 1675:  /* avx_dpps256 */
    case 1716:  /* avx_vtestps256 */
    case 1722:  /* avx_roundps256 */
    case 1803:  /* xop_frczv8sf2 */
    case 1823:  /* xop_vpermil2v8sf3 */
    case 1850:  /* vec_dupv8si */
    case 1851:  /* vec_dupv8sf */
    case 1852:  /* vec_dupv4di */
    case 1853:  /* vec_dupv4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1888:  /* vec_set_lo_v8sf */
    case 1890:  /* vec_set_hi_v8sf */
    case 1897:  /* avx_maskloadps256 */
    case 1905:  /* avx_maskstoreps256 */
    case 1928:  /* avx_vec_concatv8sf */
    case 1931:  /* *vcvtph2ps_load */
    case 1932:  /* vcvtph2ps256 */
    case 1935:  /* vcvtps2ph256 */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2007:  /* *avx2_gatherdiv8sf_4 */
      return MODE_V8SF;

    case 20:  /* *cmpfp_xf */
    case 21:  /* *cmpfp_xf_cc */
    case 111:  /* swapxf */
    case 183:  /* *floathixf2_i387_with_temp */
    case 186:  /* *floathixf2_i387 */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 230:  /* *floatdixf2_i387_with_temp */
    case 233:  /* *floatsixf2_i387 */
    case 236:  /* *floatdixf2_i387 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 242:  /* *floatunssixf2_1 */
    case 455:  /* *absxf2_1 */
    case 456:  /* *negxf2_1 */
    case 459:  /* *absextendsfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 462:  /* *negextenddfxf2 */
    case 761:  /* *fop_xf_comm_i387 */
    case 762:  /* *fop_xf_1_i387 */
    case 775:  /* sqrtxf2 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 781:  /* fpremxf4_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 783:  /* *sinxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 789:  /* sincosxf3 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 792:  /* fptanxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 808:  /* *fscalexf4_i387 */
    case 811:  /* rintxf2 */
    case 821:  /* frndintxf2_floor */
    case 822:  /* frndintxf2_floor_i387 */
    case 832:  /* frndintxf2_ceil */
    case 833:  /* frndintxf2_ceil_i387 */
    case 843:  /* frndintxf2_trunc */
    case 844:  /* frndintxf2_trunc_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 846:  /* frndintxf2_mask_pm_i387 */
    case 849:  /* fxamxf2_i387 */
    case 894:  /* *movxfcc_1 */
      return MODE_XF;

    case 23:  /* *cmpfp_df */
    case 25:  /* *cmpfp_df_cc */
    case 113:  /* *swapdf */
    case 136:  /* *extendsfdf2_sse */
    case 147:  /* *truncxfdf2_mixed */
    case 149:  /* truncxfdf2_i387_noop */
    case 151:  /* *truncxfdf2_i387 */
    case 155:  /* fix_truncdfdi_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 182:  /* *floathidf2_i387_with_temp */
    case 185:  /* *floathidf2_i387 */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 200:  /* *floatdidf2_mixed_with_temp */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 212:  /* *floatsidf2_vector_sse */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 218:  /* *floatsidf2_sse_interunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 232:  /* *floatsidf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 241:  /* *floatunssidf2_1 */
    case 453:  /* *absdf2_1 */
    case 454:  /* *negdf2_1 */
    case 457:  /* *absextendsfdf2 */
    case 458:  /* *negextendsfdf2 */
    case 586:  /* setcc_df_sse */
    case 738:  /* *fop_df_comm_mixed */
    case 740:  /* *fop_df_comm_sse */
    case 742:  /* *fop_df_comm_i387 */
    case 744:  /* *fop_df_1_mixed */
    case 747:  /* *fop_df_1_sse */
    case 749:  /* *fop_df_1_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 772:  /* *fop_xf_6_i387 */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 780:  /* *sqrtdf2_sse */
    case 810:  /* sse4_1_rounddf2 */
    case 848:  /* fxamdf2_i387 */
    case 851:  /* fxamdf2_i387_with_temp */
    case 852:  /* movmsk_df */
    case 902:  /* smaxdf3 */
    case 903:  /* smindf3 */
    case 905:  /* *ieee_smindf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1140:  /* sse2_vmdivv2df3 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1210:  /* sse2_comi */
    case 1212:  /* sse2_ucomi */
    case 1238:  /* *fma_fmadd_df */
    case 1244:  /* *fma_fmsub_df */
    case 1250:  /* *fma_fnmadd_df */
    case 1256:  /* *fma_fnmsub_df */
    case 1305:  /* sse2_cvtsi2sd */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1323:  /* sse2_cvtss2sd */
    case 1660:  /* sse4a_movntdf */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1800:  /* xop_frczdf2 */
    case 2018:  /* loaddi_via_fpu */
      return MODE_DF;

    case 22:  /* *cmpfp_sf */
    case 24:  /* *cmpfp_sf_cc */
    case 112:  /* *swapsf */
    case 140:  /* *truncdfsf_fast_mixed */
    case 141:  /* *truncdfsf_fast_sse */
    case 142:  /* *truncdfsf_fast_i387 */
    case 143:  /* *truncdfsf_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 145:  /* *truncdfsf2_i387_1 */
    case 146:  /* *truncxfsf2_mixed */
    case 148:  /* truncxfsf2_i387_noop */
    case 150:  /* *truncxfsf2_i387 */
    case 154:  /* fix_truncsfdi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 181:  /* *floathisf2_i387_with_temp */
    case 184:  /* *floathisf2_i387 */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 211:  /* *floatsisf2_vector_sse */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 217:  /* *floatsisf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 231:  /* *floatsisf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 240:  /* *floatunssisf2_1 */
    case 451:  /* *abssf2_1 */
    case 452:  /* *negsf2_1 */
    case 585:  /* setcc_sf_sse */
    case 737:  /* *fop_sf_comm_mixed */
    case 739:  /* *fop_sf_comm_sse */
    case 741:  /* *fop_sf_comm_i387 */
    case 743:  /* *fop_sf_1_mixed */
    case 745:  /* *rcpsf2_sse */
    case 746:  /* *fop_sf_1_sse */
    case 748:  /* *fop_sf_1_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 760:  /* *fop_df_6_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 778:  /* *rsqrtsf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 809:  /* sse4_1_roundsf2 */
    case 847:  /* fxamsf2_i387 */
    case 850:  /* fxamsf2_i387_with_temp */
    case 900:  /* smaxsf3 */
    case 901:  /* sminsf3 */
    case 904:  /* *ieee_sminsf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1139:  /* sse_vmdivv4sf3 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1209:  /* sse_comi */
    case 1211:  /* sse_ucomi */
    case 1237:  /* *fma_fmadd_sf */
    case 1243:  /* *fma_fmsub_sf */
    case 1249:  /* *fma_fnmadd_sf */
    case 1255:  /* *fma_fnmsub_sf */
    case 1287:  /* sse_cvttps2pi */
    case 1288:  /* sse_cvtsi2ss */
    case 1289:  /* sse_cvtsi2ssq */
    case 1322:  /* sse2_cvtsd2ss */
    case 1346:  /* sse_movss */
    case 1659:  /* sse4a_movntsf */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1799:  /* xop_frczsf2 */
      return MODE_SF;

    case 59:  /* *movoi_internal_avx */
    case 1099:  /* avx_loaddqu256 */
    case 1101:  /* avx_storedqu256 */
    case 1103:  /* avx_lddqu256 */
    case 1111:  /* avx_movntv4di */
    case 1298:  /* avx_cvtps2dq256 */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1362:  /* vec_extract_hi_v4di */
    case 1366:  /* vec_extract_hi_v8si */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1377:  /* avx2_interleave_highv4di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1392:  /* *addv32qi3 */
    case 1393:  /* *subv32qi3 */
    case 1396:  /* *addv16hi3 */
    case 1397:  /* *subv16hi3 */
    case 1400:  /* *addv8si3 */
    case 1401:  /* *subv8si3 */
    case 1404:  /* *addv4di3 */
    case 1405:  /* *subv4di3 */
    case 1426:  /* *mulv16hi3 */
    case 1428:  /* *smulv16hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1436:  /* *avx2_pmaddwd */
    case 1438:  /* *avx2_mulv8si3 */
    case 1443:  /* ashrv16hi3 */
    case 1445:  /* ashrv8si3 */
    case 1447:  /* ashlv16hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1451:  /* ashlv8si3 */
    case 1452:  /* lshrv8si3 */
    case 1455:  /* ashlv4di3 */
    case 1456:  /* lshrv4di3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1539:  /* avx2_packsswb */
    case 1541:  /* avx2_packssdw */
    case 1543:  /* avx2_packuswb */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1553:  /* avx2_interleave_highv8si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1570:  /* avx2_pshufd_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1595:  /* avx2_psadbw */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1647:  /* avx2_palignrv2ti */
    case 1650:  /* absv32qi2 */
    case 1652:  /* absv16hi2 */
    case 1654:  /* absv8si2 */
    case 1679:  /* avx2_movntdqa */
    case 1681:  /* avx2_mpsadbw */
    case 1683:  /* avx2_packusdw */
    case 1685:  /* avx2_pblendvb */
    case 1688:  /* *avx2_pblendw */
    case 1689:  /* avx2_pblenddv8si */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1720:  /* avx_ptest256 */
    case 1834:  /* *avx_vzeroall */
    case 1835:  /* avx_vzeroupper */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1844:  /* avx2_permvarv8si */
    case 1845:  /* avx2_permv4df */
    case 1846:  /* avx2_permvarv8sf */
    case 1847:  /* avx2_permv4di_1 */
    case 1848:  /* avx2_permv2ti */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1858:  /* avx_vbroadcastf128_v32qi */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1883:  /* vec_set_lo_v4di */
    case 1885:  /* vec_set_hi_v4di */
    case 1887:  /* vec_set_lo_v8si */
    case 1889:  /* vec_set_hi_v8si */
    case 1891:  /* vec_set_lo_v16hi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1894:  /* vec_set_hi_v32qi */
    case 1901:  /* avx2_maskloadd256 */
    case 1902:  /* avx2_maskloadq256 */
    case 1909:  /* avx2_maskstored256 */
    case 1910:  /* avx2_maskstoreq256 */
    case 1914:  /* avx2_ashrvv8si */
    case 1916:  /* avx2_ashlvv8si */
    case 1917:  /* avx2_lshrvv8si */
    case 1920:  /* avx2_ashlvv4di */
    case 1921:  /* avx2_lshrvv4di */
    case 1924:  /* avx_vec_concatv32qi */
    case 1925:  /* avx_vec_concatv16hi */
    case 1926:  /* avx_vec_concatv8si */
    case 1927:  /* avx_vec_concatv4di */
    case 1940:  /* *avx2_gathersiv4di */
    case 1941:  /* *avx2_gathersiv4di */
    case 1948:  /* *avx2_gathersiv8si */
    case 1949:  /* *avx2_gathersiv8si */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
      return MODE_OI;

    case 41:  /* *pushti2 */
    case 1089:  /* sse2_movq128 */
    case 1100:  /* sse2_loaddqu */
    case 1102:  /* sse2_storedqu */
    case 1104:  /* sse3_lddqu */
    case 1112:  /* sse2_movntv2di */
    case 1299:  /* sse2_cvtps2dq */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1304:  /* sse2_cvttpd2pi */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1378:  /* vec_interleave_highv2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1394:  /* *addv16qi3 */
    case 1395:  /* *subv16qi3 */
    case 1398:  /* *addv8hi3 */
    case 1399:  /* *subv8hi3 */
    case 1402:  /* *addv4si3 */
    case 1403:  /* *subv4si3 */
    case 1406:  /* *addv2di3 */
    case 1407:  /* *subv2di3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1427:  /* *mulv8hi3 */
    case 1430:  /* *smulv8hi3_highpart */
    case 1431:  /* *umulv8hi3_highpart */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1444:  /* ashrv8hi3 */
    case 1446:  /* ashrv4si3 */
    case 1449:  /* ashlv8hi3 */
    case 1450:  /* lshrv8hi3 */
    case 1453:  /* ashlv4si3 */
    case 1454:  /* lshrv4si3 */
    case 1457:  /* ashlv2di3 */
    case 1458:  /* lshrv2di3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1479:  /* *smaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1485:  /* *umaxv16qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1535:  /* *andnottf3 */
    case 1536:  /* *andtf3 */
    case 1537:  /* *iortf3 */
    case 1538:  /* *xortf3 */
    case 1540:  /* sse2_packsswb */
    case 1542:  /* sse2_packssdw */
    case 1544:  /* sse2_packuswb */
    case 1546:  /* vec_interleave_highv16qi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1554:  /* vec_interleave_highv4si */
    case 1556:  /* vec_interleave_lowv4si */
    case 1557:  /* sse4_1_pinsrb */
    case 1558:  /* sse2_pinsrw */
    case 1559:  /* sse4_1_pinsrd */
    case 1560:  /* sse4_1_pinsrq */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1564:  /* *sse2_pextrw_si */
    case 1565:  /* *sse2_pextrw_di */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1567:  /* *sse4_1_pextrd */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1569:  /* *sse4_1_pextrq */
    case 1571:  /* sse2_pshufd_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1575:  /* sse2_pshufhw_1 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1596:  /* sse2_psadbw */
    case 1603:  /* *sse2_maskmovdqu */
    case 1604:  /* *sse2_maskmovdqu */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1648:  /* ssse3_palignrti */
    case 1651:  /* absv16qi2 */
    case 1653:  /* absv8hi2 */
    case 1655:  /* absv4si2 */
    case 1663:  /* sse4a_extrqi */
    case 1664:  /* sse4a_extrq */
    case 1665:  /* sse4a_insertqi */
    case 1666:  /* sse4a_insertq */
    case 1680:  /* sse4_1_movntdqa */
    case 1682:  /* sse4_1_mpsadbw */
    case 1684:  /* sse4_1_packusdw */
    case 1686:  /* sse4_1_pblendvb */
    case 1687:  /* sse4_1_pblendw */
    case 1690:  /* avx2_pblenddv4si */
    case 1691:  /* sse4_1_phminposuw */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1721:  /* sse4_1_ptest */
    case 1728:  /* sse4_2_pcmpestr */
    case 1729:  /* sse4_2_pcmpestri */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1731:  /* sse4_2_pcmpestr_cconly */
    case 1732:  /* sse4_2_pcmpistr */
    case 1733:  /* sse4_2_pcmpistri */
    case 1734:  /* sse4_2_pcmpistrm */
    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1736:  /* xop_pmacsww */
    case 1737:  /* xop_pmacssww */
    case 1738:  /* xop_pmacsdd */
    case 1739:  /* xop_pmacssdd */
    case 1740:  /* xop_pmacssdql */
    case 1741:  /* xop_pmacssdqh */
    case 1742:  /* xop_pmacsdql */
    case 1743:  /* xop_pmacsdqh */
    case 1744:  /* xop_pmacsswd */
    case 1745:  /* xop_pmacswd */
    case 1746:  /* xop_pmadcsswd */
    case 1747:  /* xop_pmadcswd */
    case 1775:  /* xop_pperm */
    case 1776:  /* xop_pperm_pack_v2di_v4si */
    case 1777:  /* xop_pperm_pack_v4si_v8hi */
    case 1778:  /* xop_pperm_pack_v8hi_v16qi */
    case 1779:  /* xop_rotlv16qi3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1786:  /* xop_rotrv2di3 */
    case 1787:  /* xop_vrotlv16qi3 */
    case 1788:  /* xop_vrotlv8hi3 */
    case 1789:  /* xop_vrotlv4si3 */
    case 1790:  /* xop_vrotlv2di3 */
    case 1791:  /* xop_shav16qi3 */
    case 1792:  /* xop_shav8hi3 */
    case 1793:  /* xop_shav4si3 */
    case 1794:  /* xop_shav2di3 */
    case 1795:  /* xop_shlv16qi3 */
    case 1796:  /* xop_shlv8hi3 */
    case 1797:  /* xop_shlv4si3 */
    case 1798:  /* xop_shlv2di3 */
    case 1807:  /* xop_maskcmpv16qi3 */
    case 1808:  /* xop_maskcmpv8hi3 */
    case 1809:  /* xop_maskcmpv4si3 */
    case 1810:  /* xop_maskcmpv2di3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1822:  /* xop_pcom_tfv2di3 */
    case 1827:  /* aesenc */
    case 1828:  /* aesenclast */
    case 1829:  /* aesdec */
    case 1830:  /* aesdeclast */
    case 1831:  /* aesimc */
    case 1832:  /* aeskeygenassist */
    case 1833:  /* pclmulqdq */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1899:  /* avx2_maskloadd */
    case 1900:  /* avx2_maskloadq */
    case 1907:  /* avx2_maskstored */
    case 1908:  /* avx2_maskstoreq */
    case 1915:  /* avx2_ashrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1919:  /* avx2_lshrvv4si */
    case 1922:  /* avx2_ashlvv2di */
    case 1923:  /* avx2_lshrvv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1937:  /* *avx2_gathersiv2di */
    case 1944:  /* *avx2_gathersiv4si */
    case 1945:  /* *avx2_gathersiv4si */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
      return MODE_TI;

    case 3:  /* *cmpdi_ccno_1 */
    case 7:  /* *cmpdi_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 40:  /* *pushdi2 */
    case 42:  /* *pushdi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 46:  /* *pushsi2_rex64 */
    case 50:  /* *pushdi2_prologue */
    case 52:  /* *popdi1 */
    case 54:  /* *popdi1_epilogue */
    case 58:  /* *movdi_or */
    case 70:  /* *movabsdi_1 */
    case 74:  /* *movabsdi_2 */
    case 76:  /* *swapdi */
    case 126:  /* *extendsidi2_rex64 */
    case 128:  /* extendqidi2 */
    case 129:  /* extendhidi2 */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 169:  /* *fix_truncdi_i387_1 */
    case 170:  /* fix_truncdi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 248:  /* *adddi3_cc */
    case 251:  /* *adddi_1 */
    case 259:  /* *adddi_2 */
    case 264:  /* *adddi_3 */
    case 266:  /* *adddi_4 */
    case 273:  /* *adddi_5 */
    case 287:  /* *subdi_1 */
    case 293:  /* *subdi_2 */
    case 298:  /* *subdi_3 */
    case 306:  /* *adddi3_carry */
    case 307:  /* *subdi3_carry */
    case 313:  /* *adddi3_cconly_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 320:  /* *muldi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 327:  /* *umulditi3_1 */
    case 329:  /* *mulditi3_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 339:  /* divmoddi4_1 */
    case 342:  /* *divmoddi4 */
    case 345:  /* *divmoddi4_noext */
    case 348:  /* udivmoddi4_1 */
    case 351:  /* *udivmoddi4 */
    case 354:  /* *udivmoddi4_noext */
    case 389:  /* *iordi_1 */
    case 390:  /* *xordi_1 */
    case 405:  /* *iordi_2 */
    case 406:  /* *xordi_2 */
    case 419:  /* *iordi_3 */
    case 420:  /* *xordi_3 */
    case 436:  /* *negdi2_1 */
    case 441:  /* *negdi2_cmpz */
    case 471:  /* *one_cmpldi2_1 */
    case 477:  /* *one_cmpldi2_2 */
    case 481:  /* x86_64_shld */
    case 484:  /* *ashldi3_mask */
    case 486:  /* *bmi2_ashldi3_1 */
    case 488:  /* *ashldi3_1 */
    case 497:  /* *ashldi3_cmp */
    case 502:  /* *ashldi3_cconly */
    case 505:  /* *lshrdi3_mask */
    case 506:  /* *ashrdi3_mask */
    case 511:  /* x86_64_shrd */
    case 513:  /* ashrdi3_cvt */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 523:  /* *ashrdi3_1 */
    case 540:  /* *lshrdi3_cmp */
    case 541:  /* *ashrdi3_cmp */
    case 550:  /* *lshrdi3_cconly */
    case 551:  /* *ashrdi3_cconly */
    case 554:  /* *rotldi3_mask */
    case 555:  /* *rotrdi3_mask */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 565:  /* *rotrdi3_1 */
    case 575:  /* *btsq */
    case 576:  /* *btrq */
    case 577:  /* *btcq */
    case 579:  /* *btdi */
    case 647:  /* set_got_rex64 */
    case 648:  /* set_rip_rex64 */
    case 649:  /* set_got_offset_rex64 */
    case 656:  /* *ffsdi_1 */
    case 659:  /* ctzdi2 */
    case 662:  /* clzdi2_lzcnt */
    case 664:  /* *bmi_andn_di */
    case 666:  /* bmi_bextr_di */
    case 668:  /* *bmi_blsi_di */
    case 670:  /* *bmi_blsmsk_di */
    case 672:  /* *bmi_blsr_di */
    case 674:  /* bmi2_bzhi_di3 */
    case 676:  /* bmi2_pdep_di3 */
    case 678:  /* bmi2_pext_di3 */
    case 680:  /* tbm_bextri_di */
    case 682:  /* *tbm_blcfill_di */
    case 684:  /* *tbm_blci_di */
    case 686:  /* *tbm_blcic_di */
    case 688:  /* *tbm_blcmsk_di */
    case 690:  /* *tbm_blcs_di */
    case 692:  /* *tbm_blsfill_di */
    case 694:  /* *tbm_blsic_di */
    case 696:  /* *tbm_t1mskc_di */
    case 698:  /* *tbm_tzmsk_di */
    case 699:  /* bsr_rex64 */
    case 704:  /* popcountdi2 */
    case 707:  /* *popcountdi2_cmp */
    case 710:  /* *bswapdi2_movbe */
    case 712:  /* *bswapdi2_1 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 812:  /* *fistdi2_1 */
    case 813:  /* fistdi2 */
    case 814:  /* fistdi2_with_temp */
    case 825:  /* *fistdi2_floor_1 */
    case 826:  /* fistdi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 836:  /* *fistdi2_ceil_1 */
    case 837:  /* fistdi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 854:  /* *strmovdi_rex_1 */
    case 861:  /* *rep_movdi_rex64 */
    case 866:  /* *strsetdi_rex_1 */
    case 873:  /* *rep_stosdi_rex64 */
    case 885:  /* *x86_movdicc_0_m1 */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 892:  /* *movdicc_noc */
    case 909:  /* pro_epilogue_adjust_stack_di_add */
    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 932:  /* sse4_2_crc32di */
    case 940:  /* *lwp_llwpcbdi1 */
    case 942:  /* lwp_slwpcbdi */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 959:  /* *movv8qi_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 969:  /* sse_movntq */
    case 993:  /* *vec_dupv2sf */
    case 994:  /* *mmx_concatv2sf */
    case 997:  /* *mmx_addv8qi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1013:  /* *mmx_mulv4hi3 */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1016:  /* *mmx_pmaddwd */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1048:  /* *mmx_xorv2si3 */
    case 1049:  /* mmx_packsswb */
    case 1050:  /* mmx_packssdw */
    case 1051:  /* mmx_packuswb */
    case 1052:  /* mmx_punpckhbw */
    case 1053:  /* mmx_punpcklbw */
    case 1054:  /* mmx_punpckhwd */
    case 1055:  /* mmx_punpcklwd */
    case 1056:  /* mmx_punpckhdq */
    case 1057:  /* mmx_punpckldq */
    case 1058:  /* *mmx_pinsrw */
    case 1059:  /* mmx_pextrw */
    case 1060:  /* mmx_pshufw_1 */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1062:  /* *vec_dupv4hi */
    case 1063:  /* *vec_dupv2si */
    case 1064:  /* *mmx_concatv2si */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1069:  /* mmx_psadbw */
    case 1070:  /* mmx_pmovmskb */
    case 1071:  /* *mmx_maskmovq */
    case 1072:  /* *mmx_maskmovq */
    case 1106:  /* sse2_movntidi */
    case 1286:  /* sse_cvtps2pi */
    case 1292:  /* sse_cvtss2siq */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1295:  /* sse_cvttss2siq */
    case 1303:  /* sse2_cvtpd2pi */
    case 1309:  /* sse2_cvtsd2siq */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1312:  /* sse2_cvttsd2siq */
    case 1601:  /* avx2_pmovmskb */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1631:  /* ssse3_pmaddubsw */
    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1649:  /* ssse3_palignrdi */
    case 1656:  /* absv8qi2 */
    case 1657:  /* absv4hi2 */
    case 1658:  /* absv2si2 */
    case 2019:  /* storedi_via_fpu */
      return MODE_DI;

    case 2:  /* *cmpsi_ccno_1 */
    case 6:  /* *cmpsi_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 29:  /* *cmpfp_si */
    case 31:  /* *cmpfp_si_cc */
    case 32:  /* x86_fnstsw_1 */
    case 33:  /* x86_sahf_1 */
    case 43:  /* *pushsi2 */
    case 47:  /* *pushqi2 */
    case 48:  /* *pushhi2 */
    case 49:  /* *pushsi2_prologue */
    case 51:  /* *popsi1 */
    case 53:  /* *popsi1_epilogue */
    case 55:  /* *movsi_xor */
    case 56:  /* *movdi_xor */
    case 57:  /* *movsi_or */
    case 69:  /* *movabssi_1 */
    case 73:  /* *movabssi_2 */
    case 75:  /* *swapsi */
    case 77:  /* *swapqi_1 */
    case 78:  /* *swaphi_1 */
    case 85:  /* *movhi_extv_1 */
    case 86:  /* *movsi_extv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 90:  /* *movdi_extzv_1 */
    case 116:  /* zero_extendqidi2 */
    case 117:  /* zero_extendhidi2 */
    case 118:  /* zero_extendhisi2_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 121:  /* *zero_extendqisi2_and */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 130:  /* extendhisi2 */
    case 131:  /* *extendhisi2_zext */
    case 132:  /* extendqisi2 */
    case 133:  /* *extendqisi2_zext */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 168:  /* *fix_truncsi_i387_1 */
    case 173:  /* fix_truncsi_i387 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 243:  /* *leasi */
    case 247:  /* *addsi3_cc */
    case 250:  /* *addsi_1 */
    case 252:  /* addsi_1_zext */
    case 258:  /* *addsi_2 */
    case 260:  /* *addsi_2_zext */
    case 263:  /* *addsi_3 */
    case 265:  /* *addsi_3_zext */
    case 269:  /* *addsi_4 */
    case 272:  /* *addsi_5 */
    case 277:  /* *lea_general_1 */
    case 278:  /* *lea_general_2 */
    case 279:  /* *lea_general_3 */
    case 286:  /* *subsi_1 */
    case 288:  /* *subsi_1_zext */
    case 292:  /* *subsi_2 */
    case 294:  /* *subsi_2_zext */
    case 297:  /* *subsi_3 */
    case 299:  /* *subsi_3_zext */
    case 304:  /* *addsi3_carry */
    case 305:  /* *subsi3_carry */
    case 308:  /* *addsi3_carry_zext */
    case 309:  /* *subsi3_carry_zext */
    case 312:  /* *addsi3_cconly_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 319:  /* *mulsi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 326:  /* *umulsidi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 335:  /* *umulsi3_highpart_1 */
    case 336:  /* *smulsi3_highpart_zext */
    case 337:  /* *umulsi3_highpart_zext */
    case 338:  /* divmodsi4_1 */
    case 341:  /* *divmodsi4 */
    case 344:  /* *divmodsi4_noext */
    case 347:  /* udivmodsi4_1 */
    case 350:  /* *udivmodsi4 */
    case 353:  /* *udivmodsi4_noext */
    case 360:  /* *testsi_1 */
    case 368:  /* *andsi_1 */
    case 369:  /* *andsi_1_zext */
    case 377:  /* *andsi_2 */
    case 378:  /* *andsi_2_zext */
    case 387:  /* *iorsi_1 */
    case 388:  /* *xorsi_1 */
    case 393:  /* *iorsi_1_zext */
    case 394:  /* *xorsi_1_zext */
    case 395:  /* *iorsi_1_zext_imm */
    case 396:  /* *xorsi_1_zext_imm */
    case 403:  /* *iorsi_2 */
    case 404:  /* *xorsi_2 */
    case 407:  /* *iorsi_2_zext */
    case 408:  /* *xorsi_2_zext */
    case 409:  /* *iorsi_2_zext_imm */
    case 410:  /* *xorsi_2_zext_imm */
    case 417:  /* *iorsi_3 */
    case 418:  /* *xorsi_3 */
    case 435:  /* *negsi2_1 */
    case 437:  /* *negsi2_1_zext */
    case 440:  /* *negsi2_cmpz */
    case 442:  /* *negsi2_cmpz_zext */
    case 470:  /* *one_cmplsi2_1 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 476:  /* *one_cmplsi2_2 */
    case 478:  /* *one_cmplsi2_2_zext */
    case 482:  /* x86_shld */
    case 483:  /* *ashlsi3_mask */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 487:  /* *ashlsi3_1 */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 490:  /* *ashlsi3_1_zext */
    case 496:  /* *ashlsi3_cmp */
    case 498:  /* *ashlsi3_cmp_zext */
    case 501:  /* *ashlsi3_cconly */
    case 503:  /* *lshrsi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 512:  /* x86_shrd */
    case 514:  /* ashrsi3_cvt */
    case 515:  /* *ashrsi3_cvt_zext */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 527:  /* *ashrsi3_1_zext */
    case 538:  /* *lshrsi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 542:  /* *lshrsi3_cmp_zext */
    case 543:  /* *ashrsi3_cmp_zext */
    case 548:  /* *lshrsi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 552:  /* *rotlsi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 568:  /* *rotrsi3_1_zext */
    case 578:  /* *btsi */
    case 655:  /* *ffssi_1 */
    case 658:  /* ctzsi2 */
    case 661:  /* clzsi2_lzcnt */
    case 663:  /* *bmi_andn_si */
    case 665:  /* bmi_bextr_si */
    case 667:  /* *bmi_blsi_si */
    case 669:  /* *bmi_blsmsk_si */
    case 671:  /* *bmi_blsr_si */
    case 673:  /* bmi2_bzhi_si3 */
    case 675:  /* bmi2_pdep_si3 */
    case 677:  /* bmi2_pext_si3 */
    case 679:  /* tbm_bextri_si */
    case 681:  /* *tbm_blcfill_si */
    case 683:  /* *tbm_blci_si */
    case 685:  /* *tbm_blcic_si */
    case 687:  /* *tbm_blcmsk_si */
    case 689:  /* *tbm_blcs_si */
    case 691:  /* *tbm_blsfill_si */
    case 693:  /* *tbm_blsic_si */
    case 695:  /* *tbm_t1mskc_si */
    case 697:  /* *tbm_tzmsk_si */
    case 700:  /* bsr */
    case 703:  /* popcountsi2 */
    case 706:  /* *popcountsi2_cmp */
    case 708:  /* *popcountsi2_cmp_zext */
    case 709:  /* *bswapsi2_movbe */
    case 711:  /* *bswapsi2_1 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 752:  /* *fop_sf_2_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 756:  /* *fop_sf_3_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 816:  /* *fistsi2_1 */
    case 818:  /* fistsi2 */
    case 820:  /* fistsi2_with_temp */
    case 824:  /* *fistsi2_floor_1 */
    case 829:  /* fistsi2_floor */
    case 831:  /* fistsi2_floor_with_temp */
    case 835:  /* *fistsi2_ceil_1 */
    case 840:  /* fistsi2_ceil */
    case 842:  /* fistsi2_ceil_with_temp */
    case 855:  /* *strmovsi_1 */
    case 856:  /* *strmovsi_1 */
    case 862:  /* *rep_movsi */
    case 863:  /* *rep_movsi */
    case 867:  /* *strsetsi_1 */
    case 868:  /* *strsetsi_1 */
    case 874:  /* *rep_stossi */
    case 875:  /* *rep_stossi */
    case 884:  /* *x86_movsicc_0_m1 */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 891:  /* *movsicc_noc */
    case 908:  /* pro_epilogue_adjust_stack_si_add */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 929:  /* sse4_2_crc32qi */
    case 930:  /* sse4_2_crc32hi */
    case 931:  /* sse4_2_crc32si */
    case 939:  /* *lwp_llwpcbsi1 */
    case 941:  /* lwp_slwpcbsi */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 1105:  /* sse2_movntisi */
    case 1290:  /* sse_cvtss2si */
    case 1291:  /* sse_cvtss2si_2 */
    case 1294:  /* sse_cvttss2si */
    case 1307:  /* sse2_cvtsd2si */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1311:  /* sse2_cvttsd2si */
    case 1602:  /* sse2_pmovmskb */
      return MODE_SI;

    case 1:  /* *cmphi_ccno_1 */
    case 5:  /* *cmphi_1 */
    case 9:  /* *cmphi_minus_1 */
    case 28:  /* *cmpfp_hi */
    case 30:  /* *cmpfp_hi_cc */
    case 68:  /* *movabshi_1 */
    case 72:  /* *movabshi_2 */
    case 80:  /* *swaphi_2 */
    case 82:  /* *movstricthi_1 */
    case 84:  /* *movstricthi_xor */
    case 120:  /* *zero_extendqihi2_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 134:  /* extendqihi2 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 167:  /* *fix_trunchi_i387_1 */
    case 172:  /* fix_trunchi_i387 */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 176:  /* x86_fnstcw_1 */
    case 177:  /* x86_fldcw_1 */
    case 257:  /* *addhi_2 */
    case 262:  /* *addhi_3 */
    case 268:  /* *addhi_4 */
    case 271:  /* *addhi_5 */
    case 285:  /* *subhi_1 */
    case 291:  /* *subhi_2 */
    case 296:  /* *subhi_3 */
    case 302:  /* *addhi3_carry */
    case 303:  /* *subhi3_carry */
    case 311:  /* *addhi3_cconly_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 322:  /* *mulhi3_1 */
    case 340:  /* *divmodhi4 */
    case 343:  /* *divmodhi4_noext */
    case 349:  /* *udivmodhi4 */
    case 352:  /* *udivmodhi4_noext */
    case 359:  /* *testhi_1 */
    case 376:  /* *andhi_2 */
    case 385:  /* *iorhi_1 */
    case 386:  /* *xorhi_1 */
    case 401:  /* *iorhi_2 */
    case 402:  /* *xorhi_2 */
    case 415:  /* *iorhi_3 */
    case 416:  /* *xorhi_3 */
    case 434:  /* *neghi2_1 */
    case 439:  /* *neghi2_cmpz */
    case 469:  /* *one_cmplhi2_1 */
    case 475:  /* *one_cmplhi2_2 */
    case 495:  /* *ashlhi3_cmp */
    case 500:  /* *ashlhi3_cconly */
    case 530:  /* *lshrhi3_1 */
    case 531:  /* *ashrhi3_1 */
    case 536:  /* *lshrhi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 546:  /* *lshrhi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 571:  /* *rotlhi3_1 */
    case 572:  /* *rotrhi3_1 */
    case 657:  /* ctzhi2 */
    case 660:  /* clzhi2_lzcnt */
    case 701:  /* *bsrhi */
    case 702:  /* popcounthi2 */
    case 705:  /* *popcounthi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 717:  /* *parityhi2_cmp */
    case 750:  /* *fop_sf_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 815:  /* *fisthi2_1 */
    case 817:  /* fisthi2 */
    case 819:  /* fisthi2_with_temp */
    case 823:  /* *fisthi2_floor_1 */
    case 828:  /* fisthi2_floor */
    case 830:  /* fisthi2_floor_with_temp */
    case 834:  /* *fisthi2_ceil_1 */
    case 839:  /* fisthi2_ceil */
    case 841:  /* fisthi2_ceil_with_temp */
    case 857:  /* *strmovhi_1 */
    case 858:  /* *strmovhi_1 */
    case 869:  /* *strsethi_1 */
    case 870:  /* *strsethi_1 */
    case 890:  /* *movhicc_noc */
      return MODE_HI;

    case 0:  /* *cmpqi_ccno_1 */
    case 4:  /* *cmpqi_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 12:  /* *cmpqi_ext_1 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 14:  /* *cmpqi_ext_2 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 17:  /* *cmpqi_ext_4 */
    case 67:  /* *movabsqi_1 */
    case 71:  /* *movabsqi_2 */
    case 79:  /* *swapqi_2 */
    case 81:  /* *movstrictqi_1 */
    case 83:  /* *movstrictqi_xor */
    case 93:  /* *movsi_insv_1_rex64 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 95:  /* *movsi_insv_1 */
    case 96:  /* *movqi_insv_2 */
    case 249:  /* addqi3_cc */
    case 255:  /* *addqi_1_slp */
    case 256:  /* *addqi_2 */
    case 261:  /* *addqi_3 */
    case 267:  /* *addqi_4 */
    case 270:  /* *addqi_5 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 275:  /* addqi_ext_1 */
    case 276:  /* *addqi_ext_2 */
    case 284:  /* *subqi_1 */
    case 289:  /* *subqi_1_slp */
    case 290:  /* *subqi_2 */
    case 295:  /* *subqi_3 */
    case 300:  /* *addqi3_carry */
    case 301:  /* *subqi3_carry */
    case 310:  /* *addqi3_cconly_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 323:  /* *mulqi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 331:  /* *umulqihi3_1 */
    case 346:  /* divmodhiqi3 */
    case 355:  /* udivmodhiqi3 */
    case 358:  /* *testqi_1 */
    case 361:  /* *testqi_ext_0 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 363:  /* *testqi_ext_1 */
    case 364:  /* *testqi_ext_2 */
    case 372:  /* *andqi_1_slp */
    case 375:  /* *andqi_2 */
    case 379:  /* *andqi_2_slp */
    case 380:  /* andqi_ext_0 */
    case 381:  /* *andqi_ext_0_cc */
    case 382:  /* *andqi_ext_1_rex64 */
    case 383:  /* *andqi_ext_1 */
    case 384:  /* *andqi_ext_2 */
    case 397:  /* *iorqi_1_slp */
    case 398:  /* *xorqi_1_slp */
    case 399:  /* *iorqi_2 */
    case 400:  /* *xorqi_2 */
    case 411:  /* *iorqi_2_slp */
    case 412:  /* *xorqi_2_slp */
    case 413:  /* *iorqi_3 */
    case 414:  /* *xorqi_3 */
    case 421:  /* *iorqi_ext_0 */
    case 422:  /* *xorqi_ext_0 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 425:  /* *iorqi_ext_1 */
    case 426:  /* *xorqi_ext_1 */
    case 427:  /* *iorqi_ext_2 */
    case 428:  /* *xorqi_ext_2 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 430:  /* *xorqi_cc_ext_1 */
    case 433:  /* *negqi2_1 */
    case 438:  /* *negqi2_cmpz */
    case 474:  /* *one_cmplqi2_2 */
    case 493:  /* *ashlqi3_1_slp */
    case 494:  /* *ashlqi3_cmp */
    case 499:  /* *ashlqi3_cconly */
    case 528:  /* *lshrqi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 532:  /* *lshrqi3_1_slp */
    case 533:  /* *ashrqi3_1_slp */
    case 534:  /* *lshrqi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 544:  /* *lshrqi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 569:  /* *rotlqi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 573:  /* *rotlqi3_1_slp */
    case 574:  /* *rotrqi3_1_slp */
    case 583:  /* *setcc_qi */
    case 584:  /* *setcc_qi_slp */
    case 859:  /* *strmovqi_1 */
    case 860:  /* *strmovqi_1 */
    case 864:  /* *rep_movqi */
    case 865:  /* *rep_movqi */
    case 871:  /* *strsetqi_1 */
    case 872:  /* *strsetqi_1 */
    case 876:  /* *rep_stosqi */
    case 877:  /* *rep_stosqi */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 882:  /* *strlenqi_1 */
    case 883:  /* *strlenqi_1 */
    case 893:  /* *movqicc_noc */
      return MODE_QI;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return MODE_UNKNOWN;

    }
}

enum attr_pent_pair
get_attr_pent_pair (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (!((1 << which_alternative) & 0xf))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (!((1 << which_alternative) & 0x3))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (which_alternative == 10)
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (!((1 << which_alternative) & 0x3f))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (!((1 << which_alternative) & 0xf))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (which_alternative == 9)
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (!((1 << which_alternative) & 0x3f))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU)))) || ((which_alternative != 0) || (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
      extract_insn_cached (insn);
      if (constant_call_address_operand (operands[1], VOIDmode))
        {
	  return PENT_PAIR_PV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
      extract_insn_cached (insn);
      if (constant_call_address_operand (operands[0], VOIDmode))
        {
	  return PENT_PAIR_PV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
      extract_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (const1_operand (operands[1], VOIDmode))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if ((which_alternative == 0) && (const1_operand (operands[2], VOIDmode)))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
      extract_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (const1_operand (operands[2], VOIDmode))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
      extract_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (const_int_operand (operands[1], VOIDmode))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if ((which_alternative == 0) && (const_int_operand (operands[2], VOIDmode)))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if ((which_alternative == 1) && (const_int_operand (operands[2], VOIDmode)))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))
        {
	  return PENT_PAIR_UV;
        }
      else if (const_int_operand (operands[2], VOIDmode))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 500:  /* *ashlhi3_cconly */
    case 495:  /* *ashlhi3_cmp */
      extract_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || ((cached_type == TYPE_ISHIFT) && (const_int_operand (operands[2], VOIDmode))))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 494:  /* *ashlqi3_cmp */
      extract_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if ((cached_type = get_attr_type (insn)) == TYPE_ALU)
        {
	  return PENT_PAIR_UV;
        }
      else if ((cached_type == TYPE_ISHIFT) && (const_int_operand (operands[2], VOIDmode)))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 493:  /* *ashlqi3_1_slp */
      extract_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if ((cached_type = get_attr_type (insn)) == TYPE_ALU)
        {
	  return PENT_PAIR_UV;
        }
      else if ((cached_type == TYPE_ISHIFT1) && (const_int_operand (operands[1], VOIDmode)))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 2))
        {
	  return PENT_PAIR_UV;
        }
      else if ((cached_type == TYPE_ISHIFT) && (const_int_operand (operands[2], VOIDmode)))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if ((which_alternative != 0) || ((cached_type = get_attr_type (insn)) == TYPE_ALU))
        {
	  if (which_alternative == 0)
	    {
	      return PENT_PAIR_PU;
	    }
	  else
	    {
	      return PENT_PAIR_UV;
	    }
        }
      else if ((cached_type == TYPE_ISHIFT) && (const_int_operand (operands[2], VOIDmode)))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((which_alternative == 0) && ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))) || (which_alternative == 1))
        {
	  return PENT_PAIR_UV;
        }
      else if (((which_alternative == 0) && ((! (
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD))) || (! (const1_operand (operands[2], VOIDmode))))) && (const_int_operand (operands[2], VOIDmode)))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((cached_type = get_attr_type (insn)) == TYPE_ALU) || (which_alternative == 1))
        {
	  return PENT_PAIR_UV;
        }
      else if ((cached_type == TYPE_ISHIFT) && (const_int_operand (operands[2], VOIDmode)))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 481:  /* x86_64_shld */
      extract_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (const_int_operand (operands[2], VOIDmode))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 370:  /* *andhi_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 967:  /* *movv2sf_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((1 << which_alternative) & 0x3))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((which_alternative != 5) && (! (incdec_operand (operands[2], QImode)))) || ((which_alternative == 5) || ((which_alternative != 5) && (incdec_operand (operands[2], QImode)))))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((which_alternative != 3) && (! (incdec_operand (operands[2], HImode)))) || ((which_alternative == 3) || ((which_alternative != 3) && (incdec_operand (operands[2], HImode)))))
        {
	  if (((1 << which_alternative) & 0x7))
	    {
	      return PENT_PAIR_PU;
	    }
	  else
	    {
	      return PENT_PAIR_UV;
	    }
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((which_alternative != 2) && (! (incdec_operand (operands[2], SImode)))) || ((which_alternative == 2) || ((which_alternative != 2) && (incdec_operand (operands[2], SImode)))))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 251:  /* *adddi_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((which_alternative != 3) && (! (incdec_operand (operands[2], DImode)))) || ((which_alternative == 3) || ((which_alternative != 3) && (incdec_operand (operands[2], DImode)))))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((which_alternative != 3) && (! (incdec_operand (operands[2], SImode)))) || ((which_alternative == 3) || ((which_alternative != 3) && (incdec_operand (operands[2], SImode)))))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 123:  /* *zero_extendqisi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (which_alternative != 0)
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (which_alternative != 0)
        {
	  return PENT_PAIR_PU;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (which_alternative == 1)
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((1 << which_alternative) & 0x78))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (get_attr_type (insn) == TYPE_IMOV)
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if ((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (((which_alternative != 5) || (aligned_operand (operands[1], QImode))) && (((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))))) || ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 3) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH)))) && ((!((1 << which_alternative) & 0x28)) && ((! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) || (which_alternative != 2)))))))
        {
	  if (get_attr_type (insn) == TYPE_IMOVX)
	    {
	      return PENT_PAIR_PU;
	    }
	  else
	    {
	      return PENT_PAIR_UV;
	    }
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (get_attr_type (insn) == TYPE_IMOV)
        {
	  if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) || (get_attr_mode (insn) == MODE_HI))
	    {
	      return PENT_PAIR_PU;
	    }
	  else
	    {
	      return PENT_PAIR_UV;
	    }
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (!((1 << which_alternative) & 0xffc))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if (!((1 << which_alternative) & 0x1fff0))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[0], VOIDmode)))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
      extract_insn_cached (insn);
      if (! (memory_operand (operands[1], VOIDmode)))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (! (memory_operand (operands[1], VOIDmode))))
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 475:  /* *one_cmplhi2_2 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 376:  /* *andhi_2 */
    case 315:  /* *addhi3_cc_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 296:  /* *subhi_3 */
    case 291:  /* *subhi_2 */
    case 285:  /* *subhi_1 */
    case 271:  /* *addhi_5 */
    case 268:  /* *addhi_4 */
    case 262:  /* *addhi_3 */
    case 257:  /* *addhi_2 */
    case 120:  /* *zero_extendqihi2_and */
    case 84:  /* *movstricthi_xor */
    case 82:  /* *movstricthi_1 */
    case 72:  /* *movabshi_2 */
    case 68:  /* *movabshi_1 */
    case 9:  /* *cmphi_minus_1 */
    case 5:  /* *cmphi_1 */
    case 1:  /* *cmphi_ccno_1 */
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else
        {
	  return PENT_PAIR_PU;
        }

    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 649:  /* set_got_offset_rex64 */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 379:  /* *andqi_2_slp */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 372:  /* *andqi_1_slp */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 356:  /* *testdi_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 290:  /* *subqi_2 */
    case 289:  /* *subqi_1_slp */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 256:  /* *addqi_2 */
    case 255:  /* *addqi_1_slp */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 121:  /* *zero_extendqisi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 83:  /* *movstrictqi_xor */
    case 81:  /* *movstrictqi_1 */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 71:  /* *movabsqi_2 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 67:  /* *movabsqi_1 */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else
        {
	  return PENT_PAIR_UV;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_insn_cached (insn);
      if (get_attr_imm_disp (insn) == IMM_DISP_TRUE)
        {
	  return PENT_PAIR_NP;
        }
      else if ((((cached_type = get_attr_type (insn)) == TYPE_ALU1) || ((cached_type == TYPE_ALU) || ((cached_type == TYPE_IMOV) || ((cached_type == TYPE_ICMP) || ((cached_type == TYPE_TEST) || ((cached_type == TYPE_LEA) || (cached_type == TYPE_INCDEC))))))) || (((cached_type == TYPE_POP) || (cached_type == TYPE_PUSH)) && (! (get_attr_memory (insn) == MEMORY_BOTH))))
        {
	  if (get_attr_pent_prefix (insn) == PENT_PREFIX_TRUE)
	    {
	      return PENT_PAIR_PU;
	    }
	  else
	    {
	      return PENT_PAIR_UV;
	    }
        }
      else if (cached_type == TYPE_IBR)
        {
	  return PENT_PAIR_PV;
        }
      else if (((cached_type == TYPE_ISHIFT) && (const_int_operand (operands[2], VOIDmode))) || (((cached_type == TYPE_ROTATE) && (const1_operand (operands[2], VOIDmode))) || (((cached_type == TYPE_ISHIFT1) && (const_int_operand (operands[1], VOIDmode))) || ((cached_type == TYPE_ROTATE1) && (const1_operand (operands[1], VOIDmode))))))
        {
	  return PENT_PAIR_PU;
        }
      else if (((cached_type == TYPE_CALL) && (constant_call_address_operand (operands[0], VOIDmode))) || ((cached_type == TYPE_CALLV) && (constant_call_address_operand (operands[1], VOIDmode))))
        {
	  return PENT_PAIR_PV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 357:  /* *testqi_1_maybe_si */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PENT_PAIR_UV;
        }
      else if (which_alternative == 1)
        {
	  return PENT_PAIR_NP;
        }
      else if (which_alternative == 2)
        {
	  return PENT_PAIR_UV;
        }
      else
        {
	  return PENT_PAIR_NP;
        }

    case 358:  /* *testqi_1 */
    case 359:  /* *testhi_1 */
    case 360:  /* *testsi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PENT_PAIR_UV;
        }
      else if (which_alternative == 1)
        {
	  return PENT_PAIR_NP;
        }
      else
        {
	  return PENT_PAIR_UV;
        }

    case 641:  /* simple_return_indirect_internal */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
      return PENT_PAIR_PV;

    case 300:  /* *addqi3_carry */
    case 301:  /* *subqi3_carry */
    case 302:  /* *addhi3_carry */
    case 303:  /* *subhi3_carry */
    case 304:  /* *addsi3_carry */
    case 305:  /* *subsi3_carry */
    case 306:  /* *adddi3_carry */
    case 307:  /* *subdi3_carry */
    case 308:  /* *addsi3_carry_zext */
    case 309:  /* *subsi3_carry_zext */
    case 884:  /* *x86_movsicc_0_m1 */
    case 885:  /* *x86_movdicc_0_m1 */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 889:  /* *x86_movdicc_0_m1_neg */
      return PENT_PAIR_PU;

    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
      return PENT_PAIR_UV;

    default:
      return PENT_PAIR_NP;

    }
}

enum attr_pent_prefix
get_attr_pent_prefix (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xf)) || ((((which_alternative == 0) && (! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))) || (which_alternative == 2)) || (which_alternative == 3)))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 1388:  /* sse2_loadlpd */
    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x600))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3f))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xc00))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x6000)) || ((((which_alternative == 8) && (! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))) || (which_alternative == 6)) || ((((1 << which_alternative) & 0x30)) || ((which_alternative == 7) && (! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))))))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 747:  /* *fop_df_1_sse */
    case 746:  /* *fop_sf_1_sse */
    case 744:  /* *fop_df_1_mixed */
    case 743:  /* *fop_sf_1_mixed */
      if (get_attr_unit (insn) == UNIT_SSE)
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 738:  /* *fop_df_comm_mixed */
    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 1385:  /* sse2_storelpd */
    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 713:  /* *bswaphi_lowpart_1 */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
      extract_constrain_insn_cached (insn);
      if ((((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_K6)))) || (which_alternative != 0))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 367:  /* *anddi_1 */
    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1f)) || (((which_alternative == 5) && ((
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR)) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))))) || ((!((1 << which_alternative) & 0xce1f)) && (((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))))))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1f)) || (((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x880)) && (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (((!((1 << which_alternative) & 0x880)) || (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((((1 << which_alternative) & 0x220)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))))) || ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || ((((1 << which_alternative) & 0x220)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR)))))))))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x7f)) || ((((which_alternative == 9) && (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))) || (((which_alternative != 9) || (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) && ((((which_alternative == 7) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))))) || ((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || ((which_alternative == 7) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))))))) || (((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || (((which_alternative == 9) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x1bff))))))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x7)) || (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))))))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 1358:  /* *sse4_1_extractps */
    case 491:  /* *ashlhi3_1 */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 66:  /* *movqi_internal */
      if (get_attr_type (insn) == TYPE_IMOVX)
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) || (get_attr_mode (insn) == MODE_HI))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xffc)) || ((((((1 << which_alternative) & 0xf80)) && ((!((1 << which_alternative) & 0xcc)) && ((!((1 << which_alternative) & 0xf00)) || (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))) || ((((1 << which_alternative) & 0xc0)) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))) || ((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1fff0))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 967:  /* *movv2sf_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
    case 368:  /* *andsi_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
    case 135:  /* *extendsfdf2_mixed */
    case 114:  /* *zero_extendsidi2_rex64 */
    case 63:  /* *movdi_internal */
    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) || ((which_alternative == 1) && (! (GET_MODE (operands[1]) == SFmode))))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if ((get_attr_prefix_0f (insn) == 1) || ((get_attr_prefix_data16 (insn) == 1) || (get_attr_prefix_rep (insn) == 1)))
        {
	  return PENT_PREFIX_TRUE;
        }
      else
        {
	  return PENT_PREFIX_FALSE;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1579:  /* *sse2_storeq_rex64 */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 909:  /* pro_epilogue_adjust_stack_di_add */
    case 908:  /* pro_epilogue_adjust_stack_si_add */
    case 896:  /* *movdfcc_1 */
    case 894:  /* *movxfcc_1 */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 842:  /* fistsi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 831:  /* fistsi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
    case 820:  /* fistsi2_with_temp */
    case 818:  /* fistsi2 */
    case 816:  /* *fistsi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
    case 761:  /* *fop_xf_comm_i387 */
    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 756:  /* *fop_sf_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 749:  /* *fop_df_1_i387 */
    case 748:  /* *fop_sf_1_i387 */
    case 742:  /* *fop_df_comm_i387 */
    case 741:  /* *fop_sf_comm_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 650:  /* eh_return_internal */
    case 649:  /* set_got_offset_rex64 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 641:  /* simple_return_indirect_internal */
    case 640:  /* simple_return_pop_internal */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 492:  /* *ashlqi3_1 */
    case 490:  /* *ashlsi3_1_zext */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 433:  /* *negqi2_1 */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 379:  /* *andqi_2_slp */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 372:  /* *andqi_1_slp */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 323:  /* *mulqi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 290:  /* *subqi_2 */
    case 289:  /* *subqi_1_slp */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 284:  /* *subqi_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 256:  /* *addqi_2 */
    case 255:  /* *addqi_1_slp */
    case 254:  /* *addqi_1 */
    case 252:  /* addsi_1_zext */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 145:  /* *truncdfsf2_i387_1 */
    case 144:  /* *truncdfsf_i387 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
    case 127:  /* extendsidi2_1 */
    case 126:  /* *extendsidi2_rex64 */
    case 121:  /* *zero_extendqisi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 83:  /* *movstrictqi_xor */
    case 81:  /* *movstrictqi_1 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 71:  /* *movabsqi_2 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 67:  /* *movabsqi_1 */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
    case 42:  /* *pushdi2_rex64 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 29:  /* *cmpfp_si */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      return PENT_PREFIX_FALSE;

    default:
      return PENT_PREFIX_TRUE;

    }
}

int
get_attr_prefix_vex_w (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return 0;

    }
}

enum attr_prefix
get_attr_prefix (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 3)
        {
	  return PREFIX_VEX;
        }
      else if (((1 << which_alternative) & 0x30))
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_VEX;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_VEX;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return PREFIX_MAYBE_VEX;
        }
      else if (((1 << which_alternative) & 0x30))
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 6)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 7)
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_VEX;
        }

    case 1588:  /* *vec_concatv4si */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_VEX;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_VEX;
        }

    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return PREFIX_MAYBE_VEX;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_VEX;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 2)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else if (which_alternative == 3)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 4)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 5)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 2)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 3)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 4)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else if (((1 << which_alternative) & 0x60))
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 7)
        {
	  return PREFIX_VEX;
        }
      else
        {
	  return PREFIX_MAYBE_VEX;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 2)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 3)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 4)
        {
	  return PREFIX_VEX;
        }
      else if (((1 << which_alternative) & 0x60))
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 7)
        {
	  return PREFIX_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 2)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 3)
        {
	  return PREFIX_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 2)
        {
	  return PREFIX_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return PREFIX_MAYBE_VEX;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 5)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 6)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 7)
        {
	  return PREFIX_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 2)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 3)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 4)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return PREFIX_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 744:  /* *fop_df_1_mixed */
    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_VEX;
        }

    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 738:  /* *fop_df_comm_mixed */
    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_VEX;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_MAYBE_VEX;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 135:  /* *extendsfdf2_mixed */
    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_MAYBE_VEX;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf800))
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1e0))
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3f))
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_MAYBE_VEX;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7f))
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_MAYBE_VEX;
        }

    case 115:  /* zero_extendsidi2_1 */
    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f))
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_MAYBE_VEX;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x31e0))
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 959:  /* *movv8qi_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f00))
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 963:  /* *movv8qi_internal */
    case 964:  /* *movv4hi_internal */
    case 965:  /* *movv2si_internal */
    case 966:  /* *movv1di_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1c0))
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3f00))
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3c0))
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 1329:  /* sse_movhlps */
    case 1330:  /* sse_movlhps */
    case 1343:  /* sse_loadhps */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 2)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 3)
        {
	  return PREFIX_VEX;
        }
      else
        {
	  return PREFIX_MAYBE_VEX;
        }

    case 1352:  /* *vec_concatv4sf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 2)
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_VEX;
        }

    case 1373:  /* *vec_interleave_highv2df */
    case 1375:  /* *vec_interleave_lowv2df */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 2)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else if (which_alternative == 3)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 4)
        {
	  return PREFIX_VEX;
        }
      else
        {
	  return PREFIX_MAYBE_VEX;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 34:  /* *cmpfp_i_mixed */
    case 37:  /* *cmpfp_iu_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 1390:  /* vec_dupv2df */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_MAYBE_VEX;
        }

    case 1582:  /* *vec_extractv2di_1 */
    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 2)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 3)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 1583:  /* *vec_dupv4si */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_ORIG;
        }
      else if (which_alternative == 1)
        {
	  return PREFIX_VEX;
        }
      else if (which_alternative == 2)
        {
	  return PREFIX_MAYBE_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 326:  /* *umulsidi3_1 */
    case 327:  /* *umulditi3_1 */
    case 585:  /* setcc_sf_sse */
    case 586:  /* setcc_df_sse */
    case 739:  /* *fop_sf_comm_sse */
    case 740:  /* *fop_df_comm_sse */
    case 746:  /* *fop_sf_1_sse */
    case 747:  /* *fop_df_1_sse */
    case 900:  /* smaxsf3 */
    case 901:  /* sminsf3 */
    case 902:  /* smaxdf3 */
    case 903:  /* smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 905:  /* *ieee_smindf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 1117:  /* *addv8sf3 */
    case 1118:  /* *subv8sf3 */
    case 1119:  /* *addv4sf3 */
    case 1120:  /* *subv4sf3 */
    case 1121:  /* *addv4df3 */
    case 1122:  /* *subv4df3 */
    case 1123:  /* *addv2df3 */
    case 1124:  /* *subv2df3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1129:  /* *mulv8sf3 */
    case 1130:  /* *mulv4sf3 */
    case 1131:  /* *mulv4df3 */
    case 1132:  /* *mulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1135:  /* avx_divv8sf3 */
    case 1136:  /* sse_divv4sf3 */
    case 1137:  /* avx_divv4df3 */
    case 1138:  /* sse2_divv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
    case 1140:  /* sse2_vmdivv2df3 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1153:  /* *smaxv8sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1160:  /* *sminv2df3_finite */
    case 1161:  /* *smaxv8sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1164:  /* *sminv4sf3 */
    case 1165:  /* *smaxv4df3 */
    case 1166:  /* *sminv4df3 */
    case 1167:  /* *smaxv2df3 */
    case 1168:  /* *sminv2df3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1192:  /* sse3_hsubv4sf3 */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1213:  /* avx_andnotv8sf3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1217:  /* *andv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1220:  /* *andv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1222:  /* *xorv4sf3 */
    case 1223:  /* *andv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1225:  /* *xorv4df3 */
    case 1226:  /* *andv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1228:  /* *xorv2df3 */
    case 1229:  /* *andnotsf3 */
    case 1230:  /* *andnotdf3 */
    case 1231:  /* *andsf3 */
    case 1232:  /* *iorsf3 */
    case 1233:  /* *xorsf3 */
    case 1234:  /* *anddf3 */
    case 1235:  /* *iordf3 */
    case 1236:  /* *xordf3 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1340:  /* sse_shufps_v4si */
    case 1341:  /* sse_shufps_v4sf */
    case 1346:  /* sse_movss */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1356:  /* sse4_1_insertps */
    case 1378:  /* vec_interleave_highv2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1381:  /* sse2_shufpd_v2di */
    case 1382:  /* sse2_shufpd_v2df */
    case 1392:  /* *addv32qi3 */
    case 1393:  /* *subv32qi3 */
    case 1394:  /* *addv16qi3 */
    case 1395:  /* *subv16qi3 */
    case 1396:  /* *addv16hi3 */
    case 1397:  /* *subv16hi3 */
    case 1398:  /* *addv8hi3 */
    case 1399:  /* *subv8hi3 */
    case 1400:  /* *addv8si3 */
    case 1401:  /* *subv8si3 */
    case 1402:  /* *addv4si3 */
    case 1403:  /* *subv4si3 */
    case 1404:  /* *addv4di3 */
    case 1405:  /* *subv4di3 */
    case 1406:  /* *addv2di3 */
    case 1407:  /* *subv2di3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1426:  /* *mulv16hi3 */
    case 1427:  /* *mulv8hi3 */
    case 1428:  /* *smulv16hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1431:  /* *umulv8hi3_highpart */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1438:  /* *avx2_mulv8si3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1443:  /* ashrv16hi3 */
    case 1444:  /* ashrv8hi3 */
    case 1445:  /* ashrv8si3 */
    case 1446:  /* ashrv4si3 */
    case 1447:  /* ashlv16hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1450:  /* lshrv8hi3 */
    case 1451:  /* ashlv8si3 */
    case 1452:  /* lshrv8si3 */
    case 1453:  /* ashlv4si3 */
    case 1454:  /* lshrv4si3 */
    case 1455:  /* ashlv4di3 */
    case 1456:  /* lshrv4di3 */
    case 1457:  /* ashlv2di3 */
    case 1458:  /* lshrv2di3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1479:  /* *smaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1485:  /* *umaxv16qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1503:  /* *andnotv32qi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1506:  /* *andnotv8hi3 */
    case 1507:  /* *andnotv8si3 */
    case 1508:  /* *andnotv4si3 */
    case 1509:  /* *andnotv4di3 */
    case 1510:  /* *andnotv2di3 */
    case 1511:  /* *andv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1514:  /* *andv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1516:  /* *xorv16qi3 */
    case 1517:  /* *andv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1520:  /* *andv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1522:  /* *xorv8hi3 */
    case 1523:  /* *andv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1525:  /* *xorv8si3 */
    case 1526:  /* *andv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1528:  /* *xorv4si3 */
    case 1529:  /* *andv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1531:  /* *xorv4di3 */
    case 1532:  /* *andv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1534:  /* *xorv2di3 */
    case 1535:  /* *andnottf3 */
    case 1536:  /* *andtf3 */
    case 1537:  /* *iortf3 */
    case 1538:  /* *xortf3 */
    case 1539:  /* avx2_packsswb */
    case 1540:  /* sse2_packsswb */
    case 1541:  /* avx2_packssdw */
    case 1542:  /* sse2_packssdw */
    case 1543:  /* avx2_packuswb */
    case 1544:  /* sse2_packuswb */
    case 1546:  /* vec_interleave_highv16qi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1554:  /* vec_interleave_highv4si */
    case 1556:  /* vec_interleave_lowv4si */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1595:  /* avx2_psadbw */
    case 1596:  /* sse2_psadbw */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1647:  /* avx2_palignrv2ti */
    case 1648:  /* ssse3_palignrti */
    case 1667:  /* avx_blendps256 */
    case 1668:  /* sse4_1_blendps */
    case 1669:  /* avx_blendpd256 */
    case 1670:  /* sse4_1_blendpd */
    case 1671:  /* avx_blendvps256 */
    case 1672:  /* sse4_1_blendvps */
    case 1673:  /* avx_blendvpd256 */
    case 1674:  /* sse4_1_blendvpd */
    case 1675:  /* avx_dpps256 */
    case 1676:  /* sse4_1_dpps */
    case 1677:  /* avx_dppd256 */
    case 1678:  /* sse4_1_dppd */
    case 1681:  /* avx2_mpsadbw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1684:  /* sse4_1_packusdw */
    case 1685:  /* avx2_pblendvb */
    case 1686:  /* sse4_1_pblendvb */
    case 1687:  /* sse4_1_pblendw */
    case 1726:  /* sse4_1_roundss */
    case 1727:  /* sse4_1_roundsd */
    case 1827:  /* aesenc */
    case 1828:  /* aesenclast */
    case 1829:  /* aesdec */
    case 1830:  /* aesdeclast */
    case 1833:  /* pclmulqdq */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return PREFIX_ORIG;
        }
      else
        {
	  return PREFIX_VEX;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_mode = get_attr_mode (insn)) == MODE_OI) || ((cached_mode == MODE_V8SF) || (cached_mode == MODE_V4DF)))
        {
	  return PREFIX_VEX;
        }
      else
        {
	  return PREFIX_ORIG;
        }

    case 35:  /* *cmpfp_i_sse */
    case 38:  /* *cmpfp_iu_sse */
    case 61:  /* *movti_internal_sse */
    case 105:  /* *movtf_internal_sse2 */
    case 136:  /* *extendsfdf2_sse */
    case 141:  /* *truncdfsf_fast_sse */
    case 154:  /* fix_truncsfdi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 217:  /* *floatsisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 745:  /* *rcpsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 780:  /* *sqrtdf2_sse */
    case 809:  /* sse4_1_roundsf2 */
    case 810:  /* sse4_1_rounddf2 */
    case 852:  /* movmsk_df */
    case 1075:  /* *movv32qi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1078:  /* *movv8hi_internal */
    case 1079:  /* *movv8si_internal */
    case 1080:  /* *movv4si_internal */
    case 1081:  /* *movv4di_internal */
    case 1082:  /* *movv2di_internal */
    case 1083:  /* *movv2ti_internal */
    case 1084:  /* *movv1ti_internal */
    case 1085:  /* *movv8sf_internal */
    case 1086:  /* *movv4sf_internal */
    case 1087:  /* *movv4df_internal */
    case 1088:  /* *movv2df_internal */
    case 1089:  /* sse2_movq128 */
    case 1091:  /* avx_loadups256 */
    case 1092:  /* sse_loadups */
    case 1093:  /* avx_loadupd256 */
    case 1094:  /* sse2_loadupd */
    case 1095:  /* avx_storeups256 */
    case 1096:  /* sse_storeups */
    case 1097:  /* avx_storeupd256 */
    case 1098:  /* sse2_storeupd */
    case 1099:  /* avx_loaddqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1101:  /* avx_storedqu256 */
    case 1102:  /* sse2_storedqu */
    case 1103:  /* avx_lddqu256 */
    case 1104:  /* sse3_lddqu */
    case 1107:  /* avx_movntv8sf */
    case 1108:  /* sse_movntv4sf */
    case 1109:  /* avx_movntv4df */
    case 1110:  /* sse2_movntv2df */
    case 1111:  /* avx_movntv4di */
    case 1112:  /* sse2_movntv2di */
    case 1141:  /* avx_rcpv8sf2 */
    case 1142:  /* sse_rcpv4sf2 */
    case 1144:  /* avx_sqrtv8sf2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1146:  /* avx_sqrtv4df2 */
    case 1147:  /* sse2_sqrtv2df2 */
    case 1150:  /* avx_rsqrtv8sf2 */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1209:  /* sse_comi */
    case 1210:  /* sse2_comi */
    case 1211:  /* sse_ucomi */
    case 1212:  /* sse2_ucomi */
    case 1290:  /* sse_cvtss2si */
    case 1291:  /* sse_cvtss2si_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1294:  /* sse_cvttss2si */
    case 1295:  /* sse_cvttss2siq */
    case 1296:  /* floatv8siv8sf2 */
    case 1297:  /* floatv4siv4sf2 */
    case 1299:  /* sse2_cvtps2dq */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1307:  /* sse2_cvtsd2si */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1311:  /* sse2_cvttsd2si */
    case 1312:  /* sse2_cvttsd2siq */
    case 1315:  /* sse2_cvtdq2pd */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1328:  /* sse2_cvtps2pd */
    case 1336:  /* sse3_movshdup */
    case 1338:  /* sse3_movsldup */
    case 1342:  /* sse_storehps */
    case 1344:  /* sse_storelps */
    case 1385:  /* sse2_storelpd */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1564:  /* *sse2_pextrw_si */
    case 1565:  /* *sse2_pextrw_di */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1567:  /* *sse4_1_pextrd */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1569:  /* *sse4_1_pextrq */
    case 1571:  /* sse2_pshufd_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1575:  /* sse2_pshufhw_1 */
    case 1597:  /* avx_movmskps256 */
    case 1598:  /* sse_movmskps */
    case 1599:  /* avx_movmskpd256 */
    case 1600:  /* sse2_movmskpd */
    case 1602:  /* sse2_pmovmskb */
    case 1603:  /* *sse2_maskmovdqu */
    case 1604:  /* *sse2_maskmovdqu */
    case 1605:  /* sse_ldmxcsr */
    case 1606:  /* sse_stmxcsr */
    case 1650:  /* absv32qi2 */
    case 1651:  /* absv16qi2 */
    case 1652:  /* absv16hi2 */
    case 1653:  /* absv8hi2 */
    case 1654:  /* absv8si2 */
    case 1655:  /* absv4si2 */
    case 1679:  /* avx2_movntdqa */
    case 1680:  /* sse4_1_movntdqa */
    case 1691:  /* sse4_1_phminposuw */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1721:  /* sse4_1_ptest */
    case 1722:  /* avx_roundps256 */
    case 1723:  /* sse4_1_roundps */
    case 1724:  /* avx_roundpd256 */
    case 1725:  /* sse4_1_roundpd */
    case 1729:  /* sse4_2_pcmpestri */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1731:  /* sse4_2_pcmpestr_cconly */
    case 1733:  /* sse4_2_pcmpistri */
    case 1734:  /* sse4_2_pcmpistrm */
    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1831:  /* aesimc */
    case 1832:  /* aeskeygenassist */
      return PREFIX_MAYBE_VEX;

    case 1825:  /* xop_vpermil2v4df3 */
    case 1823:  /* xop_vpermil2v8sf3 */
    case 1804:  /* xop_frczv4df2 */
    case 1803:  /* xop_frczv8sf2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 59:  /* *movoi_internal_avx */
    case 324:  /* *bmi2_umulditi3_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 673:  /* bmi2_bzhi_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 678:  /* bmi2_pext_di3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1185:  /* avx_haddv4df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1193:  /* avx_cmpv8sf3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1298:  /* avx_cvtps2dq256 */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1313:  /* floatv4siv4df2 */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1326:  /* avx_cvtps2pd256 */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1331:  /* avx_unpckhps256 */
    case 1333:  /* avx_unpcklps256 */
    case 1335:  /* avx_movshdup256 */
    case 1337:  /* avx_movsldup256 */
    case 1339:  /* avx_shufps256_1 */
    case 1347:  /* avx2_vec_dupv8sf */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1362:  /* vec_extract_hi_v4di */
    case 1363:  /* vec_extract_hi_v4df */
    case 1366:  /* vec_extract_hi_v8si */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1372:  /* avx_unpckhpd256 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1376:  /* avx_shufpd256_1 */
    case 1377:  /* avx2_interleave_highv4di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1436:  /* *avx2_pmaddwd */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1553:  /* avx2_interleave_highv8si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1570:  /* avx2_pshufd_1 */
    case 1572:  /* avx2_pshuflw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1601:  /* avx2_pmovmskb */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1683:  /* avx2_packusdw */
    case 1688:  /* *avx2_pblendw */
    case 1689:  /* avx2_pblenddv8si */
    case 1690:  /* avx2_pblenddv4si */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1716:  /* avx_vtestps256 */
    case 1717:  /* avx_vtestps */
    case 1718:  /* avx_vtestpd256 */
    case 1719:  /* avx_vtestpd */
    case 1720:  /* avx_ptest256 */
    case 1834:  /* *avx_vzeroall */
    case 1835:  /* avx_vzeroupper */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1844:  /* avx2_permvarv8si */
    case 1845:  /* avx2_permv4df */
    case 1846:  /* avx2_permvarv8sf */
    case 1847:  /* avx2_permv4di_1 */
    case 1848:  /* avx2_permv2ti */
    case 1849:  /* avx2_vec_dupv4df */
    case 1850:  /* vec_dupv8si */
    case 1851:  /* vec_dupv8sf */
    case 1852:  /* vec_dupv4di */
    case 1853:  /* vec_dupv4df */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1858:  /* avx_vbroadcastf128_v32qi */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1869:  /* *avx_vpermilpv4df */
    case 1870:  /* *avx_vpermilpv2df */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1883:  /* vec_set_lo_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1886:  /* vec_set_hi_v4df */
    case 1887:  /* vec_set_lo_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1890:  /* vec_set_hi_v8sf */
    case 1891:  /* vec_set_lo_v16hi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1894:  /* vec_set_hi_v32qi */
    case 1895:  /* avx_maskloadps */
    case 1896:  /* avx_maskloadpd */
    case 1897:  /* avx_maskloadps256 */
    case 1898:  /* avx_maskloadpd256 */
    case 1899:  /* avx2_maskloadd */
    case 1900:  /* avx2_maskloadq */
    case 1901:  /* avx2_maskloadd256 */
    case 1902:  /* avx2_maskloadq256 */
    case 1903:  /* avx_maskstoreps */
    case 1904:  /* avx_maskstorepd */
    case 1905:  /* avx_maskstoreps256 */
    case 1906:  /* avx_maskstorepd256 */
    case 1907:  /* avx2_maskstored */
    case 1908:  /* avx2_maskstoreq */
    case 1909:  /* avx2_maskstored256 */
    case 1910:  /* avx2_maskstoreq256 */
    case 1914:  /* avx2_ashrvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1916:  /* avx2_ashlvv8si */
    case 1917:  /* avx2_lshrvv8si */
    case 1918:  /* avx2_ashlvv4si */
    case 1919:  /* avx2_lshrvv4si */
    case 1920:  /* avx2_ashlvv4di */
    case 1921:  /* avx2_lshrvv4di */
    case 1922:  /* avx2_ashlvv2di */
    case 1923:  /* avx2_lshrvv2di */
    case 1924:  /* avx_vec_concatv32qi */
    case 1925:  /* avx_vec_concatv16hi */
    case 1926:  /* avx_vec_concatv8si */
    case 1927:  /* avx_vec_concatv4di */
    case 1928:  /* avx_vec_concatv8sf */
    case 1929:  /* avx_vec_concatv4df */
    case 1930:  /* vcvtph2ps */
    case 1931:  /* *vcvtph2ps_load */
    case 1932:  /* vcvtph2ps256 */
    case 1933:  /* *vcvtps2ph */
    case 1934:  /* *vcvtps2ph_store */
    case 1935:  /* vcvtps2ph256 */
    case 1936:  /* *avx2_gathersiv2di */
    case 1937:  /* *avx2_gathersiv2di */
    case 1938:  /* *avx2_gathersiv2df */
    case 1939:  /* *avx2_gathersiv2df */
    case 1940:  /* *avx2_gathersiv4di */
    case 1941:  /* *avx2_gathersiv4di */
    case 1942:  /* *avx2_gathersiv4df */
    case 1943:  /* *avx2_gathersiv4df */
    case 1944:  /* *avx2_gathersiv4si */
    case 1945:  /* *avx2_gathersiv4si */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1948:  /* *avx2_gathersiv8si */
    case 1949:  /* *avx2_gathersiv8si */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2007:  /* *avx2_gatherdiv8sf_4 */
      return PREFIX_VEX;

    default:
      return PREFIX_ORIG;

    }
}

int
get_attr_prefix_extra (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1589:  /* *vec_concatv2di_rex64 */
    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1560:  /* sse4_1_pinsrq */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((V2DImode) == (V8HImode)))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1559:  /* sse4_1_pinsrd */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((V4SImode) == (V8HImode)))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1558:  /* sse2_pinsrw */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((V8HImode) == (V8HImode)))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1557:  /* sse4_1_pinsrb */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((V16QImode) == (V8HImode)))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1929:  /* avx_vec_concatv4df */
    case 1928:  /* avx_vec_concatv8sf */
    case 1927:  /* avx_vec_concatv4di */
    case 1926:  /* avx_vec_concatv8si */
    case 1925:  /* avx_vec_concatv16hi */
    case 1924:  /* avx_vec_concatv32qi */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3f))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0xc0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 0;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1583:  /* *vec_dupv4si */
    case 1349:  /* vec_dupv4sf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else if (which_alternative == 1)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1067:  /* *mmx_uavgv8qi3 */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 1590 "../../gcc/config/i386/mmx.md"
(TARGET_SSE))) && (! (
#line 1591 "../../gcc/config/i386/mmx.md"
(TARGET_3DNOW_A))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1826:  /* xop_vpermil2v2df3 */
    case 1825:  /* xop_vpermil2v4df3 */
    case 1824:  /* xop_vpermil2v4sf3 */
    case 1823:  /* xop_vpermil2v8sf3 */
    case 1778:  /* xop_pperm_pack_v8hi_v16qi */
    case 1777:  /* xop_pperm_pack_v4si_v8hi */
    case 1776:  /* xop_pperm_pack_v2di_v4si */
    case 1775:  /* xop_pperm */
    case 1759:  /* xop_pcmov_v2df */
    case 1758:  /* xop_pcmov_v4df256 */
    case 1757:  /* xop_pcmov_v4sf */
    case 1756:  /* xop_pcmov_v8sf256 */
    case 1755:  /* xop_pcmov_v2di */
    case 1754:  /* xop_pcmov_v4di256 */
    case 1753:  /* xop_pcmov_v4si */
    case 1752:  /* xop_pcmov_v8si256 */
    case 1751:  /* xop_pcmov_v8hi */
    case 1750:  /* xop_pcmov_v16hi256 */
    case 1749:  /* xop_pcmov_v16qi */
    case 1748:  /* xop_pcmov_v32qi256 */
    case 1747:  /* xop_pmadcswd */
    case 1746:  /* xop_pmadcsswd */
    case 1745:  /* xop_pmacswd */
    case 1744:  /* xop_pmacsswd */
    case 1743:  /* xop_pmacsdqh */
    case 1742:  /* xop_pmacsdql */
    case 1741:  /* xop_pmacssdqh */
    case 1740:  /* xop_pmacssdql */
    case 1739:  /* xop_pmacssdd */
    case 1738:  /* xop_pmacsdd */
    case 1737:  /* xop_pmacssww */
    case 1736:  /* xop_pmacsww */
    case 1284:  /* *fma4i_vmfnmsub_v2df */
    case 1283:  /* *fma4i_vmfnmsub_v4sf */
    case 1282:  /* *fma4i_vmfnmadd_v2df */
    case 1281:  /* *fma4i_vmfnmadd_v4sf */
    case 1280:  /* *fma4i_vmfmsub_v2df */
    case 1279:  /* *fma4i_vmfmsub_v4sf */
    case 1278:  /* *fma4i_vmfmadd_v2df */
    case 1277:  /* *fma4i_vmfmadd_v4sf */
    case 1276:  /* *fmai_fnmsub_v2df */
    case 1275:  /* *fmai_fnmsub_v4sf */
    case 1274:  /* *fmai_fnmadd_v2df */
    case 1273:  /* *fmai_fnmadd_v4sf */
    case 1272:  /* *fmai_fmsub_v2df */
    case 1271:  /* *fmai_fmsub_v4sf */
    case 1270:  /* *fmai_fmadd_v2df */
    case 1269:  /* *fmai_fmadd_v4sf */
    case 1268:  /* *fma_fmsubadd_v2df */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1266:  /* *fma_fmsubadd_v4sf */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1264:  /* *fma_fmaddsub_v2df */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1262:  /* *fma_fmaddsub_v4sf */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1258:  /* *fma_fnmsub_v2df */
    case 1257:  /* *fma_fnmsub_v4sf */
    case 1256:  /* *fma_fnmsub_df */
    case 1255:  /* *fma_fnmsub_sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1252:  /* *fma_fnmadd_v2df */
    case 1251:  /* *fma_fnmadd_v4sf */
    case 1250:  /* *fma_fnmadd_df */
    case 1249:  /* *fma_fnmadd_sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1246:  /* *fma_fmsub_v2df */
    case 1245:  /* *fma_fmsub_v4sf */
    case 1244:  /* *fma_fmsub_df */
    case 1243:  /* *fma_fmsub_sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1240:  /* *fma_fmadd_v2df */
    case 1239:  /* *fma_fmadd_v4sf */
    case 1238:  /* *fma_fmadd_df */
    case 1237:  /* *fma_fmadd_sf */
    case 899:  /* *xop_pcmov_df */
    case 898:  /* *xop_pcmov_sf */
    case 947:  /* rdfsbasesi */
    case 948:  /* rdfsbasedi */
    case 949:  /* rdgsbasesi */
    case 950:  /* rdgsbasedi */
    case 951:  /* wrfsbasesi */
    case 952:  /* wrfsbasedi */
    case 953:  /* wrgsbasesi */
    case 954:  /* wrgsbasedi */
    case 1787:  /* xop_vrotlv16qi3 */
    case 1788:  /* xop_vrotlv8hi3 */
    case 1789:  /* xop_vrotlv4si3 */
    case 1790:  /* xop_vrotlv2di3 */
    case 1791:  /* xop_shav16qi3 */
    case 1792:  /* xop_shav8hi3 */
    case 1793:  /* xop_shav4si3 */
    case 1794:  /* xop_shav2di3 */
    case 1795:  /* xop_shlv16qi3 */
    case 1796:  /* xop_shlv8hi3 */
    case 1797:  /* xop_shlv4si3 */
    case 1798:  /* xop_shlv2di3 */
    case 1807:  /* xop_maskcmpv16qi3 */
    case 1808:  /* xop_maskcmpv8hi3 */
    case 1809:  /* xop_maskcmpv4si3 */
    case 1810:  /* xop_maskcmpv2di3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1822:  /* xop_pcom_tfv2di3 */
      return 2;

    case 1806:  /* *xop_vmfrczv2df2 */
    case 1805:  /* *xop_vmfrczv4sf2 */
    case 1804:  /* xop_frczv4df2 */
    case 1803:  /* xop_frczv8sf2 */
    case 1802:  /* xop_frczv2df2 */
    case 1801:  /* xop_frczv4sf2 */
    case 1800:  /* xop_frczdf2 */
    case 1799:  /* xop_frczsf2 */
    case 1774:  /* xop_phsubdq */
    case 1773:  /* xop_phsubwd */
    case 1772:  /* xop_phsubbw */
    case 1771:  /* xop_phaddudq */
    case 1770:  /* xop_phadduwq */
    case 1769:  /* xop_phadduwd */
    case 1768:  /* xop_phaddubq */
    case 1767:  /* xop_phaddubd */
    case 1766:  /* xop_phaddubw */
    case 1765:  /* xop_phadddq */
    case 1764:  /* xop_phaddwq */
    case 1763:  /* xop_phaddwd */
    case 1762:  /* xop_phaddbq */
    case 1761:  /* xop_phaddbd */
    case 1760:  /* xop_phaddbw */
    case 809:  /* sse4_1_roundsf2 */
    case 810:  /* sse4_1_rounddf2 */
    case 929:  /* sse4_2_crc32qi */
    case 930:  /* sse4_2_crc32hi */
    case 931:  /* sse4_2_crc32si */
    case 932:  /* sse4_2_crc32di */
    case 955:  /* rdrandhi_1 */
    case 956:  /* rdrandsi_1 */
    case 957:  /* rdranddi_1 */
    case 970:  /* *mmx_addv2sf3 */
    case 971:  /* *mmx_subv2sf3 */
    case 972:  /* *mmx_mulv2sf3 */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 975:  /* *mmx_smaxv2sf3 */
    case 976:  /* *mmx_sminv2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 987:  /* mmx_gev2sf3 */
    case 988:  /* mmx_pf2id */
    case 989:  /* mmx_pf2iw */
    case 990:  /* mmx_pi2fw */
    case 991:  /* mmx_floatv2si2 */
    case 992:  /* mmx_pswapdv2sf2 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1356:  /* sse4_1_insertps */
    case 1362:  /* vec_extract_hi_v4di */
    case 1363:  /* vec_extract_hi_v4df */
    case 1366:  /* vec_extract_hi_v8si */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1567:  /* *sse4_1_pextrd */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1569:  /* *sse4_1_pextrq */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1631:  /* ssse3_pmaddubsw */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1647:  /* avx2_palignrv2ti */
    case 1648:  /* ssse3_palignrti */
    case 1649:  /* ssse3_palignrdi */
    case 1650:  /* absv32qi2 */
    case 1651:  /* absv16qi2 */
    case 1652:  /* absv16hi2 */
    case 1653:  /* absv8hi2 */
    case 1654:  /* absv8si2 */
    case 1655:  /* absv4si2 */
    case 1656:  /* absv8qi2 */
    case 1657:  /* absv4hi2 */
    case 1658:  /* absv2si2 */
    case 1667:  /* avx_blendps256 */
    case 1668:  /* sse4_1_blendps */
    case 1669:  /* avx_blendpd256 */
    case 1670:  /* sse4_1_blendpd */
    case 1671:  /* avx_blendvps256 */
    case 1672:  /* sse4_1_blendvps */
    case 1673:  /* avx_blendvpd256 */
    case 1674:  /* sse4_1_blendvpd */
    case 1675:  /* avx_dpps256 */
    case 1676:  /* sse4_1_dpps */
    case 1677:  /* avx_dppd256 */
    case 1678:  /* sse4_1_dppd */
    case 1679:  /* avx2_movntdqa */
    case 1680:  /* sse4_1_movntdqa */
    case 1681:  /* avx2_mpsadbw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1683:  /* avx2_packusdw */
    case 1684:  /* sse4_1_packusdw */
    case 1685:  /* avx2_pblendvb */
    case 1686:  /* sse4_1_pblendvb */
    case 1687:  /* sse4_1_pblendw */
    case 1688:  /* *avx2_pblendw */
    case 1689:  /* avx2_pblenddv8si */
    case 1690:  /* avx2_pblenddv4si */
    case 1691:  /* sse4_1_phminposuw */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1716:  /* avx_vtestps256 */
    case 1717:  /* avx_vtestps */
    case 1718:  /* avx_vtestpd256 */
    case 1719:  /* avx_vtestpd */
    case 1720:  /* avx_ptest256 */
    case 1721:  /* sse4_1_ptest */
    case 1722:  /* avx_roundps256 */
    case 1723:  /* sse4_1_roundps */
    case 1724:  /* avx_roundpd256 */
    case 1725:  /* sse4_1_roundpd */
    case 1726:  /* sse4_1_roundss */
    case 1727:  /* sse4_1_roundsd */
    case 1728:  /* sse4_2_pcmpestr */
    case 1729:  /* sse4_2_pcmpestri */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1731:  /* sse4_2_pcmpestr_cconly */
    case 1732:  /* sse4_2_pcmpistr */
    case 1733:  /* sse4_2_pcmpistri */
    case 1734:  /* sse4_2_pcmpistrm */
    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1827:  /* aesenc */
    case 1828:  /* aesenclast */
    case 1829:  /* aesdec */
    case 1830:  /* aesdeclast */
    case 1831:  /* aesimc */
    case 1832:  /* aeskeygenassist */
    case 1833:  /* pclmulqdq */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1845:  /* avx2_permv4df */
    case 1850:  /* vec_dupv8si */
    case 1851:  /* vec_dupv8sf */
    case 1852:  /* vec_dupv4di */
    case 1853:  /* vec_dupv4df */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1858:  /* avx_vbroadcastf128_v32qi */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1869:  /* *avx_vpermilpv4df */
    case 1870:  /* *avx_vpermilpv2df */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1883:  /* vec_set_lo_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1886:  /* vec_set_hi_v4df */
    case 1887:  /* vec_set_lo_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1890:  /* vec_set_hi_v8sf */
    case 1891:  /* vec_set_lo_v16hi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1894:  /* vec_set_hi_v32qi */
    case 1895:  /* avx_maskloadps */
    case 1896:  /* avx_maskloadpd */
    case 1897:  /* avx_maskloadps256 */
    case 1898:  /* avx_maskloadpd256 */
    case 1899:  /* avx2_maskloadd */
    case 1900:  /* avx2_maskloadq */
    case 1901:  /* avx2_maskloadd256 */
    case 1902:  /* avx2_maskloadq256 */
    case 1903:  /* avx_maskstoreps */
    case 1904:  /* avx_maskstorepd */
    case 1905:  /* avx_maskstoreps256 */
    case 1906:  /* avx_maskstorepd256 */
    case 1907:  /* avx2_maskstored */
    case 1908:  /* avx2_maskstoreq */
    case 1909:  /* avx2_maskstored256 */
    case 1910:  /* avx2_maskstoreq256 */
      return 1;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_SSEMULADD) || (cached_type == TYPE_SSE4ARG))
        {
	  return 2;
        }
      else if ((cached_type == TYPE_SSEIADD1) || (cached_type == TYPE_SSECVT1))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    default:
      return 0;

    }
}

int
get_attr_prefix_rex (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x9)) && (! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0xf)) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1560:  /* sse4_1_pinsrq */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((V2DImode) == (V2DImode)))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 1559:  /* sse4_1_pinsrd */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((V4SImode) == (V2DImode)))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 1558:  /* sse2_pinsrw */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((V8HImode) == (V2DImode)))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 1557:  /* sse4_1_pinsrb */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && ((V16QImode) == (V2DImode)))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 1306:  /* sse2_cvtsi2sdq */
    case 1289:  /* sse_cvtsi2ssq */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0x3ff)) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0xffc)) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0x1e7f)) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x600))
        {
	  return 
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn));
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if ((!((1 << which_alternative) & 0xfc)) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
      extract_constrain_insn_cached (insn);
      if (ext_QIreg_operand (operands[2], QImode))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 883:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 872:  /* *strsetqi_1 */
    case 860:  /* *strmovqi_1 */
      extract_constrain_insn_cached (insn);
      if (
#line 16241 "../../gcc/config/i386/i386.md"
(DImode == DImode))
        {
	  return 0;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if ((
#line 423 "../../gcc/config/i386/i386.md"
(x86_extended_QIreg_mentioned_p (insn))) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 882:  /* *strlenqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 876:  /* *rep_stosqi */
    case 871:  /* *strsetqi_1 */
    case 859:  /* *strmovqi_1 */
      extract_constrain_insn_cached (insn);
      if (
#line 16241 "../../gcc/config/i386/i386.md"
(SImode == DImode))
        {
	  return 0;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if ((
#line 423 "../../gcc/config/i386/i386.md"
(x86_extended_QIreg_mentioned_p (insn))) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 713:  /* *bswaphi_lowpart_1 */
    case 492:  /* *ashlqi3_1 */
    case 472:  /* *one_cmplqi2_1 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if (((which_alternative == 0) && (
#line 423 "../../gcc/config/i386/i386.md"
(x86_extended_QIreg_mentioned_p (insn)))) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 374:  /* *andqi_2_maybe_si */
    case 371:  /* *andqi_1 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if (((((1 << which_alternative) & 0x3)) && (
#line 423 "../../gcc/config/i386/i386.md"
(x86_extended_QIreg_mentioned_p (insn)))) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 370:  /* *andhi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && (ext_QIreg_operand (operands[1], VOIDmode)))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if ((
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))) || ((which_alternative == 2) && (ext_QIreg_operand (operands[1], QImode))))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 2) && ((
#line 7803 "../../gcc/config/i386/i386.md"
(INTVAL (operands[2]) == 0xff)) && (ext_QIreg_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if ((
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))) || ((which_alternative == 2) && (ext_QIreg_operand (operands[1], QImode))))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 3) && ((
#line 7803 "../../gcc/config/i386/i386.md"
(INTVAL (operands[2]) == 0xff)) && (ext_QIreg_operand (operands[1], VOIDmode))))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if ((((1 << which_alternative) & 0x6)) || ((
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))) || ((which_alternative == 3) && (ext_QIreg_operand (operands[1], QImode)))))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 1579:  /* *sse2_storeq_rex64 */
    case 896:  /* *movdfcc_1 */
    case 895:  /* *movdfcc_1_rex64 */
    case 356:  /* *testdi_1 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0x3)) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((GET_MODE (operands[0]) == DImode) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 357:  /* *testqi_1_maybe_si */
    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if (((((1 << which_alternative) & 0x7)) && (
#line 423 "../../gcc/config/i386/i386.md"
(x86_extended_QIreg_mentioned_p (insn)))) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 244:  /* *leadi */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((! (SImode_address_operand (operands[1], VOIDmode))) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
      extract_constrain_insn_cached (insn);
      if (
#line 16241 "../../gcc/config/i386/i386.md"
(DImode == DImode))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
      extract_constrain_insn_cached (insn);
      if (
#line 16241 "../../gcc/config/i386/i386.md"
(SImode == DImode))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (
#line 16241 "../../gcc/config/i386/i386.md"
(DImode == DImode)))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 1) && (
#line 16241 "../../gcc/config/i386/i386.md"
(SImode == DImode)))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (
#line 16241 "../../gcc/config/i386/i386.md"
(DImode == DImode)))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
      extract_constrain_insn_cached (insn);
      if ((which_alternative != 0) && (
#line 16241 "../../gcc/config/i386/i386.md"
(SImode == DImode)))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))) || ((which_alternative == 0) && (ext_QIreg_operand (operands[1], QImode))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((which_alternative == 1) || ((
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))) || ((which_alternative == 0) && (ext_QIreg_operand (operands[1], QImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((((1 << which_alternative) & 0x1858)) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 105:  /* *movtf_internal_sse2 */
    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0x7)) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((which_alternative == 1) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if (((! ((cached_type = get_attr_type (insn)) == TYPE_IMOVX)) && (
#line 423 "../../gcc/config/i386/i386.md"
(x86_extended_QIreg_mentioned_p (insn)))) || ((
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))) || (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && (ext_QIreg_operand (operands[1], QImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((((QIreg_operand (operands[0], QImode)) && (! (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))) && (
#line 423 "../../gcc/config/i386/i386.md"
(x86_extended_QIreg_mentioned_p (insn)))) || ((
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))) || (((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))) && (ext_QIreg_operand (operands[1], QImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))) || (ext_QIreg_operand (operands[1], QImode)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((((which_alternative == 6) || ((!((1 << which_alternative) & 0x78)) && ((! ((cached_type = get_attr_type (insn)) == TYPE_IMOVX)) && ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (which_alternative == 2))) || ((!((1 << which_alternative) & 0x3)) || ((! (
#line 2311 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_DEPENDENCY))) || ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)))))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (which_alternative == 2))) || ((!((1 << which_alternative) & 0x3)) || ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH))))))))) && (
#line 423 "../../gcc/config/i386/i386.md"
(x86_extended_QIreg_mentioned_p (insn)))) || ((
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))) || (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && (ext_QIreg_operand (operands[1], QImode)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))) || ((((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))) && (ext_QIreg_operand (operands[1], QImode))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((!((1 << which_alternative) & 0x1ebc)) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x180))
        {
	  return 1;
        }
      else
        {
	  if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
	    {
	      return 0;
	    }
	  else if ((!((1 << which_alternative) & 0xbf1)) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((((1 << which_alternative) & 0x3)) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 373:  /* *anddi_2 */
    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((which_alternative != 0) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 2019:  /* storedi_via_fpu */
    case 1601:  /* avx2_pmovmskb */
    case 1312:  /* sse2_cvttsd2siq */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1295:  /* sse_cvttss2siq */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1106:  /* sse2_movntidi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 932:  /* sse4_2_crc32di */
    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 909:  /* pro_epilogue_adjust_stack_di_add */
    case 892:  /* *movdicc_noc */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 873:  /* *rep_stosdi_rex64 */
    case 866:  /* *strsetdi_rex_1 */
    case 861:  /* *rep_movdi_rex64 */
    case 854:  /* *strmovdi_rex_1 */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 712:  /* *bswapdi2_1 */
    case 710:  /* *bswapdi2_movbe */
    case 707:  /* *popcountdi2_cmp */
    case 704:  /* popcountdi2 */
    case 699:  /* bsr_rex64 */
    case 698:  /* *tbm_tzmsk_di */
    case 696:  /* *tbm_t1mskc_di */
    case 694:  /* *tbm_blsic_di */
    case 692:  /* *tbm_blsfill_di */
    case 690:  /* *tbm_blcs_di */
    case 688:  /* *tbm_blcmsk_di */
    case 686:  /* *tbm_blcic_di */
    case 684:  /* *tbm_blci_di */
    case 682:  /* *tbm_blcfill_di */
    case 680:  /* tbm_bextri_di */
    case 678:  /* bmi2_pext_di3 */
    case 676:  /* bmi2_pdep_di3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 672:  /* *bmi_blsr_di */
    case 670:  /* *bmi_blsmsk_di */
    case 668:  /* *bmi_blsi_di */
    case 666:  /* bmi_bextr_di */
    case 664:  /* *bmi_andn_di */
    case 662:  /* clzdi2_lzcnt */
    case 659:  /* ctzdi2 */
    case 656:  /* *ffsdi_1 */
    case 649:  /* set_got_offset_rex64 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 579:  /* *btdi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 513:  /* ashrdi3_cvt */
    case 511:  /* x86_64_shrd */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 497:  /* *ashldi3_cmp */
    case 488:  /* *ashldi3_1 */
    case 486:  /* *bmi2_ashldi3_1 */
    case 484:  /* *ashldi3_mask */
    case 481:  /* x86_64_shld */
    case 477:  /* *one_cmpldi2_2 */
    case 471:  /* *one_cmpldi2_1 */
    case 441:  /* *negdi2_cmpz */
    case 436:  /* *negdi2_1 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 354:  /* *udivmoddi4_noext */
    case 351:  /* *udivmoddi4 */
    case 348:  /* udivmoddi4_1 */
    case 345:  /* *divmoddi4_noext */
    case 342:  /* *divmoddi4 */
    case 339:  /* divmoddi4_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 329:  /* *mulditi3_1 */
    case 327:  /* *umulditi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 320:  /* *muldi3_1 */
    case 317:  /* *adddi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 298:  /* *subdi_3 */
    case 293:  /* *subdi_2 */
    case 287:  /* *subdi_1 */
    case 273:  /* *adddi_5 */
    case 266:  /* *adddi_4 */
    case 264:  /* *adddi_3 */
    case 259:  /* *adddi_2 */
    case 251:  /* *adddi_1 */
    case 248:  /* *adddi3_cc */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 76:  /* *swapdi */
    case 74:  /* *movabsdi_2 */
    case 70:  /* *movabsdi_1 */
    case 58:  /* *movdi_or */
    case 40:  /* *pushdi2 */
    case 11:  /* *cmpdi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 3:  /* *cmpdi_ccno_1 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 893:  /* *movqicc_noc */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 499:  /* *ashlqi3_cconly */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 474:  /* *one_cmplqi2_2 */
    case 438:  /* *negqi2_cmpz */
    case 433:  /* *negqi2_1 */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 379:  /* *andqi_2_slp */
    case 375:  /* *andqi_2 */
    case 372:  /* *andqi_1_slp */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 358:  /* *testqi_1 */
    case 355:  /* udivmodhiqi3 */
    case 346:  /* divmodhiqi3 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 323:  /* *mulqi3_1 */
    case 314:  /* *addqi3_cc_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 295:  /* *subqi_3 */
    case 290:  /* *subqi_2 */
    case 289:  /* *subqi_1_slp */
    case 284:  /* *subqi_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 270:  /* *addqi_5 */
    case 267:  /* *addqi_4 */
    case 261:  /* *addqi_3 */
    case 256:  /* *addqi_2 */
    case 255:  /* *addqi_1_slp */
    case 249:  /* addqi3_cc */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 83:  /* *movstrictqi_xor */
    case 81:  /* *movstrictqi_1 */
    case 79:  /* *swapqi_2 */
    case 71:  /* *movabsqi_2 */
    case 67:  /* *movabsqi_1 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 4:  /* *cmpqi_1 */
    case 0:  /* *cmpqi_ccno_1 */
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((
#line 423 "../../gcc/config/i386/i386.md"
(x86_extended_QIreg_mentioned_p (insn))) || (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if ((((cached_mode = get_attr_mode (insn)) == MODE_DI) && (((! ((cached_type = get_attr_type (insn)) == TYPE_PUSH)) && ((! (cached_type == TYPE_POP)) && ((! (cached_type == TYPE_CALL)) && ((! (cached_type == TYPE_CALLV)) && ((! (cached_type == TYPE_LEAVE)) && (! (cached_type == TYPE_IBR))))))) && (! (get_attr_unit (insn) == UNIT_MMX)))) || ((((cached_mode = get_attr_mode (insn)) == MODE_QI) && (
#line 423 "../../gcc/config/i386/i386.md"
(x86_extended_QIreg_mentioned_p (insn)))) || ((
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn))) || (((cached_type = get_attr_type (insn)) == TYPE_IMOVX) && (ext_QIreg_operand (operands[1], QImode))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 154:  /* fix_truncsfdi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 1569:  /* *sse4_1_pextrq */
      return 1;

    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1631:  /* ssse3_pmaddubsw */
    case 1634:  /* *ssse3_pmulhrswv4hi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1649:  /* ssse3_palignrdi */
    case 1656:  /* absv8qi2 */
    case 1657:  /* absv4hi2 */
    case 1658:  /* absv2si2 */
      extract_constrain_insn_cached (insn);
      return 
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn));

    default:
      extract_constrain_insn_cached (insn);
      if (! (
#line 416 "../../gcc/config/i386/i386.md"
(TARGET_64BIT)))
        {
	  return 0;
        }
      else if (
#line 425 "../../gcc/config/i386/i386.md"
(x86_extended_reg_mentioned_p (insn)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    }
}

int
get_attr_prefix_0f (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_unit cached_unit ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x600))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3f))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
    case 1383:  /* sse2_storehpd */
    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xc00))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x6000))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 747:  /* *fop_df_1_sse */
    case 746:  /* *fop_sf_1_sse */
    case 744:  /* *fop_df_1_mixed */
    case 743:  /* *fop_sf_1_mixed */
      if (get_attr_unit (insn) == UNIT_SSE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 738:  /* *fop_df_comm_mixed */
    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 1;
        }

    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else
        {
	  return 0;
        }

    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 367:  /* *anddi_1 */
    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else
        {
	  if (which_alternative != 1)
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 110:  /* *movsf_internal */
    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x1f))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7f))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1358:  /* *sse4_1_extractps */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 91:  /* *movqi_extzv_2_rex64 */
    case 87:  /* *movqi_extv_1_rex64 */
      extract_constrain_insn_cached (insn);
      if ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 92:  /* *movqi_extzv_2 */
    case 88:  /* *movqi_extv_1 */
    case 66:  /* *movqi_internal */
      if (get_attr_type (insn) == TYPE_IMOVX)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (((which_alternative != 0) || ((
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL)) && (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH)))) && ((!((1 << which_alternative) & 0x6)) || (! (aligned_operand (operands[1], HImode)))))) && ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xffc))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1fff0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
    case 135:  /* *extendsfdf2_mixed */
    case 63:  /* *movdi_internal */
    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
    case 140:  /* *truncdfsf_fast_mixed */
    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1579:  /* *sse2_storeq_rex64 */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 918:  /* trap */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 909:  /* pro_epilogue_adjust_stack_di_add */
    case 908:  /* pro_epilogue_adjust_stack_si_add */
    case 896:  /* *movdfcc_1 */
    case 894:  /* *movxfcc_1 */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 853:  /* cld */
    case 851:  /* fxamdf2_i387_with_temp */
    case 850:  /* fxamsf2_i387_with_temp */
    case 849:  /* fxamxf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 847:  /* fxamsf2_i387 */
    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
    case 761:  /* *fop_xf_comm_i387 */
    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 756:  /* *fop_sf_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 749:  /* *fop_df_1_i387 */
    case 748:  /* *fop_sf_1_i387 */
    case 742:  /* *fop_df_comm_i387 */
    case 741:  /* *fop_sf_comm_i387 */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 650:  /* eh_return_internal */
    case 649:  /* set_got_offset_rex64 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 641:  /* simple_return_indirect_internal */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 492:  /* *ashlqi3_1 */
    case 491:  /* *ashlhi3_1 */
    case 490:  /* *ashlsi3_1_zext */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 379:  /* *andqi_2_slp */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 372:  /* *andqi_1_slp */
    case 371:  /* *andqi_1 */
    case 369:  /* *andsi_1_zext */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 323:  /* *mulqi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 289:  /* *subqi_1_slp */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 255:  /* *addqi_1_slp */
    case 254:  /* *addqi_1 */
    case 253:  /* *addhi_1 */
    case 252:  /* addsi_1_zext */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 145:  /* *truncdfsf2_i387_1 */
    case 144:  /* *truncdfsf_i387 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
    case 127:  /* extendsidi2_1 */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 118:  /* zero_extendhisi2_and */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
    case 42:  /* *pushdi2_rex64 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
    case 33:  /* x86_sahf_1 */
    case 32:  /* x86_fnstsw_1 */
    case 31:  /* *cmpfp_si_cc */
    case 30:  /* *cmpfp_hi_cc */
    case 29:  /* *cmpfp_si */
    case 28:  /* *cmpfp_hi */
    case 27:  /* *cmpfp_u_cc */
    case 26:  /* *cmpfp_u */
    case 25:  /* *cmpfp_df_cc */
    case 24:  /* *cmpfp_sf_cc */
    case 23:  /* *cmpfp_df */
    case 22:  /* *cmpfp_sf */
    case 21:  /* *cmpfp_xf_cc */
    case 20:  /* *cmpfp_xf */
    case 19:  /* *cmpfp_0_cc */
    case 18:  /* *cmpfp_0 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
    case 126:  /* *extendsidi2_rex64 */
      return 0;

    case 130:  /* extendhisi2 */
    case 131:  /* *extendhisi2_zext */
    case 134:  /* extendqihi2 */
      extract_constrain_insn_cached (insn);
      if ((! (((
#line 308 "../../gcc/config/i386/i386.md"
(ix86_schedule)) == (
(CPU_K6))))) && (which_alternative == 0))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if ((((cached_type = get_attr_type (insn)) == TYPE_IMOVX) || ((cached_type == TYPE_SETCC) || ((cached_type == TYPE_ICMOV) || (cached_type == TYPE_BITMANIP)))) || (((cached_unit = get_attr_unit (insn)) == UNIT_SSE) || (cached_unit == UNIT_MMX)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    default:
      return 1;

    }
}

int
get_attr_prefix_rep (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1576:  /* sse2_loadld */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x24))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x6fe))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1584:  /* *vec_dupv2di */
    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3b))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x39))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 4)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1192:  /* sse3_hsubv4sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1184:  /* sse3_addsubv4sf3 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1301:  /* fix_truncv4sfv4si2 */
    case 1104:  /* sse3_lddqu */
    case 1103:  /* avx_lddqu256 */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1383:  /* sse2_storehpd */
    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x30))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x30)) || ((which_alternative == 7) && (! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x2c0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 747:  /* *fop_df_1_sse */
    case 746:  /* *fop_sf_1_sse */
    case 744:  /* *fop_df_1_mixed */
    case 743:  /* *fop_sf_1_mixed */
      if (get_attr_unit (insn) == UNIT_SSE)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1385:  /* sse2_storelpd */
    case 738:  /* *fop_df_comm_mixed */
    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 1389:  /* sse2_movsd */
    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1351:  /* *vec_concatv2sf_sse */
    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1390:  /* vec_dupv2df */
    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0xce1f)) && (((which_alternative == 6) && ((! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xe78))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x1f)) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || ((((1 << which_alternative) & 0x880)) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))) || (!((1 << which_alternative) & 0xee0)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || (((which_alternative == 9) && (! (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))) || (!((1 << which_alternative) & 0x1bff))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xf00)) && (! (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 0;
        }
      else
        {
	  return 0;
        }

    case 657:  /* ctzhi2 */
    case 658:  /* ctzsi2 */
    case 659:  /* ctzdi2 */
      extract_constrain_insn_cached (insn);
      return 
#line 12043 "../../gcc/config/i386/i386.md"
(TARGET_BMI);

    case 1662:  /* sse4a_vmmovntv2df */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1660:  /* sse4a_movntdf */
    case 1659:  /* sse4a_movntsf */
    case 1346:  /* sse_movss */
    case 1323:  /* sse2_cvtss2sd */
    case 1322:  /* sse2_cvtsd2ss */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1289:  /* sse_cvtsi2ssq */
    case 1288:  /* sse_cvtsi2ss */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 1140:  /* sse2_vmdivv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 905:  /* *ieee_smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 903:  /* smindf3 */
    case 902:  /* smaxdf3 */
    case 901:  /* sminsf3 */
    case 900:  /* smaxsf3 */
    case 852:  /* movmsk_df */
    case 810:  /* sse4_1_rounddf2 */
    case 809:  /* sse4_1_roundsf2 */
    case 780:  /* *sqrtdf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 745:  /* *rcpsf2_sse */
    case 740:  /* *fop_df_comm_sse */
    case 739:  /* *fop_sf_comm_sse */
    case 586:  /* setcc_df_sse */
    case 585:  /* setcc_sf_sse */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 217:  /* *floatsisf2_sse_interunit */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 212:  /* *floatsidf2_vector_sse */
    case 211:  /* *floatsisf2_vector_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 154:  /* fix_truncsfdi_sse */
    case 141:  /* *truncdfsf_fast_sse */
    case 136:  /* *extendsfdf2_sse */
    case 639:  /* simple_return_internal_long */
    case 660:  /* clzhi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 662:  /* clzdi2_lzcnt */
    case 702:  /* popcounthi2 */
    case 703:  /* popcountsi2 */
    case 704:  /* popcountdi2 */
    case 705:  /* *popcounthi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 707:  /* *popcountdi2_cmp */
    case 708:  /* *popcountsi2_cmp_zext */
    case 861:  /* *rep_movdi_rex64 */
    case 862:  /* *rep_movsi */
    case 863:  /* *rep_movsi */
    case 864:  /* *rep_movqi */
    case 865:  /* *rep_movqi */
    case 873:  /* *rep_stosdi_rex64 */
    case 874:  /* *rep_stossi */
    case 875:  /* *rep_stossi */
    case 876:  /* *rep_stosqi */
    case 877:  /* *rep_stosqi */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 882:  /* *strlenqi_1 */
    case 883:  /* *strlenqi_1 */
    case 929:  /* sse4_2_crc32qi */
    case 930:  /* sse4_2_crc32hi */
    case 931:  /* sse4_2_crc32si */
    case 932:  /* sse4_2_crc32di */
    case 1290:  /* sse_cvtss2si */
    case 1291:  /* sse_cvtss2si_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1294:  /* sse_cvttss2si */
    case 1295:  /* sse_cvttss2siq */
    case 1307:  /* sse2_cvtsd2si */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1311:  /* sse2_cvttsd2si */
    case 1312:  /* sse2_cvttsd2siq */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1336:  /* sse3_movshdup */
    case 1338:  /* sse3_movsldup */
    case 1573:  /* sse2_pshuflw_1 */
    case 1575:  /* sse2_pshufhw_1 */
    case 1665:  /* sse4a_insertqi */
    case 1666:  /* sse4a_insertq */
      return 1;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_SSEMULADD) || ((cached_type == TYPE_SSE4ARG) || ((cached_type == TYPE_SSEIADD1) || (cached_type == TYPE_SSECVT1))))
        {
	  return 0;
        }
      else if ((get_attr_unit (insn) == UNIT_SSE) && (((cached_mode = get_attr_mode (insn)) == MODE_SF) || (cached_mode == MODE_DF)))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    default:
      return 0;

    }
}

int
get_attr_prefix_data16 (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_mode cached_mode ATTRIBUTE_UNUSED;
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7f))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1582:  /* *vec_extractv2di_1 */
    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xe))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1534:  /* *xorv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1532:  /* *andv2di3 */
    case 1510:  /* *andnotv2di3 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))
        {
	  return 1;
        }
      else
        {
	  if (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V2DImode) > 16)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 1528:  /* *xorv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1526:  /* *andv4si3 */
    case 1508:  /* *andnotv4si3 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))
        {
	  return 1;
        }
      else
        {
	  if (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V4SImode) > 16)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 1522:  /* *xorv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1520:  /* *andv8hi3 */
    case 1506:  /* *andnotv8hi3 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))
        {
	  return 1;
        }
      else
        {
	  if (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V8HImode) > 16)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 1516:  /* *xorv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1514:  /* *andv16qi3 */
    case 1504:  /* *andnotv16qi3 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))))
        {
	  return 1;
        }
      else
        {
	  if (((
#line 6571 "../../gcc/config/i386/sse.md"
(TARGET_AVX2)) || (! (
#line 6572 "../../gcc/config/i386/sse.md"
(GET_MODE_SIZE (V16QImode) > 16)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  if (((1 << which_alternative) & 0x3))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }
      else if (which_alternative == 3)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0x14)) && (! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return 1;
        }
      else
        {
	  if (which_alternative == 5)
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else if (which_alternative == 1)
        {
	  return 1;
        }
      else if (((1 << which_alternative) & 0x3c))
        {
	  if (which_alternative == 5)
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }
      else if (which_alternative == 6)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  if (((1 << which_alternative) & 0xc))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return 1;
        }
      else
        {
	  if (which_alternative == 2)
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  if (((1 << which_alternative) & 0x3))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }
      else if (which_alternative == 3)
        {
	  return 1;
        }
      else if (which_alternative == 4)
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 0;
        }
      else if (which_alternative == 2)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1302:  /* sse2_cvtpi2pd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 1;
        }

    case 1104:  /* sse3_lddqu */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1724:  /* avx_roundpd256 */
    case 1723:  /* sse4_1_roundps */
    case 1722:  /* avx_roundps256 */
    case 1111:  /* avx_movntv4di */
    case 1101:  /* avx_storedqu256 */
    case 1099:  /* avx_loaddqu256 */
      extract_constrain_insn_cached (insn);
      if (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))
        {
	  return 0;
        }
      else
        {
	  return 1;
        }

    case 1088:  /* *movv2df_internal */
      extract_constrain_insn_cached (insn);
      if (((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && ((V2DFmode) == (V2DFmode)))))) || ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DFmode) == (V4SFmode))) && (! ((V2DFmode) == (V2DFmode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1087:  /* *movv4df_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DFmode) == (V4SFmode))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1086:  /* *movv4sf_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4SFmode) == (V4SFmode))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1085:  /* *movv8sf_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SFmode) == (V4SFmode))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1084:  /* *movv1ti_internal */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && ((V1TImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V1TImode) == (V4SFmode))) && (! ((V1TImode) == (V2DFmode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1083:  /* *movv2ti_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V2TImode) == (V4SFmode))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1082:  /* *movv2di_internal */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && ((V2DImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V2DImode) == (V4SFmode))) && (! ((V2DImode) == (V2DFmode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1081:  /* *movv4di_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V4DImode) == (V4SFmode))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1080:  /* *movv4si_internal */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && ((V4SImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V4SImode) == (V4SFmode))) && (! ((V4SImode) == (V2DFmode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1079:  /* *movv8si_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V8SImode) == (V4SFmode))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1078:  /* *movv8hi_internal */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && ((V8HImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V8HImode) == (V4SFmode))) && (! ((V8HImode) == (V2DFmode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1077:  /* *movv16hi_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V16HImode) == (V4SFmode))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1076:  /* *movv16qi_internal */
      extract_constrain_insn_cached (insn);
      if (((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && ((V16QImode) == (V2DFmode)))))) || ((
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX)) || (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && ((! ((V16QImode) == (V4SFmode))) && (! ((V16QImode) == (V2DFmode))))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1075:  /* *movv32qi_internal */
      extract_constrain_insn_cached (insn);
      if ((! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))) && (((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2))) && (((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))) && (! ((V32QImode) == (V4SFmode))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 8) && (! (
#line 521 "../../gcc/config/i386/i386.md"
(TARGET_AVX))))
        {
	  return 1;
        }
      else
        {
	  if (which_alternative == 6)
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1c00))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 931:  /* sse4_2_crc32si */
    case 930:  /* sse4_2_crc32hi */
    case 929:  /* sse4_2_crc32qi */
      extract_insn_cached (insn);
      if (GET_MODE (operands[2]) == HImode)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1726:  /* sse4_1_roundss */
    case 1677:  /* avx_dppd256 */
    case 1676:  /* sse4_1_dpps */
    case 1675:  /* avx_dpps256 */
    case 1673:  /* avx_blendvpd256 */
    case 1672:  /* sse4_1_blendvps */
    case 1671:  /* avx_blendvps256 */
    case 1669:  /* avx_blendpd256 */
    case 1668:  /* sse4_1_blendps */
    case 1667:  /* avx_blendps256 */
    case 1647:  /* avx2_palignrv2ti */
    case 1642:  /* avx2_psignv8si3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1595:  /* avx2_psadbw */
    case 1583:  /* *vec_dupv4si */
    case 1543:  /* avx2_packuswb */
    case 1541:  /* avx2_packssdw */
    case 1539:  /* avx2_packsswb */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1456:  /* lshrv4di3 */
    case 1455:  /* ashlv4di3 */
    case 1452:  /* lshrv8si3 */
    case 1451:  /* ashlv8si3 */
    case 1448:  /* lshrv16hi3 */
    case 1447:  /* ashlv16hi3 */
    case 1445:  /* ashrv8si3 */
    case 1443:  /* ashrv16hi3 */
    case 1429:  /* *umulv16hi3_highpart */
    case 1428:  /* *smulv16hi3_highpart */
    case 1426:  /* *mulv16hi3 */
    case 1405:  /* *subv4di3 */
    case 1404:  /* *addv4di3 */
    case 1401:  /* *subv8si3 */
    case 1400:  /* *addv8si3 */
    case 1397:  /* *subv16hi3 */
    case 1396:  /* *addv16hi3 */
    case 1393:  /* *subv32qi3 */
    case 1392:  /* *addv32qi3 */
    case 1390:  /* vec_dupv2df */
    case 1385:  /* sse2_storelpd */
    case 1358:  /* *sse4_1_extractps */
    case 1356:  /* sse4_1_insertps */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1588:  /* *vec_concatv4si */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1584:  /* *vec_dupv2di */
    case 1576:  /* sse2_loadld */
    case 370:  /* *andhi_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 210:  /* *floatsidf2_vector_sse_with_temp */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 713:  /* *bswaphi_lowpart_1 */
    case 196:  /* *floatsidf2_vector_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative != 0)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x1f))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xf))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && ((
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR)) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((1 << which_alternative) & 0x880)) && (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS))))
        {
	  return 1;
        }
      else
        {
	  if ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((((((1 << which_alternative) & 0x220)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))))) || ((((1 << which_alternative) & 0x440)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || ((((1 << which_alternative) & 0x220)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))))))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 9) && (
#line 3329 "../../gcc/config/i386/i386.md"
(TARGET_SSE_SPLIT_REGS)))
        {
	  return 1;
        }
      else
        {
	  if ((((which_alternative == 7) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (! (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR))))) || ((which_alternative == 8) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3328 "../../gcc/config/i386/i386.md"
(TARGET_SSE_PARTIAL_REG_DEPENDENCY))))) || ((which_alternative == 7) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && (
#line 3311 "../../gcc/config/i386/i386.md"
(TARGET_SSE_LOAD0_BY_PXOR)))))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 105:  /* *movtf_internal_sse2 */
    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((((1 << which_alternative) & 0x5)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) || ((which_alternative == 1) && ((! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES))) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 65:  /* *movhi_internal */
      if (get_attr_mode (insn) == MODE_HI)
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xf80)) && ((!((1 << which_alternative) & 0xcc)) && ((!((1 << which_alternative) & 0xf00)) || (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))))
        {
	  return 1;
        }
      else
        {
	  if ((((1 << which_alternative) & 0xc0)) && (
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xa0))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 10)
        {
	  return 1;
        }
      else
        {
	  if (((1 << which_alternative) & 0xa00))
	    {
	      return 1;
	    }
	  else
	    {
	      return 0;
	    }
        }

    case 61:  /* *movti_internal_sse */
      extract_constrain_insn_cached (insn);
      if (((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && (! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))))) && ((which_alternative != 2) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if ((!((1 << which_alternative) & 0x3)) && ((
#line 640 "../../gcc/config/i386/i386.md"
(TARGET_SSE2)) && ((! (
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun)))) && ((which_alternative != 4) || (! (
#line 484 "../../gcc/config/i386/sse.md"
(TARGET_SSE_TYPELESS_STORES)))))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 38:  /* *cmpfp_iu_sse */
    case 35:  /* *cmpfp_i_sse */
      extract_insn_cached (insn);
      if (! (GET_MODE (operands[1]) == SFmode))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return 0;
        }
      else if (! (GET_MODE (operands[1]) == SFmode))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_SSEMULADD) || ((cached_type == TYPE_SSE4ARG) || ((cached_type == TYPE_SSEIADD1) || (cached_type == TYPE_SSECVT1))))
        {
	  return 0;
        }
      else if (((cached_mode = get_attr_mode (insn)) == MODE_HI) || ((get_attr_unit (insn) == UNIT_SSE) && ((cached_mode == MODE_V2DF) || (cached_mode == MODE_TI))))
        {
	  return 1;
        }
      else
        {
	  return 0;
        }

    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1945:  /* *avx2_gathersiv4si */
    case 1944:  /* *avx2_gathersiv4si */
    case 1939:  /* *avx2_gathersiv2df */
    case 1938:  /* *avx2_gathersiv2df */
    case 1937:  /* *avx2_gathersiv2di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1923:  /* avx2_lshrvv2di */
    case 1922:  /* avx2_ashlvv2di */
    case 1919:  /* avx2_lshrvv4si */
    case 1918:  /* avx2_ashlvv4si */
    case 1915:  /* avx2_ashrvv4si */
    case 1908:  /* avx2_maskstoreq */
    case 1907:  /* avx2_maskstored */
    case 1904:  /* avx_maskstorepd */
    case 1900:  /* avx2_maskloadq */
    case 1899:  /* avx2_maskloadd */
    case 1896:  /* avx_maskloadpd */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1870:  /* *avx_vpermilpv2df */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1833:  /* pclmulqdq */
    case 1832:  /* aeskeygenassist */
    case 1831:  /* aesimc */
    case 1830:  /* aesdeclast */
    case 1829:  /* aesdec */
    case 1828:  /* aesenclast */
    case 1827:  /* aesenc */
    case 1786:  /* xop_rotrv2di3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1779:  /* xop_rotlv16qi3 */
    case 1727:  /* sse4_1_roundsd */
    case 1725:  /* sse4_1_roundpd */
    case 1721:  /* sse4_1_ptest */
    case 1719:  /* avx_vtestpd */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1691:  /* sse4_1_phminposuw */
    case 1690:  /* avx2_pblenddv4si */
    case 1687:  /* sse4_1_pblendw */
    case 1686:  /* sse4_1_pblendvb */
    case 1684:  /* sse4_1_packusdw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1680:  /* sse4_1_movntdqa */
    case 1678:  /* sse4_1_dppd */
    case 1674:  /* sse4_1_blendvpd */
    case 1670:  /* sse4_1_blendpd */
    case 1648:  /* ssse3_palignrti */
    case 1643:  /* ssse3_psignv4si3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1600:  /* sse2_movmskpd */
    case 1596:  /* sse2_psadbw */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1569:  /* *sse4_1_pextrq */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1567:  /* *sse4_1_pextrd */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1560:  /* sse4_1_pinsrq */
    case 1559:  /* sse4_1_pinsrd */
    case 1558:  /* sse2_pinsrw */
    case 1557:  /* sse4_1_pinsrb */
    case 1556:  /* vec_interleave_lowv4si */
    case 1554:  /* vec_interleave_highv4si */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1544:  /* sse2_packuswb */
    case 1542:  /* sse2_packssdw */
    case 1540:  /* sse2_packsswb */
    case 1538:  /* *xortf3 */
    case 1537:  /* *iortf3 */
    case 1536:  /* *andtf3 */
    case 1535:  /* *andnottf3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1486:  /* *uminv16qi3 */
    case 1485:  /* *umaxv16qi3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1479:  /* *smaxv8hi3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1458:  /* lshrv2di3 */
    case 1457:  /* ashlv2di3 */
    case 1454:  /* lshrv4si3 */
    case 1453:  /* ashlv4si3 */
    case 1450:  /* lshrv8hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1446:  /* ashrv4si3 */
    case 1444:  /* ashrv8hi3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1437:  /* *sse2_pmaddwd */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1431:  /* *umulv8hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1427:  /* *mulv8hi3 */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1407:  /* *subv2di3 */
    case 1406:  /* *addv2di3 */
    case 1403:  /* *subv4si3 */
    case 1402:  /* *addv4si3 */
    case 1399:  /* *subv8hi3 */
    case 1398:  /* *addv8hi3 */
    case 1395:  /* *subv16qi3 */
    case 1394:  /* *addv16qi3 */
    case 1382:  /* sse2_shufpd_v2df */
    case 1381:  /* sse2_shufpd_v2di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1378:  /* vec_interleave_highv2di */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1315:  /* sse2_cvtdq2pd */
    case 1299:  /* sse2_cvtps2dq */
    case 1236:  /* *xordf3 */
    case 1235:  /* *iordf3 */
    case 1234:  /* *anddf3 */
    case 1230:  /* *andnotdf3 */
    case 1228:  /* *xorv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1226:  /* *andv2df3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1212:  /* sse2_ucomi */
    case 1210:  /* sse2_comi */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1196:  /* avx_cmpv2df3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1168:  /* *sminv2df3 */
    case 1167:  /* *smaxv2df3 */
    case 1160:  /* *sminv2df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1147:  /* sse2_sqrtv2df2 */
    case 1138:  /* sse2_divv2df3 */
    case 1132:  /* *mulv2df3 */
    case 1124:  /* *subv2df3 */
    case 1123:  /* *addv2df3 */
    case 1112:  /* sse2_movntv2di */
    case 1110:  /* sse2_movntv2df */
    case 1102:  /* sse2_storedqu */
    case 1100:  /* sse2_loaddqu */
    case 1098:  /* sse2_storeupd */
    case 1094:  /* sse2_loadupd */
    case 1089:  /* sse2_movq128 */
    case 890:  /* *movhicc_noc */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 841:  /* fisthi2_ceil_with_temp */
    case 839:  /* fisthi2_ceil */
    case 834:  /* *fisthi2_ceil_1 */
    case 830:  /* fisthi2_floor_with_temp */
    case 828:  /* fisthi2_floor */
    case 823:  /* *fisthi2_floor_1 */
    case 819:  /* fisthi2_with_temp */
    case 817:  /* fisthi2 */
    case 815:  /* *fisthi2_1 */
    case 765:  /* *fop_xf_3_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 717:  /* *parityhi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 705:  /* *popcounthi2_cmp */
    case 702:  /* popcounthi2 */
    case 701:  /* *bsrhi */
    case 660:  /* clzhi2_lzcnt */
    case 657:  /* ctzhi2 */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 500:  /* *ashlhi3_cconly */
    case 495:  /* *ashlhi3_cmp */
    case 475:  /* *one_cmplhi2_2 */
    case 469:  /* *one_cmplhi2_1 */
    case 439:  /* *neghi2_cmpz */
    case 434:  /* *neghi2_1 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 376:  /* *andhi_2 */
    case 359:  /* *testhi_1 */
    case 352:  /* *udivmodhi4_noext */
    case 349:  /* *udivmodhi4 */
    case 343:  /* *divmodhi4_noext */
    case 340:  /* *divmodhi4 */
    case 322:  /* *mulhi3_1 */
    case 315:  /* *addhi3_cc_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 296:  /* *subhi_3 */
    case 291:  /* *subhi_2 */
    case 285:  /* *subhi_1 */
    case 271:  /* *addhi_5 */
    case 268:  /* *addhi_4 */
    case 262:  /* *addhi_3 */
    case 257:  /* *addhi_2 */
    case 177:  /* x86_fldcw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 172:  /* fix_trunchi_i387 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 134:  /* extendqihi2 */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 120:  /* *zero_extendqihi2_and */
    case 84:  /* *movstricthi_xor */
    case 82:  /* *movstricthi_1 */
    case 80:  /* *swaphi_2 */
    case 72:  /* *movabshi_2 */
    case 68:  /* *movabshi_1 */
    case 30:  /* *cmpfp_hi_cc */
    case 28:  /* *cmpfp_hi */
    case 9:  /* *cmphi_minus_1 */
    case 5:  /* *cmphi_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 1303:  /* sse2_cvtpd2pi */
    case 1304:  /* sse2_cvttpd2pi */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1564:  /* *sse2_pextrw_si */
    case 1565:  /* *sse2_pextrw_di */
    case 1571:  /* sse2_pshufd_1 */
    case 1602:  /* sse2_pmovmskb */
    case 1603:  /* *sse2_maskmovdqu */
    case 1604:  /* *sse2_maskmovdqu */
    case 1650:  /* absv32qi2 */
    case 1651:  /* absv16qi2 */
    case 1652:  /* absv16hi2 */
    case 1653:  /* absv8hi2 */
    case 1654:  /* absv8si2 */
    case 1655:  /* absv4si2 */
    case 1663:  /* sse4a_extrqi */
    case 1664:  /* sse4a_extrq */
    case 1728:  /* sse4_2_pcmpestr */
    case 1729:  /* sse4_2_pcmpestri */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1731:  /* sse4_2_pcmpestr_cconly */
    case 1732:  /* sse4_2_pcmpistr */
    case 1733:  /* sse4_2_pcmpistri */
    case 1734:  /* sse4_2_pcmpistrm */
    case 1735:  /* sse4_2_pcmpistr_cconly */
      return 1;

    default:
      return 0;

    }
}

enum attr_type
get_attr_type (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 1863:  /* avx_vbroadcastf128_v4df */
    case 1862:  /* avx_vbroadcastf128_v8sf */
    case 1861:  /* avx_vbroadcastf128_v4di */
    case 1860:  /* avx_vbroadcastf128_v8si */
    case 1859:  /* avx_vbroadcastf128_v16hi */
    case 1858:  /* avx_vbroadcastf128_v32qi */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_SSEMOV;
        }
      else
        {
	  return TYPE_SSELOG1;
        }

    case 1590:  /* vec_concatv2di */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_SSEMOV;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return TYPE_SSELOG;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 1584:  /* *vec_dupv2di */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return TYPE_SSELOG1;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 1582:  /* *vec_extractv2di_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_SSEMOV;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return TYPE_SSEISHFT1;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_SSEMOV;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return TYPE_SSEISHFT1;
        }
      else if (which_alternative == 3)
        {
	  return TYPE_SSEMOV;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_OTHER;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_SSEMOV;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return TYPE_SSELOG;
        }
      else if (which_alternative == 4)
        {
	  return TYPE_SSEMOV;
        }
      else if (which_alternative == 5)
        {
	  return TYPE_FMOV;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return TYPE_SSEMOV;
        }
      else if (which_alternative == 3)
        {
	  return TYPE_FMOV;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_SSEMOV;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return TYPE_SSELOG1;
        }
      else if (which_alternative == 3)
        {
	  return TYPE_SSEMOV;
        }
      else if (which_alternative == 4)
        {
	  return TYPE_FMOV;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_SSELOG;
        }
      else
        {
	  return TYPE_OTHER;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return TYPE_SSELOG;
        }
      else if (which_alternative == 4)
        {
	  return TYPE_SSEMOV;
        }
      else if (which_alternative == 5)
        {
	  return TYPE_MMXCVT;
        }
      else
        {
	  return TYPE_MMXMOV;
        }

    case 1588:  /* *vec_concatv4si */
    case 1345:  /* sse_loadlps */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_SSELOG;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_MMXCVT;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return TYPE_SSELOG1;
        }
      else if (which_alternative == 4)
        {
	  return TYPE_MMXMOV;
        }
      else if (which_alternative == 5)
        {
	  return TYPE_SSEMOV;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 896:  /* *movdfcc_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_FCMOV;
        }
      else
        {
	  return TYPE_MULTI;
        }

    case 897:  /* *movsfcc_1_387 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_FCMOV;
        }
      else
        {
	  return TYPE_ICMOV;
        }

    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_BITMANIP;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 370:  /* *andhi_1 */
    case 368:  /* *andsi_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_ALU;
        }
      else
        {
	  return TYPE_IMOVX;
        }

    case 367:  /* *anddi_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return TYPE_ALU;
        }
      else
        {
	  return TYPE_IMOVX;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_FMOV;
        }
      else if (which_alternative == 1)
        {
	  return TYPE_MULTI;
        }
      else
        {
	  return TYPE_SSEICVT;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_FMOV;
        }
      else if (which_alternative == 1)
        {
	  return TYPE_SSECVT;
        }
      else
        {
	  return TYPE_MULTI;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_FMOV;
        }
      else
        {
	  return TYPE_SSECVT;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return TYPE_MULTI;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return TYPE_MMXMOV;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_IMOVX;
        }
      else if (which_alternative == 1)
        {
	  return TYPE_IMOV;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return TYPE_MMXMOV;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return TYPE_FMOV;
        }
      else
        {
	  return TYPE_MULTI;
        }

    case 1864:  /* *avx_vperm_broadcast_v4sf */
    case 105:  /* *movtf_internal_sse2 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_SSEMOV;
        }
      else
        {
	  return TYPE_SSELOG1;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_SSEMOV;
        }
      else if (which_alternative == 2)
        {
	  return TYPE_SSELOG1;
        }
      else
        {
	  return TYPE_OTHER;
        }

    case 66:  /* *movqi_internal */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 5) && (! (aligned_operand (operands[1], QImode))))
        {
	  return TYPE_IMOVX;
        }
      else if ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || ((which_alternative == 3) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2293 "../../gcc/config/i386/i386.md"
(TARGET_QIMODE_MATH))))))
        {
	  return TYPE_IMOV;
        }
      else if ((((1 << which_alternative) & 0x28)) || ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (which_alternative == 2)))
        {
	  return TYPE_IMOVX;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 65:  /* *movhi_internal */
      extract_constrain_insn_cached (insn);
      if ((
#line 481 "../../gcc/config/i386/sse.md"
(optimize_function_for_size_p (cfun))) || (((which_alternative == 0) && ((! (
#line 2292 "../../gcc/config/i386/i386.md"
(TARGET_PARTIAL_REG_STALL))) || (! (
#line 2235 "../../gcc/config/i386/i386.md"
(TARGET_HIMODE_MATH))))) || ((((1 << which_alternative) & 0x6)) && (aligned_operand (operands[1], HImode)))))
        {
	  return TYPE_IMOV;
        }
      else if ((
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)) && (((1 << which_alternative) & 0x5)))
        {
	  return TYPE_IMOVX;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_OTHER;
        }
      else if (which_alternative == 2)
        {
	  return TYPE_SSELOG1;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 1088:  /* *movv2df_internal */
    case 1087:  /* *movv4df_internal */
    case 1086:  /* *movv4sf_internal */
    case 1085:  /* *movv8sf_internal */
    case 1084:  /* *movv1ti_internal */
    case 1083:  /* *movv2ti_internal */
    case 1082:  /* *movv2di_internal */
    case 1081:  /* *movv4di_internal */
    case 1080:  /* *movv4si_internal */
    case 1079:  /* *movv8si_internal */
    case 1078:  /* *movv8hi_internal */
    case 1077:  /* *movv16hi_internal */
    case 1076:  /* *movv16qi_internal */
    case 1075:  /* *movv32qi_internal */
    case 61:  /* *movti_internal_sse */
    case 59:  /* *movoi_internal_avx */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_SSELOG1;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 0:  /* *cmpqi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 3:  /* *cmpdi_ccno_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_TEST;
        }
      else
        {
	  return TYPE_ICMP;
        }

    case 34:  /* *cmpfp_i_mixed */
    case 37:  /* *cmpfp_iu_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_FCMP;
        }
      else
        {
	  return TYPE_SSECOMI;
        }

    case 42:  /* *pushdi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_PUSH;
        }
      else
        {
	  return TYPE_MULTI;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 4)
        {
	  return TYPE_MMX;
        }
      else if (((1 << which_alternative) & 0x1e0))
        {
	  return TYPE_MMXMOV;
        }
      else if (which_alternative == 9)
        {
	  return TYPE_SSELOG1;
        }
      else if (((1 << which_alternative) & 0x7c00))
        {
	  return TYPE_SSEMOV;
        }
      else if (((1 << which_alternative) & 0x18000))
        {
	  return TYPE_SSECVT;
        }
      else if (pic_32bit_operand (operands[1], VOIDmode))
        {
	  return TYPE_LEA;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_MULTI;
        }
      else if (which_alternative == 2)
        {
	  return TYPE_MMX;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return TYPE_MMXMOV;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return TYPE_SSELOG1;
        }
      else if (((1 << which_alternative) & 0x6000))
        {
	  return TYPE_SSECVT;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return TYPE_MMX;
        }
      else if (((1 << which_alternative) & 0x38))
        {
	  return TYPE_MMXMOV;
        }
      else if (which_alternative == 6)
        {
	  return TYPE_SSELOG1;
        }
      else if (((1 << which_alternative) & 0xf80))
        {
	  return TYPE_SSEMOV;
        }
      else if (pic_32bit_operand (operands[1], VOIDmode))
        {
	  return TYPE_LEA;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 87:  /* *movqi_extv_1_rex64 */
    case 91:  /* *movqi_extzv_2_rex64 */
      extract_constrain_insn_cached (insn);
      if ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX)))
        {
	  return TYPE_IMOVX;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 88:  /* *movqi_extv_1 */
    case 92:  /* *movqi_extzv_2 */
      extract_constrain_insn_cached (insn);
      if ((register_operand (operands[0], QImode)) && ((! (QIreg_operand (operands[0], QImode))) || (
#line 2548 "../../gcc/config/i386/i386.md"
(TARGET_MOVX))))
        {
	  return TYPE_IMOVX;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 102:  /* *pushsf_rex64 */
    case 103:  /* *pushsf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_MULTI;
        }
      else if (which_alternative == 1)
        {
	  return TYPE_PUSH;
        }
      else
        {
	  return TYPE_MULTI;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return TYPE_FMOV;
        }
      else if (((1 << which_alternative) & 0x78))
        {
	  return TYPE_IMOV;
        }
      else if (which_alternative == 7)
        {
	  return TYPE_SSELOG1;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return TYPE_FMOV;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return TYPE_MULTI;
        }
      else if (((1 << which_alternative) & 0x220))
        {
	  return TYPE_SSELOG1;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return TYPE_FMOV;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return TYPE_IMOV;
        }
      else if (which_alternative == 5)
        {
	  return TYPE_SSELOG1;
        }
      else if (((1 << which_alternative) & 0xce00))
        {
	  return TYPE_MMXMOV;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 123:  /* *zero_extendqisi2_movzbl_and */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_IMOVX;
        }
      else
        {
	  return TYPE_ALU1;
        }

    case 140:  /* *truncdfsf_fast_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_FMOV;
        }
      else
        {
	  return TYPE_SSECVT;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 195:  /* *floatsisf2_vector_mixed */
    case 196:  /* *floatsidf2_vector_mixed */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 208:  /* *floatdidf2_mixed_nointerunit */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_FMOV;
        }
      else
        {
	  return TYPE_SSEICVT;
        }

    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
    case 181:  /* *floathisf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 230:  /* *floatdixf2_i387_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_FMOV;
        }
      else
        {
	  return TYPE_MULTI;
        }

    case 250:  /* *addsi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return TYPE_LEA;
        }
      else if (incdec_operand (operands[2], SImode))
        {
	  return TYPE_INCDEC;
        }
      else
        {
	  return TYPE_ALU;
        }

    case 251:  /* *adddi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return TYPE_LEA;
        }
      else if (incdec_operand (operands[2], DImode))
        {
	  return TYPE_INCDEC;
        }
      else
        {
	  return TYPE_ALU;
        }

    case 252:  /* addsi_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return TYPE_LEA;
        }
      else if (incdec_operand (operands[2], SImode))
        {
	  return TYPE_INCDEC;
        }
      else
        {
	  return TYPE_ALU;
        }

    case 253:  /* *addhi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return TYPE_LEA;
        }
      else if (incdec_operand (operands[2], HImode))
        {
	  return TYPE_INCDEC;
        }
      else
        {
	  return TYPE_ALU;
        }

    case 254:  /* *addqi_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return TYPE_LEA;
        }
      else if (incdec_operand (operands[2], QImode))
        {
	  return TYPE_INCDEC;
        }
      else
        {
	  return TYPE_ALU;
        }

    case 255:  /* *addqi_1_slp */
      extract_insn_cached (insn);
      if (incdec_operand (operands[1], QImode))
        {
	  return TYPE_INCDEC;
        }
      else
        {
	  return TYPE_ALU1;
        }

    case 257:  /* *addhi_2 */
    case 262:  /* *addhi_3 */
    case 268:  /* *addhi_4 */
    case 271:  /* *addhi_5 */
      extract_insn_cached (insn);
      if (incdec_operand (operands[2], HImode))
        {
	  return TYPE_INCDEC;
        }
      else
        {
	  return TYPE_ALU;
        }

    case 258:  /* *addsi_2 */
    case 260:  /* *addsi_2_zext */
    case 263:  /* *addsi_3 */
    case 265:  /* *addsi_3_zext */
    case 269:  /* *addsi_4 */
    case 272:  /* *addsi_5 */
      extract_insn_cached (insn);
      if (incdec_operand (operands[2], SImode))
        {
	  return TYPE_INCDEC;
        }
      else
        {
	  return TYPE_ALU;
        }

    case 259:  /* *adddi_2 */
    case 264:  /* *adddi_3 */
    case 266:  /* *adddi_4 */
    case 273:  /* *adddi_5 */
      extract_insn_cached (insn);
      if (incdec_operand (operands[2], DImode))
        {
	  return TYPE_INCDEC;
        }
      else
        {
	  return TYPE_ALU;
        }

    case 256:  /* *addqi_2 */
    case 261:  /* *addqi_3 */
    case 267:  /* *addqi_4 */
    case 270:  /* *addqi_5 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 275:  /* addqi_ext_1 */
      extract_insn_cached (insn);
      if (incdec_operand (operands[2], QImode))
        {
	  return TYPE_INCDEC;
        }
      else
        {
	  return TYPE_ALU;
        }

    case 326:  /* *umulsidi3_1 */
    case 327:  /* *umulditi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_IMUL;
        }
      else
        {
	  return TYPE_IMULX;
        }

    case 487:  /* *ashlsi3_1 */
    case 488:  /* *ashldi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return TYPE_LEA;
        }
      else if (which_alternative == 2)
        {
	  return TYPE_ISHIFTX;
        }
      else if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (register_operand (operands[0], VOIDmode))) && (const1_operand (operands[2], VOIDmode)))
        {
	  return TYPE_ALU;
        }
      else
        {
	  return TYPE_ISHIFT;
        }

    case 490:  /* *ashlsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return TYPE_LEA;
        }
      else if (which_alternative == 2)
        {
	  return TYPE_ISHIFTX;
        }
      else if ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))
        {
	  return TYPE_ALU;
        }
      else
        {
	  return TYPE_ISHIFT;
        }

    case 491:  /* *ashlhi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 1)
        {
	  return TYPE_LEA;
        }
      else if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (register_operand (operands[0], VOIDmode))) && (const1_operand (operands[2], VOIDmode)))
        {
	  return TYPE_ALU;
        }
      else
        {
	  return TYPE_ISHIFT;
        }

    case 492:  /* *ashlqi3_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 2)
        {
	  return TYPE_LEA;
        }
      else if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (register_operand (operands[0], VOIDmode))) && (const1_operand (operands[2], VOIDmode)))
        {
	  return TYPE_ALU;
        }
      else
        {
	  return TYPE_ISHIFT;
        }

    case 493:  /* *ashlqi3_1_slp */
      extract_constrain_insn_cached (insn);
      if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (register_operand (operands[0], VOIDmode))) && (const1_operand (operands[1], VOIDmode)))
        {
	  return TYPE_ALU;
        }
      else
        {
	  return TYPE_ISHIFT1;
        }

    case 498:  /* *ashlsi3_cmp_zext */
      extract_constrain_insn_cached (insn);
      if ((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (const1_operand (operands[2], VOIDmode)))
        {
	  return TYPE_ALU;
        }
      else
        {
	  return TYPE_ISHIFT;
        }

    case 494:  /* *ashlqi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 497:  /* *ashldi3_cmp */
    case 499:  /* *ashlqi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 502:  /* *ashldi3_cconly */
      extract_constrain_insn_cached (insn);
      if (((
#line 9565 "../../gcc/config/i386/i386.md"
(TARGET_DOUBLE_WITH_ADD)) && (register_operand (operands[0], VOIDmode))) && (const1_operand (operands[2], VOIDmode)))
        {
	  return TYPE_ALU;
        }
      else
        {
	  return TYPE_ISHIFT;
        }

    case 513:  /* ashrdi3_cvt */
    case 514:  /* ashrsi3_cvt */
    case 515:  /* *ashrsi3_cvt_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_IMOVX;
        }
      else
        {
	  return TYPE_ISHIFT;
        }

    case 520:  /* *lshrsi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 523:  /* *ashrdi3_1 */
    case 526:  /* *lshrsi3_1_zext */
    case 527:  /* *ashrsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_ISHIFT;
        }
      else
        {
	  return TYPE_ISHIFTX;
        }

    case 562:  /* *rotlsi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 564:  /* *rotldi3_1 */
    case 565:  /* *rotrdi3_1 */
    case 567:  /* *rotlsi3_1_zext */
    case 568:  /* *rotrsi3_1_zext */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_ROTATE;
        }
      else
        {
	  return TYPE_ROTATEX;
        }

    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6))
        {
	  if (mult_operator (operands[3], SFmode))
	    {
	      return TYPE_SSEMUL;
	    }
	  else
	    {
	      return TYPE_SSEADD;
	    }
        }
      else
        {
	  if (mult_operator (operands[3], SFmode))
	    {
	      return TYPE_FMUL;
	    }
	  else
	    {
	      return TYPE_FOP;
	    }
        }

    case 738:  /* *fop_df_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x6))
        {
	  if (mult_operator (operands[3], DFmode))
	    {
	      return TYPE_SSEMUL;
	    }
	  else
	    {
	      return TYPE_SSEADD;
	    }
        }
      else
        {
	  if (mult_operator (operands[3], DFmode))
	    {
	      return TYPE_FMUL;
	    }
	  else
	    {
	      return TYPE_FOP;
	    }
        }

    case 739:  /* *fop_sf_comm_sse */
      extract_insn_cached (insn);
      if (mult_operator (operands[3], SFmode))
        {
	  return TYPE_SSEMUL;
        }
      else
        {
	  return TYPE_SSEADD;
        }

    case 740:  /* *fop_df_comm_sse */
      extract_insn_cached (insn);
      if (mult_operator (operands[3], DFmode))
        {
	  return TYPE_SSEMUL;
        }
      else
        {
	  return TYPE_SSEADD;
        }

    case 741:  /* *fop_sf_comm_i387 */
      extract_insn_cached (insn);
      if (mult_operator (operands[3], SFmode))
        {
	  return TYPE_FMUL;
        }
      else
        {
	  return TYPE_FOP;
        }

    case 742:  /* *fop_df_comm_i387 */
      extract_insn_cached (insn);
      if (mult_operator (operands[3], DFmode))
        {
	  return TYPE_FMUL;
        }
      else
        {
	  return TYPE_FOP;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode)))
        {
	  return TYPE_SSEMUL;
        }
      else if ((((1 << which_alternative) & 0xc)) && (div_operator (operands[3], SFmode)))
        {
	  return TYPE_SSEDIV;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return TYPE_SSEADD;
        }
      else if (mult_operator (operands[3], SFmode))
        {
	  return TYPE_FMUL;
        }
      else if (div_operator (operands[3], SFmode))
        {
	  return TYPE_FDIV;
        }
      else
        {
	  return TYPE_FOP;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if ((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode)))
        {
	  return TYPE_SSEMUL;
        }
      else if ((((1 << which_alternative) & 0xc)) && (div_operator (operands[3], DFmode)))
        {
	  return TYPE_SSEDIV;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return TYPE_SSEADD;
        }
      else if (mult_operator (operands[3], DFmode))
        {
	  return TYPE_FMUL;
        }
      else if (div_operator (operands[3], DFmode))
        {
	  return TYPE_FDIV;
        }
      else
        {
	  return TYPE_FOP;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (mult_operator (operands[3], SFmode))
        {
	  return TYPE_SSEMUL;
        }
      else if (div_operator (operands[3], SFmode))
        {
	  return TYPE_SSEDIV;
        }
      else
        {
	  return TYPE_SSEADD;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (mult_operator (operands[3], DFmode))
        {
	  return TYPE_SSEMUL;
        }
      else if (div_operator (operands[3], DFmode))
        {
	  return TYPE_SSEDIV;
        }
      else
        {
	  return TYPE_SSEADD;
        }

    case 748:  /* *fop_sf_1_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 756:  /* *fop_sf_3_i387 */
      extract_insn_cached (insn);
      if (mult_operator (operands[3], SFmode))
        {
	  return TYPE_FMUL;
        }
      else if (div_operator (operands[3], SFmode))
        {
	  return TYPE_FDIV;
        }
      else
        {
	  return TYPE_FOP;
        }

    case 749:  /* *fop_df_1_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 760:  /* *fop_df_6_i387 */
      extract_insn_cached (insn);
      if (mult_operator (operands[3], DFmode))
        {
	  return TYPE_FMUL;
        }
      else if (div_operator (operands[3], DFmode))
        {
	  return TYPE_FDIV;
        }
      else
        {
	  return TYPE_FOP;
        }

    case 761:  /* *fop_xf_comm_i387 */
      extract_insn_cached (insn);
      if (mult_operator (operands[3], XFmode))
        {
	  return TYPE_FMUL;
        }
      else
        {
	  return TYPE_FOP;
        }

    case 762:  /* *fop_xf_1_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 772:  /* *fop_xf_6_i387 */
      extract_insn_cached (insn);
      if (mult_operator (operands[3], XFmode))
        {
	  return TYPE_FMUL;
        }
      else if (div_operator (operands[3], XFmode))
        {
	  return TYPE_FDIV;
        }
      else
        {
	  return TYPE_FOP;
        }

    case 908:  /* pro_epilogue_adjust_stack_si_add */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))
        {
	  return TYPE_ALU;
        }
      else if (const0_operand (operands[2], SImode))
        {
	  return TYPE_IMOV;
        }
      else
        {
	  return TYPE_LEA;
        }

    case 909:  /* pro_epilogue_adjust_stack_di_add */
      extract_constrain_insn_cached (insn);
      if ((which_alternative == 0) && (! (
#line 16677 "../../gcc/config/i386/i386.md"
(TARGET_OPT_AGU))))
        {
	  return TYPE_ALU;
        }
      else if (const0_operand (operands[2], DImode))
        {
	  return TYPE_IMOV;
        }
      else
        {
	  return TYPE_LEA;
        }

    case 959:  /* *movv8qi_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_IMOV;
        }
      else if (which_alternative == 2)
        {
	  return TYPE_MMX;
        }
      else if (((1 << which_alternative) & 0x38))
        {
	  return TYPE_MMXMOV;
        }
      else if (((1 << which_alternative) & 0xc0))
        {
	  return TYPE_SSECVT;
        }
      else if (which_alternative == 8)
        {
	  return TYPE_SSELOG1;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 963:  /* *movv8qi_internal */
    case 964:  /* *movv4hi_internal */
    case 965:  /* *movv2si_internal */
    case 966:  /* *movv1di_internal */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_MMX;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return TYPE_MMXMOV;
        }
      else if (((1 << which_alternative) & 0x30))
        {
	  return TYPE_SSECVT;
        }
      else if (((1 << which_alternative) & 0x240))
        {
	  return TYPE_SSELOG1;
        }
      else if (((1 << which_alternative) & 0x6000))
        {
	  return TYPE_MULTI;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 967:  /* *movv2sf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return TYPE_IMOV;
        }
      else if (which_alternative == 2)
        {
	  return TYPE_MMX;
        }
      else if (((1 << which_alternative) & 0x38))
        {
	  return TYPE_MMXMOV;
        }
      else if (((1 << which_alternative) & 0xc0))
        {
	  return TYPE_SSECVT;
        }
      else if (which_alternative == 9)
        {
	  return TYPE_SSELOG1;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_MMX;
        }
      else if (((1 << which_alternative) & 0xe))
        {
	  return TYPE_MMXMOV;
        }
      else if (((1 << which_alternative) & 0x30))
        {
	  return TYPE_SSECVT;
        }
      else if (which_alternative == 6)
        {
	  return TYPE_SSELOG1;
        }
      else if (((1 << which_alternative) & 0xc00))
        {
	  return TYPE_MULTI;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_MMXCVT;
        }
      else if (which_alternative == 1)
        {
	  return TYPE_SSELOG1;
        }
      else if (which_alternative == 2)
        {
	  return TYPE_MMXMOV;
        }
      else if (which_alternative == 3)
        {
	  return TYPE_SSEMOV;
        }
      else if (which_alternative == 4)
        {
	  return TYPE_FMOV;
        }
      else
        {
	  return TYPE_IMOV;
        }

    case 994:  /* *mmx_concatv2sf */
    case 1064:  /* *mmx_concatv2si */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_MMXCVT;
        }
      else
        {
	  return TYPE_MMXMOV;
        }

    case 1353:  /* vec_setv4si_0 */
    case 1354:  /* vec_setv4sf_0 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc1))
        {
	  return TYPE_SSELOG;
        }
      else if (which_alternative == 9)
        {
	  return TYPE_IMOV;
        }
      else if (which_alternative == 10)
        {
	  return TYPE_FMOV;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return TYPE_SSELOG;
        }
      else if (which_alternative == 9)
        {
	  return TYPE_FMOV;
        }
      else if (which_alternative == 10)
        {
	  return TYPE_IMOV;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 1389:  /* sse2_movsd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return TYPE_SSELOG;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 1375:  /* *vec_interleave_lowv2df */
    case 1373:  /* *vec_interleave_highv2df */
    case 1391:  /* *vec_concatv2df */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return TYPE_SSELOG;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 1349:  /* vec_dupv4sf */
    case 1583:  /* *vec_dupv4si */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_SSELOG1;
        }
      else if (which_alternative == 1)
        {
	  return TYPE_SSEMOV;
        }
      else
        {
	  return TYPE_SSELOG1;
        }

    case 1351:  /* *vec_concatv2sf_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1587:  /* *vec_concatv2si_sse */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_SSELOG;
        }
      else if (which_alternative == 1)
        {
	  return TYPE_SSEMOV;
        }
      else if (which_alternative == 2)
        {
	  return TYPE_MMXCVT;
        }
      else
        {
	  return TYPE_MMXMOV;
        }

    case 1589:  /* *vec_concatv2di_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x63))
        {
	  return TYPE_SSELOG;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 1924:  /* avx_vec_concatv32qi */
    case 1925:  /* avx_vec_concatv16hi */
    case 1926:  /* avx_vec_concatv8si */
    case 1927:  /* avx_vec_concatv4di */
    case 1928:  /* avx_vec_concatv8sf */
    case 1929:  /* avx_vec_concatv4df */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return TYPE_SSELOG;
        }
      else
        {
	  return TYPE_SSEMOV;
        }

    case 1023:  /* mmx_ashrv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1049:  /* mmx_packsswb */
    case 1050:  /* mmx_packssdw */
    case 1051:  /* mmx_packuswb */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1069:  /* mmx_psadbw */
      return TYPE_MMXSHFT;

    case 988:  /* mmx_pf2id */
    case 989:  /* mmx_pf2iw */
    case 990:  /* mmx_pi2fw */
    case 991:  /* mmx_floatv2si2 */
    case 992:  /* mmx_pswapdv2sf2 */
    case 993:  /* *vec_dupv2sf */
    case 1052:  /* mmx_punpckhbw */
    case 1053:  /* mmx_punpcklbw */
    case 1054:  /* mmx_punpckhwd */
    case 1055:  /* mmx_punpcklwd */
    case 1056:  /* mmx_punpckhdq */
    case 1057:  /* mmx_punpckldq */
    case 1058:  /* *mmx_pinsrw */
    case 1059:  /* mmx_pextrw */
    case 1060:  /* mmx_pshufw_1 */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1062:  /* *vec_dupv4hi */
    case 1063:  /* *vec_dupv2si */
    case 1070:  /* mmx_pmovmskb */
    case 1071:  /* *mmx_maskmovq */
    case 1072:  /* *mmx_maskmovq */
      return TYPE_MMXCVT;

    case 985:  /* *mmx_eqv2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 987:  /* mmx_gev2sf3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1036:  /* mmx_gtv2si3 */
      return TYPE_MMXCMP;

    case 972:  /* *mmx_mulv2sf3 */
    case 1013:  /* *mmx_mulv4hi3 */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1016:  /* *mmx_pmaddwd */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1018:  /* *sse2_umulv1siv1di3 */
      return TYPE_MMXMUL;

    case 970:  /* *mmx_addv2sf3 */
    case 971:  /* *mmx_subv2sf3 */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 975:  /* *mmx_smaxv2sf3 */
    case 976:  /* *mmx_sminv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 997:  /* *mmx_addv8qi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1048:  /* *mmx_xorv2si3 */
      return TYPE_MMXADD;

    case 969:  /* sse_movntq */
      return TYPE_MMXMOV;

    case 920:  /* *prefetch_3dnow */
    case 977:  /* mmx_rcpv2sf2 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 1073:  /* *mmx_emms */
    case 1074:  /* *mmx_femms */
      return TYPE_MMX;

    case 939:  /* *lwp_llwpcbsi1 */
    case 940:  /* *lwp_llwpcbdi1 */
    case 941:  /* lwp_slwpcbsi */
    case 942:  /* lwp_slwpcbdi */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 946:  /* *lwp_lwpinsdi3_1 */
      return TYPE_LWP;

    case 898:  /* *xop_pcmov_sf */
    case 899:  /* *xop_pcmov_df */
    case 1748:  /* xop_pcmov_v32qi256 */
    case 1749:  /* xop_pcmov_v16qi */
    case 1750:  /* xop_pcmov_v16hi256 */
    case 1751:  /* xop_pcmov_v8hi */
    case 1752:  /* xop_pcmov_v8si256 */
    case 1753:  /* xop_pcmov_v4si */
    case 1754:  /* xop_pcmov_v4di256 */
    case 1755:  /* xop_pcmov_v2di */
    case 1756:  /* xop_pcmov_v8sf256 */
    case 1757:  /* xop_pcmov_v4sf */
    case 1758:  /* xop_pcmov_v4df256 */
    case 1759:  /* xop_pcmov_v2df */
    case 1775:  /* xop_pperm */
    case 1776:  /* xop_pperm_pack_v2di_v4si */
    case 1777:  /* xop_pperm_pack_v4si_v8hi */
    case 1778:  /* xop_pperm_pack_v8hi_v16qi */
    case 1807:  /* xop_maskcmpv16qi3 */
    case 1808:  /* xop_maskcmpv8hi3 */
    case 1809:  /* xop_maskcmpv4si3 */
    case 1810:  /* xop_maskcmpv2di3 */
    case 1823:  /* xop_vpermil2v8sf3 */
    case 1824:  /* xop_vpermil2v4sf3 */
    case 1825:  /* xop_vpermil2v4df3 */
    case 1826:  /* xop_vpermil2v2df3 */
      return TYPE_SSE4ARG;

    case 1237:  /* *fma_fmadd_sf */
    case 1238:  /* *fma_fmadd_df */
    case 1239:  /* *fma_fmadd_v4sf */
    case 1240:  /* *fma_fmadd_v2df */
    case 1241:  /* *fma_fmadd_v8sf */
    case 1242:  /* *fma_fmadd_v4df */
    case 1243:  /* *fma_fmsub_sf */
    case 1244:  /* *fma_fmsub_df */
    case 1245:  /* *fma_fmsub_v4sf */
    case 1246:  /* *fma_fmsub_v2df */
    case 1247:  /* *fma_fmsub_v8sf */
    case 1248:  /* *fma_fmsub_v4df */
    case 1249:  /* *fma_fnmadd_sf */
    case 1250:  /* *fma_fnmadd_df */
    case 1251:  /* *fma_fnmadd_v4sf */
    case 1252:  /* *fma_fnmadd_v2df */
    case 1253:  /* *fma_fnmadd_v8sf */
    case 1254:  /* *fma_fnmadd_v4df */
    case 1255:  /* *fma_fnmsub_sf */
    case 1256:  /* *fma_fnmsub_df */
    case 1257:  /* *fma_fnmsub_v4sf */
    case 1258:  /* *fma_fnmsub_v2df */
    case 1259:  /* *fma_fnmsub_v8sf */
    case 1260:  /* *fma_fnmsub_v4df */
    case 1261:  /* *fma_fmaddsub_v8sf */
    case 1262:  /* *fma_fmaddsub_v4sf */
    case 1263:  /* *fma_fmaddsub_v4df */
    case 1264:  /* *fma_fmaddsub_v2df */
    case 1265:  /* *fma_fmsubadd_v8sf */
    case 1266:  /* *fma_fmsubadd_v4sf */
    case 1267:  /* *fma_fmsubadd_v4df */
    case 1268:  /* *fma_fmsubadd_v2df */
    case 1269:  /* *fmai_fmadd_v4sf */
    case 1270:  /* *fmai_fmadd_v2df */
    case 1271:  /* *fmai_fmsub_v4sf */
    case 1272:  /* *fmai_fmsub_v2df */
    case 1273:  /* *fmai_fnmadd_v4sf */
    case 1274:  /* *fmai_fnmadd_v2df */
    case 1275:  /* *fmai_fnmsub_v4sf */
    case 1276:  /* *fmai_fnmsub_v2df */
    case 1277:  /* *fma4i_vmfmadd_v4sf */
    case 1278:  /* *fma4i_vmfmadd_v2df */
    case 1279:  /* *fma4i_vmfmsub_v4sf */
    case 1280:  /* *fma4i_vmfmsub_v2df */
    case 1281:  /* *fma4i_vmfnmadd_v4sf */
    case 1282:  /* *fma4i_vmfnmadd_v2df */
    case 1283:  /* *fma4i_vmfnmsub_v4sf */
    case 1284:  /* *fma4i_vmfnmsub_v2df */
    case 1736:  /* xop_pmacsww */
    case 1737:  /* xop_pmacssww */
    case 1738:  /* xop_pmacsdd */
    case 1739:  /* xop_pmacssdd */
    case 1740:  /* xop_pmacssdql */
    case 1741:  /* xop_pmacssdqh */
    case 1742:  /* xop_pmacsdql */
    case 1743:  /* xop_pmacsdqh */
    case 1744:  /* xop_pmacsswd */
    case 1745:  /* xop_pmacswd */
    case 1746:  /* xop_pmadcsswd */
    case 1747:  /* xop_pmadcswd */
      return TYPE_SSEMULADD;

    case 1665:  /* sse4a_insertqi */
    case 1666:  /* sse4a_insertq */
      return TYPE_SSEINS;

    case 1135:  /* avx_divv8sf3 */
    case 1136:  /* sse_divv4sf3 */
    case 1137:  /* avx_divv4df3 */
    case 1138:  /* sse2_divv2df3 */
    case 1139:  /* sse_vmdivv4sf3 */
    case 1140:  /* sse2_vmdivv2df3 */
      return TYPE_SSEDIV;

    case 154:  /* fix_truncsfdi_sse */
    case 155:  /* fix_truncdfdi_sse */
    case 156:  /* fix_truncsfsi_sse */
    case 157:  /* fix_truncdfsi_sse */
    case 209:  /* *floatsisf2_vector_sse_with_temp */
    case 210:  /* *floatsidf2_vector_sse_with_temp */
    case 211:  /* *floatsisf2_vector_sse */
    case 212:  /* *floatsidf2_vector_sse */
    case 213:  /* *floatsisf2_sse_with_temp */
    case 214:  /* *floatsidf2_sse_with_temp */
    case 215:  /* *floatdisf2_sse_with_temp */
    case 216:  /* *floatdidf2_sse_with_temp */
    case 217:  /* *floatsisf2_sse_interunit */
    case 218:  /* *floatsidf2_sse_interunit */
    case 219:  /* *floatdisf2_sse_interunit */
    case 220:  /* *floatdidf2_sse_interunit */
    case 221:  /* *floatsisf2_sse_nointerunit */
    case 222:  /* *floatsidf2_sse_nointerunit */
    case 223:  /* *floatdisf2_sse_nointerunit */
    case 224:  /* *floatdidf2_sse_nointerunit */
    case 1288:  /* sse_cvtsi2ss */
    case 1289:  /* sse_cvtsi2ssq */
    case 1290:  /* sse_cvtss2si */
    case 1291:  /* sse_cvtss2si_2 */
    case 1292:  /* sse_cvtss2siq */
    case 1293:  /* sse_cvtss2siq_2 */
    case 1294:  /* sse_cvttss2si */
    case 1295:  /* sse_cvttss2siq */
    case 1305:  /* sse2_cvtsi2sd */
    case 1306:  /* sse2_cvtsi2sdq */
    case 1307:  /* sse2_cvtsd2si */
    case 1308:  /* sse2_cvtsd2si_2 */
    case 1309:  /* sse2_cvtsd2siq */
    case 1310:  /* sse2_cvtsd2siq_2 */
    case 1311:  /* sse2_cvttsd2si */
    case 1312:  /* sse2_cvttsd2siq */
      return TYPE_SSEICVT;

    case 1799:  /* xop_frczsf2 */
    case 1800:  /* xop_frczdf2 */
    case 1801:  /* xop_frczv4sf2 */
    case 1802:  /* xop_frczv2df2 */
    case 1803:  /* xop_frczv8sf2 */
    case 1804:  /* xop_frczv4df2 */
    case 1805:  /* *xop_vmfrczv4sf2 */
    case 1806:  /* *xop_vmfrczv2df2 */
      return TYPE_SSECVT1;

    case 136:  /* *extendsfdf2_sse */
    case 141:  /* *truncdfsf_fast_sse */
    case 809:  /* sse4_1_roundsf2 */
    case 810:  /* sse4_1_rounddf2 */
    case 1111:  /* avx_movntv4di */
    case 1112:  /* sse2_movntv2di */
    case 1285:  /* sse_cvtpi2ps */
    case 1286:  /* sse_cvtps2pi */
    case 1287:  /* sse_cvttps2pi */
    case 1296:  /* floatv8siv8sf2 */
    case 1297:  /* floatv4siv4sf2 */
    case 1298:  /* avx_cvtps2dq256 */
    case 1299:  /* sse2_cvtps2dq */
    case 1300:  /* fix_truncv8sfv8si2 */
    case 1301:  /* fix_truncv4sfv4si2 */
    case 1302:  /* sse2_cvtpi2pd */
    case 1303:  /* sse2_cvtpd2pi */
    case 1304:  /* sse2_cvttpd2pi */
    case 1313:  /* floatv4siv4df2 */
    case 1314:  /* avx_cvtdq2pd256_2 */
    case 1315:  /* sse2_cvtdq2pd */
    case 1316:  /* avx_cvtpd2dq256 */
    case 1317:  /* *avx_cvtpd2dq256_2 */
    case 1318:  /* *sse2_cvtpd2dq */
    case 1319:  /* fix_truncv4dfv4si2 */
    case 1320:  /* *avx_cvttpd2dq256_2 */
    case 1321:  /* *sse2_cvttpd2dq */
    case 1322:  /* sse2_cvtsd2ss */
    case 1323:  /* sse2_cvtss2sd */
    case 1324:  /* avx_cvtpd2ps256 */
    case 1325:  /* *sse2_cvtpd2ps */
    case 1326:  /* avx_cvtps2pd256 */
    case 1327:  /* *avx_cvtps2pd256_2 */
    case 1328:  /* sse2_cvtps2pd */
    case 1722:  /* avx_roundps256 */
    case 1723:  /* sse4_1_roundps */
    case 1724:  /* avx_roundpd256 */
    case 1725:  /* sse4_1_roundpd */
    case 1726:  /* sse4_1_roundss */
    case 1727:  /* sse4_1_roundsd */
    case 1930:  /* vcvtph2ps */
    case 1931:  /* *vcvtph2ps_load */
    case 1932:  /* vcvtph2ps256 */
    case 1933:  /* *vcvtps2ph */
    case 1934:  /* *vcvtps2ph_store */
    case 1935:  /* vcvtps2ph256 */
      return TYPE_SSECVT;

    case 35:  /* *cmpfp_i_sse */
    case 38:  /* *cmpfp_iu_sse */
    case 1209:  /* sse_comi */
    case 1210:  /* sse2_comi */
    case 1211:  /* sse_ucomi */
    case 1212:  /* sse2_ucomi */
    case 1716:  /* avx_vtestps256 */
    case 1717:  /* avx_vtestps */
    case 1718:  /* avx_vtestpd256 */
    case 1719:  /* avx_vtestpd */
    case 1720:  /* avx_ptest256 */
    case 1721:  /* sse4_1_ptest */
      return TYPE_SSECOMI;

    case 585:  /* setcc_sf_sse */
    case 586:  /* setcc_df_sse */
    case 1193:  /* avx_cmpv8sf3 */
    case 1194:  /* avx_cmpv4sf3 */
    case 1195:  /* avx_cmpv4df3 */
    case 1196:  /* avx_cmpv2df3 */
    case 1197:  /* avx_vmcmpv4sf3 */
    case 1198:  /* avx_vmcmpv2df3 */
    case 1199:  /* *avx_maskcmpv8sf3_comm */
    case 1200:  /* *sse_maskcmpv4sf3_comm */
    case 1201:  /* *avx_maskcmpv4df3_comm */
    case 1202:  /* *sse2_maskcmpv2df3_comm */
    case 1203:  /* avx_maskcmpv8sf3 */
    case 1204:  /* sse_maskcmpv4sf3 */
    case 1205:  /* avx_maskcmpv4df3 */
    case 1206:  /* sse2_maskcmpv2df3 */
    case 1207:  /* sse_vmmaskcmpv4sf3 */
    case 1208:  /* sse2_vmmaskcmpv2df3 */
    case 1487:  /* *avx2_eqv32qi3 */
    case 1488:  /* *avx2_eqv16hi3 */
    case 1489:  /* *avx2_eqv8si3 */
    case 1490:  /* *avx2_eqv4di3 */
    case 1491:  /* *sse4_1_eqv2di3 */
    case 1492:  /* *sse2_eqv16qi3 */
    case 1493:  /* *sse2_eqv8hi3 */
    case 1494:  /* *sse2_eqv4si3 */
    case 1495:  /* sse4_2_gtv2di3 */
    case 1496:  /* avx2_gtv32qi3 */
    case 1497:  /* avx2_gtv16hi3 */
    case 1498:  /* avx2_gtv8si3 */
    case 1499:  /* avx2_gtv4di3 */
    case 1500:  /* sse2_gtv16qi3 */
    case 1501:  /* sse2_gtv8hi3 */
    case 1502:  /* sse2_gtv4si3 */
    case 1811:  /* xop_maskcmp_unsv16qi3 */
    case 1812:  /* xop_maskcmp_unsv8hi3 */
    case 1813:  /* xop_maskcmp_unsv4si3 */
    case 1814:  /* xop_maskcmp_unsv2di3 */
    case 1815:  /* xop_maskcmp_uns2v16qi3 */
    case 1816:  /* xop_maskcmp_uns2v8hi3 */
    case 1817:  /* xop_maskcmp_uns2v4si3 */
    case 1818:  /* xop_maskcmp_uns2v2di3 */
    case 1819:  /* xop_pcom_tfv16qi3 */
    case 1820:  /* xop_pcom_tfv8hi3 */
    case 1821:  /* xop_pcom_tfv4si3 */
    case 1822:  /* xop_pcom_tfv2di3 */
      return TYPE_SSECMP;

    case 1129:  /* *mulv8sf3 */
    case 1130:  /* *mulv4sf3 */
    case 1131:  /* *mulv4df3 */
    case 1132:  /* *mulv2df3 */
    case 1133:  /* sse_vmmulv4sf3 */
    case 1134:  /* sse2_vmmulv2df3 */
    case 1675:  /* avx_dpps256 */
    case 1676:  /* sse4_1_dpps */
    case 1677:  /* avx_dppd256 */
    case 1678:  /* sse4_1_dppd */
      return TYPE_SSEMUL;

    case 900:  /* smaxsf3 */
    case 901:  /* sminsf3 */
    case 902:  /* smaxdf3 */
    case 903:  /* smindf3 */
    case 904:  /* *ieee_sminsf3 */
    case 905:  /* *ieee_smindf3 */
    case 906:  /* *ieee_smaxsf3 */
    case 907:  /* *ieee_smaxdf3 */
    case 1117:  /* *addv8sf3 */
    case 1118:  /* *subv8sf3 */
    case 1119:  /* *addv4sf3 */
    case 1120:  /* *subv4sf3 */
    case 1121:  /* *addv4df3 */
    case 1122:  /* *subv4df3 */
    case 1123:  /* *addv2df3 */
    case 1124:  /* *subv2df3 */
    case 1125:  /* sse_vmaddv4sf3 */
    case 1126:  /* sse_vmsubv4sf3 */
    case 1127:  /* sse2_vmaddv2df3 */
    case 1128:  /* sse2_vmsubv2df3 */
    case 1153:  /* *smaxv8sf3_finite */
    case 1154:  /* *sminv8sf3_finite */
    case 1155:  /* *smaxv4sf3_finite */
    case 1156:  /* *sminv4sf3_finite */
    case 1157:  /* *smaxv4df3_finite */
    case 1158:  /* *sminv4df3_finite */
    case 1159:  /* *smaxv2df3_finite */
    case 1160:  /* *sminv2df3_finite */
    case 1161:  /* *smaxv8sf3 */
    case 1162:  /* *sminv8sf3 */
    case 1163:  /* *smaxv4sf3 */
    case 1164:  /* *sminv4sf3 */
    case 1165:  /* *smaxv4df3 */
    case 1166:  /* *sminv4df3 */
    case 1167:  /* *smaxv2df3 */
    case 1168:  /* *sminv2df3 */
    case 1173:  /* *ieee_sminv8sf3 */
    case 1174:  /* *ieee_sminv4sf3 */
    case 1175:  /* *ieee_sminv4df3 */
    case 1176:  /* *ieee_sminv2df3 */
    case 1177:  /* *ieee_smaxv8sf3 */
    case 1178:  /* *ieee_smaxv4sf3 */
    case 1179:  /* *ieee_smaxv4df3 */
    case 1180:  /* *ieee_smaxv2df3 */
    case 1181:  /* avx_addsubv4df3 */
    case 1182:  /* sse3_addsubv2df3 */
    case 1183:  /* avx_addsubv8sf3 */
    case 1184:  /* sse3_addsubv4sf3 */
    case 1185:  /* avx_haddv4df3 */
    case 1186:  /* avx_hsubv4df3 */
    case 1187:  /* sse3_haddv2df3 */
    case 1188:  /* sse3_hsubv2df3 */
    case 1189:  /* avx_haddv8sf3 */
    case 1190:  /* avx_hsubv8sf3 */
    case 1191:  /* sse3_haddv4sf3 */
    case 1192:  /* sse3_hsubv4sf3 */
      return TYPE_SSEADD;

    case 852:  /* movmsk_df */
    case 1089:  /* sse2_movq128 */
    case 1091:  /* avx_loadups256 */
    case 1092:  /* sse_loadups */
    case 1093:  /* avx_loadupd256 */
    case 1094:  /* sse2_loadupd */
    case 1095:  /* avx_storeups256 */
    case 1096:  /* sse_storeups */
    case 1097:  /* avx_storeupd256 */
    case 1098:  /* sse2_storeupd */
    case 1099:  /* avx_loaddqu256 */
    case 1100:  /* sse2_loaddqu */
    case 1101:  /* avx_storedqu256 */
    case 1102:  /* sse2_storedqu */
    case 1103:  /* avx_lddqu256 */
    case 1104:  /* sse3_lddqu */
    case 1105:  /* sse2_movntisi */
    case 1106:  /* sse2_movntidi */
    case 1107:  /* avx_movntv8sf */
    case 1108:  /* sse_movntv4sf */
    case 1109:  /* avx_movntv4df */
    case 1110:  /* sse2_movntv2df */
    case 1329:  /* sse_movhlps */
    case 1330:  /* sse_movlhps */
    case 1342:  /* sse_storehps */
    case 1343:  /* sse_loadhps */
    case 1344:  /* sse_storelps */
    case 1346:  /* sse_movss */
    case 1352:  /* *vec_concatv4sf */
    case 1384:  /* *vec_extractv2df_1_sse */
    case 1386:  /* *vec_extractv2df_0_sse */
    case 1576:  /* sse2_loadld */
    case 1597:  /* avx_movmskps256 */
    case 1598:  /* sse_movmskps */
    case 1599:  /* avx_movmskpd256 */
    case 1600:  /* sse2_movmskpd */
    case 1601:  /* avx2_pmovmskb */
    case 1602:  /* sse2_pmovmskb */
    case 1603:  /* *sse2_maskmovdqu */
    case 1604:  /* *sse2_maskmovdqu */
    case 1659:  /* sse4a_movntsf */
    case 1660:  /* sse4a_movntdf */
    case 1661:  /* sse4a_vmmovntv4sf */
    case 1662:  /* sse4a_vmmovntv2df */
    case 1667:  /* avx_blendps256 */
    case 1668:  /* sse4_1_blendps */
    case 1669:  /* avx_blendpd256 */
    case 1670:  /* sse4_1_blendpd */
    case 1671:  /* avx_blendvps256 */
    case 1672:  /* sse4_1_blendvps */
    case 1673:  /* avx_blendvpd256 */
    case 1674:  /* sse4_1_blendvpd */
    case 1679:  /* avx2_movntdqa */
    case 1680:  /* sse4_1_movntdqa */
    case 1685:  /* avx2_pblendvb */
    case 1686:  /* sse4_1_pblendvb */
    case 1687:  /* sse4_1_pblendw */
    case 1688:  /* *avx2_pblendw */
    case 1689:  /* avx2_pblenddv8si */
    case 1690:  /* avx2_pblenddv4si */
    case 1692:  /* avx2_sign_extendv16qiv16hi2 */
    case 1693:  /* avx2_zero_extendv16qiv16hi2 */
    case 1694:  /* sse4_1_sign_extendv8qiv8hi2 */
    case 1695:  /* sse4_1_zero_extendv8qiv8hi2 */
    case 1696:  /* avx2_sign_extendv8qiv8si2 */
    case 1697:  /* avx2_zero_extendv8qiv8si2 */
    case 1698:  /* sse4_1_sign_extendv4qiv4si2 */
    case 1699:  /* sse4_1_zero_extendv4qiv4si2 */
    case 1700:  /* avx2_sign_extendv8hiv8si2 */
    case 1701:  /* avx2_zero_extendv8hiv8si2 */
    case 1702:  /* sse4_1_sign_extendv4hiv4si2 */
    case 1703:  /* sse4_1_zero_extendv4hiv4si2 */
    case 1704:  /* avx2_sign_extendv4qiv4di2 */
    case 1705:  /* avx2_zero_extendv4qiv4di2 */
    case 1706:  /* sse4_1_sign_extendv2qiv2di2 */
    case 1707:  /* sse4_1_zero_extendv2qiv2di2 */
    case 1708:  /* avx2_sign_extendv4hiv4di2 */
    case 1709:  /* avx2_zero_extendv4hiv4di2 */
    case 1710:  /* sse4_1_sign_extendv2hiv2di2 */
    case 1711:  /* sse4_1_zero_extendv2hiv2di2 */
    case 1712:  /* avx2_sign_extendv4siv4di2 */
    case 1713:  /* avx2_zero_extendv4siv4di2 */
    case 1714:  /* sse4_1_sign_extendv2siv2di2 */
    case 1715:  /* sse4_1_zero_extendv2siv2di2 */
    case 1836:  /* avx2_pbroadcastv32qi */
    case 1837:  /* avx2_pbroadcastv16qi */
    case 1838:  /* avx2_pbroadcastv16hi */
    case 1839:  /* avx2_pbroadcastv8hi */
    case 1840:  /* avx2_pbroadcastv8si */
    case 1841:  /* avx2_pbroadcastv4si */
    case 1842:  /* avx2_pbroadcastv4di */
    case 1843:  /* avx2_pbroadcastv2di */
    case 1850:  /* vec_dupv8si */
    case 1851:  /* vec_dupv8sf */
    case 1852:  /* vec_dupv4di */
    case 1853:  /* vec_dupv4df */
    case 1854:  /* avx2_vbroadcasti128_v32qi */
    case 1855:  /* avx2_vbroadcasti128_v16hi */
    case 1856:  /* avx2_vbroadcasti128_v8si */
    case 1857:  /* avx2_vbroadcasti128_v4di */
    case 1936:  /* *avx2_gathersiv2di */
    case 1937:  /* *avx2_gathersiv2di */
    case 1938:  /* *avx2_gathersiv2df */
    case 1939:  /* *avx2_gathersiv2df */
    case 1940:  /* *avx2_gathersiv4di */
    case 1941:  /* *avx2_gathersiv4di */
    case 1942:  /* *avx2_gathersiv4df */
    case 1943:  /* *avx2_gathersiv4df */
    case 1944:  /* *avx2_gathersiv4si */
    case 1945:  /* *avx2_gathersiv4si */
    case 1946:  /* *avx2_gathersiv4sf */
    case 1947:  /* *avx2_gathersiv4sf */
    case 1948:  /* *avx2_gathersiv8si */
    case 1949:  /* *avx2_gathersiv8si */
    case 1950:  /* *avx2_gathersiv8sf */
    case 1951:  /* *avx2_gathersiv8sf */
    case 1952:  /* *avx2_gathersiv2di_2 */
    case 1953:  /* *avx2_gathersiv2di_2 */
    case 1954:  /* *avx2_gathersiv2df_2 */
    case 1955:  /* *avx2_gathersiv2df_2 */
    case 1956:  /* *avx2_gathersiv4di_2 */
    case 1957:  /* *avx2_gathersiv4di_2 */
    case 1958:  /* *avx2_gathersiv4df_2 */
    case 1959:  /* *avx2_gathersiv4df_2 */
    case 1960:  /* *avx2_gathersiv4si_2 */
    case 1961:  /* *avx2_gathersiv4si_2 */
    case 1962:  /* *avx2_gathersiv4sf_2 */
    case 1963:  /* *avx2_gathersiv4sf_2 */
    case 1964:  /* *avx2_gathersiv8si_2 */
    case 1965:  /* *avx2_gathersiv8si_2 */
    case 1966:  /* *avx2_gathersiv8sf_2 */
    case 1967:  /* *avx2_gathersiv8sf_2 */
    case 1968:  /* *avx2_gatherdiv2di */
    case 1969:  /* *avx2_gatherdiv2di */
    case 1970:  /* *avx2_gatherdiv2df */
    case 1971:  /* *avx2_gatherdiv2df */
    case 1972:  /* *avx2_gatherdiv4di */
    case 1973:  /* *avx2_gatherdiv4di */
    case 1974:  /* *avx2_gatherdiv4df */
    case 1975:  /* *avx2_gatherdiv4df */
    case 1976:  /* *avx2_gatherdiv4si */
    case 1977:  /* *avx2_gatherdiv4si */
    case 1978:  /* *avx2_gatherdiv4sf */
    case 1979:  /* *avx2_gatherdiv4sf */
    case 1980:  /* *avx2_gatherdiv8si */
    case 1981:  /* *avx2_gatherdiv8si */
    case 1982:  /* *avx2_gatherdiv8sf */
    case 1983:  /* *avx2_gatherdiv8sf */
    case 1984:  /* *avx2_gatherdiv2di_2 */
    case 1985:  /* *avx2_gatherdiv2di_2 */
    case 1986:  /* *avx2_gatherdiv2df_2 */
    case 1987:  /* *avx2_gatherdiv2df_2 */
    case 1988:  /* *avx2_gatherdiv4di_2 */
    case 1989:  /* *avx2_gatherdiv4di_2 */
    case 1990:  /* *avx2_gatherdiv4df_2 */
    case 1991:  /* *avx2_gatherdiv4df_2 */
    case 1992:  /* *avx2_gatherdiv4si_2 */
    case 1993:  /* *avx2_gatherdiv4si_2 */
    case 1994:  /* *avx2_gatherdiv4sf_2 */
    case 1995:  /* *avx2_gatherdiv4sf_2 */
    case 1996:  /* *avx2_gatherdiv8si_2 */
    case 1997:  /* *avx2_gatherdiv8si_2 */
    case 1998:  /* *avx2_gatherdiv8sf_2 */
    case 1999:  /* *avx2_gatherdiv8sf_2 */
    case 2000:  /* *avx2_gatherdiv8si_3 */
    case 2001:  /* *avx2_gatherdiv8si_3 */
    case 2002:  /* *avx2_gatherdiv8sf_3 */
    case 2003:  /* *avx2_gatherdiv8sf_3 */
    case 2004:  /* *avx2_gatherdiv8si_4 */
    case 2005:  /* *avx2_gatherdiv8si_4 */
    case 2006:  /* *avx2_gatherdiv8sf_4 */
    case 2007:  /* *avx2_gatherdiv8sf_4 */
      return TYPE_SSEMOV;

    case 745:  /* *rcpsf2_sse */
    case 778:  /* *rsqrtsf2_sse */
    case 779:  /* *sqrtsf2_sse */
    case 780:  /* *sqrtdf2_sse */
    case 919:  /* *prefetch_sse */
    case 1141:  /* avx_rcpv8sf2 */
    case 1142:  /* sse_rcpv4sf2 */
    case 1143:  /* sse_vmrcpv4sf2 */
    case 1144:  /* avx_sqrtv8sf2 */
    case 1145:  /* sse_sqrtv4sf2 */
    case 1146:  /* avx_sqrtv4df2 */
    case 1147:  /* sse2_sqrtv2df2 */
    case 1148:  /* sse_vmsqrtv4sf2 */
    case 1149:  /* sse2_vmsqrtv2df2 */
    case 1150:  /* avx_rsqrtv8sf2 */
    case 1151:  /* sse_rsqrtv4sf2 */
    case 1152:  /* sse_vmrsqrtv4sf2 */
    case 1169:  /* sse_vmsmaxv4sf3 */
    case 1170:  /* sse_vmsminv4sf3 */
    case 1171:  /* sse2_vmsmaxv2df3 */
    case 1172:  /* sse2_vmsminv2df3 */
    case 1335:  /* avx_movshdup256 */
    case 1336:  /* sse3_movshdup */
    case 1337:  /* avx_movsldup256 */
    case 1338:  /* sse3_movsldup */
    case 1605:  /* sse_ldmxcsr */
    case 1606:  /* sse_stmxcsr */
    case 1607:  /* sse2_clflush */
    case 1663:  /* sse4a_extrqi */
    case 1664:  /* sse4a_extrq */
    case 1834:  /* *avx_vzeroall */
    case 1835:  /* avx_vzeroupper */
    case 2008:  /* *sse2_lfence */
    case 2009:  /* *sse_sfence */
    case 2010:  /* mfence_sse2 */
      return TYPE_SSE;

    case 1426:  /* *mulv16hi3 */
    case 1427:  /* *mulv8hi3 */
    case 1428:  /* *smulv16hi3_highpart */
    case 1429:  /* *umulv16hi3_highpart */
    case 1430:  /* *smulv8hi3_highpart */
    case 1431:  /* *umulv8hi3_highpart */
    case 1432:  /* *avx_umulv4siv4di3 */
    case 1433:  /* *sse2_umulv2siv2di3 */
    case 1434:  /* *avx2_mulv4siv4di3 */
    case 1435:  /* *sse4_1_mulv2siv2di3 */
    case 1438:  /* *avx2_mulv8si3 */
    case 1439:  /* *sse4_1_mulv4si3 */
    case 1632:  /* *avx2_umulhrswv16hi3 */
    case 1633:  /* *ssse3_pmulhrswv8hi3 */
    case 1634:  /* *ssse3_pmulhrswv4hi3 */
      return TYPE_SSEIMUL;

    case 1443:  /* ashrv16hi3 */
    case 1444:  /* ashrv8hi3 */
    case 1445:  /* ashrv8si3 */
    case 1446:  /* ashrv4si3 */
    case 1447:  /* ashlv16hi3 */
    case 1448:  /* lshrv16hi3 */
    case 1449:  /* ashlv8hi3 */
    case 1450:  /* lshrv8hi3 */
    case 1451:  /* ashlv8si3 */
    case 1452:  /* lshrv8si3 */
    case 1453:  /* ashlv4si3 */
    case 1454:  /* lshrv4si3 */
    case 1455:  /* ashlv4di3 */
    case 1456:  /* lshrv4di3 */
    case 1457:  /* ashlv2di3 */
    case 1458:  /* lshrv2di3 */
    case 1459:  /* avx2_ashlv2ti3 */
    case 1460:  /* sse2_ashlv1ti3 */
    case 1461:  /* avx2_lshrv2ti3 */
    case 1462:  /* sse2_lshrv1ti3 */
    case 1647:  /* avx2_palignrv2ti */
    case 1648:  /* ssse3_palignrti */
    case 1649:  /* ssse3_palignrdi */
    case 1779:  /* xop_rotlv16qi3 */
    case 1780:  /* xop_rotlv8hi3 */
    case 1781:  /* xop_rotlv4si3 */
    case 1782:  /* xop_rotlv2di3 */
    case 1783:  /* xop_rotrv16qi3 */
    case 1784:  /* xop_rotrv8hi3 */
    case 1785:  /* xop_rotrv4si3 */
    case 1786:  /* xop_rotrv2di3 */
    case 1787:  /* xop_vrotlv16qi3 */
    case 1788:  /* xop_vrotlv8hi3 */
    case 1789:  /* xop_vrotlv4si3 */
    case 1790:  /* xop_vrotlv2di3 */
    case 1791:  /* xop_shav16qi3 */
    case 1792:  /* xop_shav8hi3 */
    case 1793:  /* xop_shav4si3 */
    case 1794:  /* xop_shav2di3 */
    case 1795:  /* xop_shlv16qi3 */
    case 1796:  /* xop_shlv8hi3 */
    case 1797:  /* xop_shlv4si3 */
    case 1798:  /* xop_shlv2di3 */
    case 1914:  /* avx2_ashrvv8si */
    case 1915:  /* avx2_ashrvv4si */
    case 1916:  /* avx2_ashlvv8si */
    case 1917:  /* avx2_lshrvv8si */
    case 1918:  /* avx2_ashlvv4si */
    case 1919:  /* avx2_lshrvv4si */
    case 1920:  /* avx2_ashlvv4di */
    case 1921:  /* avx2_lshrvv4di */
    case 1922:  /* avx2_ashlvv2di */
    case 1923:  /* avx2_lshrvv2di */
      return TYPE_SSEISHFT;

    case 1760:  /* xop_phaddbw */
    case 1761:  /* xop_phaddbd */
    case 1762:  /* xop_phaddbq */
    case 1763:  /* xop_phaddwd */
    case 1764:  /* xop_phaddwq */
    case 1765:  /* xop_phadddq */
    case 1766:  /* xop_phaddubw */
    case 1767:  /* xop_phaddubd */
    case 1768:  /* xop_phaddubq */
    case 1769:  /* xop_phadduwd */
    case 1770:  /* xop_phadduwq */
    case 1771:  /* xop_phaddudq */
    case 1772:  /* xop_phsubbw */
    case 1773:  /* xop_phsubwd */
    case 1774:  /* xop_phsubdq */
      return TYPE_SSEIADD1;

    case 1392:  /* *addv32qi3 */
    case 1393:  /* *subv32qi3 */
    case 1394:  /* *addv16qi3 */
    case 1395:  /* *subv16qi3 */
    case 1396:  /* *addv16hi3 */
    case 1397:  /* *subv16hi3 */
    case 1398:  /* *addv8hi3 */
    case 1399:  /* *subv8hi3 */
    case 1400:  /* *addv8si3 */
    case 1401:  /* *subv8si3 */
    case 1402:  /* *addv4si3 */
    case 1403:  /* *subv4si3 */
    case 1404:  /* *addv4di3 */
    case 1405:  /* *subv4di3 */
    case 1406:  /* *addv2di3 */
    case 1407:  /* *subv2di3 */
    case 1408:  /* *avx2_ssaddv32qi3 */
    case 1409:  /* *avx2_usaddv32qi3 */
    case 1410:  /* *avx2_sssubv32qi3 */
    case 1411:  /* *avx2_ussubv32qi3 */
    case 1412:  /* *sse2_ssaddv16qi3 */
    case 1413:  /* *sse2_usaddv16qi3 */
    case 1414:  /* *sse2_sssubv16qi3 */
    case 1415:  /* *sse2_ussubv16qi3 */
    case 1416:  /* *avx2_ssaddv16hi3 */
    case 1417:  /* *avx2_usaddv16hi3 */
    case 1418:  /* *avx2_sssubv16hi3 */
    case 1419:  /* *avx2_ussubv16hi3 */
    case 1420:  /* *sse2_ssaddv8hi3 */
    case 1421:  /* *sse2_usaddv8hi3 */
    case 1422:  /* *sse2_sssubv8hi3 */
    case 1423:  /* *sse2_ussubv8hi3 */
    case 1436:  /* *avx2_pmaddwd */
    case 1437:  /* *sse2_pmaddwd */
    case 1463:  /* *avx2_smaxv32qi3 */
    case 1464:  /* *avx2_sminv32qi3 */
    case 1465:  /* *avx2_umaxv32qi3 */
    case 1466:  /* *avx2_uminv32qi3 */
    case 1467:  /* *avx2_smaxv16hi3 */
    case 1468:  /* *avx2_sminv16hi3 */
    case 1469:  /* *avx2_umaxv16hi3 */
    case 1470:  /* *avx2_uminv16hi3 */
    case 1471:  /* *avx2_smaxv8si3 */
    case 1472:  /* *avx2_sminv8si3 */
    case 1473:  /* *avx2_umaxv8si3 */
    case 1474:  /* *avx2_uminv8si3 */
    case 1475:  /* *sse4_1_smaxv16qi3 */
    case 1476:  /* *sse4_1_sminv16qi3 */
    case 1477:  /* *sse4_1_smaxv4si3 */
    case 1478:  /* *sse4_1_sminv4si3 */
    case 1479:  /* *smaxv8hi3 */
    case 1480:  /* *sminv8hi3 */
    case 1481:  /* *sse4_1_umaxv8hi3 */
    case 1482:  /* *sse4_1_uminv8hi3 */
    case 1483:  /* *sse4_1_umaxv4si3 */
    case 1484:  /* *sse4_1_uminv4si3 */
    case 1485:  /* *umaxv16qi3 */
    case 1486:  /* *uminv16qi3 */
    case 1591:  /* *avx2_uavgv32qi3 */
    case 1592:  /* *sse2_uavgv16qi3 */
    case 1593:  /* *avx2_uavgv16hi3 */
    case 1594:  /* *sse2_uavgv8hi3 */
    case 1595:  /* avx2_psadbw */
    case 1596:  /* sse2_psadbw */
    case 1611:  /* avx2_phaddwv16hi3 */
    case 1612:  /* ssse3_phaddwv8hi3 */
    case 1613:  /* ssse3_phaddwv4hi3 */
    case 1614:  /* avx2_phadddv8si3 */
    case 1615:  /* ssse3_phadddv4si3 */
    case 1616:  /* ssse3_phadddv2si3 */
    case 1617:  /* avx2_phaddswv16hi3 */
    case 1618:  /* ssse3_phaddswv8hi3 */
    case 1619:  /* ssse3_phaddswv4hi3 */
    case 1620:  /* avx2_phsubwv16hi3 */
    case 1621:  /* ssse3_phsubwv8hi3 */
    case 1622:  /* ssse3_phsubwv4hi3 */
    case 1623:  /* avx2_phsubdv8si3 */
    case 1624:  /* ssse3_phsubdv4si3 */
    case 1625:  /* ssse3_phsubdv2si3 */
    case 1626:  /* avx2_phsubswv16hi3 */
    case 1627:  /* ssse3_phsubswv8hi3 */
    case 1628:  /* ssse3_phsubswv4hi3 */
    case 1629:  /* avx2_pmaddubsw256 */
    case 1630:  /* ssse3_pmaddubsw128 */
    case 1631:  /* ssse3_pmaddubsw */
      return TYPE_SSEIADD;

    case 929:  /* sse4_2_crc32qi */
    case 930:  /* sse4_2_crc32hi */
    case 931:  /* sse4_2_crc32si */
    case 932:  /* sse4_2_crc32di */
    case 1347:  /* avx2_vec_dupv8sf */
    case 1348:  /* avx2_vec_dupv4sf */
    case 1390:  /* vec_dupv2df */
    case 1570:  /* avx2_pshufd_1 */
    case 1571:  /* sse2_pshufd_1 */
    case 1635:  /* avx2_pshufbv32qi3 */
    case 1636:  /* ssse3_pshufbv16qi3 */
    case 1637:  /* ssse3_pshufbv8qi3 */
    case 1638:  /* avx2_psignv32qi3 */
    case 1639:  /* ssse3_psignv16qi3 */
    case 1640:  /* avx2_psignv16hi3 */
    case 1641:  /* ssse3_psignv8hi3 */
    case 1642:  /* avx2_psignv8si3 */
    case 1643:  /* ssse3_psignv4si3 */
    case 1644:  /* ssse3_psignv8qi3 */
    case 1645:  /* ssse3_psignv4hi3 */
    case 1646:  /* ssse3_psignv2si3 */
    case 1650:  /* absv32qi2 */
    case 1651:  /* absv16qi2 */
    case 1652:  /* absv16hi2 */
    case 1653:  /* absv8hi2 */
    case 1654:  /* absv8si2 */
    case 1655:  /* absv4si2 */
    case 1656:  /* absv8qi2 */
    case 1657:  /* absv4hi2 */
    case 1658:  /* absv2si2 */
    case 1681:  /* avx2_mpsadbw */
    case 1682:  /* sse4_1_mpsadbw */
    case 1691:  /* sse4_1_phminposuw */
    case 1827:  /* aesenc */
    case 1828:  /* aesenclast */
    case 1829:  /* aesdec */
    case 1830:  /* aesdeclast */
    case 1831:  /* aesimc */
    case 1832:  /* aeskeygenassist */
    case 1833:  /* pclmulqdq */
    case 1849:  /* avx2_vec_dupv4df */
    case 1895:  /* avx_maskloadps */
    case 1896:  /* avx_maskloadpd */
    case 1897:  /* avx_maskloadps256 */
    case 1898:  /* avx_maskloadpd256 */
    case 1899:  /* avx2_maskloadd */
    case 1900:  /* avx2_maskloadq */
    case 1901:  /* avx2_maskloadd256 */
    case 1902:  /* avx2_maskloadq256 */
    case 1903:  /* avx_maskstoreps */
    case 1904:  /* avx_maskstorepd */
    case 1905:  /* avx_maskstoreps256 */
    case 1906:  /* avx_maskstorepd256 */
    case 1907:  /* avx2_maskstored */
    case 1908:  /* avx2_maskstoreq */
    case 1909:  /* avx2_maskstored256 */
    case 1910:  /* avx2_maskstoreq256 */
      return TYPE_SSELOG1;

    case 1213:  /* avx_andnotv8sf3 */
    case 1214:  /* sse_andnotv4sf3 */
    case 1215:  /* avx_andnotv4df3 */
    case 1216:  /* sse2_andnotv2df3 */
    case 1217:  /* *andv8sf3 */
    case 1218:  /* *iorv8sf3 */
    case 1219:  /* *xorv8sf3 */
    case 1220:  /* *andv4sf3 */
    case 1221:  /* *iorv4sf3 */
    case 1222:  /* *xorv4sf3 */
    case 1223:  /* *andv4df3 */
    case 1224:  /* *iorv4df3 */
    case 1225:  /* *xorv4df3 */
    case 1226:  /* *andv2df3 */
    case 1227:  /* *iorv2df3 */
    case 1228:  /* *xorv2df3 */
    case 1229:  /* *andnotsf3 */
    case 1230:  /* *andnotdf3 */
    case 1231:  /* *andsf3 */
    case 1232:  /* *iorsf3 */
    case 1233:  /* *xorsf3 */
    case 1234:  /* *anddf3 */
    case 1235:  /* *iordf3 */
    case 1236:  /* *xordf3 */
    case 1331:  /* avx_unpckhps256 */
    case 1332:  /* vec_interleave_highv4sf */
    case 1333:  /* avx_unpcklps256 */
    case 1334:  /* vec_interleave_lowv4sf */
    case 1339:  /* avx_shufps256_1 */
    case 1340:  /* sse_shufps_v4si */
    case 1341:  /* sse_shufps_v4sf */
    case 1355:  /* *vec_setv4sf_sse4_1 */
    case 1356:  /* sse4_1_insertps */
    case 1362:  /* vec_extract_hi_v4di */
    case 1363:  /* vec_extract_hi_v4df */
    case 1366:  /* vec_extract_hi_v8si */
    case 1367:  /* vec_extract_hi_v8sf */
    case 1369:  /* vec_extract_hi_v16hi */
    case 1371:  /* vec_extract_hi_v32qi */
    case 1372:  /* avx_unpckhpd256 */
    case 1374:  /* *avx_unpcklpd256 */
    case 1376:  /* avx_shufpd256_1 */
    case 1377:  /* avx2_interleave_highv4di */
    case 1378:  /* vec_interleave_highv2di */
    case 1379:  /* avx2_interleave_lowv4di */
    case 1380:  /* vec_interleave_lowv2di */
    case 1381:  /* sse2_shufpd_v2di */
    case 1382:  /* sse2_shufpd_v2df */
    case 1503:  /* *andnotv32qi3 */
    case 1504:  /* *andnotv16qi3 */
    case 1505:  /* *andnotv16hi3 */
    case 1506:  /* *andnotv8hi3 */
    case 1507:  /* *andnotv8si3 */
    case 1508:  /* *andnotv4si3 */
    case 1509:  /* *andnotv4di3 */
    case 1510:  /* *andnotv2di3 */
    case 1511:  /* *andv32qi3 */
    case 1512:  /* *iorv32qi3 */
    case 1513:  /* *xorv32qi3 */
    case 1514:  /* *andv16qi3 */
    case 1515:  /* *iorv16qi3 */
    case 1516:  /* *xorv16qi3 */
    case 1517:  /* *andv16hi3 */
    case 1518:  /* *iorv16hi3 */
    case 1519:  /* *xorv16hi3 */
    case 1520:  /* *andv8hi3 */
    case 1521:  /* *iorv8hi3 */
    case 1522:  /* *xorv8hi3 */
    case 1523:  /* *andv8si3 */
    case 1524:  /* *iorv8si3 */
    case 1525:  /* *xorv8si3 */
    case 1526:  /* *andv4si3 */
    case 1527:  /* *iorv4si3 */
    case 1528:  /* *xorv4si3 */
    case 1529:  /* *andv4di3 */
    case 1530:  /* *iorv4di3 */
    case 1531:  /* *xorv4di3 */
    case 1532:  /* *andv2di3 */
    case 1533:  /* *iorv2di3 */
    case 1534:  /* *xorv2di3 */
    case 1535:  /* *andnottf3 */
    case 1536:  /* *andtf3 */
    case 1537:  /* *iortf3 */
    case 1538:  /* *xortf3 */
    case 1539:  /* avx2_packsswb */
    case 1540:  /* sse2_packsswb */
    case 1541:  /* avx2_packssdw */
    case 1542:  /* sse2_packssdw */
    case 1543:  /* avx2_packuswb */
    case 1544:  /* sse2_packuswb */
    case 1545:  /* avx2_interleave_highv32qi */
    case 1546:  /* vec_interleave_highv16qi */
    case 1547:  /* avx2_interleave_lowv32qi */
    case 1548:  /* vec_interleave_lowv16qi */
    case 1549:  /* avx2_interleave_highv16hi */
    case 1550:  /* vec_interleave_highv8hi */
    case 1551:  /* avx2_interleave_lowv16hi */
    case 1552:  /* vec_interleave_lowv8hi */
    case 1553:  /* avx2_interleave_highv8si */
    case 1554:  /* vec_interleave_highv4si */
    case 1555:  /* avx2_interleave_lowv8si */
    case 1556:  /* vec_interleave_lowv4si */
    case 1557:  /* sse4_1_pinsrb */
    case 1558:  /* sse2_pinsrw */
    case 1559:  /* sse4_1_pinsrd */
    case 1560:  /* sse4_1_pinsrq */
    case 1561:  /* *sse4_1_pextrb_si */
    case 1562:  /* *sse4_1_pextrb_di */
    case 1563:  /* *sse4_1_pextrb_memory */
    case 1564:  /* *sse2_pextrw_si */
    case 1565:  /* *sse2_pextrw_di */
    case 1566:  /* *sse4_1_pextrw_memory */
    case 1567:  /* *sse4_1_pextrd */
    case 1568:  /* *sse4_1_pextrd_zext */
    case 1569:  /* *sse4_1_pextrq */
    case 1572:  /* avx2_pshuflw_1 */
    case 1573:  /* sse2_pshuflw_1 */
    case 1574:  /* avx2_pshufhw_1 */
    case 1575:  /* sse2_pshufhw_1 */
    case 1683:  /* avx2_packusdw */
    case 1684:  /* sse4_1_packusdw */
    case 1728:  /* sse4_2_pcmpestr */
    case 1729:  /* sse4_2_pcmpestri */
    case 1730:  /* sse4_2_pcmpestrm */
    case 1731:  /* sse4_2_pcmpestr_cconly */
    case 1732:  /* sse4_2_pcmpistr */
    case 1733:  /* sse4_2_pcmpistri */
    case 1734:  /* sse4_2_pcmpistrm */
    case 1735:  /* sse4_2_pcmpistr_cconly */
    case 1844:  /* avx2_permvarv8si */
    case 1845:  /* avx2_permv4df */
    case 1846:  /* avx2_permvarv8sf */
    case 1847:  /* avx2_permv4di_1 */
    case 1848:  /* avx2_permv2ti */
    case 1867:  /* *avx_vpermilpv8sf */
    case 1868:  /* *avx_vpermilpv4sf */
    case 1869:  /* *avx_vpermilpv4df */
    case 1870:  /* *avx_vpermilpv2df */
    case 1871:  /* avx_vpermilvarv8sf3 */
    case 1872:  /* avx_vpermilvarv4sf3 */
    case 1873:  /* avx_vpermilvarv4df3 */
    case 1874:  /* avx_vpermilvarv2df3 */
    case 1875:  /* *avx_vperm2f128v8si_full */
    case 1876:  /* *avx_vperm2f128v8sf_full */
    case 1877:  /* *avx_vperm2f128v4df_full */
    case 1878:  /* *avx_vperm2f128v8si_nozero */
    case 1879:  /* *avx_vperm2f128v8sf_nozero */
    case 1880:  /* *avx_vperm2f128v4df_nozero */
    case 1881:  /* avx2_vec_set_lo_v4di */
    case 1882:  /* avx2_vec_set_hi_v4di */
    case 1883:  /* vec_set_lo_v4di */
    case 1884:  /* vec_set_lo_v4df */
    case 1885:  /* vec_set_hi_v4di */
    case 1886:  /* vec_set_hi_v4df */
    case 1887:  /* vec_set_lo_v8si */
    case 1888:  /* vec_set_lo_v8sf */
    case 1889:  /* vec_set_hi_v8si */
    case 1890:  /* vec_set_hi_v8sf */
    case 1891:  /* vec_set_lo_v16hi */
    case 1892:  /* vec_set_hi_v16hi */
    case 1893:  /* vec_set_lo_v32qi */
    case 1894:  /* vec_set_hi_v32qi */
      return TYPE_SSELOG;

    case 821:  /* frndintxf2_floor */
    case 822:  /* frndintxf2_floor_i387 */
    case 832:  /* frndintxf2_ceil */
    case 833:  /* frndintxf2_ceil_i387 */
    case 843:  /* frndintxf2_trunc */
    case 844:  /* frndintxf2_trunc_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 846:  /* frndintxf2_mask_pm_i387 */
      return TYPE_FRNDINT;

    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
      return TYPE_FISTTP;

    case 167:  /* *fix_trunchi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 170:  /* fix_truncdi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 172:  /* fix_trunchi_i387 */
    case 173:  /* fix_truncsi_i387 */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 823:  /* *fisthi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 825:  /* *fistdi2_floor_1 */
    case 826:  /* fistdi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 828:  /* fisthi2_floor */
    case 829:  /* fistsi2_floor */
    case 830:  /* fisthi2_floor_with_temp */
    case 831:  /* fistsi2_floor_with_temp */
    case 834:  /* *fisthi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 836:  /* *fistdi2_ceil_1 */
    case 837:  /* fistdi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 839:  /* fisthi2_ceil */
    case 840:  /* fistsi2_ceil */
    case 841:  /* fisthi2_ceil_with_temp */
    case 842:  /* fistsi2_ceil_with_temp */
      return TYPE_FISTP;

    case 111:  /* swapxf */
    case 112:  /* *swapsf */
    case 113:  /* *swapdf */
      return TYPE_FXCH;

    case 36:  /* *cmpfp_i_i387 */
    case 39:  /* *cmpfp_iu_387 */
      return TYPE_FCMP;

    case 894:  /* *movxfcc_1 */
      return TYPE_FCMOV;

    case 775:  /* sqrtxf2 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 781:  /* fpremxf4_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 783:  /* *sinxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 789:  /* sincosxf3 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 792:  /* fptanxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 808:  /* *fscalexf4_i387 */
    case 811:  /* rintxf2 */
    case 812:  /* *fistdi2_1 */
    case 813:  /* fistdi2 */
    case 814:  /* fistdi2_with_temp */
    case 815:  /* *fisthi2_1 */
    case 816:  /* *fistsi2_1 */
    case 817:  /* fisthi2 */
    case 818:  /* fistsi2 */
    case 819:  /* fisthi2_with_temp */
    case 820:  /* fistsi2_with_temp */
      return TYPE_FPSPC;

    case 451:  /* *abssf2_1 */
    case 452:  /* *negsf2_1 */
    case 453:  /* *absdf2_1 */
    case 454:  /* *negdf2_1 */
    case 455:  /* *absxf2_1 */
    case 456:  /* *negxf2_1 */
    case 457:  /* *absextendsfdf2 */
    case 458:  /* *negextendsfdf2 */
    case 459:  /* *absextendsfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 462:  /* *negextenddfxf2 */
      return TYPE_FSGN;

    case 137:  /* *extendsfdf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 139:  /* *extenddfxf2_i387 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 145:  /* *truncdfsf2_i387_1 */
    case 148:  /* truncxfsf2_i387_noop */
    case 149:  /* truncxfdf2_i387_noop */
    case 150:  /* *truncxfsf2_i387 */
    case 151:  /* *truncxfdf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 186:  /* *floathixf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 236:  /* *floatdixf2_i387 */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 2018:  /* loaddi_via_fpu */
    case 2019:  /* storedi_via_fpu */
      return TYPE_FMOV;

    case 660:  /* clzhi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 662:  /* clzdi2_lzcnt */
    case 663:  /* *bmi_andn_si */
    case 664:  /* *bmi_andn_di */
    case 665:  /* bmi_bextr_si */
    case 666:  /* bmi_bextr_di */
    case 667:  /* *bmi_blsi_si */
    case 668:  /* *bmi_blsi_di */
    case 669:  /* *bmi_blsmsk_si */
    case 670:  /* *bmi_blsmsk_di */
    case 671:  /* *bmi_blsr_si */
    case 672:  /* *bmi_blsr_di */
    case 673:  /* bmi2_bzhi_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 678:  /* bmi2_pext_di3 */
    case 679:  /* tbm_bextri_si */
    case 680:  /* tbm_bextri_di */
    case 681:  /* *tbm_blcfill_si */
    case 682:  /* *tbm_blcfill_di */
    case 683:  /* *tbm_blci_si */
    case 684:  /* *tbm_blci_di */
    case 685:  /* *tbm_blcic_si */
    case 686:  /* *tbm_blcic_di */
    case 687:  /* *tbm_blcmsk_si */
    case 688:  /* *tbm_blcmsk_di */
    case 689:  /* *tbm_blcs_si */
    case 690:  /* *tbm_blcs_di */
    case 691:  /* *tbm_blsfill_si */
    case 692:  /* *tbm_blsfill_di */
    case 693:  /* *tbm_blsic_si */
    case 694:  /* *tbm_blsic_di */
    case 695:  /* *tbm_t1mskc_si */
    case 696:  /* *tbm_t1mskc_di */
    case 697:  /* *tbm_tzmsk_si */
    case 698:  /* *tbm_tzmsk_di */
    case 702:  /* popcounthi2 */
    case 703:  /* popcountsi2 */
    case 704:  /* popcountdi2 */
    case 705:  /* *popcounthi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 707:  /* *popcountdi2_cmp */
    case 708:  /* *popcountsi2_cmp_zext */
    case 711:  /* *bswapsi2_1 */
    case 712:  /* *bswapdi2_1 */
      return TYPE_BITMANIP;

    case 854:  /* *strmovdi_rex_1 */
    case 855:  /* *strmovsi_1 */
    case 856:  /* *strmovsi_1 */
    case 857:  /* *strmovhi_1 */
    case 858:  /* *strmovhi_1 */
    case 859:  /* *strmovqi_1 */
    case 860:  /* *strmovqi_1 */
    case 861:  /* *rep_movdi_rex64 */
    case 862:  /* *rep_movsi */
    case 863:  /* *rep_movsi */
    case 864:  /* *rep_movqi */
    case 865:  /* *rep_movqi */
    case 866:  /* *strsetdi_rex_1 */
    case 867:  /* *strsetsi_1 */
    case 868:  /* *strsetsi_1 */
    case 869:  /* *strsethi_1 */
    case 870:  /* *strsethi_1 */
    case 871:  /* *strsetqi_1 */
    case 872:  /* *strsetqi_1 */
    case 873:  /* *rep_stosdi_rex64 */
    case 874:  /* *rep_stossi */
    case 875:  /* *rep_stossi */
    case 876:  /* *rep_stosqi */
    case 877:  /* *rep_stosqi */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 882:  /* *strlenqi_1 */
    case 883:  /* *strlenqi_1 */
      return TYPE_STR;

    case 651:  /* leave */
    case 652:  /* leave_rex64 */
      return TYPE_LEAVE;

    case 621:  /* *call_value_vzeroupper */
    case 622:  /* *call_value_vzeroupper */
    case 623:  /* *call_value */
    case 624:  /* *call_value */
    case 625:  /* *sibcall_value_vzeroupper */
    case 626:  /* *sibcall_value_vzeroupper */
    case 627:  /* *sibcall_value */
    case 628:  /* *sibcall_value */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 631:  /* *call_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 634:  /* *sibcall_value_pop */
      return TYPE_CALLV;

    case 607:  /* *call_vzeroupper */
    case 608:  /* *call_vzeroupper */
    case 609:  /* *call */
    case 610:  /* *call */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 613:  /* *sibcall_vzeroupper */
    case 614:  /* *sibcall_vzeroupper */
    case 615:  /* *sibcall */
    case 616:  /* *sibcall */
    case 617:  /* *call_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
      return TYPE_CALL;

    case 51:  /* *popsi1 */
    case 52:  /* *popdi1 */
    case 53:  /* *popsi1_epilogue */
    case 54:  /* *popdi1_epilogue */
      return TYPE_POP;

    case 43:  /* *pushsi2 */
    case 44:  /* *pushqi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 46:  /* *pushsi2_rex64 */
    case 47:  /* *pushqi2 */
    case 48:  /* *pushhi2 */
    case 49:  /* *pushsi2_prologue */
    case 50:  /* *pushdi2_prologue */
      return TYPE_PUSH;

    case 890:  /* *movhicc_noc */
    case 891:  /* *movsicc_noc */
    case 892:  /* *movdicc_noc */
    case 893:  /* *movqicc_noc */
      return TYPE_ICMOV;

    case 583:  /* *setcc_qi */
    case 584:  /* *setcc_qi_slp */
      return TYPE_SETCC;

    case 587:  /* *jcc_1 */
    case 588:  /* *jcc_2 */
    case 602:  /* jump */
    case 603:  /* *indirect_jump */
    case 604:  /* *indirect_jump */
    case 605:  /* *tablejump_1 */
    case 606:  /* *tablejump_1 */
    case 641:  /* simple_return_indirect_internal */
      return TYPE_IBR;

    case 14:  /* *cmpqi_ext_2 */
    case 356:  /* *testdi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 358:  /* *testqi_1 */
    case 359:  /* *testhi_1 */
    case 360:  /* *testsi_1 */
    case 361:  /* *testqi_ext_0 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 363:  /* *testqi_ext_1 */
    case 364:  /* *testqi_ext_2 */
      return TYPE_TEST;

    case 4:  /* *cmpqi_1 */
    case 5:  /* *cmphi_1 */
    case 6:  /* *cmpsi_1 */
    case 7:  /* *cmpdi_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 12:  /* *cmpqi_ext_1 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 17:  /* *cmpqi_ext_4 */
      return TYPE_ICMP;

    case 343:  /* *divmodhi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 345:  /* *divmoddi4_noext */
    case 346:  /* divmodhiqi3 */
    case 352:  /* *udivmodhi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 354:  /* *udivmoddi4_noext */
    case 355:  /* udivmodhiqi3 */
      return TYPE_IDIV;

    case 324:  /* *bmi2_umulditi3_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
      return TYPE_IMULX;

    case 319:  /* *mulsi3_1 */
    case 320:  /* *muldi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 322:  /* *mulhi3_1 */
    case 323:  /* *mulqi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 329:  /* *mulditi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 331:  /* *umulqihi3_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 335:  /* *umulsi3_highpart_1 */
    case 336:  /* *smulsi3_highpart_zext */
    case 337:  /* *umulsi3_highpart_zext */
      return TYPE_IMUL;

    case 573:  /* *rotlqi3_1_slp */
    case 574:  /* *rotrqi3_1_slp */
      return TYPE_ROTATE1;

    case 560:  /* *bmi2_rorxsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 566:  /* *bmi2_rorxsi3_1_zext */
      return TYPE_ROTATEX;

    case 552:  /* *rotlsi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 554:  /* *rotldi3_mask */
    case 555:  /* *rotrdi3_mask */
    case 569:  /* *rotlqi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 572:  /* *rotrhi3_1 */
      return TYPE_ROTATE;

    case 532:  /* *lshrqi3_1_slp */
    case 533:  /* *ashrqi3_1_slp */
      return TYPE_ISHIFT1;

    case 485:  /* *bmi2_ashlsi3_1 */
    case 486:  /* *bmi2_ashldi3_1 */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
      return TYPE_ISHIFTX;

    case 481:  /* x86_64_shld */
    case 482:  /* x86_shld */
    case 483:  /* *ashlsi3_mask */
    case 484:  /* *ashldi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 506:  /* *ashrdi3_mask */
    case 511:  /* x86_64_shrd */
    case 512:  /* x86_shrd */
    case 528:  /* *lshrqi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 531:  /* *ashrhi3_1 */
    case 534:  /* *lshrqi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 541:  /* *ashrdi3_cmp */
    case 542:  /* *lshrsi3_cmp_zext */
    case 543:  /* *ashrsi3_cmp_zext */
    case 544:  /* *lshrqi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 551:  /* *ashrdi3_cconly */
      return TYPE_ISHIFT;

    case 243:  /* *leasi */
    case 244:  /* *leadi */
    case 277:  /* *lea_general_1 */
    case 278:  /* *lea_general_2 */
    case 279:  /* *lea_general_3 */
    case 280:  /* *lea_general_4 */
    case 281:  /* *lea_general_4 */
    case 647:  /* set_got_rex64 */
    case 648:  /* set_rip_rex64 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
      return TYPE_LEA;

    case 85:  /* *movhi_extv_1 */
    case 86:  /* *movsi_extv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 90:  /* *movdi_extzv_1 */
    case 116:  /* zero_extendqidi2 */
    case 117:  /* zero_extendhidi2 */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 126:  /* *extendsidi2_rex64 */
    case 128:  /* extendqidi2 */
    case 129:  /* extendhidi2 */
    case 130:  /* extendhisi2 */
    case 131:  /* *extendhisi2_zext */
    case 132:  /* extendqisi2 */
    case 133:  /* *extendqisi2_zext */
    case 134:  /* extendqihi2 */
      return TYPE_IMOVX;

    case 67:  /* *movabsqi_1 */
    case 68:  /* *movabshi_1 */
    case 69:  /* *movabssi_1 */
    case 70:  /* *movabsdi_1 */
    case 71:  /* *movabsqi_2 */
    case 72:  /* *movabshi_2 */
    case 73:  /* *movabssi_2 */
    case 74:  /* *movabsdi_2 */
    case 75:  /* *swapsi */
    case 76:  /* *swapdi */
    case 77:  /* *swapqi_1 */
    case 78:  /* *swaphi_1 */
    case 79:  /* *swapqi_2 */
    case 80:  /* *swaphi_2 */
    case 81:  /* *movstrictqi_1 */
    case 82:  /* *movstricthi_1 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 95:  /* *movsi_insv_1 */
    case 96:  /* *movqi_insv_2 */
    case 649:  /* set_got_offset_rex64 */
    case 723:  /* *load_tp_x32 */
    case 724:  /* *load_tp_x32_zext */
    case 725:  /* *load_tp_si */
    case 726:  /* *load_tp_di */
      return TYPE_IMOV;

    case 433:  /* *negqi2_1 */
    case 434:  /* *neghi2_1 */
    case 435:  /* *negsi2_1 */
    case 436:  /* *negdi2_1 */
    case 437:  /* *negsi2_1_zext */
    case 438:  /* *negqi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 441:  /* *negdi2_cmpz */
    case 442:  /* *negsi2_cmpz_zext */
    case 469:  /* *one_cmplhi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 472:  /* *one_cmplqi2_1 */
    case 473:  /* *one_cmplsi2_1_zext */
      return TYPE_NEGNOT;

    case 55:  /* *movsi_xor */
    case 56:  /* *movdi_xor */
    case 57:  /* *movsi_or */
    case 58:  /* *movdi_or */
    case 83:  /* *movstrictqi_xor */
    case 84:  /* *movstricthi_xor */
    case 118:  /* zero_extendhisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 121:  /* *zero_extendqisi2_and */
    case 289:  /* *subqi_1_slp */
    case 372:  /* *andqi_1_slp */
    case 379:  /* *andqi_2_slp */
    case 397:  /* *iorqi_1_slp */
    case 398:  /* *xorqi_1_slp */
    case 411:  /* *iorqi_2_slp */
    case 412:  /* *xorqi_2_slp */
    case 474:  /* *one_cmplqi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 477:  /* *one_cmpldi2_2 */
    case 478:  /* *one_cmplsi2_2_zext */
    case 575:  /* *btsq */
    case 576:  /* *btrq */
    case 577:  /* *btcq */
    case 578:  /* *btsi */
    case 579:  /* *btdi */
    case 655:  /* *ffssi_1 */
    case 656:  /* *ffsdi_1 */
    case 657:  /* ctzhi2 */
    case 658:  /* ctzsi2 */
    case 659:  /* ctzdi2 */
    case 699:  /* bsr_rex64 */
    case 700:  /* bsr */
    case 701:  /* *bsrhi */
      return TYPE_ALU1;

    case 247:  /* *addsi3_cc */
    case 248:  /* *adddi3_cc */
    case 249:  /* addqi3_cc */
    case 276:  /* *addqi_ext_2 */
    case 284:  /* *subqi_1 */
    case 285:  /* *subhi_1 */
    case 286:  /* *subsi_1 */
    case 287:  /* *subdi_1 */
    case 288:  /* *subsi_1_zext */
    case 290:  /* *subqi_2 */
    case 291:  /* *subhi_2 */
    case 292:  /* *subsi_2 */
    case 293:  /* *subdi_2 */
    case 294:  /* *subsi_2_zext */
    case 295:  /* *subqi_3 */
    case 296:  /* *subhi_3 */
    case 297:  /* *subsi_3 */
    case 298:  /* *subdi_3 */
    case 299:  /* *subsi_3_zext */
    case 300:  /* *addqi3_carry */
    case 301:  /* *subqi3_carry */
    case 302:  /* *addhi3_carry */
    case 303:  /* *subhi3_carry */
    case 304:  /* *addsi3_carry */
    case 305:  /* *subsi3_carry */
    case 306:  /* *adddi3_carry */
    case 307:  /* *subdi3_carry */
    case 308:  /* *addsi3_carry_zext */
    case 309:  /* *subsi3_carry_zext */
    case 310:  /* *addqi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 369:  /* *andsi_1_zext */
    case 371:  /* *andqi_1 */
    case 373:  /* *anddi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 375:  /* *andqi_2 */
    case 376:  /* *andhi_2 */
    case 377:  /* *andsi_2 */
    case 378:  /* *andsi_2_zext */
    case 380:  /* andqi_ext_0 */
    case 381:  /* *andqi_ext_0_cc */
    case 382:  /* *andqi_ext_1_rex64 */
    case 383:  /* *andqi_ext_1 */
    case 384:  /* *andqi_ext_2 */
    case 385:  /* *iorhi_1 */
    case 386:  /* *xorhi_1 */
    case 387:  /* *iorsi_1 */
    case 388:  /* *xorsi_1 */
    case 389:  /* *iordi_1 */
    case 390:  /* *xordi_1 */
    case 391:  /* *iorqi_1 */
    case 392:  /* *xorqi_1 */
    case 393:  /* *iorsi_1_zext */
    case 394:  /* *xorsi_1_zext */
    case 395:  /* *iorsi_1_zext_imm */
    case 396:  /* *xorsi_1_zext_imm */
    case 399:  /* *iorqi_2 */
    case 400:  /* *xorqi_2 */
    case 401:  /* *iorhi_2 */
    case 402:  /* *xorhi_2 */
    case 403:  /* *iorsi_2 */
    case 404:  /* *xorsi_2 */
    case 405:  /* *iordi_2 */
    case 406:  /* *xordi_2 */
    case 407:  /* *iorsi_2_zext */
    case 408:  /* *xorsi_2_zext */
    case 409:  /* *iorsi_2_zext_imm */
    case 410:  /* *xorsi_2_zext_imm */
    case 413:  /* *iorqi_3 */
    case 414:  /* *xorqi_3 */
    case 415:  /* *iorhi_3 */
    case 416:  /* *xorhi_3 */
    case 417:  /* *iorsi_3 */
    case 418:  /* *xorsi_3 */
    case 419:  /* *iordi_3 */
    case 420:  /* *xordi_3 */
    case 421:  /* *iorqi_ext_0 */
    case 422:  /* *xorqi_ext_0 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 425:  /* *iorqi_ext_1 */
    case 426:  /* *xorqi_ext_1 */
    case 427:  /* *iorqi_ext_2 */
    case 428:  /* *xorqi_ext_2 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 430:  /* *xorqi_cc_ext_1 */
    case 727:  /* *add_tp_x32 */
    case 728:  /* *add_tp_x32_zext */
    case 729:  /* *add_tp_si */
    case 730:  /* *add_tp_di */
    case 884:  /* *x86_movsicc_0_m1 */
    case 885:  /* *x86_movdicc_0_m1 */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 911:  /* pro_epilogue_adjust_stack_di_sub */
      return TYPE_ALU;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    case 18:  /* *cmpfp_0 */
    case 19:  /* *cmpfp_0_cc */
    case 20:  /* *cmpfp_xf */
    case 21:  /* *cmpfp_xf_cc */
    case 22:  /* *cmpfp_sf */
    case 23:  /* *cmpfp_df */
    case 24:  /* *cmpfp_sf_cc */
    case 25:  /* *cmpfp_df_cc */
    case 26:  /* *cmpfp_u */
    case 27:  /* *cmpfp_u_cc */
    case 28:  /* *cmpfp_hi */
    case 29:  /* *cmpfp_si */
    case 30:  /* *cmpfp_hi_cc */
    case 31:  /* *cmpfp_si_cc */
    case 40:  /* *pushdi2 */
    case 41:  /* *pushti2 */
    case 97:  /* *pushtf */
    case 98:  /* *pushxf */
    case 99:  /* *pushxf_nointeger */
    case 100:  /* *pushdf_rex64 */
    case 101:  /* *pushdf */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 240:  /* *floatunssisf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 242:  /* *floatunssixf2_1 */
    case 338:  /* divmodsi4_1 */
    case 339:  /* divmoddi4_1 */
    case 340:  /* *divmodhi4 */
    case 341:  /* *divmodsi4 */
    case 342:  /* *divmoddi4 */
    case 347:  /* udivmodsi4_1 */
    case 348:  /* udivmoddi4_1 */
    case 349:  /* *udivmodhi4 */
    case 350:  /* *udivmodsi4 */
    case 351:  /* *udivmoddi4 */
    case 479:  /* *ashldi3_doubleword */
    case 480:  /* *ashlti3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 510:  /* *ashrti3_doubleword */
    case 645:  /* set_got */
    case 646:  /* set_got_labelled */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 847:  /* fxamsf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 849:  /* fxamxf2_i387 */
    case 850:  /* fxamsf2_i387_with_temp */
    case 851:  /* fxamdf2_i387_with_temp */
    case 912:  /* allocate_stack_worker_probe_si */
    case 913:  /* allocate_stack_worker_probe_di */
    case 914:  /* adjust_stack_and_probesi */
    case 915:  /* adjust_stack_and_probedi */
    case 916:  /* probe_stack_rangesi */
    case 917:  /* probe_stack_rangedi */
    case 921:  /* stack_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 924:  /* stack_tls_protect_set_di */
    case 925:  /* stack_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 928:  /* stack_tls_protect_test_di */
      return TYPE_MULTI;

    default:
      return TYPE_OTHER;

    }
}

enum attr_use_carry
get_attr_use_carry (rtx insn ATTRIBUTE_UNUSED)
{
  switch (recog_memoized (insn))
    {
    case 300:  /* *addqi3_carry */
    case 301:  /* *subqi3_carry */
    case 302:  /* *addhi3_carry */
    case 303:  /* *subhi3_carry */
    case 304:  /* *addsi3_carry */
    case 305:  /* *subsi3_carry */
    case 306:  /* *adddi3_carry */
    case 307:  /* *subdi3_carry */
    case 308:  /* *addsi3_carry_zext */
    case 884:  /* *x86_movsicc_0_m1 */
    case 885:  /* *x86_movdicc_0_m1 */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 889:  /* *x86_movdicc_0_m1_neg */
      return USE_CARRY_1;

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
    default:
      return USE_CARRY_0;

    }
}

enum attr_unit
get_attr_unit (rtx insn ATTRIBUTE_UNUSED)
{
  enum attr_type cached_type ATTRIBUTE_UNUSED;

  switch (recog_memoized (insn))
    {
    case 1581:  /* *vec_extractv2di_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xf))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 1579:  /* *sse2_storeq_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return UNIT_UNKNOWN;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 1388:  /* sse2_loadlpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 9)
        {
	  return UNIT_I387;
        }
      else if (!((1 << which_alternative) & 0x600))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 1387:  /* sse2_loadhpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 5)
        {
	  return UNIT_I387;
        }
      else if (((1 << which_alternative) & 0x1f))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 1385:  /* sse2_storelpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 3)
        {
	  return UNIT_I387;
        }
      else if (((1 << which_alternative) & 0x7))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 1383:  /* sse2_storehpd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 4)
        {
	  return UNIT_I387;
        }
      else if (((1 << which_alternative) & 0xf))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 1358:  /* *sse4_1_extractps */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_UNKNOWN;
        }

    case 1354:  /* vec_setv4sf_0 */
    case 1353:  /* vec_setv4si_0 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 10)
        {
	  return UNIT_I387;
        }
      else if (!((1 << which_alternative) & 0x600))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 1587:  /* *vec_concatv2si_sse */
    case 1586:  /* *vec_concatv2si_sse2 */
    case 1351:  /* *vec_concatv2sf_sse */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_MMX;
        }

    case 1585:  /* *vec_concatv2si_sse4_1 */
    case 1350:  /* *vec_concatv2sf_sse4_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1f))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_MMX;
        }

    case 1302:  /* sse2_cvtpi2pd */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return UNIT_MMX;
        }
      else
        {
	  return UNIT_SSE;
        }

    case 1066:  /* *vec_extractv2si_1 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x2e))
        {
	  return UNIT_SSE;
        }
      else if (((1 << which_alternative) & 0x11))
        {
	  return UNIT_MMX;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 996:  /* *vec_extractv2sf_1 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 4)
        {
	  return UNIT_I387;
        }
      else if (((1 << which_alternative) & 0xa))
        {
	  return UNIT_SSE;
        }
      else if (((1 << which_alternative) & 0x5))
        {
	  return UNIT_MMX;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 968:  /* *movv2sf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x30))
        {
	  return UNIT_MMX;
        }
      else
        {
	  if (!((1 << which_alternative) & 0xc0f))
	    {
	      return UNIT_SSE;
	    }
	  else if (((1 << which_alternative) & 0xf))
	    {
	      return UNIT_MMX;
	    }
	  else
	    {
	      return UNIT_INTEGER;
	    }
        }

    case 966:  /* *movv1di_internal */
    case 965:  /* *movv2si_internal */
    case 964:  /* *movv4hi_internal */
    case 963:  /* *movv8qi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x30))
        {
	  return UNIT_MMX;
        }
      else
        {
	  if (!((1 << which_alternative) & 0x600f))
	    {
	      return UNIT_SSE;
	    }
	  else if (((1 << which_alternative) & 0xf))
	    {
	      return UNIT_MMX;
	    }
	  else
	    {
	      return UNIT_INTEGER;
	    }
        }

    case 967:  /* *movv2sf_internal_rex64 */
    case 962:  /* *movv1di_internal_rex64 */
    case 961:  /* *movv2si_internal_rex64 */
    case 960:  /* *movv4hi_internal_rex64 */
    case 959:  /* *movv8qi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xc0))
        {
	  return UNIT_MMX;
        }
      else
        {
	  if (!((1 << which_alternative) & 0x3f))
	    {
	      return UNIT_SSE;
	    }
	  else if (((1 << which_alternative) & 0x3c))
	    {
	      return UNIT_MMX;
	    }
	  else
	    {
	      return UNIT_INTEGER;
	    }
        }

    case 897:  /* *movsfcc_1_387 */
    case 896:  /* *movdfcc_1 */
    case 895:  /* *movdfcc_1_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return UNIT_I387;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 772:  /* *fop_xf_6_i387 */
    case 771:  /* *fop_xf_6_i387 */
    case 770:  /* *fop_xf_5_i387 */
    case 769:  /* *fop_xf_5_i387 */
    case 768:  /* *fop_xf_4_i387 */
    case 767:  /* *fop_xf_4_i387 */
    case 766:  /* *fop_xf_3_i387 */
    case 765:  /* *fop_xf_3_i387 */
    case 764:  /* *fop_xf_2_i387 */
    case 763:  /* *fop_xf_2_i387 */
    case 762:  /* *fop_xf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || ((mult_operator (operands[3], XFmode)) || (cached_type == TYPE_FDIV)))
        {
	  return UNIT_I387;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 760:  /* *fop_df_6_i387 */
    case 759:  /* *fop_df_5_i387 */
    case 758:  /* *fop_df_4_i387 */
    case 757:  /* *fop_df_3_i387 */
    case 755:  /* *fop_df_3_i387 */
    case 753:  /* *fop_df_2_i387 */
    case 751:  /* *fop_df_2_i387 */
    case 749:  /* *fop_df_1_i387 */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || ((mult_operator (operands[3], DFmode)) || (cached_type == TYPE_FDIV)))
        {
	  return UNIT_I387;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 756:  /* *fop_sf_3_i387 */
    case 754:  /* *fop_sf_3_i387 */
    case 752:  /* *fop_sf_2_i387 */
    case 750:  /* *fop_sf_2_i387 */
    case 748:  /* *fop_sf_1_i387 */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || ((mult_operator (operands[3], SFmode)) || (cached_type == TYPE_FDIV)))
        {
	  return UNIT_I387;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 747:  /* *fop_df_1_sse */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) || ((mult_operator (operands[3], DFmode)) || (cached_type == TYPE_SSEDIV)))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 746:  /* *fop_sf_1_sse */
      extract_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_SSEADD) || ((mult_operator (operands[3], SFmode)) || (cached_type == TYPE_SSEDIV)))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 744:  /* *fop_df_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) || (cached_type == TYPE_FDIV)))
        {
	  return UNIT_I387;
        }
      else if ((cached_type == TYPE_SSEADD) || (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], DFmode))) || (cached_type == TYPE_SSEDIV)))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 743:  /* *fop_sf_1_mixed */
      extract_constrain_insn_cached (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FOP) || (((!((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) || (cached_type == TYPE_FDIV)))
        {
	  return UNIT_I387;
        }
      else if ((cached_type == TYPE_SSEADD) || (((((1 << which_alternative) & 0xc)) && (mult_operator (operands[3], SFmode))) || (cached_type == TYPE_SSEDIV)))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 738:  /* *fop_df_comm_mixed */
    case 737:  /* *fop_sf_comm_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return UNIT_I387;
        }
      else if (((1 << which_alternative) & 0x6))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 204:  /* *floatdidf2_mixed_interunit */
    case 203:  /* *floatdisf2_mixed_interunit */
    case 202:  /* *floatsidf2_mixed_interunit */
    case 201:  /* *floatsisf2_mixed_interunit */
    case 196:  /* *floatsidf2_vector_mixed */
    case 195:  /* *floatsisf2_vector_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return UNIT_I387;
        }
      else
        {
	  return UNIT_SSE;
        }

    case 200:  /* *floatdidf2_mixed_with_temp */
    case 199:  /* *floatdisf2_mixed_with_temp */
    case 198:  /* *floatsidf2_mixed_with_temp */
    case 197:  /* *floatsisf2_mixed_with_temp */
    case 194:  /* *floatsidf2_vector_mixed_with_temp */
    case 193:  /* *floatsisf2_vector_mixed_with_temp */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return UNIT_I387;
        }
      else if (which_alternative == 1)
        {
	  return UNIT_I387;
        }
      else
        {
	  return UNIT_SSE;
        }

    case 230:  /* *floatdixf2_i387_with_temp */
    case 229:  /* *floatdidf2_i387_with_temp */
    case 228:  /* *floatdisf2_i387_with_temp */
    case 227:  /* *floatsixf2_i387_with_temp */
    case 226:  /* *floatsidf2_i387_with_temp */
    case 225:  /* *floatsisf2_i387_with_temp */
    case 183:  /* *floathixf2_i387_with_temp */
    case 182:  /* *floathidf2_i387_with_temp */
    case 181:  /* *floathisf2_i387_with_temp */
    case 147:  /* *truncxfdf2_mixed */
    case 146:  /* *truncxfsf2_mixed */
    case 144:  /* *truncdfsf_i387 */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return UNIT_I387;
        }
      else
        {
	  return UNIT_I387;
        }

    case 143:  /* *truncdfsf_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  if (which_alternative == 0)
	    {
	      return UNIT_I387;
	    }
	  else
	    {
	      return UNIT_SSE;
	    }
        }
      else
        {
	  return UNIT_I387;
        }

    case 135:  /* *extendsfdf2_mixed */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x3))
        {
	  return UNIT_I387;
        }
      else
        {
	  return UNIT_SSE;
        }

    case 115:  /* zero_extendsidi2_1 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x1f))
        {
	  return UNIT_SSE;
        }
      else if (((1 << which_alternative) & 0x18))
        {
	  return UNIT_MMX;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 114:  /* *zero_extendsidi2_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0xf))
        {
	  return UNIT_SSE;
        }
      else if (((1 << which_alternative) & 0xc))
        {
	  return UNIT_MMX;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 110:  /* *movsf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return UNIT_I387;
        }
      else if (!((1 << which_alternative) & 0xce1f))
        {
	  return UNIT_SSE;
        }
      else if (((1 << which_alternative) & 0xce00))
        {
	  return UNIT_MMX;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 109:  /* *movdf_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return UNIT_I387;
        }
      else if (!((1 << which_alternative) & 0x1f))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 108:  /* *movdf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return UNIT_I387;
        }
      else if (!((1 << which_alternative) & 0x7f))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 107:  /* *movxf_internal */
    case 106:  /* *movxf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return UNIT_I387;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 104:  /* *movtf_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x7))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_UNKNOWN;
        }

    case 103:  /* *pushsf */
    case 102:  /* *pushsf_rex64 */
    case 101:  /* *pushdf */
    case 100:  /* *pushdf_rex64 */
    case 99:  /* *pushxf_nointeger */
    case 98:  /* *pushxf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return UNIT_I387;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 97:  /* *pushtf */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 64:  /* *movsi_internal */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0xfc0))
        {
	  return UNIT_SSE;
        }
      else if (((1 << which_alternative) & 0x3c))
        {
	  return UNIT_MMX;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 63:  /* *movdi_internal */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x1f))
        {
	  return UNIT_SSE;
        }
      else if (((1 << which_alternative) & 0x1c))
        {
	  return UNIT_MMX;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 62:  /* *movdi_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (((1 << which_alternative) & 0x1fe00))
        {
	  return UNIT_SSE;
        }
      else if (((1 << which_alternative) & 0x1f0))
        {
	  return UNIT_MMX;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 60:  /* *movti_internal_rex64 */
      extract_constrain_insn_cached (insn);
      if (!((1 << which_alternative) & 0x3))
        {
	  return UNIT_SSE;
        }
      else
        {
	  return UNIT_UNKNOWN;
        }

    case 208:  /* *floatdidf2_mixed_nointerunit */
    case 207:  /* *floatdisf2_mixed_nointerunit */
    case 206:  /* *floatsidf2_mixed_nointerunit */
    case 205:  /* *floatsisf2_mixed_nointerunit */
    case 140:  /* *truncdfsf_fast_mixed */
    case 37:  /* *cmpfp_iu_mixed */
    case 34:  /* *cmpfp_i_mixed */
      extract_constrain_insn_cached (insn);
      if (which_alternative == 0)
        {
	  return UNIT_I387;
        }
      else
        {
	  return UNIT_SSE;
        }

    case -1:
      if (GET_CODE (PATTERN (insn)) != ASM_INPUT
          && asm_noperands (PATTERN (insn)) < 0)
        fatal_insn_not_found (insn);
      if (((cached_type = get_attr_type (insn)) == TYPE_FMOV) || ((cached_type == TYPE_FOP) || ((cached_type == TYPE_FSGN) || ((cached_type == TYPE_FMUL) || ((cached_type == TYPE_FDIV) || ((cached_type == TYPE_FPSPC) || ((cached_type == TYPE_FCMOV) || ((cached_type == TYPE_FCMP) || ((cached_type == TYPE_FXCH) || ((cached_type == TYPE_FISTP) || ((cached_type == TYPE_FISTTP) || (cached_type == TYPE_FRNDINT))))))))))))
        {
	  return UNIT_I387;
        }
      else if ((cached_type == TYPE_SSELOG) || ((cached_type == TYPE_SSELOG1) || ((cached_type == TYPE_SSEIADD) || ((cached_type == TYPE_SSEIADD1) || ((cached_type == TYPE_SSEISHFT) || ((cached_type == TYPE_SSEISHFT1) || ((cached_type == TYPE_SSEIMUL) || ((cached_type == TYPE_SSE) || ((cached_type == TYPE_SSEMOV) || ((cached_type == TYPE_SSEADD) || ((cached_type == TYPE_SSEMUL) || ((cached_type == TYPE_SSECMP) || ((cached_type == TYPE_SSECOMI) || ((cached_type == TYPE_SSECVT) || ((cached_type == TYPE_SSECVT1) || ((cached_type == TYPE_SSEICVT) || ((cached_type == TYPE_SSEDIV) || ((cached_type == TYPE_SSEINS) || ((cached_type == TYPE_SSEMULADD) || (cached_type == TYPE_SSE4ARG))))))))))))))))))))
        {
	  return UNIT_SSE;
        }
      else if ((cached_type == TYPE_MMX) || ((cached_type == TYPE_MMXMOV) || ((cached_type == TYPE_MMXADD) || ((cached_type == TYPE_MMXMUL) || ((cached_type == TYPE_MMXCMP) || ((cached_type == TYPE_MMXCVT) || (cached_type == TYPE_MMXSHFT)))))))
        {
	  return UNIT_MMX;
        }
      else if (cached_type == TYPE_OTHER)
        {
	  return UNIT_UNKNOWN;
        }
      else
        {
	  return UNIT_INTEGER;
        }

    case 2057:  /* atomic_xordi */
    case 2056:  /* atomic_ordi */
    case 2055:  /* atomic_anddi */
    case 2054:  /* atomic_xorsi */
    case 2053:  /* atomic_orsi */
    case 2052:  /* atomic_andsi */
    case 2051:  /* atomic_xorhi */
    case 2050:  /* atomic_orhi */
    case 2049:  /* atomic_andhi */
    case 2048:  /* atomic_xorqi */
    case 2047:  /* atomic_orqi */
    case 2046:  /* atomic_andqi */
    case 2045:  /* atomic_subdi */
    case 2044:  /* atomic_subsi */
    case 2043:  /* atomic_subhi */
    case 2042:  /* atomic_subqi */
    case 2041:  /* atomic_adddi */
    case 2040:  /* atomic_addsi */
    case 2039:  /* atomic_addhi */
    case 2038:  /* atomic_addqi */
    case 2037:  /* atomic_exchangedi */
    case 2036:  /* atomic_exchangesi */
    case 2035:  /* atomic_exchangehi */
    case 2034:  /* atomic_exchangeqi */
    case 2033:  /* *atomic_fetch_add_cmpdi */
    case 2032:  /* *atomic_fetch_add_cmpsi */
    case 2031:  /* *atomic_fetch_add_cmphi */
    case 2030:  /* *atomic_fetch_add_cmpqi */
    case 2029:  /* atomic_fetch_adddi */
    case 2028:  /* atomic_fetch_addsi */
    case 2027:  /* atomic_fetch_addhi */
    case 2026:  /* atomic_fetch_addqi */
    case 2025:  /* atomic_compare_and_swapti_doubleword */
    case 2024:  /* atomic_compare_and_swapdi_doubleword */
    case 2023:  /* atomic_compare_and_swapdi_1 */
    case 2022:  /* atomic_compare_and_swapsi_1 */
    case 2021:  /* atomic_compare_and_swaphi_1 */
    case 2020:  /* atomic_compare_and_swapqi_1 */
    case 2017:  /* atomic_storedi_fpu */
    case 2016:  /* atomic_storedi_1 */
    case 2015:  /* atomic_storesi_1 */
    case 2014:  /* atomic_storehi_1 */
    case 2013:  /* atomic_storeqi_1 */
    case 2012:  /* atomic_loaddi_fpu */
    case 2011:  /* mfence_nosse */
    case 1913:  /* avx_pd256_pd */
    case 1912:  /* avx_ps256_ps */
    case 1911:  /* avx_si256_si */
    case 1866:  /* *avx_vperm_broadcast_v4df */
    case 1865:  /* *avx_vperm_broadcast_v8sf */
    case 1610:  /* sse3_monitor64 */
    case 1609:  /* sse3_monitor */
    case 1608:  /* sse3_mwait */
    case 1580:  /* *sse2_storeq */
    case 1578:  /* *vec_ext_v4si_mem */
    case 1577:  /* sse2_stored */
    case 1442:  /* mulv2di3 */
    case 1441:  /* mulv4di3 */
    case 1440:  /* *sse2_mulv4si3 */
    case 1425:  /* mulv16qi3 */
    case 1424:  /* mulv32qi3 */
    case 1370:  /* vec_extract_lo_v32qi */
    case 1368:  /* vec_extract_lo_v16hi */
    case 1365:  /* vec_extract_lo_v8sf */
    case 1364:  /* vec_extract_lo_v8si */
    case 1361:  /* vec_extract_lo_v4df */
    case 1360:  /* vec_extract_lo_v4di */
    case 1359:  /* *vec_extract_v4sf_mem */
    case 1357:  /* *vec_extractv4sf_0 */
    case 1116:  /* *absnegv2df2 */
    case 1115:  /* *absnegv4df2 */
    case 1114:  /* *absnegv4sf2 */
    case 1113:  /* *absnegv8sf2 */
    case 1090:  /* movdi_to_sse */
    case 1065:  /* *vec_extractv2si_0 */
    case 995:  /* *vec_extractv2sf_0 */
    case 958:  /* *pause */
    case 957:  /* rdranddi_1 */
    case 956:  /* rdrandsi_1 */
    case 955:  /* rdrandhi_1 */
    case 954:  /* wrgsbasedi */
    case 953:  /* wrgsbasesi */
    case 952:  /* wrfsbasedi */
    case 951:  /* wrfsbasesi */
    case 950:  /* rdgsbasedi */
    case 949:  /* rdgsbasesi */
    case 948:  /* rdfsbasedi */
    case 947:  /* rdfsbasesi */
    case 938:  /* *rdtscp_rex64 */
    case 937:  /* *rdtscp */
    case 936:  /* *rdtsc_rex64 */
    case 935:  /* *rdtsc */
    case 934:  /* *rdpmc_rex64 */
    case 933:  /* *rdpmc */
    case 918:  /* trap */
    case 853:  /* cld */
    case 736:  /* *tls_dynamic_gnu2_combine_64 */
    case 733:  /* *tls_dynamic_gnu2_combine_32 */
    case 722:  /* *tls_local_dynamic_32_once */
    case 717:  /* *parityhi2_cmp */
    case 716:  /* paritysi2_cmp */
    case 715:  /* paritydi2_cmp */
    case 714:  /* bswaphi_lowpart */
    case 713:  /* *bswaphi_lowpart_1 */
    case 654:  /* ffssi2_no_cmove */
    case 653:  /* split_stack_return */
    case 650:  /* eh_return_internal */
    case 644:  /* pad */
    case 643:  /* nops */
    case 642:  /* nop */
    case 640:  /* simple_return_pop_internal */
    case 639:  /* simple_return_internal_long */
    case 638:  /* simple_return_internal */
    case 637:  /* prologue_use */
    case 636:  /* *memory_blockage */
    case 635:  /* blockage */
    case 601:  /* *fp_jcc_4_si_387 */
    case 600:  /* *fp_jcc_4_hi_387 */
    case 599:  /* *fp_jcc_3_387 */
    case 598:  /* *fp_jcc_2r_387 */
    case 597:  /* *fp_jcc_2_387 */
    case 596:  /* *fp_jcc_1r_387 */
    case 595:  /* *fp_jcc_1_387 */
    case 594:  /* *jcc_btsi_mask_1 */
    case 593:  /* *jcc_btsi_1 */
    case 592:  /* *jcc_btdi_mask */
    case 591:  /* *jcc_btsi_mask */
    case 590:  /* *jcc_btdi */
    case 589:  /* *jcc_btsi */
    case 582:  /* *setcc_si_1_movzbl */
    case 581:  /* *setcc_si_1_and */
    case 580:  /* *setcc_di_1 */
    case 559:  /* ix86_rotrti3_doubleword */
    case 558:  /* ix86_rotrdi3_doubleword */
    case 557:  /* ix86_rotlti3_doubleword */
    case 556:  /* ix86_rotldi3_doubleword */
    case 468:  /* copysigntf3_var */
    case 467:  /* copysigndf3_var */
    case 466:  /* copysignsf3_var */
    case 465:  /* copysigntf3_const */
    case 464:  /* copysigndf3_const */
    case 463:  /* copysignsf3_const */
    case 450:  /* *absnegtf2_sse */
    case 449:  /* *absnegxf2_i387 */
    case 448:  /* *absnegdf2_i387 */
    case 447:  /* *absnegsf2_i387 */
    case 446:  /* *absnegdf2_sse */
    case 445:  /* *absnegsf2_sse */
    case 444:  /* *absnegdf2_mixed */
    case 443:  /* *absnegsf2_mixed */
    case 432:  /* *negti2_doubleword */
    case 431:  /* *negdi2_doubleword */
    case 366:  /* *testqi_ext_3 */
    case 365:  /* *testqi_ext_3_rex64 */
    case 283:  /* *subti3_doubleword */
    case 282:  /* *subdi3_doubleword */
    case 246:  /* *addti3_doubleword */
    case 245:  /* *adddi3_doubleword */
    case 192:  /* *floatdixf2_1 */
    case 191:  /* *floatdidf2_1 */
    case 190:  /* *floatdisf2_1 */
    case 189:  /* *floatsixf2_1 */
    case 188:  /* *floatsidf2_1 */
    case 187:  /* *floatsisf2_1 */
    case 180:  /* *floathixf2_1 */
    case 179:  /* *floathidf2_1 */
    case 178:  /* *floathisf2_1 */
    case 153:  /* *fixuns_truncdf_1 */
    case 152:  /* *fixuns_truncsf_1 */
    case 127:  /* extendsidi2_1 */
    case 33:  /* x86_sahf_1 */
      return UNIT_UNKNOWN;

    case 1074:  /* *mmx_femms */
    case 1073:  /* *mmx_emms */
    case 1072:  /* *mmx_maskmovq */
    case 1071:  /* *mmx_maskmovq */
    case 1070:  /* mmx_pmovmskb */
    case 1069:  /* mmx_psadbw */
    case 1068:  /* *mmx_uavgv4hi3 */
    case 1067:  /* *mmx_uavgv8qi3 */
    case 1064:  /* *mmx_concatv2si */
    case 1063:  /* *vec_dupv2si */
    case 1062:  /* *vec_dupv4hi */
    case 1061:  /* mmx_pswapdv2si2 */
    case 1060:  /* mmx_pshufw_1 */
    case 1059:  /* mmx_pextrw */
    case 1058:  /* *mmx_pinsrw */
    case 1057:  /* mmx_punpckldq */
    case 1056:  /* mmx_punpckhdq */
    case 1055:  /* mmx_punpcklwd */
    case 1054:  /* mmx_punpckhwd */
    case 1053:  /* mmx_punpcklbw */
    case 1052:  /* mmx_punpckhbw */
    case 1051:  /* mmx_packuswb */
    case 1050:  /* mmx_packssdw */
    case 1049:  /* mmx_packsswb */
    case 1048:  /* *mmx_xorv2si3 */
    case 1047:  /* *mmx_iorv2si3 */
    case 1046:  /* *mmx_andv2si3 */
    case 1045:  /* *mmx_xorv4hi3 */
    case 1044:  /* *mmx_iorv4hi3 */
    case 1043:  /* *mmx_andv4hi3 */
    case 1042:  /* *mmx_xorv8qi3 */
    case 1041:  /* *mmx_iorv8qi3 */
    case 1040:  /* *mmx_andv8qi3 */
    case 1039:  /* mmx_andnotv2si3 */
    case 1038:  /* mmx_andnotv4hi3 */
    case 1037:  /* mmx_andnotv8qi3 */
    case 1036:  /* mmx_gtv2si3 */
    case 1035:  /* mmx_gtv4hi3 */
    case 1034:  /* mmx_gtv8qi3 */
    case 1033:  /* *mmx_eqv2si3 */
    case 1032:  /* *mmx_eqv4hi3 */
    case 1031:  /* *mmx_eqv8qi3 */
    case 1030:  /* mmx_lshrv1di3 */
    case 1029:  /* mmx_ashlv1di3 */
    case 1028:  /* mmx_lshrv2si3 */
    case 1027:  /* mmx_ashlv2si3 */
    case 1026:  /* mmx_lshrv4hi3 */
    case 1025:  /* mmx_ashlv4hi3 */
    case 1024:  /* mmx_ashrv2si3 */
    case 1023:  /* mmx_ashrv4hi3 */
    case 1022:  /* *mmx_uminv8qi3 */
    case 1021:  /* *mmx_umaxv8qi3 */
    case 1020:  /* *mmx_sminv4hi3 */
    case 1019:  /* *mmx_smaxv4hi3 */
    case 1018:  /* *sse2_umulv1siv1di3 */
    case 1017:  /* *mmx_pmulhrwv4hi3 */
    case 1016:  /* *mmx_pmaddwd */
    case 1015:  /* *mmx_umulv4hi3_highpart */
    case 1014:  /* *mmx_smulv4hi3_highpart */
    case 1013:  /* *mmx_mulv4hi3 */
    case 1012:  /* *mmx_ussubv4hi3 */
    case 1011:  /* *mmx_sssubv4hi3 */
    case 1010:  /* *mmx_usaddv4hi3 */
    case 1009:  /* *mmx_ssaddv4hi3 */
    case 1008:  /* *mmx_ussubv8qi3 */
    case 1007:  /* *mmx_sssubv8qi3 */
    case 1006:  /* *mmx_usaddv8qi3 */
    case 1005:  /* *mmx_ssaddv8qi3 */
    case 1004:  /* *mmx_subv1di3 */
    case 1003:  /* *mmx_addv1di3 */
    case 1002:  /* *mmx_subv2si3 */
    case 1001:  /* *mmx_addv2si3 */
    case 1000:  /* *mmx_subv4hi3 */
    case 999:  /* *mmx_addv4hi3 */
    case 998:  /* *mmx_subv8qi3 */
    case 997:  /* *mmx_addv8qi3 */
    case 994:  /* *mmx_concatv2sf */
    case 993:  /* *vec_dupv2sf */
    case 992:  /* mmx_pswapdv2sf2 */
    case 991:  /* mmx_floatv2si2 */
    case 990:  /* mmx_pi2fw */
    case 989:  /* mmx_pf2iw */
    case 988:  /* mmx_pf2id */
    case 987:  /* mmx_gev2sf3 */
    case 986:  /* mmx_gtv2sf3 */
    case 985:  /* *mmx_eqv2sf3 */
    case 984:  /* mmx_addsubv2sf3 */
    case 983:  /* mmx_hsubv2sf3 */
    case 982:  /* mmx_haddv2sf3 */
    case 981:  /* mmx_rsqit1v2sf3 */
    case 980:  /* mmx_rsqrtv2sf2 */
    case 979:  /* mmx_rcpit2v2sf3 */
    case 978:  /* mmx_rcpit1v2sf3 */
    case 977:  /* mmx_rcpv2sf2 */
    case 976:  /* *mmx_sminv2sf3 */
    case 975:  /* *mmx_smaxv2sf3 */
    case 974:  /* *mmx_sminv2sf3_finite */
    case 973:  /* *mmx_smaxv2sf3_finite */
    case 972:  /* *mmx_mulv2sf3 */
    case 971:  /* *mmx_subv2sf3 */
    case 970:  /* *mmx_addv2sf3 */
    case 969:  /* sse_movntq */
    case 920:  /* *prefetch_3dnow */
    case 1286:  /* sse_cvtps2pi */
    case 1287:  /* sse_cvttps2pi */
    case 1303:  /* sse2_cvtpd2pi */
    case 1304:  /* sse2_cvttpd2pi */
      return UNIT_MMX;

    case 2019:  /* storedi_via_fpu */
    case 2018:  /* loaddi_via_fpu */
    case 894:  /* *movxfcc_1 */
    case 846:  /* frndintxf2_mask_pm_i387 */
    case 845:  /* frndintxf2_mask_pm */
    case 844:  /* frndintxf2_trunc_i387 */
    case 843:  /* frndintxf2_trunc */
    case 842:  /* fistsi2_ceil_with_temp */
    case 841:  /* fisthi2_ceil_with_temp */
    case 840:  /* fistsi2_ceil */
    case 839:  /* fisthi2_ceil */
    case 838:  /* fistdi2_ceil_with_temp */
    case 837:  /* fistdi2_ceil */
    case 836:  /* *fistdi2_ceil_1 */
    case 835:  /* *fistsi2_ceil_1 */
    case 834:  /* *fisthi2_ceil_1 */
    case 833:  /* frndintxf2_ceil_i387 */
    case 832:  /* frndintxf2_ceil */
    case 831:  /* fistsi2_floor_with_temp */
    case 830:  /* fisthi2_floor_with_temp */
    case 829:  /* fistsi2_floor */
    case 828:  /* fisthi2_floor */
    case 827:  /* fistdi2_floor_with_temp */
    case 826:  /* fistdi2_floor */
    case 825:  /* *fistdi2_floor_1 */
    case 824:  /* *fistsi2_floor_1 */
    case 823:  /* *fisthi2_floor_1 */
    case 822:  /* frndintxf2_floor_i387 */
    case 821:  /* frndintxf2_floor */
    case 820:  /* fistsi2_with_temp */
    case 819:  /* fisthi2_with_temp */
    case 818:  /* fistsi2 */
    case 817:  /* fisthi2 */
    case 816:  /* *fistsi2_1 */
    case 815:  /* *fisthi2_1 */
    case 814:  /* fistdi2_with_temp */
    case 813:  /* fistdi2 */
    case 812:  /* *fistdi2_1 */
    case 811:  /* rintxf2 */
    case 808:  /* *fscalexf4_i387 */
    case 807:  /* *f2xm1xf2_i387 */
    case 806:  /* fxtract_extenddfxf3_i387 */
    case 805:  /* fxtract_extendsfxf3_i387 */
    case 804:  /* fxtractxf3_i387 */
    case 803:  /* fyl2xp1_extenddfxf3_i387 */
    case 802:  /* fyl2xp1_extendsfxf3_i387 */
    case 801:  /* fyl2xp1xf3_i387 */
    case 800:  /* fyl2x_extenddfxf3_i387 */
    case 799:  /* fyl2x_extendsfxf3_i387 */
    case 798:  /* fyl2xxf3_i387 */
    case 797:  /* fpatan_extenddfxf3_i387 */
    case 796:  /* fpatan_extendsfxf3_i387 */
    case 795:  /* *fpatanxf3_i387 */
    case 794:  /* fptan_extenddfxf4_i387 */
    case 793:  /* fptan_extendsfxf4_i387 */
    case 792:  /* fptanxf4_i387 */
    case 791:  /* sincos_extenddfxf3_i387 */
    case 790:  /* sincos_extendsfxf3_i387 */
    case 789:  /* sincosxf3 */
    case 788:  /* *cos_extenddfxf2_i387 */
    case 787:  /* *cos_extendsfxf2_i387 */
    case 786:  /* *cosxf2_i387 */
    case 785:  /* *sin_extenddfxf2_i387 */
    case 784:  /* *sin_extendsfxf2_i387 */
    case 783:  /* *sinxf2_i387 */
    case 782:  /* fprem1xf4_i387 */
    case 781:  /* fpremxf4_i387 */
    case 777:  /* sqrt_extenddfxf2_i387 */
    case 776:  /* sqrt_extendsfxf2_i387 */
    case 775:  /* sqrtxf2 */
    case 774:  /* truncxfdf2_i387_noop_unspec */
    case 773:  /* truncxfsf2_i387_noop_unspec */
    case 761:  /* *fop_xf_comm_i387 */
    case 742:  /* *fop_df_comm_i387 */
    case 741:  /* *fop_sf_comm_i387 */
    case 462:  /* *negextenddfxf2 */
    case 461:  /* *absextenddfxf2 */
    case 460:  /* *negextendsfxf2 */
    case 459:  /* *absextendsfxf2 */
    case 458:  /* *negextendsfdf2 */
    case 457:  /* *absextendsfdf2 */
    case 456:  /* *negxf2_1 */
    case 455:  /* *absxf2_1 */
    case 454:  /* *negdf2_1 */
    case 453:  /* *absdf2_1 */
    case 452:  /* *negsf2_1 */
    case 451:  /* *abssf2_1 */
    case 236:  /* *floatdixf2_i387 */
    case 235:  /* *floatdidf2_i387 */
    case 234:  /* *floatdisf2_i387 */
    case 233:  /* *floatsixf2_i387 */
    case 232:  /* *floatsidf2_i387 */
    case 231:  /* *floatsisf2_i387 */
    case 186:  /* *floathixf2_i387 */
    case 185:  /* *floathidf2_i387 */
    case 184:  /* *floathisf2_i387 */
    case 175:  /* fix_truncsi_i387_with_temp */
    case 174:  /* fix_trunchi_i387_with_temp */
    case 173:  /* fix_truncsi_i387 */
    case 172:  /* fix_trunchi_i387 */
    case 171:  /* fix_truncdi_i387_with_temp */
    case 170:  /* fix_truncdi_i387 */
    case 169:  /* *fix_truncdi_i387_1 */
    case 168:  /* *fix_truncsi_i387_1 */
    case 167:  /* *fix_trunchi_i387_1 */
    case 166:  /* fix_truncdi_i387_fisttp_with_temp */
    case 165:  /* fix_truncsi_i387_fisttp_with_temp */
    case 164:  /* fix_trunchi_i387_fisttp_with_temp */
    case 163:  /* fix_truncdi_i387_fisttp */
    case 162:  /* fix_truncsi_i387_fisttp */
    case 161:  /* fix_trunchi_i387_fisttp */
    case 160:  /* fix_truncdi_fisttp_i387_1 */
    case 159:  /* fix_truncsi_fisttp_i387_1 */
    case 158:  /* fix_trunchi_fisttp_i387_1 */
    case 151:  /* *truncxfdf2_i387 */
    case 150:  /* *truncxfsf2_i387 */
    case 149:  /* truncxfdf2_i387_noop */
    case 148:  /* truncxfsf2_i387_noop */
    case 145:  /* *truncdfsf2_i387_1 */
    case 142:  /* *truncdfsf_fast_i387 */
    case 139:  /* *extenddfxf2_i387 */
    case 138:  /* *extendsfxf2_i387 */
    case 137:  /* *extendsfdf2_i387 */
    case 113:  /* *swapdf */
    case 112:  /* *swapsf */
    case 111:  /* swapxf */
    case 39:  /* *cmpfp_iu_387 */
    case 36:  /* *cmpfp_i_i387 */
    case 18:  /* *cmpfp_0 */
    case 19:  /* *cmpfp_0_cc */
    case 20:  /* *cmpfp_xf */
    case 21:  /* *cmpfp_xf_cc */
    case 22:  /* *cmpfp_sf */
    case 23:  /* *cmpfp_df */
    case 24:  /* *cmpfp_sf_cc */
    case 25:  /* *cmpfp_df_cc */
    case 26:  /* *cmpfp_u */
    case 27:  /* *cmpfp_u_cc */
    case 28:  /* *cmpfp_hi */
    case 29:  /* *cmpfp_si */
    case 30:  /* *cmpfp_hi_cc */
    case 31:  /* *cmpfp_si_cc */
    case 32:  /* x86_fnstsw_1 */
    case 176:  /* x86_fnstcw_1 */
    case 177:  /* x86_fldcw_1 */
    case 237:  /* floatdisf2_i387_with_xmm */
    case 238:  /* floatdidf2_i387_with_xmm */
    case 239:  /* floatdixf2_i387_with_xmm */
    case 847:  /* fxamsf2_i387 */
    case 848:  /* fxamdf2_i387 */
    case 849:  /* fxamxf2_i387 */
    case 850:  /* fxamsf2_i387_with_temp */
    case 851:  /* fxamdf2_i387_with_temp */
      return UNIT_I387;

    case 946:  /* *lwp_lwpinsdi3_1 */
    case 945:  /* *lwp_lwpinssi3_1 */
    case 944:  /* *lwp_lwpvaldi3_1 */
    case 943:  /* *lwp_lwpvalsi3_1 */
    case 942:  /* lwp_slwpcbdi */
    case 941:  /* lwp_slwpcbsi */
    case 940:  /* *lwp_llwpcbdi1 */
    case 939:  /* *lwp_llwpcbsi1 */
    case 928:  /* stack_tls_protect_test_di */
    case 927:  /* stack_tls_protect_test_si */
    case 926:  /* stack_protect_test_di */
    case 925:  /* stack_protect_test_si */
    case 924:  /* stack_tls_protect_set_di */
    case 923:  /* stack_tls_protect_set_si */
    case 922:  /* stack_protect_set_di */
    case 921:  /* stack_protect_set_si */
    case 917:  /* probe_stack_rangedi */
    case 916:  /* probe_stack_rangesi */
    case 915:  /* adjust_stack_and_probedi */
    case 914:  /* adjust_stack_and_probesi */
    case 913:  /* allocate_stack_worker_probe_di */
    case 912:  /* allocate_stack_worker_probe_si */
    case 911:  /* pro_epilogue_adjust_stack_di_sub */
    case 910:  /* pro_epilogue_adjust_stack_si_sub */
    case 909:  /* pro_epilogue_adjust_stack_di_add */
    case 908:  /* pro_epilogue_adjust_stack_si_add */
    case 893:  /* *movqicc_noc */
    case 892:  /* *movdicc_noc */
    case 891:  /* *movsicc_noc */
    case 890:  /* *movhicc_noc */
    case 889:  /* *x86_movdicc_0_m1_neg */
    case 888:  /* *x86_movsicc_0_m1_neg */
    case 887:  /* *x86_movdicc_0_m1_se */
    case 886:  /* *x86_movsicc_0_m1_se */
    case 885:  /* *x86_movdicc_0_m1 */
    case 884:  /* *x86_movsicc_0_m1 */
    case 883:  /* *strlenqi_1 */
    case 882:  /* *strlenqi_1 */
    case 881:  /* *cmpstrnqi_1 */
    case 880:  /* *cmpstrnqi_1 */
    case 879:  /* *cmpstrnqi_nz_1 */
    case 878:  /* *cmpstrnqi_nz_1 */
    case 877:  /* *rep_stosqi */
    case 876:  /* *rep_stosqi */
    case 875:  /* *rep_stossi */
    case 874:  /* *rep_stossi */
    case 873:  /* *rep_stosdi_rex64 */
    case 872:  /* *strsetqi_1 */
    case 871:  /* *strsetqi_1 */
    case 870:  /* *strsethi_1 */
    case 869:  /* *strsethi_1 */
    case 868:  /* *strsetsi_1 */
    case 867:  /* *strsetsi_1 */
    case 866:  /* *strsetdi_rex_1 */
    case 865:  /* *rep_movqi */
    case 864:  /* *rep_movqi */
    case 863:  /* *rep_movsi */
    case 862:  /* *rep_movsi */
    case 861:  /* *rep_movdi_rex64 */
    case 860:  /* *strmovqi_1 */
    case 859:  /* *strmovqi_1 */
    case 858:  /* *strmovhi_1 */
    case 857:  /* *strmovhi_1 */
    case 856:  /* *strmovsi_1 */
    case 855:  /* *strmovsi_1 */
    case 854:  /* *strmovdi_rex_1 */
    case 735:  /* *tls_dynamic_gnu2_call_64 */
    case 734:  /* *tls_dynamic_gnu2_lea_64 */
    case 732:  /* *tls_dynamic_gnu2_call_32 */
    case 731:  /* *tls_dynamic_gnu2_lea_32 */
    case 730:  /* *add_tp_di */
    case 729:  /* *add_tp_si */
    case 728:  /* *add_tp_x32_zext */
    case 727:  /* *add_tp_x32 */
    case 726:  /* *load_tp_di */
    case 725:  /* *load_tp_si */
    case 724:  /* *load_tp_x32_zext */
    case 723:  /* *load_tp_x32 */
    case 721:  /* *tls_local_dynamic_base_64 */
    case 720:  /* *tls_local_dynamic_base_32_gnu */
    case 719:  /* *tls_global_dynamic_64 */
    case 718:  /* *tls_global_dynamic_32_gnu */
    case 712:  /* *bswapdi2_1 */
    case 711:  /* *bswapsi2_1 */
    case 710:  /* *bswapdi2_movbe */
    case 709:  /* *bswapsi2_movbe */
    case 708:  /* *popcountsi2_cmp_zext */
    case 707:  /* *popcountdi2_cmp */
    case 706:  /* *popcountsi2_cmp */
    case 705:  /* *popcounthi2_cmp */
    case 704:  /* popcountdi2 */
    case 703:  /* popcountsi2 */
    case 702:  /* popcounthi2 */
    case 701:  /* *bsrhi */
    case 700:  /* bsr */
    case 699:  /* bsr_rex64 */
    case 698:  /* *tbm_tzmsk_di */
    case 697:  /* *tbm_tzmsk_si */
    case 696:  /* *tbm_t1mskc_di */
    case 695:  /* *tbm_t1mskc_si */
    case 694:  /* *tbm_blsic_di */
    case 693:  /* *tbm_blsic_si */
    case 692:  /* *tbm_blsfill_di */
    case 691:  /* *tbm_blsfill_si */
    case 690:  /* *tbm_blcs_di */
    case 689:  /* *tbm_blcs_si */
    case 688:  /* *tbm_blcmsk_di */
    case 687:  /* *tbm_blcmsk_si */
    case 686:  /* *tbm_blcic_di */
    case 685:  /* *tbm_blcic_si */
    case 684:  /* *tbm_blci_di */
    case 683:  /* *tbm_blci_si */
    case 682:  /* *tbm_blcfill_di */
    case 681:  /* *tbm_blcfill_si */
    case 680:  /* tbm_bextri_di */
    case 679:  /* tbm_bextri_si */
    case 678:  /* bmi2_pext_di3 */
    case 677:  /* bmi2_pext_si3 */
    case 676:  /* bmi2_pdep_di3 */
    case 675:  /* bmi2_pdep_si3 */
    case 674:  /* bmi2_bzhi_di3 */
    case 673:  /* bmi2_bzhi_si3 */
    case 672:  /* *bmi_blsr_di */
    case 671:  /* *bmi_blsr_si */
    case 670:  /* *bmi_blsmsk_di */
    case 669:  /* *bmi_blsmsk_si */
    case 668:  /* *bmi_blsi_di */
    case 667:  /* *bmi_blsi_si */
    case 666:  /* bmi_bextr_di */
    case 665:  /* bmi_bextr_si */
    case 664:  /* *bmi_andn_di */
    case 663:  /* *bmi_andn_si */
    case 662:  /* clzdi2_lzcnt */
    case 661:  /* clzsi2_lzcnt */
    case 660:  /* clzhi2_lzcnt */
    case 659:  /* ctzdi2 */
    case 658:  /* ctzsi2 */
    case 657:  /* ctzhi2 */
    case 656:  /* *ffsdi_1 */
    case 655:  /* *ffssi_1 */
    case 652:  /* leave_rex64 */
    case 651:  /* leave */
    case 649:  /* set_got_offset_rex64 */
    case 648:  /* set_rip_rex64 */
    case 647:  /* set_got_rex64 */
    case 646:  /* set_got_labelled */
    case 645:  /* set_got */
    case 641:  /* simple_return_indirect_internal */
    case 634:  /* *sibcall_value_pop */
    case 633:  /* *sibcall_value_pop_vzeroupper */
    case 632:  /* *call_value_pop */
    case 631:  /* *call_value_pop_vzeroupper */
    case 630:  /* *call_value_rex64_ms_sysv */
    case 629:  /* *call_value_rex64_ms_sysv_vzeroupper */
    case 628:  /* *sibcall_value */
    case 627:  /* *sibcall_value */
    case 626:  /* *sibcall_value_vzeroupper */
    case 625:  /* *sibcall_value_vzeroupper */
    case 624:  /* *call_value */
    case 623:  /* *call_value */
    case 622:  /* *call_value_vzeroupper */
    case 621:  /* *call_value_vzeroupper */
    case 620:  /* *sibcall_pop */
    case 619:  /* *sibcall_pop_vzeroupper */
    case 618:  /* *call_pop */
    case 617:  /* *call_pop_vzeroupper */
    case 616:  /* *sibcall */
    case 615:  /* *sibcall */
    case 614:  /* *sibcall_vzeroupper */
    case 613:  /* *sibcall_vzeroupper */
    case 612:  /* *call_rex64_ms_sysv */
    case 611:  /* *call_rex64_ms_sysv_vzeroupper */
    case 610:  /* *call */
    case 609:  /* *call */
    case 608:  /* *call_vzeroupper */
    case 607:  /* *call_vzeroupper */
    case 606:  /* *tablejump_1 */
    case 605:  /* *tablejump_1 */
    case 604:  /* *indirect_jump */
    case 603:  /* *indirect_jump */
    case 602:  /* jump */
    case 588:  /* *jcc_2 */
    case 587:  /* *jcc_1 */
    case 584:  /* *setcc_qi_slp */
    case 583:  /* *setcc_qi */
    case 579:  /* *btdi */
    case 578:  /* *btsi */
    case 577:  /* *btcq */
    case 576:  /* *btrq */
    case 575:  /* *btsq */
    case 574:  /* *rotrqi3_1_slp */
    case 573:  /* *rotlqi3_1_slp */
    case 572:  /* *rotrhi3_1 */
    case 571:  /* *rotlhi3_1 */
    case 570:  /* *rotrqi3_1 */
    case 569:  /* *rotlqi3_1 */
    case 568:  /* *rotrsi3_1_zext */
    case 567:  /* *rotlsi3_1_zext */
    case 566:  /* *bmi2_rorxsi3_1_zext */
    case 565:  /* *rotrdi3_1 */
    case 564:  /* *rotldi3_1 */
    case 563:  /* *rotrsi3_1 */
    case 562:  /* *rotlsi3_1 */
    case 561:  /* *bmi2_rorxdi3_1 */
    case 560:  /* *bmi2_rorxsi3_1 */
    case 555:  /* *rotrdi3_mask */
    case 554:  /* *rotldi3_mask */
    case 553:  /* *rotrsi3_mask */
    case 552:  /* *rotlsi3_mask */
    case 551:  /* *ashrdi3_cconly */
    case 550:  /* *lshrdi3_cconly */
    case 549:  /* *ashrsi3_cconly */
    case 548:  /* *lshrsi3_cconly */
    case 547:  /* *ashrhi3_cconly */
    case 546:  /* *lshrhi3_cconly */
    case 545:  /* *ashrqi3_cconly */
    case 544:  /* *lshrqi3_cconly */
    case 543:  /* *ashrsi3_cmp_zext */
    case 542:  /* *lshrsi3_cmp_zext */
    case 541:  /* *ashrdi3_cmp */
    case 540:  /* *lshrdi3_cmp */
    case 539:  /* *ashrsi3_cmp */
    case 538:  /* *lshrsi3_cmp */
    case 537:  /* *ashrhi3_cmp */
    case 536:  /* *lshrhi3_cmp */
    case 535:  /* *ashrqi3_cmp */
    case 534:  /* *lshrqi3_cmp */
    case 533:  /* *ashrqi3_1_slp */
    case 532:  /* *lshrqi3_1_slp */
    case 531:  /* *ashrhi3_1 */
    case 530:  /* *lshrhi3_1 */
    case 529:  /* *ashrqi3_1 */
    case 528:  /* *lshrqi3_1 */
    case 527:  /* *ashrsi3_1_zext */
    case 526:  /* *lshrsi3_1_zext */
    case 525:  /* *bmi2_ashrsi3_1_zext */
    case 524:  /* *bmi2_lshrsi3_1_zext */
    case 523:  /* *ashrdi3_1 */
    case 522:  /* *lshrdi3_1 */
    case 521:  /* *ashrsi3_1 */
    case 520:  /* *lshrsi3_1 */
    case 519:  /* *bmi2_ashrdi3_1 */
    case 518:  /* *bmi2_lshrdi3_1 */
    case 517:  /* *bmi2_ashrsi3_1 */
    case 516:  /* *bmi2_lshrsi3_1 */
    case 515:  /* *ashrsi3_cvt_zext */
    case 514:  /* ashrsi3_cvt */
    case 513:  /* ashrdi3_cvt */
    case 512:  /* x86_shrd */
    case 511:  /* x86_64_shrd */
    case 510:  /* *ashrti3_doubleword */
    case 509:  /* *lshrti3_doubleword */
    case 508:  /* *ashrdi3_doubleword */
    case 507:  /* *lshrdi3_doubleword */
    case 506:  /* *ashrdi3_mask */
    case 505:  /* *lshrdi3_mask */
    case 504:  /* *ashrsi3_mask */
    case 503:  /* *lshrsi3_mask */
    case 502:  /* *ashldi3_cconly */
    case 501:  /* *ashlsi3_cconly */
    case 500:  /* *ashlhi3_cconly */
    case 499:  /* *ashlqi3_cconly */
    case 498:  /* *ashlsi3_cmp_zext */
    case 497:  /* *ashldi3_cmp */
    case 496:  /* *ashlsi3_cmp */
    case 495:  /* *ashlhi3_cmp */
    case 494:  /* *ashlqi3_cmp */
    case 493:  /* *ashlqi3_1_slp */
    case 492:  /* *ashlqi3_1 */
    case 491:  /* *ashlhi3_1 */
    case 490:  /* *ashlsi3_1_zext */
    case 489:  /* *bmi2_ashlsi3_1_zext */
    case 488:  /* *ashldi3_1 */
    case 487:  /* *ashlsi3_1 */
    case 486:  /* *bmi2_ashldi3_1 */
    case 485:  /* *bmi2_ashlsi3_1 */
    case 484:  /* *ashldi3_mask */
    case 483:  /* *ashlsi3_mask */
    case 482:  /* x86_shld */
    case 481:  /* x86_64_shld */
    case 480:  /* *ashlti3_doubleword */
    case 479:  /* *ashldi3_doubleword */
    case 478:  /* *one_cmplsi2_2_zext */
    case 477:  /* *one_cmpldi2_2 */
    case 476:  /* *one_cmplsi2_2 */
    case 475:  /* *one_cmplhi2_2 */
    case 474:  /* *one_cmplqi2_2 */
    case 473:  /* *one_cmplsi2_1_zext */
    case 472:  /* *one_cmplqi2_1 */
    case 471:  /* *one_cmpldi2_1 */
    case 470:  /* *one_cmplsi2_1 */
    case 469:  /* *one_cmplhi2_1 */
    case 442:  /* *negsi2_cmpz_zext */
    case 441:  /* *negdi2_cmpz */
    case 440:  /* *negsi2_cmpz */
    case 439:  /* *neghi2_cmpz */
    case 438:  /* *negqi2_cmpz */
    case 437:  /* *negsi2_1_zext */
    case 436:  /* *negdi2_1 */
    case 435:  /* *negsi2_1 */
    case 434:  /* *neghi2_1 */
    case 433:  /* *negqi2_1 */
    case 430:  /* *xorqi_cc_ext_1 */
    case 429:  /* *xorqi_cc_ext_1_rex64 */
    case 428:  /* *xorqi_ext_2 */
    case 427:  /* *iorqi_ext_2 */
    case 426:  /* *xorqi_ext_1 */
    case 425:  /* *iorqi_ext_1 */
    case 424:  /* *xorqi_ext_1_rex64 */
    case 423:  /* *iorqi_ext_1_rex64 */
    case 422:  /* *xorqi_ext_0 */
    case 421:  /* *iorqi_ext_0 */
    case 420:  /* *xordi_3 */
    case 419:  /* *iordi_3 */
    case 418:  /* *xorsi_3 */
    case 417:  /* *iorsi_3 */
    case 416:  /* *xorhi_3 */
    case 415:  /* *iorhi_3 */
    case 414:  /* *xorqi_3 */
    case 413:  /* *iorqi_3 */
    case 412:  /* *xorqi_2_slp */
    case 411:  /* *iorqi_2_slp */
    case 410:  /* *xorsi_2_zext_imm */
    case 409:  /* *iorsi_2_zext_imm */
    case 408:  /* *xorsi_2_zext */
    case 407:  /* *iorsi_2_zext */
    case 406:  /* *xordi_2 */
    case 405:  /* *iordi_2 */
    case 404:  /* *xorsi_2 */
    case 403:  /* *iorsi_2 */
    case 402:  /* *xorhi_2 */
    case 401:  /* *iorhi_2 */
    case 400:  /* *xorqi_2 */
    case 399:  /* *iorqi_2 */
    case 398:  /* *xorqi_1_slp */
    case 397:  /* *iorqi_1_slp */
    case 396:  /* *xorsi_1_zext_imm */
    case 395:  /* *iorsi_1_zext_imm */
    case 394:  /* *xorsi_1_zext */
    case 393:  /* *iorsi_1_zext */
    case 392:  /* *xorqi_1 */
    case 391:  /* *iorqi_1 */
    case 390:  /* *xordi_1 */
    case 389:  /* *iordi_1 */
    case 388:  /* *xorsi_1 */
    case 387:  /* *iorsi_1 */
    case 386:  /* *xorhi_1 */
    case 385:  /* *iorhi_1 */
    case 384:  /* *andqi_ext_2 */
    case 383:  /* *andqi_ext_1 */
    case 382:  /* *andqi_ext_1_rex64 */
    case 381:  /* *andqi_ext_0_cc */
    case 380:  /* andqi_ext_0 */
    case 379:  /* *andqi_2_slp */
    case 378:  /* *andsi_2_zext */
    case 377:  /* *andsi_2 */
    case 376:  /* *andhi_2 */
    case 375:  /* *andqi_2 */
    case 374:  /* *andqi_2_maybe_si */
    case 373:  /* *anddi_2 */
    case 372:  /* *andqi_1_slp */
    case 371:  /* *andqi_1 */
    case 370:  /* *andhi_1 */
    case 369:  /* *andsi_1_zext */
    case 368:  /* *andsi_1 */
    case 367:  /* *anddi_1 */
    case 364:  /* *testqi_ext_2 */
    case 363:  /* *testqi_ext_1 */
    case 362:  /* *testqi_ext_1_rex64 */
    case 361:  /* *testqi_ext_0 */
    case 360:  /* *testsi_1 */
    case 359:  /* *testhi_1 */
    case 358:  /* *testqi_1 */
    case 357:  /* *testqi_1_maybe_si */
    case 356:  /* *testdi_1 */
    case 355:  /* udivmodhiqi3 */
    case 354:  /* *udivmoddi4_noext */
    case 353:  /* *udivmodsi4_noext */
    case 352:  /* *udivmodhi4_noext */
    case 351:  /* *udivmoddi4 */
    case 350:  /* *udivmodsi4 */
    case 349:  /* *udivmodhi4 */
    case 348:  /* udivmoddi4_1 */
    case 347:  /* udivmodsi4_1 */
    case 346:  /* divmodhiqi3 */
    case 345:  /* *divmoddi4_noext */
    case 344:  /* *divmodsi4_noext */
    case 343:  /* *divmodhi4_noext */
    case 342:  /* *divmoddi4 */
    case 341:  /* *divmodsi4 */
    case 340:  /* *divmodhi4 */
    case 339:  /* divmoddi4_1 */
    case 338:  /* divmodsi4_1 */
    case 337:  /* *umulsi3_highpart_zext */
    case 336:  /* *smulsi3_highpart_zext */
    case 335:  /* *umulsi3_highpart_1 */
    case 334:  /* *smulsi3_highpart_1 */
    case 333:  /* *umuldi3_highpart_1 */
    case 332:  /* *smuldi3_highpart_1 */
    case 331:  /* *umulqihi3_1 */
    case 330:  /* *mulqihi3_1 */
    case 329:  /* *mulditi3_1 */
    case 328:  /* *mulsidi3_1 */
    case 327:  /* *umulditi3_1 */
    case 326:  /* *umulsidi3_1 */
    case 325:  /* *bmi2_umulsidi3_1 */
    case 324:  /* *bmi2_umulditi3_1 */
    case 323:  /* *mulqi3_1 */
    case 322:  /* *mulhi3_1 */
    case 321:  /* *mulsi3_1_zext */
    case 320:  /* *muldi3_1 */
    case 319:  /* *mulsi3_1 */
    case 318:  /* *addsi3_zext_cc_overflow */
    case 317:  /* *adddi3_cc_overflow */
    case 316:  /* *addsi3_cc_overflow */
    case 315:  /* *addhi3_cc_overflow */
    case 314:  /* *addqi3_cc_overflow */
    case 313:  /* *adddi3_cconly_overflow */
    case 312:  /* *addsi3_cconly_overflow */
    case 311:  /* *addhi3_cconly_overflow */
    case 310:  /* *addqi3_cconly_overflow */
    case 309:  /* *subsi3_carry_zext */
    case 308:  /* *addsi3_carry_zext */
    case 307:  /* *subdi3_carry */
    case 306:  /* *adddi3_carry */
    case 305:  /* *subsi3_carry */
    case 304:  /* *addsi3_carry */
    case 303:  /* *subhi3_carry */
    case 302:  /* *addhi3_carry */
    case 301:  /* *subqi3_carry */
    case 300:  /* *addqi3_carry */
    case 299:  /* *subsi_3_zext */
    case 298:  /* *subdi_3 */
    case 297:  /* *subsi_3 */
    case 296:  /* *subhi_3 */
    case 295:  /* *subqi_3 */
    case 294:  /* *subsi_2_zext */
    case 293:  /* *subdi_2 */
    case 292:  /* *subsi_2 */
    case 291:  /* *subhi_2 */
    case 290:  /* *subqi_2 */
    case 289:  /* *subqi_1_slp */
    case 288:  /* *subsi_1_zext */
    case 287:  /* *subdi_1 */
    case 286:  /* *subsi_1 */
    case 285:  /* *subhi_1 */
    case 284:  /* *subqi_1 */
    case 281:  /* *lea_general_4 */
    case 280:  /* *lea_general_4 */
    case 279:  /* *lea_general_3 */
    case 278:  /* *lea_general_2 */
    case 277:  /* *lea_general_1 */
    case 276:  /* *addqi_ext_2 */
    case 275:  /* addqi_ext_1 */
    case 274:  /* *addqi_ext_1_rex64 */
    case 273:  /* *adddi_5 */
    case 272:  /* *addsi_5 */
    case 271:  /* *addhi_5 */
    case 270:  /* *addqi_5 */
    case 269:  /* *addsi_4 */
    case 268:  /* *addhi_4 */
    case 267:  /* *addqi_4 */
    case 266:  /* *adddi_4 */
    case 265:  /* *addsi_3_zext */
    case 264:  /* *adddi_3 */
    case 263:  /* *addsi_3 */
    case 262:  /* *addhi_3 */
    case 261:  /* *addqi_3 */
    case 260:  /* *addsi_2_zext */
    case 259:  /* *adddi_2 */
    case 258:  /* *addsi_2 */
    case 257:  /* *addhi_2 */
    case 256:  /* *addqi_2 */
    case 255:  /* *addqi_1_slp */
    case 254:  /* *addqi_1 */
    case 253:  /* *addhi_1 */
    case 252:  /* addsi_1_zext */
    case 251:  /* *adddi_1 */
    case 250:  /* *addsi_1 */
    case 249:  /* addqi3_cc */
    case 248:  /* *adddi3_cc */
    case 247:  /* *addsi3_cc */
    case 244:  /* *leadi */
    case 243:  /* *leasi */
    case 242:  /* *floatunssixf2_1 */
    case 241:  /* *floatunssidf2_1 */
    case 240:  /* *floatunssisf2_1 */
    case 134:  /* extendqihi2 */
    case 133:  /* *extendqisi2_zext */
    case 132:  /* extendqisi2 */
    case 131:  /* *extendhisi2_zext */
    case 130:  /* extendhisi2 */
    case 129:  /* extendhidi2 */
    case 128:  /* extendqidi2 */
    case 126:  /* *extendsidi2_rex64 */
    case 125:  /* *zero_extendqisi2_movzbl */
    case 124:  /* *zero_extendqihi2_movzbl */
    case 123:  /* *zero_extendqisi2_movzbl_and */
    case 122:  /* *zero_extendqihi2_movzbl_and */
    case 121:  /* *zero_extendqisi2_and */
    case 120:  /* *zero_extendqihi2_and */
    case 119:  /* *zero_extendhisi2_movzwl */
    case 118:  /* zero_extendhisi2_and */
    case 117:  /* zero_extendhidi2 */
    case 116:  /* zero_extendqidi2 */
    case 96:  /* *movqi_insv_2 */
    case 95:  /* *movsi_insv_1 */
    case 94:  /* *movdi_insv_1_rex64 */
    case 93:  /* *movsi_insv_1_rex64 */
    case 92:  /* *movqi_extzv_2 */
    case 91:  /* *movqi_extzv_2_rex64 */
    case 90:  /* *movdi_extzv_1 */
    case 89:  /* *movsi_extzv_1 */
    case 88:  /* *movqi_extv_1 */
    case 87:  /* *movqi_extv_1_rex64 */
    case 86:  /* *movsi_extv_1 */
    case 85:  /* *movhi_extv_1 */
    case 84:  /* *movstricthi_xor */
    case 83:  /* *movstrictqi_xor */
    case 82:  /* *movstricthi_1 */
    case 81:  /* *movstrictqi_1 */
    case 80:  /* *swaphi_2 */
    case 79:  /* *swapqi_2 */
    case 78:  /* *swaphi_1 */
    case 77:  /* *swapqi_1 */
    case 76:  /* *swapdi */
    case 75:  /* *swapsi */
    case 74:  /* *movabsdi_2 */
    case 73:  /* *movabssi_2 */
    case 72:  /* *movabshi_2 */
    case 71:  /* *movabsqi_2 */
    case 70:  /* *movabsdi_1 */
    case 69:  /* *movabssi_1 */
    case 68:  /* *movabshi_1 */
    case 67:  /* *movabsqi_1 */
    case 66:  /* *movqi_internal */
    case 65:  /* *movhi_internal */
    case 58:  /* *movdi_or */
    case 57:  /* *movsi_or */
    case 56:  /* *movdi_xor */
    case 55:  /* *movsi_xor */
    case 54:  /* *popdi1_epilogue */
    case 53:  /* *popsi1_epilogue */
    case 52:  /* *popdi1 */
    case 51:  /* *popsi1 */
    case 50:  /* *pushdi2_prologue */
    case 49:  /* *pushsi2_prologue */
    case 48:  /* *pushhi2 */
    case 47:  /* *pushqi2 */
    case 46:  /* *pushsi2_rex64 */
    case 45:  /* *pushhi2_rex64 */
    case 44:  /* *pushqi2_rex64 */
    case 43:  /* *pushsi2 */
    case 42:  /* *pushdi2_rex64 */
    case 41:  /* *pushti2 */
    case 40:  /* *pushdi2 */
    case 17:  /* *cmpqi_ext_4 */
    case 16:  /* *cmpqi_ext_3_insn_rex64 */
    case 15:  /* *cmpqi_ext_3_insn */
    case 14:  /* *cmpqi_ext_2 */
    case 13:  /* *cmpqi_ext_1_rex64 */
    case 12:  /* *cmpqi_ext_1 */
    case 11:  /* *cmpdi_minus_1 */
    case 10:  /* *cmpsi_minus_1 */
    case 9:  /* *cmphi_minus_1 */
    case 8:  /* *cmpqi_minus_1 */
    case 7:  /* *cmpdi_1 */
    case 6:  /* *cmpsi_1 */
    case 5:  /* *cmphi_1 */
    case 4:  /* *cmpqi_1 */
    case 3:  /* *cmpdi_ccno_1 */
    case 2:  /* *cmpsi_ccno_1 */
    case 1:  /* *cmphi_ccno_1 */
    case 0:  /* *cmpqi_ccno_1 */
      return UNIT_INTEGER;

    default:
      return UNIT_SSE;

    }
}

EXPORTED_CONST int length_unit_log = 0;
