/* Generated automatically by the program `genoutput'
   from the machine description file `md'.  */

#include "config.h"
#include "system.h"
#include "coretypes.h"
#include "tm.h"
#include "flags.h"
#include "ggc.h"
#include "rtl.h"
#include "expr.h"
#include "insn-codes.h"
#include "tm_p.h"
#include "function.h"
#include "regs.h"
#include "hard-reg-set.h"
#include "insn-config.h"

#include "conditions.h"
#include "insn-attr.h"

#include "recog.h"

#include "diagnostic-core.h"
#include "output.h"
#include "target.h"
#include "tm-constrs.h"

static const char * const output_0[] = {
  "test{b}\t%0, %0",
  "cmp{b}\t{%1, %0|%0, %1}",
};

static const char * const output_1[] = {
  "test{w}\t%0, %0",
  "cmp{w}\t{%1, %0|%0, %1}",
};

static const char * const output_2[] = {
  "test{l}\t%0, %0",
  "cmp{l}\t{%1, %0|%0, %1}",
};

static const char * const output_3[] = {
  "test{q}\t%0, %0",
  "cmp{q}\t{%1, %0|%0, %1}",
};

static const char *
output_18 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1213 "../../gcc/config/i386/i386.md"
 return output_fp_compare (insn, operands, false, false);
}

static const char *
output_20 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1260 "../../gcc/config/i386/i386.md"
 return output_fp_compare (insn, operands, false, false);
}

static const char *
output_22 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1294 "../../gcc/config/i386/i386.md"
 return output_fp_compare (insn, operands, false, false);
}

static const char *
output_23 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1294 "../../gcc/config/i386/i386.md"
 return output_fp_compare (insn, operands, false, false);
}

static const char *
output_26 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1329 "../../gcc/config/i386/i386.md"
 return output_fp_compare (insn, operands, false, true);
}

static const char *
output_28 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1379 "../../gcc/config/i386/i386.md"
 return output_fp_compare (insn, operands, false, false);
}

static const char *
output_29 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1379 "../../gcc/config/i386/i386.md"
 return output_fp_compare (insn, operands, false, false);
}

static const char *
output_33 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1433 "../../gcc/config/i386/i386.md"
{
#ifndef HAVE_AS_IX86_SAHF
  if (TARGET_64BIT)
    return ASM_BYTE "0x9e";
  else
#endif
  return "sahf";
}
}

static const char *
output_34 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1457 "../../gcc/config/i386/i386.md"
 return output_fp_compare (insn, operands, true, false);
}

static const char *
output_35 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1486 "../../gcc/config/i386/i386.md"
 return output_fp_compare (insn, operands, true, false);
}

static const char *
output_36 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1510 "../../gcc/config/i386/i386.md"
 return output_fp_compare (insn, operands, true, false);
}

static const char *
output_37 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1530 "../../gcc/config/i386/i386.md"
 return output_fp_compare (insn, operands, true, true);
}

static const char *
output_38 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1559 "../../gcc/config/i386/i386.md"
 return output_fp_compare (insn, operands, true, true);
}

static const char *
output_39 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1583 "../../gcc/config/i386/i386.md"
 return output_fp_compare (insn, operands, true, true);
}

static const char * const output_42[] = {
  "push{q}\t%1",
  "#",
};

static const char *
output_59 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1804 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      if (misaligned_operand (operands[0], OImode)
	  || misaligned_operand (operands[1], OImode))
	return "vmovdqu\t{%1, %0|%0, %1}";
      else
	return "vmovdqa\t{%1, %0|%0, %1}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_60 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1828 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
    case 1:
      return "#";
    case 2:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 3:
    case 4:
      /* TDmode values are passed as TImode on the stack.  Moving them
	 to stack may result in unaligned memory access.  */
      if (misaligned_operand (operands[0], TImode)
	  || misaligned_operand (operands[1], TImode))
	{
	  if (get_attr_mode (insn) == MODE_V4SF)
	    return "%vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqu\t{%1, %0|%0, %1}";
	}
      else
	{
	  if (get_attr_mode (insn) == MODE_V4SF)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_61 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1886 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      /* TDmode values are passed as TImode on the stack.  Moving them
	 to stack may result in unaligned memory access.  */
      if (misaligned_operand (operands[0], TImode)
	  || misaligned_operand (operands[1], TImode))
	{
	  if (get_attr_mode (insn) == MODE_V4SF)
	    return "%vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqu\t{%1, %0|%0, %1}";
	}
      else
	{
	  if (get_attr_mode (insn) == MODE_V4SF)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_62 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1931 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_SSECVT:
      if (SSE_REG_P (operands[0]))
	return "movq2dq\t{%1, %0|%0, %1}";
      else
	return "movdq2q\t{%1, %0|%0, %1}";

    case TYPE_SSEMOV:
      if (get_attr_mode (insn) == MODE_TI)
	return "%vmovdqa\t{%1, %0|%0, %1}";
      /* Handle broken assemblers that require movd instead of movq.  */
      if (GENERAL_REG_P (operands[0]) || GENERAL_REG_P (operands[1]))
	return "%vmovd\t{%1, %0|%0, %1}";
      else
	return "%vmovq\t{%1, %0|%0, %1}";

    case TYPE_MMXMOV:
      /* Handle broken assemblers that require movd instead of movq.  */
      if (GENERAL_REG_P (operands[0]) || GENERAL_REG_P (operands[1]))
	return "movd\t{%1, %0|%0, %1}";
      else
	return "movq\t{%1, %0|%0, %1}";

    case TYPE_SSELOG1:
      return standard_sse_constant_opcode (insn, operands[1]);

    case TYPE_MMX:
      return "pxor\t%0, %0";

    case TYPE_LEA:
      return "lea{q}\t{%E1, %0|%0, %E1}";

    default:
      gcc_assert (!flag_pic || LEGITIMATE_PIC_OPERAND_P (operands[1]));
      if (get_attr_mode (insn) == MODE_SI)
	return "mov{l}\t{%k1, %k0|%k0, %k1}";
      else if (which_alternative == 2)
	return "movabs{q}\t{%1, %0|%0, %1}";
      else if (ix86_use_lea_for_mov (insn, operands))
	return "lea{q}\t{%E1, %0|%0, %E1}";
      else
	return "mov{q}\t{%1, %0|%0, %1}";
    }
}
}

static const char *
output_63 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2056 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_SSECVT:
      if (SSE_REG_P (operands[0]))
	return "movq2dq\t{%1, %0|%0, %1}";
      else
	return "movdq2q\t{%1, %0|%0, %1}";

    case TYPE_SSEMOV:
      switch (get_attr_mode (insn))
	{
	case MODE_TI:
	  return "%vmovdqa\t{%1, %0|%0, %1}";
	case MODE_DI:
	   return "%vmovq\t{%1, %0|%0, %1}";
	case MODE_V4SF:
	  return "movaps\t{%1, %0|%0, %1}";
	case MODE_V2SF:
	  return "movlps\t{%1, %0|%0, %1}";
	default:
	  gcc_unreachable ();
	}

    case TYPE_MMXMOV:
      return "movq\t{%1, %0|%0, %1}";

    case TYPE_SSELOG1:
      return standard_sse_constant_opcode (insn, operands[1]);

    case TYPE_MMX:
      return "pxor\t%0, %0";

    case TYPE_MULTI:
      return "#";

    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_64 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2137 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_SSELOG1:
      return standard_sse_constant_opcode (insn, operands[1]);

    case TYPE_SSEMOV:
      switch (get_attr_mode (insn))
	{
	case MODE_TI:
	  return "%vmovdqa\t{%1, %0|%0, %1}";
	case MODE_V4SF:
	  return "%vmovaps\t{%1, %0|%0, %1}";
	case MODE_SI:
          return "%vmovd\t{%1, %0|%0, %1}";
	case MODE_SF:
          return "%vmovss\t{%1, %0|%0, %1}";
	default:
	  gcc_unreachable ();
	}

    case TYPE_MMX:
      return "pxor\t%0, %0";

    case TYPE_MMXMOV:
      if (get_attr_mode (insn) == MODE_DI)
	return "movq\t{%1, %0|%0, %1}";
      return "movd\t{%1, %0|%0, %1}";

    case TYPE_LEA:
      return "lea{l}\t{%E1, %0|%0, %E1}";

    default:
      gcc_assert (!flag_pic || LEGITIMATE_PIC_OPERAND_P (operands[1]));
      if (ix86_use_lea_for_mov (insn, operands))
	return "lea{l}\t{%E1, %0|%0, %E1}";
      else
	return "mov{l}\t{%1, %0|%0, %1}";
    }
}
}

static const char *
output_65 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2216 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_IMOVX:
      /* movzwl is faster than movw on p2 due to partial word stalls,
	 though not as fast as an aligned movl.  */
      return "movz{wl|x}\t{%1, %k0|%k0, %1}";
    default:
      if (get_attr_mode (insn) == MODE_SI)
        return "mov{l}\t{%k1, %k0|%k0, %k1}";
      else
        return "mov{w}\t{%1, %0|%0, %1}";
    }
}
}

static const char *
output_66 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2272 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_IMOVX:
      gcc_assert (ANY_QI_REG_P (operands[1]) || MEM_P (operands[1]));
      return "movz{bl|x}\t{%1, %k0|%k0, %1}";
    default:
      if (get_attr_mode (insn) == MODE_SI)
        return "mov{l}\t{%k1, %k0|%k0, %k1}";
      else
        return "mov{b}\t{%1, %0|%0, %1}";
    }
}
}

static const char * const output_67[] = {
  "movabs{b}\t{%1, %P0|[%P0], %1}",
  "mov{b}\t{%1, %a0|BYTE PTR %a0, %1}",
};

static const char * const output_68[] = {
  "movabs{w}\t{%1, %P0|[%P0], %1}",
  "mov{w}\t{%1, %a0|WORD PTR %a0, %1}",
};

static const char * const output_69[] = {
  "movabs{l}\t{%1, %P0|[%P0], %1}",
  "mov{l}\t{%1, %a0|DWORD PTR %a0, %1}",
};

static const char * const output_70[] = {
  "movabs{q}\t{%1, %P0|[%P0], %1}",
  "mov{q}\t{%1, %a0|QWORD PTR %a0, %1}",
};

static const char * const output_71[] = {
  "movabs{b}\t{%P1, %0|%0, [%P1]}",
  "mov{b}\t{%a1, %0|%0, BYTE PTR %a1}",
};

static const char * const output_72[] = {
  "movabs{w}\t{%P1, %0|%0, [%P1]}",
  "mov{w}\t{%a1, %0|%0, WORD PTR %a1}",
};

static const char * const output_73[] = {
  "movabs{l}\t{%P1, %0|%0, [%P1]}",
  "mov{l}\t{%a1, %0|%0, DWORD PTR %a1}",
};

static const char * const output_74[] = {
  "movabs{q}\t{%P1, %0|%0, [%P1]}",
  "mov{q}\t{%a1, %0|%0, QWORD PTR %a1}",
};

static const char *
output_87 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2448 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_IMOVX:
      return "movs{bl|x}\t{%h1, %k0|%k0, %h1}";
    default:
      return "mov{b}\t{%h1, %0|%0, %h1}";
    }
}
}

static const char *
output_88 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2473 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_IMOVX:
      return "movs{bl|x}\t{%h1, %k0|%k0, %h1}";
    default:
      return "mov{b}\t{%h1, %0|%0, %h1}";
    }
}
}

static const char *
output_91 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2510 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_IMOVX:
      return "movz{bl|x}\t{%h1, %k0|%k0, %h1}";
    default:
      return "mov{b}\t{%h1, %0|%0, %h1}";
    }
}
}

static const char *
output_92 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2536 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_IMOVX:
      return "movz{bl|x}\t{%h1, %k0|%k0, %h1}";
    default:
      return "mov{b}\t{%h1, %0|%0, %h1}";
    }
}
}

static const char *
output_93 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2568 "../../gcc/config/i386/i386.md"
{
  if (CONST_INT_P (operands[1]))
    operands[1] = simplify_gen_subreg (QImode, operands[1], SImode, 0);
  return "mov{b}\t{%b1, %h0|%h0, %b1}";
}
}

static const char *
output_94 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2568 "../../gcc/config/i386/i386.md"
{
  if (CONST_INT_P (operands[1]))
    operands[1] = simplify_gen_subreg (QImode, operands[1], DImode, 0);
  return "mov{b}\t{%b1, %h0|%h0, %b1}";
}
}

static const char *
output_95 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2582 "../../gcc/config/i386/i386.md"
{
  if (CONST_INT_P (operands[1]))
    operands[1] = simplify_gen_subreg (QImode, operands[1], SImode, 0);
  return "mov{b}\t{%b1, %h0|%h0, %b1}";
}
}

static const char *
output_97 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2607 "../../gcc/config/i386/i386.md"
{
  /* This insn should be already split before reg-stack.  */
  gcc_unreachable ();
}
}

static const char *
output_98 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2627 "../../gcc/config/i386/i386.md"
{
  /* This insn should be already split before reg-stack.  */
  gcc_unreachable ();
}
}

static const char *
output_99 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2645 "../../gcc/config/i386/i386.md"
{
  /* This insn should be already split before reg-stack.  */
  gcc_unreachable ();
}
}

static const char *
output_100 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2666 "../../gcc/config/i386/i386.md"
{
  /* This insn should be already split before reg-stack.  */
  gcc_unreachable ();
}
}

static const char *
output_101 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2682 "../../gcc/config/i386/i386.md"
{
  /* This insn should be already split before reg-stack.  */
  gcc_unreachable ();
}
}

static const char *
output_102 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2703 "../../gcc/config/i386/i386.md"
{
  /* Anything else should be already split before reg-stack.  */
  gcc_assert (which_alternative == 1);
  return "push{q}\t%q1";
}
}

static const char *
output_103 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2716 "../../gcc/config/i386/i386.md"
{
  /* Anything else should be already split before reg-stack.  */
  gcc_assert (which_alternative == 1);
  return "push{l}\t%1";
}
}

static const char *
output_104 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2781 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
    case 1:
      /* Handle misaligned load/store since we
         don't have movmisaligntf pattern. */
      if (misaligned_operand (operands[0], TFmode)
	  || misaligned_operand (operands[1], TFmode))
	{
	  if (get_attr_mode (insn) == MODE_V4SF)
	    return "%vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqu\t{%1, %0|%0, %1}";
	}
      else
	{
	  if (get_attr_mode (insn) == MODE_V4SF)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";
	}

    case 2:
      return standard_sse_constant_opcode (insn, operands[1]);

    case 3:
    case 4:
	return "#";

    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_105 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2844 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
    case 1:
      /* Handle misaligned load/store since we
         don't have movmisaligntf pattern. */
      if (misaligned_operand (operands[0], TFmode)
	  || misaligned_operand (operands[1], TFmode))
	{
	  if (get_attr_mode (insn) == MODE_V4SF)
	    return "%vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqu\t{%1, %0|%0, %1}";
	}
      else
	{
	  if (get_attr_mode (insn) == MODE_V4SF)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";
	}

    case 2:
      return standard_sse_constant_opcode (insn, operands[1]);

    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_106 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2902 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
    case 1:
      return output_387_reg_move (insn, operands);

    case 2:
      return standard_80387_constant_opcode (operands[1]);

    case 3:
    case 4:
      return "#";

    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_107 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2936 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
    case 1:
      return output_387_reg_move (insn, operands);

    case 2:
      return standard_80387_constant_opcode (operands[1]);

    case 3:
    case 4:
      return "#";

    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_108 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2972 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
    case 1:
      return output_387_reg_move (insn, operands);

    case 2:
      return standard_80387_constant_opcode (operands[1]);

    case 3:
    case 4:
      return "mov{q}\t{%1, %0|%0, %1}";

    case 5:
      return "mov{l}\t{%1, %k0|%k0, %1}";

    case 6:
      return "movabs{q}\t{%1, %0|%0, %1}";

    case 7:
      return standard_sse_constant_opcode (insn, operands[1]);

    case 8:
    case 9:
    case 10:
      switch (get_attr_mode (insn))
	{
	case MODE_V2DF:
	  if (!TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovapd\t{%1, %0|%0, %1}";
	case MODE_V4SF:
	  return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_DI:
	  return "%vmovq\t{%1, %0|%0, %1}";
	case MODE_DF:
	  if (TARGET_AVX && REG_P (operands[0]) && REG_P (operands[1]))
	    return "vmovsd\t{%1, %0, %0|%0, %0, %1}";
	  return "%vmovsd\t{%1, %0|%0, %1}";
	case MODE_V1DF:
	  return "%vmovlpd\t{%1, %d0|%d0, %1}";
	case MODE_V2SF:
	  return "%vmovlps\t{%1, %d0|%d0, %1}";
	default:
	  gcc_unreachable ();
	}

    case 11:
    case 12:
      /* Handle broken assemblers that require movd instead of movq.  */
      return "%vmovd\t{%1, %0|%0, %1}";

    default:
      gcc_unreachable();
    }
}
}

static const char *
output_109 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3115 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
    case 1:
      return output_387_reg_move (insn, operands);

    case 2:
      return standard_80387_constant_opcode (operands[1]);

    case 3:
    case 4:
      return "#";

    case 5:
    case 9:
      return standard_sse_constant_opcode (insn, operands[1]);

    case 6:
    case 7:
    case 8:
    case 10:
    case 11:
    case 12:
      switch (get_attr_mode (insn))
	{
	case MODE_V2DF:
	  if (!TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovapd\t{%1, %0|%0, %1}";
	case MODE_V4SF:
	  return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_DI:
	  return "%vmovq\t{%1, %0|%0, %1}";
	case MODE_DF:
	  if (TARGET_AVX && REG_P (operands[0]) && REG_P (operands[1]))
	    return "vmovsd\t{%1, %0, %0|%0, %0, %1}";
	  return "%vmovsd\t{%1, %0|%0, %1}";
	case MODE_V1DF:
	  return "%vmovlpd\t{%1, %d0|%d0, %1}";
	case MODE_V2SF:
	  return "%vmovlps\t{%1, %d0|%d0, %1}";
	default:
	  gcc_unreachable ();
	}

    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_110 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3247 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
    case 1:
      return output_387_reg_move (insn, operands);

    case 2:
      return standard_80387_constant_opcode (operands[1]);

    case 3:
    case 4:
      return "mov{l}\t{%1, %0|%0, %1}";

    case 5:
      return standard_sse_constant_opcode (insn, operands[1]);

    case 6:
      if (get_attr_mode (insn) == MODE_V4SF)
	return "%vmovaps\t{%1, %0|%0, %1}";
      if (TARGET_AVX)
	return "vmovss\t{%1, %0, %0|%0, %0, %1}";

    case 7:
    case 8:
      return "%vmovss\t{%1, %0|%0, %1}";

    case 9:
    case 10:
    case 14:
    case 15:
      return "movd\t{%1, %0|%0, %1}";

    case 11:
      return "movq\t{%1, %0|%0, %1}";

    case 12:
    case 13:
      return "%vmovd\t{%1, %0|%0, %1}";

    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_111 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3410 "../../gcc/config/i386/i386.md"
{
  if (STACK_TOP_P (operands[0]))
    return "fxch\t%1";
  else
    return "fxch\t%0";
}
}

static const char *
output_112 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3425 "../../gcc/config/i386/i386.md"
{
  if (STACK_TOP_P (operands[0]))
    return "fxch\t%1";
  else
    return "fxch\t%0";
}
}

static const char *
output_113 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3425 "../../gcc/config/i386/i386.md"
{
  if (STACK_TOP_P (operands[0]))
    return "fxch\t%1";
  else
    return "fxch\t%0";
}
}

static const char * const output_114[] = {
  "mov{l}\t{%1, %k0|%k0, %1}",
  "#",
  "movd\t{%1, %0|%0, %1}",
  "movd\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
};

static const char * const output_115[] = {
  "#",
  "#",
  "#",
  "movd\t{%1, %0|%0, %1}",
  "movd\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
};

static const char * const output_126[] = {
  "{cltq|cdqe}",
  "movs{lq|x}\t{%1, %0|%0, %1}",
};

static const char *
output_130 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3749 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_prefix_0f (insn))
    {
    case 0:
      return "{cwtl|cwde}";
    default:
      return "movs{wl|x}\t{%1, %0|%0, %1}";
    }
}
}

static const char *
output_131 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3777 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_prefix_0f (insn))
    {
    case 0:
      return "{cwtl|cwde}";
    default:
      return "movs{wl|x}\t{%1, %k0|%k0, %1}";
    }
}
}

static const char *
output_134 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3820 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_prefix_0f (insn))
    {
    case 0:
      return "{cbtw|cbw}";
    default:
      return "movs{bw|x}\t{%1, %0|%0, %1}";
    }
}
}

static const char *
output_135 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3934 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
    case 1:
      return output_387_reg_move (insn, operands);

    case 2:
      return "%vcvtss2sd\t{%1, %d0|%d0, %1}";

    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_137 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3965 "../../gcc/config/i386/i386.md"
 return output_387_reg_move (insn, operands);
}

static const char *
output_138 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3994 "../../gcc/config/i386/i386.md"
 return output_387_reg_move (insn, operands);
}

static const char *
output_139 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3994 "../../gcc/config/i386/i386.md"
 return output_387_reg_move (insn, operands);
}

static const char *
output_140 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4080 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
      return output_387_reg_move (insn, operands);
    case 1:
      return "%vcvtsd2ss\t{%1, %d0|%d0, %1}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_142 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4112 "../../gcc/config/i386/i386.md"
 return output_387_reg_move (insn, operands);
}

static const char *
output_143 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4122 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
      return output_387_reg_move (insn, operands);
    case 1:
      return "%vcvtsd2ss\t{%1, %d0|%d0, %1}";

    default:
      return "#";
    }
}
}

static const char *
output_144 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4146 "../../gcc/config/i386/i386.md"
{
  switch (which_alternative)
    {
    case 0:
      return output_387_reg_move (insn, operands);

    default:
      return "#";
    }
}
}

static const char *
output_145 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4167 "../../gcc/config/i386/i386.md"
 return output_387_reg_move (insn, operands);
}

static const char *
output_146 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4208 "../../gcc/config/i386/i386.md"
{
  gcc_assert (!which_alternative);
  return output_387_reg_move (insn, operands);
}
}

static const char *
output_147 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4222 "../../gcc/config/i386/i386.md"
{
  gcc_assert (!which_alternative);
  return output_387_reg_move (insn, operands);
}
}

static const char *
output_148 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4236 "../../gcc/config/i386/i386.md"
 return output_387_reg_move (insn, operands);
}

static const char *
output_149 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4236 "../../gcc/config/i386/i386.md"
 return output_387_reg_move (insn, operands);
}

static const char *
output_150 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4245 "../../gcc/config/i386/i386.md"
 return output_387_reg_move (insn, operands);
}

static const char *
output_151 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4245 "../../gcc/config/i386/i386.md"
 return output_387_reg_move (insn, operands);
}

static const char *
output_161 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4503 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, true);
}

static const char *
output_162 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4503 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, true);
}

static const char *
output_163 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4503 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, true);
}

static const char *
output_170 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4587 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, false);
}

static const char *
output_172 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4642 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, false);
}

static const char *
output_173 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4642 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, false);
}

static const char * const output_195[] = {
  "fild%Z1\t%1",
  "#",
};

static const char * const output_196[] = {
  "fild%Z1\t%1",
  "#",
};

static const char * const output_201[] = {
  "fild%Z1\t%1",
  "%vcvtsi2ss\t{%1, %d0|%d0, %1}",
  "%vcvtsi2ss\t{%1, %d0|%d0, %1}",
};

static const char * const output_202[] = {
  "fild%Z1\t%1",
  "%vcvtsi2sd\t{%1, %d0|%d0, %1}",
  "%vcvtsi2sd\t{%1, %d0|%d0, %1}",
};

static const char * const output_203[] = {
  "fild%Z1\t%1",
  "%vcvtsi2ss{q}\t{%1, %d0|%d0, %1}",
  "%vcvtsi2ss{q}\t{%1, %d0|%d0, %1}",
};

static const char * const output_204[] = {
  "fild%Z1\t%1",
  "%vcvtsi2sd{q}\t{%1, %d0|%d0, %1}",
  "%vcvtsi2sd{q}\t{%1, %d0|%d0, %1}",
};

static const char * const output_205[] = {
  "fild%Z1\t%1",
  "%vcvtsi2ss\t{%1, %d0|%d0, %1}",
};

static const char * const output_206[] = {
  "fild%Z1\t%1",
  "%vcvtsi2sd\t{%1, %d0|%d0, %1}",
};

static const char * const output_207[] = {
  "fild%Z1\t%1",
  "%vcvtsi2ss{q}\t{%1, %d0|%d0, %1}",
};

static const char * const output_208[] = {
  "fild%Z1\t%1",
  "%vcvtsi2sd{q}\t{%1, %d0|%d0, %1}",
};

static const char * const output_225[] = {
  "fild%Z1\t%1",
  "#",
};

static const char * const output_226[] = {
  "fild%Z1\t%1",
  "#",
};

static const char * const output_227[] = {
  "fild%Z1\t%1",
  "#",
};

static const char * const output_228[] = {
  "fild%Z1\t%1",
  "#",
};

static const char * const output_229[] = {
  "fild%Z1\t%1",
  "#",
};

static const char * const output_230[] = {
  "fild%Z1\t%1",
  "#",
};

static const char *
output_243 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5435 "../../gcc/config/i386/i386.md"
{
  rtx addr = operands[1];

  if (SImode_address_operand (addr, VOIDmode))
    {
      gcc_assert (TARGET_64BIT);
      return "lea{l}\t{%E1, %k0|%k0, %E1}";
    }
  else 
    return "lea{l}\t{%E1, %0|%0, %E1}";
}
}

static const char *
output_244 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5435 "../../gcc/config/i386/i386.md"
{
  rtx addr = operands[1];

  if (SImode_address_operand (addr, VOIDmode))
    {
      gcc_assert (TARGET_64BIT);
      return "lea{l}\t{%E1, %k0|%k0, %E1}";
    }
  else 
    return "lea{q}\t{%E1, %0|%0, %E1}";
}
}

static const char *
output_250 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5524 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_LEA:
      return "#";

    case TYPE_INCDEC:
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (operands[2] == const1_rtx)
        return "inc{l}\t%0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{l}\t%0";
	}

    default:
      /* For most processors, ADD is faster than LEA.  This alternative
	 was added to use ADD as much as possible.  */
      if (which_alternative == 2)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}
        
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], SImode))
        return "sub{l}\t{%2, %0|%0, %2}";

      return "add{l}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_251 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5524 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_LEA:
      return "#";

    case TYPE_INCDEC:
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (operands[2] == const1_rtx)
        return "inc{q}\t%0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{q}\t%0";
	}

    default:
      /* For most processors, ADD is faster than LEA.  This alternative
	 was added to use ADD as much as possible.  */
      if (which_alternative == 2)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}
        
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], DImode))
        return "sub{q}\t{%2, %0|%0, %2}";

      return "add{q}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_252 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5583 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_LEA:
      return "#";

    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{l}\t%k0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{l}\t%k0";
	}

    default:
      /* For most processors, ADD is faster than LEA.  This alternative
	 was added to use ADD as much as possible.  */
      if (which_alternative == 1)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}

      if (x86_maybe_negate_const_int (&operands[2], SImode))
        return "sub{l}\t{%2, %k0|%k0, %2}";

      return "add{l}\t{%2, %k0|%k0, %2}";
    }
}
}

static const char *
output_253 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5633 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_LEA:
      return "#";

    case TYPE_INCDEC:
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (operands[2] == const1_rtx)
	return "inc{w}\t%0";
      else
	{
	  gcc_assert (operands[2] == constm1_rtx);
	  return "dec{w}\t%0";
	}

    default:
      /* For most processors, ADD is faster than LEA.  This alternative
	 was added to use ADD as much as possible.  */
      if (which_alternative == 2)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}

      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], HImode))
	return "sub{w}\t{%2, %0|%0, %2}";

      return "add{w}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_254 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5686 "../../gcc/config/i386/i386.md"
{
  bool widen = (which_alternative == 3 || which_alternative == 4);

  switch (get_attr_type (insn))
    {
    case TYPE_LEA:
      return "#";

    case TYPE_INCDEC:
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (operands[2] == const1_rtx)
	return widen ? "inc{l}\t%k0" : "inc{b}\t%0";
      else
	{
	  gcc_assert (operands[2] == constm1_rtx);
	  return widen ? "dec{l}\t%k0" : "dec{b}\t%0";
	}

    default:
      /* For most processors, ADD is faster than LEA.  These alternatives
	 were added to use ADD as much as possible.  */
      if (which_alternative == 2 || which_alternative == 4)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}

      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], QImode))
	{
	  if (widen)
	    return "sub{l}\t{%2, %k0|%k0, %2}";
	  else
	    return "sub{b}\t{%2, %0|%0, %2}";
	}
      if (widen)
        return "add{l}\t{%k2, %k0|%k0, %k2}";
      else
        return "add{b}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_255 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5748 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[1] == const1_rtx)
	return "inc{b}\t%0";
      else
	{
	  gcc_assert (operands[1] == constm1_rtx);
	  return "dec{b}\t%0";
	}

    default:
      if (x86_maybe_negate_const_int (&operands[1], QImode))
	return "sub{b}\t{%1, %0|%0, %1}";

      return "add{b}\t{%1, %0|%0, %1}";
    }
}
}

static const char *
output_256 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5836 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{b}\t%0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{b}\t%0";
	}

    default:
      if (which_alternative == 2)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}
        
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], QImode))
        return "sub{b}\t{%2, %0|%0, %2}";

      return "add{b}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_257 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5836 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{w}\t%0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{w}\t%0";
	}

    default:
      if (which_alternative == 2)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}
        
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], HImode))
        return "sub{w}\t{%2, %0|%0, %2}";

      return "add{w}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_258 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5836 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{l}\t%0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{l}\t%0";
	}

    default:
      if (which_alternative == 2)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}
        
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], SImode))
        return "sub{l}\t{%2, %0|%0, %2}";

      return "add{l}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_259 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5836 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{q}\t%0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{q}\t%0";
	}

    default:
      if (which_alternative == 2)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}
        
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], DImode))
        return "sub{q}\t{%2, %0|%0, %2}";

      return "add{q}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_260 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5884 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{l}\t%k0";
      else
	{
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{l}\t%k0";
	}

    default:
      if (which_alternative == 1)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}

      if (x86_maybe_negate_const_int (&operands[2], SImode))
        return "sub{l}\t{%2, %k0|%k0, %2}";

      return "add{l}\t{%2, %k0|%k0, %2}";
    }
}
}

static const char *
output_261 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5928 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{b}\t%0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{b}\t%0";
	}

    default:
      if (which_alternative == 1)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}

      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], QImode))
        return "sub{b}\t{%2, %0|%0, %2}";

      return "add{b}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_262 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5928 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{w}\t%0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{w}\t%0";
	}

    default:
      if (which_alternative == 1)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}

      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], HImode))
        return "sub{w}\t{%2, %0|%0, %2}";

      return "add{w}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_263 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5928 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{l}\t%0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{l}\t%0";
	}

    default:
      if (which_alternative == 1)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}

      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], SImode))
        return "sub{l}\t{%2, %0|%0, %2}";

      return "add{l}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_264 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5928 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{q}\t%0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{q}\t%0";
	}

    default:
      if (which_alternative == 1)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}

      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], DImode))
        return "sub{q}\t{%2, %0|%0, %2}";

      return "add{q}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_265 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5975 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{l}\t%k0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{l}\t%k0";
	}

    default:
      if (which_alternative == 1)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}

      if (x86_maybe_negate_const_int (&operands[2], SImode))
        return "sub{l}\t{%2, %k0|%k0, %2}";

      return "add{l}\t{%2, %k0|%k0, %2}";
    }
}
}

static const char *
output_266 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6026 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == constm1_rtx)
        return "inc{q}\t%0";
      else
        {
	  gcc_assert (operands[2] == const1_rtx);
          return "dec{q}\t%0";
	}

    default:
      if (x86_maybe_negate_const_int (&operands[2], DImode))
	return "add{q}\t{%2, %0|%0, %2}";

      return "sub{q}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_267 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6070 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == constm1_rtx)
        return "inc{b}\t%0";
      else
        {
	  gcc_assert (operands[2] == const1_rtx);
          return "dec{b}\t%0";
	}

    default:
      if (x86_maybe_negate_const_int (&operands[2], QImode))
	return "add{b}\t{%2, %0|%0, %2}";

      return "sub{b}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_268 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6070 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == constm1_rtx)
        return "inc{w}\t%0";
      else
        {
	  gcc_assert (operands[2] == const1_rtx);
          return "dec{w}\t%0";
	}

    default:
      if (x86_maybe_negate_const_int (&operands[2], HImode))
	return "add{w}\t{%2, %0|%0, %2}";

      return "sub{w}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_269 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6070 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == constm1_rtx)
        return "inc{l}\t%0";
      else
        {
	  gcc_assert (operands[2] == const1_rtx);
          return "dec{l}\t%0";
	}

    default:
      if (x86_maybe_negate_const_int (&operands[2], SImode))
	return "add{l}\t{%2, %0|%0, %2}";

      return "sub{l}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_270 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6110 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{b}\t%0";
      else
        {
          gcc_assert (operands[2] == constm1_rtx);
          return "dec{b}\t%0";
	}

    default:
      if (which_alternative == 1)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}

      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], QImode))
        return "sub{b}\t{%2, %0|%0, %2}";

      return "add{b}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_271 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6110 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{w}\t%0";
      else
        {
          gcc_assert (operands[2] == constm1_rtx);
          return "dec{w}\t%0";
	}

    default:
      if (which_alternative == 1)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}

      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], HImode))
        return "sub{w}\t{%2, %0|%0, %2}";

      return "add{w}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_272 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6110 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{l}\t%0";
      else
        {
          gcc_assert (operands[2] == constm1_rtx);
          return "dec{l}\t%0";
	}

    default:
      if (which_alternative == 1)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}

      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], SImode))
        return "sub{l}\t{%2, %0|%0, %2}";

      return "add{l}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_273 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6110 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
        return "inc{q}\t%0";
      else
        {
          gcc_assert (operands[2] == constm1_rtx);
          return "dec{q}\t%0";
	}

    default:
      if (which_alternative == 1)
	{
	  rtx tmp;
	  tmp = operands[1], operands[1] = operands[2], operands[2] = tmp;
	}

      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], DImode))
        return "sub{q}\t{%2, %0|%0, %2}";

      return "add{q}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_274 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6159 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
	return "inc{b}\t%h0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{b}\t%h0";
        }

    default:
      return "add{b}\t{%2, %h0|%h0, %2}";
    }
}
}

static const char *
output_275 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6194 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_INCDEC:
      if (operands[2] == const1_rtx)
	return "inc{b}\t%h0";
      else
        {
	  gcc_assert (operands[2] == constm1_rtx);
          return "dec{b}\t%h0";
	}

    default:
      return "add{b}\t{%2, %h0|%h0, %2}";
    }
}
}

static const char * const output_319[] = {
  "imul{l}\t{%2, %1, %0|%0, %1, %2}",
  "imul{l}\t{%2, %1, %0|%0, %1, %2}",
  "imul{l}\t{%2, %0|%0, %2}",
};

static const char * const output_320[] = {
  "imul{q}\t{%2, %1, %0|%0, %1, %2}",
  "imul{q}\t{%2, %1, %0|%0, %1, %2}",
  "imul{q}\t{%2, %0|%0, %2}",
};

static const char * const output_321[] = {
  "imul{l}\t{%2, %1, %k0|%k0, %1, %2}",
  "imul{l}\t{%2, %1, %k0|%k0, %1, %2}",
  "imul{l}\t{%2, %k0|%k0, %2}",
};

static const char * const output_322[] = {
  "imul{w}\t{%2, %1, %0|%0, %1, %2}",
  "imul{w}\t{%2, %1, %0|%0, %1, %2}",
  "imul{w}\t{%2, %0|%0, %2}",
};

static const char * const output_326[] = {
  "mul{l}\t%2",
  "#",
};

static const char * const output_327[] = {
  "mul{q}\t%2",
  "#",
};

static const char * const output_356[] = {
  "test{l}\t{%k1, %k0|%k0, %k1}",
  "test{l}\t{%k1, %k0|%k0, %k1}",
  "test{q}\t{%1, %0|%0, %1}",
  "test{q}\t{%1, %0|%0, %1}",
  "test{q}\t{%1, %0|%0, %1}",
};

static const char *
output_357 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 7441 "../../gcc/config/i386/i386.md"
{
  if (which_alternative == 3)
    {
      if (CONST_INT_P (operands[1]) && INTVAL (operands[1]) < 0)
	operands[1] = GEN_INT (INTVAL (operands[1]) & 0xff);
      return "test{l}\t{%1, %k0|%k0, %1}";
    }
  return "test{b}\t{%1, %0|%0, %1}";
}
}

static const char *
output_367 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 7719 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_IMOVX:
      {
	enum machine_mode mode;

	gcc_assert (CONST_INT_P (operands[2]));
	if (INTVAL (operands[2]) == (HOST_WIDE_INT) 0xffffffff)
	  mode = SImode;
	else if (INTVAL (operands[2]) == 0xffff)
	  mode = HImode;
	else
	  {
	    gcc_assert (INTVAL (operands[2]) == 0xff);
	    mode = QImode;
	  }

	operands[1] = gen_lowpart (mode, operands[1]);
	if (mode == SImode)
	  return "mov{l}\t{%1, %k0|%k0, %1}";
	else if (mode == HImode)
	  return "movz{wl|x}\t{%1, %k0|%k0, %1}";
	else
	  return "movz{bl|x}\t{%1, %k0|%k0, %1}";
      }

    default:
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (get_attr_mode (insn) == MODE_SI)
	return "and{l}\t{%k2, %k0|%k0, %k2}";
      else
	return "and{q}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_368 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 7771 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_IMOVX:
      {
	enum machine_mode mode;

	gcc_assert (CONST_INT_P (operands[2]));
        if (INTVAL (operands[2]) == 0xffff)
	  mode = HImode;
	else
	  {
	    gcc_assert (INTVAL (operands[2]) == 0xff);
	    mode = QImode;
	  }

	operands[1] = gen_lowpart (mode, operands[1]);
	if (mode == HImode)
	  return "movz{wl|x}\t{%1, %0|%0, %1}";
	else
	  return "movz{bl|x}\t{%1, %0|%0, %1}";
      }

    default:
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      return "and{l}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_370 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 7828 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_IMOVX:
      gcc_assert (CONST_INT_P (operands[2]));
      gcc_assert (INTVAL (operands[2]) == 0xff);
      return "movz{bl|x}\t{%b1, %k0|%k0, %b1}";

    default:
      gcc_assert (rtx_equal_p (operands[0], operands[1]));

      return "and{w}\t{%2, %0|%0, %2}";
    }
}
}

static const char * const output_371[] = {
  "and{b}\t{%2, %0|%0, %2}",
  "and{b}\t{%2, %0|%0, %2}",
  "and{l}\t{%k2, %k0|%k0, %k2}",
};

static const char * const output_373[] = {
  "and{l}\t{%k2, %k0|%k0, %k2}",
  "and{q}\t{%2, %0|%0, %2}",
  "and{q}\t{%2, %0|%0, %2}",
};

static const char *
output_374 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 7958 "../../gcc/config/i386/i386.md"
{
  if (which_alternative == 2)
    {
      if (CONST_INT_P (operands[2]) && INTVAL (operands[2]) < 0)
        operands[2] = GEN_INT (INTVAL (operands[2]) & 0xff);
      return "and{l}\t{%2, %k0|%k0, %2}";
    }
  return "and{b}\t{%2, %0|%0, %2}";
}
}

static const char * const output_391[] = {
  "or{b}\t{%2, %0|%0, %2}",
  "or{b}\t{%2, %0|%0, %2}",
  "or{l}\t{%k2, %k0|%k0, %k2}",
};

static const char * const output_392[] = {
  "xor{b}\t{%2, %0|%0, %2}",
  "xor{b}\t{%2, %0|%0, %2}",
  "xor{l}\t{%k2, %k0|%k0, %k2}",
};

static const char * const output_472[] = {
  "not{b}\t%0",
  "not{l}\t%k0",
};

static const char *
output_483 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9102 "../../gcc/config/i386/i386.md"
{
  return "sal{l}\t{%b2, %0|%0, %b2}";
}
}

static const char *
output_484 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9102 "../../gcc/config/i386/i386.md"
{
  return "sal{q}\t{%b2, %0|%0, %b2}";
}
}

static const char *
output_487 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9123 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_LEA:
    case TYPE_ISHIFTX:
      return "#";

    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      return "add{l}\t%0, %0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{l}\t%0";
      else
	return "sal{l}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_488 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9123 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_LEA:
    case TYPE_ISHIFTX:
      return "#";

    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      return "add{q}\t%0, %0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{q}\t%0";
      else
	return "sal{q}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_490 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9194 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_LEA:
    case TYPE_ISHIFTX:
      return "#";

    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      return "add{l}\t%k0, %k0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{l}\t%k0";
      else
	return "sal{l}\t{%2, %k0|%k0, %2}";
    }
}
}

static const char *
output_491 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9253 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_LEA:
      return "#";

    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      return "add{w}\t%0, %0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{w}\t%0";
      else
	return "sal{w}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_492 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9298 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_LEA:
      return "#";

    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      if (REG_P (operands[1]) && !ANY_QI_REG_P (operands[1]))
        return "add{l}\t%k0, %k0";
      else
        return "add{b}\t%0, %0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	{
	  if (get_attr_mode (insn) == MODE_SI)
	    return "sal{l}\t%k0";
	  else
	    return "sal{b}\t%0";
	}
      else
	{
	  if (get_attr_mode (insn) == MODE_SI)
	    return "sal{l}\t{%2, %k0|%k0, %2}";
	  else
	    return "sal{b}\t{%2, %0|%0, %2}";
	}
    }
}
}

static const char *
output_493 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9359 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ALU:
      gcc_assert (operands[1] == const1_rtx);
      return "add{b}\t%0, %0";

    default:
      if (operands[1] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{b}\t%0";
      else
	return "sal{b}\t{%1, %0|%0, %1}";
    }
}
}

static const char *
output_494 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9455 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      return "add{b}\t%0, %0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{b}\t%0";
      else
	return "sal{b}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_495 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9455 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      return "add{w}\t%0, %0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{w}\t%0";
      else
	return "sal{w}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_496 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9455 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      return "add{l}\t%0, %0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{l}\t%0";
      else
	return "sal{l}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_497 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9455 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      return "add{q}\t%0, %0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{q}\t%0";
      else
	return "sal{q}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_498 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9504 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      return "add{l}\t%k0, %k0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{l}\t%k0";
      else
	return "sal{l}\t{%2, %k0|%k0, %2}";
    }
}
}

static const char *
output_499 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9549 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      return "add{b}\t%0, %0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{b}\t%0";
      else
	return "sal{b}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_500 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9549 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      return "add{w}\t%0, %0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{w}\t%0";
      else
	return "sal{w}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_501 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9549 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      return "add{l}\t%0, %0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{l}\t%0";
      else
	return "sal{l}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_502 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9549 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ALU:
      gcc_assert (operands[2] == const1_rtx);
      return "add{q}\t%0, %0";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sal{q}\t%0";
      else
	return "sal{q}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_503 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9604 "../../gcc/config/i386/i386.md"
{
  return "shr{l}\t{%b2, %0|%0, %b2}";
}
}

static const char *
output_504 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9604 "../../gcc/config/i386/i386.md"
{
  return "sar{l}\t{%b2, %0|%0, %b2}";
}
}

static const char *
output_505 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9604 "../../gcc/config/i386/i386.md"
{
  return "shr{q}\t{%b2, %0|%0, %b2}";
}
}

static const char *
output_506 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9604 "../../gcc/config/i386/i386.md"
{
  return "sar{q}\t{%b2, %0|%0, %b2}";
}
}

static const char * const output_513[] = {
  "{cqto|cqo}",
  "sar{q}\t{%2, %0|%0, %2}",
};

static const char * const output_514[] = {
  "{cltd|cdq}",
  "sar{l}\t{%2, %0|%0, %2}",
};

static const char * const output_515[] = {
  "{cltd|cdq}",
  "sar{l}\t{%2, %k0|%k0, %2}",
};

static const char *
output_520 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9768 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ISHIFTX:
      return "#";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "shr{l}\t%0";
      else
	return "shr{l}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_521 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9768 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ISHIFTX:
      return "#";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sar{l}\t%0";
      else
	return "sar{l}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_522 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9768 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ISHIFTX:
      return "#";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "shr{q}\t%0";
      else
	return "shr{q}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_523 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9768 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ISHIFTX:
      return "#";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sar{q}\t%0";
      else
	return "sar{q}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_526 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9821 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ISHIFTX:
      return "#";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "shr{l}\t%k0";
      else
	return "shr{l}\t{%2, %k0|%k0, %2}";
    }
}
}

static const char *
output_527 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9821 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ISHIFTX:
      return "#";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "sar{l}\t%k0";
      else
	return "sar{l}\t{%2, %k0|%k0, %2}";
    }
}
}

static const char *
output_528 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9865 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "shr{b}\t%0";
  else
    return "shr{b}\t{%2, %0|%0, %2}";
}
}

static const char *
output_529 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9865 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "sar{b}\t%0";
  else
    return "sar{b}\t{%2, %0|%0, %2}";
}
}

static const char *
output_530 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9865 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "shr{w}\t%0";
  else
    return "shr{w}\t{%2, %0|%0, %2}";
}
}

static const char *
output_531 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9865 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "sar{w}\t%0";
  else
    return "sar{w}\t{%2, %0|%0, %2}";
}
}

static const char *
output_532 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9891 "../../gcc/config/i386/i386.md"
{
  if (operands[1] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "shr{b}\t%0";
  else
    return "shr{b}\t{%1, %0|%0, %1}";
}
}

static const char *
output_533 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9891 "../../gcc/config/i386/i386.md"
{
  if (operands[1] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "sar{b}\t%0";
  else
    return "sar{b}\t{%1, %0|%0, %1}";
}
}

static const char *
output_534 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9926 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "shr{b}\t%0";
  else
    return "shr{b}\t{%2, %0|%0, %2}";
}
}

static const char *
output_535 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9926 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "sar{b}\t%0";
  else
    return "sar{b}\t{%2, %0|%0, %2}";
}
}

static const char *
output_536 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9926 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "shr{w}\t%0";
  else
    return "shr{w}\t{%2, %0|%0, %2}";
}
}

static const char *
output_537 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9926 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "sar{w}\t%0";
  else
    return "sar{w}\t{%2, %0|%0, %2}";
}
}

static const char *
output_538 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9926 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "shr{l}\t%0";
  else
    return "shr{l}\t{%2, %0|%0, %2}";
}
}

static const char *
output_539 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9926 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "sar{l}\t%0";
  else
    return "sar{l}\t{%2, %0|%0, %2}";
}
}

static const char *
output_540 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9926 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "shr{q}\t%0";
  else
    return "shr{q}\t{%2, %0|%0, %2}";
}
}

static const char *
output_541 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9926 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "sar{q}\t%0";
  else
    return "sar{q}\t{%2, %0|%0, %2}";
}
}

static const char *
output_542 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9958 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "shr{l}\t%k0";
  else
    return "shr{l}\t{%2, %k0|%k0, %2}";
}
}

static const char *
output_543 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9958 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "sar{l}\t%k0";
  else
    return "sar{l}\t{%2, %k0|%k0, %2}";
}
}

static const char *
output_544 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9988 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "shr{b}\t%0";
  else
    return "shr{b}\t{%2, %0|%0, %2}";
}
}

static const char *
output_545 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9988 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "sar{b}\t%0";
  else
    return "sar{b}\t{%2, %0|%0, %2}";
}
}

static const char *
output_546 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9988 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "shr{w}\t%0";
  else
    return "shr{w}\t{%2, %0|%0, %2}";
}
}

static const char *
output_547 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9988 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "sar{w}\t%0";
  else
    return "sar{w}\t{%2, %0|%0, %2}";
}
}

static const char *
output_548 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9988 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "shr{l}\t%0";
  else
    return "shr{l}\t{%2, %0|%0, %2}";
}
}

static const char *
output_549 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9988 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "sar{l}\t%0";
  else
    return "sar{l}\t{%2, %0|%0, %2}";
}
}

static const char *
output_550 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9988 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "shr{q}\t%0";
  else
    return "shr{q}\t{%2, %0|%0, %2}";
}
}

static const char *
output_551 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9988 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "sar{q}\t%0";
  else
    return "sar{q}\t{%2, %0|%0, %2}";
}
}

static const char *
output_552 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10059 "../../gcc/config/i386/i386.md"
{
  return "rol{l}\t{%b2, %0|%0, %b2}";
}
}

static const char *
output_553 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10059 "../../gcc/config/i386/i386.md"
{
  return "ror{l}\t{%b2, %0|%0, %b2}";
}
}

static const char *
output_554 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10059 "../../gcc/config/i386/i386.md"
{
  return "rol{q}\t{%b2, %0|%0, %b2}";
}
}

static const char *
output_555 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10059 "../../gcc/config/i386/i386.md"
{
  return "ror{q}\t{%b2, %0|%0, %b2}";
}
}

static const char *
output_562 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10140 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ROTATEX:
      return "#";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "rol{l}\t%0";
      else
	return "rol{l}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_563 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10140 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ROTATEX:
      return "#";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "ror{l}\t%0";
      else
	return "ror{l}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_564 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10140 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ROTATEX:
      return "#";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "rol{q}\t%0";
      else
	return "rol{q}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_565 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10140 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ROTATEX:
      return "#";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "ror{q}\t%0";
      else
	return "ror{q}\t{%2, %0|%0, %2}";
    }
}
}

static const char *
output_567 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10206 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ROTATEX:
      return "#";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "rol{l}\t%k0";
      else
	return "rol{l}\t{%2, %k0|%k0, %2}";
    }
}
}

static const char *
output_568 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10206 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_ROTATEX:
      return "#";

    default:
      if (operands[2] == const1_rtx
	  && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
	return "ror{l}\t%k0";
      else
	return "ror{l}\t{%2, %k0|%k0, %2}";
    }
}
}

static const char *
output_569 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10263 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "rol{b}\t%0";
  else
    return "rol{b}\t{%2, %0|%0, %2}";
}
}

static const char *
output_570 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10263 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "ror{b}\t%0";
  else
    return "ror{b}\t{%2, %0|%0, %2}";
}
}

static const char *
output_571 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10263 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "rol{w}\t%0";
  else
    return "rol{w}\t{%2, %0|%0, %2}";
}
}

static const char *
output_572 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10263 "../../gcc/config/i386/i386.md"
{
  if (operands[2] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "ror{w}\t%0";
  else
    return "ror{w}\t{%2, %0|%0, %2}";
}
}

static const char *
output_573 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10289 "../../gcc/config/i386/i386.md"
{
  if (operands[1] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "rol{b}\t%0";
  else
    return "rol{b}\t{%1, %0|%0, %1}";
}
}

static const char *
output_574 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 10289 "../../gcc/config/i386/i386.md"
{
  if (operands[1] == const1_rtx
      && (TARGET_SHIFT1 || optimize_function_for_size_p (cfun)))
    return "ror{b}\t%0";
  else
    return "ror{b}\t{%1, %0|%0, %1}";
}
}

static const char * const output_585[] = {
  "cmp%D3ss\t{%2, %0|%0, %2}",
  "vcmp%D3ss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_586[] = {
  "cmp%D3sd\t{%2, %0|%0, %2}",
  "vcmp%D3sd\t{%2, %1, %0|%0, %1, %2}",
};

static const char *
output_609 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11286 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[0]);
}

static const char *
output_610 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11286 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[0]);
}

static const char *
output_612 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11331 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[0]);
}

static const char *
output_615 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11350 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[0]);
}

static const char *
output_616 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11350 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[0]);
}

static const char *
output_618 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11388 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[0]);
}

static const char *
output_620 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11413 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[0]);
}

static const char *
output_623 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11460 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[1]);
}

static const char *
output_624 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11460 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[1]);
}

static const char *
output_627 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11481 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[1]);
}

static const char *
output_628 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11481 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[1]);
}

static const char *
output_630 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11528 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[1]);
}

static const char *
output_632 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11569 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[1]);
}

static const char *
output_634 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11596 "../../gcc/config/i386/i386.md"
 return ix86_output_call_insn (insn, operands[1]);
}

static const char *
output_643 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11769 "../../gcc/config/i386/i386.md"
{
  int num = INTVAL (operands[0]);

  gcc_assert (num >= 1 && num <= 8);

  while (num--)
    fputs ("\tnop\n", asm_out_file);

  return "";
}
}

static const char *
output_644 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11790 "../../gcc/config/i386/i386.md"
{
#ifdef ASM_OUTPUT_MAX_SKIP_PAD
  ASM_OUTPUT_MAX_SKIP_PAD (asm_out_file, 4, (int)INTVAL (operands[0]));
#else
  /* It is tempting to use ASM_OUTPUT_ALIGN here, but we don't want to do that.
     The align insn is used to avoid 3 jump instructions in the row to improve
     branch prediction and the benefits hardly outweigh the cost of extra 8
     nops on the average inserted by full alignment pseudo operation.  */
#endif
  return "";
}
}

static const char *
output_645 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11813 "../../gcc/config/i386/i386.md"
 return output_set_got (operands[0], NULL_RTX);
}

static const char *
output_646 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11823 "../../gcc/config/i386/i386.md"
 return output_set_got (operands[0], operands[1]);
}

static const char *
output_653 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11926 "../../gcc/config/i386/i386.md"
{
  if (operands[0] == const0_rtx)
    return "ret";
  else
    return "ret\t%0";
}
}

static const char *
output_657 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12035 "../../gcc/config/i386/i386.md"
{
  if (TARGET_BMI)
    return "tzcnt{w}\t{%1, %0|%0, %1}";
  else
    return "bsf{w}\t{%1, %0|%0, %1}";
}
}

static const char *
output_658 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12035 "../../gcc/config/i386/i386.md"
{
  if (TARGET_BMI)
    return "tzcnt{l}\t{%1, %0|%0, %1}";
  else
    return "bsf{l}\t{%1, %0|%0, %1}";
}
}

static const char *
output_659 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12035 "../../gcc/config/i386/i386.md"
{
  if (TARGET_BMI)
    return "tzcnt{q}\t{%1, %0|%0, %1}";
  else
    return "bsf{q}\t{%1, %0|%0, %1}";
}
}

static const char *
output_679 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12182 "../../gcc/config/i386/i386.md"
{
  operands[2] = GEN_INT (INTVAL (operands[2]) << 8 | INTVAL (operands[3]));
  return "bextr\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char *
output_680 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12182 "../../gcc/config/i386/i386.md"
{
  operands[2] = GEN_INT (INTVAL (operands[2]) << 8 | INTVAL (operands[3]));
  return "bextr\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char *
output_702 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12350 "../../gcc/config/i386/i386.md"
{
#if TARGET_MACHO
  return "popcnt\t{%1, %0|%0, %1}";
#else
  return "popcnt{w}\t{%1, %0|%0, %1}";
#endif
}
}

static const char *
output_703 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12350 "../../gcc/config/i386/i386.md"
{
#if TARGET_MACHO
  return "popcnt\t{%1, %0|%0, %1}";
#else
  return "popcnt{l}\t{%1, %0|%0, %1}";
#endif
}
}

static const char *
output_704 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12350 "../../gcc/config/i386/i386.md"
{
#if TARGET_MACHO
  return "popcnt\t{%1, %0|%0, %1}";
#else
  return "popcnt{q}\t{%1, %0|%0, %1}";
#endif
}
}

static const char *
output_705 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12370 "../../gcc/config/i386/i386.md"
{
#if TARGET_MACHO
  return "popcnt\t{%1, %0|%0, %1}";
#else
  return "popcnt{w}\t{%1, %0|%0, %1}";
#endif
}
}

static const char *
output_706 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12370 "../../gcc/config/i386/i386.md"
{
#if TARGET_MACHO
  return "popcnt\t{%1, %0|%0, %1}";
#else
  return "popcnt{l}\t{%1, %0|%0, %1}";
#endif
}
}

static const char *
output_707 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12370 "../../gcc/config/i386/i386.md"
{
#if TARGET_MACHO
  return "popcnt\t{%1, %0|%0, %1}";
#else
  return "popcnt{q}\t{%1, %0|%0, %1}";
#endif
}
}

static const char *
output_708 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12389 "../../gcc/config/i386/i386.md"
{
#if TARGET_MACHO
  return "popcnt\t{%1, %0|%0, %1}";
#else
  return "popcnt{l}\t{%1, %0|%0, %1}";
#endif
}
}

static const char * const output_709[] = {
  "bswap\t%0",
  "movbe\t{%1, %0|%0, %1}",
  "movbe\t{%1, %0|%0, %1}",
};

static const char * const output_710[] = {
  "bswap\t%0",
  "movbe\t{%1, %0|%0, %1}",
  "movbe\t{%1, %0|%0, %1}",
};

static const char * const output_713[] = {
  "xchg{b}\t{%h0, %b0|%b0, %h0}",
  "rol{w}\t{$8, %0|%0, 8}",
};

static const char *
output_718 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12590 "../../gcc/config/i386/i386.md"
{
  output_asm_insn
    ("lea{l}\t{%E2@tlsgd(,%1,1), %0|%0, %E2@tlsgd[%1*1]}", operands);
  if (TARGET_SUN_TLS)
#ifdef HAVE_AS_IX86_TLSGDPLT
    return "call\t%a2@tlsgdplt";
#else
    return "call\t%p3@plt";
#endif
  return "call\t%P3";
}
}

static const char *
output_719 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12623 "../../gcc/config/i386/i386.md"
{
  if (!TARGET_X32)
    fputs (ASM_BYTE "0x66\n", asm_out_file);
  output_asm_insn
    ("lea{q}\t{%E1@tlsgd(%%rip), %%rdi|rdi, %E1@tlsgd[rip]}", operands);
  fputs (ASM_SHORT "0x6666\n", asm_out_file);
  fputs ("\trex64\n", asm_out_file);
  if (TARGET_SUN_TLS)
    return "call\t%p2@plt";
  return "call\t%P2";
}
}

static const char *
output_720 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12657 "../../gcc/config/i386/i386.md"
{
  output_asm_insn
    ("lea{l}\t{%&@tlsldm(%1), %0|%0, %&@tlsldm[%1]}", operands);
  if (TARGET_SUN_TLS)
#ifdef HAVE_AS_IX86_TLSLDMPLT
    return "call\t%&@tlsldmplt";
#else
    return "call\t%p2@plt";
#endif
  return "call\t%P2";
}
}

static const char *
output_721 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12689 "../../gcc/config/i386/i386.md"
{
  output_asm_insn
    ("lea{q}\t{%&@tlsld(%%rip), %%rdi|rdi, %&@tlsld[rip]}", operands);
  if (TARGET_SUN_TLS)
    return "call\t%p1@plt";
  return "call\t%P1";
}
}

static const char *
output_737 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12973 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_738 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12973 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_739 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12994 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_740 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12994 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_741 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13011 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_742 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13011 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_743 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13026 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_744 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13026 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_746 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13064 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_747 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13064 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_748 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13086 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_749 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13086 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_750 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13106 "../../gcc/config/i386/i386.md"
 return which_alternative ? "#" : output_387_binary_op (insn, operands);
}

static const char *
output_751 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13106 "../../gcc/config/i386/i386.md"
 return which_alternative ? "#" : output_387_binary_op (insn, operands);
}

static const char *
output_752 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13106 "../../gcc/config/i386/i386.md"
 return which_alternative ? "#" : output_387_binary_op (insn, operands);
}

static const char *
output_753 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13106 "../../gcc/config/i386/i386.md"
 return which_alternative ? "#" : output_387_binary_op (insn, operands);
}

static const char *
output_754 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13126 "../../gcc/config/i386/i386.md"
 return which_alternative ? "#" : output_387_binary_op (insn, operands);
}

static const char *
output_755 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13126 "../../gcc/config/i386/i386.md"
 return which_alternative ? "#" : output_387_binary_op (insn, operands);
}

static const char *
output_756 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13126 "../../gcc/config/i386/i386.md"
 return which_alternative ? "#" : output_387_binary_op (insn, operands);
}

static const char *
output_757 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13126 "../../gcc/config/i386/i386.md"
 return which_alternative ? "#" : output_387_binary_op (insn, operands);
}

static const char *
output_758 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13146 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_759 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13164 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_760 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13183 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_761 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13200 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_762 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13214 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_763 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13231 "../../gcc/config/i386/i386.md"
 return which_alternative ? "#" : output_387_binary_op (insn, operands);
}

static const char *
output_764 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13231 "../../gcc/config/i386/i386.md"
 return which_alternative ? "#" : output_387_binary_op (insn, operands);
}

static const char *
output_765 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13249 "../../gcc/config/i386/i386.md"
 return which_alternative ? "#" : output_387_binary_op (insn, operands);
}

static const char *
output_766 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13249 "../../gcc/config/i386/i386.md"
 return which_alternative ? "#" : output_387_binary_op (insn, operands);
}

static const char *
output_767 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13267 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_768 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13267 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_769 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13284 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_770 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13284 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_771 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13302 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_772 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13302 "../../gcc/config/i386/i386.md"
 return output_387_binary_op (insn, operands);
}

static const char *
output_773 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13364 "../../gcc/config/i386/i386.md"
 return output_387_reg_move (insn, operands);
}

static const char *
output_774 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 13364 "../../gcc/config/i386/i386.md"
 return output_387_reg_move (insn, operands);
}

static const char *
output_813 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 14707 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, false);
}

static const char *
output_817 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 14766 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, false);
}

static const char *
output_818 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 14766 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, false);
}

static const char *
output_826 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 14967 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, false);
}

static const char *
output_828 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 15026 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, false);
}

static const char *
output_829 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 15026 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, false);
}

static const char *
output_837 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 15225 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, false);
}

static const char *
output_839 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 15284 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, false);
}

static const char *
output_840 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 15284 "../../gcc/config/i386/i386.md"
 return output_fix_trunc (insn, operands, false);
}

static const char * const output_890[] = {
  "cmov%O2%C1\t{%2, %0|%0, %2}",
  "cmov%O2%c1\t{%3, %0|%0, %3}",
};

static const char * const output_891[] = {
  "cmov%O2%C1\t{%2, %0|%0, %2}",
  "cmov%O2%c1\t{%3, %0|%0, %3}",
};

static const char * const output_892[] = {
  "cmov%O2%C1\t{%2, %0|%0, %2}",
  "cmov%O2%c1\t{%3, %0|%0, %3}",
};

static const char * const output_894[] = {
  "fcmov%F1\t{%2, %0|%0, %2}",
  "fcmov%f1\t{%3, %0|%0, %3}",
};

static const char * const output_895[] = {
  "fcmov%F1\t{%2, %0|%0, %2}",
  "fcmov%f1\t{%3, %0|%0, %3}",
  "cmov%O2%C1\t{%2, %0|%0, %2}",
  "cmov%O2%c1\t{%3, %0|%0, %3}",
};

static const char * const output_896[] = {
  "fcmov%F1\t{%2, %0|%0, %2}",
  "fcmov%f1\t{%3, %0|%0, %3}",
  "#",
  "#",
};

static const char * const output_897[] = {
  "fcmov%F1\t{%2, %0|%0, %2}",
  "fcmov%f1\t{%3, %0|%0, %3}",
  "cmov%O2%C1\t{%2, %0|%0, %2}",
  "cmov%O2%c1\t{%3, %0|%0, %3}",
};

static const char * const output_900[] = {
  "maxss\t{%2, %0|%0, %2}",
  "vmaxss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_901[] = {
  "minss\t{%2, %0|%0, %2}",
  "vminss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_902[] = {
  "maxsd\t{%2, %0|%0, %2}",
  "vmaxsd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_903[] = {
  "minsd\t{%2, %0|%0, %2}",
  "vminsd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_904[] = {
  "minss\t{%2, %0|%0, %2}",
  "vminss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_905[] = {
  "minsd\t{%2, %0|%0, %2}",
  "vminsd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_906[] = {
  "maxss\t{%2, %0|%0, %2}",
  "vmaxss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_907[] = {
  "maxsd\t{%2, %0|%0, %2}",
  "vmaxsd\t{%2, %1, %0|%0, %1, %2}",
};

static const char *
output_908 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 16657 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_IMOV:
      return "mov{l}\t{%1, %0|%0, %1}";

    case TYPE_ALU:
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], SImode))
	return "sub{l}\t{%2, %0|%0, %2}";

      return "add{l}\t{%2, %0|%0, %2}";

    default:
      operands[2] = SET_SRC (XVECEXP (PATTERN (insn), 0, 0));
      return "lea{l}\t{%E2, %0|%0, %E2}";
    }
}
}

static const char *
output_909 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 16657 "../../gcc/config/i386/i386.md"
{
  switch (get_attr_type (insn))
    {
    case TYPE_IMOV:
      return "mov{q}\t{%1, %0|%0, %1}";

    case TYPE_ALU:
      gcc_assert (rtx_equal_p (operands[0], operands[1]));
      if (x86_maybe_negate_const_int (&operands[2], DImode))
	return "sub{q}\t{%2, %0|%0, %2}";

      return "add{q}\t{%2, %0|%0, %2}";

    default:
      operands[2] = SET_SRC (XVECEXP (PATTERN (insn), 0, 0));
      return "lea{q}\t{%E2, %0|%0, %E2}";
    }
}
}

static const char *
output_914 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 16773 "../../gcc/config/i386/i386.md"
 return output_adjust_stack_and_probe (operands[0]);
}

static const char *
output_915 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 16773 "../../gcc/config/i386/i386.md"
 return output_adjust_stack_and_probe (operands[0]);
}

static const char *
output_916 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 16783 "../../gcc/config/i386/i386.md"
 return output_probe_stack_range (operands[0], operands[2]);
}

static const char *
output_917 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 16783 "../../gcc/config/i386/i386.md"
 return output_probe_stack_range (operands[0], operands[2]);
}

static const char *
output_918 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 17696 "../../gcc/config/i386/i386.md"
{ return ASM_SHORT "0x0b0f"; }
}

static const char *
output_919 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 17728 "../../gcc/config/i386/i386.md"
{
  static const char * const patterns[4] = {
   "prefetchnta\t%a0", "prefetcht2\t%a0", "prefetcht1\t%a0", "prefetcht0\t%a0"
  };

  int locality = INTVAL (operands[1]);
  gcc_assert (IN_RANGE (locality, 0, 3));

  return patterns[locality];
}
}

static const char *
output_920 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 17749 "../../gcc/config/i386/i386.md"
{
  if (INTVAL (operands[1]) == 0)
    return "prefetch\t%a0";
  else
    return "prefetchw\t%a0";
}
}

static const char * const output_959[] = {
  "mov{q}\t{%1, %0|%0, %1}",
  "mov{q}\t{%1, %0|%0, %1}",
  "pxor\t%0, %0",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movdq2q\t{%1, %0|%0, %1}",
  "movq2dq\t{%1, %0|%0, %1}",
  "%vpxor\t%0, %d0",
  "%vmovq\t{%1, %0|%0, %1}",
  "%vmovq\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
};

static const char * const output_960[] = {
  "mov{q}\t{%1, %0|%0, %1}",
  "mov{q}\t{%1, %0|%0, %1}",
  "pxor\t%0, %0",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movdq2q\t{%1, %0|%0, %1}",
  "movq2dq\t{%1, %0|%0, %1}",
  "%vpxor\t%0, %d0",
  "%vmovq\t{%1, %0|%0, %1}",
  "%vmovq\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
};

static const char * const output_961[] = {
  "mov{q}\t{%1, %0|%0, %1}",
  "mov{q}\t{%1, %0|%0, %1}",
  "pxor\t%0, %0",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movdq2q\t{%1, %0|%0, %1}",
  "movq2dq\t{%1, %0|%0, %1}",
  "%vpxor\t%0, %d0",
  "%vmovq\t{%1, %0|%0, %1}",
  "%vmovq\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
};

static const char * const output_962[] = {
  "mov{q}\t{%1, %0|%0, %1}",
  "mov{q}\t{%1, %0|%0, %1}",
  "pxor\t%0, %0",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movdq2q\t{%1, %0|%0, %1}",
  "movq2dq\t{%1, %0|%0, %1}",
  "%vpxor\t%0, %d0",
  "%vmovq\t{%1, %0|%0, %1}",
  "%vmovq\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
};

static const char * const output_963[] = {
  "pxor\t%0, %0",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movdq2q\t{%1, %0|%0, %1}",
  "movq2dq\t{%1, %0|%0, %1}",
  "%vpxor\t%0, %d0",
  "%vmovq\t{%1, %0|%0, %1}",
  "%vmovq\t{%1, %0|%0, %1}",
  "xorps\t%0, %0",
  "movaps\t{%1, %0|%0, %1}",
  "movlps\t{%1, %0|%0, %1}",
  "movlps\t{%1, %0|%0, %1}",
  "#",
  "#",
};

static const char * const output_964[] = {
  "pxor\t%0, %0",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movdq2q\t{%1, %0|%0, %1}",
  "movq2dq\t{%1, %0|%0, %1}",
  "%vpxor\t%0, %d0",
  "%vmovq\t{%1, %0|%0, %1}",
  "%vmovq\t{%1, %0|%0, %1}",
  "xorps\t%0, %0",
  "movaps\t{%1, %0|%0, %1}",
  "movlps\t{%1, %0|%0, %1}",
  "movlps\t{%1, %0|%0, %1}",
  "#",
  "#",
};

static const char * const output_965[] = {
  "pxor\t%0, %0",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movdq2q\t{%1, %0|%0, %1}",
  "movq2dq\t{%1, %0|%0, %1}",
  "%vpxor\t%0, %d0",
  "%vmovq\t{%1, %0|%0, %1}",
  "%vmovq\t{%1, %0|%0, %1}",
  "xorps\t%0, %0",
  "movaps\t{%1, %0|%0, %1}",
  "movlps\t{%1, %0|%0, %1}",
  "movlps\t{%1, %0|%0, %1}",
  "#",
  "#",
};

static const char * const output_966[] = {
  "pxor\t%0, %0",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movdq2q\t{%1, %0|%0, %1}",
  "movq2dq\t{%1, %0|%0, %1}",
  "%vpxor\t%0, %d0",
  "%vmovq\t{%1, %0|%0, %1}",
  "%vmovq\t{%1, %0|%0, %1}",
  "xorps\t%0, %0",
  "movaps\t{%1, %0|%0, %1}",
  "movlps\t{%1, %0|%0, %1}",
  "movlps\t{%1, %0|%0, %1}",
  "#",
  "#",
};

static const char * const output_967[] = {
  "mov{q}\t{%1, %0|%0, %1}",
  "mov{q}\t{%1, %0|%0, %1}",
  "pxor\t%0, %0",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movdq2q\t{%1, %0|%0, %1}",
  "movq2dq\t{%1, %0|%0, %1}",
  "%vxorps\t%0, %d0",
  "%vmovaps\t{%1, %0|%0, %1}",
  "%vmovlps\t{%1, %d0|%d0, %1}",
  "%vmovlps\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
};

static const char * const output_968[] = {
  "pxor\t%0, %0",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movq\t{%1, %0|%0, %1}",
  "movdq2q\t{%1, %0|%0, %1}",
  "movq2dq\t{%1, %0|%0, %1}",
  "%vxorps\t%0, %d0",
  "%vmovaps\t{%1, %0|%0, %1}",
  "%vmovlps\t{%1, %d0|%d0, %1}",
  "%vmovlps\t{%1, %0|%0, %1}",
  "#",
  "#",
};

static const char * const output_971[] = {
  "pfsub\t{%2, %0|%0, %2}",
  "pfsubr\t{%1, %0|%0, %1}",
};

static const char * const output_994[] = {
  "punpckldq\t{%2, %0|%0, %2}",
  "movd\t{%1, %0|%0, %1}",
};

static const char * const output_996[] = {
  "punpckhdq\t%0, %0",
  "unpckhps\t%0, %0",
  "#",
  "#",
  "#",
  "#",
};

static const char *
output_1058 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1283 "../../gcc/config/i386/mmx.md"
{
  operands[3] = GEN_INT (exact_log2 (INTVAL (operands[3])));
  if (MEM_P (operands[2]))
    return "pinsrw\t{%3, %2, %0|%0, %2, %3}";
  else
    return "pinsrw\t{%3, %k2, %0|%0, %k2, %3}";
}
}

static const char *
output_1060 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1330 "../../gcc/config/i386/mmx.md"
{
  int mask = 0;
  mask |= INTVAL (operands[2]) << 0;
  mask |= INTVAL (operands[3]) << 2;
  mask |= INTVAL (operands[4]) << 4;
  mask |= INTVAL (operands[5]) << 6;
  operands[2] = GEN_INT (mask);

  return "pshufw\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char * const output_1064[] = {
  "punpckldq\t{%2, %0|%0, %2}",
  "movd\t{%1, %0|%0, %1}",
};

static const char * const output_1066[] = {
  "punpckhdq\t%0, %0",
  "punpckhdq\t%0, %0",
  "pshufd\t{$85, %1, %0|%0, %1, 85}",
  "unpckhps\t%0, %0",
  "#",
  "#",
  "#",
};

static const char *
output_1067 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1579 "../../gcc/config/i386/mmx.md"
{
  /* These two instructions have the same operation, but their encoding
     is different.  Prefer the one that is de facto standard.  */
  if (TARGET_SSE || TARGET_3DNOW_A)
    return "pavgb\t{%2, %0|%0, %2}";
  else
    return "pavgusb\t{%2, %0|%0, %2}";
}
}

static const char *
output_1075 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V32QImode)
		  || misaligned_operand (operands[1], V32QImode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V32QImode)
		  || misaligned_operand (operands[1], V32QImode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V32QImode)
		  || misaligned_operand (operands[1], V32QImode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1076 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V16QImode)
		  || misaligned_operand (operands[1], V16QImode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V16QImode)
		  || misaligned_operand (operands[1], V16QImode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V16QImode)
		  || misaligned_operand (operands[1], V16QImode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1077 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V16HImode)
		  || misaligned_operand (operands[1], V16HImode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V16HImode)
		  || misaligned_operand (operands[1], V16HImode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V16HImode)
		  || misaligned_operand (operands[1], V16HImode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1078 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V8HImode)
		  || misaligned_operand (operands[1], V8HImode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V8HImode)
		  || misaligned_operand (operands[1], V8HImode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V8HImode)
		  || misaligned_operand (operands[1], V8HImode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1079 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V8SImode)
		  || misaligned_operand (operands[1], V8SImode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V8SImode)
		  || misaligned_operand (operands[1], V8SImode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V8SImode)
		  || misaligned_operand (operands[1], V8SImode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1080 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V4SImode)
		  || misaligned_operand (operands[1], V4SImode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V4SImode)
		  || misaligned_operand (operands[1], V4SImode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V4SImode)
		  || misaligned_operand (operands[1], V4SImode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1081 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V4DImode)
		  || misaligned_operand (operands[1], V4DImode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V4DImode)
		  || misaligned_operand (operands[1], V4DImode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V4DImode)
		  || misaligned_operand (operands[1], V4DImode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1082 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V2DImode)
		  || misaligned_operand (operands[1], V2DImode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V2DImode)
		  || misaligned_operand (operands[1], V2DImode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V2DImode)
		  || misaligned_operand (operands[1], V2DImode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1083 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V2TImode)
		  || misaligned_operand (operands[1], V2TImode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V2TImode)
		  || misaligned_operand (operands[1], V2TImode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V2TImode)
		  || misaligned_operand (operands[1], V2TImode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1084 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V1TImode)
		  || misaligned_operand (operands[1], V1TImode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V1TImode)
		  || misaligned_operand (operands[1], V1TImode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V1TImode)
		  || misaligned_operand (operands[1], V1TImode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1085 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V8SFmode)
		  || misaligned_operand (operands[1], V8SFmode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V8SFmode)
		  || misaligned_operand (operands[1], V8SFmode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V8SFmode)
		  || misaligned_operand (operands[1], V8SFmode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1086 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V4SFmode)
		  || misaligned_operand (operands[1], V4SFmode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V4SFmode)
		  || misaligned_operand (operands[1], V4SFmode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V4SFmode)
		  || misaligned_operand (operands[1], V4SFmode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1087 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V4DFmode)
		  || misaligned_operand (operands[1], V4DFmode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V4DFmode)
		  || misaligned_operand (operands[1], V4DFmode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V4DFmode)
		  || misaligned_operand (operands[1], V4DFmode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1088 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 429 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return standard_sse_constant_opcode (insn, operands[1]);
    case 1:
    case 2:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	case MODE_V4SF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V2DFmode)
		  || misaligned_operand (operands[1], V2DFmode)))
	    return "vmovups\t{%1, %0|%0, %1}";
	  else
	    return "%vmovaps\t{%1, %0|%0, %1}";

	case MODE_V4DF:
	case MODE_V2DF:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V2DFmode)
		  || misaligned_operand (operands[1], V2DFmode)))
	    return "vmovupd\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovapd\t{%1, %0|%0, %1}";

	case MODE_OI:
	case MODE_TI:
	  if (TARGET_AVX
	      && (misaligned_operand (operands[0], V2DFmode)
		  || misaligned_operand (operands[1], V2DFmode)))
	    return "vmovdqu\t{%1, %0|%0, %1}";
	  else if (TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL)
	    return "%vmovaps\t{%1, %0|%0, %1}";
	  else
	    return "%vmovdqa\t{%1, %0|%0, %1}";

	default:
	  gcc_unreachable ();
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char * const output_1117[] = {
  "addps\t{%2, %0|%0, %2}",
  "vaddps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1118[] = {
  "subps\t{%2, %0|%0, %2}",
  "vsubps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1119[] = {
  "addps\t{%2, %0|%0, %2}",
  "vaddps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1120[] = {
  "subps\t{%2, %0|%0, %2}",
  "vsubps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1121[] = {
  "addpd\t{%2, %0|%0, %2}",
  "vaddpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1122[] = {
  "subpd\t{%2, %0|%0, %2}",
  "vsubpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1123[] = {
  "addpd\t{%2, %0|%0, %2}",
  "vaddpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1124[] = {
  "subpd\t{%2, %0|%0, %2}",
  "vsubpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1125[] = {
  "addss\t{%2, %0|%0, %2}",
  "vaddss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1126[] = {
  "subss\t{%2, %0|%0, %2}",
  "vsubss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1127[] = {
  "addsd\t{%2, %0|%0, %2}",
  "vaddsd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1128[] = {
  "subsd\t{%2, %0|%0, %2}",
  "vsubsd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1129[] = {
  "mulps\t{%2, %0|%0, %2}",
  "vmulps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1130[] = {
  "mulps\t{%2, %0|%0, %2}",
  "vmulps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1131[] = {
  "mulpd\t{%2, %0|%0, %2}",
  "vmulpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1132[] = {
  "mulpd\t{%2, %0|%0, %2}",
  "vmulpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1133[] = {
  "mulss\t{%2, %0|%0, %2}",
  "vmulss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1134[] = {
  "mulsd\t{%2, %0|%0, %2}",
  "vmulsd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1135[] = {
  "divps\t{%2, %0|%0, %2}",
  "vdivps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1136[] = {
  "divps\t{%2, %0|%0, %2}",
  "vdivps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1137[] = {
  "divpd\t{%2, %0|%0, %2}",
  "vdivpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1138[] = {
  "divpd\t{%2, %0|%0, %2}",
  "vdivpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1139[] = {
  "divss\t{%2, %0|%0, %2}",
  "vdivss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1140[] = {
  "divsd\t{%2, %0|%0, %2}",
  "vdivsd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1143[] = {
  "rcpss\t{%1, %0|%0, %1}",
  "vrcpss\t{%1, %2, %0|%0, %2, %1}",
};

static const char * const output_1148[] = {
  "sqrtss\t{%1, %0|%0, %1}",
  "vsqrtss\t{%1, %2, %0|%0, %2, %1}",
};

static const char * const output_1149[] = {
  "sqrtsd\t{%1, %0|%0, %1}",
  "vsqrtsd\t{%1, %2, %0|%0, %2, %1}",
};

static const char * const output_1152[] = {
  "rsqrtss\t{%1, %0|%0, %1}",
  "vrsqrtss\t{%1, %2, %0|%0, %2, %1}",
};

static const char * const output_1153[] = {
  "maxps\t{%2, %0|%0, %2}",
  "vmaxps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1154[] = {
  "minps\t{%2, %0|%0, %2}",
  "vminps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1155[] = {
  "maxps\t{%2, %0|%0, %2}",
  "vmaxps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1156[] = {
  "minps\t{%2, %0|%0, %2}",
  "vminps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1157[] = {
  "maxpd\t{%2, %0|%0, %2}",
  "vmaxpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1158[] = {
  "minpd\t{%2, %0|%0, %2}",
  "vminpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1159[] = {
  "maxpd\t{%2, %0|%0, %2}",
  "vmaxpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1160[] = {
  "minpd\t{%2, %0|%0, %2}",
  "vminpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1161[] = {
  "maxps\t{%2, %0|%0, %2}",
  "vmaxps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1162[] = {
  "minps\t{%2, %0|%0, %2}",
  "vminps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1163[] = {
  "maxps\t{%2, %0|%0, %2}",
  "vmaxps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1164[] = {
  "minps\t{%2, %0|%0, %2}",
  "vminps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1165[] = {
  "maxpd\t{%2, %0|%0, %2}",
  "vmaxpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1166[] = {
  "minpd\t{%2, %0|%0, %2}",
  "vminpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1167[] = {
  "maxpd\t{%2, %0|%0, %2}",
  "vmaxpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1168[] = {
  "minpd\t{%2, %0|%0, %2}",
  "vminpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1169[] = {
  "maxss\t{%2, %0|%0, %2}",
  "vmaxss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1170[] = {
  "minss\t{%2, %0|%0, %2}",
  "vminss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1171[] = {
  "maxsd\t{%2, %0|%0, %2}",
  "vmaxsd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1172[] = {
  "minsd\t{%2, %0|%0, %2}",
  "vminsd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1173[] = {
  "minps\t{%2, %0|%0, %2}",
  "vminps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1174[] = {
  "minps\t{%2, %0|%0, %2}",
  "vminps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1175[] = {
  "minpd\t{%2, %0|%0, %2}",
  "vminpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1176[] = {
  "minpd\t{%2, %0|%0, %2}",
  "vminpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1177[] = {
  "maxps\t{%2, %0|%0, %2}",
  "vmaxps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1178[] = {
  "maxps\t{%2, %0|%0, %2}",
  "vmaxps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1179[] = {
  "maxpd\t{%2, %0|%0, %2}",
  "vmaxpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1180[] = {
  "maxpd\t{%2, %0|%0, %2}",
  "vmaxpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1182[] = {
  "addsubpd\t{%2, %0|%0, %2}",
  "vaddsubpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1184[] = {
  "addsubps\t{%2, %0|%0, %2}",
  "vaddsubps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1187[] = {
  "haddpd\t{%2, %0|%0, %2}",
  "vhaddpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1188[] = {
  "hsubpd\t{%2, %0|%0, %2}",
  "vhsubpd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1191[] = {
  "haddps\t{%2, %0|%0, %2}",
  "vhaddps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1192[] = {
  "hsubps\t{%2, %0|%0, %2}",
  "vhsubps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1199[] = {
  "cmp%D3ps\t{%2, %0|%0, %2}",
  "vcmp%D3ps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1200[] = {
  "cmp%D3ps\t{%2, %0|%0, %2}",
  "vcmp%D3ps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1201[] = {
  "cmp%D3pd\t{%2, %0|%0, %2}",
  "vcmp%D3pd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1202[] = {
  "cmp%D3pd\t{%2, %0|%0, %2}",
  "vcmp%D3pd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1203[] = {
  "cmp%D3ps\t{%2, %0|%0, %2}",
  "vcmp%D3ps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1204[] = {
  "cmp%D3ps\t{%2, %0|%0, %2}",
  "vcmp%D3ps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1205[] = {
  "cmp%D3pd\t{%2, %0|%0, %2}",
  "vcmp%D3pd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1206[] = {
  "cmp%D3pd\t{%2, %0|%0, %2}",
  "vcmp%D3pd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1207[] = {
  "cmp%D3ss\t{%2, %0|%0, %2}",
  "vcmp%D3ss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1208[] = {
  "cmp%D3sd\t{%2, %0|%0, %2}",
  "vcmp%D3sd\t{%2, %1, %0|%0, %1, %2}",
};

static const char *
output_1213 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1561 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "ps";

  switch (which_alternative)
    {
    case 0:
      insn = "andn%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vandn%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1214 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1561 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "ps";

  switch (which_alternative)
    {
    case 0:
      insn = "andn%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vandn%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1215 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1561 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "pd";

  switch (which_alternative)
    {
    case 0:
      insn = "andn%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vandn%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1216 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1561 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "pd";

  switch (which_alternative)
    {
    case 0:
      insn = "andn%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vandn%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1217 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1601 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "ps";

  switch (which_alternative)
    {
    case 0:
      insn = "and%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vand%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1218 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1601 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "ps";

  switch (which_alternative)
    {
    case 0:
      insn = "or%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vor%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1219 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1601 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "ps";

  switch (which_alternative)
    {
    case 0:
      insn = "xor%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vxor%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1220 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1601 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "ps";

  switch (which_alternative)
    {
    case 0:
      insn = "and%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vand%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1221 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1601 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "ps";

  switch (which_alternative)
    {
    case 0:
      insn = "or%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vor%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1222 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1601 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "ps";

  switch (which_alternative)
    {
    case 0:
      insn = "xor%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vxor%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1223 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1601 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "pd";

  switch (which_alternative)
    {
    case 0:
      insn = "and%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vand%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1224 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1601 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "pd";

  switch (which_alternative)
    {
    case 0:
      insn = "or%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vor%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1225 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1601 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "pd";

  switch (which_alternative)
    {
    case 0:
      insn = "xor%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vxor%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1226 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1601 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "pd";

  switch (which_alternative)
    {
    case 0:
      insn = "and%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vand%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1227 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1601 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "pd";

  switch (which_alternative)
    {
    case 0:
      insn = "or%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vor%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1228 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1601 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "pd";

  switch (which_alternative)
    {
    case 0:
      insn = "xor%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vxor%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1229 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1657 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "ps";

  switch (which_alternative)
    {
    case 0:
      insn = "andn%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vandn%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1230 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1657 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "pd";

  switch (which_alternative)
    {
    case 0:
      insn = "andn%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vandn%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1231 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1689 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "ps";

  switch (which_alternative)
    {
    case 0:
      insn = "and%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vand%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1232 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1689 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "ps";

  switch (which_alternative)
    {
    case 0:
      insn = "or%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vor%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1233 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1689 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "ps";

  switch (which_alternative)
    {
    case 0:
      insn = "xor%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vxor%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1234 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1689 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "pd";

  switch (which_alternative)
    {
    case 0:
      insn = "and%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vand%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1235 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1689 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "pd";

  switch (which_alternative)
    {
    case 0:
      insn = "or%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vor%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char *
output_1236 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 1689 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *insn;
  const char *suffix
    = TARGET_SSE_PACKED_SINGLE_INSN_OPTIMAL ? "ps" : "pd";

  switch (which_alternative)
    {
    case 0:
      insn = "xor%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      insn = "vxor%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), insn, suffix);
  return buf;
}
}

static const char * const output_1237[] = {
  "vfmadd132ss\t{%2, %3, %0|%0, %3, %2}",
  "vfmadd213ss\t{%3, %2, %0|%0, %2, %3}",
  "vfmadd231ss\t{%2, %1, %0|%0, %1, %2}",
  "vfmaddss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmaddss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1238[] = {
  "vfmadd132sd\t{%2, %3, %0|%0, %3, %2}",
  "vfmadd213sd\t{%3, %2, %0|%0, %2, %3}",
  "vfmadd231sd\t{%2, %1, %0|%0, %1, %2}",
  "vfmaddsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmaddsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1239[] = {
  "vfmadd132ps\t{%2, %3, %0|%0, %3, %2}",
  "vfmadd213ps\t{%3, %2, %0|%0, %2, %3}",
  "vfmadd231ps\t{%2, %1, %0|%0, %1, %2}",
  "vfmaddps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmaddps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1240[] = {
  "vfmadd132pd\t{%2, %3, %0|%0, %3, %2}",
  "vfmadd213pd\t{%3, %2, %0|%0, %2, %3}",
  "vfmadd231pd\t{%2, %1, %0|%0, %1, %2}",
  "vfmaddpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmaddpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1241[] = {
  "vfmadd132ps\t{%2, %3, %0|%0, %3, %2}",
  "vfmadd213ps\t{%3, %2, %0|%0, %2, %3}",
  "vfmadd231ps\t{%2, %1, %0|%0, %1, %2}",
  "vfmaddps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmaddps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1242[] = {
  "vfmadd132pd\t{%2, %3, %0|%0, %3, %2}",
  "vfmadd213pd\t{%3, %2, %0|%0, %2, %3}",
  "vfmadd231pd\t{%2, %1, %0|%0, %1, %2}",
  "vfmaddpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmaddpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1243[] = {
  "vfmsub132ss\t{%2, %3, %0|%0, %3, %2}",
  "vfmsub213ss\t{%3, %2, %0|%0, %2, %3}",
  "vfmsub231ss\t{%2, %1, %0|%0, %1, %2}",
  "vfmsubss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmsubss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1244[] = {
  "vfmsub132sd\t{%2, %3, %0|%0, %3, %2}",
  "vfmsub213sd\t{%3, %2, %0|%0, %2, %3}",
  "vfmsub231sd\t{%2, %1, %0|%0, %1, %2}",
  "vfmsubsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmsubsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1245[] = {
  "vfmsub132ps\t{%2, %3, %0|%0, %3, %2}",
  "vfmsub213ps\t{%3, %2, %0|%0, %2, %3}",
  "vfmsub231ps\t{%2, %1, %0|%0, %1, %2}",
  "vfmsubps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmsubps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1246[] = {
  "vfmsub132pd\t{%2, %3, %0|%0, %3, %2}",
  "vfmsub213pd\t{%3, %2, %0|%0, %2, %3}",
  "vfmsub231pd\t{%2, %1, %0|%0, %1, %2}",
  "vfmsubpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmsubpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1247[] = {
  "vfmsub132ps\t{%2, %3, %0|%0, %3, %2}",
  "vfmsub213ps\t{%3, %2, %0|%0, %2, %3}",
  "vfmsub231ps\t{%2, %1, %0|%0, %1, %2}",
  "vfmsubps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmsubps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1248[] = {
  "vfmsub132pd\t{%2, %3, %0|%0, %3, %2}",
  "vfmsub213pd\t{%3, %2, %0|%0, %2, %3}",
  "vfmsub231pd\t{%2, %1, %0|%0, %1, %2}",
  "vfmsubpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmsubpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1249[] = {
  "vfnmadd132ss\t{%2, %3, %0|%0, %3, %2}",
  "vfnmadd213ss\t{%3, %2, %0|%0, %2, %3}",
  "vfnmadd231ss\t{%2, %1, %0|%0, %1, %2}",
  "vfnmaddss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfnmaddss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1250[] = {
  "vfnmadd132sd\t{%2, %3, %0|%0, %3, %2}",
  "vfnmadd213sd\t{%3, %2, %0|%0, %2, %3}",
  "vfnmadd231sd\t{%2, %1, %0|%0, %1, %2}",
  "vfnmaddsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfnmaddsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1251[] = {
  "vfnmadd132ps\t{%2, %3, %0|%0, %3, %2}",
  "vfnmadd213ps\t{%3, %2, %0|%0, %2, %3}",
  "vfnmadd231ps\t{%2, %1, %0|%0, %1, %2}",
  "vfnmaddps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfnmaddps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1252[] = {
  "vfnmadd132pd\t{%2, %3, %0|%0, %3, %2}",
  "vfnmadd213pd\t{%3, %2, %0|%0, %2, %3}",
  "vfnmadd231pd\t{%2, %1, %0|%0, %1, %2}",
  "vfnmaddpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfnmaddpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1253[] = {
  "vfnmadd132ps\t{%2, %3, %0|%0, %3, %2}",
  "vfnmadd213ps\t{%3, %2, %0|%0, %2, %3}",
  "vfnmadd231ps\t{%2, %1, %0|%0, %1, %2}",
  "vfnmaddps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfnmaddps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1254[] = {
  "vfnmadd132pd\t{%2, %3, %0|%0, %3, %2}",
  "vfnmadd213pd\t{%3, %2, %0|%0, %2, %3}",
  "vfnmadd231pd\t{%2, %1, %0|%0, %1, %2}",
  "vfnmaddpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfnmaddpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1255[] = {
  "vfnmsub132ss\t{%2, %3, %0|%0, %3, %2}",
  "vfnmsub213ss\t{%3, %2, %0|%0, %2, %3}",
  "vfnmsub231ss\t{%2, %1, %0|%0, %1, %2}",
  "vfnmsubss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfnmsubss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1256[] = {
  "vfnmsub132sd\t{%2, %3, %0|%0, %3, %2}",
  "vfnmsub213sd\t{%3, %2, %0|%0, %2, %3}",
  "vfnmsub231sd\t{%2, %1, %0|%0, %1, %2}",
  "vfnmsubsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfnmsubsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1257[] = {
  "vfnmsub132ps\t{%2, %3, %0|%0, %3, %2}",
  "vfnmsub213ps\t{%3, %2, %0|%0, %2, %3}",
  "vfnmsub231ps\t{%2, %1, %0|%0, %1, %2}",
  "vfnmsubps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfnmsubps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1258[] = {
  "vfnmsub132pd\t{%2, %3, %0|%0, %3, %2}",
  "vfnmsub213pd\t{%3, %2, %0|%0, %2, %3}",
  "vfnmsub231pd\t{%2, %1, %0|%0, %1, %2}",
  "vfnmsubpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfnmsubpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1259[] = {
  "vfnmsub132ps\t{%2, %3, %0|%0, %3, %2}",
  "vfnmsub213ps\t{%3, %2, %0|%0, %2, %3}",
  "vfnmsub231ps\t{%2, %1, %0|%0, %1, %2}",
  "vfnmsubps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfnmsubps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1260[] = {
  "vfnmsub132pd\t{%2, %3, %0|%0, %3, %2}",
  "vfnmsub213pd\t{%3, %2, %0|%0, %2, %3}",
  "vfnmsub231pd\t{%2, %1, %0|%0, %1, %2}",
  "vfnmsubpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfnmsubpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1261[] = {
  "vfmaddsub132ps\t{%2, %3, %0|%0, %3, %2}",
  "vfmaddsub213ps\t{%3, %2, %0|%0, %2, %3}",
  "vfmaddsub231ps\t{%2, %1, %0|%0, %1, %2}",
  "vfmaddsubps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmaddsubps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1262[] = {
  "vfmaddsub132ps\t{%2, %3, %0|%0, %3, %2}",
  "vfmaddsub213ps\t{%3, %2, %0|%0, %2, %3}",
  "vfmaddsub231ps\t{%2, %1, %0|%0, %1, %2}",
  "vfmaddsubps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmaddsubps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1263[] = {
  "vfmaddsub132pd\t{%2, %3, %0|%0, %3, %2}",
  "vfmaddsub213pd\t{%3, %2, %0|%0, %2, %3}",
  "vfmaddsub231pd\t{%2, %1, %0|%0, %1, %2}",
  "vfmaddsubpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmaddsubpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1264[] = {
  "vfmaddsub132pd\t{%2, %3, %0|%0, %3, %2}",
  "vfmaddsub213pd\t{%3, %2, %0|%0, %2, %3}",
  "vfmaddsub231pd\t{%2, %1, %0|%0, %1, %2}",
  "vfmaddsubpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmaddsubpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1265[] = {
  "vfmsubadd132ps\t{%2, %3, %0|%0, %3, %2}",
  "vfmsubadd213ps\t{%3, %2, %0|%0, %2, %3}",
  "vfmsubadd231ps\t{%2, %1, %0|%0, %1, %2}",
  "vfmsubaddps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmsubaddps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1266[] = {
  "vfmsubadd132ps\t{%2, %3, %0|%0, %3, %2}",
  "vfmsubadd213ps\t{%3, %2, %0|%0, %2, %3}",
  "vfmsubadd231ps\t{%2, %1, %0|%0, %1, %2}",
  "vfmsubaddps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmsubaddps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1267[] = {
  "vfmsubadd132pd\t{%2, %3, %0|%0, %3, %2}",
  "vfmsubadd213pd\t{%3, %2, %0|%0, %2, %3}",
  "vfmsubadd231pd\t{%2, %1, %0|%0, %1, %2}",
  "vfmsubaddpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmsubaddpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1268[] = {
  "vfmsubadd132pd\t{%2, %3, %0|%0, %3, %2}",
  "vfmsubadd213pd\t{%3, %2, %0|%0, %2, %3}",
  "vfmsubadd231pd\t{%2, %1, %0|%0, %1, %2}",
  "vfmsubaddpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
  "vfmsubaddpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1269[] = {
  "vfmadd132ss\t{%2, %3, %0|%0, %3, %2}",
  "vfmadd213ss\t{%3, %2, %0|%0, %2, %3}",
};

static const char * const output_1270[] = {
  "vfmadd132sd\t{%2, %3, %0|%0, %3, %2}",
  "vfmadd213sd\t{%3, %2, %0|%0, %2, %3}",
};

static const char * const output_1271[] = {
  "vfmsub132ss\t{%2, %3, %0|%0, %3, %2}",
  "vfmsub213ss\t{%3, %2, %0|%0, %2, %3}",
};

static const char * const output_1272[] = {
  "vfmsub132sd\t{%2, %3, %0|%0, %3, %2}",
  "vfmsub213sd\t{%3, %2, %0|%0, %2, %3}",
};

static const char * const output_1273[] = {
  "vfnmadd132ss\t{%2, %3, %0|%0, %3, %2}",
  "vfnmadd213ss\t{%3, %2, %0|%0, %2, %3}",
};

static const char * const output_1274[] = {
  "vfnmadd132sd\t{%2, %3, %0|%0, %3, %2}",
  "vfnmadd213sd\t{%3, %2, %0|%0, %2, %3}",
};

static const char * const output_1275[] = {
  "vfnmsub132ss\t{%2, %3, %0|%0, %3, %2}",
  "vfnmsub213ss\t{%3, %2, %0|%0, %2, %3}",
};

static const char * const output_1276[] = {
  "vfnmsub132sd\t{%2, %3, %0|%0, %3, %2}",
  "vfnmsub213sd\t{%3, %2, %0|%0, %2, %3}",
};

static const char * const output_1288[] = {
  "cvtsi2ss\t{%2, %0|%0, %2}",
  "cvtsi2ss\t{%2, %0|%0, %2}",
  "vcvtsi2ss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1289[] = {
  "cvtsi2ssq\t{%2, %0|%0, %2}",
  "cvtsi2ssq\t{%2, %0|%0, %2}",
  "vcvtsi2ssq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1305[] = {
  "cvtsi2sd\t{%2, %0|%0, %2}",
  "cvtsi2sd\t{%2, %0|%0, %2}",
  "vcvtsi2sd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1306[] = {
  "cvtsi2sdq\t{%2, %0|%0, %2}",
  "cvtsi2sdq\t{%2, %0|%0, %2}",
  "vcvtsi2sdq\t{%2, %1, %0|%0, %1, %2}",
};

static const char *
output_1318 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2574 "../../gcc/config/i386/sse.md"
{
  if (TARGET_AVX)
    return "vcvtpd2dq{x}\t{%1, %0|%0, %1}";
  else
    return "cvtpd2dq\t{%1, %0|%0, %1}";
}
}

static const char *
output_1321 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2631 "../../gcc/config/i386/sse.md"
{
  if (TARGET_AVX)
    return "vcvttpd2dq{x}\t{%1, %0|%0, %1}";
  else
    return "cvttpd2dq\t{%1, %0|%0, %1}";
}
}

static const char * const output_1322[] = {
  "cvtsd2ss\t{%2, %0|%0, %2}",
  "cvtsd2ss\t{%2, %0|%0, %2}",
  "vcvtsd2ss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1323[] = {
  "cvtss2sd\t{%2, %0|%0, %2}",
  "cvtss2sd\t{%2, %0|%0, %2}",
  "vcvtss2sd\t{%2, %1, %0|%0, %1, %2}",
};

static const char *
output_1325 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 2713 "../../gcc/config/i386/sse.md"
{
  if (TARGET_AVX)
    return "vcvtpd2ps{x}\t{%1, %0|%0, %1}";
  else
    return "cvtpd2ps\t{%1, %0|%0, %1}";
}
}

static const char * const output_1329[] = {
  "movhlps\t{%2, %0|%0, %2}",
  "vmovhlps\t{%2, %1, %0|%0, %1, %2}",
  "movlps\t{%H2, %0|%0, %H2}",
  "vmovlps\t{%H2, %1, %0|%0, %1, %H2}",
  "%vmovhps\t{%2, %0|%0, %2}",
};

static const char * const output_1330[] = {
  "movlhps\t{%2, %0|%0, %2}",
  "vmovlhps\t{%2, %1, %0|%0, %1, %2}",
  "movhps\t{%2, %0|%0, %2}",
  "vmovhps\t{%2, %1, %0|%0, %1, %2}",
  "%vmovlps\t{%2, %H0|%H0, %2}",
};

static const char * const output_1332[] = {
  "unpckhps\t{%2, %0|%0, %2}",
  "vunpckhps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1334[] = {
  "unpcklps\t{%2, %0|%0, %2}",
  "vunpcklps\t{%2, %1, %0|%0, %1, %2}",
};

static const char *
output_1339 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3540 "../../gcc/config/i386/sse.md"
{
  int mask;
  mask = INTVAL (operands[3]);
  mask |= INTVAL (operands[4]) << 2;
  mask |= (INTVAL (operands[5]) - 8) << 4;
  mask |= (INTVAL (operands[6]) - 8) << 6;
  operands[3] = GEN_INT (mask);

  return "vshufps\t{%3, %2, %1, %0|%0, %1, %2, %3}";
}
}

static const char *
output_1340 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3582 "../../gcc/config/i386/sse.md"
{
  int mask = 0;
  mask |= INTVAL (operands[3]) << 0;
  mask |= INTVAL (operands[4]) << 2;
  mask |= (INTVAL (operands[5]) - 4) << 4;
  mask |= (INTVAL (operands[6]) - 4) << 6;
  operands[3] = GEN_INT (mask);

  switch (which_alternative)
    {
    case 0:
      return "shufps\t{%3, %2, %0|%0, %2, %3}";
    case 1:
      return "vshufps\t{%3, %2, %1, %0|%0, %1, %2, %3}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1341 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3582 "../../gcc/config/i386/sse.md"
{
  int mask = 0;
  mask |= INTVAL (operands[3]) << 0;
  mask |= INTVAL (operands[4]) << 2;
  mask |= (INTVAL (operands[5]) - 4) << 4;
  mask |= (INTVAL (operands[6]) - 4) << 6;
  operands[3] = GEN_INT (mask);

  switch (which_alternative)
    {
    case 0:
      return "shufps\t{%3, %2, %0|%0, %2, %3}";
    case 1:
      return "vshufps\t{%3, %2, %1, %0|%0, %1, %2, %3}";
    default:
      gcc_unreachable ();
    }
}
}

static const char * const output_1342[] = {
  "%vmovhps\t{%1, %0|%0, %1}",
  "%vmovhlps\t{%1, %d0|%d0, %1}",
  "%vmovlps\t{%H1, %d0|%d0, %H1}",
};

static const char * const output_1343[] = {
  "movhps\t{%2, %0|%0, %2}",
  "vmovhps\t{%2, %1, %0|%0, %1, %2}",
  "movlhps\t{%2, %0|%0, %2}",
  "vmovlhps\t{%2, %1, %0|%0, %1, %2}",
  "%vmovlps\t{%2, %H0|%H0, %2}",
};

static const char * const output_1344[] = {
  "%vmovlps\t{%1, %0|%0, %1}",
  "%vmovaps\t{%1, %0|%0, %1}",
  "%vmovlps\t{%1, %d0|%d0, %1}",
};

static const char * const output_1345[] = {
  "shufps\t{$0xe4, %1, %0|%0, %1, 0xe4}",
  "vshufps\t{$0xe4, %1, %2, %0|%0, %2, %1, 0xe4}",
  "movlps\t{%2, %0|%0, %2}",
  "vmovlps\t{%2, %1, %0|%0, %1, %2}",
  "%vmovlps\t{%2, %0|%0, %2}",
};

static const char * const output_1346[] = {
  "movss\t{%2, %0|%0, %2}",
  "vmovss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1349[] = {
  "vshufps\t{$0, %1, %1, %0|%0, %1, %1, 0}",
  "vbroadcastss\t{%1, %0|%0, %1}",
  "shufps\t{$0, %0, %0|%0, %0, 0}",
};

static const char * const output_1350[] = {
  "unpcklps\t{%2, %0|%0, %2}",
  "vunpcklps\t{%2, %1, %0|%0, %1, %2}",
  "insertps\t{$0x10, %2, %0|%0, %2, 0x10}",
  "vinsertps\t{$0x10, %2, %1, %0|%0, %1, %2, 0x10}",
  "%vmovss\t{%1, %0|%0, %1}",
  "punpckldq\t{%2, %0|%0, %2}",
  "movd\t{%1, %0|%0, %1}",
};

static const char * const output_1351[] = {
  "unpcklps\t{%2, %0|%0, %2}",
  "movss\t{%1, %0|%0, %1}",
  "punpckldq\t{%2, %0|%0, %2}",
  "movd\t{%1, %0|%0, %1}",
};

static const char * const output_1352[] = {
  "movlhps\t{%2, %0|%0, %2}",
  "vmovlhps\t{%2, %1, %0|%0, %1, %2}",
  "movhps\t{%2, %0|%0, %2}",
  "vmovhps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1353[] = {
  "%vinsertps\t{$0xe, %d2, %0|%0, %d2, 0xe}",
  "%vmovd\t{%2, %0|%0, %2}",
  "%vmovd\t{%2, %0|%0, %2}",
  "movss\t{%2, %0|%0, %2}",
  "movss\t{%2, %0|%0, %2}",
  "vmovss\t{%2, %1, %0|%0, %1, %2}",
  "pinsrd\t{$0, %2, %0|%0, %2, 0}",
  "vpinsrd\t{$0, %2, %1, %0|%0, %1, %2, 0}",
  "#",
  "#",
  "#",
};

static const char * const output_1354[] = {
  "%vinsertps\t{$0xe, %d2, %0|%0, %d2, 0xe}",
  "%vmovss\t{%2, %0|%0, %2}",
  "%vmovd\t{%2, %0|%0, %2}",
  "movss\t{%2, %0|%0, %2}",
  "movss\t{%2, %0|%0, %2}",
  "vmovss\t{%2, %1, %0|%0, %1, %2}",
  "pinsrd\t{$0, %2, %0|%0, %2, 0}",
  "vpinsrd\t{$0, %2, %1, %0|%0, %1, %2, 0}",
  "#",
  "#",
  "#",
};

static const char *
output_1355 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3873 "../../gcc/config/i386/sse.md"
{
  operands[3] = GEN_INT (exact_log2 (INTVAL (operands[3])) << 4);
  switch (which_alternative)
    {
    case 0:
      return "insertps\t{%3, %2, %0|%0, %2, %3}";
    case 1:
      return "vinsertps\t{%3, %2, %1, %0|%0, %1, %2, %3}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1356 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 3900 "../../gcc/config/i386/sse.md"
{
  if (MEM_P (operands[2]))
    {
      unsigned count_s = INTVAL (operands[3]) >> 6;
      if (count_s)
	operands[3] = GEN_INT (INTVAL (operands[3]) & 0x3f);
      operands[2] = adjust_address_nv (operands[2], SFmode, count_s * 4);
    }
  switch (which_alternative)
    {
    case 0:
      return "insertps\t{%3, %2, %0|%0, %2, %3}";
    case 1:
      return "vinsertps\t{%3, %2, %1, %0|%0, %1, %2, %3}";
    default:
      gcc_unreachable ();
    }
}
}

static const char * const output_1358[] = {
  "%vextractps\t{%2, %1, %0|%0, %1, %2}",
  "#",
  "#",
};

static const char * const output_1373[] = {
  "unpckhpd\t{%2, %0|%0, %2}",
  "vunpckhpd\t{%2, %1, %0|%0, %1, %2}",
  "%vmovddup\t{%H1, %0|%0, %H1}",
  "movlpd\t{%H1, %0|%0, %H1}",
  "vmovlpd\t{%H1, %2, %0|%0, %2, %H1}",
  "%vmovhpd\t{%1, %0|%0, %1}",
};

static const char * const output_1374[] = {
  "vunpcklpd\t{%2, %1, %0|%0, %1, %2}",
  "vmovddup\t{%1, %0|%0, %1}",
};

static const char * const output_1375[] = {
  "unpcklpd\t{%2, %0|%0, %2}",
  "vunpcklpd\t{%2, %1, %0|%0, %1, %2}",
  "%vmovddup\t{%1, %0|%0, %1}",
  "movhpd\t{%2, %0|%0, %2}",
  "vmovhpd\t{%2, %1, %0|%0, %1, %2}",
  "%vmovlpd\t{%2, %H0|%H0, %2}",
};

static const char *
output_1376 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4470 "../../gcc/config/i386/sse.md"
{
  int mask;
  mask = INTVAL (operands[3]);
  mask |= (INTVAL (operands[4]) - 4) << 1;
  mask |= (INTVAL (operands[5]) - 2) << 2;
  mask |= (INTVAL (operands[6]) - 6) << 3;
  operands[3] = GEN_INT (mask);

  return "vshufpd\t{%3, %2, %1, %0|%0, %1, %2, %3}";
}
}

static const char * const output_1378[] = {
  "punpckhqdq\t{%2, %0|%0, %2}",
  "vpunpckhqdq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1380[] = {
  "punpcklqdq\t{%2, %0|%0, %2}",
  "vpunpcklqdq\t{%2, %1, %0|%0, %1, %2}",
};

static const char *
output_1381 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4577 "../../gcc/config/i386/sse.md"
{
  int mask;
  mask = INTVAL (operands[3]);
  mask |= (INTVAL (operands[4]) - 2) << 1;
  operands[3] = GEN_INT (mask);

  switch (which_alternative)
    {
    case 0:
      return "shufpd\t{%3, %2, %0|%0, %2, %3}";
    case 1:
      return "vshufpd\t{%3, %2, %1, %0|%0, %1, %2, %3}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1382 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 4577 "../../gcc/config/i386/sse.md"
{
  int mask;
  mask = INTVAL (operands[3]);
  mask |= (INTVAL (operands[4]) - 2) << 1;
  operands[3] = GEN_INT (mask);

  switch (which_alternative)
    {
    case 0:
      return "shufpd\t{%3, %2, %0|%0, %2, %3}";
    case 1:
      return "vshufpd\t{%3, %2, %1, %0|%0, %1, %2, %3}";
    default:
      gcc_unreachable ();
    }
}
}

static const char * const output_1383[] = {
  "%vmovhpd\t{%1, %0|%0, %1}",
  "unpckhpd\t%0, %0",
  "vunpckhpd\t{%d1, %0|%0, %d1}",
  "#",
  "#",
  "#",
};

static const char * const output_1384[] = {
  "movhps\t{%1, %0|%0, %1}",
  "movhlps\t{%1, %0|%0, %1}",
  "movlps\t{%H1, %0|%0, %H1}",
};

static const char * const output_1385[] = {
  "%vmovlpd\t{%1, %0|%0, %1}",
  "#",
  "#",
  "#",
  "#",
};

static const char * const output_1386[] = {
  "movlps\t{%1, %0|%0, %1}",
  "movaps\t{%1, %0|%0, %1}",
  "movlps\t{%1, %0|%0, %1}",
};

static const char * const output_1387[] = {
  "movhpd\t{%2, %0|%0, %2}",
  "vmovhpd\t{%2, %1, %0|%0, %1, %2}",
  "unpcklpd\t{%2, %0|%0, %2}",
  "vunpcklpd\t{%2, %1, %0|%0, %1, %2}",
  "#",
  "#",
  "#",
};

static const char * const output_1388[] = {
  "%vmovsd\t{%2, %0|%0, %2}",
  "movlpd\t{%2, %0|%0, %2}",
  "vmovlpd\t{%2, %1, %0|%0, %1, %2}",
  "movsd\t{%2, %0|%0, %2}",
  "vmovsd\t{%2, %1, %0|%0, %1, %2}",
  "shufpd\t{$2, %1, %0|%0, %1, 2}",
  "movhpd\t{%H1, %0|%0, %H1}",
  "vmovhpd\t{%H1, %2, %0|%0, %2, %H1}",
  "#",
  "#",
  "#",
};

static const char * const output_1389[] = {
  "movsd\t{%2, %0|%0, %2}",
  "vmovsd\t{%2, %1, %0|%0, %1, %2}",
  "movlpd\t{%2, %0|%0, %2}",
  "vmovlpd\t{%2, %1, %0|%0, %1, %2}",
  "%vmovlpd\t{%2, %0|%0, %2}",
  "shufpd\t{$2, %1, %0|%0, %1, 2}",
  "movhps\t{%H1, %0|%0, %H1}",
  "vmovhps\t{%H1, %2, %0|%0, %2, %H1}",
  "%vmovhps\t{%1, %H0|%H0, %1}",
};

static const char * const output_1390[] = {
  "unpcklpd\t%0, %0",
  "%vmovddup\t{%1, %0|%0, %1}",
};

static const char * const output_1391[] = {
  "unpcklpd\t{%2, %0|%0, %2}",
  "vunpcklpd\t{%2, %1, %0|%0, %1, %2}",
  "%vmovddup\t{%1, %0|%0, %1}",
  "movhpd\t{%2, %0|%0, %2}",
  "vmovhpd\t{%2, %1, %0|%0, %1, %2}",
  "%vmovsd\t{%1, %0|%0, %1}",
  "movlhps\t{%2, %0|%0, %2}",
  "movhps\t{%2, %0|%0, %2}",
};

static const char * const output_1392[] = {
  "paddb\t{%2, %0|%0, %2}",
  "vpaddb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1393[] = {
  "psubb\t{%2, %0|%0, %2}",
  "vpsubb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1394[] = {
  "paddb\t{%2, %0|%0, %2}",
  "vpaddb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1395[] = {
  "psubb\t{%2, %0|%0, %2}",
  "vpsubb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1396[] = {
  "paddw\t{%2, %0|%0, %2}",
  "vpaddw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1397[] = {
  "psubw\t{%2, %0|%0, %2}",
  "vpsubw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1398[] = {
  "paddw\t{%2, %0|%0, %2}",
  "vpaddw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1399[] = {
  "psubw\t{%2, %0|%0, %2}",
  "vpsubw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1400[] = {
  "paddd\t{%2, %0|%0, %2}",
  "vpaddd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1401[] = {
  "psubd\t{%2, %0|%0, %2}",
  "vpsubd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1402[] = {
  "paddd\t{%2, %0|%0, %2}",
  "vpaddd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1403[] = {
  "psubd\t{%2, %0|%0, %2}",
  "vpsubd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1404[] = {
  "paddq\t{%2, %0|%0, %2}",
  "vpaddq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1405[] = {
  "psubq\t{%2, %0|%0, %2}",
  "vpsubq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1406[] = {
  "paddq\t{%2, %0|%0, %2}",
  "vpaddq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1407[] = {
  "psubq\t{%2, %0|%0, %2}",
  "vpsubq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1408[] = {
  "paddsb\t{%2, %0|%0, %2}",
  "vpaddsb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1409[] = {
  "paddusb\t{%2, %0|%0, %2}",
  "vpaddusb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1410[] = {
  "psubsb\t{%2, %0|%0, %2}",
  "vpsubsb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1411[] = {
  "psubusb\t{%2, %0|%0, %2}",
  "vpsubusb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1412[] = {
  "paddsb\t{%2, %0|%0, %2}",
  "vpaddsb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1413[] = {
  "paddusb\t{%2, %0|%0, %2}",
  "vpaddusb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1414[] = {
  "psubsb\t{%2, %0|%0, %2}",
  "vpsubsb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1415[] = {
  "psubusb\t{%2, %0|%0, %2}",
  "vpsubusb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1416[] = {
  "paddsw\t{%2, %0|%0, %2}",
  "vpaddsw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1417[] = {
  "paddusw\t{%2, %0|%0, %2}",
  "vpaddusw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1418[] = {
  "psubsw\t{%2, %0|%0, %2}",
  "vpsubsw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1419[] = {
  "psubusw\t{%2, %0|%0, %2}",
  "vpsubusw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1420[] = {
  "paddsw\t{%2, %0|%0, %2}",
  "vpaddsw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1421[] = {
  "paddusw\t{%2, %0|%0, %2}",
  "vpaddusw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1422[] = {
  "psubsw\t{%2, %0|%0, %2}",
  "vpsubsw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1423[] = {
  "psubusw\t{%2, %0|%0, %2}",
  "vpsubusw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1426[] = {
  "pmullw\t{%2, %0|%0, %2}",
  "vpmullw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1427[] = {
  "pmullw\t{%2, %0|%0, %2}",
  "vpmullw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1428[] = {
  "pmulhw\t{%2, %0|%0, %2}",
  "vpmulhw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1429[] = {
  "pmulhuw\t{%2, %0|%0, %2}",
  "vpmulhuw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1430[] = {
  "pmulhw\t{%2, %0|%0, %2}",
  "vpmulhw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1431[] = {
  "pmulhuw\t{%2, %0|%0, %2}",
  "vpmulhuw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1433[] = {
  "pmuludq\t{%2, %0|%0, %2}",
  "vpmuludq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1435[] = {
  "pmuldq\t{%2, %0|%0, %2}",
  "vpmuldq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1437[] = {
  "pmaddwd\t{%2, %0|%0, %2}",
  "vpmaddwd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1438[] = {
  "pmulld\t{%2, %0|%0, %2}",
  "vpmulld\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1439[] = {
  "pmulld\t{%2, %0|%0, %2}",
  "vpmulld\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1443[] = {
  "psraw\t{%2, %0|%0, %2}",
  "vpsraw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1444[] = {
  "psraw\t{%2, %0|%0, %2}",
  "vpsraw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1445[] = {
  "psrad\t{%2, %0|%0, %2}",
  "vpsrad\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1446[] = {
  "psrad\t{%2, %0|%0, %2}",
  "vpsrad\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1447[] = {
  "psllw\t{%2, %0|%0, %2}",
  "vpsllw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1448[] = {
  "psrlw\t{%2, %0|%0, %2}",
  "vpsrlw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1449[] = {
  "psllw\t{%2, %0|%0, %2}",
  "vpsllw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1450[] = {
  "psrlw\t{%2, %0|%0, %2}",
  "vpsrlw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1451[] = {
  "pslld\t{%2, %0|%0, %2}",
  "vpslld\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1452[] = {
  "psrld\t{%2, %0|%0, %2}",
  "vpsrld\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1453[] = {
  "pslld\t{%2, %0|%0, %2}",
  "vpslld\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1454[] = {
  "psrld\t{%2, %0|%0, %2}",
  "vpsrld\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1455[] = {
  "psllq\t{%2, %0|%0, %2}",
  "vpsllq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1456[] = {
  "psrlq\t{%2, %0|%0, %2}",
  "vpsrlq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1457[] = {
  "psllq\t{%2, %0|%0, %2}",
  "vpsllq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1458[] = {
  "psrlq\t{%2, %0|%0, %2}",
  "vpsrlq\t{%2, %1, %0|%0, %1, %2}",
};

static const char *
output_1459 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5894 "../../gcc/config/i386/sse.md"
{
  operands[2] = GEN_INT (INTVAL (operands[2]) / 8);

  switch (which_alternative)
    {
    case 0:
      return "pslldq\t{%2, %0|%0, %2}";
    case 1:
      return "vpslldq\t{%2, %1, %0|%0, %1, %2}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1460 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5894 "../../gcc/config/i386/sse.md"
{
  operands[2] = GEN_INT (INTVAL (operands[2]) / 8);

  switch (which_alternative)
    {
    case 0:
      return "pslldq\t{%2, %0|%0, %2}";
    case 1:
      return "vpslldq\t{%2, %1, %0|%0, %1, %2}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1461 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5931 "../../gcc/config/i386/sse.md"
{
  operands[2] = GEN_INT (INTVAL (operands[2]) / 8);

  switch (which_alternative)
    {
    case 0:
      return "psrldq\t{%2, %0|%0, %2}";
    case 1:
      return "vpsrldq\t{%2, %1, %0|%0, %1, %2}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1462 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 5931 "../../gcc/config/i386/sse.md"
{
  operands[2] = GEN_INT (INTVAL (operands[2]) / 8);

  switch (which_alternative)
    {
    case 0:
      return "psrldq\t{%2, %0|%0, %2}";
    case 1:
      return "vpsrldq\t{%2, %1, %0|%0, %1, %2}";
    default:
      gcc_unreachable ();
    }
}
}

static const char * const output_1475[] = {
  "pmaxsb\t{%2, %0|%0, %2}",
  "vpmaxsb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1476[] = {
  "pminsb\t{%2, %0|%0, %2}",
  "vpminsb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1477[] = {
  "pmaxsd\t{%2, %0|%0, %2}",
  "vpmaxsd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1478[] = {
  "pminsd\t{%2, %0|%0, %2}",
  "vpminsd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1479[] = {
  "pmaxsw\t{%2, %0|%0, %2}",
  "vpmaxsw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1480[] = {
  "pminsw\t{%2, %0|%0, %2}",
  "vpminsw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1481[] = {
  "pmaxuw\t{%2, %0|%0, %2}",
  "vpmaxuw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1482[] = {
  "pminuw\t{%2, %0|%0, %2}",
  "vpminuw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1483[] = {
  "pmaxud\t{%2, %0|%0, %2}",
  "vpmaxud\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1484[] = {
  "pminud\t{%2, %0|%0, %2}",
  "vpminud\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1485[] = {
  "pmaxub\t{%2, %0|%0, %2}",
  "vpmaxub\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1486[] = {
  "pminub\t{%2, %0|%0, %2}",
  "vpminub\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1491[] = {
  "pcmpeqq\t{%2, %0|%0, %2}",
  "vpcmpeqq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1492[] = {
  "pcmpeqb\t{%2, %0|%0, %2}",
  "vpcmpeqb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1493[] = {
  "pcmpeqw\t{%2, %0|%0, %2}",
  "vpcmpeqw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1494[] = {
  "pcmpeqd\t{%2, %0|%0, %2}",
  "vpcmpeqd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1495[] = {
  "pcmpgtq\t{%2, %0|%0, %2}",
  "vpcmpgtq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1500[] = {
  "pcmpgtb\t{%2, %0|%0, %2}",
  "vpcmpgtb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1501[] = {
  "pcmpgtw\t{%2, %0|%0, %2}",
  "vpcmpgtw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1502[] = {
  "pcmpgtd\t{%2, %0|%0, %2}",
  "vpcmpgtd\t{%2, %1, %0|%0, %1, %2}",
};

static const char *
output_1503 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6444 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pandn";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andnps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1504 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6444 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pandn";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andnps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1505 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6444 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pandn";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andnps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1506 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6444 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pandn";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andnps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1507 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6444 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pandn";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andnps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1508 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6444 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pandn";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andnps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1509 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6444 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pandn";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andnps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1510 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6444 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pandn";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andnps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1511 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pand";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1512 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "por";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "orps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1513 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pxor";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "xorps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1514 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pand";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1515 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "por";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "orps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1516 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pxor";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "xorps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1517 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pand";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1518 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "por";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "orps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1519 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pxor";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "xorps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1520 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pand";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1521 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "por";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "orps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1522 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pxor";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "xorps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1523 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pand";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1524 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "por";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "orps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1525 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pxor";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "xorps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1526 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pand";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1527 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "por";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "orps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1528 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pxor";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "xorps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1529 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pand";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1530 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "por";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "orps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1531 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pxor";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "xorps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1532 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pand";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "andps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1533 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "por";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "orps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char *
output_1534 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6519 "../../gcc/config/i386/sse.md"
{
  static char buf[32];
  const char *ops;
  const char *tmp;

  switch (get_attr_mode (insn))
    {
    case MODE_OI:
      gcc_assert (TARGET_AVX2);
    case MODE_TI:
      gcc_assert (TARGET_SSE2);

      tmp = "pxor";
      break;

   case MODE_V8SF:
      gcc_assert (TARGET_AVX);
   case MODE_V4SF:
      gcc_assert (TARGET_SSE);

      tmp = "xorps";
      break;

   default:
      gcc_unreachable ();
   }

  switch (which_alternative)
    {
    case 0:
      ops = "%s\t{%%2, %%0|%%0, %%2}";
      break;
    case 1:
      ops = "v%s\t{%%2, %%1, %%0|%%0, %%1, %%2}";
      break;
    default:
      gcc_unreachable ();
    }

  snprintf (buf, sizeof (buf), ops, tmp);
  return buf;
}
}

static const char * const output_1535[] = {
  "pandn\t{%2, %0|%0, %2}",
  "vpandn\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1536[] = {
  "pand\t{%2, %0|%0, %2}",
  "vpand\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1537[] = {
  "por\t{%2, %0|%0, %2}",
  "vpor\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1538[] = {
  "pxor\t{%2, %0|%0, %2}",
  "vpxor\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1539[] = {
  "packsswb\t{%2, %0|%0, %2}",
  "vpacksswb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1540[] = {
  "packsswb\t{%2, %0|%0, %2}",
  "vpacksswb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1541[] = {
  "packssdw\t{%2, %0|%0, %2}",
  "vpackssdw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1542[] = {
  "packssdw\t{%2, %0|%0, %2}",
  "vpackssdw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1543[] = {
  "packuswb\t{%2, %0|%0, %2}",
  "vpackuswb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1544[] = {
  "packuswb\t{%2, %0|%0, %2}",
  "vpackuswb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1546[] = {
  "punpckhbw\t{%2, %0|%0, %2}",
  "vpunpckhbw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1548[] = {
  "punpcklbw\t{%2, %0|%0, %2}",
  "vpunpcklbw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1550[] = {
  "punpckhwd\t{%2, %0|%0, %2}",
  "vpunpckhwd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1552[] = {
  "punpcklwd\t{%2, %0|%0, %2}",
  "vpunpcklwd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1554[] = {
  "punpckhdq\t{%2, %0|%0, %2}",
  "vpunpckhdq\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1556[] = {
  "punpckldq\t{%2, %0|%0, %2}",
  "vpunpckldq\t{%2, %1, %0|%0, %1, %2}",
};

static const char *
output_1557 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6992 "../../gcc/config/i386/sse.md"
{
  operands[3] = GEN_INT (exact_log2 (INTVAL (operands[3])));

  switch (which_alternative)
    {
    case 0:
      if (GET_MODE_SIZE (QImode) < GET_MODE_SIZE (SImode))
	return "pinsrb\t{%3, %k2, %0|%0, %k2, %3}";
      /* FALLTHRU */
    case 1:
      return "pinsrb\t{%3, %2, %0|%0, %2, %3}";
    case 2:
      if (GET_MODE_SIZE (QImode) < GET_MODE_SIZE (SImode))
	return "vpinsrb\t{%3, %k2, %1, %0|%0, %1, %k2, %3}";
      /* FALLTHRU */
    case 3:
      return "vpinsrb\t{%3, %2, %1, %0|%0, %1, %2, %3}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1558 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6992 "../../gcc/config/i386/sse.md"
{
  operands[3] = GEN_INT (exact_log2 (INTVAL (operands[3])));

  switch (which_alternative)
    {
    case 0:
      if (GET_MODE_SIZE (HImode) < GET_MODE_SIZE (SImode))
	return "pinsrw\t{%3, %k2, %0|%0, %k2, %3}";
      /* FALLTHRU */
    case 1:
      return "pinsrw\t{%3, %2, %0|%0, %2, %3}";
    case 2:
      if (GET_MODE_SIZE (HImode) < GET_MODE_SIZE (SImode))
	return "vpinsrw\t{%3, %k2, %1, %0|%0, %1, %k2, %3}";
      /* FALLTHRU */
    case 3:
      return "vpinsrw\t{%3, %2, %1, %0|%0, %1, %2, %3}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1559 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6992 "../../gcc/config/i386/sse.md"
{
  operands[3] = GEN_INT (exact_log2 (INTVAL (operands[3])));

  switch (which_alternative)
    {
    case 0:
      if (GET_MODE_SIZE (SImode) < GET_MODE_SIZE (SImode))
	return "pinsrd\t{%3, %k2, %0|%0, %k2, %3}";
      /* FALLTHRU */
    case 1:
      return "pinsrd\t{%3, %2, %0|%0, %2, %3}";
    case 2:
      if (GET_MODE_SIZE (SImode) < GET_MODE_SIZE (SImode))
	return "vpinsrd\t{%3, %k2, %1, %0|%0, %1, %k2, %3}";
      /* FALLTHRU */
    case 3:
      return "vpinsrd\t{%3, %2, %1, %0|%0, %1, %2, %3}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1560 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 6992 "../../gcc/config/i386/sse.md"
{
  operands[3] = GEN_INT (exact_log2 (INTVAL (operands[3])));

  switch (which_alternative)
    {
    case 0:
      if (GET_MODE_SIZE (DImode) < GET_MODE_SIZE (SImode))
	return "pinsrq\t{%3, %k2, %0|%0, %k2, %3}";
      /* FALLTHRU */
    case 1:
      return "pinsrq\t{%3, %2, %0|%0, %2, %3}";
    case 2:
      if (GET_MODE_SIZE (DImode) < GET_MODE_SIZE (SImode))
	return "vpinsrq\t{%3, %k2, %1, %0|%0, %1, %k2, %3}";
      /* FALLTHRU */
    case 3:
      return "vpinsrq\t{%3, %2, %1, %0|%0, %1, %2, %3}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1570 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 7169 "../../gcc/config/i386/sse.md"
{
  int mask = 0;
  mask |= INTVAL (operands[2]) << 0;
  mask |= INTVAL (operands[3]) << 2;
  mask |= INTVAL (operands[4]) << 4;
  mask |= INTVAL (operands[5]) << 6;
  operands[2] = GEN_INT (mask);

  return "vpshufd\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char *
output_1571 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 7208 "../../gcc/config/i386/sse.md"
{
  int mask = 0;
  mask |= INTVAL (operands[2]) << 0;
  mask |= INTVAL (operands[3]) << 2;
  mask |= INTVAL (operands[4]) << 4;
  mask |= INTVAL (operands[5]) << 6;
  operands[2] = GEN_INT (mask);

  return "%vpshufd\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char *
output_1572 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 7268 "../../gcc/config/i386/sse.md"
{
  int mask = 0;
  mask |= INTVAL (operands[2]) << 0;
  mask |= INTVAL (operands[3]) << 2;
  mask |= INTVAL (operands[4]) << 4;
  mask |= INTVAL (operands[5]) << 6;
  operands[2] = GEN_INT (mask);

  return "vpshuflw\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char *
output_1573 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 7311 "../../gcc/config/i386/sse.md"
{
  int mask = 0;
  mask |= INTVAL (operands[2]) << 0;
  mask |= INTVAL (operands[3]) << 2;
  mask |= INTVAL (operands[4]) << 4;
  mask |= INTVAL (operands[5]) << 6;
  operands[2] = GEN_INT (mask);

  return "%vpshuflw\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char *
output_1574 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 7372 "../../gcc/config/i386/sse.md"
{
  int mask = 0;
  mask |= (INTVAL (operands[2]) - 4) << 0;
  mask |= (INTVAL (operands[3]) - 4) << 2;
  mask |= (INTVAL (operands[4]) - 4) << 4;
  mask |= (INTVAL (operands[5]) - 4) << 6;
  operands[2] = GEN_INT (mask);

  return "vpshufhw\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char *
output_1575 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 7415 "../../gcc/config/i386/sse.md"
{
  int mask = 0;
  mask |= (INTVAL (operands[2]) - 4) << 0;
  mask |= (INTVAL (operands[3]) - 4) << 2;
  mask |= (INTVAL (operands[4]) - 4) << 4;
  mask |= (INTVAL (operands[5]) - 4) << 6;
  operands[2] = GEN_INT (mask);

  return "%vpshufhw\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char * const output_1576[] = {
  "%vmovd\t{%2, %0|%0, %2}",
  "%vmovd\t{%2, %0|%0, %2}",
  "movss\t{%2, %0|%0, %2}",
  "movss\t{%2, %0|%0, %2}",
  "vmovss\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1579[] = {
  "#",
  "#",
  "mov{q}\t{%1, %0|%0, %1}",
};

static const char * const output_1581[] = {
  "%vmovhps\t{%1, %0|%0, %1}",
  "psrldq\t{$8, %0|%0, 8}",
  "vpsrldq\t{$8, %1, %0|%0, %1, 8}",
  "%vmovq\t{%H1, %0|%0, %H1}",
  "mov{q}\t{%H1, %0|%0, %H1}",
};

static const char * const output_1582[] = {
  "%vmovhps\t{%1, %0|%0, %1}",
  "psrldq\t{$8, %0|%0, 8}",
  "vpsrldq\t{$8, %1, %0|%0, %1, 8}",
  "%vmovq\t{%H1, %0|%0, %H1}",
  "movhlps\t{%1, %0|%0, %1}",
  "movlps\t{%H1, %0|%0, %H1}",
};

static const char * const output_1583[] = {
  "%vpshufd\t{$0, %1, %0|%0, %1, 0}",
  "vbroadcastss\t{%1, %0|%0, %1}",
  "shufps\t{$0, %0, %0|%0, %0, 0}",
};

static const char * const output_1584[] = {
  "punpcklqdq\t%0, %0",
  "vpunpcklqdq\t{%d1, %0|%0, %d1}",
  "%vmovddup\t{%1, %0|%0, %1}",
  "movlhps\t%0, %0",
};

static const char * const output_1585[] = {
  "pinsrd\t{$1, %2, %0|%0, %2, 1}",
  "vpinsrd\t{$1, %2, %1, %0|%0, %1, %2, 1}",
  "punpckldq\t{%2, %0|%0, %2}",
  "vpunpckldq\t{%2, %1, %0|%0, %1, %2}",
  "%vmovd\t{%1, %0|%0, %1}",
  "punpckldq\t{%2, %0|%0, %2}",
  "movd\t{%1, %0|%0, %1}",
};

static const char * const output_1586[] = {
  "punpckldq\t{%2, %0|%0, %2}",
  "movd\t{%1, %0|%0, %1}",
  "punpckldq\t{%2, %0|%0, %2}",
  "movd\t{%1, %0|%0, %1}",
};

static const char * const output_1587[] = {
  "unpcklps\t{%2, %0|%0, %2}",
  "movss\t{%1, %0|%0, %1}",
  "punpckldq\t{%2, %0|%0, %2}",
  "movd\t{%1, %0|%0, %1}",
};

static const char * const output_1588[] = {
  "punpcklqdq\t{%2, %0|%0, %2}",
  "vpunpcklqdq\t{%2, %1, %0|%0, %1, %2}",
  "movlhps\t{%2, %0|%0, %2}",
  "movhps\t{%2, %0|%0, %2}",
  "vmovhps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1589[] = {
  "pinsrq\t{$1, %2, %0|%0, %2, 1}",
  "vpinsrq\t{$1, %2, %1, %0|%0, %1, %2, 1}",
  "%vmovq\t{%1, %0|%0, %1}",
  "%vmovd\t{%1, %0|%0, %1}",
  "movq2dq\t{%1, %0|%0, %1}",
  "punpcklqdq\t{%2, %0|%0, %2}",
  "vpunpcklqdq\t{%2, %1, %0|%0, %1, %2}",
  "movhps\t{%2, %0|%0, %2}",
  "vmovhps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1590[] = {
  "%vmovq\t{%1, %0|%0, %1}",
  "movq2dq\t{%1, %0|%0, %1}",
  "punpcklqdq\t{%2, %0|%0, %2}",
  "vpunpcklqdq\t{%2, %1, %0|%0, %1, %2}",
  "movlhps\t{%2, %0|%0, %2}",
  "movhps\t{%2, %0|%0, %2}",
  "vmovhps\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1592[] = {
  "pavgb\t{%2, %0|%0, %2}",
  "vpavgb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1594[] = {
  "pavgw\t{%2, %0|%0, %2}",
  "vpavgw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1595[] = {
  "psadbw\t{%2, %0|%0, %2}",
  "vpsadbw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1596[] = {
  "psadbw\t{%2, %0|%0, %2}",
  "vpsadbw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1612[] = {
  "phaddw\t{%2, %0|%0, %2}",
  "vphaddw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1615[] = {
  "phaddd\t{%2, %0|%0, %2}",
  "vphaddd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1618[] = {
  "phaddsw\t{%2, %0|%0, %2}",
  "vphaddsw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1621[] = {
  "phsubw\t{%2, %0|%0, %2}",
  "vphsubw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1624[] = {
  "phsubd\t{%2, %0|%0, %2}",
  "vphsubd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1627[] = {
  "phsubsw\t{%2, %0|%0, %2}",
  "vphsubsw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1630[] = {
  "pmaddubsw\t{%2, %0|%0, %2}",
  "vpmaddubsw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1633[] = {
  "pmulhrsw\t{%2, %0|%0, %2}",
  "vpmulhrsw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1635[] = {
  "pshufb\t{%2, %0|%0, %2}",
  "vpshufb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1636[] = {
  "pshufb\t{%2, %0|%0, %2}",
  "vpshufb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1638[] = {
  "psignb\t{%2, %0|%0, %2}",
  "vpsignb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1639[] = {
  "psignb\t{%2, %0|%0, %2}",
  "vpsignb\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1640[] = {
  "psignw\t{%2, %0|%0, %2}",
  "vpsignw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1641[] = {
  "psignw\t{%2, %0|%0, %2}",
  "vpsignw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1642[] = {
  "psignd\t{%2, %0|%0, %2}",
  "vpsignd\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1643[] = {
  "psignd\t{%2, %0|%0, %2}",
  "vpsignd\t{%2, %1, %0|%0, %1, %2}",
};

static const char *
output_1647 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9306 "../../gcc/config/i386/sse.md"
{
  operands[3] = GEN_INT (INTVAL (operands[3]) / 8);

  switch (which_alternative)
    {
    case 0:
      return "palignr\t{%3, %2, %0|%0, %2, %3}";
    case 1:
      return "vpalignr\t{%3, %2, %1, %0|%0, %1, %2, %3}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1648 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9306 "../../gcc/config/i386/sse.md"
{
  operands[3] = GEN_INT (INTVAL (operands[3]) / 8);

  switch (which_alternative)
    {
    case 0:
      return "palignr\t{%3, %2, %0|%0, %2, %3}";
    case 1:
      return "vpalignr\t{%3, %2, %1, %0|%0, %1, %2, %3}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1649 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9335 "../../gcc/config/i386/sse.md"
{
  operands[3] = GEN_INT (INTVAL (operands[3]) / 8);
  return "palignr\t{%3, %2, %0|%0, %2, %3}";
}
}

static const char * const output_1667[] = {
  "blendps\t{%3, %2, %0|%0, %2, %3}",
  "vblendps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1668[] = {
  "blendps\t{%3, %2, %0|%0, %2, %3}",
  "vblendps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1669[] = {
  "blendpd\t{%3, %2, %0|%0, %2, %3}",
  "vblendpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1670[] = {
  "blendpd\t{%3, %2, %0|%0, %2, %3}",
  "vblendpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1671[] = {
  "blendvps\t{%3, %2, %0|%0, %2, %3}",
  "vblendvps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1672[] = {
  "blendvps\t{%3, %2, %0|%0, %2, %3}",
  "vblendvps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1673[] = {
  "blendvpd\t{%3, %2, %0|%0, %2, %3}",
  "vblendvpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1674[] = {
  "blendvpd\t{%3, %2, %0|%0, %2, %3}",
  "vblendvpd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1675[] = {
  "dpps\t{%3, %2, %0|%0, %2, %3}",
  "vdpps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1676[] = {
  "dpps\t{%3, %2, %0|%0, %2, %3}",
  "vdpps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1677[] = {
  "dppd\t{%3, %2, %0|%0, %2, %3}",
  "vdppd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1678[] = {
  "dppd\t{%3, %2, %0|%0, %2, %3}",
  "vdppd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1681[] = {
  "mpsadbw\t{%3, %2, %0|%0, %2, %3}",
  "vmpsadbw\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1682[] = {
  "mpsadbw\t{%3, %2, %0|%0, %2, %3}",
  "vmpsadbw\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1684[] = {
  "packusdw\t{%2, %0|%0, %2}",
  "vpackusdw\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1685[] = {
  "pblendvb\t{%3, %2, %0|%0, %2, %3}",
  "vpblendvb\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1686[] = {
  "pblendvb\t{%3, %2, %0|%0, %2, %3}",
  "vpblendvb\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1687[] = {
  "pblendw\t{%3, %2, %0|%0, %2, %3}",
  "vpblendw\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char *
output_1688 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 9625 "../../gcc/config/i386/sse.md"
{
  operands[3] = GEN_INT (INTVAL (operands[3]) & 0xff);
  return "vpblendw\t{%3, %2, %1, %0|%0, %1, %2, %3}";
}
}

static const char * const output_1726[] = {
  "roundss\t{%3, %2, %0|%0, %2, %3}",
  "vroundss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1727[] = {
  "roundsd\t{%3, %2, %0|%0, %2, %3}",
  "vroundsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char * const output_1731[] = {
  "%vpcmpestrm\t{%6, %4, %2|%2, %4, %6}",
  "%vpcmpestrm\t{%6, %4, %2|%2, %4, %6}",
  "%vpcmpestri\t{%6, %4, %2|%2, %4, %6}",
  "%vpcmpestri\t{%6, %4, %2|%2, %4, %6}",
};

static const char * const output_1735[] = {
  "%vpcmpistrm\t{%4, %3, %2|%2, %3, %4}",
  "%vpcmpistrm\t{%4, %3, %2|%2, %3, %4}",
  "%vpcmpistri\t{%4, %3, %2|%2, %3, %4}",
  "%vpcmpistri\t{%4, %3, %2|%2, %3, %4}",
};

static const char *
output_1783 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11169 "../../gcc/config/i386/sse.md"
{
  operands[3]
    = GEN_INT (GET_MODE_BITSIZE (QImode) - INTVAL (operands[2]));
  return "vprotb\t{%3, %1, %0|%0, %1, %3}";
}
}

static const char *
output_1784 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11169 "../../gcc/config/i386/sse.md"
{
  operands[3]
    = GEN_INT (GET_MODE_BITSIZE (HImode) - INTVAL (operands[2]));
  return "vprotw\t{%3, %1, %0|%0, %1, %3}";
}
}

static const char *
output_1785 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11169 "../../gcc/config/i386/sse.md"
{
  operands[3]
    = GEN_INT (GET_MODE_BITSIZE (SImode) - INTVAL (operands[2]));
  return "vprotd\t{%3, %1, %0|%0, %1, %3}";
}
}

static const char *
output_1786 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11169 "../../gcc/config/i386/sse.md"
{
  operands[3]
    = GEN_INT (GET_MODE_BITSIZE (DImode) - INTVAL (operands[2]));
  return "vprotq\t{%3, %1, %0|%0, %1, %3}";
}
}

static const char *
output_1819 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11539 "../../gcc/config/i386/sse.md"
{
  return ((INTVAL (operands[3]) != 0)
	  ? "vpcomtrueb\t{%2, %1, %0|%0, %1, %2}"
	  : "vpcomfalseb\t{%2, %1, %0|%0, %1, %2}");
}
}

static const char *
output_1820 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11539 "../../gcc/config/i386/sse.md"
{
  return ((INTVAL (operands[3]) != 0)
	  ? "vpcomtruew\t{%2, %1, %0|%0, %1, %2}"
	  : "vpcomfalsew\t{%2, %1, %0|%0, %1, %2}");
}
}

static const char *
output_1821 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11539 "../../gcc/config/i386/sse.md"
{
  return ((INTVAL (operands[3]) != 0)
	  ? "vpcomtrued\t{%2, %1, %0|%0, %1, %2}"
	  : "vpcomfalsed\t{%2, %1, %0|%0, %1, %2}");
}
}

static const char *
output_1822 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11539 "../../gcc/config/i386/sse.md"
{
  return ((INTVAL (operands[3]) != 0)
	  ? "vpcomtrueq\t{%2, %1, %0|%0, %1, %2}"
	  : "vpcomfalseq\t{%2, %1, %0|%0, %1, %2}");
}
}

static const char * const output_1827[] = {
  "aesenc\t{%2, %0|%0, %2}",
  "vaesenc\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1828[] = {
  "aesenclast\t{%2, %0|%0, %2}",
  "vaesenclast\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1829[] = {
  "aesdec\t{%2, %0|%0, %2}",
  "vaesdec\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1830[] = {
  "aesdeclast\t{%2, %0|%0, %2}",
  "vaesdeclast\t{%2, %1, %0|%0, %1, %2}",
};

static const char * const output_1833[] = {
  "pclmulqdq\t{%3, %2, %0|%0, %2, %3}",
  "vpclmulqdq\t{%3, %2, %1, %0|%0, %1, %2, %3}",
};

static const char *
output_1847 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11791 "../../gcc/config/i386/sse.md"
{
  int mask = 0;
  mask |= INTVAL (operands[2]) << 0;
  mask |= INTVAL (operands[3]) << 2;
  mask |= INTVAL (operands[4]) << 4;
  mask |= INTVAL (operands[5]) << 6;
  operands[2] = GEN_INT (mask);
  return "vpermq\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char * const output_1850[] = {
  "vbroadcastss\t{%1, %0|%0, %1}",
  "#",
};

static const char * const output_1851[] = {
  "vbroadcastss\t{%1, %0|%0, %1}",
  "#",
};

static const char * const output_1852[] = {
  "vbroadcastsd\t{%1, %0|%0, %1}",
  "#",
};

static const char * const output_1853[] = {
  "vbroadcastsd\t{%1, %0|%0, %1}",
  "#",
};

static const char * const output_1858[] = {
  "vbroadcast%~128\t{%1, %0|%0, %1}",
  "vinsert%~128\t{$1, %1, %0, %0|%0, %0, %1, 1}",
  "vperm2%~128\t{$0, %t1, %t1, %0|%0, %t1, %t1, 0}",
};

static const char * const output_1859[] = {
  "vbroadcast%~128\t{%1, %0|%0, %1}",
  "vinsert%~128\t{$1, %1, %0, %0|%0, %0, %1, 1}",
  "vperm2%~128\t{$0, %t1, %t1, %0|%0, %t1, %t1, 0}",
};

static const char * const output_1860[] = {
  "vbroadcast%~128\t{%1, %0|%0, %1}",
  "vinsert%~128\t{$1, %1, %0, %0|%0, %0, %1, 1}",
  "vperm2%~128\t{$0, %t1, %t1, %0|%0, %t1, %t1, 0}",
};

static const char * const output_1861[] = {
  "vbroadcast%~128\t{%1, %0|%0, %1}",
  "vinsert%~128\t{$1, %1, %0, %0|%0, %0, %1, 1}",
  "vperm2%~128\t{$0, %t1, %t1, %0|%0, %t1, %t1, 0}",
};

static const char * const output_1862[] = {
  "vbroadcastf128\t{%1, %0|%0, %1}",
  "vinsertf128\t{$1, %1, %0, %0|%0, %0, %1, 1}",
  "vperm2f128\t{$0, %t1, %t1, %0|%0, %t1, %t1, 0}",
};

static const char * const output_1863[] = {
  "vbroadcastf128\t{%1, %0|%0, %1}",
  "vinsertf128\t{$1, %1, %0, %0|%0, %0, %1, 1}",
  "vperm2f128\t{$0, %t1, %t1, %0|%0, %t1, %t1, 0}",
};

static const char *
output_1864 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 11895 "../../gcc/config/i386/sse.md"
{
  int elt = INTVAL (operands[3]);
  switch (which_alternative)
    {
    case 0:
    case 1:
      operands[1] = adjust_address_nv (operands[1], SFmode, elt * 4);
      return "vbroadcastss\t{%1, %0|%0, %1}";
    case 2:
      operands[2] = GEN_INT (elt * 0x55);
      return "vpermilps\t{%2, %1, %0|%0, %1, %2}";
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1867 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12008 "../../gcc/config/i386/sse.md"
{
  int mask = avx_vpermilp_parallel (operands[2], V8SFmode) - 1;
  operands[2] = GEN_INT (mask);
  return "vpermilps\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char *
output_1868 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12008 "../../gcc/config/i386/sse.md"
{
  int mask = avx_vpermilp_parallel (operands[2], V4SFmode) - 1;
  operands[2] = GEN_INT (mask);
  return "vpermilps\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char *
output_1869 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12008 "../../gcc/config/i386/sse.md"
{
  int mask = avx_vpermilp_parallel (operands[2], V4DFmode) - 1;
  operands[2] = GEN_INT (mask);
  return "vpermilpd\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char *
output_1870 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12008 "../../gcc/config/i386/sse.md"
{
  int mask = avx_vpermilp_parallel (operands[2], V2DFmode) - 1;
  operands[2] = GEN_INT (mask);
  return "vpermilpd\t{%2, %1, %0|%0, %1, %2}";
}
}

static const char *
output_1878 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12094 "../../gcc/config/i386/sse.md"
{
  int mask = avx_vperm2f128_parallel (operands[3], V8SImode) - 1;
  if (mask == 0x12)
    return "vinsert%~128\t{$0, %x2, %1, %0|%0, %1, %x2, 0}";
  if (mask == 0x20)
    return "vinsert%~128\t{$1, %x2, %1, %0|%0, %1, %x2, 1}";
  operands[3] = GEN_INT (mask);
  return "vperm2%~128\t{%3, %2, %1, %0|%0, %1, %2, %3}";
}
}

static const char *
output_1879 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12094 "../../gcc/config/i386/sse.md"
{
  int mask = avx_vperm2f128_parallel (operands[3], V8SFmode) - 1;
  if (mask == 0x12)
    return "vinsertf128\t{$0, %x2, %1, %0|%0, %1, %x2, 0}";
  if (mask == 0x20)
    return "vinsertf128\t{$1, %x2, %1, %0|%0, %1, %x2, 1}";
  operands[3] = GEN_INT (mask);
  return "vperm2f128\t{%3, %2, %1, %0|%0, %1, %2, %3}";
}
}

static const char *
output_1880 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12094 "../../gcc/config/i386/sse.md"
{
  int mask = avx_vperm2f128_parallel (operands[3], V4DFmode) - 1;
  if (mask == 0x12)
    return "vinsertf128\t{$0, %x2, %1, %0|%0, %1, %x2, 0}";
  if (mask == 0x20)
    return "vinsertf128\t{$1, %x2, %1, %0|%0, %1, %x2, 1}";
  operands[3] = GEN_INT (mask);
  return "vperm2f128\t{%3, %2, %1, %0|%0, %1, %2, %3}";
}
}

static const char *
output_1924 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12439 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return "vinsert%~128\t{$0x1, %2, %t1, %0|%0, %t1, %2, 0x1}";
    case 1:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	  return "vmovaps\t{%1, %x0|%x0, %1}";
	case MODE_V4DF:
	  return "vmovapd\t{%1, %x0|%x0, %1}";
	default:
	  return "vmovdqa\t{%1, %x0|%x0, %1}";
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1925 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12439 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return "vinsert%~128\t{$0x1, %2, %t1, %0|%0, %t1, %2, 0x1}";
    case 1:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	  return "vmovaps\t{%1, %x0|%x0, %1}";
	case MODE_V4DF:
	  return "vmovapd\t{%1, %x0|%x0, %1}";
	default:
	  return "vmovdqa\t{%1, %x0|%x0, %1}";
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1926 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12439 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return "vinsert%~128\t{$0x1, %2, %t1, %0|%0, %t1, %2, 0x1}";
    case 1:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	  return "vmovaps\t{%1, %x0|%x0, %1}";
	case MODE_V4DF:
	  return "vmovapd\t{%1, %x0|%x0, %1}";
	default:
	  return "vmovdqa\t{%1, %x0|%x0, %1}";
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1927 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12439 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return "vinsert%~128\t{$0x1, %2, %t1, %0|%0, %t1, %2, 0x1}";
    case 1:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	  return "vmovaps\t{%1, %x0|%x0, %1}";
	case MODE_V4DF:
	  return "vmovapd\t{%1, %x0|%x0, %1}";
	default:
	  return "vmovdqa\t{%1, %x0|%x0, %1}";
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1928 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12439 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return "vinsertf128\t{$0x1, %2, %t1, %0|%0, %t1, %2, 0x1}";
    case 1:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	  return "vmovaps\t{%1, %x0|%x0, %1}";
	case MODE_V4DF:
	  return "vmovapd\t{%1, %x0|%x0, %1}";
	default:
	  return "vmovdqa\t{%1, %x0|%x0, %1}";
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1929 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12439 "../../gcc/config/i386/sse.md"
{
  switch (which_alternative)
    {
    case 0:
      return "vinsertf128\t{$0x1, %2, %t1, %0|%0, %t1, %2, 0x1}";
    case 1:
      switch (get_attr_mode (insn))
	{
	case MODE_V8SF:
	  return "vmovaps\t{%1, %x0|%x0, %1}";
	case MODE_V4DF:
	  return "vmovapd\t{%1, %x0|%x0, %1}";
	default:
	  return "vmovdqa\t{%1, %x0|%x0, %1}";
	}
    default:
      gcc_unreachable ();
    }
}
}

static const char *
output_1984 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V2DImode != V2DImode)
    return "vpgatherqq\t{%4, %6, %x0|%x0, %6, %4}";
  return "vpgatherqq\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1985 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V2DImode != V2DImode)
    return "vpgatherqq\t{%4, %6, %x0|%x0, %6, %4}";
  return "vpgatherqq\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1986 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V2DFmode != V2DFmode)
    return "vgatherqpd\t{%4, %6, %x0|%x0, %6, %4}";
  return "vgatherqpd\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1987 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V2DFmode != V2DFmode)
    return "vgatherqpd\t{%4, %6, %x0|%x0, %6, %4}";
  return "vgatherqpd\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1988 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V4DImode != V4DImode)
    return "vpgatherqq\t{%4, %6, %x0|%x0, %6, %4}";
  return "vpgatherqq\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1989 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V4DImode != V4DImode)
    return "vpgatherqq\t{%4, %6, %x0|%x0, %6, %4}";
  return "vpgatherqq\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1990 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V4DFmode != V4DFmode)
    return "vgatherqpd\t{%4, %6, %x0|%x0, %6, %4}";
  return "vgatherqpd\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1991 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V4DFmode != V4DFmode)
    return "vgatherqpd\t{%4, %6, %x0|%x0, %6, %4}";
  return "vgatherqpd\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1992 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V4SImode != V4SImode)
    return "vpgatherqd\t{%4, %6, %x0|%x0, %6, %4}";
  return "vpgatherqd\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1993 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V4SImode != V4SImode)
    return "vpgatherqd\t{%4, %6, %x0|%x0, %6, %4}";
  return "vpgatherqd\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1994 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V4SFmode != V4SFmode)
    return "vgatherqps\t{%4, %6, %x0|%x0, %6, %4}";
  return "vgatherqps\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1995 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V4SFmode != V4SFmode)
    return "vgatherqps\t{%4, %6, %x0|%x0, %6, %4}";
  return "vgatherqps\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1996 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V8SImode != V4SImode)
    return "vpgatherqd\t{%4, %6, %x0|%x0, %6, %4}";
  return "vpgatherqd\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1997 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V8SImode != V4SImode)
    return "vpgatherqd\t{%4, %6, %x0|%x0, %6, %4}";
  return "vpgatherqd\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1998 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V8SFmode != V4SFmode)
    return "vgatherqps\t{%4, %6, %x0|%x0, %6, %4}";
  return "vgatherqps\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_1999 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 12679 "../../gcc/config/i386/sse.md"
{
  if (V8SFmode != V4SFmode)
    return "vgatherqps\t{%4, %6, %x0|%x0, %6, %4}";
  return "vgatherqps\t{%4, %6, %0|%0, %6, %4}";
}
}

static const char *
output_2019 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 301 "../../gcc/config/i386/sync.md"
{
  gcc_assert (find_regno_note (insn, REG_DEAD, FIRST_STACK_REG) != NULL_RTX);

  return "fistp%Z0\t%0";
}
}

static const char *
output_2024 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 425 "../../gcc/config/i386/sync.md"
{
  bool swap = REGNO (operands[5]) != BX_REG;

  if (swap)
    output_asm_insn ("xchg{l}\t%%ebx, %5", operands);

  output_asm_insn ("lock{%;} cmpxchg8b\t%2", operands);

  if (swap)
    output_asm_insn ("xchg{l}\t%%ebx, %5", operands);

  return "";
}
}

static const char *
output_2025 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 425 "../../gcc/config/i386/sync.md"
{
  bool swap = REGNO (operands[5]) != BX_REG;

  if (swap)
    output_asm_insn ("xchg{q}\t%%rbx, %5", operands);

  output_asm_insn ("lock{%;} cmpxchg16b\t%2", operands);

  if (swap)
    output_asm_insn ("xchg{q}\t%%rbx, %5", operands);

  return "";
}
}

static const char *
output_2030 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 498 "../../gcc/config/i386/sync.md"
{
  if (TARGET_USE_INCDEC)
    {
      if (operands[1] == const1_rtx)
	return "lock{%;} inc{b}\t%0";
      if (operands[1] == constm1_rtx)
	return "lock{%;} dec{b}\t%0";
    }

  if (x86_maybe_negate_const_int (&operands[1], QImode))
    return "lock{%;} sub{b}\t{%1, %0|%0, %1}";

  return "lock{%;} add{b}\t{%1, %0|%0, %1}";
}
}

static const char *
output_2031 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 498 "../../gcc/config/i386/sync.md"
{
  if (TARGET_USE_INCDEC)
    {
      if (operands[1] == const1_rtx)
	return "lock{%;} inc{w}\t%0";
      if (operands[1] == constm1_rtx)
	return "lock{%;} dec{w}\t%0";
    }

  if (x86_maybe_negate_const_int (&operands[1], HImode))
    return "lock{%;} sub{w}\t{%1, %0|%0, %1}";

  return "lock{%;} add{w}\t{%1, %0|%0, %1}";
}
}

static const char *
output_2032 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 498 "../../gcc/config/i386/sync.md"
{
  if (TARGET_USE_INCDEC)
    {
      if (operands[1] == const1_rtx)
	return "lock{%;} inc{l}\t%0";
      if (operands[1] == constm1_rtx)
	return "lock{%;} dec{l}\t%0";
    }

  if (x86_maybe_negate_const_int (&operands[1], SImode))
    return "lock{%;} sub{l}\t{%1, %0|%0, %1}";

  return "lock{%;} add{l}\t{%1, %0|%0, %1}";
}
}

static const char *
output_2033 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 498 "../../gcc/config/i386/sync.md"
{
  if (TARGET_USE_INCDEC)
    {
      if (operands[1] == const1_rtx)
	return "lock{%;} inc{q}\t%0";
      if (operands[1] == constm1_rtx)
	return "lock{%;} dec{q}\t%0";
    }

  if (x86_maybe_negate_const_int (&operands[1], DImode))
    return "lock{%;} sub{q}\t{%1, %0|%0, %1}";

  return "lock{%;} add{q}\t{%1, %0|%0, %1}";
}
}

static const char *
output_2038 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 535 "../../gcc/config/i386/sync.md"
{
  if (TARGET_USE_INCDEC)
    {
      if (operands[1] == const1_rtx)
	return "lock{%;} inc{b}\t%0";
      if (operands[1] == constm1_rtx)
	return "lock{%;} dec{b}\t%0";
    }

  if (x86_maybe_negate_const_int (&operands[1], QImode))
    return "lock{%;} sub{b}\t{%1, %0|%0, %1}";

  return "lock{%;} add{b}\t{%1, %0|%0, %1}";
}
}

static const char *
output_2039 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 535 "../../gcc/config/i386/sync.md"
{
  if (TARGET_USE_INCDEC)
    {
      if (operands[1] == const1_rtx)
	return "lock{%;} inc{w}\t%0";
      if (operands[1] == constm1_rtx)
	return "lock{%;} dec{w}\t%0";
    }

  if (x86_maybe_negate_const_int (&operands[1], HImode))
    return "lock{%;} sub{w}\t{%1, %0|%0, %1}";

  return "lock{%;} add{w}\t{%1, %0|%0, %1}";
}
}

static const char *
output_2040 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 535 "../../gcc/config/i386/sync.md"
{
  if (TARGET_USE_INCDEC)
    {
      if (operands[1] == const1_rtx)
	return "lock{%;} inc{l}\t%0";
      if (operands[1] == constm1_rtx)
	return "lock{%;} dec{l}\t%0";
    }

  if (x86_maybe_negate_const_int (&operands[1], SImode))
    return "lock{%;} sub{l}\t{%1, %0|%0, %1}";

  return "lock{%;} add{l}\t{%1, %0|%0, %1}";
}
}

static const char *
output_2041 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 535 "../../gcc/config/i386/sync.md"
{
  if (TARGET_USE_INCDEC)
    {
      if (operands[1] == const1_rtx)
	return "lock{%;} inc{q}\t%0";
      if (operands[1] == constm1_rtx)
	return "lock{%;} dec{q}\t%0";
    }

  if (x86_maybe_negate_const_int (&operands[1], DImode))
    return "lock{%;} sub{q}\t{%1, %0|%0, %1}";

  return "lock{%;} add{q}\t{%1, %0|%0, %1}";
}
}

static const char *
output_2042 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 559 "../../gcc/config/i386/sync.md"
{
  if (TARGET_USE_INCDEC)
    {
      if (operands[1] == const1_rtx)
	return "lock{%;} dec{b}\t%0";
      if (operands[1] == constm1_rtx)
	return "lock{%;} inc{b}\t%0";
    }

  if (x86_maybe_negate_const_int (&operands[1], QImode))
    return "lock{%;} add{b}\t{%1, %0|%0, %1}";

  return "lock{%;} sub{b}\t{%1, %0|%0, %1}";
}
}

static const char *
output_2043 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 559 "../../gcc/config/i386/sync.md"
{
  if (TARGET_USE_INCDEC)
    {
      if (operands[1] == const1_rtx)
	return "lock{%;} dec{w}\t%0";
      if (operands[1] == constm1_rtx)
	return "lock{%;} inc{w}\t%0";
    }

  if (x86_maybe_negate_const_int (&operands[1], HImode))
    return "lock{%;} add{w}\t{%1, %0|%0, %1}";

  return "lock{%;} sub{w}\t{%1, %0|%0, %1}";
}
}

static const char *
output_2044 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 559 "../../gcc/config/i386/sync.md"
{
  if (TARGET_USE_INCDEC)
    {
      if (operands[1] == const1_rtx)
	return "lock{%;} dec{l}\t%0";
      if (operands[1] == constm1_rtx)
	return "lock{%;} inc{l}\t%0";
    }

  if (x86_maybe_negate_const_int (&operands[1], SImode))
    return "lock{%;} add{l}\t{%1, %0|%0, %1}";

  return "lock{%;} sub{l}\t{%1, %0|%0, %1}";
}
}

static const char *
output_2045 (rtx *operands ATTRIBUTE_UNUSED, rtx insn ATTRIBUTE_UNUSED)
{
#line 559 "../../gcc/config/i386/sync.md"
{
  if (TARGET_USE_INCDEC)
    {
      if (operands[1] == const1_rtx)
	return "lock{%;} dec{q}\t%0";
      if (operands[1] == constm1_rtx)
	return "lock{%;} inc{q}\t%0";
    }

  if (x86_maybe_negate_const_int (&operands[1], DImode))
    return "lock{%;} add{q}\t{%1, %0|%0, %1}";

  return "lock{%;} sub{q}\t{%1, %0|%0, %1}";
}
}



static const struct insn_operand_data operand_data[] = 
{
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "q,?mq",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    const0_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,?mr",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    const0_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,?mr",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const0_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,?mr",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    const0_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "qm,q",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qn,qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm,r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "rn,rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "re,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "re,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "Qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    ext_register_operand,
    "Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "Q",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const0_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "Qmn",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    ext_register_operand,
    "Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "Qn",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const0_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "fm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "fm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    float_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    float_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f,x",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "f,xm",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    push_operand,
    "=<",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    general_no_elim_operand,
    "riF*o",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    push_operand,
    "=<",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    general_no_elim_operand,
    "riF*o",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    push_operand,
    "=<,!<",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    general_no_elim_operand,
    "re*m,n",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    push_operand,
    "=<",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    general_no_elim_operand,
    "ri*m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    push_operand,
    "=X",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_no_elim_operand,
    "rn",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    push_operand,
    "=X",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_no_elim_operand,
    "rn",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    push_operand,
    "=X",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_no_elim_operand,
    "re",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    push_operand,
    "=<",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    general_no_elim_operand,
    "re*m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    push_operand,
    "=<",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    general_no_elim_operand,
    "re*m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r*m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    pop_operand,
    ">",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r*m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    pop_operand,
    ">",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const0_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const0_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    OImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "C,xm,x",
    OImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=!r,o,x,x,m",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "riFo,re,C,xm,x",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "C,xm,x",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,r,r,m,*y,m*y,?*y,?r,?*Ym,*x,m,*x,*x,?r,?*Yi,?*x,?*Ym",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "Z,rem,i,re,C,*y,m,*Ym,r,C,*x,*x,m,*Yi,r,*Ym,*x",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,o,*y,m*y,*y,*x,m,*x,*x,*x,m,*x,*x,?*x,?*Ym",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "riFo,riF,C,*y,m,C,*x,*x,m,C,*x,*x,m,*Ym,*x",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,m,*y,*y,?rm,?*y,*x,*x,?r,m,?*Yi,*x",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "g,re,C,*y,*y,rm,C,*x,*Yi,*x,r,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,r,r,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "r,rn,rm,rn",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=q,q,q,r,r,?r,m",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "q,qn,qm,q,rn,qm,qn",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_movabs_operand,
    "i,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "a,rn",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_movabs_operand,
    "i,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "a,rn",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_movabs_operand,
    "i,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "a,re",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_movabs_operand,
    "i,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "a,re",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a,r",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_movabs_operand,
    "i,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a,r",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_movabs_operand,
    "i,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a,r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_movabs_operand,
    "i,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_movabs_operand,
    "i,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+r",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+r",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+r",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+r",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+q",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+q",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "+qm,q",
    QImode,
    1,
    0,
    1,
    1
  },
  {
    general_operand,
    "qn,m",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "+rm,r",
    HImode,
    1,
    0,
    1,
    1
  },
  {
    general_operand,
    "rn,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "+q",
    QImode,
    1,
    0,
    1,
    0
  },
  {
    const0_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+r",
    HImode,
    1,
    0,
    1,
    0
  },
  {
    const0_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=R",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=R",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=Q,?R",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "Q,Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=Qm,?r",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    ext_register_operand,
    "Q,Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=R",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=Qm,?R",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    ext_register_operand,
    "Q,Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "+Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "Qn",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "+Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "Qn",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "+Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "Qmn",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    ext_register_operand,
    "+Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "Q",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    push_operand,
    "=<,<,<",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    general_no_elim_operand,
    "x,Fo,*r",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    push_operand,
    "=<,<",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    general_no_elim_operand,
    "f,ro",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    push_operand,
    "=<,<",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    general_no_elim_operand,
    "f,*rFo",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    push_operand,
    "=<,<,<",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_no_elim_operand,
    "f,Yd*rFm,x",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    push_operand,
    "=<,<,<",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_no_elim_operand,
    "f,Yd*rFo,x",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    push_operand,
    "=X,X,X",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_no_elim_operand,
    "f,rF,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    push_operand,
    "=<,<,<",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_no_elim_operand,
    "f,rFm,x",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,m,x,?*r,!o",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "xm,x,C,*roF,*r",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,m,x",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "xm,x,C",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=f,m,f,?Yx*r,!o",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "fm,f,G,Yx*roF,Yx*rC",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=f,m,f,?Yx*r,!o",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "fm,f,G,Yx*roF,Yx*rF",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=?Yf*f,?m,?Yf*f,?r,?m,?r,?r,x,x,x,m,Yi,r",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "Yf*fm,Yf*f,G,rm,rC,C,F,C,x,m,x,r,Yi",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=Yf*f,m,Yf*f,?Yd*r,!o,x,x,x,m,*x,*x,*x,m",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "Yf*fm,Yf*f,G,Yd*roF,Yd*rF,C,x,m,x,C,*x,m,*x",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=Yf*f,m,Yf*f,?r,?m,x,x,x,m,!*y,!m,!*y,?Yi,?r,!*Ym,!r",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "Yf*fm,Yf*f,G,rmF,rF,C,x,m,x,m,*y,*y,r,Yi,r,*Ym",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "+f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    fp_register_operand,
    "+f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    fp_register_operand,
    "+f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    fp_register_operand,
    "+f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    fp_register_operand,
    "+f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=r,o,?*Ym,?!*y,?*Yi,*x",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm,0,r,m,r,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,?r,?o,?*Ym,?!*y,?*Yi,*x",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,rm,r,r,m,r,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,?&q",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,?&q",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "qm,0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "qm,0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=*a,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "*0,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=*A,r,?r,?*o",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,0,r,r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=X,X,X,&r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=*a,r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "*0,rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=*a,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "*0,rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=*a,r",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "*0,qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=f,m,x",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "fm,f,xm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=f,m",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "fm,f",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=f,m",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "fm,f",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=f,m",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "fm,f",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=fm,x",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "f,xm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=fm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "f",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=m,x,?f,?x,?*r",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "f,xm,f,f,f",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,X,m,m,m",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=m,?f,?x,?*r",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "f,f,f,f",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m,m,m",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=m,?f,?x,?*r",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f,f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=X,m,m,m",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=m,?f,?x,?*r",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f,f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=X,m,m,m",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=&x,&x",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=x,&x",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "m,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=&x,&x",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=x,&x",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "m,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&1f",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "=m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&1f",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "=m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&1f",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=m,?r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=X,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&1f,&1f",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=m,?r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=X,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&1f,&1f",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=m,?r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=X,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&1f,&1f",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "=m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&1f",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=m,?r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&1f,&1f",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "=m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=m,?r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=m,?r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f,f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f,f,x,x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r,r,m,!x",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m,m,X,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f,x,x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r,r,m,!x",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m,m,X,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f,x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r,r,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m,m,X",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f,x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r,r,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m,m,X",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f,x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r,r,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m,m,X",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f,x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r,r,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m,m,X",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,r,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,r,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,r,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,r,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m,!x",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m,X,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m,!x",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m,X,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m,X",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m,X",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m,X",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m,X",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=X,x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "=X,x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=X,x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "=X,x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=X,x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "=X,x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=X,x",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=X,x",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=X,x",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    lea_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    lea_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,o",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "ronF,rnF",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,o",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "roe,re",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "re,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "re,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=qm,q",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qn,qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,rm,r,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,r,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "rme,re,0,le",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,rm,r,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,r,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "rme,re,0,le",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,r,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "rme,0,le",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm,r,r,Yp",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,r,Yp",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "rn,rm,0,ln",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=qm,q,q,r,r,Yp",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,q,0,r,Yp",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qn,qm,0,rn,0,ln",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "+qm,q",
    QImode,
    1,
    0,
    1,
    1
  },
  {
    general_operand,
    "qn,qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=q,qm,q",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,q",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qmn,qn,0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,rm,r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "rmn,rn,0",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,rm,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "rme,re,0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,rm,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "rme,re,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "rme,0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=q,q",
    QImode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "%0,q",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qmn,0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=r,r",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "%0,r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "rmn,0",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=r,r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "%0,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "rme,0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=r,r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "%0,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "rme,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=rm",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_immediate_operand,
    "e",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=qm",
    QImode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "n",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=rm",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "0",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "n",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=rm",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "=Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "0",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "Qn",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "=Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "0",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "Qmn",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    ext_register_operand,
    "=Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "%0",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    index_register_operand,
    "l",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "i",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    index_register_operand,
    "l",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const248_operand,
    "n",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "ri",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    index_register_operand,
    "l",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const248_operand,
    "n",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "i",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    index_register_operand,
    "l",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "n",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "n",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=r,o",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "ronF,rnF",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,o",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "roe,re",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=qm,q",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qn,qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm,r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "rn,rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "re,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "re,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_general_operand,
    "rme",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=qm,q",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qn,qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    ix86_carry_flag_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=qm,q",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qn,qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    ix86_carry_flag_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=rm,r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "rn,rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    ix86_carry_flag_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=rm,r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "rn,rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    ix86_carry_flag_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=rm,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "re,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    ix86_carry_flag_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=rm,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "re,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    ix86_carry_flag_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=rm,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "re,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    ix86_carry_flag_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=rm,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "re,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    ix86_carry_flag_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "rme",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    ix86_carry_flag_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_general_operand,
    "rme",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    ix86_carry_flag_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    scratch_operand,
    "=q",
    QImode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qmn",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=r",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "rmn",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "rme",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "rme",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r,r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%rm,rm,0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "K,e,mr",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%rm,rm,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "K,e,mr",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%rm,rm,0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "K,e,mr",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r,r",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%rm,rm,0",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "K,n,mr",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=a",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%d",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%d",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=A,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,d",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=A,r",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,d",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=A",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=A",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=d",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%a",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=1",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=d",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%a",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=1",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=d",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%a",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=1",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=&d",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=&d",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=&d",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=d",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "1",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=d",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "1",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=d",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "1",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%!*a,r,!*a,r,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_szext_general_operand,
    "Z,Z,e,e,re",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%!*a,q,qm,r",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "n,n,qn,n",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%!*a,q,qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "n,n,qn",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%!*a,r,rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "n,n,rn",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%!*a,r,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "e,e,re",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    ext_register_operand,
    "Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "n",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "Qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "rm",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=r,rm,r,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,0,qm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_szext_general_operand,
    "Z,re,rm,L",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm,r,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,qm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "re,rm,L",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm,r,r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,qm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "rn,rm,L",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=qm,q,r",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qn,qmn,rn",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "+qm,q",
    QImode,
    1,
    0,
    1,
    1
  },
  {
    general_operand,
    "qn,qmn",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,r,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_szext_general_operand,
    "Z,rem,re",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=q,qm,*r",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qmn,qn,n",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=q,qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qmn,qn",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "rmn,rn",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "rme,re",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "+q,qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "qmn,qn",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    ext_register_operand,
    "=Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "0",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "n",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "=Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "0",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "=Q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "0",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "Qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "rme,re",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=q,m,r",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "%0,0,0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qmn,qn,rn",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "%0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_zext_immediate_operand,
    "Z",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "+q,m",
    QImode,
    1,
    0,
    1,
    1
  },
  {
    general_operand,
    "qmn,qn",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_zext_immediate_operand,
    "Z",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "+q,qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "qmn,qn",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    ext_register_operand,
    "=q",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "0",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "qmn",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=ro",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=ro",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x,f,!r",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x,0,0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,0,X,X",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    absneg_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x,f,!r",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x,0,0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,0,X,X",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    absneg_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x,!r",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x,0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "xm,0,X",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    absneg_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x,!r",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x,0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "xm,0,X",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    absneg_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,!r",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    absneg_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,!r",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    absneg_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,!r",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    absneg_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,0",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    absneg_operator,
    "",
    TFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    vector_move_operand,
    "xmC",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    vector_move_operand,
    "xmC",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    vector_move_operand,
    "xmC",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=x,x,x,x,x",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "x,0,0,x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1,1,x,1,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "X,xm,xm,0,0",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,xm,1,xm,1",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=x,x,x,x,x",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "x,0,0,x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1,1,x,1,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "X,xm,xm,0,0",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,xm,1,xm,1",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,x,x",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=x,x,x,x,x",
    TFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "x,0,0,x,x",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1,1,x,1,x",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "X,xm,xm,0,0",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,xm,1,xm,1",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=qm,r",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=&r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    reg_or_pm1_operand,
    "n,0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "Jc,Jc",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=&r,r",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    reg_or_pm1_operand,
    "n,0",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "Oc,Oc",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "+r*m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "Jc",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "+r*m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "Ic",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm,r,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,l,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cI,M,r",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm,r,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,l,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cJ,M,r",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r,r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,l,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cI,M,r",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm,Yp",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,l",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cI,M",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=qm,r,Yp",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0,l",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cI,cI,M",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "+qm",
    QImode,
    1,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cI",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    const_1_to_31_operand,
    "I",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    const_1_to_31_operand,
    "I",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_1_to_31_operand,
    "I",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    const_1_to_63_operand,
    "J",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_1_to_31_operand,
    "I",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=q",
    QImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_1_to_31_operand,
    "I",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=r",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_1_to_31_operand,
    "I",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_1_to_31_operand,
    "I",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_1_to_63_operand,
    "J",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "Jc",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "Oc",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=*d,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "*a,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=*d,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "*a,0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=*d,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "*a,0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cI,r",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cJ,r",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cI,r",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cI",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cI",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_1_to_31_operand,
    "I",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=r",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    const_1_to_63_operand,
    "J",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "I",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "J",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cI,I",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cJ,J",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "I",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "cI,I",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_63_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_nonmemory_operand,
    "rN",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_nonmemory_operand,
    "rN",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=q",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_operator,
    "",
    DImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=q",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_operator,
    "",
    SImode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    ix86_comparison_operator,
    "",
    QImode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "+qm",
    QImode,
    1,
    0,
    1,
    1
  },
  {
    ix86_comparison_operator,
    "",
    QImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    sse_comparison_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    sse_comparison_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    bt_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    bt_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    bt_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    bt_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "fm",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=a",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=a",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const0_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=a",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    ix86_swapped_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    float_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=a,a",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    ix86_swapped_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    float_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=a,a",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    indirect_branch_operand,
    "rw",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    indirect_branch_operand,
    "rw",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    indirect_branch_operand,
    "rw",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    indirect_branch_operand,
    "rw",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    call_insn_operand,
    "lzw",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    call_insn_operand,
    "rzw",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    sibcall_insn_operand,
    "Uz",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    sibcall_insn_operand,
    "Uz",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    call_insn_operand,
    "lzm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    call_insn_operand,
    "lzm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    sibcall_insn_operand,
    "Uz",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    call_insn_operand,
    "rzw",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    sibcall_insn_operand,
    "Uz",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    sibcall_insn_operand,
    "Uz",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    call_insn_operand,
    "lzm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    sibcall_insn_operand,
    "Uz",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    BLKmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&q",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=r",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "n",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=r,r,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,m,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=r,r,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,m,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "+Q,r",
    HImode,
    1,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "=&r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "=Q",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "=&Q",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=Q",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "b",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    constant_call_address_operand,
    "z",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=d",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "=c",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    constant_call_address_operand,
    "z",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "b",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    constant_call_address_operand,
    "z",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=d",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "=c",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    constant_call_address_operand,
    "z",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "b",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    constant_call_address_operand,
    "z",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=d",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "=c",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "b",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "b",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=&a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "b",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    tls_modbase_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=&a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    tls_modbase_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f,x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,0,x",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "fm,xm,xm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,0,x",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "fm,xm,xm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "fm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "fm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f,x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,fm,0,x",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "fm,0,xm,xm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f,x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,fm,0,x",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "fm,0,xm,xm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,fm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "fm,0",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,fm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "fm,0",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "fm,0",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "fm,0",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "fm,0",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "%0",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f,0",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,0",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,0",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "fm,0",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "fm,0",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "fm,0",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "fm,0",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "fm,0",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "fm,0",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    binary_fp_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=u",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=u",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=u",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=u",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    const_double_operand,
    "F",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=u",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_double_operand,
    "F",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=u",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    const_double_operand,
    "F",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "u",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=2",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "u",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=2",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "u",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=2",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "u",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=2",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "u",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=2",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_15_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_15_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&1f",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=m,?r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=X,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&1f,&1f",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "=m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&1f",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=m,?r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&1f,&1f",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "=m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=m,?r",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=m,?r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=S",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=S",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=S",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "2",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=S",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "2",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=S",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "2",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=S",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "2",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=&c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=&c",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=D",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "i",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_carry_flag_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_carry_flag_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=r,r",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "rm,0",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "rm,0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "rm,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "r,0",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,r",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    fcmov_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "f,0",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,f",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=f,f,r,r",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    fcmov_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "f,0,rm,0",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,f,0,rm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f,&r,&r",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    fcmov_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "f,0,rm,0",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,f,0,rm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=f,f,r,r",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    fcmov_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "f,0,rm,0",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,f,0,rm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r,r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_nonmemory_operand,
    "re,le",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_nonmemory_operand,
    "re,le",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "n",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    address_operand,
    "p",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    address_operand,
    "p",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "=m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "=m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "=m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    flags_reg_operand,
    "",
    CCZmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    flags_reg_operand,
    "",
    CCZmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    flags_reg_operand,
    "",
    CCZmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    flags_reg_operand,
    "",
    CCZmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=A",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=d",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=A",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=d",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm,r,!?y,!y,!?y,m,!y,*x,x,x,m,r,Yi",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "Cr,m,C,!y,m,!?y,*x,!y,C,xm,x,Yi,r",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm,r,!?y,!y,!?y,m,!y,*x,x,x,m,r,Yi",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "Cr,m,C,!y,m,!?y,*x,!y,C,xm,x,Yi,r",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm,r,!?y,!y,!?y,m,!y,*x,x,x,m,r,Yi",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "Cr,m,C,!y,m,!?y,*x,!y,C,xm,x,Yi,r",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm,r,!?y,!y,!?y,m,!y,*x,x,x,m,r,Yi",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "Cr,m,C,!y,m,!?y,*x,!y,C,xm,x,Yi,r",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=!?y,!y,!?y,m,!y,*x,*x,*x,m,*x,*x,*x,m,r,m",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "C,!y,m,!?y,*x,!y,C,*xm,*x,C,*x,m,*x,irm,r",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=!?y,!y,!?y,m,!y,*x,*x,*x,m,*x,*x,*x,m,r,m",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "C,!y,m,!?y,*x,!y,C,*xm,*x,C,*x,m,*x,irm,r",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=!?y,!y,!?y,m,!y,*x,*x,*x,m,*x,*x,*x,m,r,m",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "C,!y,m,!?y,*x,!y,C,*xm,*x,C,*x,m,*x,irm,r",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=!?y,!y,!?y,m,!y,*x,*x,*x,m,*x,*x,*x,m,r,m",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "C,!y,m,!?y,*x,!y,C,*xm,*x,C,*x,m,*x,irm,r",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm,r,!?y,!y,!?y,m,!y,*x,x,x,x,m,r,Yi",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "Cr,m,C,!y,m,!?y,*x,!y,C,x,m,x,Yi,r",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=!?y,!y,!?y,m,!y,*x,*x,*x,*x,m,r,m",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "C,!y,m,!?y,*x,!y,C,*x,m,*x,irm,r",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "y",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=y",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "ym",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y,y",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,ym",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "ym,0",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "ym",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=y,y",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,rm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "ym,C",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,m,y,m,f,r",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x,ym,y,m,m",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=y,x,y,x,f,r",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0,o,o,o,o",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "ym",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "ym",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "ym",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "ym",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "ym",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "ym",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V1DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "ym",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V1DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "ym",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "ym",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V1DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "ym",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "yN",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "yN",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=y",
    V1DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V1DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "yN",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=y",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "y",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=y",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=y,y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "ym,C",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,m,y,m,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x,ym,y,m",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=y,x,x,x,y,x,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,0,x,0,o,o,o",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V1DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "y",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "D",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "y",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "y",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "D",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "y",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "y",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    emms_operation,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V2TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V2TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V1TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V1TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_or_sse_const_operand,
    "C,xm,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=?x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=&x,X",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x,x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,xm,x,m",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,0,xm,x",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    absneg_operator,
    "",
    V8SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x,x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,xm,x,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,0,xm,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    absneg_operator,
    "",
    V4SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x,x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,xm,x,m",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,0,xm,x",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    absneg_operator,
    "",
    V4DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x,x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,xm,x,m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,0,xm,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    absneg_operator,
    "",
    V2DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_31_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_31_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_31_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_31_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "%0,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    sse_comparison_operator,
    "",
    V8SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "%0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    sse_comparison_operator,
    "",
    V4SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "%0,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    sse_comparison_operator,
    "",
    V4DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "%0,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    sse_comparison_operator,
    "",
    V2DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    sse_comparison_operator,
    "",
    V8SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    sse_comparison_operator,
    "",
    V4SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    sse_comparison_operator,
    "",
    V4DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    sse_comparison_operator,
    "",
    V2DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "%0,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "%0,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x,x,x,x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,0,x,x,x",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x,xm,x,m",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,xm,0,xm,x",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,x,x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,0,x,x,x",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x,xm,x,m",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,xm,0,xm,x",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,0,x,x,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x,xm,x,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,xm,0,xm,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,0,x,x,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x,xm,x,m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,xm,0,xm,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,0,x,x,x",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x,xm,x,m",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,xm,0,xm,x",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,0,x,x,x",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x,xm,x,m",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,xm,0,xm,x",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,0",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,0",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const0_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const0_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "y,m",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m,rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r,r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    const0_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const0_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m,xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m,xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const0_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,x,x,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,x,0,x,0",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,x,o,o,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,x,x,o",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,x,0,x,0",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,x,m,m,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_8_to_11_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_8_to_11_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_12_to_15_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_12_to_15_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=m,x,x",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,x,o",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,x,x,o",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,x,0,x,0",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "m,m,x,x,x",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=m,x,x",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,x,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,x,x,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,x,0,x,0",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,x,m,m,x",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m,0",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,x,x,*y,*y",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x,0,x,m,0,m",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "x,x,m,m,C,*ym,C",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,*y,*y",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,m,0,m",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    reg_or_0_operand,
    "x,C,*y,C",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x,x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x,0,x",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,x,m,m",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,x,x,x,x,x,x,m,m,m",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "C,C,C,C,0,x,0,x,0,0,0",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "x,m,*r,m,x,x,*rm,*rm,!x,!*re,!*fF",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,x,x,x,x,x,x,m,m,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "C,C,C,C,0,x,0,x,0,0,0",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "x,m,*r,m,x,x,*rm,*rm,!x,!*re,!*fF",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=x,m,f,r",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x,m,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=rm,x,x",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x,0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "n,n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,*r,f",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "o,o,o",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "n,n,n",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=x,m",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,m",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x,x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=x,m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=x,m",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,m",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x,x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=x,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=x,m",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,m",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x,x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=x,x,x,x,x,m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,x,o,o,o,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,x,1,0,x,0",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,1",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,x,x,x,o",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,x,m,0,x,0",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,x,1,m,m,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_1_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_5_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_2_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_6_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_1_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_2_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_1_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_2_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=m,x,x,x,*f,r",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,0,x,o,o,o",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=m,x,x",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,x,o",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=m,x,x,*f,r",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,x,m,m,m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=m,x,x",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,x,m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,x,x,o,o,o",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,x,0,x,0,0,0",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "m,m,x,x,x,*f,r",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,x,x,x,x,x,x,m,m,m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "C,0,x,0,x,x,o,o,0,0,0",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "m,m,m,x,x,0,0,x,x,*f,r",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,x,x,x,m,x,x,x,o",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "0,x,0,x,0,x,o,o,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,x,m,m,x,0,0,x,0",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,xm",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,x,x,x,x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x,m,0,x,m,0,0",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "x,x,1,m,m,C,x,m",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "xN,xN",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "xN,xN",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "xN,xN",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "xN,xN",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "xN,xN",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "xN,xN",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V2TImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V2TImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_mul_8_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V1TImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V1TImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_mul_8_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0,x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m,r,m",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x,x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0,x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m,r,m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x,x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0,x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m,r,m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x,x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,0,x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "r,m,r,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_15_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_15_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_15_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_7_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_7_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_7_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=rm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_1_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_8_to_11_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_8_to_11_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_8_to_11_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_8_to_11_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_12_to_15_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_12_to_15_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_12_to_15_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_12_to_15_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_4_to_7_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,Yi,x,x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    reg_or_0_operand,
    "C,C,C,0,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,r,m,x,x",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=xm,r",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x,Yi",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "o",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=xm,*r,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,Yi,o",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=xm",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=m,x,x,x,r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,0,x,o,o",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=m,x,x,x,x,x",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,0,x,o,x,o",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m,0",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x,m,0",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,x,x,*y,*y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x,0,x,rm,0,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "rm,rm,x,x,C,*ym,C",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,*y,*y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,rm,0,rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    reg_or_0_operand,
    "x,C,*y,C",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x,*y,*y",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,m,0,*rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    reg_or_0_operand,
    "x,C,*y,C",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x,x,x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x,0,0,x",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,x,x,m,m",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x,Yi,!x,x,x,x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "0,x,xm,r,*y,0,x,0,x",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "rm,rm,C,C,C,x,x,m,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,?x,x,x,x,x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,*y,0,x,0,0,x",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    vector_move_operand,
    "C,C,x,x,x,m,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "D",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "D",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "d",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "d",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2TImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V2TImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2TImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_mul_8_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_mul_8_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=y",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_mul_8_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=y",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "ym",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_15_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_15_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand_maybe_avx,
    "=x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand_maybe_avx,
    "0,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimm_not_xmm0_operand_maybe_avx,
    "xm,xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "Yz,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand_maybe_avx,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand_maybe_avx,
    "0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimm_not_xmm0_operand_maybe_avx,
    "xm,xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "Yz,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand_maybe_avx,
    "=x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand_maybe_avx,
    "0,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimm_not_xmm0_operand_maybe_avx,
    "xm,xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "Yz,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand_maybe_avx,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand_maybe_avx,
    "0,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimm_not_xmm0_operand_maybe_avx,
    "xm,xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "Yz,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%0,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    reg_not_xmm0_operand_maybe_avx,
    "=x,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand_maybe_avx,
    "0,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimm_not_xmm0_operand_maybe_avx,
    "xm,xm",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "Yz,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand_maybe_avx,
    "=x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand_maybe_avx,
    "0,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimm_not_xmm0_operand_maybe_avx,
    "xm,xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "Yz,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    avx2_pblendw_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_15_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_15_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_15_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_15_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_15_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_15_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c,c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=Yz,Yz",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand,
    "x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a,a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimm_not_xmm0_operand,
    "x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "d,d",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c,c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a,a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "d,d",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=Yz,Yz",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a,a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "d,d",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=Yz,Yz,X,X",
    V16QImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "=X,X,c,c",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "x,x,x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "a,a,a,a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m,x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "d,d,d,d",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "n,n,n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c,c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=Yz,Yz",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand,
    "x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimm_not_xmm0_operand,
    "x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=c,c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=Yz,Yz",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=Yz,Yz,X,X",
    V16QImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "=X,X,c,c",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "x,x,x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m,x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n,n,n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "x,m",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_7_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_15_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_31_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_63_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,x",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "x,m",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const0_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const0_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_int_operator,
    "",
    V16QImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_int_operator,
    "",
    V8HImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_int_operator,
    "",
    V4SImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_int_operator,
    "",
    V2DImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_uns_operator,
    "",
    V16QImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_uns_operator,
    "",
    V8HImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_uns_operator,
    "",
    V4SImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_uns_operator,
    "",
    V2DImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "%x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "0,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm,xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n,n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vzeroall_operation,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "=x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "xm",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?x",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?x",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?x",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,?x",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,0,?x",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,0,?x",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,0,?x",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,0,?x",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,0,?x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,0,?x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,o,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    avx_vbroadcast_operand,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    const_int_operand,
    "C,n,n",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,o,?x",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    avx_vbroadcast_operand,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    const_int_operand,
    "C,n,n",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x,x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "m,o,?x",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    avx_vbroadcast_operand,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    const_int_operand,
    "C,n,n",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=x,m",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,m",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "=x,m",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "xm,x",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    vector_move_operand,
    "xm,C",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    vector_move_operand,
    "xm,C",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    vector_move_operand,
    "xm,C",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    vector_move_operand,
    "xm,C",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    vector_move_operand,
    "xm,C",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x,x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x,x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    vector_move_operand,
    "xm,C",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=x",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "N",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const0_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "N",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=xm",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "N",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=&x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4DFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    DFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "0",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SImode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    register_operand,
    "=&x",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=&x",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    vsib_address_operand,
    "p",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand,
    "n",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_mem_operator,
    "",
    SFmode,
    0,
    1,
    0,
    1
  },
  {
    nonimmediate_operand,
    "=x,m,?r",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "m,m,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=X,X,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=X,xf,xf",
    DFmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "=m",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "qn",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "rn",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_nonmemory_operand,
    "re",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_nonmemory_operand,
    "re",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=m,m,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "x,m,?r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "=X,X,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "=X,xf,xf",
    DFmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "=m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "f",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "q",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=a,a",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=d,d",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    cmpxchg8b_pic_memory_operand,
    "+m,m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1,1",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "b,!*r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "c,c",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=X,&5",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=a,a",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=d,d",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    cmpxchg8b_pic_memory_operand,
    "+m,m",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0,0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "1,1",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "b,!*r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "c,c",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=X,&5",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "=q",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "0",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "0",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "i",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "i",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "i",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=q",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "0",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "qn",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "rn",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "re",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "+m",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "re",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    ext_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonmemory_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonmemory_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    cmp_fp_expander_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    cmp_fp_expander_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    cmp_fp_expander_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    cmp_fp_expander_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    cmp_fp_expander_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    cmp_fp_expander_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    cmp_fp_expander_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    cmp_fp_expander_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const0_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const0_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const0_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    float_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    float_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    push_operand,
    "",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    push_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    push_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    OImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    OImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    CDImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    CDImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "=m",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "r",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=&r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "=r",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "m",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "=r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    1,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    1,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    ext_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    push_operand,
    "",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    sse_reg_operand,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    push_operand,
    "",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    push_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    any_fp_register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    push_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    any_fp_register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    push_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    push_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    any_fp_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    fp_register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    fp_register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    push_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    push_operand,
    "",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    push_operand,
    "",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    fp_register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    scratch_operand,
    "",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    scratch_operand,
    "",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "x",
    DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "x",
    DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "x",
    SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "x",
    SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    VOIDmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    VOIDmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    VOIDmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    VOIDmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    VOIDmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    VOIDmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    VOIDmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    VOIDmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    fp_register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    fp_register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    fp_register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    fp_register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    fp_register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    lea_address_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    lea_address_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_nonmemory_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_nonmemory_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_nonmemory_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    index_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    index_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const248_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    index_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const248_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    index_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_carry_flag_operator,
    "",
    QImode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_carry_flag_operator,
    "",
    HImode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_carry_flag_operator,
    "",
    SImode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_carry_flag_operator,
    "",
    DImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_general_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_nonmemory_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_szext_general_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    ext_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    compare_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    compare_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_szext_general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_szext_general_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    fp_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    absneg_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    absneg_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    absneg_operator,
    "",
    SFmode,
    0,
    1,
    0,
    0
  },
  {
    0,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    absneg_operator,
    "",
    DFmode,
    0,
    1,
    0,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    absneg_operator,
    "",
    XFmode,
    0,
    1,
    0,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    vector_move_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    vector_move_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    vector_move_operand,
    "",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    TFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    compare_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    compare_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    compare_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    compare_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    compare_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    shiftdi_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    ashldi_input_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    reg_or_pm1_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    index_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    index_register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    shiftdi_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    shiftdi_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_1_to_31_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    const_1_to_63_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const8_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const8_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    ext_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const8_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const8_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_63_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_operator,
    "",
    DImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_operator,
    "",
    SImode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    ix86_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    QImode,
    1,
    0,
    1,
    1
  },
  {
    ix86_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    bt_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    bt_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    bt_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    bt_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=a",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    ix86_swapped_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    float_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=a",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    ix86_swapped_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    float_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=a",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    ix86_swapped_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    float_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=a",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    ix86_swapped_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    float_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "=a",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    indirect_branch_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_operator,
    "",
    QImode,
    0,
    1,
    0,
    0
  },
  {
    q_regs_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    call_insn_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    call_insn_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    sibcall_insn_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    sibcall_insn_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    0,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    call_insn_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    sibcall_insn_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    call_insn_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    sibcall_insn_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    sibcall_insn_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    0,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    call_insn_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    sibcall_insn_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    constant_call_address_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    constant_call_address_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    constant_call_address_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    constant_call_address_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    constant_call_address_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    tls_modbase_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    tls_symbolic_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    tls_modbase_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    scratch_operand,
    "",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    scratch_operand,
    "",
    XFmode,
    0,
    0,
    0,
    0
  },
  {
     register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
     register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    XFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    VOIDmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    VOIDmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    VOIDmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    VOIDmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    BLKmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    BLKmode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    BLKmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    BLKmode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    BLKmode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    BLKmode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    BLKmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    BLKmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    BLKmode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    BLKmode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_general_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_general_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_carry_flag_operator,
    "",
    SImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_carry_flag_operator,
    "",
    DImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    ix86_fp_comparison_operator,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    XFmode,
    0,
    0,
    1,
    0
  },
  {
    register_and_not_any_fp_reg_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    fcmov_comparison_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    fp_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    fp_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    binary_fp_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    memory_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    ordered_comparison_operator,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    aligned_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    promotable_binary_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    compare_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    aligned_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    compare_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    aligned_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    push_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "q",
    QImode,
    0,
    0,
    0,
    0
  },
  {
    push_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "r",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    push_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    push_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    push_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "r",
    SFmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "q",
    QImode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    compare_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    compare_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    compare_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    immediate_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    flags_reg_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    compare_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    ext_register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    arith_or_logical_operator,
    "",
    SImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    commutative_operator,
    "",
    SImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    commutative_operator,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    arith_or_logical_operator,
    "",
    SImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    plusminuslogic_operator,
    "",
    QImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    plusminuslogic_operator,
    "",
    HImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_nonmemory_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    plusminuslogic_operator,
    "",
    SImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    x86_64_nonmemory_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    plusminuslogic_operator,
    "",
    DImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    plusminuslogic_operator,
    "",
    QImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    plusminuslogic_operator,
    "",
    HImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    plusminuslogic_operator,
    "",
    SImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    plusminuslogic_operator,
    "",
    DImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    plusminuslogic_operator,
    "",
    SImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    plusminuslogic_operator,
    "",
    SImode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    1,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    const_int_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    const_int_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    scratch_operand,
    "r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const359_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const359_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const359_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    const359_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    immediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    HImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_general_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "r",
    SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    x86_64_general_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "r",
    DImode,
    0,
    0,
    0,
    0
  },
  {
    address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "r",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "=qm",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "r",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "rm",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "i",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V1DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V1DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V1DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V1DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V1TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V1TImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    zero_extended_scalar_load_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    zero_extended_scalar_load_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2TImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V1TImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    absneg_operator,
    "",
    V8SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    absneg_operator,
    "",
    V4SFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    absneg_operator,
    "",
    V4DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    absneg_operator,
    "",
    V2DFmode,
    0,
    1,
    0,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonmemory_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_3_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_1_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_1_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_1_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_1_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_1_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_1_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "x",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "xm",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_1_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    DFmode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_mul_8_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_mul_8_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_mul_8_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_mul_8_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    general_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    0,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    TFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    TFmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_3_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_15_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_15_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_15_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_15_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimm_not_xmm0_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    reg_not_xmm0_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimm_not_xmm0_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    general_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    nonmemory_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    avx_vbroadcast_operand,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    avx_vbroadcast_operand,
    "",
    VOIDmode,
    0,
    1,
    0,
    0
  },
  {
    const_int_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V16QImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_1_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_1_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_1_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_1_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_1_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    1
  },
  {
    const_0_to_1_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V32QImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V16HImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    0,
    "",
    VOIDmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const_0_to_255_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V2DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V4DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V4DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V2DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V2DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V2DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V4DImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4DFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V4DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V4SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V2DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V4SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V8SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SImode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V8SImode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    V8SFmode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    vsib_address_operand,
    "",
    VOIDmode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    V4DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    V4SFmode,
    0,
    0,
    1,
    0
  },
  {
    const1248_operand ,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    scratch_operand,
    "",
    V8SFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    nonimmediate_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    DFmode,
    0,
    0,
    0,
    0
  },
  {
    memory_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    scratch_operand,
    "",
    DFmode,
    0,
    0,
    0,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    TImode,
    0,
    0,
    1,
    1
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    TImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    QImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    QImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    HImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    HImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    SImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
  {
    register_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    memory_operand,
    "",
    DImode,
    0,
    0,
    1,
    1
  },
  {
    const_int_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    DImode,
    0,
    0,
    1,
    0
  },
  {
    const_int_operand,
    "",
    SImode,
    0,
    0,
    1,
    0
  },
};


#if GCC_VERSION >= 2007
__extension__
#endif

const struct insn_data_d insn_data[] = 
{
  /* ../../gcc/config/i386/i386.md:966 */
  {
    "*cmpqi_ccno_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_0 },
#else
    { 0, output_0, 0 },
#endif
    0,
    &operand_data[1],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:966 */
  {
    "*cmphi_ccno_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1 },
#else
    { 0, output_1, 0 },
#endif
    0,
    &operand_data[3],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:966 */
  {
    "*cmpsi_ccno_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_2 },
#else
    { 0, output_2, 0 },
#endif
    0,
    &operand_data[5],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:966 */
  {
    "*cmpdi_ccno_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_3 },
#else
    { 0, output_3, 0 },
#endif
    0,
    &operand_data[7],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:978 */
  {
    "*cmpqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cmp{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[9],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:978 */
  {
    "*cmphi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cmp{w}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[11],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:978 */
  {
    "*cmpsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cmp{l}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[13],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:978 */
  {
    "*cmpdi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cmp{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[15],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:987 */
  {
    "*cmpqi_minus_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cmp{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[9],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:987 */
  {
    "*cmphi_minus_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cmp{w}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[11],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:987 */
  {
    "*cmpsi_minus_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cmp{l}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[13],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:987 */
  {
    "*cmpdi_minus_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cmp{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[15],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:998 */
  {
    "*cmpqi_ext_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cmp{b}\t{%h1, %0|%0, %h1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[17],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1012 */
  {
    "*cmpqi_ext_1_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cmp{b}\t{%h1, %0|%0, %h1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[19],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1026 */
  {
    "*cmpqi_ext_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "test{b}\t%h0, %h0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[20],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1051 */
  {
    "*cmpqi_ext_3_insn",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cmp{b}\t{%1, %h0|%h0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[22],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1066 */
  {
    "*cmpqi_ext_3_insn_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cmp{b}\t{%1, %h0|%h0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[24],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1081 */
  {
    "*cmpqi_ext_4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cmp{b}\t{%h1, %h0|%h0, %h1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[26],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1204 */
  {
    "*cmpfp_0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_18 },
#else
    { 0, 0, output_18 },
#endif
    0,
    &operand_data[28],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:1224 */
  {
    "*cmpfp_0_cc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[28],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1252 */
  {
    "*cmpfp_xf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_20 },
#else
    { 0, 0, output_20 },
#endif
    0,
    &operand_data[31],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:1265 */
  {
    "*cmpfp_xf_cc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[31],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1286 */
  {
    "*cmpfp_sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_22 },
#else
    { 0, 0, output_22 },
#endif
    0,
    &operand_data[34],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:1286 */
  {
    "*cmpfp_df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_23 },
#else
    { 0, 0, output_23 },
#endif
    0,
    &operand_data[37],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:1299 */
  {
    "*cmpfp_sf_cc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[34],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1299 */
  {
    "*cmpfp_df_cc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[37],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1320 */
  {
    "*cmpfp_u",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_26 },
#else
    { 0, 0, output_26 },
#endif
    0,
    &operand_data[40],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:1340 */
  {
    "*cmpfp_u_cc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[40],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1368 */
  {
    "*cmpfp_hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_28 },
#else
    { 0, 0, output_28 },
#endif
    0,
    &operand_data[43],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:1368 */
  {
    "*cmpfp_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_29 },
#else
    { 0, 0, output_29 },
#endif
    0,
    &operand_data[47],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:1385 */
  {
    "*cmpfp_hi_cc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[43],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1385 */
  {
    "*cmpfp_si_cc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[47],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1415 */
  {
    "x86_fnstsw_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fnstsw\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_x86_fnstsw_1,
    &operand_data[28],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1428 */
  {
    "x86_sahf_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_33 },
#else
    { 0, 0, output_33 },
#endif
    (insn_gen_fn) gen_x86_sahf_1,
    &operand_data[51],
    1,
    1,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:1450 */
  {
    "*cmpfp_i_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_34 },
#else
    { 0, 0, output_34 },
#endif
    0,
    &operand_data[52],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:1479 */
  {
    "*cmpfp_i_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_35 },
#else
    { 0, 0, output_35 },
#endif
    0,
    &operand_data[54],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:1502 */
  {
    "*cmpfp_i_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_36 },
#else
    { 0, 0, output_36 },
#endif
    0,
    &operand_data[41],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:1523 */
  {
    "*cmpfp_iu_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_37 },
#else
    { 0, 0, output_37 },
#endif
    0,
    &operand_data[52],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:1552 */
  {
    "*cmpfp_iu_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_38 },
#else
    { 0, 0, output_38 },
#endif
    0,
    &operand_data[54],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:1575 */
  {
    "*cmpfp_iu_387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_39 },
#else
    { 0, 0, output_39 },
#endif
    0,
    &operand_data[41],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:1598 */
  {
    "*pushdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[56],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1598 */
  {
    "*pushti2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[58],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1614 */
  {
    "*pushdi2_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_42 },
#else
    { 0, output_42, 0 },
#endif
    0,
    &operand_data[60],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:1680 */
  {
    "*pushsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "push{l}\t%1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[62],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1693 */
  {
    "*pushqi2_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "push{q}\t%q1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[64],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1693 */
  {
    "*pushhi2_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "push{q}\t%q1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[66],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1693 */
  {
    "*pushsi2_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "push{q}\t%q1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[68],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1701 */
  {
    "*pushqi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "push{l}\t%k1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[64],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1701 */
  {
    "*pushhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "push{l}\t%k1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[66],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1709 */
  {
    "*pushsi2_prologue",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "push{l}\t%1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[70],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1709 */
  {
    "*pushdi2_prologue",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "push{q}\t%1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[72],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1718 */
  {
    "*popsi1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pop{l}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[74],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1718 */
  {
    "*popdi1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pop{q}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[76],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1726 */
  {
    "*popsi1_epilogue",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pop{l}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[74],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1726 */
  {
    "*popdi1_epilogue",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pop{q}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[76],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1779 */
  {
    "*movsi_xor",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{l}\t%k0, %k0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[78],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1779 */
  {
    "*movdi_xor",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{l}\t%k0, %k0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[80],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1789 */
  {
    "*movsi_or",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{l}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[82],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1789 */
  {
    "*movdi_or",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[84],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:1800 */
  {
    "*movoi_internal_avx",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_59 },
#else
    { 0, 0, output_59 },
#endif
    0,
    &operand_data[86],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:1824 */
  {
    "*movti_internal_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_60 },
#else
    { 0, 0, output_60 },
#endif
    0,
    &operand_data[88],
    2,
    2,
    0,
    5,
    3
  },
  /* ../../gcc/config/i386/i386.md:1881 */
  {
    "*movti_internal_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_61 },
#else
    { 0, 0, output_61 },
#endif
    0,
    &operand_data[90],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:1925 */
  {
    "*movdi_internal_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_62 },
#else
    { 0, 0, output_62 },
#endif
    0,
    &operand_data[92],
    2,
    2,
    0,
    17,
    3
  },
  /* ../../gcc/config/i386/i386.md:2050 */
  {
    "*movdi_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_63 },
#else
    { 0, 0, output_63 },
#endif
    0,
    &operand_data[94],
    2,
    2,
    0,
    15,
    3
  },
  /* ../../gcc/config/i386/i386.md:2131 */
  {
    "*movsi_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_64 },
#else
    { 0, 0, output_64 },
#endif
    0,
    &operand_data[96],
    2,
    2,
    0,
    12,
    3
  },
  /* ../../gcc/config/i386/i386.md:2212 */
  {
    "*movhi_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_65 },
#else
    { 0, 0, output_65 },
#endif
    0,
    &operand_data[98],
    2,
    2,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/i386.md:2268 */
  {
    "*movqi_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_66 },
#else
    { 0, 0, output_66 },
#endif
    0,
    &operand_data[100],
    2,
    2,
    0,
    7,
    3
  },
  /* ../../gcc/config/i386/i386.md:2327 */
  {
    "*movabsqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_67 },
#else
    { 0, output_67, 0 },
#endif
    0,
    &operand_data[102],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:2327 */
  {
    "*movabshi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_68 },
#else
    { 0, output_68, 0 },
#endif
    0,
    &operand_data[104],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:2327 */
  {
    "*movabssi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_69 },
#else
    { 0, output_69, 0 },
#endif
    0,
    &operand_data[106],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:2327 */
  {
    "*movabsdi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_70 },
#else
    { 0, output_70, 0 },
#endif
    0,
    &operand_data[108],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:2341 */
  {
    "*movabsqi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_71 },
#else
    { 0, output_71, 0 },
#endif
    0,
    &operand_data[110],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:2341 */
  {
    "*movabshi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_72 },
#else
    { 0, output_72, 0 },
#endif
    0,
    &operand_data[112],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:2341 */
  {
    "*movabssi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_73 },
#else
    { 0, output_73, 0 },
#endif
    0,
    &operand_data[114],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:2341 */
  {
    "*movabsdi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_74 },
#else
    { 0, output_74, 0 },
#endif
    0,
    &operand_data[116],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:2355 */
  {
    "*swapsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xchg{l}\t%1, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[118],
    2,
    2,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:2355 */
  {
    "*swapdi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xchg{q}\t%1, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[120],
    2,
    2,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:2369 */
  {
    "*swapqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xchg{l}\t%k1, %k0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[122],
    2,
    2,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:2369 */
  {
    "*swaphi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xchg{l}\t%k1, %k0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[124],
    2,
    2,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:2385 */
  {
    "*swapqi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xchg{b}\t%1, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[126],
    2,
    2,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:2385 */
  {
    "*swaphi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xchg{w}\t%1, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[124],
    2,
    2,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:2412 */
  {
    "*movstrictqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[128],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:2412 */
  {
    "*movstricthi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{w}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[130],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:2422 */
  {
    "*movstrictqi_xor",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{b}\t%0, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[132],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:2422 */
  {
    "*movstricthi_xor",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{w}\t%0, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[134],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:2432 */
  {
    "*movhi_extv_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movs{bl|x}\t{%h1, %k0|%k0, %h1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[136],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:2432 */
  {
    "*movsi_extv_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movs{bl|x}\t{%h1, %k0|%k0, %h1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[138],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:2442 */
  {
    "*movqi_extv_1_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_87 },
#else
    { 0, 0, output_87 },
#endif
    0,
    &operand_data[140],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:2467 */
  {
    "*movqi_extv_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_88 },
#else
    { 0, 0, output_88 },
#endif
    0,
    &operand_data[142],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:2493 */
  {
    "*movsi_extzv_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movz{bl|x}\t{%h1, %k0|%k0, %h1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[138],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:2493 */
  {
    "*movdi_extzv_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movz{bl|x}\t{%h1, %k0|%k0, %h1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[144],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:2503 */
  {
    "*movqi_extzv_2_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_91 },
#else
    { 0, 0, output_91 },
#endif
    0,
    &operand_data[140],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:2529 */
  {
    "*movqi_extzv_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_92 },
#else
    { 0, 0, output_92 },
#endif
    0,
    &operand_data[146],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:2562 */
  {
    "*movsi_insv_1_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_93 },
#else
    { 0, 0, output_93 },
#endif
    0,
    &operand_data[148],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:2562 */
  {
    "*movdi_insv_1_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_94 },
#else
    { 0, 0, output_94 },
#endif
    0,
    &operand_data[150],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:2576 */
  {
    "*movsi_insv_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_95 },
#else
    { 0, 0, output_95 },
#endif
    0,
    &operand_data[152],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:2590 */
  {
    "*movqi_insv_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{b}\t{%h1, %h0|%h0, %h1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[154],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:2603 */
  {
    "*pushtf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_97 },
#else
    { 0, 0, output_97 },
#endif
    0,
    &operand_data[156],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:2623 */
  {
    "*pushxf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_98 },
#else
    { 0, 0, output_98 },
#endif
    0,
    &operand_data[158],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:2641 */
  {
    "*pushxf_nointeger",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_99 },
#else
    { 0, 0, output_99 },
#endif
    0,
    &operand_data[160],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:2662 */
  {
    "*pushdf_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_100 },
#else
    { 0, 0, output_100 },
#endif
    0,
    &operand_data[162],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:2678 */
  {
    "*pushdf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_101 },
#else
    { 0, 0, output_101 },
#endif
    0,
    &operand_data[164],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:2699 */
  {
    "*pushsf_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_102 },
#else
    { 0, 0, output_102 },
#endif
    0,
    &operand_data[166],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:2712 */
  {
    "*pushsf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_103 },
#else
    { 0, 0, output_103 },
#endif
    0,
    &operand_data[168],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:2769 */
  {
    "*movtf_internal_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_104 },
#else
    { 0, 0, output_104 },
#endif
    0,
    &operand_data[170],
    2,
    2,
    0,
    5,
    3
  },
  /* ../../gcc/config/i386/i386.md:2831 */
  {
    "*movtf_internal_sse2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_105 },
#else
    { 0, 0, output_105 },
#endif
    0,
    &operand_data[172],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:2890 */
  {
    "*movxf_internal_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_106 },
#else
    { 0, 0, output_106 },
#endif
    0,
    &operand_data[174],
    2,
    2,
    0,
    5,
    3
  },
  /* ../../gcc/config/i386/i386.md:2924 */
  {
    "*movxf_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_107 },
#else
    { 0, 0, output_107 },
#endif
    0,
    &operand_data[176],
    2,
    2,
    0,
    5,
    3
  },
  /* ../../gcc/config/i386/i386.md:2957 */
  {
    "*movdf_internal_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_108 },
#else
    { 0, 0, output_108 },
#endif
    0,
    &operand_data[178],
    2,
    2,
    0,
    13,
    3
  },
  /* ../../gcc/config/i386/i386.md:3098 */
  {
    "*movdf_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_109 },
#else
    { 0, 0, output_109 },
#endif
    0,
    &operand_data[180],
    2,
    2,
    0,
    13,
    3
  },
  /* ../../gcc/config/i386/i386.md:3232 */
  {
    "*movsf_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_110 },
#else
    { 0, 0, output_110 },
#endif
    0,
    &operand_data[182],
    2,
    2,
    0,
    16,
    3
  },
  /* ../../gcc/config/i386/i386.md:3404 */
  {
    "swapxf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_111 },
#else
    { 0, 0, output_111 },
#endif
    (insn_gen_fn) gen_swapxf,
    &operand_data[184],
    2,
    2,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:3419 */
  {
    "*swapsf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_112 },
#else
    { 0, 0, output_112 },
#endif
    0,
    &operand_data[186],
    2,
    2,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:3419 */
  {
    "*swapdf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_113 },
#else
    { 0, 0, output_113 },
#endif
    0,
    &operand_data[188],
    2,
    2,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:3448 */
  {
    "*zero_extendsidi2_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_114 },
#else
    { 0, output_114, 0 },
#endif
    0,
    &operand_data[190],
    2,
    2,
    0,
    6,
    2
  },
  /* ../../gcc/config/i386/i386.md:3473 */
  {
    "zero_extendsidi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_115 },
#else
    { 0, output_115, 0 },
#endif
    (insn_gen_fn) gen_zero_extendsidi2_1,
    &operand_data[192],
    2,
    2,
    0,
    7,
    2
  },
  /* ../../gcc/config/i386/i386.md:3511 */
  {
    "zero_extendqidi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movz{bl|x}\t{%1, %k0|%k0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_zero_extendqidi2,
    &operand_data[194],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:3511 */
  {
    "zero_extendhidi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movz{wl|x}\t{%1, %k0|%k0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_zero_extendhidi2,
    &operand_data[196],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:3533 */
  {
    "zero_extendhisi2_and",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_zero_extendhisi2_and,
    &operand_data[198],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:3546 */
  {
    "*zero_extendhisi2_movzwl",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movz{wl|x}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[200],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:3561 */
  {
    "*zero_extendqihi2_and",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[202],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:3561 */
  {
    "*zero_extendqisi2_and",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[204],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:3587 */
  {
    "*zero_extendqihi2_movzbl_and",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[206],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:3587 */
  {
    "*zero_extendqisi2_movzbl_and",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[208],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:3608 */
  {
    "*zero_extendqihi2_movzbl",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movz{bl|x}\t{%1, %k0|%k0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[210],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:3608 */
  {
    "*zero_extendqisi2_movzbl",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movz{bl|x}\t{%1, %k0|%k0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[212],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:3641 */
  {
    "*extendsidi2_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_126 },
#else
    { 0, output_126, 0 },
#endif
    0,
    &operand_data[214],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:3653 */
  {
    "extendsidi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_extendsidi2_1,
    &operand_data[216],
    2,
    3,
    0,
    4,
    1
  },
  /* ../../gcc/config/i386/i386.md:3736 */
  {
    "extendqidi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movs{bq|x}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_extendqidi2,
    &operand_data[194],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:3736 */
  {
    "extendhidi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movs{wq|x}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_extendhidi2,
    &operand_data[196],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:3745 */
  {
    "extendhisi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_130 },
#else
    { 0, 0, output_130 },
#endif
    (insn_gen_fn) gen_extendhisi2,
    &operand_data[219],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:3771 */
  {
    "*extendhisi2_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_131 },
#else
    { 0, 0, output_131 },
#endif
    0,
    &operand_data[221],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:3799 */
  {
    "extendqisi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movs{bl|x}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_extendqisi2,
    &operand_data[212],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:3807 */
  {
    "*extendqisi2_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movs{bl|x}\t{%1, %k0|%k0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[194],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:3816 */
  {
    "extendqihi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_134 },
#else
    { 0, 0, output_134 },
#endif
    (insn_gen_fn) gen_extendqihi2,
    &operand_data[223],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:3929 */
  {
    "*extendsfdf2_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_135 },
#else
    { 0, 0, output_135 },
#endif
    0,
    &operand_data[225],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:3952 */
  {
    "*extendsfdf2_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtss2sd\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[227],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:3961 */
  {
    "*extendsfdf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_137 },
#else
    { 0, 0, output_137 },
#endif
    0,
    &operand_data[229],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:3989 */
  {
    "*extendsfxf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_138 },
#else
    { 0, 0, output_138 },
#endif
    0,
    &operand_data[231],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:3989 */
  {
    "*extenddfxf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_139 },
#else
    { 0, 0, output_139 },
#endif
    0,
    &operand_data[233],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:4075 */
  {
    "*truncdfsf_fast_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_140 },
#else
    { 0, 0, output_140 },
#endif
    0,
    &operand_data[235],
    2,
    2,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:4097 */
  {
    "*truncdfsf_fast_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtsd2ss\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[237],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:4107 */
  {
    "*truncdfsf_fast_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_142 },
#else
    { 0, 0, output_142 },
#endif
    0,
    &operand_data[239],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:4116 */
  {
    "*truncdfsf_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_143 },
#else
    { 0, 0, output_143 },
#endif
    0,
    &operand_data[241],
    3,
    3,
    0,
    5,
    3
  },
  /* ../../gcc/config/i386/i386.md:4140 */
  {
    "*truncdfsf_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_144 },
#else
    { 0, 0, output_144 },
#endif
    0,
    &operand_data[244],
    3,
    3,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/i386.md:4160 */
  {
    "*truncdfsf2_i387_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_145 },
#else
    { 0, 0, output_145 },
#endif
    0,
    &operand_data[247],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:4202 */
  {
    "*truncxfsf2_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_146 },
#else
    { 0, 0, output_146 },
#endif
    0,
    &operand_data[249],
    3,
    3,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/i386.md:4216 */
  {
    "*truncxfdf2_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_147 },
#else
    { 0, 0, output_147 },
#endif
    0,
    &operand_data[252],
    3,
    3,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/i386.md:4231 */
  {
    "truncxfsf2_i387_noop",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_148 },
#else
    { 0, 0, output_148 },
#endif
    (insn_gen_fn) gen_truncxfsf2_i387_noop,
    &operand_data[255],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:4231 */
  {
    "truncxfdf2_i387_noop",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_149 },
#else
    { 0, 0, output_149 },
#endif
    (insn_gen_fn) gen_truncxfdf2_i387_noop,
    &operand_data[257],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:4240 */
  {
    "*truncxfsf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_150 },
#else
    { 0, 0, output_150 },
#endif
    0,
    &operand_data[259],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:4240 */
  {
    "*truncxfdf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_151 },
#else
    { 0, 0, output_151 },
#endif
    0,
    &operand_data[261],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:4382 */
  {
    "*fixuns_truncsf_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[263],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4382 */
  {
    "*fixuns_truncdf_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[268],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4412 */
  {
    "fix_truncsfdi_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvttss2si{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncsfdi_sse,
    &operand_data[273],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4412 */
  {
    "fix_truncdfdi_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvttsd2si{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncdfdi_sse,
    &operand_data[275],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4426 */
  {
    "fix_truncsfsi_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvttss2si\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncsfsi_sse,
    &operand_data[277],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4426 */
  {
    "fix_truncdfsi_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvttsd2si\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncdfsi_sse,
    &operand_data[279],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4467 */
  {
    "fix_trunchi_fisttp_i387_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_trunchi_fisttp_i387_1,
    &operand_data[281],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4467 */
  {
    "fix_truncsi_fisttp_i387_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncsi_fisttp_i387_1,
    &operand_data[283],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4467 */
  {
    "fix_truncdi_fisttp_i387_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncdi_fisttp_i387_1,
    &operand_data[285],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4494 */
  {
    "fix_trunchi_i387_fisttp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_161 },
#else
    { 0, 0, output_161 },
#endif
    (insn_gen_fn) gen_fix_trunchi_i387_fisttp,
    &operand_data[287],
    2,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:4494 */
  {
    "fix_truncsi_i387_fisttp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_162 },
#else
    { 0, 0, output_162 },
#endif
    (insn_gen_fn) gen_fix_truncsi_i387_fisttp,
    &operand_data[290],
    2,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:4494 */
  {
    "fix_truncdi_i387_fisttp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_163 },
#else
    { 0, 0, output_163 },
#endif
    (insn_gen_fn) gen_fix_truncdi_i387_fisttp,
    &operand_data[293],
    2,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:4507 */
  {
    "fix_trunchi_i387_fisttp_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_trunchi_i387_fisttp_with_temp,
    &operand_data[296],
    3,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4507 */
  {
    "fix_truncsi_i387_fisttp_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncsi_i387_fisttp_with_temp,
    &operand_data[300],
    3,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4507 */
  {
    "fix_truncdi_i387_fisttp_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncdi_i387_fisttp_with_temp,
    &operand_data[304],
    3,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4545 */
  {
    "*fix_trunchi_i387_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[281],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4545 */
  {
    "*fix_truncsi_i387_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[283],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4545 */
  {
    "*fix_truncdi_i387_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[285],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4578 */
  {
    "fix_truncdi_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_170 },
#else
    { 0, 0, output_170 },
#endif
    (insn_gen_fn) gen_fix_truncdi_i387,
    &operand_data[308],
    4,
    5,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:4592 */
  {
    "fix_truncdi_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncdi_i387_with_temp,
    &operand_data[313],
    5,
    6,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4634 */
  {
    "fix_trunchi_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_172 },
#else
    { 0, 0, output_172 },
#endif
    (insn_gen_fn) gen_fix_trunchi_i387,
    &operand_data[319],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:4634 */
  {
    "fix_truncsi_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_173 },
#else
    { 0, 0, output_173 },
#endif
    (insn_gen_fn) gen_fix_truncsi_i387,
    &operand_data[323],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:4647 */
  {
    "fix_trunchi_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_trunchi_i387_with_temp,
    &operand_data[327],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4647 */
  {
    "fix_truncsi_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncsi_i387_with_temp,
    &operand_data[332],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4684 */
  {
    "x86_fnstcw_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fnstcw\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_x86_fnstcw_1,
    &operand_data[287],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:4695 */
  {
    "x86_fldcw_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fldcw\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_x86_fldcw_1,
    &operand_data[45],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:4721 */
  {
    "*floathisf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[337],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4721 */
  {
    "*floathidf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[339],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4721 */
  {
    "*floathixf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[341],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4735 */
  {
    "*floathisf2_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[343],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4735 */
  {
    "*floathidf2_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[346],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4735 */
  {
    "*floathixf2_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[349],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:4748 */
  {
    "*floathisf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fild%Z1\t%1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[352],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:4748 */
  {
    "*floathidf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fild%Z1\t%1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[354],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:4748 */
  {
    "*floathixf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fild%Z1\t%1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[356],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:4810 */
  {
    "*floatsisf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[358],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4810 */
  {
    "*floatsidf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[360],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4810 */
  {
    "*floatsixf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[362],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4810 */
  {
    "*floatdisf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[364],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4810 */
  {
    "*floatdidf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[366],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4810 */
  {
    "*floatdixf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[368],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:4847 */
  {
    "*floatsisf2_vector_mixed_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[370],
    3,
    3,
    0,
    5,
    1
  },
  /* ../../gcc/config/i386/i386.md:4847 */
  {
    "*floatsidf2_vector_mixed_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[373],
    3,
    3,
    0,
    5,
    1
  },
  /* ../../gcc/config/i386/i386.md:4863 */
  {
    "*floatsisf2_vector_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_195 },
#else
    { 0, output_195, 0 },
#endif
    0,
    &operand_data[376],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:4863 */
  {
    "*floatsidf2_vector_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_196 },
#else
    { 0, output_196, 0 },
#endif
    0,
    &operand_data[378],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:4879 */
  {
    "*floatsisf2_mixed_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[380],
    3,
    3,
    0,
    4,
    1
  },
  /* ../../gcc/config/i386/i386.md:4879 */
  {
    "*floatsidf2_mixed_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[383],
    3,
    3,
    0,
    4,
    1
  },
  /* ../../gcc/config/i386/i386.md:4879 */
  {
    "*floatdisf2_mixed_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[386],
    3,
    3,
    0,
    4,
    1
  },
  /* ../../gcc/config/i386/i386.md:4879 */
  {
    "*floatdidf2_mixed_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[389],
    3,
    3,
    0,
    4,
    1
  },
  /* ../../gcc/config/i386/i386.md:4922 */
  {
    "*floatsisf2_mixed_interunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_201 },
#else
    { 0, output_201, 0 },
#endif
    0,
    &operand_data[392],
    2,
    2,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:4922 */
  {
    "*floatsidf2_mixed_interunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_202 },
#else
    { 0, output_202, 0 },
#endif
    0,
    &operand_data[394],
    2,
    2,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:4922 */
  {
    "*floatdisf2_mixed_interunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_203 },
#else
    { 0, output_203, 0 },
#endif
    0,
    &operand_data[396],
    2,
    2,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:4922 */
  {
    "*floatdidf2_mixed_interunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_204 },
#else
    { 0, output_204, 0 },
#endif
    0,
    &operand_data[398],
    2,
    2,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:4948 */
  {
    "*floatsisf2_mixed_nointerunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_205 },
#else
    { 0, output_205, 0 },
#endif
    0,
    &operand_data[376],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:4948 */
  {
    "*floatsidf2_mixed_nointerunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_206 },
#else
    { 0, output_206, 0 },
#endif
    0,
    &operand_data[378],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:4948 */
  {
    "*floatdisf2_mixed_nointerunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_207 },
#else
    { 0, output_207, 0 },
#endif
    0,
    &operand_data[400],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:4948 */
  {
    "*floatdidf2_mixed_nointerunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_208 },
#else
    { 0, output_208, 0 },
#endif
    0,
    &operand_data[402],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:4972 */
  {
    "*floatsisf2_vector_sse_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[404],
    3,
    3,
    0,
    3,
    1
  },
  /* ../../gcc/config/i386/i386.md:4972 */
  {
    "*floatsidf2_vector_sse_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[407],
    3,
    3,
    0,
    3,
    1
  },
  /* ../../gcc/config/i386/i386.md:4987 */
  {
    "*floatsisf2_vector_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[410],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:4987 */
  {
    "*floatsidf2_vector_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[412],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:5138 */
  {
    "*floatsisf2_sse_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[414],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5138 */
  {
    "*floatsidf2_sse_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[417],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5138 */
  {
    "*floatdisf2_sse_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[420],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5138 */
  {
    "*floatdidf2_sse_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[423],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5153 */
  {
    "*floatsisf2_sse_interunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtsi2ss\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[414],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5153 */
  {
    "*floatsidf2_sse_interunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtsi2sd\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[417],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5153 */
  {
    "*floatdisf2_sse_interunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtsi2ss{q}\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[420],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5153 */
  {
    "*floatdidf2_sse_interunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtsi2sd{q}\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[423],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5188 */
  {
    "*floatsisf2_sse_nointerunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtsi2ss\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[410],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:5188 */
  {
    "*floatsidf2_sse_nointerunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtsi2sd\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[412],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:5188 */
  {
    "*floatdisf2_sse_nointerunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtsi2ss{q}\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[426],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:5188 */
  {
    "*floatdidf2_sse_nointerunit",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtsi2sd{q}\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[428],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:5236 */
  {
    "*floatsisf2_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_225 },
#else
    { 0, output_225, 0 },
#endif
    0,
    &operand_data[430],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:5236 */
  {
    "*floatsidf2_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_226 },
#else
    { 0, output_226, 0 },
#endif
    0,
    &operand_data[433],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:5236 */
  {
    "*floatsixf2_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_227 },
#else
    { 0, output_227, 0 },
#endif
    0,
    &operand_data[436],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:5236 */
  {
    "*floatdisf2_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_228 },
#else
    { 0, output_228, 0 },
#endif
    0,
    &operand_data[439],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:5236 */
  {
    "*floatdidf2_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_229 },
#else
    { 0, output_229, 0 },
#endif
    0,
    &operand_data[442],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:5236 */
  {
    "*floatdixf2_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_230 },
#else
    { 0, output_230, 0 },
#endif
    0,
    &operand_data[445],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:5251 */
  {
    "*floatsisf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fild%Z1\t%1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[448],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:5251 */
  {
    "*floatsidf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fild%Z1\t%1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[450],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:5251 */
  {
    "*floatsixf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fild%Z1\t%1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[452],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:5251 */
  {
    "*floatdisf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fild%Z1\t%1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[454],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:5251 */
  {
    "*floatdidf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fild%Z1\t%1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[456],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:5251 */
  {
    "*floatdixf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fild%Z1\t%1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[458],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:5284 */
  {
    "floatdisf2_i387_with_xmm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_floatdisf2_i387_with_xmm,
    &operand_data[460],
    3,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5284 */
  {
    "floatdidf2_i387_with_xmm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_floatdidf2_i387_with_xmm,
    &operand_data[465],
    3,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5284 */
  {
    "floatdixf2_i387_with_xmm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_floatdixf2_i387_with_xmm,
    &operand_data[470],
    3,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5343 */
  {
    "*floatunssisf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[475],
    3,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5343 */
  {
    "*floatunssidf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[479],
    3,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5343 */
  {
    "*floatunssixf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[483],
    3,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5431 */
  {
    "*leasi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_243 },
#else
    { 0, 0, output_243 },
#endif
    0,
    &operand_data[487],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:5431 */
  {
    "*leadi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_244 },
#else
    { 0, 0, output_244 },
#endif
    0,
    &operand_data[489],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:5468 */
  {
    "*adddi3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[491],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5468 */
  {
    "*addti3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[494],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5491 */
  {
    "*addsi3_cc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[497],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5491 */
  {
    "*adddi3_cc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[500],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5504 */
  {
    "addqi3_cc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_addqi3_cc,
    &operand_data[503],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:5517 */
  {
    "*addsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_250 },
#else
    { 0, 0, output_250 },
#endif
    0,
    &operand_data[506],
    3,
    3,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/i386.md:5517 */
  {
    "*adddi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_251 },
#else
    { 0, 0, output_251 },
#endif
    0,
    &operand_data[509],
    3,
    3,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/i386.md:5576 */
  {
    "addsi_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_252 },
#else
    { 0, 0, output_252 },
#endif
    (insn_gen_fn) gen_addsi_1_zext,
    &operand_data[512],
    3,
    3,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:5627 */
  {
    "*addhi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_253 },
#else
    { 0, 0, output_253 },
#endif
    0,
    &operand_data[515],
    3,
    3,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/i386.md:5680 */
  {
    "*addqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_254 },
#else
    { 0, 0, output_254 },
#endif
    0,
    &operand_data[518],
    3,
    3,
    0,
    6,
    3
  },
  /* ../../gcc/config/i386/i386.md:5741 */
  {
    "*addqi_1_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_255 },
#else
    { 0, 0, output_255 },
#endif
    0,
    &operand_data[521],
    2,
    2,
    1,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:5825 */
  {
    "*addqi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_256 },
#else
    { 0, 0, output_256 },
#endif
    0,
    &operand_data[523],
    3,
    3,
    2,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:5825 */
  {
    "*addhi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_257 },
#else
    { 0, 0, output_257 },
#endif
    0,
    &operand_data[526],
    3,
    3,
    2,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:5825 */
  {
    "*addsi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_258 },
#else
    { 0, 0, output_258 },
#endif
    0,
    &operand_data[529],
    3,
    3,
    2,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:5825 */
  {
    "*adddi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_259 },
#else
    { 0, 0, output_259 },
#endif
    0,
    &operand_data[532],
    3,
    3,
    2,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:5874 */
  {
    "*addsi_2_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_260 },
#else
    { 0, 0, output_260 },
#endif
    0,
    &operand_data[535],
    3,
    3,
    2,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:5920 */
  {
    "*addqi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_261 },
#else
    { 0, 0, output_261 },
#endif
    0,
    &operand_data[538],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:5920 */
  {
    "*addhi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_262 },
#else
    { 0, 0, output_262 },
#endif
    0,
    &operand_data[541],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:5920 */
  {
    "*addsi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_263 },
#else
    { 0, 0, output_263 },
#endif
    0,
    &operand_data[544],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:5920 */
  {
    "*adddi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_264 },
#else
    { 0, 0, output_264 },
#endif
    0,
    &operand_data[547],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:5966 */
  {
    "*addsi_3_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_265 },
#else
    { 0, 0, output_265 },
#endif
    0,
    &operand_data[535],
    3,
    3,
    2,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:6018 */
  {
    "*adddi_4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_266 },
#else
    { 0, 0, output_266 },
#endif
    0,
    &operand_data[550],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:6063 */
  {
    "*addqi_4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_267 },
#else
    { 0, 0, output_267 },
#endif
    0,
    &operand_data[553],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:6063 */
  {
    "*addhi_4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_268 },
#else
    { 0, 0, output_268 },
#endif
    0,
    &operand_data[556],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:6063 */
  {
    "*addsi_4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_269 },
#else
    { 0, 0, output_269 },
#endif
    0,
    &operand_data[559],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:6100 */
  {
    "*addqi_5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_270 },
#else
    { 0, 0, output_270 },
#endif
    0,
    &operand_data[538],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:6100 */
  {
    "*addhi_5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_271 },
#else
    { 0, 0, output_271 },
#endif
    0,
    &operand_data[541],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:6100 */
  {
    "*addsi_5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_272 },
#else
    { 0, 0, output_272 },
#endif
    0,
    &operand_data[544],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:6100 */
  {
    "*adddi_5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_273 },
#else
    { 0, 0, output_273 },
#endif
    0,
    &operand_data[547],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:6147 */
  {
    "*addqi_ext_1_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_274 },
#else
    { 0, 0, output_274 },
#endif
    0,
    &operand_data[562],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:6182 */
  {
    "addqi_ext_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_275 },
#else
    { 0, 0, output_275 },
#endif
    (insn_gen_fn) gen_addqi_ext_1,
    &operand_data[565],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:6217 */
  {
    "*addqi_ext_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{b}\t{%h2, %h0|%h0, %h2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[568],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6239 */
  {
    "*lea_general_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[571],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6271 */
  {
    "*lea_general_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[575],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6301 */
  {
    "*lea_general_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[579],
    5,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6336 */
  {
    "*lea_general_4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[584],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6336 */
  {
    "*lea_general_4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[584],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6387 */
  {
    "*subdi3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[588],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6387 */
  {
    "*subti3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[591],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6409 */
  {
    "*subqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[594],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6409 */
  {
    "*subhi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[597],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6409 */
  {
    "*subsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[600],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6409 */
  {
    "*subdi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[603],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6420 */
  {
    "*subsi_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[606],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6431 */
  {
    "*subqi_1_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[521],
    2,
    2,
    1,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6442 */
  {
    "*subqi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[594],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6442 */
  {
    "*subhi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[597],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6442 */
  {
    "*subsi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[600],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6442 */
  {
    "*subdi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[603],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6457 */
  {
    "*subsi_2_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[606],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6473 */
  {
    "*subqi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[594],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6473 */
  {
    "*subhi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[597],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6473 */
  {
    "*subsi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[600],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6473 */
  {
    "*subdi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[603],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6485 */
  {
    "*subsi_3_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{l}\t{%2, %1|%1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[606],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6513 */
  {
    "*addqi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "adc{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[609],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6513 */
  {
    "*subqi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sbb{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[613],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6513 */
  {
    "*addhi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "adc{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[617],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6513 */
  {
    "*subhi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sbb{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[621],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6513 */
  {
    "*addsi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "adc{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[625],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6513 */
  {
    "*subsi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sbb{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[629],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6513 */
  {
    "*adddi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "adc{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[633],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6513 */
  {
    "*subdi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sbb{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[637],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6529 */
  {
    "*addsi3_carry_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "adc{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[641],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6544 */
  {
    "*subsi3_carry_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sbb{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[645],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6560 */
  {
    "*addqi3_cconly_overflow",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[649],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6560 */
  {
    "*addhi3_cconly_overflow",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[652],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6560 */
  {
    "*addsi3_cconly_overflow",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[655],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6560 */
  {
    "*adddi3_cconly_overflow",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[658],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6573 */
  {
    "*addqi3_cc_overflow",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[503],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6573 */
  {
    "*addhi3_cc_overflow",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[617],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6573 */
  {
    "*addsi3_cc_overflow",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[497],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6573 */
  {
    "*adddi3_cc_overflow",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[500],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:6587 */
  {
    "*addsi3_zext_cc_overflow",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[641],
    3,
    3,
    3,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6645 */
  {
    "*mulsi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_319 },
#else
    { 0, output_319, 0 },
#endif
    0,
    &operand_data[661],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:6645 */
  {
    "*muldi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_320 },
#else
    { 0, output_320, 0 },
#endif
    0,
    &operand_data[664],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:6675 */
  {
    "*mulsi3_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_321 },
#else
    { 0, output_321, 0 },
#endif
    0,
    &operand_data[667],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:6716 */
  {
    "*mulhi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_322 },
#else
    { 0, output_322, 0 },
#endif
    0,
    &operand_data[670],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:6746 */
  {
    "*mulqi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mul{b}\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[673],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6783 */
  {
    "*bmi2_umulditi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mulx\t{%3, %0, %1|%1, %0, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[676],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6801 */
  {
    "*bmi2_umulsidi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mulx\t{%3, %0, %1|%1, %0, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[680],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6819 */
  {
    "*umulsidi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_326 },
#else
    { 0, output_326, 0 },
#endif
    0,
    &operand_data[684],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:6819 */
  {
    "*umulditi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_327 },
#else
    { 0, output_327, 0 },
#endif
    0,
    &operand_data[687],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:6870 */
  {
    "*mulsidi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "imul{l}\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[690],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6870 */
  {
    "*mulditi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "imul{q}\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[693],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6890 */
  {
    "*mulqihi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "imul{b}\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[696],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6890 */
  {
    "*umulqihi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mul{b}\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[696],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6926 */
  {
    "*smuldi3_highpart_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "imul{q}\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[699],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6926 */
  {
    "*umuldi3_highpart_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mul{q}\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[699],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6951 */
  {
    "*smulsi3_highpart_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "imul{l}\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[703],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6951 */
  {
    "*umulsi3_highpart_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mul{l}\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[703],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6975 */
  {
    "*smulsi3_highpart_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "imul{l}\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[707],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:6975 */
  {
    "*umulsi3_highpart_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mul{l}\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[707],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7080 */
  {
    "divmodsi4_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_divmodsi4_1,
    &operand_data[711],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7080 */
  {
    "divmoddi4_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_divmoddi4_1,
    &operand_data[715],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7115 */
  {
    "*divmodhi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[719],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7115 */
  {
    "*divmodsi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[711],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7115 */
  {
    "*divmoddi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[715],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7150 */
  {
    "*divmodhi4_noext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "idiv{w}\t%3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[723],
    5,
    5,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7150 */
  {
    "*divmodsi4_noext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "idiv{l}\t%3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[728],
    5,
    5,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7150 */
  {
    "*divmoddi4_noext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "idiv{q}\t%3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[733],
    5,
    5,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7206 */
  {
    "divmodhiqi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "idiv{b}\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_divmodhiqi3,
    &operand_data[738],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7253 */
  {
    "udivmodsi4_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_udivmodsi4_1,
    &operand_data[711],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7253 */
  {
    "udivmoddi4_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_udivmoddi4_1,
    &operand_data[715],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7275 */
  {
    "*udivmodhi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[719],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7275 */
  {
    "*udivmodsi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[711],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7275 */
  {
    "*udivmoddi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[715],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7296 */
  {
    "*udivmodhi4_noext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "div{w}\t%3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[723],
    5,
    5,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7296 */
  {
    "*udivmodsi4_noext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "div{l}\t%3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[728],
    5,
    5,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7296 */
  {
    "*udivmoddi4_noext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "div{q}\t%3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[733],
    5,
    5,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7347 */
  {
    "udivmodhiqi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "div{b}\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_udivmodhiqi3,
    &operand_data[738],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7411 */
  {
    "*testdi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_356 },
#else
    { 0, output_356, 0 },
#endif
    0,
    &operand_data[741],
    2,
    2,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/i386.md:7430 */
  {
    "*testqi_1_maybe_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_357 },
#else
    { 0, 0, output_357 },
#endif
    0,
    &operand_data[743],
    2,
    2,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/i386.md:7455 */
  {
    "*testqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "test{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[745],
    2,
    2,
    0,
    3,
    1
  },
  /* ../../gcc/config/i386/i386.md:7455 */
  {
    "*testhi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "test{w}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[747],
    2,
    2,
    0,
    3,
    1
  },
  /* ../../gcc/config/i386/i386.md:7455 */
  {
    "*testsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "test{l}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[749],
    2,
    2,
    0,
    3,
    1
  },
  /* ../../gcc/config/i386/i386.md:7481 */
  {
    "*testqi_ext_0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "test{b}\t{%1, %h0|%h0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[751],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7499 */
  {
    "*testqi_ext_1_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "test{b}\t{%1, %h0|%h0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[18],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7515 */
  {
    "*testqi_ext_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "test{b}\t{%1, %h0|%h0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[753],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7531 */
  {
    "*testqi_ext_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "test{b}\t{%h1, %h0|%h0, %h1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[26],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7549 */
  {
    "*testqi_ext_3_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[755],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7571 */
  {
    "*testqi_ext_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[758],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7712 */
  {
    "*anddi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_367 },
#else
    { 0, 0, output_367 },
#endif
    0,
    &operand_data[761],
    3,
    3,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/i386.md:7765 */
  {
    "*andsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_368 },
#else
    { 0, 0, output_368 },
#endif
    0,
    &operand_data[764],
    3,
    3,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:7811 */
  {
    "*andsi_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "and{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[641],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7822 */
  {
    "*andhi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_370 },
#else
    { 0, 0, output_370 },
#endif
    0,
    &operand_data[767],
    3,
    3,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:7853 */
  {
    "*andqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_371 },
#else
    { 0, output_371, 0 },
#endif
    0,
    &operand_data[770],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:7866 */
  {
    "*andqi_1_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "and{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[773],
    2,
    2,
    1,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:7917 */
  {
    "*anddi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_373 },
#else
    { 0, output_373, 0 },
#endif
    0,
    &operand_data[775],
    3,
    3,
    2,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:7946 */
  {
    "*andqi_2_maybe_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_374 },
#else
    { 0, 0, output_374 },
#endif
    0,
    &operand_data[778],
    3,
    3,
    2,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:7970 */
  {
    "*andqi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "and{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[781],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:7970 */
  {
    "*andhi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "and{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[784],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:7970 */
  {
    "*andsi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "and{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[787],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:7985 */
  {
    "*andsi_2_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "and{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[641],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:7999 */
  {
    "*andqi_2_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "and{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[790],
    2,
    2,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8017 */
  {
    "andqi_ext_0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "and{b}\t{%2, %h0|%h0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_andqi_ext_0,
    &operand_data[792],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8037 */
  {
    "*andqi_ext_0_cc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "and{b}\t{%2, %h0|%h0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[792],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8063 */
  {
    "*andqi_ext_1_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "and{b}\t{%2, %h0|%h0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[795],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8081 */
  {
    "*andqi_ext_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "and{b}\t{%2, %h0|%h0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[798],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8099 */
  {
    "*andqi_ext_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "and{b}\t{%h2, %h0|%h0, %h2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[568],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8180 */
  {
    "*iorhi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[784],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8180 */
  {
    "*xorhi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[784],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8180 */
  {
    "*iorsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[787],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8180 */
  {
    "*xorsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[787],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8180 */
  {
    "*iordi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[801],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8180 */
  {
    "*xordi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[801],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8192 */
  {
    "*iorqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_391 },
#else
    { 0, output_391, 0 },
#endif
    0,
    &operand_data[804],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:8192 */
  {
    "*xorqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_392 },
#else
    { 0, output_392, 0 },
#endif
    0,
    &operand_data[804],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:8206 */
  {
    "*iorsi_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[641],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8206 */
  {
    "*xorsi_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[641],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8217 */
  {
    "*iorsi_1_zext_imm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[807],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8217 */
  {
    "*xorsi_1_zext_imm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[807],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8228 */
  {
    "*iorqi_1_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[810],
    2,
    2,
    1,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8228 */
  {
    "*xorqi_1_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[810],
    2,
    2,
    1,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8239 */
  {
    "*iorqi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[781],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8239 */
  {
    "*xorqi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[781],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8239 */
  {
    "*iorhi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[784],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8239 */
  {
    "*xorhi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[784],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8239 */
  {
    "*iorsi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[787],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8239 */
  {
    "*xorsi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[787],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8239 */
  {
    "*iordi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[801],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8239 */
  {
    "*xordi_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[801],
    3,
    3,
    2,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8255 */
  {
    "*iorsi_2_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[641],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8255 */
  {
    "*xorsi_2_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[641],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8268 */
  {
    "*iorsi_2_zext_imm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[812],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8268 */
  {
    "*xorsi_2_zext_imm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{l}\t{%2, %k0|%k0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[812],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8282 */
  {
    "*iorqi_2_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[815],
    2,
    2,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8282 */
  {
    "*xorqi_2_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[815],
    2,
    2,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8296 */
  {
    "*iorqi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[649],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8296 */
  {
    "*xorqi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[649],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8296 */
  {
    "*iorhi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[652],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8296 */
  {
    "*xorhi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[652],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8296 */
  {
    "*iorsi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[655],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8296 */
  {
    "*xorsi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[655],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8296 */
  {
    "*iordi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[658],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8296 */
  {
    "*xordi_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[658],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8309 */
  {
    "*iorqi_ext_0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{b}\t{%2, %h0|%h0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[792],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8309 */
  {
    "*xorqi_ext_0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{b}\t{%2, %h0|%h0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[792],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8327 */
  {
    "*iorqi_ext_1_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{b}\t{%2, %h0|%h0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[795],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8327 */
  {
    "*xorqi_ext_1_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{b}\t{%2, %h0|%h0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[795],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8346 */
  {
    "*iorqi_ext_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{b}\t{%2, %h0|%h0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[798],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8346 */
  {
    "*xorqi_ext_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{b}\t{%2, %h0|%h0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[798],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8365 */
  {
    "*iorqi_ext_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "or{b}\t{%h2, %h0|%h0, %h2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[795],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8365 */
  {
    "*xorqi_ext_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{b}\t{%h2, %h0|%h0, %h2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[795],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8448 */
  {
    "*xorqi_cc_ext_1_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{b}\t{%2, %h0|%h0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[562],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8473 */
  {
    "*xorqi_cc_ext_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{b}\t{%2, %h0|%h0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[817],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8506 */
  {
    "*negdi2_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[820],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8506 */
  {
    "*negti2_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[822],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8529 */
  {
    "*negqi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "neg{b}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[824],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8529 */
  {
    "*neghi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "neg{w}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[826],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8529 */
  {
    "*negsi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "neg{l}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[828],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8529 */
  {
    "*negdi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "neg{q}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[830],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8539 */
  {
    "*negsi2_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "neg{l}\t%k0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[832],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8555 */
  {
    "*negqi2_cmpz",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "neg{b}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[824],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8555 */
  {
    "*neghi2_cmpz",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "neg{w}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[826],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8555 */
  {
    "*negsi2_cmpz",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "neg{l}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[828],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8555 */
  {
    "*negdi2_cmpz",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "neg{q}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[830],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8567 */
  {
    "*negsi2_cmpz_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "neg{l}\t%k0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[832],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8593 */
  {
    "*absnegsf2_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[834],
    4,
    4,
    0,
    4,
    1
  },
  /* ../../gcc/config/i386/i386.md:8593 */
  {
    "*absnegdf2_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[838],
    4,
    4,
    0,
    4,
    1
  },
  /* ../../gcc/config/i386/i386.md:8602 */
  {
    "*absnegsf2_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[842],
    4,
    4,
    0,
    3,
    1
  },
  /* ../../gcc/config/i386/i386.md:8602 */
  {
    "*absnegdf2_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[846],
    4,
    4,
    0,
    3,
    1
  },
  /* ../../gcc/config/i386/i386.md:8611 */
  {
    "*absnegsf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[850],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8611 */
  {
    "*absnegdf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[854],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8611 */
  {
    "*absnegxf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[858],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8626 */
  {
    "*absnegtf2_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[862],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8764 */
  {
    "*abssf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fabs",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[866],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8764 */
  {
    "*negsf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fchs",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[866],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8764 */
  {
    "*absdf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fabs",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[868],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8764 */
  {
    "*negdf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fchs",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[868],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8764 */
  {
    "*absxf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fabs",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[870],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8764 */
  {
    "*negxf2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fchs",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[870],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8774 */
  {
    "*absextendsfdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fabs",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[872],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8774 */
  {
    "*negextendsfdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fchs",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[872],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8783 */
  {
    "*absextendsfxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fabs",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[874],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8783 */
  {
    "*negextendsfxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fchs",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[874],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8792 */
  {
    "*absextenddfxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fabs",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[876],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8792 */
  {
    "*negextenddfxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fchs",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[876],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8814 */
  {
    "copysignsf3_const",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_copysignsf3_const,
    &operand_data[878],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8814 */
  {
    "copysigndf3_const",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_copysigndf3_const,
    &operand_data[882],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8814 */
  {
    "copysigntf3_const",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_copysigntf3_const,
    &operand_data[886],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8828 */
  {
    "copysignsf3_var",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_copysignsf3_var,
    &operand_data[890],
    6,
    6,
    0,
    5,
    1
  },
  /* ../../gcc/config/i386/i386.md:8828 */
  {
    "copysigndf3_var",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_copysigndf3_var,
    &operand_data[896],
    6,
    6,
    0,
    5,
    1
  },
  /* ../../gcc/config/i386/i386.md:8828 */
  {
    "copysigntf3_var",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_copysigntf3_var,
    &operand_data[902],
    6,
    6,
    0,
    5,
    1
  },
  /* ../../gcc/config/i386/i386.md:8864 */
  {
    "*one_cmplhi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "not{w}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[826],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8864 */
  {
    "*one_cmplsi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "not{l}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[828],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8864 */
  {
    "*one_cmpldi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "not{q}\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[830],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8873 */
  {
    "*one_cmplqi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_472 },
#else
    { 0, output_472, 0 },
#endif
    0,
    &operand_data[908],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:8884 */
  {
    "*one_cmplsi2_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "not{l}\t%k0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[606],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8893 */
  {
    "*one_cmplqi2_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[824],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8893 */
  {
    "*one_cmplhi2_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[826],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8893 */
  {
    "*one_cmplsi2_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[828],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8893 */
  {
    "*one_cmpldi2_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[830],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8920 */
  {
    "*one_cmplsi2_2_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[606],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:8977 */
  {
    "*ashldi3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[910],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:8977 */
  {
    "*ashlti3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[913],
    3,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:9011 */
  {
    "x86_64_shld",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "shld{q}\t{%s2%1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_x86_64_shld,
    &operand_data[916],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9027 */
  {
    "x86_shld",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "shld{l}\t{%s2%1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_x86_shld,
    &operand_data[919],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9090 */
  {
    "*ashlsi3_mask",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_483 },
#else
    { 0, 0, output_483 },
#endif
    0,
    &operand_data[922],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9090 */
  {
    "*ashldi3_mask",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_484 },
#else
    { 0, 0, output_484 },
#endif
    0,
    &operand_data[926],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9108 */
  {
    "*bmi2_ashlsi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "shlx\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9108 */
  {
    "*bmi2_ashldi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "shlx\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9117 */
  {
    "*ashlsi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_487 },
#else
    { 0, 0, output_487 },
#endif
    0,
    &operand_data[936],
    3,
    3,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:9117 */
  {
    "*ashldi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_488 },
#else
    { 0, 0, output_488 },
#endif
    0,
    &operand_data[939],
    3,
    3,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:9177 */
  {
    "*bmi2_ashlsi3_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "shlx\t{%2, %1, %k0|%k0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[942],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9187 */
  {
    "*ashlsi3_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_490 },
#else
    { 0, 0, output_490 },
#endif
    0,
    &operand_data[945],
    3,
    3,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:9247 */
  {
    "*ashlhi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_491 },
#else
    { 0, 0, output_491 },
#endif
    0,
    &operand_data[948],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:9292 */
  {
    "*ashlqi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_492 },
#else
    { 0, 0, output_492 },
#endif
    0,
    &operand_data[951],
    3,
    3,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:9349 */
  {
    "*ashlqi3_1_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_493 },
#else
    { 0, 0, output_493 },
#endif
    0,
    &operand_data[954],
    2,
    2,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9440 */
  {
    "*ashlqi3_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_494 },
#else
    { 0, 0, output_494 },
#endif
    0,
    &operand_data[956],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9440 */
  {
    "*ashlhi3_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_495 },
#else
    { 0, 0, output_495 },
#endif
    0,
    &operand_data[959],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9440 */
  {
    "*ashlsi3_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_496 },
#else
    { 0, 0, output_496 },
#endif
    0,
    &operand_data[962],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9440 */
  {
    "*ashldi3_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_497 },
#else
    { 0, 0, output_497 },
#endif
    0,
    &operand_data[965],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9488 */
  {
    "*ashlsi3_cmp_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_498 },
#else
    { 0, 0, output_498 },
#endif
    0,
    &operand_data[968],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9536 */
  {
    "*ashlqi3_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_499 },
#else
    { 0, 0, output_499 },
#endif
    0,
    &operand_data[971],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9536 */
  {
    "*ashlhi3_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_500 },
#else
    { 0, 0, output_500 },
#endif
    0,
    &operand_data[974],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9536 */
  {
    "*ashlsi3_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_501 },
#else
    { 0, 0, output_501 },
#endif
    0,
    &operand_data[977],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9536 */
  {
    "*ashldi3_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_502 },
#else
    { 0, 0, output_502 },
#endif
    0,
    &operand_data[980],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9592 */
  {
    "*lshrsi3_mask",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_503 },
#else
    { 0, 0, output_503 },
#endif
    0,
    &operand_data[922],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9592 */
  {
    "*ashrsi3_mask",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_504 },
#else
    { 0, 0, output_504 },
#endif
    0,
    &operand_data[922],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9592 */
  {
    "*lshrdi3_mask",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_505 },
#else
    { 0, 0, output_505 },
#endif
    0,
    &operand_data[926],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9592 */
  {
    "*ashrdi3_mask",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_506 },
#else
    { 0, 0, output_506 },
#endif
    0,
    &operand_data[926],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9610 */
  {
    "*lshrdi3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[983],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9610 */
  {
    "*ashrdi3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[983],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9610 */
  {
    "*lshrti3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[986],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9610 */
  {
    "*ashrti3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[986],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9638 */
  {
    "x86_64_shrd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "shrd{q}\t{%s2%1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_x86_64_shrd,
    &operand_data[916],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9654 */
  {
    "x86_shrd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "shrd{l}\t{%s2%1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_x86_shrd,
    &operand_data[919],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9671 */
  {
    "ashrdi3_cvt",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_513 },
#else
    { 0, output_513, 0 },
#endif
    (insn_gen_fn) gen_ashrdi3_cvt,
    &operand_data[989],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:9688 */
  {
    "ashrsi3_cvt",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_514 },
#else
    { 0, output_514, 0 },
#endif
    (insn_gen_fn) gen_ashrsi3_cvt,
    &operand_data[992],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:9705 */
  {
    "*ashrsi3_cvt_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_515 },
#else
    { 0, output_515, 0 },
#endif
    0,
    &operand_data[995],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:9752 */
  {
    "*bmi2_lshrsi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "shrx\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9752 */
  {
    "*bmi2_ashrsi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sarx\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9752 */
  {
    "*bmi2_lshrdi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "shrx\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9752 */
  {
    "*bmi2_ashrdi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sarx\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9761 */
  {
    "*lshrsi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_520 },
#else
    { 0, 0, output_520 },
#endif
    0,
    &operand_data[998],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:9761 */
  {
    "*ashrsi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_521 },
#else
    { 0, 0, output_521 },
#endif
    0,
    &operand_data[998],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:9761 */
  {
    "*lshrdi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_522 },
#else
    { 0, 0, output_522 },
#endif
    0,
    &operand_data[1001],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:9761 */
  {
    "*ashrdi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_523 },
#else
    { 0, 0, output_523 },
#endif
    0,
    &operand_data[1001],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:9804 */
  {
    "*bmi2_lshrsi3_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "shrx\t{%2, %1, %k0|%k0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[942],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9804 */
  {
    "*bmi2_ashrsi3_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sarx\t{%2, %1, %k0|%k0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[942],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:9814 */
  {
    "*lshrsi3_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_526 },
#else
    { 0, 0, output_526 },
#endif
    0,
    &operand_data[1004],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:9814 */
  {
    "*ashrsi3_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_527 },
#else
    { 0, 0, output_527 },
#endif
    0,
    &operand_data[1004],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:9858 */
  {
    "*lshrqi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_528 },
#else
    { 0, 0, output_528 },
#endif
    0,
    &operand_data[1007],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9858 */
  {
    "*ashrqi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_529 },
#else
    { 0, 0, output_529 },
#endif
    0,
    &operand_data[1007],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9858 */
  {
    "*lshrhi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_530 },
#else
    { 0, 0, output_530 },
#endif
    0,
    &operand_data[1010],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9858 */
  {
    "*ashrhi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_531 },
#else
    { 0, 0, output_531 },
#endif
    0,
    &operand_data[1010],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9882 */
  {
    "*lshrqi3_1_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_532 },
#else
    { 0, 0, output_532 },
#endif
    0,
    &operand_data[954],
    2,
    2,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9882 */
  {
    "*ashrqi3_1_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_533 },
#else
    { 0, 0, output_533 },
#endif
    0,
    &operand_data[954],
    2,
    2,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9911 */
  {
    "*lshrqi3_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_534 },
#else
    { 0, 0, output_534 },
#endif
    0,
    &operand_data[956],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9911 */
  {
    "*ashrqi3_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_535 },
#else
    { 0, 0, output_535 },
#endif
    0,
    &operand_data[956],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9911 */
  {
    "*lshrhi3_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_536 },
#else
    { 0, 0, output_536 },
#endif
    0,
    &operand_data[959],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9911 */
  {
    "*ashrhi3_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_537 },
#else
    { 0, 0, output_537 },
#endif
    0,
    &operand_data[959],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9911 */
  {
    "*lshrsi3_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_538 },
#else
    { 0, 0, output_538 },
#endif
    0,
    &operand_data[962],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9911 */
  {
    "*ashrsi3_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_539 },
#else
    { 0, 0, output_539 },
#endif
    0,
    &operand_data[962],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9911 */
  {
    "*lshrdi3_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_540 },
#else
    { 0, 0, output_540 },
#endif
    0,
    &operand_data[965],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9911 */
  {
    "*ashrdi3_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_541 },
#else
    { 0, 0, output_541 },
#endif
    0,
    &operand_data[965],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9943 */
  {
    "*lshrsi3_cmp_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_542 },
#else
    { 0, 0, output_542 },
#endif
    0,
    &operand_data[968],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9943 */
  {
    "*ashrsi3_cmp_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_543 },
#else
    { 0, 0, output_543 },
#endif
    0,
    &operand_data[968],
    3,
    3,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9975 */
  {
    "*lshrqi3_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_544 },
#else
    { 0, 0, output_544 },
#endif
    0,
    &operand_data[971],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9975 */
  {
    "*ashrqi3_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_545 },
#else
    { 0, 0, output_545 },
#endif
    0,
    &operand_data[971],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9975 */
  {
    "*lshrhi3_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_546 },
#else
    { 0, 0, output_546 },
#endif
    0,
    &operand_data[974],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9975 */
  {
    "*ashrhi3_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_547 },
#else
    { 0, 0, output_547 },
#endif
    0,
    &operand_data[974],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9975 */
  {
    "*lshrsi3_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_548 },
#else
    { 0, 0, output_548 },
#endif
    0,
    &operand_data[977],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9975 */
  {
    "*ashrsi3_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_549 },
#else
    { 0, 0, output_549 },
#endif
    0,
    &operand_data[977],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9975 */
  {
    "*lshrdi3_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_550 },
#else
    { 0, 0, output_550 },
#endif
    0,
    &operand_data[980],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:9975 */
  {
    "*ashrdi3_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_551 },
#else
    { 0, 0, output_551 },
#endif
    0,
    &operand_data[980],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:10047 */
  {
    "*rotlsi3_mask",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_552 },
#else
    { 0, 0, output_552 },
#endif
    0,
    &operand_data[922],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:10047 */
  {
    "*rotrsi3_mask",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_553 },
#else
    { 0, 0, output_553 },
#endif
    0,
    &operand_data[922],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:10047 */
  {
    "*rotldi3_mask",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_554 },
#else
    { 0, 0, output_554 },
#endif
    0,
    &operand_data[926],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:10047 */
  {
    "*rotrdi3_mask",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_555 },
#else
    { 0, 0, output_555 },
#endif
    0,
    &operand_data[926],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:10068 */
  {
    "ix86_rotldi3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ix86_rotldi3_doubleword,
    &operand_data[1013],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10068 */
  {
    "ix86_rotlti3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ix86_rotlti3_doubleword,
    &operand_data[1017],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10096 */
  {
    "ix86_rotrdi3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ix86_rotrdi3_doubleword,
    &operand_data[1013],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10096 */
  {
    "ix86_rotrti3_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ix86_rotrti3_doubleword,
    &operand_data[1017],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10124 */
  {
    "*bmi2_rorxsi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rorx\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1021],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10124 */
  {
    "*bmi2_rorxdi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rorx\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1024],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10133 */
  {
    "*rotlsi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_562 },
#else
    { 0, 0, output_562 },
#endif
    0,
    &operand_data[1027],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:10133 */
  {
    "*rotrsi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_563 },
#else
    { 0, 0, output_563 },
#endif
    0,
    &operand_data[1027],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:10133 */
  {
    "*rotldi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_564 },
#else
    { 0, 0, output_564 },
#endif
    0,
    &operand_data[1030],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:10133 */
  {
    "*rotrdi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_565 },
#else
    { 0, 0, output_565 },
#endif
    0,
    &operand_data[1030],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:10189 */
  {
    "*bmi2_rorxsi3_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rorx\t{%2, %1, %k0|%k0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1033],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10199 */
  {
    "*rotlsi3_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_567 },
#else
    { 0, 0, output_567 },
#endif
    0,
    &operand_data[1036],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:10199 */
  {
    "*rotrsi3_1_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_568 },
#else
    { 0, 0, output_568 },
#endif
    0,
    &operand_data[1036],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:10257 */
  {
    "*rotlqi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_569 },
#else
    { 0, 0, output_569 },
#endif
    0,
    &operand_data[1007],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:10257 */
  {
    "*rotrqi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_570 },
#else
    { 0, 0, output_570 },
#endif
    0,
    &operand_data[1007],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:10257 */
  {
    "*rotlhi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_571 },
#else
    { 0, 0, output_571 },
#endif
    0,
    &operand_data[1010],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:10257 */
  {
    "*rotrhi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_572 },
#else
    { 0, 0, output_572 },
#endif
    0,
    &operand_data[1010],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:10280 */
  {
    "*rotlqi3_1_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_573 },
#else
    { 0, 0, output_573 },
#endif
    0,
    &operand_data[954],
    2,
    2,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:10280 */
  {
    "*rotrqi3_1_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_574 },
#else
    { 0, 0, output_574 },
#endif
    0,
    &operand_data[954],
    2,
    2,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:10393 */
  {
    "*btsq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bts{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1039],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10405 */
  {
    "*btrq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "btr{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1039],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10417 */
  {
    "*btcq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "btc{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1039],
    2,
    2,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10528 */
  {
    "*btsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bt{l}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1041],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10528 */
  {
    "*btdi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bt{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1043],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10547 */
  {
    "*setcc_di_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1045],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10561 */
  {
    "*setcc_si_1_and",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1047],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10578 */
  {
    "*setcc_si_1_movzbl",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1047],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10593 */
  {
    "*setcc_qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "set%C1\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1049],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10602 */
  {
    "*setcc_qi_slp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "set%C1\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1051],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10682 */
  {
    "setcc_sf_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_585 },
#else
    { 0, output_585, 0 },
#endif
    (insn_gen_fn) gen_setcc_sf_sse,
    &operand_data[1053],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:10682 */
  {
    "setcc_df_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_586 },
#else
    { 0, output_586, 0 },
#endif
    (insn_gen_fn) gen_setcc_df_sse,
    &operand_data[1057],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:10700 */
  {
    "*jcc_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%+j%C1\t%l0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1061],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:10718 */
  {
    "*jcc_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%+j%c1\t%l0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1061],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:10788 */
  {
    "*jcc_btsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1063],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10788 */
  {
    "*jcc_btdi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1067],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10822 */
  {
    "*jcc_btsi_mask",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1071],
    5,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10822 */
  {
    "*jcc_btdi_mask",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1076],
    5,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10856 */
  {
    "*jcc_btsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1063],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10889 */
  {
    "*jcc_btsi_mask_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1071],
    5,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10925 */
  {
    "*fp_jcc_1_387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1081],
    4,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10943 */
  {
    "*fp_jcc_1r_387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1081],
    4,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10961 */
  {
    "*fp_jcc_2_387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1086],
    4,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10976 */
  {
    "*fp_jcc_2r_387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1086],
    4,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:10991 */
  {
    "*fp_jcc_3_387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1091],
    4,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11048 */
  {
    "*fp_jcc_4_hi_387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1096],
    5,
    6,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:11048 */
  {
    "*fp_jcc_4_si_387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1102],
    5,
    6,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:11117 */
  {
    "jump",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "jmp\t%l0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_jump,
    &operand_data[852],
    1,
    1,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:11135 */
  {
    "*indirect_jump",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "jmp\t%A0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1108],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11135 */
  {
    "*indirect_jump",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "jmp\t%A0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1109],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11182 */
  {
    "*tablejump_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "jmp\t%A0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1110],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11182 */
  {
    "*tablejump_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "jmp\t%A0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1112],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11270 */
  {
    "*call_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1114],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11270 */
  {
    "*call_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1117],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11282 */
  {
    "*call",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_609 },
#else
    { 0, 0, output_609 },
#endif
    0,
    &operand_data[1114],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11282 */
  {
    "*call",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_610 },
#else
    { 0, 0, output_610 },
#endif
    0,
    &operand_data[1117],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11289 */
  {
    "*call_rex64_ms_sysv_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1117],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11314 */
  {
    "*call_rex64_ms_sysv",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_612 },
#else
    { 0, 0, output_612 },
#endif
    0,
    &operand_data[1117],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11334 */
  {
    "*sibcall_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1120],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11334 */
  {
    "*sibcall_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1123],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11346 */
  {
    "*sibcall",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_615 },
#else
    { 0, 0, output_615 },
#endif
    0,
    &operand_data[1120],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11346 */
  {
    "*sibcall",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_616 },
#else
    { 0, 0, output_616 },
#endif
    0,
    &operand_data[1123],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11366 */
  {
    "*call_pop_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1126],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11381 */
  {
    "*call_pop",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_618 },
#else
    { 0, 0, output_618 },
#endif
    0,
    &operand_data[1130],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11391 */
  {
    "*sibcall_pop_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1133],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11406 */
  {
    "*sibcall_pop",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_620 },
#else
    { 0, 0, output_620 },
#endif
    0,
    &operand_data[1133],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11442 */
  {
    "*call_value_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1113],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11442 */
  {
    "*call_value_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1137],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11455 */
  {
    "*call_value",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_623 },
#else
    { 0, 0, output_623 },
#endif
    0,
    &operand_data[1113],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11455 */
  {
    "*call_value",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_624 },
#else
    { 0, 0, output_624 },
#endif
    0,
    &operand_data[1137],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11463 */
  {
    "*sibcall_value_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1141],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11463 */
  {
    "*sibcall_value_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1145],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11476 */
  {
    "*sibcall_value",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_627 },
#else
    { 0, 0, output_627 },
#endif
    0,
    &operand_data[1141],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11476 */
  {
    "*sibcall_value",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_628 },
#else
    { 0, 0, output_628 },
#endif
    0,
    &operand_data[1145],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11484 */
  {
    "*call_value_rex64_ms_sysv_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1137],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11510 */
  {
    "*call_value_rex64_ms_sysv",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_630 },
#else
    { 0, 0, output_630 },
#endif
    0,
    &operand_data[1137],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11545 */
  {
    "*call_value_pop_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1149],
    5,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11561 */
  {
    "*call_value_pop",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_632 },
#else
    { 0, 0, output_632 },
#endif
    0,
    &operand_data[1149],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11572 */
  {
    "*sibcall_value_pop_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1154],
    5,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11588 */
  {
    "*sibcall_value_pop",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_634 },
#else
    { 0, 0, output_634 },
#endif
    0,
    &operand_data[1154],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11649 */
  {
    "blockage",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_blockage,
    &operand_data[0],
    0,
    0,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:11666 */
  {
    "*memory_blockage",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1159],
    1,
    1,
    1,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:11675 */
  {
    "prologue_use",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_prologue_use,
    &operand_data[852],
    1,
    1,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:11715 */
  {
    "simple_return_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "ret",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_simple_return_internal,
    &operand_data[0],
    0,
    0,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:11727 */
  {
    "simple_return_internal_long",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rep\n\tret",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_simple_return_internal_long,
    &operand_data[0],
    0,
    0,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:11738 */
  {
    "simple_return_pop_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "ret\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_simple_return_pop_internal,
    &operand_data[83],
    1,
    1,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:11748 */
  {
    "simple_return_indirect_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "jmp\t%A0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_simple_return_indirect_internal,
    &operand_data[920],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11756 */
  {
    "nop",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "nop",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_nop,
    &operand_data[0],
    0,
    0,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:11765 */
  {
    "nops",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_643 },
#else
    { 0, 0, output_643 },
#endif
    (insn_gen_fn) gen_nops,
    &operand_data[1116],
    1,
    1,
    0,
    0,
    3
  },
  /* ../../gcc/config/i386/i386.md:11787 */
  {
    "pad",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_644 },
#else
    { 0, 0, output_644 },
#endif
    (insn_gen_fn) gen_pad,
    &operand_data[852],
    1,
    1,
    0,
    0,
    3
  },
  /* ../../gcc/config/i386/i386.md:11808 */
  {
    "set_got",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_645 },
#else
    { 0, 0, output_645 },
#endif
    (insn_gen_fn) gen_set_got,
    &operand_data[78],
    1,
    1,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11817 */
  {
    "set_got_labelled",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_646 },
#else
    { 0, 0, output_646 },
#endif
    (insn_gen_fn) gen_set_got_labelled,
    &operand_data[1160],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:11827 */
  {
    "set_got_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lea{q}\t{_GLOBAL_OFFSET_TABLE_(%%rip), %0|%0, _GLOBAL_OFFSET_TABLE_[rip]}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_set_got_rex64,
    &operand_data[80],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11836 */
  {
    "set_rip_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lea{q}\t{%l1(%%rip), %0|%0, %l1[rip]}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_set_rip_rex64,
    &operand_data[1162],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11845 */
  {
    "set_got_offset_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movabs{q}\t{$_GLOBAL_OFFSET_TABLE_-%l1, %0|%0, OFFSET FLAT:_GLOBAL_OFFSET_TABLE_-%l1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_set_got_offset_rex64,
    &operand_data[1162],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:11886 */
  {
    "eh_return_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_eh_return_internal,
    &operand_data[0],
    0,
    0,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:11894 */
  {
    "leave",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "leave",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_leave,
    &operand_data[0],
    0,
    0,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:11902 */
  {
    "leave_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "leave",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_leave_rex64,
    &operand_data[0],
    0,
    0,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:11922 */
  {
    "split_stack_return",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_653 },
#else
    { 0, 0, output_653 },
#endif
    (insn_gen_fn) gen_split_stack_return,
    &operand_data[83],
    1,
    1,
    0,
    0,
    3
  },
  /* ../../gcc/config/i386/i386.md:11994 */
  {
    "ffssi2_no_cmove",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ffssi2_no_cmove,
    &operand_data[1164],
    2,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12018 */
  {
    "*ffssi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bsf{l}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12018 */
  {
    "*ffsdi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bsf{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12030 */
  {
    "ctzhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_657 },
#else
    { 0, 0, output_657 },
#endif
    (insn_gen_fn) gen_ctzhi2,
    &operand_data[1167],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12030 */
  {
    "ctzsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_658 },
#else
    { 0, 0, output_658 },
#endif
    (insn_gen_fn) gen_ctzsi2,
    &operand_data[930],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12030 */
  {
    "ctzdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_659 },
#else
    { 0, 0, output_659 },
#endif
    (insn_gen_fn) gen_ctzdi2,
    &operand_data[933],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12066 */
  {
    "clzhi2_lzcnt",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lzcnt{w}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_clzhi2_lzcnt,
    &operand_data[1167],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12066 */
  {
    "clzsi2_lzcnt",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lzcnt{l}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_clzsi2_lzcnt,
    &operand_data[930],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12066 */
  {
    "clzdi2_lzcnt",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lzcnt{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_clzdi2_lzcnt,
    &operand_data[933],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12077 */
  {
    "*bmi_andn_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "andn\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1169],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12077 */
  {
    "*bmi_andn_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "andn\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1172],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12089 */
  {
    "bmi_bextr_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bextr\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_bmi_bextr_si,
    &operand_data[930],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12089 */
  {
    "bmi_bextr_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bextr\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_bmi_bextr_di,
    &operand_data[933],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12100 */
  {
    "*bmi_blsi_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blsi\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12100 */
  {
    "*bmi_blsi_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blsi\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12112 */
  {
    "*bmi_blsmsk_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blsmsk\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12112 */
  {
    "*bmi_blsmsk_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blsmsk\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12125 */
  {
    "*bmi_blsr_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blsr\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12125 */
  {
    "*bmi_blsr_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blsr\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12139 */
  {
    "bmi2_bzhi_si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bzhi\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_bmi2_bzhi_si3,
    &operand_data[930],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12139 */
  {
    "bmi2_bzhi_di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bzhi\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_bmi2_bzhi_di3,
    &operand_data[933],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12151 */
  {
    "bmi2_pdep_si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pdep\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_bmi2_pdep_si3,
    &operand_data[1169],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12151 */
  {
    "bmi2_pdep_di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pdep\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_bmi2_pdep_di3,
    &operand_data[1172],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12162 */
  {
    "bmi2_pext_si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pext\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_bmi2_pext_si3,
    &operand_data[1169],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12162 */
  {
    "bmi2_pext_di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pext\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_bmi2_pext_di3,
    &operand_data[1172],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12174 */
  {
    "tbm_bextri_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_679 },
#else
    { 0, 0, output_679 },
#endif
    (insn_gen_fn) gen_tbm_bextri_si,
    &operand_data[1175],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12174 */
  {
    "tbm_bextri_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_680 },
#else
    { 0, 0, output_680 },
#endif
    (insn_gen_fn) gen_tbm_bextri_di,
    &operand_data[1179],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12189 */
  {
    "*tbm_blcfill_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blcfill\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12189 */
  {
    "*tbm_blcfill_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blcfill\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12202 */
  {
    "*tbm_blci_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blci\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12202 */
  {
    "*tbm_blci_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blci\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12216 */
  {
    "*tbm_blcic_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blcic\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12216 */
  {
    "*tbm_blcic_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blcic\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12230 */
  {
    "*tbm_blcmsk_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blcmsk\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12230 */
  {
    "*tbm_blcmsk_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blcmsk\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12243 */
  {
    "*tbm_blcs_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blcs\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12243 */
  {
    "*tbm_blcs_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blcs\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12256 */
  {
    "*tbm_blsfill_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blsfill\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12256 */
  {
    "*tbm_blsfill_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blsfill\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12269 */
  {
    "*tbm_blsic_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blsic\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12269 */
  {
    "*tbm_blsic_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "blsic\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12283 */
  {
    "*tbm_t1mskc_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "t1mskc\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12283 */
  {
    "*tbm_t1mskc_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "t1mskc\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12297 */
  {
    "*tbm_tzmsk_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "tzmsk\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12297 */
  {
    "*tbm_tzmsk_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "tzmsk\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12311 */
  {
    "bsr_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bsr{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_bsr_rex64,
    &operand_data[933],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12322 */
  {
    "bsr",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bsr{l}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_bsr,
    &operand_data[930],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12333 */
  {
    "*bsrhi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bsr{w}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1167],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12344 */
  {
    "popcounthi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_702 },
#else
    { 0, 0, output_702 },
#endif
    (insn_gen_fn) gen_popcounthi2,
    &operand_data[1167],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12344 */
  {
    "popcountsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_703 },
#else
    { 0, 0, output_703 },
#endif
    (insn_gen_fn) gen_popcountsi2,
    &operand_data[930],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12344 */
  {
    "popcountdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_704 },
#else
    { 0, 0, output_704 },
#endif
    (insn_gen_fn) gen_popcountdi2,
    &operand_data[933],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12361 */
  {
    "*popcounthi2_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_705 },
#else
    { 0, 0, output_705 },
#endif
    0,
    &operand_data[1167],
    2,
    2,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12361 */
  {
    "*popcountsi2_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_706 },
#else
    { 0, 0, output_706 },
#endif
    0,
    &operand_data[930],
    2,
    2,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12361 */
  {
    "*popcountdi2_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_707 },
#else
    { 0, 0, output_707 },
#endif
    0,
    &operand_data[933],
    2,
    2,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12381 */
  {
    "*popcountsi2_cmp_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_708 },
#else
    { 0, 0, output_708 },
#endif
    0,
    &operand_data[942],
    2,
    2,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12417 */
  {
    "*bswapsi2_movbe",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_709 },
#else
    { 0, output_709, 0 },
#endif
    0,
    &operand_data[1183],
    2,
    2,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:12417 */
  {
    "*bswapdi2_movbe",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_710 },
#else
    { 0, output_710, 0 },
#endif
    0,
    &operand_data[1185],
    2,
    2,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/i386.md:12432 */
  {
    "*bswapsi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bswap\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1187],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12432 */
  {
    "*bswapdi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "bswap\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[832],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12441 */
  {
    "*bswaphi_lowpart_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_713 },
#else
    { 0, output_713, 0 },
#endif
    0,
    &operand_data[1189],
    1,
    1,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:12452 */
  {
    "bswaphi_lowpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rol{w}\t{$8, %0|%0, 8}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_bswaphi_lowpart,
    &operand_data[134],
    1,
    1,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12508 */
  {
    "paritydi2_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_paritydi2_cmp,
    &operand_data[1190],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12539 */
  {
    "paritysi2_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_paritysi2_cmp,
    &operand_data[1194],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12563 */
  {
    "*parityhi2_cmp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xor{b}\t{%h0, %b0|%b0, %h0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1197],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12579 */
  {
    "*tls_global_dynamic_32_gnu",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_718 },
#else
    { 0, 0, output_718 },
#endif
    0,
    &operand_data[1199],
    4,
    6,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12615 */
  {
    "*tls_global_dynamic_64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_719 },
#else
    { 0, 0, output_719 },
#endif
    0,
    &operand_data[1205],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12647 */
  {
    "*tls_local_dynamic_base_32_gnu",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_720 },
#else
    { 0, 0, output_720 },
#endif
    0,
    &operand_data[1209],
    3,
    5,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12682 */
  {
    "*tls_local_dynamic_base_64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_721 },
#else
    { 0, 0, output_721 },
#endif
    0,
    &operand_data[1214],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:12710 */
  {
    "*tls_local_dynamic_32_once",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1217],
    4,
    6,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12737 */
  {
    "*load_tp_x32",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{l}\t{%%fs:0, %0|%0, DWORD PTR fs:0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[78],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12748 */
  {
    "*load_tp_x32_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{l}\t{%%fs:0, %k0|%k0, DWORD PTR fs:0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[80],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12759 */
  {
    "*load_tp_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{l}\t{%%gs:0, %0|%0, DWORD PTR gs:0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[78],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12759 */
  {
    "*load_tp_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{q}\t{%%fs:0, %0|%0, QWORD PTR fs:0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[80],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12770 */
  {
    "*add_tp_x32",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{l}\t{%%fs:0, %0|%0, DWORD PTR fs:0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1187],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12783 */
  {
    "*add_tp_x32_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{l}\t{%%fs:0, %k0|%k0, DWORD PTR fs:0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[606],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12797 */
  {
    "*add_tp_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{l}\t{%%gs:0, %0|%0, DWORD PTR gs:0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1187],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12797 */
  {
    "*add_tp_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "add{q}\t{%%fs:0, %0|%0, QWORD PTR fs:0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[832],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12846 */
  {
    "*tls_dynamic_gnu2_lea_32",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lea{l}\t{%E2@TLSDESC(%1), %0|%0, %E2@TLSDESC[%1]}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1223],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12859 */
  {
    "*tls_dynamic_gnu2_call_32",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "call\t{*%a1@TLSCALL(%2)|[DWORD PTR [%2+%a1@TLSCALL]]}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1226],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12874 */
  {
    "*tls_dynamic_gnu2_combine_32",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1230],
    5,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12910 */
  {
    "*tls_dynamic_gnu2_lea_64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lea{q}\t{%E1@TLSDESC(%%rip), %0|%0, %E1@TLSDESC[rip]}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1235],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12921 */
  {
    "*tls_dynamic_gnu2_call_64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "call\t{*%a1@TLSCALL(%2)|[QWORD PTR [%2+%a1@TLSCALL]]}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1237],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12934 */
  {
    "*tls_dynamic_gnu2_combine_64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1240],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:12965 */
  {
    "*fop_sf_comm_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_737 },
#else
    { 0, 0, output_737 },
#endif
    0,
    &operand_data[1244],
    4,
    4,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:12965 */
  {
    "*fop_df_comm_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_738 },
#else
    { 0, 0, output_738 },
#endif
    0,
    &operand_data[1248],
    4,
    4,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/i386.md:12986 */
  {
    "*fop_sf_comm_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_739 },
#else
    { 0, 0, output_739 },
#endif
    0,
    &operand_data[1252],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:12986 */
  {
    "*fop_df_comm_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_740 },
#else
    { 0, 0, output_740 },
#endif
    0,
    &operand_data[1256],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13003 */
  {
    "*fop_sf_comm_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_741 },
#else
    { 0, 0, output_741 },
#endif
    0,
    &operand_data[1260],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:13003 */
  {
    "*fop_df_comm_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_742 },
#else
    { 0, 0, output_742 },
#endif
    0,
    &operand_data[1264],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:13018 */
  {
    "*fop_sf_1_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_743 },
#else
    { 0, 0, output_743 },
#endif
    0,
    &operand_data[1268],
    4,
    4,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/i386.md:13018 */
  {
    "*fop_df_1_mixed",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_744 },
#else
    { 0, 0, output_744 },
#endif
    0,
    &operand_data[1272],
    4,
    4,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/i386.md:13046 */
  {
    "*rcpsf2_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vrcpss\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1276],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13057 */
  {
    "*fop_sf_1_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_746 },
#else
    { 0, 0, output_746 },
#endif
    0,
    &operand_data[1278],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13057 */
  {
    "*fop_df_1_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_747 },
#else
    { 0, 0, output_747 },
#endif
    0,
    &operand_data[1282],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13077 */
  {
    "*fop_sf_1_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_748 },
#else
    { 0, 0, output_748 },
#endif
    0,
    &operand_data[1286],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13077 */
  {
    "*fop_df_1_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_749 },
#else
    { 0, 0, output_749 },
#endif
    0,
    &operand_data[1290],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13097 */
  {
    "*fop_sf_2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_750 },
#else
    { 0, 0, output_750 },
#endif
    0,
    &operand_data[1294],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13097 */
  {
    "*fop_df_2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_751 },
#else
    { 0, 0, output_751 },
#endif
    0,
    &operand_data[1298],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13097 */
  {
    "*fop_sf_2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_752 },
#else
    { 0, 0, output_752 },
#endif
    0,
    &operand_data[1302],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13097 */
  {
    "*fop_df_2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_753 },
#else
    { 0, 0, output_753 },
#endif
    0,
    &operand_data[1306],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13117 */
  {
    "*fop_sf_3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_754 },
#else
    { 0, 0, output_754 },
#endif
    0,
    &operand_data[1310],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13117 */
  {
    "*fop_df_3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_755 },
#else
    { 0, 0, output_755 },
#endif
    0,
    &operand_data[1314],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13117 */
  {
    "*fop_sf_3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_756 },
#else
    { 0, 0, output_756 },
#endif
    0,
    &operand_data[1318],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13117 */
  {
    "*fop_df_3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_757 },
#else
    { 0, 0, output_757 },
#endif
    0,
    &operand_data[1322],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13137 */
  {
    "*fop_df_4_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_758 },
#else
    { 0, 0, output_758 },
#endif
    0,
    &operand_data[1326],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13156 */
  {
    "*fop_df_5_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_759 },
#else
    { 0, 0, output_759 },
#endif
    0,
    &operand_data[1330],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13174 */
  {
    "*fop_df_6_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_760 },
#else
    { 0, 0, output_760 },
#endif
    0,
    &operand_data[1334],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13193 */
  {
    "*fop_xf_comm_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_761 },
#else
    { 0, 0, output_761 },
#endif
    0,
    &operand_data[1338],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:13207 */
  {
    "*fop_xf_1_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_762 },
#else
    { 0, 0, output_762 },
#endif
    0,
    &operand_data[1342],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13224 */
  {
    "*fop_xf_2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_763 },
#else
    { 0, 0, output_763 },
#endif
    0,
    &operand_data[1346],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13224 */
  {
    "*fop_xf_2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_764 },
#else
    { 0, 0, output_764 },
#endif
    0,
    &operand_data[1350],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13242 */
  {
    "*fop_xf_3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_765 },
#else
    { 0, 0, output_765 },
#endif
    0,
    &operand_data[1354],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13242 */
  {
    "*fop_xf_3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_766 },
#else
    { 0, 0, output_766 },
#endif
    0,
    &operand_data[1358],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13260 */
  {
    "*fop_xf_4_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_767 },
#else
    { 0, 0, output_767 },
#endif
    0,
    &operand_data[1362],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13260 */
  {
    "*fop_xf_4_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_768 },
#else
    { 0, 0, output_768 },
#endif
    0,
    &operand_data[1366],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13277 */
  {
    "*fop_xf_5_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_769 },
#else
    { 0, 0, output_769 },
#endif
    0,
    &operand_data[1370],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13277 */
  {
    "*fop_xf_5_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_770 },
#else
    { 0, 0, output_770 },
#endif
    0,
    &operand_data[1374],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13294 */
  {
    "*fop_xf_6_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_771 },
#else
    { 0, 0, output_771 },
#endif
    0,
    &operand_data[1378],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13294 */
  {
    "*fop_xf_6_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_772 },
#else
    { 0, 0, output_772 },
#endif
    0,
    &operand_data[1382],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:13359 */
  {
    "truncxfsf2_i387_noop_unspec",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_773 },
#else
    { 0, 0, output_773 },
#endif
    (insn_gen_fn) gen_truncxfsf2_i387_noop_unspec,
    &operand_data[255],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:13359 */
  {
    "truncxfdf2_i387_noop_unspec",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_774 },
#else
    { 0, 0, output_774 },
#endif
    (insn_gen_fn) gen_truncxfdf2_i387_noop_unspec,
    &operand_data[257],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:13368 */
  {
    "sqrtxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fsqrt",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sqrtxf2,
    &operand_data[870],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13379 */
  {
    "sqrt_extendsfxf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fsqrt",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sqrt_extendsfxf2_i387,
    &operand_data[874],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13379 */
  {
    "sqrt_extenddfxf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fsqrt",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sqrt_extenddfxf2_i387,
    &operand_data[876],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13392 */
  {
    "*rsqrtsf2_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vrsqrtss\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1276],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13413 */
  {
    "*sqrtsf2_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vsqrtss\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1276],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13413 */
  {
    "*sqrtdf2_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vsqrtsd\t{%1, %d0|%d0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1386],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13456 */
  {
    "fpremxf4_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fprem",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fpremxf4_i387,
    &operand_data[1388],
    4,
    4,
    4,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13527 */
  {
    "fprem1xf4_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fprem1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fprem1xf4_i387,
    &operand_data[1388],
    4,
    4,
    4,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13599 */
  {
    "*sinxf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fsin",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[870],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13608 */
  {
    "*sin_extendsfxf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fsin",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[874],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13608 */
  {
    "*sin_extenddfxf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fsin",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[876],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13621 */
  {
    "*cosxf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fcos",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[870],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13630 */
  {
    "*cos_extendsfxf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fcos",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[874],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13630 */
  {
    "*cos_extenddfxf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fcos",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[876],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13649 */
  {
    "sincosxf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fsincos",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sincosxf3,
    &operand_data[1388],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13681 */
  {
    "sincos_extendsfxf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fsincos",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sincos_extendsfxf3_i387,
    &operand_data[1392],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13681 */
  {
    "sincos_extenddfxf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fsincos",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sincos_extenddfxf3_i387,
    &operand_data[1395],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13738 */
  {
    "fptanxf4_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fptan",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fptanxf4_i387,
    &operand_data[1398],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13751 */
  {
    "fptan_extendsfxf4_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fptan",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fptan_extendsfxf4_i387,
    &operand_data[1402],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13751 */
  {
    "fptan_extenddfxf4_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fptan",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fptan_extenddfxf4_i387,
    &operand_data[1406],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13799 */
  {
    "*fpatanxf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fpatan",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1410],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13811 */
  {
    "fpatan_extendsfxf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fpatan",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fpatan_extendsfxf3_i387,
    &operand_data[1414],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13811 */
  {
    "fpatan_extenddfxf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fpatan",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fpatan_extenddfxf3_i387,
    &operand_data[1418],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13971 */
  {
    "fyl2xxf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fyl2x",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fyl2xxf3_i387,
    &operand_data[1410],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13983 */
  {
    "fyl2x_extendsfxf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fyl2x",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fyl2x_extendsfxf3_i387,
    &operand_data[1422],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:13983 */
  {
    "fyl2x_extenddfxf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fyl2x",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fyl2x_extenddfxf3_i387,
    &operand_data[1426],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14088 */
  {
    "fyl2xp1xf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fyl2xp1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fyl2xp1xf3_i387,
    &operand_data[1410],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14100 */
  {
    "fyl2xp1_extendsfxf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fyl2xp1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fyl2xp1_extendsfxf3_i387,
    &operand_data[1422],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14100 */
  {
    "fyl2xp1_extenddfxf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fyl2xp1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fyl2xp1_extenddfxf3_i387,
    &operand_data[1426],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14150 */
  {
    "fxtractxf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fxtract",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fxtractxf3_i387,
    &operand_data[1388],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14162 */
  {
    "fxtract_extendsfxf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fxtract",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fxtract_extendsfxf3_i387,
    &operand_data[1392],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14162 */
  {
    "fxtract_extenddfxf3_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fxtract",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fxtract_extenddfxf3_i387,
    &operand_data[1395],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14243 */
  {
    "*f2xm1xf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "f2xm1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[870],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14253 */
  {
    "*fscalexf4_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fscale",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1388],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14588 */
  {
    "sse4_1_roundsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vroundss\t{%2, %1, %d0|%d0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_roundsf2,
    &operand_data[1430],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14588 */
  {
    "sse4_1_rounddf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vroundsd\t{%2, %1, %d0|%d0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_rounddf2,
    &operand_data[1433],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14600 */
  {
    "rintxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "frndint",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_rintxf2,
    &operand_data[870],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14678 */
  {
    "*fistdi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1436],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:14701 */
  {
    "fistdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_813 },
#else
    { 0, 0, output_813 },
#endif
    (insn_gen_fn) gen_fistdi2,
    &operand_data[1438],
    2,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:14711 */
  {
    "fistdi2_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fistdi2_with_temp,
    &operand_data[1441],
    3,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:14743 */
  {
    "*fisthi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[340],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:14743 */
  {
    "*fistsi2_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[361],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:14761 */
  {
    "fisthi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_817 },
#else
    { 0, 0, output_817 },
#endif
    (insn_gen_fn) gen_fisthi2,
    &operand_data[1445],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:14761 */
  {
    "fistsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_818 },
#else
    { 0, 0, output_818 },
#endif
    (insn_gen_fn) gen_fistsi2,
    &operand_data[1447],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:14770 */
  {
    "fisthi2_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fisthi2_with_temp,
    &operand_data[1449],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14770 */
  {
    "fistsi2_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fistsi2_with_temp,
    &operand_data[1452],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14836 */
  {
    "frndintxf2_floor",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_frndintxf2_floor,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:14861 */
  {
    "frndintxf2_floor_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fldcw\t%3\n\tfrndint\n\tfldcw\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_frndintxf2_floor_i387,
    &operand_data[1457],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:14926 */
  {
    "*fisthi2_floor_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1461],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:14926 */
  {
    "*fistsi2_floor_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1463],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:14926 */
  {
    "*fistdi2_floor_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1436],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:14958 */
  {
    "fistdi2_floor",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_826 },
#else
    { 0, 0, output_826 },
#endif
    (insn_gen_fn) gen_fistdi2_floor,
    &operand_data[1465],
    4,
    5,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:14972 */
  {
    "fistdi2_floor_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fistdi2_floor_with_temp,
    &operand_data[1470],
    5,
    6,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:15018 */
  {
    "fisthi2_floor",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_828 },
#else
    { 0, 0, output_828 },
#endif
    (insn_gen_fn) gen_fisthi2_floor,
    &operand_data[1476],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:15018 */
  {
    "fistsi2_floor",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_829 },
#else
    { 0, 0, output_829 },
#endif
    (insn_gen_fn) gen_fistsi2_floor,
    &operand_data[1480],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:15031 */
  {
    "fisthi2_floor_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fisthi2_floor_with_temp,
    &operand_data[1484],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:15031 */
  {
    "fistsi2_floor_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fistsi2_floor_with_temp,
    &operand_data[1489],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:15094 */
  {
    "frndintxf2_ceil",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_frndintxf2_ceil,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:15119 */
  {
    "frndintxf2_ceil_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fldcw\t%3\n\tfrndint\n\tfldcw\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_frndintxf2_ceil_i387,
    &operand_data[1457],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15184 */
  {
    "*fisthi2_ceil_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1461],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:15184 */
  {
    "*fistsi2_ceil_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1463],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:15184 */
  {
    "*fistdi2_ceil_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1436],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:15216 */
  {
    "fistdi2_ceil",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_837 },
#else
    { 0, 0, output_837 },
#endif
    (insn_gen_fn) gen_fistdi2_ceil,
    &operand_data[1465],
    4,
    5,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:15230 */
  {
    "fistdi2_ceil_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fistdi2_ceil_with_temp,
    &operand_data[1470],
    5,
    6,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:15276 */
  {
    "fisthi2_ceil",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_839 },
#else
    { 0, 0, output_839 },
#endif
    (insn_gen_fn) gen_fisthi2_ceil,
    &operand_data[1476],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:15276 */
  {
    "fistsi2_ceil",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_840 },
#else
    { 0, 0, output_840 },
#endif
    (insn_gen_fn) gen_fistsi2_ceil,
    &operand_data[1480],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:15289 */
  {
    "fisthi2_ceil_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fisthi2_ceil_with_temp,
    &operand_data[1484],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:15289 */
  {
    "fistsi2_ceil_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fistsi2_ceil_with_temp,
    &operand_data[1489],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:15350 */
  {
    "frndintxf2_trunc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_frndintxf2_trunc,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:15375 */
  {
    "frndintxf2_trunc_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fldcw\t%3\n\tfrndint\n\tfldcw\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_frndintxf2_trunc_i387,
    &operand_data[1457],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15441 */
  {
    "frndintxf2_mask_pm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_frndintxf2_mask_pm,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:15466 */
  {
    "frndintxf2_mask_pm_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fldcw\t%3\n\tfrndint\n\tfclex\n\tfldcw\t%2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_frndintxf2_mask_pm_i387,
    &operand_data[1457],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15507 */
  {
    "fxamsf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fxam\n\tfnstsw\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fxamsf2_i387,
    &operand_data[34],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15507 */
  {
    "fxamdf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fxam\n\tfnstsw\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fxamdf2_i387,
    &operand_data[37],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15507 */
  {
    "fxamxf2_i387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fxam\n\tfnstsw\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fxamxf2_i387,
    &operand_data[31],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15519 */
  {
    "fxamsf2_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fxamsf2_i387_with_temp,
    &operand_data[1494],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:15519 */
  {
    "fxamdf2_i387_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fxamdf2_i387_with_temp,
    &operand_data[1496],
    2,
    2,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:15615 */
  {
    "movmsk_df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovmskpd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_movmsk_df,
    &operand_data[1498],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15666 */
  {
    "cld",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cld",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_cld,
    &operand_data[0],
    0,
    0,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/i386.md:15732 */
  {
    "*strmovdi_rex_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movsq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1500],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15748 */
  {
    "*strmovsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movs{l|d}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1504],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15748 */
  {
    "*strmovsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movs{l|d}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1500],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15763 */
  {
    "*strmovhi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movsw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1504],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15763 */
  {
    "*strmovhi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movsw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1500],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15778 */
  {
    "*strmovqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movsb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1504],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15778 */
  {
    "*strmovqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movsb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1500],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15810 */
  {
    "*rep_movdi_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rep{%;} movsq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1508],
    6,
    6,
    4,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15830 */
  {
    "*rep_movsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rep{%;} movs{l|d}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1514],
    6,
    6,
    4,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15830 */
  {
    "*rep_movsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rep{%;} movs{l|d}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1508],
    6,
    6,
    4,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15849 */
  {
    "*rep_movqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rep{%;} movsb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1514],
    6,
    6,
    4,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15849 */
  {
    "*rep_movqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rep{%;} movsb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1508],
    6,
    6,
    4,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15921 */
  {
    "*strsetdi_rex_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "stosq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1520],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15935 */
  {
    "*strsetsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "stos{l|d}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1523],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15935 */
  {
    "*strsetsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "stos{l|d}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1526],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15948 */
  {
    "*strsethi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "stosw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1529],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15948 */
  {
    "*strsethi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "stosw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1532],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15961 */
  {
    "*strsetqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "stosb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1535],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15961 */
  {
    "*strsetqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "stosb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1538],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:15989 */
  {
    "*rep_stosdi_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rep{%;} stosq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1541],
    5,
    5,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16007 */
  {
    "*rep_stossi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rep{%;} stos{l|d}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1546],
    5,
    5,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16007 */
  {
    "*rep_stossi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rep{%;} stos{l|d}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1551],
    5,
    5,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16024 */
  {
    "*rep_stosqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rep{%;} stosb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1556],
    5,
    5,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16024 */
  {
    "*rep_stosqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rep{%;} stosb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1561],
    5,
    5,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16145 */
  {
    "*cmpstrnqi_nz_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "repz{%;} cmpsb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1566],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16145 */
  {
    "*cmpstrnqi_nz_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "repz{%;} cmpsb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1573],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16182 */
  {
    "*cmpstrnqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "repz{%;} cmpsb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1566],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16182 */
  {
    "*cmpstrnqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "repz{%;} cmpsb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1573],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16227 */
  {
    "*strlenqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "repnz{%;} scasb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1580],
    6,
    6,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16227 */
  {
    "*strlenqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "repnz{%;} scasb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1586],
    6,
    6,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16348 */
  {
    "*x86_movsicc_0_m1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sbb{l}\t%0, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1592],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16348 */
  {
    "*x86_movdicc_0_m1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sbb{q}\t%0, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1594],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16367 */
  {
    "*x86_movsicc_0_m1_se",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sbb{l}\t%0, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1592],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16367 */
  {
    "*x86_movdicc_0_m1_se",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sbb{q}\t%0, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1594],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16384 */
  {
    "*x86_movsicc_0_m1_neg",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sbb{l}\t%0, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1592],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16384 */
  {
    "*x86_movdicc_0_m1_neg",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sbb{q}\t%0, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1594],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16399 */
  {
    "*movhicc_noc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_890 },
#else
    { 0, output_890, 0 },
#endif
    0,
    &operand_data[1596],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:16399 */
  {
    "*movsicc_noc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_891 },
#else
    { 0, output_891, 0 },
#endif
    0,
    &operand_data[1600],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:16399 */
  {
    "*movdicc_noc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_892 },
#else
    { 0, output_892, 0 },
#endif
    0,
    &operand_data[1604],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:16412 */
  {
    "*movqicc_noc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1608],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/i386.md:16451 */
  {
    "*movxfcc_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_894 },
#else
    { 0, output_894, 0 },
#endif
    0,
    &operand_data[1612],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:16464 */
  {
    "*movdfcc_1_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_895 },
#else
    { 0, output_895, 0 },
#endif
    0,
    &operand_data[1616],
    4,
    4,
    0,
    4,
    2
  },
  /* ../../gcc/config/i386/i386.md:16480 */
  {
    "*movdfcc_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_896 },
#else
    { 0, output_896, 0 },
#endif
    0,
    &operand_data[1620],
    4,
    4,
    0,
    4,
    2
  },
  /* ../../gcc/config/i386/i386.md:16512 */
  {
    "*movsfcc_1_387",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_897 },
#else
    { 0, output_897, 0 },
#endif
    0,
    &operand_data[1624],
    4,
    4,
    0,
    4,
    2
  },
  /* ../../gcc/config/i386/i386.md:16532 */
  {
    "*xop_pcmov_sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%1, %3, %2, %0|%0, %2, %3, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1628],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16532 */
  {
    "*xop_pcmov_df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%1, %3, %2, %0|%0, %2, %3, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1632],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16547 */
  {
    "smaxsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_900 },
#else
    { 0, output_900, 0 },
#endif
    (insn_gen_fn) gen_smaxsf3,
    &operand_data[1252],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:16547 */
  {
    "sminsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_901 },
#else
    { 0, output_901, 0 },
#endif
    (insn_gen_fn) gen_sminsf3,
    &operand_data[1252],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:16547 */
  {
    "smaxdf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_902 },
#else
    { 0, output_902, 0 },
#endif
    (insn_gen_fn) gen_smaxdf3,
    &operand_data[1256],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:16547 */
  {
    "smindf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_903 },
#else
    { 0, output_903, 0 },
#endif
    (insn_gen_fn) gen_smindf3,
    &operand_data[1256],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:16567 */
  {
    "*ieee_sminsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_904 },
#else
    { 0, output_904, 0 },
#endif
    0,
    &operand_data[1053],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:16567 */
  {
    "*ieee_smindf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_905 },
#else
    { 0, output_905, 0 },
#endif
    0,
    &operand_data[1057],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:16582 */
  {
    "*ieee_smaxsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_906 },
#else
    { 0, output_906, 0 },
#endif
    0,
    &operand_data[1053],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:16582 */
  {
    "*ieee_smaxdf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_907 },
#else
    { 0, output_907, 0 },
#endif
    0,
    &operand_data[1057],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/i386.md:16650 */
  {
    "pro_epilogue_adjust_stack_si_add",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_908 },
#else
    { 0, 0, output_908 },
#endif
    (insn_gen_fn) gen_pro_epilogue_adjust_stack_si_add,
    &operand_data[1636],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:16650 */
  {
    "pro_epilogue_adjust_stack_di_add",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_909 },
#else
    { 0, 0, output_909 },
#endif
    (insn_gen_fn) gen_pro_epilogue_adjust_stack_di_add,
    &operand_data[1639],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/i386.md:16693 */
  {
    "pro_epilogue_adjust_stack_si_sub",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_pro_epilogue_adjust_stack_si_sub,
    &operand_data[1642],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16693 */
  {
    "pro_epilogue_adjust_stack_di_sub",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sub{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_pro_epilogue_adjust_stack_di_sub,
    &operand_data[1645],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16704 */
  {
    "allocate_stack_worker_probe_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "call\t___chkstk_ms",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_allocate_stack_worker_probe_si,
    &operand_data[1648],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16704 */
  {
    "allocate_stack_worker_probe_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "call\t___chkstk_ms",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_allocate_stack_worker_probe_di,
    &operand_data[1650],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:16764 */
  {
    "adjust_stack_and_probesi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_914 },
#else
    { 0, 0, output_914 },
#endif
    (insn_gen_fn) gen_adjust_stack_and_probesi,
    &operand_data[1652],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:16764 */
  {
    "adjust_stack_and_probedi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_915 },
#else
    { 0, 0, output_915 },
#endif
    (insn_gen_fn) gen_adjust_stack_and_probedi,
    &operand_data[1655],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:16776 */
  {
    "probe_stack_rangesi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_916 },
#else
    { 0, 0, output_916 },
#endif
    (insn_gen_fn) gen_probe_stack_rangesi,
    &operand_data[1652],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:16776 */
  {
    "probe_stack_rangedi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_917 },
#else
    { 0, 0, output_917 },
#endif
    (insn_gen_fn) gen_probe_stack_rangedi,
    &operand_data[1655],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:17693 */
  {
    "trap",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_918 },
#else
    { 0, 0, output_918 },
#endif
    (insn_gen_fn) gen_trap,
    &operand_data[0],
    0,
    0,
    0,
    0,
    3
  },
  /* ../../gcc/config/i386/i386.md:17723 */
  {
    "*prefetch_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_919 },
#else
    { 0, 0, output_919 },
#endif
    0,
    &operand_data[1658],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:17744 */
  {
    "*prefetch_3dnow",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_920 },
#else
    { 0, 0, output_920 },
#endif
    0,
    &operand_data[1660],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/i386.md:17782 */
  {
    "stack_protect_set_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{l}\t{%1, %2|%2, %1}\n\tmov{l}\t{%2, %0|%0, %2}\n\txor{l}\t%k2, %k2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_stack_protect_set_si,
    &operand_data[1662],
    2,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17782 */
  {
    "stack_protect_set_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{q}\t{%1, %2|%2, %1}\n\tmov{q}\t{%2, %0|%0, %2}\n\txor{l}\t%k2, %k2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_stack_protect_set_di,
    &operand_data[1665],
    2,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17792 */
  {
    "stack_tls_protect_set_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{l}\t{%@:%P1, %2|%2, DWORD PTR %@:%P1}\n\tmov{l}\t{%2, %0|%0, %2}\n\txor{l}\t%k2, %k2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_stack_tls_protect_set_si,
    &operand_data[1668],
    2,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17792 */
  {
    "stack_tls_protect_set_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{q}\t{%@:%P1, %2|%2, QWORD PTR %@:%P1}\n\tmov{q}\t{%2, %0|%0, %2}\n\txor{l}\t%k2, %k2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_stack_tls_protect_set_di,
    &operand_data[1671],
    2,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17830 */
  {
    "stack_protect_test_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{l}\t{%1, %3|%3, %1}\n\txor{l}\t{%2, %3|%3, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_stack_protect_test_si,
    &operand_data[1674],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17830 */
  {
    "stack_protect_test_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{q}\t{%1, %3|%3, %1}\n\txor{q}\t{%2, %3|%3, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_stack_protect_test_di,
    &operand_data[1678],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17840 */
  {
    "stack_tls_protect_test_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{l}\t{%1, %3|%3, %1}\n\txor{l}\t{%@:%P2, %3|%3, DWORD PTR %@:%P2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_stack_tls_protect_test_si,
    &operand_data[1682],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17840 */
  {
    "stack_tls_protect_test_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{q}\t{%1, %3|%3, %1}\n\txor{q}\t{%@:%P2, %3|%3, QWORD PTR %@:%P2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_stack_tls_protect_test_di,
    &operand_data[1686],
    3,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17850 */
  {
    "sse4_2_crc32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "crc32{b}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_2_crc32qi,
    &operand_data[1690],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17850 */
  {
    "sse4_2_crc32hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "crc32{w}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_2_crc32hi,
    &operand_data[1693],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17850 */
  {
    "sse4_2_crc32si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "crc32{l}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_2_crc32si,
    &operand_data[1696],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17871 */
  {
    "sse4_2_crc32di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "crc32{q}\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_2_crc32di,
    &operand_data[1699],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17920 */
  {
    "*rdpmc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rdpmc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1702],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17929 */
  {
    "*rdpmc_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rdpmc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1704],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17966 */
  {
    "*rdtsc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rdtsc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[690],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:17974 */
  {
    "*rdtsc_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rdtsc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[733],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18025 */
  {
    "*rdtscp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rdtscp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1707],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18035 */
  {
    "*rdtscp_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rdtscp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1709],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18058 */
  {
    "*lwp_llwpcbsi1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "llwpcb\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[920],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18058 */
  {
    "*lwp_llwpcbdi1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "llwpcb\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[917],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18082 */
  {
    "lwp_slwpcbsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "slwpcb\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_lwp_slwpcbsi,
    &operand_data[78],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18082 */
  {
    "lwp_slwpcbdi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "slwpcb\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_lwp_slwpcbdi,
    &operand_data[80],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18100 */
  {
    "*lwp_lwpvalsi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lwpval\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1712],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18100 */
  {
    "*lwp_lwpvaldi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lwpval\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1715],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18122 */
  {
    "*lwp_lwpinssi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lwpins\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1712],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18122 */
  {
    "*lwp_lwpinsdi3_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lwpins\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1715],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18135 */
  {
    "rdfsbasesi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rdfsbase %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_rdfsbasesi,
    &operand_data[78],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18135 */
  {
    "rdfsbasedi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rdfsbase %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_rdfsbasedi,
    &operand_data[80],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18143 */
  {
    "rdgsbasesi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rdgsbase %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_rdgsbasesi,
    &operand_data[78],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18143 */
  {
    "rdgsbasedi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rdgsbase %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_rdgsbasedi,
    &operand_data[80],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18151 */
  {
    "wrfsbasesi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "wrfsbase %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_wrfsbasesi,
    &operand_data[920],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18151 */
  {
    "wrfsbasedi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "wrfsbase %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_wrfsbasedi,
    &operand_data[917],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18159 */
  {
    "wrgsbasesi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "wrgsbase %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_wrgsbasesi,
    &operand_data[920],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18159 */
  {
    "wrgsbasedi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "wrgsbase %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_wrgsbasedi,
    &operand_data[917],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18167 */
  {
    "rdrandhi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rdrand\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_rdrandhi_1,
    &operand_data[210],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18167 */
  {
    "rdrandsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rdrand\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_rdrandsi_1,
    &operand_data[78],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18167 */
  {
    "rdranddi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rdrand\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_rdranddi_1,
    &operand_data[80],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:18188 */
  {
    "*pause",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "rep; nop",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1159],
    1,
    1,
    1,
    0,
    1
  },
  /* ../../gcc/config/i386/mmx.md:81 */
  {
    "*movv8qi_internal_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_959 },
#else
    { 0, output_959, 0 },
#endif
    0,
    &operand_data[1718],
    2,
    2,
    0,
    13,
    2
  },
  /* ../../gcc/config/i386/mmx.md:81 */
  {
    "*movv4hi_internal_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_960 },
#else
    { 0, output_960, 0 },
#endif
    0,
    &operand_data[1720],
    2,
    2,
    0,
    13,
    2
  },
  /* ../../gcc/config/i386/mmx.md:81 */
  {
    "*movv2si_internal_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_961 },
#else
    { 0, output_961, 0 },
#endif
    0,
    &operand_data[1722],
    2,
    2,
    0,
    13,
    2
  },
  /* ../../gcc/config/i386/mmx.md:81 */
  {
    "*movv1di_internal_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_962 },
#else
    { 0, output_962, 0 },
#endif
    0,
    &operand_data[1724],
    2,
    2,
    0,
    13,
    2
  },
  /* ../../gcc/config/i386/mmx.md:137 */
  {
    "*movv8qi_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_963 },
#else
    { 0, output_963, 0 },
#endif
    0,
    &operand_data[1726],
    2,
    2,
    0,
    15,
    2
  },
  /* ../../gcc/config/i386/mmx.md:137 */
  {
    "*movv4hi_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_964 },
#else
    { 0, output_964, 0 },
#endif
    0,
    &operand_data[1728],
    2,
    2,
    0,
    15,
    2
  },
  /* ../../gcc/config/i386/mmx.md:137 */
  {
    "*movv2si_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_965 },
#else
    { 0, output_965, 0 },
#endif
    0,
    &operand_data[1730],
    2,
    2,
    0,
    15,
    2
  },
  /* ../../gcc/config/i386/mmx.md:137 */
  {
    "*movv1di_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_966 },
#else
    { 0, output_966, 0 },
#endif
    0,
    &operand_data[1732],
    2,
    2,
    0,
    15,
    2
  },
  /* ../../gcc/config/i386/mmx.md:213 */
  {
    "*movv2sf_internal_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_967 },
#else
    { 0, output_967, 0 },
#endif
    0,
    &operand_data[1734],
    2,
    2,
    0,
    14,
    2
  },
  /* ../../gcc/config/i386/mmx.md:268 */
  {
    "*movv2sf_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_968 },
#else
    { 0, output_968, 0 },
#endif
    0,
    &operand_data[1736],
    2,
    2,
    0,
    12,
    2
  },
  /* ../../gcc/config/i386/mmx.md:346 */
  {
    "sse_movntq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movntq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_movntq,
    &operand_data[1738],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:369 */
  {
    "*mmx_addv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfadd\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1740],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:391 */
  {
    "*mmx_subv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_971 },
#else
    { 0, output_971, 0 },
#endif
    0,
    &operand_data[1743],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/mmx.md:410 */
  {
    "*mmx_mulv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfmul\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1740],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:436 */
  {
    "*mmx_smaxv2sf3_finite",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfmax\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1740],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:436 */
  {
    "*mmx_sminv2sf3_finite",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfmin\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1740],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:448 */
  {
    "*mmx_smaxv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfmax\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1746],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:448 */
  {
    "*mmx_sminv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfmin\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1746],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:459 */
  {
    "mmx_rcpv2sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfrcp\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_rcpv2sf2,
    &operand_data[1749],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:469 */
  {
    "mmx_rcpit1v2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfrcpit1\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_rcpit1v2sf3,
    &operand_data[1746],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:480 */
  {
    "mmx_rcpit2v2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfrcpit2\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_rcpit2v2sf3,
    &operand_data[1746],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:491 */
  {
    "mmx_rsqrtv2sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfrsqrt\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_rsqrtv2sf2,
    &operand_data[1749],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:501 */
  {
    "mmx_rsqit1v2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfrsqit1\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_rsqit1v2sf3,
    &operand_data[1746],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:512 */
  {
    "mmx_haddv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfacc\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_haddv2sf3,
    &operand_data[1746],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:531 */
  {
    "mmx_hsubv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfnacc\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_hsubv2sf3,
    &operand_data[1746],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:550 */
  {
    "mmx_addsubv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfpnacc\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_addsubv2sf3,
    &operand_data[1746],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:577 */
  {
    "*mmx_eqv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfcmpeq\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1751],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:587 */
  {
    "mmx_gtv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfcmpgt\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_gtv2sf3,
    &operand_data[1754],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:597 */
  {
    "mmx_gev2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pfcmpge\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_gev2sf3,
    &operand_data[1754],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:613 */
  {
    "mmx_pf2id",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pf2id\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_pf2id,
    &operand_data[1757],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:622 */
  {
    "mmx_pf2iw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pf2iw\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_pf2iw,
    &operand_data[1757],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:634 */
  {
    "mmx_pi2fw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pi2fw\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_pi2fw,
    &operand_data[1759],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:646 */
  {
    "mmx_floatv2si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pi2fd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_floatv2si2,
    &operand_data[1759],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:661 */
  {
    "mmx_pswapdv2sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pswapd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_pswapdv2sf2,
    &operand_data[1749],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:671 */
  {
    "*vec_dupv2sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "punpckldq\t%0, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1761],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:680 */
  {
    "*mmx_concatv2sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_994 },
#else
    { 0, output_994, 0 },
#endif
    0,
    &operand_data[1763],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/mmx.md:705 */
  {
    "*vec_extractv2sf_0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1766],
    2,
    2,
    0,
    6,
    1
  },
  /* ../../gcc/config/i386/mmx.md:726 */
  {
    "*vec_extractv2sf_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_996 },
#else
    { 0, output_996, 0 },
#endif
    0,
    &operand_data[1768],
    2,
    2,
    0,
    6,
    2
  },
  /* ../../gcc/config/i386/mmx.md:789 */
  {
    "*mmx_addv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "paddb\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1770],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:789 */
  {
    "*mmx_subv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psubb\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1773],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:789 */
  {
    "*mmx_addv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "paddw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:789 */
  {
    "*mmx_subv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psubw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1779],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:789 */
  {
    "*mmx_addv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "paddd\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1782],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:789 */
  {
    "*mmx_subv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psubd\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1785],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:789 */
  {
    "*mmx_addv1di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "paddq\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1788],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:789 */
  {
    "*mmx_subv1di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psubq\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1791],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:808 */
  {
    "*mmx_ssaddv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "paddsb\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1770],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:808 */
  {
    "*mmx_usaddv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "paddusb\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1770],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:808 */
  {
    "*mmx_sssubv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psubsb\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1773],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:808 */
  {
    "*mmx_ussubv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psubusb\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1773],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:808 */
  {
    "*mmx_ssaddv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "paddsw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:808 */
  {
    "*mmx_usaddv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "paddusw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:808 */
  {
    "*mmx_sssubv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psubsw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1779],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:808 */
  {
    "*mmx_ussubv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psubusw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1779],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:825 */
  {
    "*mmx_mulv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pmullw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:847 */
  {
    "*mmx_smulv4hi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pmulhw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:875 */
  {
    "*mmx_umulv4hi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pmulhuw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:913 */
  {
    "*mmx_pmaddwd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pmaddwd\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1794],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:953 */
  {
    "*mmx_pmulhrwv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pmulhrw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:986 */
  {
    "*sse2_umulv1siv1di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pmuludq\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1797],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1010 */
  {
    "*mmx_smaxv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pmaxsw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1010 */
  {
    "*mmx_sminv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pminsw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1029 */
  {
    "*mmx_umaxv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pmaxub\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1770],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1029 */
  {
    "*mmx_uminv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pminub\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1770],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1040 */
  {
    "mmx_ashrv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psraw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_ashrv4hi3,
    &operand_data[1800],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1040 */
  {
    "mmx_ashrv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psrad\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_ashrv2si3,
    &operand_data[1803],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1054 */
  {
    "mmx_ashlv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psllw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_ashlv4hi3,
    &operand_data[1800],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1054 */
  {
    "mmx_lshrv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psrlw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_lshrv4hi3,
    &operand_data[1800],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1054 */
  {
    "mmx_ashlv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pslld\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_ashlv2si3,
    &operand_data[1803],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1054 */
  {
    "mmx_lshrv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psrld\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_lshrv2si3,
    &operand_data[1803],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1054 */
  {
    "mmx_ashlv1di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psllq\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_ashlv1di3,
    &operand_data[1806],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1054 */
  {
    "mmx_lshrv1di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psrlq\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_lshrv1di3,
    &operand_data[1806],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1082 */
  {
    "*mmx_eqv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pcmpeqb\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1770],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1082 */
  {
    "*mmx_eqv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pcmpeqw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1082 */
  {
    "*mmx_eqv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pcmpeqd\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1782],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1092 */
  {
    "mmx_gtv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pcmpgtb\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_gtv8qi3,
    &operand_data[1809],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1092 */
  {
    "mmx_gtv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pcmpgtw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_gtv4hi3,
    &operand_data[1812],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1092 */
  {
    "mmx_gtv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pcmpgtd\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_gtv2si3,
    &operand_data[1815],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1108 */
  {
    "mmx_andnotv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pandn\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_andnotv8qi3,
    &operand_data[1809],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1108 */
  {
    "mmx_andnotv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pandn\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_andnotv4hi3,
    &operand_data[1812],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1108 */
  {
    "mmx_andnotv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pandn\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_andnotv2si3,
    &operand_data[1815],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1126 */
  {
    "*mmx_andv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pand\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1770],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1126 */
  {
    "*mmx_iorv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "por\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1770],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1126 */
  {
    "*mmx_xorv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pxor\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1770],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1126 */
  {
    "*mmx_andv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pand\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1126 */
  {
    "*mmx_iorv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "por\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1126 */
  {
    "*mmx_xorv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pxor\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1126 */
  {
    "*mmx_andv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pand\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1782],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1126 */
  {
    "*mmx_iorv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "por\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1782],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1126 */
  {
    "*mmx_xorv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pxor\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1782],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1142 */
  {
    "mmx_packsswb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "packsswb\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_packsswb,
    &operand_data[1818],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1154 */
  {
    "mmx_packssdw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "packssdw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_packssdw,
    &operand_data[1821],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1166 */
  {
    "mmx_packuswb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "packuswb\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_packuswb,
    &operand_data[1818],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1178 */
  {
    "mmx_punpckhbw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "punpckhbw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_punpckhbw,
    &operand_data[1809],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1193 */
  {
    "mmx_punpcklbw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "punpcklbw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_punpcklbw,
    &operand_data[1809],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1208 */
  {
    "mmx_punpckhwd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "punpckhwd\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_punpckhwd,
    &operand_data[1812],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1221 */
  {
    "mmx_punpcklwd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "punpcklwd\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_punpcklwd,
    &operand_data[1812],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1234 */
  {
    "mmx_punpckhdq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "punpckhdq\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_punpckhdq,
    &operand_data[1815],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1247 */
  {
    "mmx_punpckldq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "punpckldq\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_punpckldq,
    &operand_data[1815],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1273 */
  {
    "*mmx_pinsrw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1058 },
#else
    { 0, 0, output_1058 },
#endif
    0,
    &operand_data[1824],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/mmx.md:1294 */
  {
    "mmx_pextrw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pextrw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_pextrw,
    &operand_data[1828],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1321 */
  {
    "mmx_pshufw_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1060 },
#else
    { 0, 0, output_1060 },
#endif
    (insn_gen_fn) gen_mmx_pshufw_1,
    &operand_data[1831],
    6,
    6,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/mmx.md:1344 */
  {
    "mmx_pswapdv2si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pswapd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_pswapdv2si2,
    &operand_data[1837],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1355 */
  {
    "*vec_dupv4hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pshufw\t{$0, %0, %0|%0, %0, 0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1839],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1366 */
  {
    "*vec_dupv2si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "punpckldq\t%0, %0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1841],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1375 */
  {
    "*mmx_concatv2si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1064 },
#else
    { 0, output_1064, 0 },
#endif
    0,
    &operand_data[1843],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/mmx.md:1400 */
  {
    "*vec_extractv2si_0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1846],
    2,
    2,
    0,
    5,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1421 */
  {
    "*vec_extractv2si_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1066 },
#else
    { 0, output_1066, 0 },
#endif
    0,
    &operand_data[1848],
    2,
    2,
    0,
    7,
    2
  },
  /* ../../gcc/config/i386/mmx.md:1562 */
  {
    "*mmx_uavgv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1067 },
#else
    { 0, 0, output_1067 },
#endif
    0,
    &operand_data[1770],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/mmx.md:1612 */
  {
    "*mmx_uavgv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pavgw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1631 */
  {
    "mmx_psadbw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psadbw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_psadbw,
    &operand_data[1850],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1641 */
  {
    "mmx_pmovmskb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pmovmskb\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mmx_pmovmskb,
    &operand_data[1853],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1658 */
  {
    "*mmx_maskmovq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "maskmovq\t{%2, %1|%1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1855],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1658 */
  {
    "*mmx_maskmovq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "maskmovq\t{%2, %1|%1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1858],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1694 */
  {
    "*mmx_emms",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "emms",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1861],
    1,
    1,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/mmx.md:1727 */
  {
    "*mmx_femms",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "femms",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1861],
    1,
    1,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv32qi_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1075 },
#else
    { 0, 0, output_1075 },
#endif
    0,
    &operand_data[1862],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv16qi_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1076 },
#else
    { 0, 0, output_1076 },
#endif
    0,
    &operand_data[1864],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv16hi_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1077 },
#else
    { 0, 0, output_1077 },
#endif
    0,
    &operand_data[1866],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv8hi_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1078 },
#else
    { 0, 0, output_1078 },
#endif
    0,
    &operand_data[1868],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv8si_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1079 },
#else
    { 0, 0, output_1079 },
#endif
    0,
    &operand_data[1870],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv4si_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1080 },
#else
    { 0, 0, output_1080 },
#endif
    0,
    &operand_data[1872],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv4di_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1081 },
#else
    { 0, 0, output_1081 },
#endif
    0,
    &operand_data[1874],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv2di_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1082 },
#else
    { 0, 0, output_1082 },
#endif
    0,
    &operand_data[1876],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv2ti_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1083 },
#else
    { 0, 0, output_1083 },
#endif
    0,
    &operand_data[1878],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv1ti_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1084 },
#else
    { 0, 0, output_1084 },
#endif
    0,
    &operand_data[1880],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv8sf_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1085 },
#else
    { 0, 0, output_1085 },
#endif
    0,
    &operand_data[1882],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv4sf_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1086 },
#else
    { 0, 0, output_1086 },
#endif
    0,
    &operand_data[1884],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv4df_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1087 },
#else
    { 0, 0, output_1087 },
#endif
    0,
    &operand_data[1886],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:423 */
  {
    "*movv2df_internal",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1088 },
#else
    { 0, 0, output_1088 },
#endif
    0,
    &operand_data[1888],
    2,
    2,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:493 */
  {
    "sse2_movq128",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_movq128,
    &operand_data[1890],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:515 */
  {
    "movdi_to_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_movdi_to_sse,
    &operand_data[1892],
    2,
    3,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:584 */
  {
    "avx_loadups256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovups\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_loadups256,
    &operand_data[1895],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:584 */
  {
    "sse_loadups",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovups\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_loadups,
    &operand_data[1897],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:584 */
  {
    "avx_loadupd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovupd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_loadupd256,
    &operand_data[1899],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:584 */
  {
    "sse2_loadupd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovupd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_loadupd,
    &operand_data[1901],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:596 */
  {
    "avx_storeups256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovups\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_storeups256,
    &operand_data[1903],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:596 */
  {
    "sse_storeups",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovups\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_storeups,
    &operand_data[1905],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:596 */
  {
    "avx_storeupd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovupd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_storeupd256,
    &operand_data[1907],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:596 */
  {
    "sse2_storeupd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovupd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_storeupd,
    &operand_data[1909],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:608 */
  {
    "avx_loaddqu256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovdqu\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_loaddqu256,
    &operand_data[1911],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:608 */
  {
    "sse2_loaddqu",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovdqu\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_loaddqu,
    &operand_data[1913],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:624 */
  {
    "avx_storedqu256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovdqu\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_storedqu256,
    &operand_data[1915],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:624 */
  {
    "sse2_storedqu",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovdqu\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_storedqu,
    &operand_data[1917],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:640 */
  {
    "avx_lddqu256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vlddqu\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_lddqu256,
    &operand_data[1911],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:640 */
  {
    "sse3_lddqu",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vlddqu\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse3_lddqu,
    &operand_data[1913],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:661 */
  {
    "sse2_movntisi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movnti\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_movntisi,
    &operand_data[1919],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:661 */
  {
    "sse2_movntidi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movnti\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_movntidi,
    &operand_data[1921],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:671 */
  {
    "avx_movntv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovntps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_movntv8sf,
    &operand_data[1903],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:671 */
  {
    "sse_movntv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovntps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_movntv4sf,
    &operand_data[1905],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:671 */
  {
    "avx_movntv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovntpd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_movntv4df,
    &operand_data[1907],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:671 */
  {
    "sse2_movntv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovntpd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_movntv2df,
    &operand_data[1909],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:681 */
  {
    "avx_movntv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovntdq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_movntv4di,
    &operand_data[1923],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:681 */
  {
    "sse2_movntv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovntdq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_movntv2di,
    &operand_data[1925],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:728 */
  {
    "*absnegv8sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1927],
    4,
    4,
    0,
    4,
    1
  },
  /* ../../gcc/config/i386/sse.md:728 */
  {
    "*absnegv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1931],
    4,
    4,
    0,
    4,
    1
  },
  /* ../../gcc/config/i386/sse.md:728 */
  {
    "*absnegv4df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1935],
    4,
    4,
    0,
    4,
    1
  },
  /* ../../gcc/config/i386/sse.md:728 */
  {
    "*absnegv2df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1939],
    4,
    4,
    0,
    4,
    1
  },
  /* ../../gcc/config/i386/sse.md:774 */
  {
    "*addv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1117 },
#else
    { 0, output_1117, 0 },
#endif
    0,
    &operand_data[1943],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:774 */
  {
    "*subv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1118 },
#else
    { 0, output_1118, 0 },
#endif
    0,
    &operand_data[1946],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:774 */
  {
    "*addv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1119 },
#else
    { 0, output_1119, 0 },
#endif
    0,
    &operand_data[1949],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:774 */
  {
    "*subv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1120 },
#else
    { 0, output_1120, 0 },
#endif
    0,
    &operand_data[1952],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:774 */
  {
    "*addv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1121 },
#else
    { 0, output_1121, 0 },
#endif
    0,
    &operand_data[1955],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:774 */
  {
    "*subv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1122 },
#else
    { 0, output_1122, 0 },
#endif
    0,
    &operand_data[1958],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:774 */
  {
    "*addv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1123 },
#else
    { 0, output_1123, 0 },
#endif
    0,
    &operand_data[1961],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:774 */
  {
    "*subv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1124 },
#else
    { 0, output_1124, 0 },
#endif
    0,
    &operand_data[1964],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:788 */
  {
    "sse_vmaddv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1125 },
#else
    { 0, output_1125, 0 },
#endif
    (insn_gen_fn) gen_sse_vmaddv4sf3,
    &operand_data[1967],
    3,
    3,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:788 */
  {
    "sse_vmsubv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1126 },
#else
    { 0, output_1126, 0 },
#endif
    (insn_gen_fn) gen_sse_vmsubv4sf3,
    &operand_data[1967],
    3,
    3,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:788 */
  {
    "sse2_vmaddv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1127 },
#else
    { 0, output_1127, 0 },
#endif
    (insn_gen_fn) gen_sse2_vmaddv2df3,
    &operand_data[1970],
    3,
    3,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:788 */
  {
    "sse2_vmsubv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1128 },
#else
    { 0, output_1128, 0 },
#endif
    (insn_gen_fn) gen_sse2_vmsubv2df3,
    &operand_data[1970],
    3,
    3,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:813 */
  {
    "*mulv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1129 },
#else
    { 0, output_1129, 0 },
#endif
    0,
    &operand_data[1943],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:813 */
  {
    "*mulv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1130 },
#else
    { 0, output_1130, 0 },
#endif
    0,
    &operand_data[1949],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:813 */
  {
    "*mulv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1131 },
#else
    { 0, output_1131, 0 },
#endif
    0,
    &operand_data[1955],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:813 */
  {
    "*mulv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1132 },
#else
    { 0, output_1132, 0 },
#endif
    0,
    &operand_data[1961],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:827 */
  {
    "sse_vmmulv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1133 },
#else
    { 0, output_1133, 0 },
#endif
    (insn_gen_fn) gen_sse_vmmulv4sf3,
    &operand_data[1967],
    3,
    3,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:827 */
  {
    "sse2_vmmulv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1134 },
#else
    { 0, output_1134, 0 },
#endif
    (insn_gen_fn) gen_sse2_vmmulv2df3,
    &operand_data[1970],
    3,
    3,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:870 */
  {
    "avx_divv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1135 },
#else
    { 0, output_1135, 0 },
#endif
    (insn_gen_fn) gen_avx_divv8sf3,
    &operand_data[1973],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:870 */
  {
    "sse_divv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1136 },
#else
    { 0, output_1136, 0 },
#endif
    (insn_gen_fn) gen_sse_divv4sf3,
    &operand_data[1967],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:870 */
  {
    "avx_divv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1137 },
#else
    { 0, output_1137, 0 },
#endif
    (insn_gen_fn) gen_avx_divv4df3,
    &operand_data[1976],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:870 */
  {
    "sse2_divv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1138 },
#else
    { 0, output_1138, 0 },
#endif
    (insn_gen_fn) gen_sse2_divv2df3,
    &operand_data[1970],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:884 */
  {
    "sse_vmdivv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1139 },
#else
    { 0, output_1139, 0 },
#endif
    (insn_gen_fn) gen_sse_vmdivv4sf3,
    &operand_data[1967],
    3,
    3,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:884 */
  {
    "sse2_vmdivv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1140 },
#else
    { 0, output_1140, 0 },
#endif
    (insn_gen_fn) gen_sse2_vmdivv2df3,
    &operand_data[1970],
    3,
    3,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:901 */
  {
    "avx_rcpv8sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vrcpps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_rcpv8sf2,
    &operand_data[1979],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:901 */
  {
    "sse_rcpv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vrcpps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_rcpv4sf2,
    &operand_data[1981],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:912 */
  {
    "sse_vmrcpv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1143 },
#else
    { 0, output_1143, 0 },
#endif
    (insn_gen_fn) gen_sse_vmrcpv4sf2,
    &operand_data[1983],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:950 */
  {
    "avx_sqrtv8sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vsqrtps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_sqrtv8sf2,
    &operand_data[1979],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:950 */
  {
    "sse_sqrtv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vsqrtps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_sqrtv4sf2,
    &operand_data[1981],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:950 */
  {
    "avx_sqrtv4df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vsqrtpd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_sqrtv4df2,
    &operand_data[1986],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:950 */
  {
    "sse2_sqrtv2df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vsqrtpd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_sqrtv2df2,
    &operand_data[1988],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:960 */
  {
    "sse_vmsqrtv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1148 },
#else
    { 0, output_1148, 0 },
#endif
    (insn_gen_fn) gen_sse_vmsqrtv4sf2,
    &operand_data[1983],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:960 */
  {
    "sse2_vmsqrtv2df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1149 },
#else
    { 0, output_1149, 0 },
#endif
    (insn_gen_fn) gen_sse2_vmsqrtv2df2,
    &operand_data[1990],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:987 */
  {
    "avx_rsqrtv8sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vrsqrtps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_rsqrtv8sf2,
    &operand_data[1979],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:987 */
  {
    "sse_rsqrtv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vrsqrtps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_rsqrtv4sf2,
    &operand_data[1981],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:997 */
  {
    "sse_vmrsqrtv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1152 },
#else
    { 0, output_1152, 0 },
#endif
    (insn_gen_fn) gen_sse_vmrsqrtv4sf2,
    &operand_data[1983],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1029 */
  {
    "*smaxv8sf3_finite",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1153 },
#else
    { 0, output_1153, 0 },
#endif
    0,
    &operand_data[1943],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1029 */
  {
    "*sminv8sf3_finite",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1154 },
#else
    { 0, output_1154, 0 },
#endif
    0,
    &operand_data[1943],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1029 */
  {
    "*smaxv4sf3_finite",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1155 },
#else
    { 0, output_1155, 0 },
#endif
    0,
    &operand_data[1949],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1029 */
  {
    "*sminv4sf3_finite",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1156 },
#else
    { 0, output_1156, 0 },
#endif
    0,
    &operand_data[1949],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1029 */
  {
    "*smaxv4df3_finite",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1157 },
#else
    { 0, output_1157, 0 },
#endif
    0,
    &operand_data[1955],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1029 */
  {
    "*sminv4df3_finite",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1158 },
#else
    { 0, output_1158, 0 },
#endif
    0,
    &operand_data[1955],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1029 */
  {
    "*smaxv2df3_finite",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1159 },
#else
    { 0, output_1159, 0 },
#endif
    0,
    &operand_data[1961],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1029 */
  {
    "*sminv2df3_finite",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1160 },
#else
    { 0, output_1160, 0 },
#endif
    0,
    &operand_data[1961],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1044 */
  {
    "*smaxv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1161 },
#else
    { 0, output_1161, 0 },
#endif
    0,
    &operand_data[1973],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1044 */
  {
    "*sminv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1162 },
#else
    { 0, output_1162, 0 },
#endif
    0,
    &operand_data[1973],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1044 */
  {
    "*smaxv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1163 },
#else
    { 0, output_1163, 0 },
#endif
    0,
    &operand_data[1967],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1044 */
  {
    "*sminv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1164 },
#else
    { 0, output_1164, 0 },
#endif
    0,
    &operand_data[1967],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1044 */
  {
    "*smaxv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1165 },
#else
    { 0, output_1165, 0 },
#endif
    0,
    &operand_data[1976],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1044 */
  {
    "*sminv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1166 },
#else
    { 0, output_1166, 0 },
#endif
    0,
    &operand_data[1976],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1044 */
  {
    "*smaxv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1167 },
#else
    { 0, output_1167, 0 },
#endif
    0,
    &operand_data[1970],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1044 */
  {
    "*sminv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1168 },
#else
    { 0, output_1168, 0 },
#endif
    0,
    &operand_data[1970],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1058 */
  {
    "sse_vmsmaxv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1169 },
#else
    { 0, output_1169, 0 },
#endif
    (insn_gen_fn) gen_sse_vmsmaxv4sf3,
    &operand_data[1967],
    3,
    3,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1058 */
  {
    "sse_vmsminv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1170 },
#else
    { 0, output_1170, 0 },
#endif
    (insn_gen_fn) gen_sse_vmsminv4sf3,
    &operand_data[1967],
    3,
    3,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1058 */
  {
    "sse2_vmsmaxv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1171 },
#else
    { 0, output_1171, 0 },
#endif
    (insn_gen_fn) gen_sse2_vmsmaxv2df3,
    &operand_data[1970],
    3,
    3,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1058 */
  {
    "sse2_vmsminv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1172 },
#else
    { 0, output_1172, 0 },
#endif
    (insn_gen_fn) gen_sse2_vmsminv2df3,
    &operand_data[1970],
    3,
    3,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1081 */
  {
    "*ieee_sminv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1173 },
#else
    { 0, output_1173, 0 },
#endif
    0,
    &operand_data[1973],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1081 */
  {
    "*ieee_sminv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1174 },
#else
    { 0, output_1174, 0 },
#endif
    0,
    &operand_data[1967],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1081 */
  {
    "*ieee_sminv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1175 },
#else
    { 0, output_1175, 0 },
#endif
    0,
    &operand_data[1976],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1081 */
  {
    "*ieee_sminv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1176 },
#else
    { 0, output_1176, 0 },
#endif
    0,
    &operand_data[1970],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1096 */
  {
    "*ieee_smaxv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1177 },
#else
    { 0, output_1177, 0 },
#endif
    0,
    &operand_data[1973],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1096 */
  {
    "*ieee_smaxv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1178 },
#else
    { 0, output_1178, 0 },
#endif
    0,
    &operand_data[1967],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1096 */
  {
    "*ieee_smaxv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1179 },
#else
    { 0, output_1179, 0 },
#endif
    0,
    &operand_data[1976],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1096 */
  {
    "*ieee_smaxv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1180 },
#else
    { 0, output_1180, 0 },
#endif
    0,
    &operand_data[1970],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1111 */
  {
    "avx_addsubv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vaddsubpd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_addsubv4df3,
    &operand_data[1993],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1125 */
  {
    "sse3_addsubv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1182 },
#else
    { 0, output_1182, 0 },
#endif
    (insn_gen_fn) gen_sse3_addsubv2df3,
    &operand_data[1970],
    3,
    3,
    2,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1143 */
  {
    "avx_addsubv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vaddsubps\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_addsubv8sf3,
    &operand_data[1996],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1157 */
  {
    "sse3_addsubv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1184 },
#else
    { 0, output_1184, 0 },
#endif
    (insn_gen_fn) gen_sse3_addsubv4sf3,
    &operand_data[1967],
    3,
    3,
    2,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1175 */
  {
    "avx_haddv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vhaddpd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_haddv4df3,
    &operand_data[1993],
    3,
    3,
    6,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1175 */
  {
    "avx_hsubv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vhsubpd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_hsubv4df3,
    &operand_data[1993],
    3,
    3,
    6,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1202 */
  {
    "sse3_haddv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1187 },
#else
    { 0, output_1187, 0 },
#endif
    (insn_gen_fn) gen_sse3_haddv2df3,
    &operand_data[1970],
    3,
    3,
    2,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1202 */
  {
    "sse3_hsubv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1188 },
#else
    { 0, output_1188, 0 },
#endif
    (insn_gen_fn) gen_sse3_hsubv2df3,
    &operand_data[1970],
    3,
    3,
    2,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1224 */
  {
    "avx_haddv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vhaddps\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_haddv8sf3,
    &operand_data[1996],
    3,
    3,
    14,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1224 */
  {
    "avx_hsubv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vhsubps\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_hsubv8sf3,
    &operand_data[1996],
    3,
    3,
    14,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1267 */
  {
    "sse3_haddv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1191 },
#else
    { 0, output_1191, 0 },
#endif
    (insn_gen_fn) gen_sse3_haddv4sf3,
    &operand_data[1967],
    3,
    3,
    6,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1267 */
  {
    "sse3_hsubv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1192 },
#else
    { 0, output_1192, 0 },
#endif
    (insn_gen_fn) gen_sse3_hsubv4sf3,
    &operand_data[1967],
    3,
    3,
    6,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1394 */
  {
    "avx_cmpv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcmpps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_cmpv8sf3,
    &operand_data[1996],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1394 */
  {
    "avx_cmpv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcmpps\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_cmpv4sf3,
    &operand_data[2000],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1394 */
  {
    "avx_cmpv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcmppd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_cmpv4df3,
    &operand_data[2004],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1394 */
  {
    "avx_cmpv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcmppd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_cmpv2df3,
    &operand_data[2008],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1408 */
  {
    "avx_vmcmpv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcmpss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_vmcmpv4sf3,
    &operand_data[2000],
    4,
    4,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1408 */
  {
    "avx_vmcmpv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcmpsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_vmcmpv2df3,
    &operand_data[2008],
    4,
    4,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1425 */
  {
    "*avx_maskcmpv8sf3_comm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1199 },
#else
    { 0, output_1199, 0 },
#endif
    0,
    &operand_data[2012],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1425 */
  {
    "*sse_maskcmpv4sf3_comm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1200 },
#else
    { 0, output_1200, 0 },
#endif
    0,
    &operand_data[2016],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1425 */
  {
    "*avx_maskcmpv4df3_comm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1201 },
#else
    { 0, output_1201, 0 },
#endif
    0,
    &operand_data[2020],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1425 */
  {
    "*sse2_maskcmpv2df3_comm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1202 },
#else
    { 0, output_1202, 0 },
#endif
    0,
    &operand_data[2024],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1441 */
  {
    "avx_maskcmpv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1203 },
#else
    { 0, output_1203, 0 },
#endif
    (insn_gen_fn) gen_avx_maskcmpv8sf3,
    &operand_data[2028],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1441 */
  {
    "sse_maskcmpv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1204 },
#else
    { 0, output_1204, 0 },
#endif
    (insn_gen_fn) gen_sse_maskcmpv4sf3,
    &operand_data[2032],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1441 */
  {
    "avx_maskcmpv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1205 },
#else
    { 0, output_1205, 0 },
#endif
    (insn_gen_fn) gen_avx_maskcmpv4df3,
    &operand_data[2036],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1441 */
  {
    "sse2_maskcmpv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1206 },
#else
    { 0, output_1206, 0 },
#endif
    (insn_gen_fn) gen_sse2_maskcmpv2df3,
    &operand_data[2040],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1456 */
  {
    "sse_vmmaskcmpv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1207 },
#else
    { 0, output_1207, 0 },
#endif
    (insn_gen_fn) gen_sse_vmmaskcmpv4sf3,
    &operand_data[2032],
    4,
    4,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1456 */
  {
    "sse2_vmmaskcmpv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1208 },
#else
    { 0, output_1208, 0 },
#endif
    (insn_gen_fn) gen_sse2_vmmaskcmpv2df3,
    &operand_data[2040],
    4,
    4,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1474 */
  {
    "sse_comi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcomiss\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_comi,
    &operand_data[2001],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1474 */
  {
    "sse2_comi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcomisd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_comi,
    &operand_data[2009],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1494 */
  {
    "sse_ucomi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vucomiss\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_ucomi,
    &operand_data[2001],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1494 */
  {
    "sse2_ucomi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vucomisd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_ucomi,
    &operand_data[2009],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:1554 */
  {
    "avx_andnotv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1213 },
#else
    { 0, 0, output_1213 },
#endif
    (insn_gen_fn) gen_avx_andnotv8sf3,
    &operand_data[1973],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1554 */
  {
    "sse_andnotv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1214 },
#else
    { 0, 0, output_1214 },
#endif
    (insn_gen_fn) gen_sse_andnotv4sf3,
    &operand_data[1967],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1554 */
  {
    "avx_andnotv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1215 },
#else
    { 0, 0, output_1215 },
#endif
    (insn_gen_fn) gen_avx_andnotv4df3,
    &operand_data[1976],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1554 */
  {
    "sse2_andnotv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1216 },
#else
    { 0, 0, output_1216 },
#endif
    (insn_gen_fn) gen_sse2_andnotv2df3,
    &operand_data[1970],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1595 */
  {
    "*andv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1217 },
#else
    { 0, 0, output_1217 },
#endif
    0,
    &operand_data[1943],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1595 */
  {
    "*iorv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1218 },
#else
    { 0, 0, output_1218 },
#endif
    0,
    &operand_data[1943],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1595 */
  {
    "*xorv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1219 },
#else
    { 0, 0, output_1219 },
#endif
    0,
    &operand_data[1943],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1595 */
  {
    "*andv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1220 },
#else
    { 0, 0, output_1220 },
#endif
    0,
    &operand_data[1949],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1595 */
  {
    "*iorv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1221 },
#else
    { 0, 0, output_1221 },
#endif
    0,
    &operand_data[1949],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1595 */
  {
    "*xorv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1222 },
#else
    { 0, 0, output_1222 },
#endif
    0,
    &operand_data[1949],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1595 */
  {
    "*andv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1223 },
#else
    { 0, 0, output_1223 },
#endif
    0,
    &operand_data[1955],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1595 */
  {
    "*iorv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1224 },
#else
    { 0, 0, output_1224 },
#endif
    0,
    &operand_data[1955],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1595 */
  {
    "*xorv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1225 },
#else
    { 0, 0, output_1225 },
#endif
    0,
    &operand_data[1955],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1595 */
  {
    "*andv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1226 },
#else
    { 0, 0, output_1226 },
#endif
    0,
    &operand_data[1961],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1595 */
  {
    "*iorv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1227 },
#else
    { 0, 0, output_1227 },
#endif
    0,
    &operand_data[1961],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1595 */
  {
    "*xorv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1228 },
#else
    { 0, 0, output_1228 },
#endif
    0,
    &operand_data[1961],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1650 */
  {
    "*andnotsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1229 },
#else
    { 0, 0, output_1229 },
#endif
    0,
    &operand_data[2044],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1650 */
  {
    "*andnotdf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1230 },
#else
    { 0, 0, output_1230 },
#endif
    0,
    &operand_data[2047],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1683 */
  {
    "*andsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1231 },
#else
    { 0, 0, output_1231 },
#endif
    0,
    &operand_data[2050],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1683 */
  {
    "*iorsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1232 },
#else
    { 0, 0, output_1232 },
#endif
    0,
    &operand_data[2050],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1683 */
  {
    "*xorsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1233 },
#else
    { 0, 0, output_1233 },
#endif
    0,
    &operand_data[2050],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1683 */
  {
    "*anddf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1234 },
#else
    { 0, 0, output_1234 },
#endif
    0,
    &operand_data[2053],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1683 */
  {
    "*iordf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1235 },
#else
    { 0, 0, output_1235 },
#endif
    0,
    &operand_data[2053],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1683 */
  {
    "*xordf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1236 },
#else
    { 0, 0, output_1236 },
#endif
    0,
    &operand_data[2053],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:1770 */
  {
    "*fma_fmadd_sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1237 },
#else
    { 0, output_1237, 0 },
#endif
    0,
    &operand_data[2056],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1770 */
  {
    "*fma_fmadd_df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1238 },
#else
    { 0, output_1238, 0 },
#endif
    0,
    &operand_data[2060],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1770 */
  {
    "*fma_fmadd_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1239 },
#else
    { 0, output_1239, 0 },
#endif
    0,
    &operand_data[2064],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1770 */
  {
    "*fma_fmadd_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1240 },
#else
    { 0, output_1240, 0 },
#endif
    0,
    &operand_data[2068],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1770 */
  {
    "*fma_fmadd_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1241 },
#else
    { 0, output_1241, 0 },
#endif
    0,
    &operand_data[2072],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1770 */
  {
    "*fma_fmadd_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1242 },
#else
    { 0, output_1242, 0 },
#endif
    0,
    &operand_data[2076],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1787 */
  {
    "*fma_fmsub_sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1243 },
#else
    { 0, output_1243, 0 },
#endif
    0,
    &operand_data[2056],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1787 */
  {
    "*fma_fmsub_df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1244 },
#else
    { 0, output_1244, 0 },
#endif
    0,
    &operand_data[2060],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1787 */
  {
    "*fma_fmsub_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1245 },
#else
    { 0, output_1245, 0 },
#endif
    0,
    &operand_data[2064],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1787 */
  {
    "*fma_fmsub_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1246 },
#else
    { 0, output_1246, 0 },
#endif
    0,
    &operand_data[2068],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1787 */
  {
    "*fma_fmsub_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1247 },
#else
    { 0, output_1247, 0 },
#endif
    0,
    &operand_data[2072],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1787 */
  {
    "*fma_fmsub_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1248 },
#else
    { 0, output_1248, 0 },
#endif
    0,
    &operand_data[2076],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1805 */
  {
    "*fma_fnmadd_sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1249 },
#else
    { 0, output_1249, 0 },
#endif
    0,
    &operand_data[2056],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1805 */
  {
    "*fma_fnmadd_df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1250 },
#else
    { 0, output_1250, 0 },
#endif
    0,
    &operand_data[2060],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1805 */
  {
    "*fma_fnmadd_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1251 },
#else
    { 0, output_1251, 0 },
#endif
    0,
    &operand_data[2064],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1805 */
  {
    "*fma_fnmadd_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1252 },
#else
    { 0, output_1252, 0 },
#endif
    0,
    &operand_data[2068],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1805 */
  {
    "*fma_fnmadd_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1253 },
#else
    { 0, output_1253, 0 },
#endif
    0,
    &operand_data[2072],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1805 */
  {
    "*fma_fnmadd_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1254 },
#else
    { 0, output_1254, 0 },
#endif
    0,
    &operand_data[2076],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1823 */
  {
    "*fma_fnmsub_sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1255 },
#else
    { 0, output_1255, 0 },
#endif
    0,
    &operand_data[2056],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1823 */
  {
    "*fma_fnmsub_df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1256 },
#else
    { 0, output_1256, 0 },
#endif
    0,
    &operand_data[2060],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1823 */
  {
    "*fma_fnmsub_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1257 },
#else
    { 0, output_1257, 0 },
#endif
    0,
    &operand_data[2064],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1823 */
  {
    "*fma_fnmsub_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1258 },
#else
    { 0, output_1258, 0 },
#endif
    0,
    &operand_data[2068],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1823 */
  {
    "*fma_fnmsub_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1259 },
#else
    { 0, output_1259, 0 },
#endif
    0,
    &operand_data[2072],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1823 */
  {
    "*fma_fnmsub_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1260 },
#else
    { 0, output_1260, 0 },
#endif
    0,
    &operand_data[2076],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1862 */
  {
    "*fma_fmaddsub_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1261 },
#else
    { 0, output_1261, 0 },
#endif
    0,
    &operand_data[2072],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1862 */
  {
    "*fma_fmaddsub_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1262 },
#else
    { 0, output_1262, 0 },
#endif
    0,
    &operand_data[2064],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1862 */
  {
    "*fma_fmaddsub_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1263 },
#else
    { 0, output_1263, 0 },
#endif
    0,
    &operand_data[2076],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1862 */
  {
    "*fma_fmaddsub_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1264 },
#else
    { 0, output_1264, 0 },
#endif
    0,
    &operand_data[2068],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1880 */
  {
    "*fma_fmsubadd_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1265 },
#else
    { 0, output_1265, 0 },
#endif
    0,
    &operand_data[2072],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1880 */
  {
    "*fma_fmsubadd_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1266 },
#else
    { 0, output_1266, 0 },
#endif
    0,
    &operand_data[2064],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1880 */
  {
    "*fma_fmsubadd_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1267 },
#else
    { 0, output_1267, 0 },
#endif
    0,
    &operand_data[2076],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1880 */
  {
    "*fma_fmsubadd_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1268 },
#else
    { 0, output_1268, 0 },
#endif
    0,
    &operand_data[2068],
    4,
    4,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:1913 */
  {
    "*fmai_fmadd_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1269 },
#else
    { 0, output_1269, 0 },
#endif
    0,
    &operand_data[2080],
    4,
    4,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1913 */
  {
    "*fmai_fmadd_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1270 },
#else
    { 0, output_1270, 0 },
#endif
    0,
    &operand_data[2084],
    4,
    4,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1929 */
  {
    "*fmai_fmsub_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1271 },
#else
    { 0, output_1271, 0 },
#endif
    0,
    &operand_data[2080],
    4,
    4,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1929 */
  {
    "*fmai_fmsub_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1272 },
#else
    { 0, output_1272, 0 },
#endif
    0,
    &operand_data[2084],
    4,
    4,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1946 */
  {
    "*fmai_fnmadd_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1273 },
#else
    { 0, output_1273, 0 },
#endif
    0,
    &operand_data[2080],
    4,
    4,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1946 */
  {
    "*fmai_fnmadd_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1274 },
#else
    { 0, output_1274, 0 },
#endif
    0,
    &operand_data[2084],
    4,
    4,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1963 */
  {
    "*fmai_fnmsub_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1275 },
#else
    { 0, output_1275, 0 },
#endif
    0,
    &operand_data[2080],
    4,
    4,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1963 */
  {
    "*fmai_fnmsub_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1276 },
#else
    { 0, output_1276, 0 },
#endif
    0,
    &operand_data[2084],
    4,
    4,
    1,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:1998 */
  {
    "*fma4i_vmfmadd_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfmaddss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2088],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:1998 */
  {
    "*fma4i_vmfmadd_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfmaddsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2093],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2012 */
  {
    "*fma4i_vmfmsub_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfmsubss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2088],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2012 */
  {
    "*fma4i_vmfmsub_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfmsubsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2093],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2027 */
  {
    "*fma4i_vmfnmadd_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfnmaddss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2088],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2027 */
  {
    "*fma4i_vmfnmadd_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfnmaddsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2093],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2042 */
  {
    "*fma4i_vmfnmsub_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfnmsubss\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2088],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2042 */
  {
    "*fma4i_vmfnmsub_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfnmsubsd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2093],
    5,
    5,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2064 */
  {
    "sse_cvtpi2ps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cvtpi2ps\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_cvtpi2ps,
    &operand_data[2098],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2076 */
  {
    "sse_cvtps2pi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cvtps2pi\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_cvtps2pi,
    &operand_data[2101],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2088 */
  {
    "sse_cvttps2pi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cvttps2pi\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_cvttps2pi,
    &operand_data[2101],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2100 */
  {
    "sse_cvtsi2ss",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1288 },
#else
    { 0, output_1288, 0 },
#endif
    (insn_gen_fn) gen_sse_cvtsi2ss,
    &operand_data[2103],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:2120 */
  {
    "sse_cvtsi2ssq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1289 },
#else
    { 0, output_1289, 0 },
#endif
    (insn_gen_fn) gen_sse_cvtsi2ssq,
    &operand_data[2106],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:2142 */
  {
    "sse_cvtss2si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtss2si\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_cvtss2si,
    &operand_data[2109],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2158 */
  {
    "sse_cvtss2si_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtss2si\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_cvtss2si_2,
    &operand_data[277],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2172 */
  {
    "sse_cvtss2siq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtss2si{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_cvtss2siq,
    &operand_data[2111],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2188 */
  {
    "sse_cvtss2siq_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtss2si{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_cvtss2siq_2,
    &operand_data[273],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2202 */
  {
    "sse_cvttss2si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvttss2si\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_cvttss2si,
    &operand_data[2109],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2218 */
  {
    "sse_cvttss2siq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvttss2si{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_cvttss2siq,
    &operand_data[2111],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2234 */
  {
    "floatv8siv8sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtdq2ps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_floatv8siv8sf2,
    &operand_data[2113],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2234 */
  {
    "floatv4siv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtdq2ps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_floatv4siv4sf2,
    &operand_data[2115],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2253 */
  {
    "avx_cvtps2dq256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtps2dq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_cvtps2dq256,
    &operand_data[2117],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2263 */
  {
    "sse2_cvtps2dq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtps2dq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtps2dq,
    &operand_data[2119],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2278 */
  {
    "fix_truncv8sfv8si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvttps2dq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncv8sfv8si2,
    &operand_data[2117],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2287 */
  {
    "fix_truncv4sfv4si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvttps2dq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncv4sfv4si2,
    &operand_data[2119],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2326 */
  {
    "sse2_cvtpi2pd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cvtpi2pd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtpi2pd,
    &operand_data[2121],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2336 */
  {
    "sse2_cvtpd2pi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cvtpd2pi\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtpd2pi,
    &operand_data[2123],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2348 */
  {
    "sse2_cvttpd2pi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "cvttpd2pi\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvttpd2pi,
    &operand_data[2123],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2359 */
  {
    "sse2_cvtsi2sd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1305 },
#else
    { 0, output_1305, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtsi2sd,
    &operand_data[2125],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:2379 */
  {
    "sse2_cvtsi2sdq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1306 },
#else
    { 0, output_1306, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtsi2sdq,
    &operand_data[2128],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:2401 */
  {
    "sse2_cvtsd2si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtsd2si\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtsd2si,
    &operand_data[2131],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2417 */
  {
    "sse2_cvtsd2si_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtsd2si\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtsd2si_2,
    &operand_data[279],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2431 */
  {
    "sse2_cvtsd2siq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtsd2si{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtsd2siq,
    &operand_data[2133],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2447 */
  {
    "sse2_cvtsd2siq_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtsd2si{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtsd2siq_2,
    &operand_data[275],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2461 */
  {
    "sse2_cvttsd2si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvttsd2si\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvttsd2si,
    &operand_data[2131],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2477 */
  {
    "sse2_cvttsd2siq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvttsd2si{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvttsd2siq,
    &operand_data[2133],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:2493 */
  {
    "floatv4siv4df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtdq2pd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_floatv4siv4df2,
    &operand_data[2135],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2502 */
  {
    "avx_cvtdq2pd256_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtdq2pd\t{%x1, %0|%0, %x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_cvtdq2pd256_2,
    &operand_data[2137],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2515 */
  {
    "sse2_cvtdq2pd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtdq2pd\t{%1, %0|%0, %q1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtdq2pd,
    &operand_data[2139],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2527 */
  {
    "avx_cvtpd2dq256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtpd2dq{y}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_cvtpd2dq256,
    &operand_data[2141],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2546 */
  {
    "*avx_cvtpd2dq256_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtpd2dq{y}\t{%1, %x0|%x0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2143],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2567 */
  {
    "*sse2_cvtpd2dq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1318 },
#else
    { 0, 0, output_1318 },
#endif
    0,
    &operand_data[2146],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:2589 */
  {
    "fix_truncv4dfv4si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvttpd2dq{y}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncv4dfv4si2,
    &operand_data[2141],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2606 */
  {
    "*avx_cvttpd2dq256_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvttpd2dq{y}\t{%1, %x0|%x0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2143],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2625 */
  {
    "*sse2_cvttpd2dq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1321 },
#else
    { 0, 0, output_1321 },
#endif
    0,
    &operand_data[2146],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:2644 */
  {
    "sse2_cvtsd2ss",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1322 },
#else
    { 0, output_1322, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtsd2ss,
    &operand_data[2149],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:2665 */
  {
    "sse2_cvtss2sd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1323 },
#else
    { 0, output_1323, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtss2sd,
    &operand_data[2152],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:2687 */
  {
    "avx_cvtpd2ps256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtpd2ps{y}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_cvtpd2ps256,
    &operand_data[2155],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2706 */
  {
    "*sse2_cvtpd2ps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1325 },
#else
    { 0, 0, output_1325 },
#endif
    0,
    &operand_data[2157],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:2727 */
  {
    "avx_cvtps2pd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtps2pd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_cvtps2pd256,
    &operand_data[2160],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2737 */
  {
    "*avx_cvtps2pd256_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtps2pd\t{%x1, %0|%0, %x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2162],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:2750 */
  {
    "sse2_cvtps2pd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vcvtps2pd\t{%1, %0|%0, %q1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtps2pd,
    &operand_data[2164],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:3229 */
  {
    "sse_movhlps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1329 },
#else
    { 0, output_1329, 0 },
#endif
    (insn_gen_fn) gen_sse_movhlps,
    &operand_data[2166],
    3,
    3,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:3274 */
  {
    "sse_movlhps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1330 },
#else
    { 0, output_1330, 0 },
#endif
    (insn_gen_fn) gen_sse_movlhps,
    &operand_data[2169],
    3,
    3,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:3297 */
  {
    "avx_unpckhps256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vunpckhps\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_unpckhps256,
    &operand_data[1996],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:3347 */
  {
    "vec_interleave_highv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1332 },
#else
    { 0, output_1332, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_highv4sf,
    &operand_data[1967],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:3365 */
  {
    "avx_unpcklps256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vunpcklps\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_unpcklps256,
    &operand_data[1996],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:3415 */
  {
    "vec_interleave_lowv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1334 },
#else
    { 0, output_1334, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_lowv4sf,
    &operand_data[1967],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:3434 */
  {
    "avx_movshdup256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vmovshdup\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_movshdup256,
    &operand_data[1979],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:3450 */
  {
    "sse3_movshdup",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovshdup\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse3_movshdup,
    &operand_data[1981],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:3467 */
  {
    "avx_movsldup256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vmovsldup\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_movsldup256,
    &operand_data[1979],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:3483 */
  {
    "sse3_movsldup",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovsldup\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse3_movsldup,
    &operand_data[1981],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:3521 */
  {
    "avx_shufps256_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1339 },
#else
    { 0, 0, output_1339 },
#endif
    (insn_gen_fn) gen_avx_shufps256_1,
    &operand_data[2172],
    11,
    11,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:3571 */
  {
    "sse_shufps_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1340 },
#else
    { 0, 0, output_1340 },
#endif
    (insn_gen_fn) gen_sse_shufps_v4si,
    &operand_data[2183],
    7,
    7,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:3571 */
  {
    "sse_shufps_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1341 },
#else
    { 0, 0, output_1341 },
#endif
    (insn_gen_fn) gen_sse_shufps_v4sf,
    &operand_data[2190],
    7,
    7,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:3606 */
  {
    "sse_storehps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1342 },
#else
    { 0, output_1342, 0 },
#endif
    (insn_gen_fn) gen_sse_storehps,
    &operand_data[2197],
    2,
    2,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:3640 */
  {
    "sse_loadhps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1343 },
#else
    { 0, output_1343, 0 },
#endif
    (insn_gen_fn) gen_sse_loadhps,
    &operand_data[2199],
    3,
    3,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:3659 */
  {
    "sse_storelps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1344 },
#else
    { 0, output_1344, 0 },
#endif
    (insn_gen_fn) gen_sse_storelps,
    &operand_data[2202],
    2,
    2,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:3693 */
  {
    "sse_loadlps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1345 },
#else
    { 0, output_1345, 0 },
#endif
    (insn_gen_fn) gen_sse_loadlps,
    &operand_data[2204],
    3,
    3,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:3713 */
  {
    "sse_movss",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1346 },
#else
    { 0, output_1346, 0 },
#endif
    (insn_gen_fn) gen_sse_movss,
    &operand_data[2207],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:3728 */
  {
    "avx2_vec_dupv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vbroadcastss\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_vec_dupv8sf,
    &operand_data[2210],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:3728 */
  {
    "avx2_vec_dupv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vbroadcastss\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_vec_dupv4sf,
    &operand_data[2000],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:3740 */
  {
    "vec_dupv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1349 },
#else
    { 0, output_1349, 0 },
#endif
    (insn_gen_fn) gen_vec_dupv4sf,
    &operand_data[2212],
    2,
    2,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:3758 */
  {
    "*vec_concatv2sf_sse4_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1350 },
#else
    { 0, output_1350, 0 },
#endif
    0,
    &operand_data[2214],
    3,
    3,
    0,
    7,
    2
  },
  /* ../../gcc/config/i386/sse.md:3783 */
  {
    "*vec_concatv2sf_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1351 },
#else
    { 0, output_1351, 0 },
#endif
    0,
    &operand_data[2217],
    3,
    3,
    0,
    4,
    2
  },
  /* ../../gcc/config/i386/sse.md:3797 */
  {
    "*vec_concatv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1352 },
#else
    { 0, output_1352, 0 },
#endif
    0,
    &operand_data[2220],
    3,
    3,
    0,
    4,
    2
  },
  /* ../../gcc/config/i386/sse.md:3824 */
  {
    "vec_setv4si_0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1353 },
#else
    { 0, output_1353, 0 },
#endif
    (insn_gen_fn) gen_vec_setv4si_0,
    &operand_data[2223],
    3,
    3,
    0,
    11,
    2
  },
  /* ../../gcc/config/i386/sse.md:3824 */
  {
    "vec_setv4sf_0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1354 },
#else
    { 0, output_1354, 0 },
#endif
    (insn_gen_fn) gen_vec_setv4sf_0,
    &operand_data[2226],
    3,
    3,
    0,
    11,
    2
  },
  /* ../../gcc/config/i386/sse.md:3863 */
  {
    "*vec_setv4sf_sse4_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1355 },
#else
    { 0, 0, output_1355 },
#endif
    0,
    &operand_data[2229],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:3893 */
  {
    "sse4_1_insertps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1356 },
#else
    { 0, 0, output_1356 },
#endif
    (insn_gen_fn) gen_sse4_1_insertps,
    &operand_data[2233],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:3952 */
  {
    "*vec_extractv4sf_0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2237],
    2,
    2,
    0,
    4,
    1
  },
  /* ../../gcc/config/i386/sse.md:3971 */
  {
    "*sse4_1_extractps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1358 },
#else
    { 0, output_1358, 0 },
#endif
    0,
    &operand_data[2239],
    3,
    3,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:4011 */
  {
    "*vec_extract_v4sf_mem",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2242],
    3,
    3,
    0,
    3,
    1
  },
  /* ../../gcc/config/i386/sse.md:4051 */
  {
    "vec_extract_lo_v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_extract_lo_v4di,
    &operand_data[2245],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:4051 */
  {
    "vec_extract_lo_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_extract_lo_v4df,
    &operand_data[2247],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:4070 */
  {
    "vec_extract_hi_v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vextract%~128\t{$0x1, %1, %0|%0, %1, 0x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_extract_hi_v4di,
    &operand_data[2249],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:4070 */
  {
    "vec_extract_hi_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vextractf128\t{$0x1, %1, %0|%0, %1, 0x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_extract_hi_v4df,
    &operand_data[2251],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:4084 */
  {
    "vec_extract_lo_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_extract_lo_v8si,
    &operand_data[2253],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:4084 */
  {
    "vec_extract_lo_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_extract_lo_v8sf,
    &operand_data[2255],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:4104 */
  {
    "vec_extract_hi_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vextract%~128\t{$0x1, %1, %0|%0, %1, 0x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_extract_hi_v8si,
    &operand_data[2257],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:4104 */
  {
    "vec_extract_hi_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vextractf128\t{$0x1, %1, %0|%0, %1, 0x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_extract_hi_v8sf,
    &operand_data[2259],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:4119 */
  {
    "vec_extract_lo_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_extract_lo_v16hi,
    &operand_data[2261],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:4141 */
  {
    "vec_extract_hi_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vextract%~128\t{$0x1, %1, %0|%0, %1, 0x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_extract_hi_v16hi,
    &operand_data[2263],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:4158 */
  {
    "vec_extract_lo_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_extract_lo_v32qi,
    &operand_data[2265],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:4184 */
  {
    "vec_extract_hi_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vextract%~128\t{$0x1, %1, %0|%0, %1, 0x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_extract_hi_v32qi,
    &operand_data[2267],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:4232 */
  {
    "avx_unpckhpd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vunpckhpd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_unpckhpd256,
    &operand_data[1993],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:4289 */
  {
    "*vec_interleave_highv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1373 },
#else
    { 0, output_1373, 0 },
#endif
    0,
    &operand_data[2269],
    3,
    3,
    0,
    6,
    2
  },
  /* ../../gcc/config/i386/sse.md:4332 */
  {
    "*avx_unpcklpd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1374 },
#else
    { 0, output_1374, 0 },
#endif
    0,
    &operand_data[2272],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4390 */
  {
    "*vec_interleave_lowv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1375 },
#else
    { 0, output_1375, 0 },
#endif
    0,
    &operand_data[2275],
    3,
    3,
    0,
    6,
    2
  },
  /* ../../gcc/config/i386/sse.md:4459 */
  {
    "avx_shufpd256_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1376 },
#else
    { 0, 0, output_1376 },
#endif
    (insn_gen_fn) gen_avx_shufpd256_1,
    &operand_data[2278],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:4500 */
  {
    "avx2_interleave_highv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpunpckhqdq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_interleave_highv4di,
    &operand_data[2285],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:4516 */
  {
    "vec_interleave_highv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1378 },
#else
    { 0, output_1378, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_highv2di,
    &operand_data[2288],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4534 */
  {
    "avx2_interleave_lowv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpunpcklqdq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_interleave_lowv4di,
    &operand_data[2285],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:4550 */
  {
    "vec_interleave_lowv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1380 },
#else
    { 0, output_1380, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_lowv2di,
    &operand_data[2288],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4568 */
  {
    "sse2_shufpd_v2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1381 },
#else
    { 0, 0, output_1381 },
#endif
    (insn_gen_fn) gen_sse2_shufpd_v2di,
    &operand_data[2288],
    5,
    5,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:4568 */
  {
    "sse2_shufpd_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1382 },
#else
    { 0, 0, output_1382 },
#endif
    (insn_gen_fn) gen_sse2_shufpd_v2df,
    &operand_data[2293],
    5,
    5,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:4601 */
  {
    "sse2_storehpd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1383 },
#else
    { 0, output_1383, 0 },
#endif
    (insn_gen_fn) gen_sse2_storehpd,
    &operand_data[2298],
    2,
    2,
    0,
    6,
    2
  },
  /* ../../gcc/config/i386/sse.md:4634 */
  {
    "*vec_extractv2df_1_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1384 },
#else
    { 0, output_1384, 0 },
#endif
    0,
    &operand_data[2300],
    2,
    2,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:4650 */
  {
    "sse2_storelpd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1385 },
#else
    { 0, output_1385, 0 },
#endif
    (insn_gen_fn) gen_sse2_storelpd,
    &operand_data[2302],
    2,
    2,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:4684 */
  {
    "*vec_extractv2df_0_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1386 },
#else
    { 0, output_1386, 0 },
#endif
    0,
    &operand_data[2304],
    2,
    2,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:4720 */
  {
    "sse2_loadhpd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1387 },
#else
    { 0, output_1387, 0 },
#endif
    (insn_gen_fn) gen_sse2_loadhpd,
    &operand_data[2306],
    3,
    3,
    0,
    7,
    2
  },
  /* ../../gcc/config/i386/sse.md:4776 */
  {
    "sse2_loadlpd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1388 },
#else
    { 0, output_1388, 0 },
#endif
    (insn_gen_fn) gen_sse2_loadlpd,
    &operand_data[2309],
    3,
    3,
    0,
    11,
    2
  },
  /* ../../gcc/config/i386/sse.md:4823 */
  {
    "sse2_movsd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1389 },
#else
    { 0, output_1389, 0 },
#endif
    (insn_gen_fn) gen_sse2_movsd,
    &operand_data[2312],
    3,
    3,
    0,
    9,
    2
  },
  /* ../../gcc/config/i386/sse.md:4856 */
  {
    "vec_dupv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1390 },
#else
    { 0, output_1390, 0 },
#endif
    (insn_gen_fn) gen_vec_dupv2df,
    &operand_data[2315],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4869 */
  {
    "*vec_concatv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1391 },
#else
    { 0, output_1391, 0 },
#endif
    0,
    &operand_data[2317],
    3,
    3,
    0,
    8,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*addv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1392 },
#else
    { 0, output_1392, 0 },
#endif
    0,
    &operand_data[2320],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*subv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1393 },
#else
    { 0, output_1393, 0 },
#endif
    0,
    &operand_data[2323],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*addv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1394 },
#else
    { 0, output_1394, 0 },
#endif
    0,
    &operand_data[2326],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*subv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1395 },
#else
    { 0, output_1395, 0 },
#endif
    0,
    &operand_data[2329],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*addv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1396 },
#else
    { 0, output_1396, 0 },
#endif
    0,
    &operand_data[2332],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*subv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1397 },
#else
    { 0, output_1397, 0 },
#endif
    0,
    &operand_data[2335],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*addv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1398 },
#else
    { 0, output_1398, 0 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*subv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1399 },
#else
    { 0, output_1399, 0 },
#endif
    0,
    &operand_data[2341],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*addv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1400 },
#else
    { 0, output_1400, 0 },
#endif
    0,
    &operand_data[2344],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*subv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1401 },
#else
    { 0, output_1401, 0 },
#endif
    0,
    &operand_data[2347],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*addv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1402 },
#else
    { 0, output_1402, 0 },
#endif
    0,
    &operand_data[2350],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*subv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1403 },
#else
    { 0, output_1403, 0 },
#endif
    0,
    &operand_data[2353],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*addv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1404 },
#else
    { 0, output_1404, 0 },
#endif
    0,
    &operand_data[2356],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*subv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1405 },
#else
    { 0, output_1405, 0 },
#endif
    0,
    &operand_data[2359],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*addv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1406 },
#else
    { 0, output_1406, 0 },
#endif
    0,
    &operand_data[2362],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4916 */
  {
    "*subv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1407 },
#else
    { 0, output_1407, 0 },
#endif
    0,
    &operand_data[2365],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*avx2_ssaddv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1408 },
#else
    { 0, output_1408, 0 },
#endif
    0,
    &operand_data[2320],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*avx2_usaddv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1409 },
#else
    { 0, output_1409, 0 },
#endif
    0,
    &operand_data[2320],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*avx2_sssubv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1410 },
#else
    { 0, output_1410, 0 },
#endif
    0,
    &operand_data[2323],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*avx2_ussubv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1411 },
#else
    { 0, output_1411, 0 },
#endif
    0,
    &operand_data[2323],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*sse2_ssaddv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1412 },
#else
    { 0, output_1412, 0 },
#endif
    0,
    &operand_data[2326],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*sse2_usaddv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1413 },
#else
    { 0, output_1413, 0 },
#endif
    0,
    &operand_data[2326],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*sse2_sssubv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1414 },
#else
    { 0, output_1414, 0 },
#endif
    0,
    &operand_data[2329],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*sse2_ussubv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1415 },
#else
    { 0, output_1415, 0 },
#endif
    0,
    &operand_data[2329],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*avx2_ssaddv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1416 },
#else
    { 0, output_1416, 0 },
#endif
    0,
    &operand_data[2332],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*avx2_usaddv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1417 },
#else
    { 0, output_1417, 0 },
#endif
    0,
    &operand_data[2332],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*avx2_sssubv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1418 },
#else
    { 0, output_1418, 0 },
#endif
    0,
    &operand_data[2335],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*avx2_ussubv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1419 },
#else
    { 0, output_1419, 0 },
#endif
    0,
    &operand_data[2335],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*sse2_ssaddv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1420 },
#else
    { 0, output_1420, 0 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*sse2_usaddv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1421 },
#else
    { 0, output_1421, 0 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*sse2_sssubv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1422 },
#else
    { 0, output_1422, 0 },
#endif
    0,
    &operand_data[2341],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4939 */
  {
    "*sse2_ussubv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1423 },
#else
    { 0, output_1423, 0 },
#endif
    0,
    &operand_data[2341],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:4954 */
  {
    "mulv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mulv32qi3,
    &operand_data[2368],
    3,
    3,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/sse.md:4954 */
  {
    "mulv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mulv16qi3,
    &operand_data[2371],
    3,
    3,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/sse.md:5028 */
  {
    "*mulv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1426 },
#else
    { 0, output_1426, 0 },
#endif
    0,
    &operand_data[2332],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5028 */
  {
    "*mulv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1427 },
#else
    { 0, output_1427, 0 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5055 */
  {
    "*smulv16hi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1428 },
#else
    { 0, output_1428, 0 },
#endif
    0,
    &operand_data[2332],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5055 */
  {
    "*umulv16hi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1429 },
#else
    { 0, output_1429, 0 },
#endif
    0,
    &operand_data[2332],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5055 */
  {
    "*smulv8hi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1430 },
#else
    { 0, output_1430, 0 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5055 */
  {
    "*umulv8hi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1431 },
#else
    { 0, output_1431, 0 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5091 */
  {
    "*avx_umulv4siv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmuludq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2374],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5124 */
  {
    "*sse2_umulv2siv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1433 },
#else
    { 0, output_1433, 0 },
#endif
    0,
    &operand_data[2377],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5161 */
  {
    "*avx2_mulv4siv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmuldq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2380],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5196 */
  {
    "*sse4_1_mulv2siv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1435 },
#else
    { 0, output_1435, 0 },
#endif
    0,
    &operand_data[2377],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5302 */
  {
    "*avx2_pmaddwd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaddwd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2383],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5355 */
  {
    "*sse2_pmaddwd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1437 },
#else
    { 0, output_1437, 0 },
#endif
    0,
    &operand_data[2386],
    3,
    3,
    2,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5407 */
  {
    "*avx2_mulv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1438 },
#else
    { 0, output_1438, 0 },
#endif
    0,
    &operand_data[2344],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5407 */
  {
    "*sse4_1_mulv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1439 },
#else
    { 0, output_1439, 0 },
#endif
    0,
    &operand_data[2350],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5421 */
  {
    "*sse2_mulv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2389],
    3,
    3,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/sse.md:5477 */
  {
    "mulv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mulv4di3,
    &operand_data[2392],
    3,
    3,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/sse.md:5477 */
  {
    "mulv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mulv2di3,
    &operand_data[2395],
    3,
    3,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/sse.md:5839 */
  {
    "ashrv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1443 },
#else
    { 0, output_1443, 0 },
#endif
    (insn_gen_fn) gen_ashrv16hi3,
    &operand_data[2398],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5839 */
  {
    "ashrv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1444 },
#else
    { 0, output_1444, 0 },
#endif
    (insn_gen_fn) gen_ashrv8hi3,
    &operand_data[2401],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5839 */
  {
    "ashrv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1445 },
#else
    { 0, output_1445, 0 },
#endif
    (insn_gen_fn) gen_ashrv8si3,
    &operand_data[2404],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5839 */
  {
    "ashrv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1446 },
#else
    { 0, output_1446, 0 },
#endif
    (insn_gen_fn) gen_ashrv4si3,
    &operand_data[2407],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5858 */
  {
    "ashlv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1447 },
#else
    { 0, output_1447, 0 },
#endif
    (insn_gen_fn) gen_ashlv16hi3,
    &operand_data[2398],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5858 */
  {
    "lshrv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1448 },
#else
    { 0, output_1448, 0 },
#endif
    (insn_gen_fn) gen_lshrv16hi3,
    &operand_data[2398],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5858 */
  {
    "ashlv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1449 },
#else
    { 0, output_1449, 0 },
#endif
    (insn_gen_fn) gen_ashlv8hi3,
    &operand_data[2401],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5858 */
  {
    "lshrv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1450 },
#else
    { 0, output_1450, 0 },
#endif
    (insn_gen_fn) gen_lshrv8hi3,
    &operand_data[2401],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5858 */
  {
    "ashlv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1451 },
#else
    { 0, output_1451, 0 },
#endif
    (insn_gen_fn) gen_ashlv8si3,
    &operand_data[2404],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5858 */
  {
    "lshrv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1452 },
#else
    { 0, output_1452, 0 },
#endif
    (insn_gen_fn) gen_lshrv8si3,
    &operand_data[2404],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5858 */
  {
    "ashlv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1453 },
#else
    { 0, output_1453, 0 },
#endif
    (insn_gen_fn) gen_ashlv4si3,
    &operand_data[2407],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5858 */
  {
    "lshrv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1454 },
#else
    { 0, output_1454, 0 },
#endif
    (insn_gen_fn) gen_lshrv4si3,
    &operand_data[2407],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5858 */
  {
    "ashlv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1455 },
#else
    { 0, output_1455, 0 },
#endif
    (insn_gen_fn) gen_ashlv4di3,
    &operand_data[2410],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5858 */
  {
    "lshrv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1456 },
#else
    { 0, output_1456, 0 },
#endif
    (insn_gen_fn) gen_lshrv4di3,
    &operand_data[2410],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5858 */
  {
    "ashlv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1457 },
#else
    { 0, output_1457, 0 },
#endif
    (insn_gen_fn) gen_ashlv2di3,
    &operand_data[2413],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5858 */
  {
    "lshrv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1458 },
#else
    { 0, output_1458, 0 },
#endif
    (insn_gen_fn) gen_lshrv2di3,
    &operand_data[2413],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:5888 */
  {
    "avx2_ashlv2ti3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1459 },
#else
    { 0, 0, output_1459 },
#endif
    (insn_gen_fn) gen_avx2_ashlv2ti3,
    &operand_data[2416],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:5888 */
  {
    "sse2_ashlv1ti3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1460 },
#else
    { 0, 0, output_1460 },
#endif
    (insn_gen_fn) gen_sse2_ashlv1ti3,
    &operand_data[2419],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:5925 */
  {
    "avx2_lshrv2ti3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1461 },
#else
    { 0, 0, output_1461 },
#endif
    (insn_gen_fn) gen_avx2_lshrv2ti3,
    &operand_data[2416],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:5925 */
  {
    "sse2_lshrv1ti3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1462 },
#else
    { 0, 0, output_1462 },
#endif
    (insn_gen_fn) gen_sse2_lshrv1ti3,
    &operand_data[2419],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:5961 */
  {
    "*avx2_smaxv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaxsb\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2422],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5961 */
  {
    "*avx2_sminv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpminsb\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2422],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5961 */
  {
    "*avx2_umaxv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaxub\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2422],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5961 */
  {
    "*avx2_uminv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpminub\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2422],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5961 */
  {
    "*avx2_smaxv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaxsw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2425],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5961 */
  {
    "*avx2_sminv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpminsw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2425],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5961 */
  {
    "*avx2_umaxv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaxuw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2425],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5961 */
  {
    "*avx2_uminv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpminuw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2425],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5961 */
  {
    "*avx2_smaxv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaxsd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2428],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5961 */
  {
    "*avx2_sminv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpminsd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2428],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5961 */
  {
    "*avx2_umaxv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaxud\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2428],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:5961 */
  {
    "*avx2_uminv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpminud\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2428],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6047 */
  {
    "*sse4_1_smaxv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1475 },
#else
    { 0, output_1475, 0 },
#endif
    0,
    &operand_data[2326],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6047 */
  {
    "*sse4_1_sminv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1476 },
#else
    { 0, output_1476, 0 },
#endif
    0,
    &operand_data[2326],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6047 */
  {
    "*sse4_1_smaxv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1477 },
#else
    { 0, output_1477, 0 },
#endif
    0,
    &operand_data[2350],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6047 */
  {
    "*sse4_1_sminv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1478 },
#else
    { 0, output_1478, 0 },
#endif
    0,
    &operand_data[2350],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6062 */
  {
    "*smaxv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1479 },
#else
    { 0, output_1479, 0 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6062 */
  {
    "*sminv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1480 },
#else
    { 0, output_1480, 0 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6128 */
  {
    "*sse4_1_umaxv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1481 },
#else
    { 0, output_1481, 0 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6128 */
  {
    "*sse4_1_uminv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1482 },
#else
    { 0, output_1482, 0 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6128 */
  {
    "*sse4_1_umaxv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1483 },
#else
    { 0, output_1483, 0 },
#endif
    0,
    &operand_data[2350],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6128 */
  {
    "*sse4_1_uminv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1484 },
#else
    { 0, output_1484, 0 },
#endif
    0,
    &operand_data[2350],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6143 */
  {
    "*umaxv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1485 },
#else
    { 0, output_1485, 0 },
#endif
    0,
    &operand_data[2326],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6143 */
  {
    "*uminv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1486 },
#else
    { 0, output_1486, 0 },
#endif
    0,
    &operand_data[2326],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6173 */
  {
    "*avx2_eqv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmpeqb\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2422],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6173 */
  {
    "*avx2_eqv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmpeqw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2425],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6173 */
  {
    "*avx2_eqv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmpeqd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2428],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6173 */
  {
    "*avx2_eqv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmpeqq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2431],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6185 */
  {
    "*sse4_1_eqv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1491 },
#else
    { 0, output_1491, 0 },
#endif
    0,
    &operand_data[2362],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6200 */
  {
    "*sse2_eqv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1492 },
#else
    { 0, output_1492, 0 },
#endif
    0,
    &operand_data[2326],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6200 */
  {
    "*sse2_eqv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1493 },
#else
    { 0, output_1493, 0 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6200 */
  {
    "*sse2_eqv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1494 },
#else
    { 0, output_1494, 0 },
#endif
    0,
    &operand_data[2350],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6232 */
  {
    "sse4_2_gtv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1495 },
#else
    { 0, output_1495, 0 },
#endif
    (insn_gen_fn) gen_sse4_2_gtv2di3,
    &operand_data[2288],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6247 */
  {
    "avx2_gtv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmpgtb\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gtv32qi3,
    &operand_data[2434],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6247 */
  {
    "avx2_gtv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmpgtw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gtv16hi3,
    &operand_data[2437],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6247 */
  {
    "avx2_gtv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmpgtd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gtv8si3,
    &operand_data[2440],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6247 */
  {
    "avx2_gtv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmpgtq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gtv4di3,
    &operand_data[2285],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6259 */
  {
    "sse2_gtv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1500 },
#else
    { 0, output_1500, 0 },
#endif
    (insn_gen_fn) gen_sse2_gtv16qi3,
    &operand_data[2443],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6259 */
  {
    "sse2_gtv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1501 },
#else
    { 0, output_1501, 0 },
#endif
    (insn_gen_fn) gen_sse2_gtv8hi3,
    &operand_data[2446],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6259 */
  {
    "sse2_gtv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1502 },
#else
    { 0, output_1502, 0 },
#endif
    (insn_gen_fn) gen_sse2_gtv4si3,
    &operand_data[2183],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6438 */
  {
    "*andnotv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1503 },
#else
    { 0, 0, output_1503 },
#endif
    0,
    &operand_data[2449],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6438 */
  {
    "*andnotv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1504 },
#else
    { 0, 0, output_1504 },
#endif
    0,
    &operand_data[2443],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6438 */
  {
    "*andnotv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1505 },
#else
    { 0, 0, output_1505 },
#endif
    0,
    &operand_data[2452],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6438 */
  {
    "*andnotv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1506 },
#else
    { 0, 0, output_1506 },
#endif
    0,
    &operand_data[2446],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6438 */
  {
    "*andnotv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1507 },
#else
    { 0, 0, output_1507 },
#endif
    0,
    &operand_data[2455],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6438 */
  {
    "*andnotv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1508 },
#else
    { 0, 0, output_1508 },
#endif
    0,
    &operand_data[2183],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6438 */
  {
    "*andnotv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1509 },
#else
    { 0, 0, output_1509 },
#endif
    0,
    &operand_data[2458],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6438 */
  {
    "*andnotv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1510 },
#else
    { 0, 0, output_1510 },
#endif
    0,
    &operand_data[2288],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*andv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1511 },
#else
    { 0, 0, output_1511 },
#endif
    0,
    &operand_data[2320],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*iorv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1512 },
#else
    { 0, 0, output_1512 },
#endif
    0,
    &operand_data[2320],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*xorv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1513 },
#else
    { 0, 0, output_1513 },
#endif
    0,
    &operand_data[2320],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*andv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1514 },
#else
    { 0, 0, output_1514 },
#endif
    0,
    &operand_data[2326],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*iorv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1515 },
#else
    { 0, 0, output_1515 },
#endif
    0,
    &operand_data[2326],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*xorv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1516 },
#else
    { 0, 0, output_1516 },
#endif
    0,
    &operand_data[2326],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*andv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1517 },
#else
    { 0, 0, output_1517 },
#endif
    0,
    &operand_data[2332],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*iorv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1518 },
#else
    { 0, 0, output_1518 },
#endif
    0,
    &operand_data[2332],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*xorv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1519 },
#else
    { 0, 0, output_1519 },
#endif
    0,
    &operand_data[2332],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*andv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1520 },
#else
    { 0, 0, output_1520 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*iorv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1521 },
#else
    { 0, 0, output_1521 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*xorv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1522 },
#else
    { 0, 0, output_1522 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*andv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1523 },
#else
    { 0, 0, output_1523 },
#endif
    0,
    &operand_data[2344],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*iorv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1524 },
#else
    { 0, 0, output_1524 },
#endif
    0,
    &operand_data[2344],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*xorv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1525 },
#else
    { 0, 0, output_1525 },
#endif
    0,
    &operand_data[2344],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*andv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1526 },
#else
    { 0, 0, output_1526 },
#endif
    0,
    &operand_data[2350],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*iorv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1527 },
#else
    { 0, 0, output_1527 },
#endif
    0,
    &operand_data[2350],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*xorv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1528 },
#else
    { 0, 0, output_1528 },
#endif
    0,
    &operand_data[2350],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*andv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1529 },
#else
    { 0, 0, output_1529 },
#endif
    0,
    &operand_data[2356],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*iorv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1530 },
#else
    { 0, 0, output_1530 },
#endif
    0,
    &operand_data[2356],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*xorv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1531 },
#else
    { 0, 0, output_1531 },
#endif
    0,
    &operand_data[2356],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*andv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1532 },
#else
    { 0, 0, output_1532 },
#endif
    0,
    &operand_data[2362],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*iorv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1533 },
#else
    { 0, 0, output_1533 },
#endif
    0,
    &operand_data[2362],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6512 */
  {
    "*xorv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1534 },
#else
    { 0, 0, output_1534 },
#endif
    0,
    &operand_data[2362],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:6579 */
  {
    "*andnottf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1535 },
#else
    { 0, output_1535, 0 },
#endif
    0,
    &operand_data[2461],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6602 */
  {
    "*andtf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1536 },
#else
    { 0, output_1536, 0 },
#endif
    0,
    &operand_data[2464],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6602 */
  {
    "*iortf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1537 },
#else
    { 0, output_1537, 0 },
#endif
    0,
    &operand_data[2464],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6602 */
  {
    "*xortf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1538 },
#else
    { 0, output_1538, 0 },
#endif
    0,
    &operand_data[2464],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6636 */
  {
    "avx2_packsswb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1539 },
#else
    { 0, output_1539, 0 },
#endif
    (insn_gen_fn) gen_avx2_packsswb,
    &operand_data[2467],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6636 */
  {
    "sse2_packsswb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1540 },
#else
    { 0, output_1540, 0 },
#endif
    (insn_gen_fn) gen_sse2_packsswb,
    &operand_data[2470],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6653 */
  {
    "avx2_packssdw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1541 },
#else
    { 0, output_1541, 0 },
#endif
    (insn_gen_fn) gen_avx2_packssdw,
    &operand_data[2473],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6653 */
  {
    "sse2_packssdw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1542 },
#else
    { 0, output_1542, 0 },
#endif
    (insn_gen_fn) gen_sse2_packssdw,
    &operand_data[2476],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6670 */
  {
    "avx2_packuswb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1543 },
#else
    { 0, output_1543, 0 },
#endif
    (insn_gen_fn) gen_avx2_packuswb,
    &operand_data[2467],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6670 */
  {
    "sse2_packuswb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1544 },
#else
    { 0, output_1544, 0 },
#endif
    (insn_gen_fn) gen_sse2_packuswb,
    &operand_data[2470],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6687 */
  {
    "avx2_interleave_highv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpunpckhbw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_interleave_highv32qi,
    &operand_data[2434],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6715 */
  {
    "vec_interleave_highv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1546 },
#else
    { 0, output_1546, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_highv16qi,
    &operand_data[2443],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6739 */
  {
    "avx2_interleave_lowv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpunpcklbw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_interleave_lowv32qi,
    &operand_data[2434],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6767 */
  {
    "vec_interleave_lowv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1548 },
#else
    { 0, output_1548, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_lowv16qi,
    &operand_data[2443],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6791 */
  {
    "avx2_interleave_highv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpunpckhwd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_interleave_highv16hi,
    &operand_data[2437],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6811 */
  {
    "vec_interleave_highv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1550 },
#else
    { 0, output_1550, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_highv8hi,
    &operand_data[2446],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6831 */
  {
    "avx2_interleave_lowv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpunpcklwd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_interleave_lowv16hi,
    &operand_data[2437],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6851 */
  {
    "vec_interleave_lowv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1552 },
#else
    { 0, output_1552, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_lowv8hi,
    &operand_data[2446],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6871 */
  {
    "avx2_interleave_highv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpunpckhdq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_interleave_highv8si,
    &operand_data[2440],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6887 */
  {
    "vec_interleave_highv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1554 },
#else
    { 0, output_1554, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_highv4si,
    &operand_data[2183],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6905 */
  {
    "avx2_interleave_lowv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpunpckldq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_interleave_lowv8si,
    &operand_data[2440],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:6921 */
  {
    "vec_interleave_lowv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1556 },
#else
    { 0, output_1556, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_lowv4si,
    &operand_data[2183],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:6982 */
  {
    "sse4_1_pinsrb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1557 },
#else
    { 0, 0, output_1557 },
#endif
    (insn_gen_fn) gen_sse4_1_pinsrb,
    &operand_data[2479],
    4,
    4,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/sse.md:6982 */
  {
    "sse2_pinsrw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1558 },
#else
    { 0, 0, output_1558 },
#endif
    (insn_gen_fn) gen_sse2_pinsrw,
    &operand_data[2483],
    4,
    4,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/sse.md:6982 */
  {
    "sse4_1_pinsrd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1559 },
#else
    { 0, 0, output_1559 },
#endif
    (insn_gen_fn) gen_sse4_1_pinsrd,
    &operand_data[2487],
    4,
    4,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/sse.md:6982 */
  {
    "sse4_1_pinsrq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1560 },
#else
    { 0, 0, output_1560 },
#endif
    (insn_gen_fn) gen_sse4_1_pinsrq,
    &operand_data[2491],
    4,
    4,
    0,
    4,
    3
  },
  /* ../../gcc/config/i386/sse.md:7037 */
  {
    "*sse4_1_pextrb_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpextrb\t{%2, %1, %k0|%k0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2495],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7037 */
  {
    "*sse4_1_pextrb_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpextrb\t{%2, %1, %k0|%k0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2498],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7051 */
  {
    "*sse4_1_pextrb_memory",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpextrb\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2501],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7064 */
  {
    "*sse2_pextrw_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpextrw\t{%2, %1, %k0|%k0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2504],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7064 */
  {
    "*sse2_pextrw_di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpextrw\t{%2, %1, %k0|%k0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2507],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7078 */
  {
    "*sse4_1_pextrw_memory",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpextrw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2510],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7091 */
  {
    "*sse4_1_pextrd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpextrd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2513],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7104 */
  {
    "*sse4_1_pextrd_zext",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpextrd\t{%2, %1, %k0|%k0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2516],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7119 */
  {
    "*sse4_1_pextrq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpextrq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2519],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7152 */
  {
    "avx2_pshufd_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1570 },
#else
    { 0, 0, output_1570 },
#endif
    (insn_gen_fn) gen_avx2_pshufd_1,
    &operand_data[2522],
    10,
    10,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:7199 */
  {
    "sse2_pshufd_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1571 },
#else
    { 0, 0, output_1571 },
#endif
    (insn_gen_fn) gen_sse2_pshufd_1,
    &operand_data[2532],
    6,
    6,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:7243 */
  {
    "avx2_pshuflw_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1572 },
#else
    { 0, 0, output_1572 },
#endif
    (insn_gen_fn) gen_avx2_pshuflw_1,
    &operand_data[2538],
    10,
    10,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:7298 */
  {
    "sse2_pshuflw_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1573 },
#else
    { 0, 0, output_1573 },
#endif
    (insn_gen_fn) gen_sse2_pshuflw_1,
    &operand_data[2548],
    6,
    6,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:7347 */
  {
    "avx2_pshufhw_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1574 },
#else
    { 0, 0, output_1574 },
#endif
    (insn_gen_fn) gen_avx2_pshufhw_1,
    &operand_data[2554],
    10,
    10,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:7402 */
  {
    "sse2_pshufhw_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1575 },
#else
    { 0, 0, output_1575 },
#endif
    (insn_gen_fn) gen_sse2_pshufhw_1,
    &operand_data[2564],
    6,
    6,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:7442 */
  {
    "sse2_loadld",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1576 },
#else
    { 0, output_1576, 0 },
#endif
    (insn_gen_fn) gen_sse2_loadld,
    &operand_data[2570],
    3,
    3,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:7461 */
  {
    "sse2_stored",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_stored,
    &operand_data[2573],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:7475 */
  {
    "*vec_ext_v4si_mem",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2575],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7498 */
  {
    "*sse2_storeq_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1579 },
#else
    { 0, output_1579, 0 },
#endif
    0,
    &operand_data[2578],
    2,
    2,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:7511 */
  {
    "*sse2_storeq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2580],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7532 */
  {
    "*vec_extractv2di_1_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1581 },
#else
    { 0, output_1581, 0 },
#endif
    0,
    &operand_data[2582],
    2,
    2,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:7551 */
  {
    "*vec_extractv2di_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1582 },
#else
    { 0, output_1582, 0 },
#endif
    0,
    &operand_data[2584],
    2,
    2,
    0,
    6,
    2
  },
  /* ../../gcc/config/i386/sse.md:7572 */
  {
    "*vec_dupv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1583 },
#else
    { 0, output_1583, 0 },
#endif
    0,
    &operand_data[2586],
    2,
    2,
    0,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:7588 */
  {
    "*vec_dupv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1584 },
#else
    { 0, output_1584, 0 },
#endif
    0,
    &operand_data[2588],
    2,
    2,
    0,
    4,
    2
  },
  /* ../../gcc/config/i386/sse.md:7603 */
  {
    "*vec_concatv2si_sse4_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1585 },
#else
    { 0, output_1585, 0 },
#endif
    0,
    &operand_data[2590],
    3,
    3,
    0,
    7,
    2
  },
  /* ../../gcc/config/i386/sse.md:7627 */
  {
    "*vec_concatv2si_sse2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1586 },
#else
    { 0, output_1586, 0 },
#endif
    0,
    &operand_data[2593],
    3,
    3,
    0,
    4,
    2
  },
  /* ../../gcc/config/i386/sse.md:7641 */
  {
    "*vec_concatv2si_sse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1587 },
#else
    { 0, output_1587, 0 },
#endif
    0,
    &operand_data[2596],
    3,
    3,
    0,
    4,
    2
  },
  /* ../../gcc/config/i386/sse.md:7655 */
  {
    "*vec_concatv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1588 },
#else
    { 0, output_1588, 0 },
#endif
    0,
    &operand_data[2599],
    3,
    3,
    0,
    5,
    2
  },
  /* ../../gcc/config/i386/sse.md:7673 */
  {
    "*vec_concatv2di_rex64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1589 },
#else
    { 0, output_1589, 0 },
#endif
    0,
    &operand_data[2602],
    3,
    3,
    0,
    9,
    2
  },
  /* ../../gcc/config/i386/sse.md:7709 */
  {
    "vec_concatv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1590 },
#else
    { 0, output_1590, 0 },
#endif
    (insn_gen_fn) gen_vec_concatv2di,
    &operand_data[2605],
    3,
    3,
    0,
    7,
    2
  },
  /* ../../gcc/config/i386/sse.md:7810 */
  {
    "*avx2_uavgv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpavgb\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2422],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7843 */
  {
    "*sse2_uavgv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1592 },
#else
    { 0, output_1592, 0 },
#endif
    0,
    &operand_data[2326],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:7912 */
  {
    "*avx2_uavgv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpavgw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2425],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7937 */
  {
    "*sse2_uavgv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1594 },
#else
    { 0, output_1594, 0 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:7964 */
  {
    "avx2_psadbw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1595 },
#else
    { 0, output_1595, 0 },
#endif
    (insn_gen_fn) gen_avx2_psadbw,
    &operand_data[2608],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:7964 */
  {
    "sse2_psadbw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1596 },
#else
    { 0, output_1596, 0 },
#endif
    (insn_gen_fn) gen_sse2_psadbw,
    &operand_data[2611],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:7980 */
  {
    "avx_movmskps256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovmskps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_movmskps256,
    &operand_data[2614],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7980 */
  {
    "sse_movmskps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovmskps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_movmskps,
    &operand_data[2616],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7980 */
  {
    "avx_movmskpd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovmskpd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_movmskpd256,
    &operand_data[2618],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7980 */
  {
    "sse2_movmskpd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovmskpd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_movmskpd,
    &operand_data[2620],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:7991 */
  {
    "avx2_pmovmskb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmovmskb\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pmovmskb,
    &operand_data[2622],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8001 */
  {
    "sse2_pmovmskb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpmovmskb\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_pmovmskb,
    &operand_data[2495],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8020 */
  {
    "*sse2_maskmovdqu",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmaskmovdqu\t{%2, %1|%1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2624],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8020 */
  {
    "*sse2_maskmovdqu",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmaskmovdqu\t{%2, %1|%1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2627],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8036 */
  {
    "sse_ldmxcsr",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vldmxcsr\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_ldmxcsr,
    &operand_data[49],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8046 */
  {
    "sse_stmxcsr",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vstmxcsr\t%0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse_stmxcsr,
    &operand_data[290],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8056 */
  {
    "sse2_clflush",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "clflush\t%a0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse2_clflush,
    &operand_data[1658],
    1,
    1,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8066 */
  {
    "sse3_mwait",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mwait",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse3_mwait,
    &operand_data[2630],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8077 */
  {
    "sse3_monitor",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "monitor\t%0, %1, %2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse3_monitor,
    &operand_data[2630],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8086 */
  {
    "sse3_monitor64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "monitor",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse3_monitor64,
    &operand_data[2633],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8104 */
  {
    "avx2_phaddwv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphaddw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_phaddwv16hi3,
    &operand_data[2437],
    3,
    3,
    30,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8180 */
  {
    "ssse3_phaddwv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1612 },
#else
    { 0, output_1612, 0 },
#endif
    (insn_gen_fn) gen_ssse3_phaddwv8hi3,
    &operand_data[2446],
    3,
    3,
    14,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:8229 */
  {
    "ssse3_phaddwv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "phaddw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ssse3_phaddwv4hi3,
    &operand_data[1812],
    3,
    3,
    6,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8258 */
  {
    "avx2_phadddv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphaddd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_phadddv8si3,
    &operand_data[2440],
    3,
    3,
    14,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8302 */
  {
    "ssse3_phadddv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1615 },
#else
    { 0, output_1615, 0 },
#endif
    (insn_gen_fn) gen_ssse3_phadddv4si3,
    &operand_data[2183],
    3,
    3,
    6,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:8335 */
  {
    "ssse3_phadddv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "phaddd\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ssse3_phadddv2si3,
    &operand_data[1815],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8356 */
  {
    "avx2_phaddswv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphaddsw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_phaddswv16hi3,
    &operand_data[2437],
    3,
    3,
    30,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8432 */
  {
    "ssse3_phaddswv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1618 },
#else
    { 0, output_1618, 0 },
#endif
    (insn_gen_fn) gen_ssse3_phaddswv8hi3,
    &operand_data[2446],
    3,
    3,
    14,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:8481 */
  {
    "ssse3_phaddswv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "phaddsw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ssse3_phaddswv4hi3,
    &operand_data[1812],
    3,
    3,
    6,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8510 */
  {
    "avx2_phsubwv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphsubw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_phsubwv16hi3,
    &operand_data[2437],
    3,
    3,
    30,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8586 */
  {
    "ssse3_phsubwv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1621 },
#else
    { 0, output_1621, 0 },
#endif
    (insn_gen_fn) gen_ssse3_phsubwv8hi3,
    &operand_data[2446],
    3,
    3,
    14,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:8635 */
  {
    "ssse3_phsubwv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "phsubw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ssse3_phsubwv4hi3,
    &operand_data[1812],
    3,
    3,
    6,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8664 */
  {
    "avx2_phsubdv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphsubd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_phsubdv8si3,
    &operand_data[2440],
    3,
    3,
    14,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8708 */
  {
    "ssse3_phsubdv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1624 },
#else
    { 0, output_1624, 0 },
#endif
    (insn_gen_fn) gen_ssse3_phsubdv4si3,
    &operand_data[2183],
    3,
    3,
    6,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:8742 */
  {
    "ssse3_phsubdv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "phsubd\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ssse3_phsubdv2si3,
    &operand_data[1815],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8763 */
  {
    "avx2_phsubswv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphsubsw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_phsubswv16hi3,
    &operand_data[2437],
    3,
    3,
    30,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8839 */
  {
    "ssse3_phsubswv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1627 },
#else
    { 0, output_1627, 0 },
#endif
    (insn_gen_fn) gen_ssse3_phsubswv8hi3,
    &operand_data[2446],
    3,
    3,
    14,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:8888 */
  {
    "ssse3_phsubswv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "phsubsw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ssse3_phsubswv4hi3,
    &operand_data[1812],
    3,
    3,
    6,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:8917 */
  {
    "avx2_pmaddubsw256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaddubsw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pmaddubsw256,
    &operand_data[2636],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9003 */
  {
    "ssse3_pmaddubsw128",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1630 },
#else
    { 0, output_1630, 0 },
#endif
    (insn_gen_fn) gen_ssse3_pmaddubsw128,
    &operand_data[2639],
    3,
    3,
    2,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9062 */
  {
    "ssse3_pmaddubsw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pmaddubsw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ssse3_pmaddubsw,
    &operand_data[2642],
    3,
    3,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9125 */
  {
    "*avx2_umulhrswv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmulhrsw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2425],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9173 */
  {
    "*ssse3_pmulhrswv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1633 },
#else
    { 0, output_1633, 0 },
#endif
    0,
    &operand_data[2338],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9219 */
  {
    "*ssse3_pmulhrswv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pmulhrsw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1776],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9241 */
  {
    "avx2_pshufbv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1635 },
#else
    { 0, output_1635, 0 },
#endif
    (insn_gen_fn) gen_avx2_pshufbv32qi3,
    &operand_data[2449],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9241 */
  {
    "ssse3_pshufbv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1636 },
#else
    { 0, output_1636, 0 },
#endif
    (insn_gen_fn) gen_ssse3_pshufbv16qi3,
    &operand_data[2443],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9257 */
  {
    "ssse3_pshufbv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pshufb\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ssse3_pshufbv8qi3,
    &operand_data[1809],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9269 */
  {
    "avx2_psignv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1638 },
#else
    { 0, output_1638, 0 },
#endif
    (insn_gen_fn) gen_avx2_psignv32qi3,
    &operand_data[2449],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9269 */
  {
    "ssse3_psignv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1639 },
#else
    { 0, output_1639, 0 },
#endif
    (insn_gen_fn) gen_ssse3_psignv16qi3,
    &operand_data[2443],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9269 */
  {
    "avx2_psignv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1640 },
#else
    { 0, output_1640, 0 },
#endif
    (insn_gen_fn) gen_avx2_psignv16hi3,
    &operand_data[2452],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9269 */
  {
    "ssse3_psignv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1641 },
#else
    { 0, output_1641, 0 },
#endif
    (insn_gen_fn) gen_ssse3_psignv8hi3,
    &operand_data[2446],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9269 */
  {
    "avx2_psignv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1642 },
#else
    { 0, output_1642, 0 },
#endif
    (insn_gen_fn) gen_avx2_psignv8si3,
    &operand_data[2455],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9269 */
  {
    "ssse3_psignv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1643 },
#else
    { 0, output_1643, 0 },
#endif
    (insn_gen_fn) gen_ssse3_psignv4si3,
    &operand_data[2183],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9286 */
  {
    "ssse3_psignv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psignb\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ssse3_psignv8qi3,
    &operand_data[1809],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9286 */
  {
    "ssse3_psignv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psignw\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ssse3_psignv4hi3,
    &operand_data[1812],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9286 */
  {
    "ssse3_psignv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "psignd\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_ssse3_psignv2si3,
    &operand_data[1815],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9299 */
  {
    "avx2_palignrv2ti",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1647 },
#else
    { 0, 0, output_1647 },
#endif
    (insn_gen_fn) gen_avx2_palignrv2ti,
    &operand_data[2645],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:9299 */
  {
    "ssse3_palignrti",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1648 },
#else
    { 0, 0, output_1648 },
#endif
    (insn_gen_fn) gen_ssse3_palignrti,
    &operand_data[2649],
    4,
    4,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:9328 */
  {
    "ssse3_palignrdi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1649 },
#else
    { 0, 0, output_1649 },
#endif
    (insn_gen_fn) gen_ssse3_palignrdi,
    &operand_data[2653],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:9346 */
  {
    "absv32qi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpabsb\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_absv32qi2,
    &operand_data[2657],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9346 */
  {
    "absv16qi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpabsb\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_absv16qi2,
    &operand_data[2659],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9346 */
  {
    "absv16hi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpabsw\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_absv16hi2,
    &operand_data[2538],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9346 */
  {
    "absv8hi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpabsw\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_absv8hi2,
    &operand_data[2548],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9346 */
  {
    "absv8si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpabsd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_absv8si2,
    &operand_data[2522],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9346 */
  {
    "absv4si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpabsd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_absv4si2,
    &operand_data[2532],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9358 */
  {
    "absv8qi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pabsb\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_absv8qi2,
    &operand_data[2661],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9358 */
  {
    "absv4hi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pabsw\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_absv4hi2,
    &operand_data[1831],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9358 */
  {
    "absv2si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "pabsd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_absv2si2,
    &operand_data[1837],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9376 */
  {
    "sse4a_movntsf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movntss\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4a_movntsf,
    &operand_data[2663],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9376 */
  {
    "sse4a_movntdf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movntsd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4a_movntdf,
    &operand_data[2665],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9386 */
  {
    "sse4a_vmmovntv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movntss\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4a_vmmovntv4sf,
    &operand_data[2667],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9386 */
  {
    "sse4a_vmmovntv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "movntsd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4a_vmmovntv2df,
    &operand_data[2669],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9398 */
  {
    "sse4a_extrqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "extrq\t{%3, %2, %0|%0, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4a_extrqi,
    &operand_data[2671],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9411 */
  {
    "sse4a_extrq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "extrq\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4a_extrq,
    &operand_data[2675],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9422 */
  {
    "sse4a_insertqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "insertq\t{%4, %3, %2, %0|%0, %2, %3, %4}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4a_insertqi,
    &operand_data[2678],
    5,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9437 */
  {
    "sse4a_insertq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "insertq\t{%2, %0|%0, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4a_insertq,
    &operand_data[2678],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9455 */
  {
    "avx_blendps256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1667 },
#else
    { 0, output_1667, 0 },
#endif
    (insn_gen_fn) gen_avx_blendps256,
    &operand_data[2683],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9455 */
  {
    "sse4_1_blendps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1668 },
#else
    { 0, output_1668, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_blendps,
    &operand_data[2687],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9455 */
  {
    "avx_blendpd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1669 },
#else
    { 0, output_1669, 0 },
#endif
    (insn_gen_fn) gen_avx_blendpd256,
    &operand_data[2691],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9455 */
  {
    "sse4_1_blendpd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1670 },
#else
    { 0, output_1670, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_blendpd,
    &operand_data[2695],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9473 */
  {
    "avx_blendvps256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1671 },
#else
    { 0, output_1671, 0 },
#endif
    (insn_gen_fn) gen_avx_blendvps256,
    &operand_data[2699],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9473 */
  {
    "sse4_1_blendvps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1672 },
#else
    { 0, output_1672, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_blendvps,
    &operand_data[2703],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9473 */
  {
    "avx_blendvpd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1673 },
#else
    { 0, output_1673, 0 },
#endif
    (insn_gen_fn) gen_avx_blendvpd256,
    &operand_data[2707],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9473 */
  {
    "sse4_1_blendvpd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1674 },
#else
    { 0, output_1674, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_blendvpd,
    &operand_data[2711],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9492 */
  {
    "avx_dpps256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1675 },
#else
    { 0, output_1675, 0 },
#endif
    (insn_gen_fn) gen_avx_dpps256,
    &operand_data[2715],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9492 */
  {
    "sse4_1_dpps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1676 },
#else
    { 0, output_1676, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_dpps,
    &operand_data[2719],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9492 */
  {
    "avx_dppd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1677 },
#else
    { 0, output_1677, 0 },
#endif
    (insn_gen_fn) gen_avx_dppd256,
    &operand_data[2723],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9492 */
  {
    "sse4_1_dppd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1678 },
#else
    { 0, output_1678, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_dppd,
    &operand_data[2727],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9511 */
  {
    "avx2_movntdqa",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovntdqa\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_movntdqa,
    &operand_data[2731],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9511 */
  {
    "sse4_1_movntdqa",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vmovntdqa\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_movntdqa,
    &operand_data[2733],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9522 */
  {
    "avx2_mpsadbw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1681 },
#else
    { 0, output_1681, 0 },
#endif
    (insn_gen_fn) gen_avx2_mpsadbw,
    &operand_data[2735],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9522 */
  {
    "sse4_1_mpsadbw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1682 },
#else
    { 0, output_1682, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_mpsadbw,
    &operand_data[2739],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9539 */
  {
    "avx2_packusdw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpackusdw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_packusdw,
    &operand_data[2743],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9553 */
  {
    "sse4_1_packusdw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1684 },
#else
    { 0, output_1684, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_packusdw,
    &operand_data[2476],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9570 */
  {
    "avx2_pblendvb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1685 },
#else
    { 0, output_1685, 0 },
#endif
    (insn_gen_fn) gen_avx2_pblendvb,
    &operand_data[2746],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9570 */
  {
    "sse4_1_pblendvb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1686 },
#else
    { 0, output_1686, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_pblendvb,
    &operand_data[2750],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9588 */
  {
    "sse4_1_pblendw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1687 },
#else
    { 0, output_1687, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_pblendw,
    &operand_data[2754],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9618 */
  {
    "*avx2_pblendw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1688 },
#else
    { 0, 0, output_1688 },
#endif
    0,
    &operand_data[2758],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:9635 */
  {
    "avx2_pblenddv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpblendd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pblenddv8si,
    &operand_data[2762],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9635 */
  {
    "avx2_pblenddv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpblendd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pblenddv4si,
    &operand_data[2766],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9649 */
  {
    "sse4_1_phminposuw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vphminposuw\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_phminposuw,
    &operand_data[2548],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9660 */
  {
    "avx2_sign_extendv16qiv16hi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmovsxbw\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_sign_extendv16qiv16hi2,
    &operand_data[2770],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9660 */
  {
    "avx2_zero_extendv16qiv16hi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmovzxbw\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_zero_extendv16qiv16hi2,
    &operand_data[2770],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9671 */
  {
    "sse4_1_sign_extendv8qiv8hi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpmovsxbw\t{%1, %0|%0, %q1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_sign_extendv8qiv8hi2,
    &operand_data[2772],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9671 */
  {
    "sse4_1_zero_extendv8qiv8hi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpmovzxbw\t{%1, %0|%0, %q1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_zero_extendv8qiv8hi2,
    &operand_data[2772],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9691 */
  {
    "avx2_sign_extendv8qiv8si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmovsxbd\t{%1, %0|%0, %q1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_sign_extendv8qiv8si2,
    &operand_data[2774],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9691 */
  {
    "avx2_zero_extendv8qiv8si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmovzxbd\t{%1, %0|%0, %q1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_zero_extendv8qiv8si2,
    &operand_data[2774],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9711 */
  {
    "sse4_1_sign_extendv4qiv4si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpmovsxbd\t{%1, %0|%0, %k1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_sign_extendv4qiv4si2,
    &operand_data[2776],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9711 */
  {
    "sse4_1_zero_extendv4qiv4si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpmovzxbd\t{%1, %0|%0, %k1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_zero_extendv4qiv4si2,
    &operand_data[2776],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9727 */
  {
    "avx2_sign_extendv8hiv8si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmovsxwd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_sign_extendv8hiv8si2,
    &operand_data[2778],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9727 */
  {
    "avx2_zero_extendv8hiv8si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmovzxwd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_zero_extendv8hiv8si2,
    &operand_data[2778],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9738 */
  {
    "sse4_1_sign_extendv4hiv4si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpmovsxwd\t{%1, %0|%0, %q1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_sign_extendv4hiv4si2,
    &operand_data[2780],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9738 */
  {
    "sse4_1_zero_extendv4hiv4si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpmovzxwd\t{%1, %0|%0, %q1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_zero_extendv4hiv4si2,
    &operand_data[2780],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9754 */
  {
    "avx2_sign_extendv4qiv4di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmovsxbq\t{%1, %0|%0, %k1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_sign_extendv4qiv4di2,
    &operand_data[2782],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9754 */
  {
    "avx2_zero_extendv4qiv4di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmovzxbq\t{%1, %0|%0, %k1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_zero_extendv4qiv4di2,
    &operand_data[2782],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9770 */
  {
    "sse4_1_sign_extendv2qiv2di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpmovsxbq\t{%1, %0|%0, %w1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_sign_extendv2qiv2di2,
    &operand_data[2784],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9770 */
  {
    "sse4_1_zero_extendv2qiv2di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpmovzxbq\t{%1, %0|%0, %w1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_zero_extendv2qiv2di2,
    &operand_data[2784],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9784 */
  {
    "avx2_sign_extendv4hiv4di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmovsxwq\t{%1, %0|%0, %q1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_sign_extendv4hiv4di2,
    &operand_data[2786],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9784 */
  {
    "avx2_zero_extendv4hiv4di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmovzxwq\t{%1, %0|%0, %q1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_zero_extendv4hiv4di2,
    &operand_data[2786],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9800 */
  {
    "sse4_1_sign_extendv2hiv2di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpmovsxwq\t{%1, %0|%0, %k1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_sign_extendv2hiv2di2,
    &operand_data[2788],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9800 */
  {
    "sse4_1_zero_extendv2hiv2di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpmovzxwq\t{%1, %0|%0, %k1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_zero_extendv2hiv2di2,
    &operand_data[2788],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9814 */
  {
    "avx2_sign_extendv4siv4di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmovsxdq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_sign_extendv4siv4di2,
    &operand_data[2790],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9814 */
  {
    "avx2_zero_extendv4siv4di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmovzxdq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_zero_extendv4siv4di2,
    &operand_data[2790],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9824 */
  {
    "sse4_1_sign_extendv2siv2di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpmovsxdq\t{%1, %0|%0, %q1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_sign_extendv2siv2di2,
    &operand_data[2792],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9824 */
  {
    "sse4_1_zero_extendv2siv2di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpmovzxdq\t{%1, %0|%0, %q1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_zero_extendv2siv2di2,
    &operand_data[2792],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9840 */
  {
    "avx_vtestps256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vtestps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_vtestps256,
    &operand_data[1997],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9840 */
  {
    "avx_vtestps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vtestps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_vtestps,
    &operand_data[2001],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9840 */
  {
    "avx_vtestpd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vtestpd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_vtestpd256,
    &operand_data[1994],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9840 */
  {
    "avx_vtestpd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vtestpd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_vtestpd,
    &operand_data[2009],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9854 */
  {
    "avx_ptest256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vptest\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_ptest256,
    &operand_data[2286],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9866 */
  {
    "sse4_1_ptest",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vptest\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_ptest,
    &operand_data[2794],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9878 */
  {
    "avx_roundps256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vroundps\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_roundps256,
    &operand_data[2796],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9878 */
  {
    "sse4_1_roundps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vroundps\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_roundps,
    &operand_data[2799],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9878 */
  {
    "avx_roundpd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vroundpd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_roundpd256,
    &operand_data[2802],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9878 */
  {
    "sse4_1_roundpd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vroundpd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_roundpd,
    &operand_data[2805],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:9951 */
  {
    "sse4_1_roundss",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1726 },
#else
    { 0, output_1726, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_roundss,
    &operand_data[2808],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:9951 */
  {
    "sse4_1_roundsd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1727 },
#else
    { 0, output_1727, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_roundsd,
    &operand_data[2812],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:10060 */
  {
    "sse4_2_pcmpestr",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_2_pcmpestr,
    &operand_data[2816],
    7,
    7,
    10,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10120 */
  {
    "sse4_2_pcmpestri",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpcmpestri\t{%5, %3, %1|%1, %3, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_2_pcmpestri,
    &operand_data[2823],
    6,
    6,
    5,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10147 */
  {
    "sse4_2_pcmpestrm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpcmpestrm\t{%5, %3, %1|%1, %3, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_2_pcmpestrm,
    &operand_data[2829],
    6,
    6,
    5,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10174 */
  {
    "sse4_2_pcmpestr_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1731 },
#else
    { 0, output_1731, 0 },
#endif
    (insn_gen_fn) gen_sse4_2_pcmpestr_cconly,
    &operand_data[2835],
    7,
    7,
    0,
    4,
    2
  },
  /* ../../gcc/config/i386/sse.md:10199 */
  {
    "sse4_2_pcmpistr",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_2_pcmpistr,
    &operand_data[2842],
    5,
    5,
    6,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10250 */
  {
    "sse4_2_pcmpistri",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpcmpistri\t{%3, %2, %1|%1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_2_pcmpistri,
    &operand_data[2847],
    4,
    4,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10273 */
  {
    "sse4_2_pcmpistrm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vpcmpistrm\t{%3, %2, %1|%1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_sse4_2_pcmpistrm,
    &operand_data[2851],
    4,
    4,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10296 */
  {
    "sse4_2_pcmpistr_cconly",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1735 },
#else
    { 0, output_1735, 0 },
#endif
    (insn_gen_fn) gen_sse4_2_pcmpistr_cconly,
    &operand_data[2855],
    5,
    5,
    0,
    4,
    2
  },
  /* ../../gcc/config/i386/sse.md:10329 */
  {
    "xop_pmacsww",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmacsww\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pmacsww,
    &operand_data[2860],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10341 */
  {
    "xop_pmacssww",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmacssww\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pmacssww,
    &operand_data[2860],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10352 */
  {
    "xop_pmacsdd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmacsdd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pmacsdd,
    &operand_data[2864],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10364 */
  {
    "xop_pmacssdd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmacssdd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pmacssdd,
    &operand_data[2864],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10375 */
  {
    "xop_pmacssdql",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmacssdql\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pmacssdql,
    &operand_data[2868],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10394 */
  {
    "xop_pmacssdqh",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmacssdqh\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pmacssdqh,
    &operand_data[2868],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10414 */
  {
    "xop_pmacsdql",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmacsdql\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pmacsdql,
    &operand_data[2868],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10434 */
  {
    "xop_pmacsdqh",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmacsdqh\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pmacsdqh,
    &operand_data[2868],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10455 */
  {
    "xop_pmacsswd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmacsswd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pmacsswd,
    &operand_data[2872],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10479 */
  {
    "xop_pmacswd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmacswd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pmacswd,
    &operand_data[2872],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10503 */
  {
    "xop_pmadcsswd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmadcsswd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pmadcsswd,
    &operand_data[2872],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10543 */
  {
    "xop_pmadcswd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmadcswd\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pmadcswd,
    &operand_data[2872],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10584 */
  {
    "xop_pcmov_v32qi256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pcmov_v32qi256,
    &operand_data[2876],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10584 */
  {
    "xop_pcmov_v16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pcmov_v16qi,
    &operand_data[2880],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10584 */
  {
    "xop_pcmov_v16hi256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pcmov_v16hi256,
    &operand_data[2884],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10584 */
  {
    "xop_pcmov_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pcmov_v8hi,
    &operand_data[2888],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10584 */
  {
    "xop_pcmov_v8si256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pcmov_v8si256,
    &operand_data[2892],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10584 */
  {
    "xop_pcmov_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pcmov_v4si,
    &operand_data[2896],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10584 */
  {
    "xop_pcmov_v4di256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pcmov_v4di256,
    &operand_data[2900],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10584 */
  {
    "xop_pcmov_v2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pcmov_v2di,
    &operand_data[2904],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10584 */
  {
    "xop_pcmov_v8sf256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pcmov_v8sf256,
    &operand_data[2908],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10584 */
  {
    "xop_pcmov_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pcmov_v4sf,
    &operand_data[2912],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10584 */
  {
    "xop_pcmov_v4df256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pcmov_v4df256,
    &operand_data[2916],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10584 */
  {
    "xop_pcmov_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcmov\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pcmov_v2df,
    &operand_data[2920],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:10595 */
  {
    "xop_phaddbw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphaddbw\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phaddbw,
    &operand_data[2772],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10624 */
  {
    "xop_phaddbd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphaddbd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phaddbd,
    &operand_data[2776],
    2,
    2,
    3,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10661 */
  {
    "xop_phaddbq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphaddbq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phaddbq,
    &operand_data[2784],
    2,
    2,
    7,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10714 */
  {
    "xop_phaddwd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphaddwd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phaddwd,
    &operand_data[2780],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10735 */
  {
    "xop_phaddwq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphaddwq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phaddwq,
    &operand_data[2788],
    2,
    2,
    3,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10764 */
  {
    "xop_phadddq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphadddq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phadddq,
    &operand_data[2792],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10781 */
  {
    "xop_phaddubw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphaddubw\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phaddubw,
    &operand_data[2772],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10810 */
  {
    "xop_phaddubd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphaddubd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phaddubd,
    &operand_data[2776],
    2,
    2,
    3,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10847 */
  {
    "xop_phaddubq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphaddubq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phaddubq,
    &operand_data[2784],
    2,
    2,
    7,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10900 */
  {
    "xop_phadduwd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphadduwd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phadduwd,
    &operand_data[2780],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10921 */
  {
    "xop_phadduwq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphadduwq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phadduwq,
    &operand_data[2788],
    2,
    2,
    3,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10950 */
  {
    "xop_phaddudq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphaddudq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phaddudq,
    &operand_data[2792],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10967 */
  {
    "xop_phsubbw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphsubbw\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phsubbw,
    &operand_data[2772],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:10996 */
  {
    "xop_phsubwd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphsubwd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phsubwd,
    &operand_data[2780],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11017 */
  {
    "xop_phsubdq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vphsubdq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_phsubdq,
    &operand_data[2792],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11035 */
  {
    "xop_pperm",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpperm\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pperm,
    &operand_data[2924],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11048 */
  {
    "xop_pperm_pack_v2di_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpperm\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pperm_pack_v2di_v4si,
    &operand_data[2928],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11061 */
  {
    "xop_pperm_pack_v4si_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpperm\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pperm_pack_v4si_v8hi,
    &operand_data[2932],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11074 */
  {
    "xop_pperm_pack_v8hi_v16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpperm\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_pperm_pack_v8hi_v16qi,
    &operand_data[2936],
    4,
    4,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11152 */
  {
    "xop_rotlv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vprotb\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_rotlv16qi3,
    &operand_data[2940],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11152 */
  {
    "xop_rotlv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vprotw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_rotlv8hi3,
    &operand_data[2943],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11152 */
  {
    "xop_rotlv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vprotd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_rotlv4si3,
    &operand_data[2946],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11152 */
  {
    "xop_rotlv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vprotq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_rotlv2di3,
    &operand_data[2949],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11163 */
  {
    "xop_rotrv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1783 },
#else
    { 0, 0, output_1783 },
#endif
    (insn_gen_fn) gen_xop_rotrv16qi3,
    &operand_data[2940],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:11163 */
  {
    "xop_rotrv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1784 },
#else
    { 0, 0, output_1784 },
#endif
    (insn_gen_fn) gen_xop_rotrv8hi3,
    &operand_data[2943],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:11163 */
  {
    "xop_rotrv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1785 },
#else
    { 0, 0, output_1785 },
#endif
    (insn_gen_fn) gen_xop_rotrv4si3,
    &operand_data[2946],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:11163 */
  {
    "xop_rotrv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1786 },
#else
    { 0, 0, output_1786 },
#endif
    (insn_gen_fn) gen_xop_rotrv2di3,
    &operand_data[2949],
    3,
    3,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:11200 */
  {
    "xop_vrotlv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vprotb\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_vrotlv16qi3,
    &operand_data[2952],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11200 */
  {
    "xop_vrotlv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vprotw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_vrotlv8hi3,
    &operand_data[2955],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11200 */
  {
    "xop_vrotlv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vprotd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_vrotlv4si3,
    &operand_data[2958],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11200 */
  {
    "xop_vrotlv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vprotq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_vrotlv2di3,
    &operand_data[2961],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11323 */
  {
    "xop_shav16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpshab\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_shav16qi3,
    &operand_data[2952],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11323 */
  {
    "xop_shav8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpshaw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_shav8hi3,
    &operand_data[2955],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11323 */
  {
    "xop_shav4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpshad\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_shav4si3,
    &operand_data[2958],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11323 */
  {
    "xop_shav2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpshaq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_shav2di3,
    &operand_data[2961],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11342 */
  {
    "xop_shlv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpshlb\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_shlv16qi3,
    &operand_data[2952],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11342 */
  {
    "xop_shlv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpshlw\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_shlv8hi3,
    &operand_data[2955],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11342 */
  {
    "xop_shlv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpshld\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_shlv4si3,
    &operand_data[2958],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11342 */
  {
    "xop_shlv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpshlq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_shlv2di3,
    &operand_data[2961],
    3,
    3,
    3,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:11449 */
  {
    "xop_frczsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfrczss\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_frczsf2,
    &operand_data[1276],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11449 */
  {
    "xop_frczdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfrczsd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_frczdf2,
    &operand_data[1386],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11449 */
  {
    "xop_frczv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfrczps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_frczv4sf2,
    &operand_data[1981],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11449 */
  {
    "xop_frczv2df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfrczpd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_frczv2df2,
    &operand_data[1988],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11449 */
  {
    "xop_frczv8sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfrczps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_frczv8sf2,
    &operand_data[1979],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11449 */
  {
    "xop_frczv4df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfrczpd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_frczv4df2,
    &operand_data[1986],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11470 */
  {
    "*xop_vmfrczv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfrczss\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2964],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11470 */
  {
    "*xop_vmfrczv2df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vfrczsd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2967],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11483 */
  {
    "xop_maskcmpv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcom%Y1b\t{%3, %2, %0|%0, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_maskcmpv16qi3,
    &operand_data[2970],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11483 */
  {
    "xop_maskcmpv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcom%Y1w\t{%3, %2, %0|%0, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_maskcmpv8hi3,
    &operand_data[2974],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11483 */
  {
    "xop_maskcmpv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcom%Y1d\t{%3, %2, %0|%0, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_maskcmpv4si3,
    &operand_data[2978],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11483 */
  {
    "xop_maskcmpv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcom%Y1q\t{%3, %2, %0|%0, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_maskcmpv2di3,
    &operand_data[2982],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11497 */
  {
    "xop_maskcmp_unsv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcom%Y1ub\t{%3, %2, %0|%0, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_maskcmp_unsv16qi3,
    &operand_data[2986],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11497 */
  {
    "xop_maskcmp_unsv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcom%Y1uw\t{%3, %2, %0|%0, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_maskcmp_unsv8hi3,
    &operand_data[2990],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11497 */
  {
    "xop_maskcmp_unsv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcom%Y1ud\t{%3, %2, %0|%0, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_maskcmp_unsv4si3,
    &operand_data[2994],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11497 */
  {
    "xop_maskcmp_unsv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcom%Y1uq\t{%3, %2, %0|%0, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_maskcmp_unsv2di3,
    &operand_data[2998],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11514 */
  {
    "xop_maskcmp_uns2v16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcom%Y1ub\t{%3, %2, %0|%0, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_maskcmp_uns2v16qi3,
    &operand_data[2986],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11514 */
  {
    "xop_maskcmp_uns2v8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcom%Y1uw\t{%3, %2, %0|%0, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_maskcmp_uns2v8hi3,
    &operand_data[2990],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11514 */
  {
    "xop_maskcmp_uns2v4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcom%Y1ud\t{%3, %2, %0|%0, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_maskcmp_uns2v4si3,
    &operand_data[2994],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11514 */
  {
    "xop_maskcmp_uns2v2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpcom%Y1uq\t{%3, %2, %0|%0, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_maskcmp_uns2v2di3,
    &operand_data[2998],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11531 */
  {
    "xop_pcom_tfv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1819 },
#else
    { 0, 0, output_1819 },
#endif
    (insn_gen_fn) gen_xop_pcom_tfv16qi3,
    &operand_data[3002],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:11531 */
  {
    "xop_pcom_tfv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1820 },
#else
    { 0, 0, output_1820 },
#endif
    (insn_gen_fn) gen_xop_pcom_tfv8hi3,
    &operand_data[3006],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:11531 */
  {
    "xop_pcom_tfv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1821 },
#else
    { 0, 0, output_1821 },
#endif
    (insn_gen_fn) gen_xop_pcom_tfv4si3,
    &operand_data[3010],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:11531 */
  {
    "xop_pcom_tfv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1822 },
#else
    { 0, 0, output_1822 },
#endif
    (insn_gen_fn) gen_xop_pcom_tfv2di3,
    &operand_data[3014],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:11550 */
  {
    "xop_vpermil2v8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpermil2ps\t{%4, %3, %2, %1, %0|%0, %1, %2, %3, %4}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_vpermil2v8sf3,
    &operand_data[3018],
    5,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11550 */
  {
    "xop_vpermil2v4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpermil2ps\t{%4, %3, %2, %1, %0|%0, %1, %2, %3, %4}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_vpermil2v4sf3,
    &operand_data[3023],
    5,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11550 */
  {
    "xop_vpermil2v4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpermil2pd\t{%4, %3, %2, %1, %0|%0, %1, %2, %3, %4}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_vpermil2v4df3,
    &operand_data[3028],
    5,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11550 */
  {
    "xop_vpermil2v2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpermil2pd\t{%4, %3, %2, %1, %0|%0, %1, %2, %3, %4}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_xop_vpermil2v2df3,
    &operand_data[3033],
    5,
    5,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11566 */
  {
    "aesenc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1827 },
#else
    { 0, output_1827, 0 },
#endif
    (insn_gen_fn) gen_aesenc,
    &operand_data[2288],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:11581 */
  {
    "aesenclast",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1828 },
#else
    { 0, output_1828, 0 },
#endif
    (insn_gen_fn) gen_aesenclast,
    &operand_data[2288],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:11596 */
  {
    "aesdec",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1829 },
#else
    { 0, output_1829, 0 },
#endif
    (insn_gen_fn) gen_aesdec,
    &operand_data[2288],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:11611 */
  {
    "aesdeclast",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1830 },
#else
    { 0, output_1830, 0 },
#endif
    (insn_gen_fn) gen_aesdeclast,
    &operand_data[2288],
    3,
    3,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:11626 */
  {
    "aesimc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vaesimc\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_aesimc,
    &operand_data[1890],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11637 */
  {
    "aeskeygenassist",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "%vaeskeygenassist\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_aeskeygenassist,
    &operand_data[3038],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11650 */
  {
    "pclmulqdq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1833 },
#else
    { 0, output_1833, 0 },
#endif
    (insn_gen_fn) gen_pclmulqdq,
    &operand_data[3041],
    4,
    4,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:11687 */
  {
    "*avx_vzeroall",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vzeroall",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3045],
    1,
    1,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/sse.md:11700 */
  {
    "avx_vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vzeroupper",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_vzeroupper,
    &operand_data[1116],
    1,
    1,
    0,
    0,
    1
  },
  /* ../../gcc/config/i386/sse.md:11717 */
  {
    "avx2_pbroadcastv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpbroadcastb\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pbroadcastv32qi,
    &operand_data[3046],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11717 */
  {
    "avx2_pbroadcastv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpbroadcastb\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pbroadcastv16qi,
    &operand_data[2659],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11717 */
  {
    "avx2_pbroadcastv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpbroadcastw\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pbroadcastv16hi,
    &operand_data[3048],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11717 */
  {
    "avx2_pbroadcastv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpbroadcastw\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pbroadcastv8hi,
    &operand_data[2548],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11717 */
  {
    "avx2_pbroadcastv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpbroadcastd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pbroadcastv8si,
    &operand_data[3050],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11717 */
  {
    "avx2_pbroadcastv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpbroadcastd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pbroadcastv4si,
    &operand_data[2532],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11717 */
  {
    "avx2_pbroadcastv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpbroadcastq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pbroadcastv4di,
    &operand_data[3052],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11717 */
  {
    "avx2_pbroadcastv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpbroadcastq\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pbroadcastv2di,
    &operand_data[1890],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11730 */
  {
    "avx2_permvarv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpermd\t{%1, %2, %0|%0, %2, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_permvarv8si,
    &operand_data[3054],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11742 */
  {
    "avx2_permv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpermpd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_permv4df,
    &operand_data[3057],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11755 */
  {
    "avx2_permvarv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpermps\t{%1, %2, %0|%0, %2, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_permvarv8sf,
    &operand_data[3060],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11782 */
  {
    "avx2_permv4di_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1847 },
#else
    { 0, 0, output_1847 },
#endif
    (insn_gen_fn) gen_avx2_permv4di_1,
    &operand_data[3063],
    6,
    6,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:11804 */
  {
    "avx2_permv2ti",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vperm2i128\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_permv2ti,
    &operand_data[3069],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11817 */
  {
    "avx2_vec_dupv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vbroadcastsd\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_vec_dupv4df,
    &operand_data[3073],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11833 */
  {
    "vec_dupv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1850 },
#else
    { 0, output_1850, 0 },
#endif
    (insn_gen_fn) gen_vec_dupv8si,
    &operand_data[3075],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:11833 */
  {
    "vec_dupv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1851 },
#else
    { 0, output_1851, 0 },
#endif
    (insn_gen_fn) gen_vec_dupv8sf,
    &operand_data[3077],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:11833 */
  {
    "vec_dupv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1852 },
#else
    { 0, output_1852, 0 },
#endif
    (insn_gen_fn) gen_vec_dupv4di,
    &operand_data[3079],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:11833 */
  {
    "vec_dupv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1853 },
#else
    { 0, output_1853, 0 },
#endif
    (insn_gen_fn) gen_vec_dupv4df,
    &operand_data[3081],
    2,
    2,
    0,
    2,
    2
  },
  /* ../../gcc/config/i386/sse.md:11846 */
  {
    "avx2_vbroadcasti128_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vbroadcasti128\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_vbroadcasti128_v32qi,
    &operand_data[3083],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11846 */
  {
    "avx2_vbroadcasti128_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vbroadcasti128\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_vbroadcasti128_v16hi,
    &operand_data[3085],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11846 */
  {
    "avx2_vbroadcasti128_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vbroadcasti128\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_vbroadcasti128_v8si,
    &operand_data[3087],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11846 */
  {
    "avx2_vbroadcasti128_v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vbroadcasti128\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_vbroadcasti128_v4di,
    &operand_data[3089],
    2,
    2,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:11869 */
  {
    "avx_vbroadcastf128_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1858 },
#else
    { 0, output_1858, 0 },
#endif
    (insn_gen_fn) gen_avx_vbroadcastf128_v32qi,
    &operand_data[3091],
    2,
    2,
    1,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:11869 */
  {
    "avx_vbroadcastf128_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1859 },
#else
    { 0, output_1859, 0 },
#endif
    (insn_gen_fn) gen_avx_vbroadcastf128_v16hi,
    &operand_data[3093],
    2,
    2,
    1,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:11869 */
  {
    "avx_vbroadcastf128_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1860 },
#else
    { 0, output_1860, 0 },
#endif
    (insn_gen_fn) gen_avx_vbroadcastf128_v8si,
    &operand_data[3095],
    2,
    2,
    1,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:11869 */
  {
    "avx_vbroadcastf128_v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1861 },
#else
    { 0, output_1861, 0 },
#endif
    (insn_gen_fn) gen_avx_vbroadcastf128_v4di,
    &operand_data[3097],
    2,
    2,
    1,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:11869 */
  {
    "avx_vbroadcastf128_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1862 },
#else
    { 0, output_1862, 0 },
#endif
    (insn_gen_fn) gen_avx_vbroadcastf128_v8sf,
    &operand_data[3099],
    2,
    2,
    1,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:11869 */
  {
    "avx_vbroadcastf128_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .multi = output_1863 },
#else
    { 0, output_1863, 0 },
#endif
    (insn_gen_fn) gen_avx_vbroadcastf128_v4df,
    &operand_data[3101],
    2,
    2,
    1,
    3,
    2
  },
  /* ../../gcc/config/i386/sse.md:11888 */
  {
    "*avx_vperm_broadcast_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1864 },
#else
    { 0, 0, output_1864 },
#endif
    0,
    &operand_data[3103],
    4,
    4,
    0,
    3,
    3
  },
  /* ../../gcc/config/i386/sse.md:11916 */
  {
    "*avx_vperm_broadcast_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3107],
    4,
    4,
    0,
    3,
    1
  },
  /* ../../gcc/config/i386/sse.md:11916 */
  {
    "*avx_vperm_broadcast_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3111],
    4,
    4,
    0,
    3,
    1
  },
  /* ../../gcc/config/i386/sse.md:12000 */
  {
    "*avx_vpermilpv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1867 },
#else
    { 0, 0, output_1867 },
#endif
    0,
    &operand_data[3115],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12000 */
  {
    "*avx_vpermilpv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1868 },
#else
    { 0, 0, output_1868 },
#endif
    0,
    &operand_data[3119],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12000 */
  {
    "*avx_vpermilpv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1869 },
#else
    { 0, 0, output_1869 },
#endif
    0,
    &operand_data[3123],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12000 */
  {
    "*avx_vpermilpv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1870 },
#else
    { 0, 0, output_1870 },
#endif
    0,
    &operand_data[3127],
    4,
    4,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12019 */
  {
    "avx_vpermilvarv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpermilps\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_vpermilvarv8sf3,
    &operand_data[3131],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12019 */
  {
    "avx_vpermilvarv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpermilps\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_vpermilvarv4sf3,
    &operand_data[3134],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12019 */
  {
    "avx_vpermilvarv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpermilpd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_vpermilvarv4df3,
    &operand_data[3137],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12019 */
  {
    "avx_vpermilvarv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpermilpd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_vpermilvarv2df3,
    &operand_data[3140],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12069 */
  {
    "*avx_vperm2f128v8si_full",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vperm2%~128\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[2762],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12069 */
  {
    "*avx_vperm2f128v8sf_full",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vperm2f128\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3143],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12069 */
  {
    "*avx_vperm2f128v4df_full",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vperm2f128\t{%3, %2, %1, %0|%0, %1, %2, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3147],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12084 */
  {
    "*avx_vperm2f128v8si_nozero",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1878 },
#else
    { 0, 0, output_1878 },
#endif
    0,
    &operand_data[3151],
    5,
    5,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12084 */
  {
    "*avx_vperm2f128v8sf_nozero",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1879 },
#else
    { 0, 0, output_1879 },
#endif
    0,
    &operand_data[3156],
    5,
    5,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12084 */
  {
    "*avx_vperm2f128v4df_nozero",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1880 },
#else
    { 0, 0, output_1880 },
#endif
    0,
    &operand_data[3161],
    5,
    5,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12134 */
  {
    "avx2_vec_set_lo_v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinserti128\t{$0x0, %2, %1, %0|%0, %1, %2, 0x0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_vec_set_lo_v4di,
    &operand_data[3166],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12149 */
  {
    "avx2_vec_set_hi_v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinserti128\t{$0x1, %2, %1, %0|%0, %1, %2, 0x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_vec_set_hi_v4di,
    &operand_data[3166],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12164 */
  {
    "vec_set_lo_v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinsert%~128\t{$0x0, %2, %1, %0|%0, %1, %2, 0x0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_set_lo_v4di,
    &operand_data[3166],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12164 */
  {
    "vec_set_lo_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinsertf128\t{$0x0, %2, %1, %0|%0, %1, %2, 0x0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_set_lo_v4df,
    &operand_data[3169],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12179 */
  {
    "vec_set_hi_v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinsert%~128\t{$0x1, %2, %1, %0|%0, %1, %2, 0x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_set_hi_v4di,
    &operand_data[3166],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12179 */
  {
    "vec_set_hi_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinsertf128\t{$0x1, %2, %1, %0|%0, %1, %2, 0x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_set_hi_v4df,
    &operand_data[3169],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12194 */
  {
    "vec_set_lo_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinsert%~128\t{$0x0, %2, %1, %0|%0, %1, %2, 0x0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_set_lo_v8si,
    &operand_data[3172],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12194 */
  {
    "vec_set_lo_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinsertf128\t{$0x0, %2, %1, %0|%0, %1, %2, 0x0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_set_lo_v8sf,
    &operand_data[3175],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12210 */
  {
    "vec_set_hi_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinsert%~128\t{$0x1, %2, %1, %0|%0, %1, %2, 0x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_set_hi_v8si,
    &operand_data[3172],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12210 */
  {
    "vec_set_hi_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinsertf128\t{$0x1, %2, %1, %0|%0, %1, %2, 0x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_set_hi_v8sf,
    &operand_data[3175],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12226 */
  {
    "vec_set_lo_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinsert%~128\t{$0x0, %2, %1, %0|%0, %1, %2, 0x0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_set_lo_v16hi,
    &operand_data[3178],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12244 */
  {
    "vec_set_hi_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinsert%~128\t{$0x1, %2, %1, %0|%0, %1, %2, 0x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_set_hi_v16hi,
    &operand_data[3178],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12262 */
  {
    "vec_set_lo_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinsert%~128\t{$0x0, %2, %1, %0|%0, %1, %2, 0x0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_set_lo_v32qi,
    &operand_data[3181],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12284 */
  {
    "vec_set_hi_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vinsert%~128\t{$0x1, %2, %1, %0|%0, %1, %2, 0x1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vec_set_hi_v32qi,
    &operand_data[3181],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12306 */
  {
    "avx_maskloadps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vmaskmovps\t{%1, %2, %0|%0, %2, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_maskloadps,
    &operand_data[3184],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12306 */
  {
    "avx_maskloadpd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vmaskmovpd\t{%1, %2, %0|%0, %2, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_maskloadpd,
    &operand_data[3187],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12306 */
  {
    "avx_maskloadps256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vmaskmovps\t{%1, %2, %0|%0, %2, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_maskloadps256,
    &operand_data[3190],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12306 */
  {
    "avx_maskloadpd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vmaskmovpd\t{%1, %2, %0|%0, %2, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_maskloadpd256,
    &operand_data[3193],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12306 */
  {
    "avx2_maskloadd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaskmovd\t{%1, %2, %0|%0, %2, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_maskloadd,
    &operand_data[3196],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12306 */
  {
    "avx2_maskloadq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaskmovq\t{%1, %2, %0|%0, %2, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_maskloadq,
    &operand_data[3199],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12306 */
  {
    "avx2_maskloadd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaskmovd\t{%1, %2, %0|%0, %2, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_maskloadd256,
    &operand_data[3202],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12306 */
  {
    "avx2_maskloadq256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaskmovq\t{%1, %2, %0|%0, %2, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_maskloadq256,
    &operand_data[3205],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12319 */
  {
    "avx_maskstoreps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vmaskmovps\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_maskstoreps,
    &operand_data[3208],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12319 */
  {
    "avx_maskstorepd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vmaskmovpd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_maskstorepd,
    &operand_data[3211],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12319 */
  {
    "avx_maskstoreps256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vmaskmovps\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_maskstoreps256,
    &operand_data[3214],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12319 */
  {
    "avx_maskstorepd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vmaskmovpd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_maskstorepd256,
    &operand_data[3217],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12319 */
  {
    "avx2_maskstored",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaskmovd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_maskstored,
    &operand_data[3220],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12319 */
  {
    "avx2_maskstoreq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaskmovq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_maskstoreq,
    &operand_data[3223],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12319 */
  {
    "avx2_maskstored256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaskmovd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_maskstored256,
    &operand_data[3226],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12319 */
  {
    "avx2_maskstoreq256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpmaskmovq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_maskstoreq256,
    &operand_data[3229],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12333 */
  {
    "avx_si256_si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_si256_si,
    &operand_data[3232],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:12333 */
  {
    "avx_ps256_ps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_ps256_ps,
    &operand_data[3234],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:12333 */
  {
    "avx_pd256_pd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx_pd256_pd,
    &operand_data[3236],
    2,
    2,
    0,
    2,
    1
  },
  /* ../../gcc/config/i386/sse.md:12411 */
  {
    "avx2_ashrvv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpsravd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_ashrvv8si,
    &operand_data[2440],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12411 */
  {
    "avx2_ashrvv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpsravd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_ashrvv4si,
    &operand_data[2766],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12422 */
  {
    "avx2_ashlvv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpsllvd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_ashlvv8si,
    &operand_data[2440],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12422 */
  {
    "avx2_lshrvv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpsrlvd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_lshrvv8si,
    &operand_data[2440],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12422 */
  {
    "avx2_ashlvv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpsllvd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_ashlvv4si,
    &operand_data[2766],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12422 */
  {
    "avx2_lshrvv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpsrlvd\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_lshrvv4si,
    &operand_data[2766],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12422 */
  {
    "avx2_ashlvv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpsllvq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_ashlvv4di,
    &operand_data[2285],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12422 */
  {
    "avx2_lshrvv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpsrlvq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_lshrvv4di,
    &operand_data[2285],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12422 */
  {
    "avx2_ashlvv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpsllvq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_ashlvv2di,
    &operand_data[3014],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12422 */
  {
    "avx2_lshrvv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpsrlvq\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_avx2_lshrvv2di,
    &operand_data[3014],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12433 */
  {
    "avx_vec_concatv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1924 },
#else
    { 0, 0, output_1924 },
#endif
    (insn_gen_fn) gen_avx_vec_concatv32qi,
    &operand_data[3238],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:12433 */
  {
    "avx_vec_concatv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1925 },
#else
    { 0, 0, output_1925 },
#endif
    (insn_gen_fn) gen_avx_vec_concatv16hi,
    &operand_data[3241],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:12433 */
  {
    "avx_vec_concatv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1926 },
#else
    { 0, 0, output_1926 },
#endif
    (insn_gen_fn) gen_avx_vec_concatv8si,
    &operand_data[3244],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:12433 */
  {
    "avx_vec_concatv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1927 },
#else
    { 0, 0, output_1927 },
#endif
    (insn_gen_fn) gen_avx_vec_concatv4di,
    &operand_data[3247],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:12433 */
  {
    "avx_vec_concatv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1928 },
#else
    { 0, 0, output_1928 },
#endif
    (insn_gen_fn) gen_avx_vec_concatv8sf,
    &operand_data[3250],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:12433 */
  {
    "avx_vec_concatv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1929 },
#else
    { 0, 0, output_1929 },
#endif
    (insn_gen_fn) gen_avx_vec_concatv4df,
    &operand_data[3253],
    3,
    3,
    0,
    2,
    3
  },
  /* ../../gcc/config/i386/sse.md:12464 */
  {
    "vcvtph2ps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtph2ps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vcvtph2ps,
    &operand_data[3256],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12477 */
  {
    "*vcvtph2ps_load",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtph2ps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3258],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12487 */
  {
    "vcvtph2ps256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtph2ps\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vcvtph2ps256,
    &operand_data[3260],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12507 */
  {
    "*vcvtps2ph",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtps2ph\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3262],
    4,
    4,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12520 */
  {
    "*vcvtps2ph_store",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtps2ph\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3266],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12531 */
  {
    "vcvtps2ph256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vcvtps2ph\t{%2, %1, %0|%0, %1, %2}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_vcvtps2ph256,
    &operand_data[3269],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdq\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3272],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdq\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3280],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdpd\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3288],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdpd\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3296],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdq\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3304],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdq\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3312],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdpd\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3320],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdpd\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3328],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdd\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3336],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdd\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3344],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdps\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3352],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdps\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3360],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdd\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3368],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdd\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3376],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdps\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3384],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12582 */
  {
    "*avx2_gathersiv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdps\t{%1, %7, %0|%0, %7, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3392],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv2di_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdq\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3400],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv2di_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdq\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3407],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv2df_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdpd\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3414],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv2df_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdpd\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3421],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv4di_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdq\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3428],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv4di_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdq\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3435],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv4df_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdpd\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3442],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv4df_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdpd\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3449],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv4si_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdd\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3456],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv4si_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdd\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3463],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv4sf_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdps\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3470],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv4sf_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdps\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3477],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv8si_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdd\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3484],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv8si_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherdd\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3491],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv8sf_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdps\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3498],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12602 */
  {
    "*avx2_gathersiv8sf_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherdps\t{%1, %6, %0|%0, %6, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3505],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherqq\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3512],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherqq\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3520],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherqpd\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3528],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherqpd\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3536],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherqq\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3544],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherqq\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3552],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherqpd\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3560],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherqpd\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3568],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherqd\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3576],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherqd\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3584],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherqps\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3592],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherqps\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3600],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherqd\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3608],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherqd\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3616],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherqps\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3624],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12644 */
  {
    "*avx2_gatherdiv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherqps\t{%5, %7, %2|%2, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3632],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv2di_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1984 },
#else
    { 0, 0, output_1984 },
#endif
    0,
    &operand_data[3640],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv2di_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1985 },
#else
    { 0, 0, output_1985 },
#endif
    0,
    &operand_data[3647],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv2df_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1986 },
#else
    { 0, 0, output_1986 },
#endif
    0,
    &operand_data[3654],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv2df_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1987 },
#else
    { 0, 0, output_1987 },
#endif
    0,
    &operand_data[3661],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv4di_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1988 },
#else
    { 0, 0, output_1988 },
#endif
    0,
    &operand_data[3668],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv4di_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1989 },
#else
    { 0, 0, output_1989 },
#endif
    0,
    &operand_data[3675],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv4df_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1990 },
#else
    { 0, 0, output_1990 },
#endif
    0,
    &operand_data[3682],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv4df_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1991 },
#else
    { 0, 0, output_1991 },
#endif
    0,
    &operand_data[3689],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv4si_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1992 },
#else
    { 0, 0, output_1992 },
#endif
    0,
    &operand_data[3696],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv4si_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1993 },
#else
    { 0, 0, output_1993 },
#endif
    0,
    &operand_data[3703],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv4sf_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1994 },
#else
    { 0, 0, output_1994 },
#endif
    0,
    &operand_data[3710],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv4sf_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1995 },
#else
    { 0, 0, output_1995 },
#endif
    0,
    &operand_data[3717],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv8si_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1996 },
#else
    { 0, 0, output_1996 },
#endif
    0,
    &operand_data[3724],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv8si_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1997 },
#else
    { 0, 0, output_1997 },
#endif
    0,
    &operand_data[3731],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv8sf_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1998 },
#else
    { 0, 0, output_1998 },
#endif
    0,
    &operand_data[3738],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12664 */
  {
    "*avx2_gatherdiv8sf_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_1999 },
#else
    { 0, 0, output_1999 },
#endif
    0,
    &operand_data[3745],
    7,
    7,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sse.md:12688 */
  {
    "*avx2_gatherdiv8si_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherqd\t{%5, %7, %0|%0, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3752],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12688 */
  {
    "*avx2_gatherdiv8si_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherqd\t{%5, %7, %0|%0, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3760],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12688 */
  {
    "*avx2_gatherdiv8sf_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherqps\t{%5, %7, %0|%0, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3768],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12688 */
  {
    "*avx2_gatherdiv8sf_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherqps\t{%5, %7, %0|%0, %7, %5}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3776],
    8,
    8,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12711 */
  {
    "*avx2_gatherdiv8si_4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherqd\t{%4, %6, %0|%0, %6, %4}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3784],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12711 */
  {
    "*avx2_gatherdiv8si_4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vpgatherqd\t{%4, %6, %0|%0, %6, %4}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3791],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12711 */
  {
    "*avx2_gatherdiv8sf_4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherqps\t{%4, %6, %0|%0, %6, %4}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3798],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sse.md:12711 */
  {
    "*avx2_gatherdiv8sf_4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "vgatherqps\t{%4, %6, %0|%0, %6, %4}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[3805],
    7,
    7,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:45 */
  {
    "*sse2_lfence",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lfence",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1159],
    1,
    1,
    1,
    0,
    1
  },
  /* ../../gcc/config/i386/sync.md:64 */
  {
    "*sse_sfence",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "sfence",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    0,
    &operand_data[1159],
    1,
    1,
    1,
    0,
    1
  },
  /* ../../gcc/config/i386/sync.md:83 */
  {
    "mfence_sse2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mfence",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mfence_sse2,
    &operand_data[1159],
    1,
    1,
    1,
    0,
    1
  },
  /* ../../gcc/config/i386/sync.md:93 */
  {
    "mfence_nosse",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} or{l}\t{$0, (%%esp)|DWORD PTR [esp], 0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_mfence_nosse,
    &operand_data[1159],
    1,
    1,
    1,
    0,
    1
  },
  /* ../../gcc/config/i386/sync.md:158 */
  {
    "atomic_loaddi_fpu",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_loaddi_fpu,
    &operand_data[3812],
    3,
    4,
    0,
    3,
    1
  },
  /* ../../gcc/config/i386/sync.md:236 */
  {
    "atomic_storeqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_storeqi_1,
    &operand_data[3816],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:236 */
  {
    "atomic_storehi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{w}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_storehi_1,
    &operand_data[3819],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:236 */
  {
    "atomic_storesi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{l}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_storesi_1,
    &operand_data[3822],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:236 */
  {
    "atomic_storedi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "mov{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_storedi_1,
    &operand_data[3825],
    3,
    3,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:244 */
  {
    "atomic_storedi_fpu",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "#",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_storedi_fpu,
    &operand_data[3828],
    3,
    4,
    0,
    3,
    1
  },
  /* ../../gcc/config/i386/sync.md:288 */
  {
    "loaddi_via_fpu",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "fild%Z1\t%1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_loaddi_via_fpu,
    &operand_data[456],
    2,
    2,
    0,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:297 */
  {
    "storedi_via_fpu",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2019 },
#else
    { 0, 0, output_2019 },
#endif
    (insn_gen_fn) gen_storedi_via_fpu,
    &operand_data[3832],
    2,
    2,
    0,
    1,
    3
  },
  /* ../../gcc/config/i386/sync.md:379 */
  {
    "atomic_compare_and_swapqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} cmpxchg{b}\t{%3, %1|%1, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_compare_and_swapqi_1,
    &operand_data[3834],
    4,
    4,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:379 */
  {
    "atomic_compare_and_swaphi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} cmpxchg{w}\t{%3, %1|%1, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_compare_and_swaphi_1,
    &operand_data[3838],
    4,
    4,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:379 */
  {
    "atomic_compare_and_swapsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} cmpxchg{l}\t{%3, %1|%1, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_compare_and_swapsi_1,
    &operand_data[3842],
    4,
    4,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:379 */
  {
    "atomic_compare_and_swapdi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} cmpxchg{q}\t{%3, %1|%1, %3}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_compare_and_swapdi_1,
    &operand_data[3846],
    4,
    4,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:408 */
  {
    "atomic_compare_and_swapdi_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2024 },
#else
    { 0, 0, output_2024 },
#endif
    (insn_gen_fn) gen_atomic_compare_and_swapdi_doubleword,
    &operand_data[3850],
    7,
    8,
    1,
    2,
    3
  },
  /* ../../gcc/config/i386/sync.md:408 */
  {
    "atomic_compare_and_swapti_doubleword",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2025 },
#else
    { 0, 0, output_2025 },
#endif
    (insn_gen_fn) gen_atomic_compare_and_swapti_doubleword,
    &operand_data[3858],
    7,
    8,
    1,
    2,
    3
  },
  /* ../../gcc/config/i386/sync.md:442 */
  {
    "atomic_fetch_addqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} xadd{b}\t{%0, %1|%1, %0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_fetch_addqi,
    &operand_data[3866],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:442 */
  {
    "atomic_fetch_addhi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} xadd{w}\t{%0, %1|%1, %0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_fetch_addhi,
    &operand_data[3870],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:442 */
  {
    "atomic_fetch_addsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} xadd{l}\t{%0, %1|%1, %0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_fetch_addsi,
    &operand_data[3874],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:442 */
  {
    "atomic_fetch_adddi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} xadd{q}\t{%0, %1|%1, %0}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_fetch_adddi,
    &operand_data[3878],
    4,
    4,
    2,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:486 */
  {
    "*atomic_fetch_add_cmpqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2030 },
#else
    { 0, 0, output_2030 },
#endif
    0,
    &operand_data[3882],
    4,
    4,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/sync.md:486 */
  {
    "*atomic_fetch_add_cmphi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2031 },
#else
    { 0, 0, output_2031 },
#endif
    0,
    &operand_data[3886],
    4,
    4,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/sync.md:486 */
  {
    "*atomic_fetch_add_cmpsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2032 },
#else
    { 0, 0, output_2032 },
#endif
    0,
    &operand_data[3890],
    4,
    4,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/sync.md:486 */
  {
    "*atomic_fetch_add_cmpdi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2033 },
#else
    { 0, 0, output_2033 },
#endif
    0,
    &operand_data[3894],
    4,
    4,
    2,
    1,
    3
  },
  /* ../../gcc/config/i386/sync.md:515 */
  {
    "atomic_exchangeqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xchg{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_exchangeqi,
    &operand_data[3898],
    4,
    4,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:515 */
  {
    "atomic_exchangehi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xchg{w}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_exchangehi,
    &operand_data[3902],
    4,
    4,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:515 */
  {
    "atomic_exchangesi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xchg{l}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_exchangesi,
    &operand_data[3906],
    4,
    4,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:515 */
  {
    "atomic_exchangedi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "xchg{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_exchangedi,
    &operand_data[3910],
    4,
    4,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:526 */
  {
    "atomic_addqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2038 },
#else
    { 0, 0, output_2038 },
#endif
    (insn_gen_fn) gen_atomic_addqi,
    &operand_data[3914],
    3,
    3,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/sync.md:526 */
  {
    "atomic_addhi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2039 },
#else
    { 0, 0, output_2039 },
#endif
    (insn_gen_fn) gen_atomic_addhi,
    &operand_data[3917],
    3,
    3,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/sync.md:526 */
  {
    "atomic_addsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2040 },
#else
    { 0, 0, output_2040 },
#endif
    (insn_gen_fn) gen_atomic_addsi,
    &operand_data[3920],
    3,
    3,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/sync.md:526 */
  {
    "atomic_adddi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2041 },
#else
    { 0, 0, output_2041 },
#endif
    (insn_gen_fn) gen_atomic_adddi,
    &operand_data[3923],
    3,
    3,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/sync.md:550 */
  {
    "atomic_subqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2042 },
#else
    { 0, 0, output_2042 },
#endif
    (insn_gen_fn) gen_atomic_subqi,
    &operand_data[3914],
    3,
    3,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/sync.md:550 */
  {
    "atomic_subhi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2043 },
#else
    { 0, 0, output_2043 },
#endif
    (insn_gen_fn) gen_atomic_subhi,
    &operand_data[3917],
    3,
    3,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/sync.md:550 */
  {
    "atomic_subsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2044 },
#else
    { 0, 0, output_2044 },
#endif
    (insn_gen_fn) gen_atomic_subsi,
    &operand_data[3920],
    3,
    3,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/sync.md:550 */
  {
    "atomic_subdi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .function = output_2045 },
#else
    { 0, 0, output_2045 },
#endif
    (insn_gen_fn) gen_atomic_subdi,
    &operand_data[3923],
    3,
    3,
    1,
    1,
    3
  },
  /* ../../gcc/config/i386/sync.md:574 */
  {
    "atomic_andqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} and{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_andqi,
    &operand_data[3914],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:574 */
  {
    "atomic_orqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} or{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_orqi,
    &operand_data[3914],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:574 */
  {
    "atomic_xorqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} xor{b}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_xorqi,
    &operand_data[3914],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:574 */
  {
    "atomic_andhi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} and{w}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_andhi,
    &operand_data[3917],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:574 */
  {
    "atomic_orhi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} or{w}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_orhi,
    &operand_data[3917],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:574 */
  {
    "atomic_xorhi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} xor{w}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_xorhi,
    &operand_data[3917],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:574 */
  {
    "atomic_andsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} and{l}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_andsi,
    &operand_data[3920],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:574 */
  {
    "atomic_orsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} or{l}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_orsi,
    &operand_data[3920],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:574 */
  {
    "atomic_xorsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} xor{l}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_xorsi,
    &operand_data[3920],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:574 */
  {
    "atomic_anddi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} and{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_anddi,
    &operand_data[3923],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:574 */
  {
    "atomic_ordi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} or{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_ordi,
    &operand_data[3923],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/sync.md:574 */
  {
    "atomic_xordi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { .single =
#else
    {
#endif
    "lock{%;} xor{q}\t{%1, %0|%0, %1}",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    },
#else
    0, 0 },
#endif
    (insn_gen_fn) gen_atomic_xordi,
    &operand_data[3923],
    3,
    3,
    1,
    1,
    1
  },
  /* ../../gcc/config/i386/i386.md:927 */
  {
    "cbranchqi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cbranchqi4,
    &operand_data[3926],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:927 */
  {
    "cbranchhi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cbranchhi4,
    &operand_data[3930],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:927 */
  {
    "cbranchsi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cbranchsi4,
    &operand_data[3934],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:927 */
  {
    "cbranchdi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cbranchdi4,
    &operand_data[3938],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:927 */
  {
    "cbranchti4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cbranchti4,
    &operand_data[3942],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:945 */
  {
    "cstoreqi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cstoreqi4,
    &operand_data[3946],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:945 */
  {
    "cstorehi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cstorehi4,
    &operand_data[3950],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:945 */
  {
    "cstoresi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cstoresi4,
    &operand_data[3954],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:945 */
  {
    "cstoredi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cstoredi4,
    &operand_data[3958],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:961 */
  {
    "cmpsi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cmpsi_1,
    &operand_data[3935],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:961 */
  {
    "cmpdi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cmpdi_1,
    &operand_data[3939],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1041 */
  {
    "cmpqi_ext_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cmpqi_ext_3,
    &operand_data[3962],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1104 */
  {
    "cbranchxf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cbranchxf4,
    &operand_data[3964],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1121 */
  {
    "cstorexf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cstorexf4,
    &operand_data[3968],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1136 */
  {
    "cbranchsf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cbranchsf4,
    &operand_data[3972],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1136 */
  {
    "cbranchdf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cbranchdf4,
    &operand_data[3976],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1153 */
  {
    "cstoresf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cstoresf4,
    &operand_data[3980],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1153 */
  {
    "cstoredf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cstoredf4,
    &operand_data[3984],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1168 */
  {
    "cbranchcc4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cbranchcc4,
    &operand_data[3988],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1182 */
  {
    "cstorecc4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cstorecc4,
    &operand_data[3992],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1224 */
  {
    "cstorecc4+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[3996],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1265 */
  {
    "cstorecc4+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[3999],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1299 */
  {
    "cstorecc4+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4002],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1299 */
  {
    "cstorecc4+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4005],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1340 */
  {
    "cstorecc4+5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4008],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1385 */
  {
    "cstorecc4+6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4011],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1385 */
  {
    "movoi-6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4015],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1606 */
  {
    "movoi-5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4019],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1628 */
  {
    "movoi-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4021],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1640 */
  {
    "movoi-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4021],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1655 */
  {
    "movoi-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4021],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1672 */
  {
    "movoi-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4024],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1737 */
  {
    "movoi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movoi,
    &operand_data[4026],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1743 */
  {
    "movti",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movti,
    &operand_data[4028],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1761 */
  {
    "movcdi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movcdi,
    &operand_data[4030],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1773 */
  {
    "movqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movqi,
    &operand_data[3927],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1773 */
  {
    "movhi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movhi,
    &operand_data[3931],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1773 */
  {
    "movsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movsi,
    &operand_data[4032],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1773 */
  {
    "movdi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movdi,
    &operand_data[4034],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:1873 */
  {
    "movdi+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4036],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2018 */
  {
    "reload_noff_store",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reload_noff_store,
    &operand_data[4038],
    3,
    3,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/i386.md:2034 */
  {
    "reload_noff_load",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reload_noff_load,
    &operand_data[4041],
    3,
    3,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/i386.md:2122 */
  {
    "reload_noff_load+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4034],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2397 */
  {
    "movstrictqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movstrictqi,
    &operand_data[4044],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2397 */
  {
    "movstricthi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movstricthi,
    &operand_data[4046],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2556 */
  {
    "movsi_insv_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movsi_insv_1,
    &operand_data[4048],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2556 */
  {
    "movdi_insv_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movdi_insv_1,
    &operand_data[4050],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2616 */
  {
    "movdi_insv_1+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4052],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2616 */
  {
    "movdi_insv_1+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4052],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2654 */
  {
    "movdi_insv_1+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4054],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2654 */
  {
    "movdi_insv_1+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4054],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2692 */
  {
    "movdi_insv_1+5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4056],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2692 */
  {
    "movtf-5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4056],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2726 */
  {
    "movtf-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4058],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2726 */
  {
    "movtf-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4058],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2734 */
  {
    "movtf-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4060],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2741 */
  {
    "movtf-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4062],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2754 */
  {
    "movtf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movtf,
    &operand_data[4064],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2763 */
  {
    "movsf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movsf,
    &operand_data[4066],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2763 */
  {
    "movdf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movdf,
    &operand_data[4068],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:2763 */
  {
    "movxf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movxf,
    &operand_data[4070],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3336 */
  {
    "movxf+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4072],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3355 */
  {
    "movxf+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4072],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3374 */
  {
    "movxf+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4074],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3374 */
  {
    "zero_extendsidi2-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4076],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3374 */
  {
    "zero_extendsidi2-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4078],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3393 */
  {
    "zero_extendsidi2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4080],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3436 */
  {
    "zero_extendsidi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_zero_extendsidi2,
    &operand_data[4082],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3465 */
  {
    "zero_extendsidi2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4084],
    0,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3492 */
  {
    "zero_extendsidi2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4085],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3501 */
  {
    "zero_extendhisi2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4087],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3520 */
  {
    "zero_extendhisi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_zero_extendhisi2,
    &operand_data[4089],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3533 */
  {
    "zero_extendhisi2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4091],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3555 */
  {
    "zero_extendqihi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_zero_extendqihi2,
    &operand_data[4092],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3555 */
  {
    "zero_extendqisi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_zero_extendqisi2,
    &operand_data[4094],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3572 */
  {
    "zero_extendqisi2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4092],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3572 */
  {
    "zero_extendqisi2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4094],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3597 */
  {
    "zero_extendqisi2+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4092],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3597 */
  {
    "extendsidi2-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4094],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3618 */
  {
    "extendsidi2-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4096],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3618 */
  {
    "extendsidi2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4098],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3629 */
  {
    "extendsidi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_extendsidi2,
    &operand_data[4085],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3662 */
  {
    "extendsidi2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4100],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3677 */
  {
    "extendsidi2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4100],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3707 */
  {
    "extendsidi2+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4103],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3848 */
  {
    "extendsidi2+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4106],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3848 */
  {
    "extendsidi2+5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4106],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3855 */
  {
    "extendsfdf2-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4108],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3855 */
  {
    "extendsfdf2-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4108],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3855 */
  {
    "extendsfdf2-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4110],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3855 */
  {
    "extendsfdf2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4110],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3863 */
  {
    "extendsfdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_extendsfdf2,
    &operand_data[4112],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3891 */
  {
    "extendsfdf2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4114],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3969 */
  {
    "extendsfxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_extendsfxf2,
    &operand_data[4116],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:3969 */
  {
    "extenddfxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_extenddfxf2,
    &operand_data[4118],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4006 */
  {
    "truncdfsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_truncdfsf2,
    &operand_data[4120],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4031 */
  {
    "truncdfsf2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4122],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4070 */
  {
    "truncdfsf2_with_temp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_truncdfsf2_with_temp,
    &operand_data[4124],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4171 */
  {
    "truncdfsf2_with_temp+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4127],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4183 */
  {
    "truncxfsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_truncxfsf2,
    &operand_data[4130],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4183 */
  {
    "truncxfdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_truncxfdf2,
    &operand_data[4132],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4249 */
  {
    "truncxfdf2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4134],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4249 */
  {
    "truncxfdf2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4137],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4258 */
  {
    "fix_truncxfdi2-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4140],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4258 */
  {
    "fix_truncxfdi2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4143],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4268 */
  {
    "fix_truncxfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncxfdi2,
    &operand_data[1436],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4281 */
  {
    "fix_truncsfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncsfdi2,
    &operand_data[4146],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4281 */
  {
    "fix_truncdfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncdfdi2,
    &operand_data[4148],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4305 */
  {
    "fix_truncxfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncxfsi2,
    &operand_data[1463],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4318 */
  {
    "fix_truncsfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncsfsi2,
    &operand_data[4150],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4318 */
  {
    "fix_truncdfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncdfsi2,
    &operand_data[4152],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4342 */
  {
    "fix_truncsfhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncsfhi2,
    &operand_data[4154],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4342 */
  {
    "fix_truncdfhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncdfhi2,
    &operand_data[4156],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4342 */
  {
    "fix_truncxfhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fix_truncxfhi2,
    &operand_data[1461],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4358 */
  {
    "fixuns_truncsfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fixuns_truncsfsi2,
    &operand_data[4158],
    2,
    5,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4358 */
  {
    "fixuns_truncdfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fixuns_truncdfsi2,
    &operand_data[4163],
    2,
    5,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4382 */
  {
    "fixuns_truncdfsi2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4168],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4382 */
  {
    "fixuns_truncsfhi2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4173],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4403 */
  {
    "fixuns_truncsfhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fixuns_truncsfhi2,
    &operand_data[4178],
    2,
    2,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4403 */
  {
    "fixuns_truncdfhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fixuns_truncdfhi2,
    &operand_data[4180],
    2,
    2,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4440 */
  {
    "fixuns_truncdfhi2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4182],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4440 */
  {
    "fixuns_truncdfhi2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4185],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4440 */
  {
    "fixuns_truncdfhi2+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4188],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4440 */
  {
    "fixuns_truncdfhi2+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4191],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4451 */
  {
    "fixuns_truncdfhi2+5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4194],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4451 */
  {
    "fixuns_truncdfhi2+6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4197],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4459 */
  {
    "fixuns_truncdfhi2+7",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4200],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4459 */
  {
    "fixuns_truncdfhi2+8",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4203],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4467 */
  {
    "fixuns_truncdfhi2+9",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[281],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4467 */
  {
    "fixuns_truncdfhi2+10",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[283],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4467 */
  {
    "fixuns_truncdfhi2+11",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[285],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4521 */
  {
    "fixuns_truncdfhi2+12",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4206],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4521 */
  {
    "fixuns_truncdfhi2+13",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4210],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4521 */
  {
    "floathisf2-13",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4214],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4531 */
  {
    "floathisf2-12",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4218],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4531 */
  {
    "floathisf2-11",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4222],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4531 */
  {
    "floathisf2-10",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4226],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4545 */
  {
    "floathisf2-9",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[281],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4545 */
  {
    "floathisf2-8",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[283],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4545 */
  {
    "floathisf2-7",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[285],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4607 */
  {
    "floathisf2-6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4230],
    0,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4621 */
  {
    "floathisf2-5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4236],
    0,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4661 */
  {
    "floathisf2-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4242],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4661 */
  {
    "floathisf2-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4247],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4673 */
  {
    "floathisf2-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4252],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4673 */
  {
    "floathisf2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4257],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4713 */
  {
    "floathisf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floathisf2,
    &operand_data[4155],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4713 */
  {
    "floathidf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floathidf2,
    &operand_data[4153],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4713 */
  {
    "floathixf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floathixf2,
    &operand_data[4262],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4721 */
  {
    "floathixf2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[337],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4721 */
  {
    "floathixf2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[339],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4721 */
  {
    "floathixf2+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[341],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4759 */
  {
    "floathixf2+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4264],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4759 */
  {
    "floathixf2+5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4267],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4759 */
  {
    "floatsisf2-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4270],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4770 */
  {
    "floatsisf2-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4273],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4770 */
  {
    "floatsisf2-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4276],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4770 */
  {
    "floatsisf2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4279],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4780 */
  {
    "floatsisf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floatsisf2,
    &operand_data[4151],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4780 */
  {
    "floatsidf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floatsidf2,
    &operand_data[4149],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4780 */
  {
    "floatsixf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floatsixf2,
    &operand_data[1462],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4780 */
  {
    "floatdisf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floatdisf2,
    &operand_data[4147],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4780 */
  {
    "floatdidf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floatdidf2,
    &operand_data[4282],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4780 */
  {
    "floatdixf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floatdixf2,
    &operand_data[4284],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4810 */
  {
    "floatdixf2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[358],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4810 */
  {
    "floatdixf2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[360],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4810 */
  {
    "floatdixf2+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[362],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4810 */
  {
    "floatdixf2+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[364],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4810 */
  {
    "floatdixf2+5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[366],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4810 */
  {
    "floatdixf2+6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[368],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4895 */
  {
    "floatdixf2+7",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4286],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4895 */
  {
    "floatdixf2+8",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4289],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4895 */
  {
    "floatdixf2+9",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4292],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4895 */
  {
    "floatdixf2+10",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4295],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4908 */
  {
    "floatdixf2+11",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4286],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4908 */
  {
    "floatdixf2+12",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4289],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4908 */
  {
    "floatdixf2+13",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4292],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:4908 */
  {
    "floatdixf2+14",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4295],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5000 */
  {
    "floatdixf2+15",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4286],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5000 */
  {
    "floatdixf2+16",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4289],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5043 */
  {
    "floatdixf2+17",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4298],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5043 */
  {
    "floatdixf2+18",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4301],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5068 */
  {
    "floatdixf2+19",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[358],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5068 */
  {
    "floatdixf2+20",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[360],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5114 */
  {
    "floatdixf2+21",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4298],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5114 */
  {
    "floatdixf2+22",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4301],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5175 */
  {
    "floatdixf2+23",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4304],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5175 */
  {
    "floatdixf2+24",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4307],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5175 */
  {
    "floatdixf2+25",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4310],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5175 */
  {
    "floatdixf2+26",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4313],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5210 */
  {
    "floatdixf2+27",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4286],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5210 */
  {
    "floatdixf2+28",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4289],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5210 */
  {
    "floatdixf2+29",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4292],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5210 */
  {
    "floatunssisf2-29",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4295],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5224 */
  {
    "floatunssisf2-28",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4298],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5224 */
  {
    "floatunssisf2-27",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4301],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5224 */
  {
    "floatunssisf2-26",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4316],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5224 */
  {
    "floatunssisf2-25",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4319],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5262 */
  {
    "floatunssisf2-24",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4322],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5262 */
  {
    "floatunssisf2-23",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4325],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5262 */
  {
    "floatunssisf2-22",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4328],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5262 */
  {
    "floatunssisf2-21",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4331],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5262 */
  {
    "floatunssisf2-20",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4334],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5262 */
  {
    "floatunssisf2-19",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4337],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5272 */
  {
    "floatunssisf2-18",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4340],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5272 */
  {
    "floatunssisf2-17",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4343],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5272 */
  {
    "floatunssisf2-16",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4346],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5272 */
  {
    "floatunssisf2-15",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4349],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5272 */
  {
    "floatunssisf2-14",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4352],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5272 */
  {
    "floatunssisf2-13",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4355],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5300 */
  {
    "floatunssisf2-12",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4358],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5300 */
  {
    "floatunssisf2-11",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4363],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5300 */
  {
    "floatunssisf2-10",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4368],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5325 */
  {
    "floatunssisf2-9",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4373],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5325 */
  {
    "floatunssisf2-8",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4378],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5325 */
  {
    "floatunssisf2-7",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4383],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5356 */
  {
    "floatunssisf2-6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4388],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5356 */
  {
    "floatunssisf2-5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4392],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5356 */
  {
    "floatunssisf2-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4396],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5371 */
  {
    "floatunssisf2-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4400],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5371 */
  {
    "floatunssisf2-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4404],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5371 */
  {
    "floatunssisf2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4408],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5389 */
  {
    "floatunssisf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floatunssisf2,
    &operand_data[4412],
    2,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5389 */
  {
    "floatunssidf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floatunssidf2,
    &operand_data[4416],
    2,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5389 */
  {
    "floatunssixf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floatunssixf2,
    &operand_data[4420],
    2,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5410 */
  {
    "floatunsdisf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floatunsdisf2,
    &operand_data[4147],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5416 */
  {
    "floatunsdidf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floatunsdidf2,
    &operand_data[4282],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5431 */
  {
    "floatunsdidf2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4424],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5431 */
  {
    "addqi3-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4426],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5461 */
  {
    "addqi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addqi3,
    &operand_data[4428],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5461 */
  {
    "addhi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addhi3,
    &operand_data[4431],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5461 */
  {
    "addsi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addsi3,
    &operand_data[4434],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5461 */
  {
    "adddi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_adddi3,
    &operand_data[4437],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5461 */
  {
    "addti3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addti3,
    &operand_data[4440],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5468 */
  {
    "addti3+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4437],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5468 */
  {
    "addti3+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4440],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5778 */
  {
    "addti3+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4443],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5778 */
  {
    "addti3+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4446],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5789 */
  {
    "addti3+5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4449],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5789 */
  {
    "addti3+6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4452],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5789 */
  {
    "addti3+7",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4455],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5789 */
  {
    "subqi3-7",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4458],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:5815 */
  {
    "subqi3-6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4461],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6239 */
  {
    "subqi3-5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4464],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6271 */
  {
    "subqi3-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4468],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6301 */
  {
    "subqi3-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4472],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6336 */
  {
    "subqi3-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4477],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6336 */
  {
    "subqi3-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4477],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6380 */
  {
    "subqi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subqi3,
    &operand_data[4428],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6380 */
  {
    "subhi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subhi3,
    &operand_data[4431],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6380 */
  {
    "subsi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subsi3,
    &operand_data[4434],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6380 */
  {
    "subdi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subdi3,
    &operand_data[4437],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6380 */
  {
    "subti3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subti3,
    &operand_data[4440],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6387 */
  {
    "subti3+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4437],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6387 */
  {
    "addqi3_carry-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4440],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6501 */
  {
    "addqi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addqi3_carry,
    &operand_data[4481],
    5,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6501 */
  {
    "subqi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subqi3_carry,
    &operand_data[4481],
    5,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6501 */
  {
    "addhi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addhi3_carry,
    &operand_data[4486],
    5,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6501 */
  {
    "subhi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subhi3_carry,
    &operand_data[4486],
    5,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6501 */
  {
    "addsi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addsi3_carry,
    &operand_data[4491],
    5,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6501 */
  {
    "subsi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subsi3_carry,
    &operand_data[4491],
    5,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6501 */
  {
    "adddi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_adddi3_carry,
    &operand_data[4496],
    5,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6501 */
  {
    "subdi3_carry",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subdi3_carry,
    &operand_data[4496],
    5,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6603 */
  {
    "addxf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addxf3,
    &operand_data[4501],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6603 */
  {
    "subxf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subxf3,
    &operand_data[4501],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6610 */
  {
    "addsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addsf3,
    &operand_data[4504],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6610 */
  {
    "subsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subsf3,
    &operand_data[4504],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6610 */
  {
    "adddf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_adddf3,
    &operand_data[4507],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6610 */
  {
    "subdf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subdf3,
    &operand_data[4507],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6620 */
  {
    "mulhi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulhi3,
    &operand_data[4510],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6620 */
  {
    "mulsi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulsi3,
    &operand_data[4513],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6620 */
  {
    "muldi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_muldi3,
    &operand_data[4516],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6627 */
  {
    "mulqi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulqi3,
    &operand_data[4519],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6764 */
  {
    "mulsidi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulsidi3,
    &operand_data[4522],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6764 */
  {
    "umulsidi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umulsidi3,
    &operand_data[4522],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6764 */
  {
    "mulditi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulditi3,
    &operand_data[4525],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6764 */
  {
    "umulditi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umulditi3,
    &operand_data[4525],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6773 */
  {
    "mulqihi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulqihi3,
    &operand_data[4528],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6773 */
  {
    "umulqihi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umulqihi3,
    &operand_data[4528],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6846 */
  {
    "umulqihi3+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4531],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6846 */
  {
    "smulsi3_highpart-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4534],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6911 */
  {
    "smulsi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smulsi3_highpart,
    &operand_data[4537],
    3,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6911 */
  {
    "umulsi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umulsi3_highpart,
    &operand_data[4537],
    3,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6911 */
  {
    "smuldi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smuldi3_highpart,
    &operand_data[4541],
    3,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:6911 */
  {
    "umuldi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umuldi3_highpart,
    &operand_data[4541],
    3,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7001 */
  {
    "mulxf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulxf3,
    &operand_data[4501],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7007 */
  {
    "mulsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulsf3,
    &operand_data[4504],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7007 */
  {
    "muldf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_muldf3,
    &operand_data[4507],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7018 */
  {
    "divxf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_divxf3,
    &operand_data[4501],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7024 */
  {
    "divdf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_divdf3,
    &operand_data[4507],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7031 */
  {
    "divsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_divsf3,
    &operand_data[4504],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7052 */
  {
    "divmodhi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_divmodhi4,
    &operand_data[4545],
    4,
    4,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7052 */
  {
    "divmodsi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_divmodsi4,
    &operand_data[4549],
    4,
    4,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7052 */
  {
    "divmoddi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_divmoddi4,
    &operand_data[4553],
    4,
    4,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7066 */
  {
    "divmoddi4+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4557],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7066 */
  {
    "divmoddi4+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4561],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7080 */
  {
    "divmoddi4+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4557],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7080 */
  {
    "divmoddi4+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4561],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7115 */
  {
    "divmodqi4-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4565],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7115 */
  {
    "divmodqi4-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4557],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7115 */
  {
    "divmodqi4-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4561],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7163 */
  {
    "divmodqi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_divmodqi4,
    &operand_data[4569],
    4,
    4,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7225 */
  {
    "udivmodhi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_udivmodhi4,
    &operand_data[4545],
    4,
    4,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7225 */
  {
    "udivmodsi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_udivmodsi4,
    &operand_data[4549],
    4,
    4,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7225 */
  {
    "udivmoddi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_udivmoddi4,
    &operand_data[4553],
    4,
    4,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7239 */
  {
    "udivmoddi4+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4557],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7239 */
  {
    "udivmoddi4+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4561],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7253 */
  {
    "udivmoddi4+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4557],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7253 */
  {
    "udivmoddi4+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4561],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7275 */
  {
    "udivmodqi4-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4565],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7275 */
  {
    "udivmodqi4-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4557],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7275 */
  {
    "udivmodqi4-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4561],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7309 */
  {
    "udivmodqi4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_udivmodqi4,
    &operand_data[4569],
    4,
    4,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7390 */
  {
    "testsi_ccno_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_testsi_ccno_1,
    &operand_data[4573],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7397 */
  {
    "testqi_ccz_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_testqi_ccz_1,
    &operand_data[4575],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7403 */
  {
    "testdi_ccno_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_testdi_ccno_1,
    &operand_data[4577],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7470 */
  {
    "testqi_ext_ccno_0",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_testqi_ext_ccno_0,
    &operand_data[4579],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7588 */
  {
    "testqi_ext_ccno_0+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4581],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7647 */
  {
    "testqi_ext_ccno_0+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4586],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7670 */
  {
    "andqi3-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4581],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7695 */
  {
    "andqi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andqi3,
    &operand_data[4428],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7695 */
  {
    "andhi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andhi3,
    &operand_data[4431],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7695 */
  {
    "andsi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andsi3,
    &operand_data[4590],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7695 */
  {
    "anddi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_anddi3,
    &operand_data[4593],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7877 */
  {
    "anddi3+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[282],
    0,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7887 */
  {
    "anddi3+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[3962],
    0,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:7897 */
  {
    "anddi3+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[3962],
    0,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8124 */
  {
    "iorqi3-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4596],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8147 */
  {
    "iorqi3-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4599],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8173 */
  {
    "iorqi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorqi3,
    &operand_data[4428],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8173 */
  {
    "xorqi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorqi3,
    &operand_data[4428],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8173 */
  {
    "iorhi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorhi3,
    &operand_data[4431],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8173 */
  {
    "xorhi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorhi3,
    &operand_data[4431],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8173 */
  {
    "iorsi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorsi3,
    &operand_data[4434],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8173 */
  {
    "xorsi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorsi3,
    &operand_data[4434],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8173 */
  {
    "iordi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iordi3,
    &operand_data[4437],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8173 */
  {
    "xordi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xordi3,
    &operand_data[4437],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8383 */
  {
    "xordi3+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4596],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8383 */
  {
    "xordi3+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4596],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8406 */
  {
    "xorqi_cc_ext_1-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4599],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8406 */
  {
    "xorqi_cc_ext_1-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4599],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8427 */
  {
    "xorqi_cc_ext_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorqi_cc_ext_1,
    &operand_data[4602],
    3,
    3,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8500 */
  {
    "negqi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negqi2,
    &operand_data[4428],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8500 */
  {
    "neghi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_neghi2,
    &operand_data[4431],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8500 */
  {
    "negsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negsi2,
    &operand_data[4434],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8500 */
  {
    "negdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negdi2,
    &operand_data[4437],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8500 */
  {
    "negti2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negti2,
    &operand_data[4028],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8506 */
  {
    "negti2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4437],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8506 */
  {
    "abssf2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4028],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8587 */
  {
    "abssf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_abssf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8587 */
  {
    "negsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negsf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8587 */
  {
    "absdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_absdf2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8587 */
  {
    "negdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negdf2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8587 */
  {
    "absxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_absxf2,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8587 */
  {
    "negxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negxf2,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8620 */
  {
    "abstf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_abstf2,
    &operand_data[4605],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8620 */
  {
    "negtf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negtf2,
    &operand_data[4605],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8637 */
  {
    "negtf2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4607],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8645 */
  {
    "negtf2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4610],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8673 */
  {
    "negtf2+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4614],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8697 */
  {
    "copysignsf3-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4617],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8735 */
  {
    "copysignsf3-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4620],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8806 */
  {
    "copysignsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_copysignsf3,
    &operand_data[4623],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8806 */
  {
    "copysigndf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_copysigndf3,
    &operand_data[4626],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8806 */
  {
    "copysigntf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_copysigntf3,
    &operand_data[4629],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8814 */
  {
    "copysigntf3+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4632],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8814 */
  {
    "copysigntf3+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4636],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8814 */
  {
    "copysigntf3+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4640],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8841 */
  {
    "one_cmplqi2-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4644],
    0,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8841 */
  {
    "one_cmplqi2-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4650],
    0,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8841 */
  {
    "one_cmplqi2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4656],
    0,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8858 */
  {
    "one_cmplqi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_one_cmplqi2,
    &operand_data[4428],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8858 */
  {
    "one_cmplhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_one_cmplhi2,
    &operand_data[4431],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8858 */
  {
    "one_cmplsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_one_cmplsi2,
    &operand_data[4434],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8858 */
  {
    "one_cmpldi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_one_cmpldi2,
    &operand_data[4437],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8905 */
  {
    "one_cmpldi2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4662],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8905 */
  {
    "one_cmpldi2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4666],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8905 */
  {
    "one_cmpldi2+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4670],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8905 */
  {
    "ashlqi3-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4674],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8932 */
  {
    "ashlqi3-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4678],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8970 */
  {
    "ashlqi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ashlqi3,
    &operand_data[4682],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8970 */
  {
    "ashlhi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ashlhi3,
    &operand_data[4685],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8970 */
  {
    "ashlsi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ashlsi3,
    &operand_data[4688],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8970 */
  {
    "ashldi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ashldi3,
    &operand_data[4691],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8970 */
  {
    "ashlti3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ashlti3,
    &operand_data[4694],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8986 */
  {
    "ashlti3+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4697],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8986 */
  {
    "ashlti3+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4700],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8999 */
  {
    "x86_shiftsi_adj_1-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4703],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:8999 */
  {
    "x86_shiftsi_adj_1-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4707],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9044 */
  {
    "x86_shiftsi_adj_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_x86_shiftsi_adj_1,
    &operand_data[4711],
    4,
    4,
    4,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9044 */
  {
    "x86_shiftdi_adj_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_x86_shiftdi_adj_1,
    &operand_data[4715],
    4,
    4,
    4,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9060 */
  {
    "x86_shiftsi_adj_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_x86_shiftsi_adj_2,
    &operand_data[4711],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9060 */
  {
    "x86_shiftdi_adj_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_x86_shiftdi_adj_2,
    &operand_data[4715],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9167 */
  {
    "x86_shiftdi_adj_2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4719],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9167 */
  {
    "x86_shiftdi_adj_2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4722],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9236 */
  {
    "x86_shiftdi_adj_2+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4725],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9393 */
  {
    "lshrqi3-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4728],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9422 */
  {
    "lshrqi3-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4731],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9584 */
  {
    "lshrqi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lshrqi3,
    &operand_data[4682],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9584 */
  {
    "ashrqi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ashrqi3,
    &operand_data[4682],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9584 */
  {
    "lshrhi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lshrhi3,
    &operand_data[4685],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9584 */
  {
    "ashrhi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ashrhi3,
    &operand_data[4685],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9584 */
  {
    "lshrsi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lshrsi3,
    &operand_data[4688],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9584 */
  {
    "ashrsi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ashrsi3,
    &operand_data[4688],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9584 */
  {
    "lshrdi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lshrdi3,
    &operand_data[4734],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9584 */
  {
    "ashrdi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ashrdi3,
    &operand_data[4734],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9584 */
  {
    "lshrti3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lshrti3,
    &operand_data[4737],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9584 */
  {
    "ashrti3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ashrti3,
    &operand_data[4737],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9610 */
  {
    "ashrti3+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4740],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9610 */
  {
    "ashrti3+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4740],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9610 */
  {
    "ashrti3+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4737],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9610 */
  {
    "ashrti3+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4737],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9626 */
  {
    "x86_shiftsi_adj_3-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4740],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9626 */
  {
    "x86_shiftsi_adj_3-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4740],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9626 */
  {
    "x86_shiftsi_adj_3-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4744],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9626 */
  {
    "x86_shiftsi_adj_3-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4744],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9723 */
  {
    "x86_shiftsi_adj_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_x86_shiftsi_adj_3,
    &operand_data[4711],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9723 */
  {
    "x86_shiftdi_adj_3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_x86_shiftdi_adj_3,
    &operand_data[4715],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9794 */
  {
    "x86_shiftdi_adj_3+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4719],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9794 */
  {
    "x86_shiftdi_adj_3+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4719],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9794 */
  {
    "x86_shiftdi_adj_3+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4722],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9794 */
  {
    "rotlti3-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4722],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9847 */
  {
    "rotlti3-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4725],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:9847 */
  {
    "rotlti3-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4725],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10007 */
  {
    "rotlti3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotlti3,
    &operand_data[4737],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10007 */
  {
    "rotrti3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotrti3,
    &operand_data[4737],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10022 */
  {
    "rotldi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotldi3,
    &operand_data[4734],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10022 */
  {
    "rotrdi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotrdi3,
    &operand_data[4734],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10039 */
  {
    "rotlqi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotlqi3,
    &operand_data[4682],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10039 */
  {
    "rotrqi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotrqi3,
    &operand_data[4682],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10039 */
  {
    "rotlhi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotlhi3,
    &operand_data[4685],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10039 */
  {
    "rotrhi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotrhi3,
    &operand_data[4685],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10039 */
  {
    "rotlsi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotlsi3,
    &operand_data[4688],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10039 */
  {
    "rotrsi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotrsi3,
    &operand_data[4688],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10068 */
  {
    "rotrsi3+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4748],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10068 */
  {
    "rotrsi3+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4752],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10096 */
  {
    "rotrsi3+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4748],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10096 */
  {
    "rotrsi3+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4752],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10167 */
  {
    "rotrsi3+5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4756],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10167 */
  {
    "rotrsi3+6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4759],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10180 */
  {
    "extv-6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4756],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10180 */
  {
    "extv-5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4759],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10233 */
  {
    "extv-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4762],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10247 */
  {
    "extv-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4762],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10306 */
  {
    "extv-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[338],
    0,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10306 */
  {
    "extv-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[338],
    0,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10318 */
  {
    "extv",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_extv,
    &operand_data[4765],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10335 */
  {
    "extzv",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_extzv,
    &operand_data[4769],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10352 */
  {
    "insv",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_insv,
    &operand_data[4773],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10431 */
  {
    "insv+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4777],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10463 */
  {
    "insv+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4777],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10495 */
  {
    "insv+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4777],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10547 */
  {
    "insv+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4780],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10561 */
  {
    "insv+5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4782],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10578 */
  {
    "insv+6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4782],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10619 */
  {
    "insv+7",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4784],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10628 */
  {
    "insv+8",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4786],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10637 */
  {
    "insv+9",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4784],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10657 */
  {
    "insv+10",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4786],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10744 */
  {
    "insv+11",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4787],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10758 */
  {
    "insv+12",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4787],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10788 */
  {
    "indirect_jump-12",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4789],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10788 */
  {
    "indirect_jump-11",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4793],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10822 */
  {
    "indirect_jump-10",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4797],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10822 */
  {
    "indirect_jump-9",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4802],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10856 */
  {
    "indirect_jump-8",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4789],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:10889 */
  {
    "indirect_jump-7",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4797],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11008 */
  {
    "indirect_jump-6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4807],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11025 */
  {
    "indirect_jump-5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4812],
    0,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11067 */
  {
    "indirect_jump-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4818],
    0,
    7,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11067 */
  {
    "indirect_jump-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4825],
    0,
    7,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11091 */
  {
    "indirect_jump-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4832],
    0,
    7,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11091 */
  {
    "indirect_jump-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4839],
    0,
    7,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11132 */
  {
    "indirect_jump",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_indirect_jump,
    &operand_data[4846],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11142 */
  {
    "tablejump",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_tablejump,
    &operand_data[4846],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11192 */
  {
    "tablejump+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4847],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11213 */
  {
    "call-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4847],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11248 */
  {
    "call",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_call,
    &operand_data[4851],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11259 */
  {
    "sibcall",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sibcall,
    &operand_data[4851],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11270 */
  {
    "sibcall+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4854],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11270 */
  {
    "sibcall+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4857],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11289 */
  {
    "sibcall+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4857],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11334 */
  {
    "call_pop-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4860],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11334 */
  {
    "call_pop-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4863],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11353 */
  {
    "call_pop",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_call_pop,
    &operand_data[4866],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11366 */
  {
    "call_pop+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4870],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11391 */
  {
    "call_value-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4874],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11418 */
  {
    "call_value",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_call_value,
    &operand_data[4878],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11430 */
  {
    "sibcall_value",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sibcall_value,
    &operand_data[4878],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11442 */
  {
    "sibcall_value+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4853],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11442 */
  {
    "sibcall_value+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4881],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11463 */
  {
    "sibcall_value+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4885],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11463 */
  {
    "call_value_pop-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4889],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11484 */
  {
    "call_value_pop-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4881],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11531 */
  {
    "call_value_pop",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_call_value_pop,
    &operand_data[4893],
    5,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11545 */
  {
    "call_value_pop+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4898],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11572 */
  {
    "untyped_call-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4903],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11601 */
  {
    "untyped_call",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_untyped_call,
    &operand_data[4908],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11657 */
  {
    "memory_blockage",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_memory_blockage,
    &operand_data[0],
    0,
    0,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11685 */
  {
    "return",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_return,
    &operand_data[0],
    0,
    0,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11702 */
  {
    "simple_return",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_simple_return,
    &operand_data[0],
    0,
    0,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11803 */
  {
    "prologue",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_prologue,
    &operand_data[0],
    0,
    0,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11857 */
  {
    "epilogue",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_epilogue,
    &operand_data[0],
    0,
    0,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11862 */
  {
    "sibcall_epilogue",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sibcall_epilogue,
    &operand_data[0],
    0,
    0,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11867 */
  {
    "eh_return",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_eh_return,
    &operand_data[282],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11886 */
  {
    "eh_return+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[0],
    0,
    0,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11912 */
  {
    "split_stack_prologue",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_split_stack_prologue,
    &operand_data[0],
    0,
    0,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11946 */
  {
    "split_stack_space_check",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_split_stack_space_check,
    &operand_data[4821],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11970 */
  {
    "ffssi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ffssi2,
    &operand_data[4532],
    2,
    2,
    7,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11970 */
  {
    "ffsdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ffsdi2,
    &operand_data[4535],
    2,
    2,
    7,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:11994 */
  {
    "ffsdi2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4911],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12046 */
  {
    "clzhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_clzhi2,
    &operand_data[4546],
    2,
    2,
    4,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12046 */
  {
    "clzsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_clzsi2,
    &operand_data[4532],
    2,
    2,
    4,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12046 */
  {
    "clzdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_clzdi2,
    &operand_data[4535],
    2,
    2,
    4,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12400 */
  {
    "bswapsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_bswapsi2,
    &operand_data[4101],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12400 */
  {
    "bswapdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_bswapdi2,
    &operand_data[4446],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12461 */
  {
    "paritydi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_paritydi2,
    &operand_data[4446],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12489 */
  {
    "paritysi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_paritysi2,
    &operand_data[4101],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12508 */
  {
    "paritysi2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4914],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12539 */
  {
    "tls_global_dynamic_32-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4918],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12604 */
  {
    "tls_global_dynamic_32",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_tls_global_dynamic_32,
    &operand_data[4920],
    4,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12638 */
  {
    "tls_global_dynamic_64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_tls_global_dynamic_64,
    &operand_data[4926],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12671 */
  {
    "tls_local_dynamic_base_32",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_tls_local_dynamic_base_32,
    &operand_data[4929],
    3,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12699 */
  {
    "tls_local_dynamic_base_64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_tls_local_dynamic_base_64,
    &operand_data[4934],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12710 */
  {
    "tls_local_dynamic_base_64+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4936],
    0,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12828 */
  {
    "tls_dynamic_gnu2_32",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_tls_dynamic_gnu2_32,
    &operand_data[4920],
    3,
    3,
    4,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12874 */
  {
    "tls_dynamic_gnu2_32+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4942],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12895 */
  {
    "tls_dynamic_gnu2_64",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_tls_dynamic_gnu2_64,
    &operand_data[4926],
    2,
    2,
    3,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:12934 */
  {
    "tls_dynamic_gnu2_64+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4947],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13312 */
  {
    "tls_dynamic_gnu2_64+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4951],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13312 */
  {
    "tls_dynamic_gnu2_64+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4955],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13333 */
  {
    "rsqrtsf2-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4959],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13333 */
  {
    "rsqrtsf2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4963],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13403 */
  {
    "rsqrtsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rsqrtsf2,
    &operand_data[4003],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13427 */
  {
    "sqrtsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sqrtsf2,
    &operand_data[4003],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13427 */
  {
    "sqrtdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sqrtdf2,
    &operand_data[4006],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13472 */
  {
    "fmodxf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmodxf3,
    &operand_data[4967],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13495 */
  {
    "fmodsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmodsf3,
    &operand_data[4970],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13495 */
  {
    "fmoddf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmoddf3,
    &operand_data[4973],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13543 */
  {
    "remainderxf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_remainderxf3,
    &operand_data[4967],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13566 */
  {
    "remaindersf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_remaindersf3,
    &operand_data[4970],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13566 */
  {
    "remainderdf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_remainderdf3,
    &operand_data[4973],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13661 */
  {
    "remainderdf3+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4501],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13671 */
  {
    "remainderdf3+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4501],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13696 */
  {
    "remainderdf3+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4502],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13696 */
  {
    "sincossf3-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4976],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13708 */
  {
    "sincossf3-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4502],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13708 */
  {
    "sincossf3-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4976],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13720 */
  {
    "sincossf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sincossf3,
    &operand_data[4979],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13720 */
  {
    "sincosdf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sincosdf3,
    &operand_data[4982],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13767 */
  {
    "tanxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_tanxf2,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13780 */
  {
    "tansf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_tansf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13780 */
  {
    "tandf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_tandf2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13827 */
  {
    "atan2xf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atan2xf3,
    &operand_data[4985],
    3,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13836 */
  {
    "atan2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atan2sf3,
    &operand_data[4979],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13836 */
  {
    "atan2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atan2df3,
    &operand_data[4982],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13852 */
  {
    "atanxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atanxf2,
    &operand_data[4989],
    2,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13865 */
  {
    "atansf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atansf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13865 */
  {
    "atandf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atandf2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13883 */
  {
    "asinxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_asinxf2,
    &operand_data[4993],
    2,
    7,
    9,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13907 */
  {
    "asinsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_asinsf2,
    &operand_data[4970],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13907 */
  {
    "asindf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_asindf2,
    &operand_data[4973],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13927 */
  {
    "acosxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_acosxf2,
    &operand_data[4993],
    2,
    7,
    9,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13951 */
  {
    "acossf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_acossf2,
    &operand_data[4970],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13951 */
  {
    "acosdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_acosdf2,
    &operand_data[4973],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:13998 */
  {
    "logxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_logxf2,
    &operand_data[4989],
    2,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14010 */
  {
    "logsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_logsf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14010 */
  {
    "logdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_logdf2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14028 */
  {
    "log10xf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_log10xf2,
    &operand_data[4989],
    2,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14040 */
  {
    "log10sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_log10sf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14040 */
  {
    "log10df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_log10df2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14058 */
  {
    "log2xf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_log2xf2,
    &operand_data[4989],
    2,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14070 */
  {
    "log2sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_log2sf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14070 */
  {
    "log2df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_log2df2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14115 */
  {
    "log1pxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_log1pxf2,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14128 */
  {
    "log1psf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_log1psf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14128 */
  {
    "log1pdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_log1pdf2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14177 */
  {
    "logbxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_logbxf2,
    &operand_data[1455],
    2,
    2,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14187 */
  {
    "logbsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_logbsf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14187 */
  {
    "logbdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_logbdf2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14203 */
  {
    "ilogbxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ilogbxf2,
    &operand_data[361],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14222 */
  {
    "ilogbsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ilogbsf2,
    &operand_data[363],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14222 */
  {
    "ilogbdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ilogbdf2,
    &operand_data[359],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14267 */
  {
    "expNcorexf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_expNcorexf3,
    &operand_data[4501],
    3,
    3,
    16,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14294 */
  {
    "expxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_expxf2,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14312 */
  {
    "expsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_expsf2,
    &operand_data[4970],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14312 */
  {
    "expdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_expdf2,
    &operand_data[4973],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14334 */
  {
    "exp10xf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_exp10xf2,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14352 */
  {
    "exp10sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_exp10sf2,
    &operand_data[4970],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14352 */
  {
    "exp10df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_exp10df2,
    &operand_data[4973],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14374 */
  {
    "exp2xf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_exp2xf2,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14392 */
  {
    "exp2sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_exp2sf2,
    &operand_data[4970],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14392 */
  {
    "exp2df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_exp2df2,
    &operand_data[4973],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14414 */
  {
    "expm1xf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_expm1xf2,
    &operand_data[1455],
    2,
    2,
    28,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14454 */
  {
    "expm1sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_expm1sf2,
    &operand_data[4970],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14454 */
  {
    "expm1df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_expm1df2,
    &operand_data[4973],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14476 */
  {
    "ldexpxf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ldexpxf3,
    &operand_data[5000],
    3,
    3,
    5,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14496 */
  {
    "ldexpsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ldexpsf3,
    &operand_data[5003],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14496 */
  {
    "ldexpdf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ldexpdf3,
    &operand_data[5006],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14519 */
  {
    "scalbxf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_scalbxf3,
    &operand_data[5009],
    3,
    3,
    3,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14536 */
  {
    "scalbsf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_scalbsf3,
    &operand_data[4970],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14536 */
  {
    "scalbdf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_scalbdf3,
    &operand_data[4973],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14561 */
  {
    "significandxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_significandxf2,
    &operand_data[1455],
    2,
    2,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14571 */
  {
    "significandsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_significandsf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14571 */
  {
    "significanddf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_significanddf2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14610 */
  {
    "rintsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rintsf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14610 */
  {
    "rintdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rintdf2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14644 */
  {
    "roundsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_roundsf2,
    &operand_data[4003],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14644 */
  {
    "rounddf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rounddf2,
    &operand_data[4006],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14644 */
  {
    "roundxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_roundxf2,
    &operand_data[5011],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14678 */
  {
    "roundxf2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[1436],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14722 */
  {
    "roundxf2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5013],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14733 */
  {
    "roundxf2+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5017],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14743 */
  {
    "roundxf2+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[340],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14743 */
  {
    "roundxf2+5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[361],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14780 */
  {
    "lrintxfhi2-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5021],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14780 */
  {
    "lrintxfhi2-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5024],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14789 */
  {
    "lrintxfhi2-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4278],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14789 */
  {
    "lrintxfhi2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5026],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14797 */
  {
    "lrintxfhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lrintxfhi2,
    &operand_data[1461],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14797 */
  {
    "lrintxfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lrintxfsi2,
    &operand_data[1463],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14797 */
  {
    "lrintxfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lrintxfdi2,
    &operand_data[1436],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14803 */
  {
    "lrintsfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lrintsfsi2,
    &operand_data[4150],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14803 */
  {
    "lrintdfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lrintdfsi2,
    &operand_data[4152],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14803 */
  {
    "lrintsfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lrintsfdi2,
    &operand_data[4146],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14803 */
  {
    "lrintdfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lrintdfdi2,
    &operand_data[4148],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14810 */
  {
    "lroundsfhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lroundsfhi2,
    &operand_data[4154],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14810 */
  {
    "lrounddfhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lrounddfhi2,
    &operand_data[4156],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14810 */
  {
    "lroundxfhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lroundxfhi2,
    &operand_data[1461],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14810 */
  {
    "lroundsfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lroundsfsi2,
    &operand_data[4150],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14810 */
  {
    "lrounddfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lrounddfsi2,
    &operand_data[4152],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14810 */
  {
    "lroundxfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lroundxfsi2,
    &operand_data[1463],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14810 */
  {
    "lroundsfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lroundsfdi2,
    &operand_data[4146],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14810 */
  {
    "lrounddfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lrounddfdi2,
    &operand_data[4148],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14810 */
  {
    "lroundxfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lroundxfdi2,
    &operand_data[1436],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14836 */
  {
    "lroundxfdi2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[1455],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14874 */
  {
    "floorxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floorxf2,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14886 */
  {
    "floorsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floorsf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14886 */
  {
    "floordf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floordf2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14926 */
  {
    "floordf2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[1461],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14926 */
  {
    "floordf2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[1463],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14926 */
  {
    "floordf2+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[1436],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:14987 */
  {
    "floordf2+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5029],
    0,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15003 */
  {
    "floordf2+5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5035],
    0,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15045 */
  {
    "lfloorxfhi2-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5041],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15045 */
  {
    "lfloorxfhi2-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5046],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15059 */
  {
    "lfloorxfhi2-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5051],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15059 */
  {
    "lfloorxfhi2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5056],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15072 */
  {
    "lfloorxfhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lfloorxfhi2,
    &operand_data[1461],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15072 */
  {
    "lfloorxfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lfloorxfsi2,
    &operand_data[1463],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15072 */
  {
    "lfloorxfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lfloorxfdi2,
    &operand_data[1436],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15081 */
  {
    "lfloorsfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lfloorsfsi2,
    &operand_data[4150],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15081 */
  {
    "lfloorsfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lfloorsfdi2,
    &operand_data[4146],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15081 */
  {
    "lfloordfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lfloordfsi2,
    &operand_data[4152],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15081 */
  {
    "lfloordfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lfloordfdi2,
    &operand_data[4148],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15094 */
  {
    "lfloordfdi2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[1455],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15132 */
  {
    "ceilxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ceilxf2,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15144 */
  {
    "ceilsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ceilsf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15144 */
  {
    "ceildf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ceildf2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15184 */
  {
    "ceildf2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[1461],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15184 */
  {
    "ceildf2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[1463],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15184 */
  {
    "ceildf2+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[1436],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15245 */
  {
    "ceildf2+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5029],
    0,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15261 */
  {
    "ceildf2+5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5035],
    0,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15303 */
  {
    "lceilxfhi2-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5041],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15303 */
  {
    "lceilxfhi2-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5046],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15317 */
  {
    "lceilxfhi2-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5051],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15317 */
  {
    "lceilxfhi2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5056],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15330 */
  {
    "lceilxfhi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lceilxfhi2,
    &operand_data[1461],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15330 */
  {
    "lceilxfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lceilxfsi2,
    &operand_data[1463],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15330 */
  {
    "lceilxfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lceilxfdi2,
    &operand_data[1436],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15339 */
  {
    "lceilsfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lceilsfsi2,
    &operand_data[4150],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15339 */
  {
    "lceilsfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lceilsfdi2,
    &operand_data[4146],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15339 */
  {
    "lceildfsi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lceildfsi2,
    &operand_data[4152],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15339 */
  {
    "lceildfdi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lceildfdi2,
    &operand_data[4148],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15350 */
  {
    "lceildfdi2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[1455],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15388 */
  {
    "btruncxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_btruncxf2,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15400 */
  {
    "btruncsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_btruncsf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15400 */
  {
    "btruncdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_btruncdf2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15441 */
  {
    "btruncdf2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[1455],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15479 */
  {
    "nearbyintxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_nearbyintxf2,
    &operand_data[1455],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15489 */
  {
    "nearbyintsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_nearbyintsf2,
    &operand_data[4504],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15489 */
  {
    "nearbyintdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_nearbyintdf2,
    &operand_data[4507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15519 */
  {
    "nearbyintdf2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[1494],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15519 */
  {
    "isinfxf2-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[1496],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15540 */
  {
    "isinfxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_isinfxf2,
    &operand_data[361],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15566 */
  {
    "isinfsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_isinfsf2,
    &operand_data[4158],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15566 */
  {
    "isinfdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_isinfdf2,
    &operand_data[4163],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15602 */
  {
    "signbitxf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_signbitxf2,
    &operand_data[361],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15628 */
  {
    "signbitdf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_signbitdf2,
    &operand_data[359],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15650 */
  {
    "signbitsf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_signbitsf2,
    &operand_data[363],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15674 */
  {
    "movmemsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmemsi,
    &operand_data[5061],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15674 */
  {
    "movmemdi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmemdi,
    &operand_data[5067],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15693 */
  {
    "strmov",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_strmov,
    &operand_data[5073],
    4,
    4,
    4,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15722 */
  {
    "strmov_singleop",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_strmov_singleop,
    &operand_data[5073],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15798 */
  {
    "rep_mov",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rep_mov,
    &operand_data[5079],
    7,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15866 */
  {
    "setmemsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_setmemsi,
    &operand_data[5086],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15866 */
  {
    "setmemdi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_setmemdi,
    &operand_data[5092],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15886 */
  {
    "strset",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_strset,
    &operand_data[5073],
    3,
    3,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15912 */
  {
    "strset_singleop",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_strset_singleop,
    &operand_data[5081],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:15979 */
  {
    "rep_stos",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rep_stos,
    &operand_data[5098],
    5,
    5,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16045 */
  {
    "cmpstrnsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cmpstrnsi,
    &operand_data[5103],
    5,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16115 */
  {
    "cmpintqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cmpintqi,
    &operand_data[3946],
    1,
    1,
    4,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16133 */
  {
    "cmpstrnqi_nz_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cmpstrnqi_nz_1,
    &operand_data[5108],
    6,
    6,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16167 */
  {
    "cmpstrnqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_cmpstrnqi_1,
    &operand_data[5108],
    6,
    6,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16205 */
  {
    "strlensi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_strlensi,
    &operand_data[5114],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16205 */
  {
    "strlendi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_strlendi,
    &operand_data[5118],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16219 */
  {
    "strlenqi_1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_strlenqi_1,
    &operand_data[5122],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16260 */
  {
    "strlenqi_1+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5125],
    0,
    9,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16289 */
  {
    "movqicc-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5125],
    0,
    9,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16325 */
  {
    "movqicc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movqicc,
    &operand_data[5133],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16325 */
  {
    "movhicc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movhicc,
    &operand_data[5137],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16325 */
  {
    "movsicc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movsicc,
    &operand_data[5141],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16325 */
  {
    "movdicc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movdicc,
    &operand_data[5145],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16337 */
  {
    "x86_movsicc_0_m1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_x86_movsicc_0_m1,
    &operand_data[5149],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16337 */
  {
    "x86_movdicc_0_m1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_x86_movdicc_0_m1,
    &operand_data[5152],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16423 */
  {
    "x86_movdicc_0_m1+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5155],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16441 */
  {
    "movsfcc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movsfcc,
    &operand_data[5159],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16441 */
  {
    "movdfcc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movdfcc,
    &operand_data[5163],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16441 */
  {
    "movxfcc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movxfcc,
    &operand_data[5167],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16496 */
  {
    "movxfcc+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5171],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16603 */
  {
    "addqicc-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5175],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16630 */
  {
    "addqicc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addqicc,
    &operand_data[5179],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16630 */
  {
    "addhicc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addhicc,
    &operand_data[5183],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16630 */
  {
    "addsicc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addsicc,
    &operand_data[5187],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16630 */
  {
    "adddicc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_adddicc,
    &operand_data[5191],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16714 */
  {
    "allocate_stack",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_allocate_stack,
    &operand_data[4599],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16751 */
  {
    "probe_stack",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_probe_stack,
    &operand_data[4073],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16786 */
  {
    "builtin_setjmp_receiver",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_builtin_setjmp_receiver,
    &operand_data[852],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16839 */
  {
    "builtin_setjmp_receiver+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5195],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16868 */
  {
    "builtin_setjmp_receiver+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5199],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16899 */
  {
    "builtin_setjmp_receiver+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5204],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16920 */
  {
    "builtin_setjmp_receiver+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4009],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16937 */
  {
    "builtin_setjmp_receiver+5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4009],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16956 */
  {
    "builtin_setjmp_receiver+6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5208],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16956 */
  {
    "builtin_setjmp_receiver+7",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5211],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16956 */
  {
    "builtin_setjmp_receiver+8",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5214],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16956 */
  {
    "builtin_setjmp_receiver+9",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5217],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16967 */
  {
    "builtin_setjmp_receiver+10",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5220],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16978 */
  {
    "builtin_setjmp_receiver+11",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5209],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16978 */
  {
    "builtin_setjmp_receiver+12",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5212],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16978 */
  {
    "builtin_setjmp_receiver+13",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5215],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16992 */
  {
    "builtin_setjmp_receiver+14",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5223],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16992 */
  {
    "builtin_setjmp_receiver+15",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5226],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:16992 */
  {
    "builtin_setjmp_receiver+16",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5229],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17003 */
  {
    "builtin_setjmp_receiver+17",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5232],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17024 */
  {
    "builtin_setjmp_receiver+18",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4428],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17024 */
  {
    "builtin_setjmp_receiver+19",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4431],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17024 */
  {
    "builtin_setjmp_receiver+20",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4434],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17045 */
  {
    "builtin_setjmp_receiver+21",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5236],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17065 */
  {
    "builtin_setjmp_receiver+22",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5240],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17082 */
  {
    "builtin_setjmp_receiver+23",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5244],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17116 */
  {
    "builtin_setjmp_receiver+24",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5248],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17129 */
  {
    "builtin_setjmp_receiver+25",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5248],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17145 */
  {
    "builtin_setjmp_receiver+26",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5252],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17162 */
  {
    "builtin_setjmp_receiver+27",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5256],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17182 */
  {
    "builtin_setjmp_receiver+28",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5260],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17198 */
  {
    "builtin_setjmp_receiver+29",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5260],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17216 */
  {
    "builtin_setjmp_receiver+30",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5264],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17216 */
  {
    "builtin_setjmp_receiver+31",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5268],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17216 */
  {
    "builtin_setjmp_receiver+32",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5272],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17216 */
  {
    "builtin_setjmp_receiver+33",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5276],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17249 */
  {
    "builtin_setjmp_receiver+34",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5280],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17249 */
  {
    "builtin_setjmp_receiver+35",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5283],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17249 */
  {
    "builtin_setjmp_receiver+36",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5286],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17249 */
  {
    "builtin_setjmp_receiver+37",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5289],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17276 */
  {
    "builtin_setjmp_receiver+38",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5292],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17276 */
  {
    "builtin_setjmp_receiver+39",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5297],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17314 */
  {
    "builtin_setjmp_receiver+40",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[3997],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17325 */
  {
    "builtin_setjmp_receiver+41",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5302],
    0,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17336 */
  {
    "builtin_setjmp_receiver+42",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[338],
    0,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17336 */
  {
    "prefetch-41",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[359],
    0,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17336 */
  {
    "prefetch-40",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[365],
    0,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17351 */
  {
    "prefetch-39",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4456],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17351 */
  {
    "prefetch-38",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4459],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17359 */
  {
    "prefetch-37",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5303],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17370 */
  {
    "prefetch-36",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4799],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17370 */
  {
    "prefetch-35",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5193],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17380 */
  {
    "prefetch-34",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5306],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17412 */
  {
    "prefetch-33",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5309],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17412 */
  {
    "prefetch-32",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5311],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17425 */
  {
    "prefetch-31",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5309],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17425 */
  {
    "prefetch-30",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5311],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17440 */
  {
    "prefetch-29",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5309],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17440 */
  {
    "prefetch-28",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5311],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17451 */
  {
    "prefetch-27",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5309],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17451 */
  {
    "prefetch-26",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5311],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17464 */
  {
    "prefetch-25",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5309],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17464 */
  {
    "prefetch-24",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5311],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17478 */
  {
    "prefetch-23",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5313],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17478 */
  {
    "prefetch-22",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5316],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17492 */
  {
    "prefetch-21",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5309],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17492 */
  {
    "prefetch-20",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5311],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17506 */
  {
    "prefetch-19",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5309],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17506 */
  {
    "prefetch-18",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5311],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17517 */
  {
    "prefetch-17",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5313],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17517 */
  {
    "prefetch-16",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5316],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17528 */
  {
    "prefetch-15",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5309],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17528 */
  {
    "prefetch-14",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5311],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17541 */
  {
    "prefetch-13",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4586],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17557 */
  {
    "prefetch-12",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5319],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17557 */
  {
    "prefetch-11",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5322],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17571 */
  {
    "prefetch-10",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5325],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17571 */
  {
    "prefetch-9",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5328],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17587 */
  {
    "prefetch-8",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5331],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17587 */
  {
    "prefetch-7",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5335],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17599 */
  {
    "prefetch-6",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5339],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17618 */
  {
    "prefetch-5",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5343],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17618 */
  {
    "prefetch-4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5347],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17618 */
  {
    "prefetch-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5351],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17647 */
  {
    "prefetch-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5355],
    0,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17647 */
  {
    "prefetch-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5361],
    0,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17699 */
  {
    "prefetch",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_prefetch,
    &operand_data[5367],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17760 */
  {
    "stack_protect_set",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_stack_protect_set,
    &operand_data[5112],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17802 */
  {
    "stack_protect_test",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_stack_protect_test,
    &operand_data[5370],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17884 */
  {
    "rdpmc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rdpmc,
    &operand_data[4085],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17940 */
  {
    "rdtsc",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rdtsc,
    &operand_data[365],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:17984 */
  {
    "rdtscp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rdtscp,
    &operand_data[5339],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/i386.md:18053 */
  {
    "lwp_llwpcb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lwp_llwpcb,
    &operand_data[573],
    1,
    1,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/i386.md:18067 */
  {
    "lwp_slwpcb",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lwp_slwpcb,
    &operand_data[571],
    1,
    1,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/i386.md:18091 */
  {
    "lwp_lwpvalsi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lwp_lwpvalsi3,
    &operand_data[5373],
    4,
    4,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/i386.md:18091 */
  {
    "lwp_lwpvaldi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lwp_lwpvaldi3,
    &operand_data[5377],
    4,
    4,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/i386.md:18112 */
  {
    "lwp_lwpinssi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lwp_lwpinssi3,
    &operand_data[5381],
    4,
    4,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/i386.md:18112 */
  {
    "lwp_lwpinsdi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lwp_lwpinsdi3,
    &operand_data[5385],
    4,
    4,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/i386.md:18177 */
  {
    "pause",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pause,
    &operand_data[0],
    0,
    0,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:71 */
  {
    "movv8qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv8qi,
    &operand_data[5389],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:71 */
  {
    "movv4hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv4hi,
    &operand_data[5391],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:71 */
  {
    "movv2si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv2si,
    &operand_data[5393],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:71 */
  {
    "movv1di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv1di,
    &operand_data[5395],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:203 */
  {
    "movv2sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv2sf,
    &operand_data[5397],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:320 */
  {
    "movv2sf+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5399],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:320 */
  {
    "movv2sf+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5401],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:320 */
  {
    "movv2sf+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5403],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:320 */
  {
    "pushv8qi1-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5405],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:320 */
  {
    "pushv8qi1-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5407],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:329 */
  {
    "pushv8qi1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv8qi1,
    &operand_data[5409],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:329 */
  {
    "pushv4hi1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv4hi1,
    &operand_data[5410],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:329 */
  {
    "pushv2si1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv2si1,
    &operand_data[5411],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:329 */
  {
    "pushv1di1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv1di1,
    &operand_data[5412],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:329 */
  {
    "pushv2sf1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv2sf1,
    &operand_data[5413],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:337 */
  {
    "movmisalignv8qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv8qi,
    &operand_data[5389],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:337 */
  {
    "movmisalignv4hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv4hi,
    &operand_data[5391],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:337 */
  {
    "movmisalignv2si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv2si,
    &operand_data[5393],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:337 */
  {
    "movmisalignv1di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv1di,
    &operand_data[5395],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:337 */
  {
    "movmisalignv2sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv2sf,
    &operand_data[5397],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:361 */
  {
    "mmx_addv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_addv2sf3,
    &operand_data[5413],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:379 */
  {
    "mmx_subv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_subv2sf3,
    &operand_data[5416],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:385 */
  {
    "mmx_subrv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_subrv2sf3,
    &operand_data[5417],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:403 */
  {
    "mmx_mulv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_mulv2sf3,
    &operand_data[5413],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:424 */
  {
    "mmx_smaxv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_smaxv2sf3,
    &operand_data[5413],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:424 */
  {
    "mmx_sminv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_sminv2sf3,
    &operand_data[5413],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:570 */
  {
    "mmx_eqv2sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_eqv2sf3,
    &operand_data[5420],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:692 */
  {
    "vec_setv2sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv2sf,
    &operand_data[5423],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:705 */
  {
    "vec_setv2sf+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5426],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:742 */
  {
    "vec_extractv2sf-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5428],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:755 */
  {
    "vec_extractv2sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv2sf,
    &operand_data[5430],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:766 */
  {
    "vec_initv2sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv2sf,
    &operand_data[5433],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:781 */
  {
    "mmx_addv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_addv8qi3,
    &operand_data[5435],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:781 */
  {
    "mmx_subv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_subv8qi3,
    &operand_data[5435],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:781 */
  {
    "mmx_addv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_addv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:781 */
  {
    "mmx_subv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_subv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:781 */
  {
    "mmx_addv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_addv2si3,
    &operand_data[5441],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:781 */
  {
    "mmx_subv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_subv2si3,
    &operand_data[5441],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:781 */
  {
    "mmx_addv1di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_addv1di3,
    &operand_data[5444],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:781 */
  {
    "mmx_subv1di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_subv1di3,
    &operand_data[5444],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:800 */
  {
    "mmx_ssaddv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_ssaddv8qi3,
    &operand_data[5435],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:800 */
  {
    "mmx_usaddv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_usaddv8qi3,
    &operand_data[5435],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:800 */
  {
    "mmx_sssubv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_sssubv8qi3,
    &operand_data[5435],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:800 */
  {
    "mmx_ussubv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_ussubv8qi3,
    &operand_data[5435],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:800 */
  {
    "mmx_ssaddv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_ssaddv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:800 */
  {
    "mmx_usaddv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_usaddv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:800 */
  {
    "mmx_sssubv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_sssubv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:800 */
  {
    "mmx_ussubv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_ussubv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:818 */
  {
    "mmx_mulv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_mulv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:834 */
  {
    "mmx_smulv4hi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_smulv4hi3_highpart,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:862 */
  {
    "mmx_umulv4hi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_umulv4hi3_highpart,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:891 */
  {
    "mmx_pmaddwd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_pmaddwd,
    &operand_data[5447],
    3,
    3,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:937 */
  {
    "mmx_pmulhrwv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_pmulhrwv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:972 */
  {
    "sse2_umulv1siv1di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_umulv1siv1di3,
    &operand_data[5450],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1002 */
  {
    "mmx_smaxv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_smaxv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1002 */
  {
    "mmx_sminv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_sminv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1021 */
  {
    "mmx_umaxv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_umaxv8qi3,
    &operand_data[5435],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1021 */
  {
    "mmx_uminv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_uminv8qi3,
    &operand_data[5435],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1074 */
  {
    "mmx_eqv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_eqv8qi3,
    &operand_data[5435],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1074 */
  {
    "mmx_eqv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_eqv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1074 */
  {
    "mmx_eqv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_eqv2si3,
    &operand_data[5441],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1118 */
  {
    "mmx_andv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_andv8qi3,
    &operand_data[5435],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1118 */
  {
    "mmx_iorv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_iorv8qi3,
    &operand_data[5435],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1118 */
  {
    "mmx_xorv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_xorv8qi3,
    &operand_data[5435],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1118 */
  {
    "mmx_andv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_andv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1118 */
  {
    "mmx_iorv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_iorv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1118 */
  {
    "mmx_xorv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_xorv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1118 */
  {
    "mmx_andv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_andv2si3,
    &operand_data[5441],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1118 */
  {
    "mmx_iorv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_iorv2si3,
    &operand_data[5441],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1118 */
  {
    "mmx_xorv2si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_xorv2si3,
    &operand_data[5441],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1260 */
  {
    "mmx_pinsrw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_pinsrw,
    &operand_data[5453],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1306 */
  {
    "mmx_pshufw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_pshufw,
    &operand_data[5457],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1387 */
  {
    "vec_setv2si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv2si,
    &operand_data[5460],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1400 */
  {
    "vec_setv2si+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5463],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1443 */
  {
    "vec_extractv2si-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5465],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1456 */
  {
    "vec_extractv2si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv2si,
    &operand_data[5467],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1467 */
  {
    "vec_initv2si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv2si,
    &operand_data[5470],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1476 */
  {
    "vec_setv4hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv4hi,
    &operand_data[5472],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1487 */
  {
    "vec_extractv4hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv4hi,
    &operand_data[5475],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1498 */
  {
    "vec_initv4hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv4hi,
    &operand_data[5478],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1507 */
  {
    "vec_setv8qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv8qi,
    &operand_data[5480],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1518 */
  {
    "vec_extractv8qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv8qi,
    &operand_data[5483],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1529 */
  {
    "vec_initv8qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv8qi,
    &operand_data[5486],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1544 */
  {
    "mmx_uavgv8qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_uavgv8qi3,
    &operand_data[5435],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1596 */
  {
    "mmx_uavgv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_uavgv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1650 */
  {
    "mmx_maskmovq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_maskmovq,
    &operand_data[5488],
    3,
    3,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1670 */
  {
    "mmx_emms",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_emms,
    &operand_data[0],
    0,
    0,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/mmx.md:1703 */
  {
    "mmx_femms",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mmx_femms,
    &operand_data[0],
    0,
    0,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv32qi,
    &operand_data[5491],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv16qi,
    &operand_data[5493],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv16hi,
    &operand_data[5495],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv8hi,
    &operand_data[5497],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv8si,
    &operand_data[5499],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv4si,
    &operand_data[5501],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv4di,
    &operand_data[5503],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv2di,
    &operand_data[5505],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv2ti",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv2ti,
    &operand_data[5507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv1ti",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv1ti,
    &operand_data[5509],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv8sf,
    &operand_data[5511],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv4sf,
    &operand_data[5513],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv4df,
    &operand_data[5515],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:414 */
  {
    "movv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movv2df,
    &operand_data[5517],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:515 */
  {
    "movv2df+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5519],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:543 */
  {
    "movv2df+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5522],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:557 */
  {
    "pushv32qi1-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5524],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv32qi1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv32qi1,
    &operand_data[2368],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv16qi1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv16qi1,
    &operand_data[2371],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv16hi1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv16hi1,
    &operand_data[5526],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv8hi1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv8hi1,
    &operand_data[5527],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv8si1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv8si1,
    &operand_data[5528],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv4si1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv4si1,
    &operand_data[2389],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv4di1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv4di1,
    &operand_data[2392],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv2di1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv2di1,
    &operand_data[2395],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv2ti1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv2ti1,
    &operand_data[5529],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv1ti1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv1ti1,
    &operand_data[5530],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv8sf1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv8sf1,
    &operand_data[5531],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv4sf1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv4sf1,
    &operand_data[5522],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv4df1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv4df1,
    &operand_data[5532],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:567 */
  {
    "pushv2df1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_pushv2df1,
    &operand_data[5524],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv32qi,
    &operand_data[5491],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv16qi,
    &operand_data[5493],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv16hi,
    &operand_data[5495],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv8hi,
    &operand_data[5497],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv8si,
    &operand_data[5499],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv4si,
    &operand_data[5501],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv4di,
    &operand_data[5503],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv2di,
    &operand_data[5505],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv2ti",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv2ti,
    &operand_data[5507],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv1ti",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv1ti,
    &operand_data[5509],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv8sf,
    &operand_data[5511],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv4sf,
    &operand_data[5513],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv4df,
    &operand_data[5515],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:575 */
  {
    "movmisalignv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_movmisalignv2df,
    &operand_data[5517],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:708 */
  {
    "storentdi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_storentdi,
    &operand_data[4084],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:708 */
  {
    "storentsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_storentsi,
    &operand_data[5533],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:708 */
  {
    "storentsf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_storentsf,
    &operand_data[5535],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:708 */
  {
    "storentdf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_storentdf,
    &operand_data[5537],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:708 */
  {
    "storentv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_storentv4di,
    &operand_data[5539],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:708 */
  {
    "storentv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_storentv2di,
    &operand_data[5541],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:708 */
  {
    "storentv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_storentv8sf,
    &operand_data[5543],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:708 */
  {
    "storentv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_storentv4sf,
    &operand_data[5545],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:708 */
  {
    "storentv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_storentv4df,
    &operand_data[5547],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:708 */
  {
    "storentv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_storentv2df,
    &operand_data[5549],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:721 */
  {
    "absv8sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_absv8sf2,
    &operand_data[5551],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:721 */
  {
    "negv8sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negv8sf2,
    &operand_data[5551],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:721 */
  {
    "absv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_absv4sf2,
    &operand_data[5553],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:721 */
  {
    "negv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negv4sf2,
    &operand_data[5553],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:721 */
  {
    "absv4df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_absv4df2,
    &operand_data[5555],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:721 */
  {
    "negv4df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negv4df2,
    &operand_data[5555],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:721 */
  {
    "absv2df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_absv2df2,
    &operand_data[5557],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:721 */
  {
    "negv2df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negv2df2,
    &operand_data[5557],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:728 */
  {
    "negv2df2+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5559],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:728 */
  {
    "negv2df2+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5563],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:728 */
  {
    "addv8sf3-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5567],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:728 */
  {
    "addv8sf3-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5571],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:766 */
  {
    "addv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addv8sf3,
    &operand_data[5559],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:766 */
  {
    "subv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subv8sf3,
    &operand_data[5559],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:766 */
  {
    "addv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addv4sf3,
    &operand_data[5563],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:766 */
  {
    "subv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subv4sf3,
    &operand_data[5563],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:766 */
  {
    "addv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addv4df3,
    &operand_data[5567],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:766 */
  {
    "subv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subv4df3,
    &operand_data[5567],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:766 */
  {
    "addv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addv2df3,
    &operand_data[5571],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:766 */
  {
    "subv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subv2df3,
    &operand_data[5571],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:805 */
  {
    "mulv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulv8sf3,
    &operand_data[5559],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:805 */
  {
    "mulv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulv4sf3,
    &operand_data[5563],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:805 */
  {
    "mulv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulv4df3,
    &operand_data[5567],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:805 */
  {
    "mulv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulv2df3,
    &operand_data[5571],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:844 */
  {
    "divv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_divv4df3,
    &operand_data[5575],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:844 */
  {
    "divv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_divv2df3,
    &operand_data[5578],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:851 */
  {
    "divv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_divv8sf3,
    &operand_data[5581],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:851 */
  {
    "divv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_divv4sf3,
    &operand_data[5584],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:929 */
  {
    "sqrtv4df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sqrtv4df2,
    &operand_data[5567],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:929 */
  {
    "sqrtv2df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sqrtv2df2,
    &operand_data[5571],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:934 */
  {
    "sqrtv8sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sqrtv8sf2,
    &operand_data[5559],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:934 */
  {
    "sqrtv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sqrtv4sf2,
    &operand_data[5563],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:977 */
  {
    "rsqrtv8sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rsqrtv8sf2,
    &operand_data[5559],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:977 */
  {
    "rsqrtv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rsqrtv4sf2,
    &operand_data[5563],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1017 */
  {
    "smaxv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smaxv8sf3,
    &operand_data[5559],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1017 */
  {
    "sminv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sminv8sf3,
    &operand_data[5559],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1017 */
  {
    "smaxv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smaxv4sf3,
    &operand_data[5563],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1017 */
  {
    "sminv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sminv4sf3,
    &operand_data[5563],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1017 */
  {
    "smaxv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smaxv4df3,
    &operand_data[5567],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1017 */
  {
    "sminv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sminv4df3,
    &operand_data[5567],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1017 */
  {
    "smaxv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smaxv2df3,
    &operand_data[5571],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1017 */
  {
    "sminv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sminv2df3,
    &operand_data[5571],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1299 */
  {
    "reduc_splus_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_splus_v4df,
    &operand_data[5555],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1312 */
  {
    "reduc_splus_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_splus_v2df,
    &operand_data[5557],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1321 */
  {
    "reduc_splus_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_splus_v8sf,
    &operand_data[5551],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1335 */
  {
    "reduc_splus_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_splus_v4sf,
    &operand_data[5553],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smax_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smax_v32qi,
    &operand_data[2368],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smin_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smin_v32qi,
    &operand_data[2368],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smax_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smax_v16hi,
    &operand_data[5587],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smin_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smin_v16hi,
    &operand_data[5587],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smax_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smax_v8si,
    &operand_data[5589],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smin_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smin_v8si,
    &operand_data[5589],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smax_v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smax_v4di,
    &operand_data[2392],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smin_v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smin_v4di,
    &operand_data[2392],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smax_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smax_v8sf,
    &operand_data[5551],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smin_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smin_v8sf,
    &operand_data[5551],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smax_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smax_v4df,
    &operand_data[5555],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smin_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smin_v4df,
    &operand_data[5555],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smax_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smax_v4sf,
    &operand_data[5553],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1358 */
  {
    "reduc_smin_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_smin_v4sf,
    &operand_data[5553],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1368 */
  {
    "reduc_umax_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_umax_v32qi,
    &operand_data[2368],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1368 */
  {
    "reduc_umin_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_umin_v32qi,
    &operand_data[2368],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1368 */
  {
    "reduc_umax_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_umax_v16hi,
    &operand_data[5587],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1368 */
  {
    "reduc_umin_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_umin_v16hi,
    &operand_data[5587],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1368 */
  {
    "reduc_umax_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_umax_v8si,
    &operand_data[5589],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1368 */
  {
    "reduc_umin_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_umin_v8si,
    &operand_data[5589],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1368 */
  {
    "reduc_umax_v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_umax_v4di,
    &operand_data[2392],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1368 */
  {
    "reduc_umin_v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_umin_v4di,
    &operand_data[2392],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1378 */
  {
    "reduc_umin_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_reduc_umin_v8hi,
    &operand_data[5591],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1514 */
  {
    "vcondv32qiv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv32qiv8sf,
    &operand_data[5593],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1514 */
  {
    "vcondv16hiv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv16hiv8sf,
    &operand_data[5599],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1514 */
  {
    "vcondv8siv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8siv8sf,
    &operand_data[5605],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1514 */
  {
    "vcondv4div8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4div8sf,
    &operand_data[5611],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1514 */
  {
    "vcondv8sfv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8sfv8sf,
    &operand_data[5617],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1514 */
  {
    "vcondv4dfv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4dfv8sf,
    &operand_data[5623],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1514 */
  {
    "vcondv32qiv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv32qiv4df,
    &operand_data[5629],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1514 */
  {
    "vcondv16hiv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv16hiv4df,
    &operand_data[5635],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1514 */
  {
    "vcondv8siv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8siv4df,
    &operand_data[5641],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1514 */
  {
    "vcondv4div4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4div4df,
    &operand_data[5647],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1514 */
  {
    "vcondv8sfv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8sfv4df,
    &operand_data[5653],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1514 */
  {
    "vcondv4dfv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4dfv4df,
    &operand_data[5659],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1531 */
  {
    "vcondv16qiv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv16qiv4sf,
    &operand_data[5665],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1531 */
  {
    "vcondv8hiv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8hiv4sf,
    &operand_data[5671],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1531 */
  {
    "vcondv4siv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4siv4sf,
    &operand_data[5677],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1531 */
  {
    "vcondv2div4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv2div4sf,
    &operand_data[5683],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1531 */
  {
    "vcondv4sfv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4sfv4sf,
    &operand_data[5689],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1531 */
  {
    "vcondv2dfv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv2dfv4sf,
    &operand_data[5695],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1531 */
  {
    "vcondv16qiv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv16qiv2df,
    &operand_data[5701],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1531 */
  {
    "vcondv8hiv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8hiv2df,
    &operand_data[5707],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1531 */
  {
    "vcondv4siv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4siv2df,
    &operand_data[5713],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1531 */
  {
    "vcondv2div2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv2div2df,
    &operand_data[5719],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1531 */
  {
    "vcondv4sfv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4sfv2df,
    &operand_data[5725],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1531 */
  {
    "vcondv2dfv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv2dfv2df,
    &operand_data[5731],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1587 */
  {
    "andv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andv8sf3,
    &operand_data[5559],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1587 */
  {
    "iorv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorv8sf3,
    &operand_data[5559],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1587 */
  {
    "xorv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorv8sf3,
    &operand_data[5559],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1587 */
  {
    "andv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andv4sf3,
    &operand_data[5563],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1587 */
  {
    "iorv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorv4sf3,
    &operand_data[5563],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1587 */
  {
    "xorv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorv4sf3,
    &operand_data[5563],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1587 */
  {
    "andv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andv4df3,
    &operand_data[5567],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1587 */
  {
    "iorv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorv4df3,
    &operand_data[5567],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1587 */
  {
    "xorv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorv4df3,
    &operand_data[5567],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1587 */
  {
    "andv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andv2df3,
    &operand_data[5571],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1587 */
  {
    "iorv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorv2df3,
    &operand_data[5571],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1587 */
  {
    "xorv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorv2df3,
    &operand_data[5571],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1627 */
  {
    "copysignv8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_copysignv8sf3,
    &operand_data[5559],
    3,
    3,
    6,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1627 */
  {
    "copysignv4sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_copysignv4sf3,
    &operand_data[5563],
    3,
    3,
    6,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1627 */
  {
    "copysignv4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_copysignv4df3,
    &operand_data[5567],
    3,
    3,
    6,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1627 */
  {
    "copysignv2df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_copysignv2df3,
    &operand_data[5571],
    3,
    3,
    6,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1727 */
  {
    "fmasf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmasf4,
    &operand_data[5737],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1727 */
  {
    "fmadf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmadf4,
    &operand_data[5741],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1727 */
  {
    "fmav4sf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmav4sf4,
    &operand_data[5745],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1727 */
  {
    "fmav2df4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmav2df4,
    &operand_data[5749],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1727 */
  {
    "fmav8sf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmav8sf4,
    &operand_data[5753],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1727 */
  {
    "fmav4df4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmav4df4,
    &operand_data[5757],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1735 */
  {
    "fmssf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmssf4,
    &operand_data[5737],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1735 */
  {
    "fmsdf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmsdf4,
    &operand_data[5741],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1735 */
  {
    "fmsv4sf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmsv4sf4,
    &operand_data[5745],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1735 */
  {
    "fmsv2df4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmsv2df4,
    &operand_data[5749],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1735 */
  {
    "fmsv8sf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmsv8sf4,
    &operand_data[5753],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1735 */
  {
    "fmsv4df4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmsv4df4,
    &operand_data[5757],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1743 */
  {
    "fnmasf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fnmasf4,
    &operand_data[5737],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1743 */
  {
    "fnmadf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fnmadf4,
    &operand_data[5741],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1743 */
  {
    "fnmav4sf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fnmav4sf4,
    &operand_data[5745],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1743 */
  {
    "fnmav2df4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fnmav2df4,
    &operand_data[5749],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1743 */
  {
    "fnmav8sf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fnmav8sf4,
    &operand_data[5753],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1743 */
  {
    "fnmav4df4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fnmav4df4,
    &operand_data[5757],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1751 */
  {
    "fnmssf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fnmssf4,
    &operand_data[5737],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1751 */
  {
    "fnmsdf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fnmsdf4,
    &operand_data[5741],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1751 */
  {
    "fnmsv4sf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fnmsv4sf4,
    &operand_data[5745],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1751 */
  {
    "fnmsv2df4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fnmsv2df4,
    &operand_data[5749],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1751 */
  {
    "fnmsv8sf4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fnmsv8sf4,
    &operand_data[5753],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1751 */
  {
    "fnmsv4df4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fnmsv4df4,
    &operand_data[5757],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1762 */
  {
    "fma4i_fmadd_sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fma4i_fmadd_sf,
    &operand_data[5737],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1762 */
  {
    "fma4i_fmadd_df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fma4i_fmadd_df,
    &operand_data[5741],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1762 */
  {
    "fma4i_fmadd_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fma4i_fmadd_v4sf,
    &operand_data[5745],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1762 */
  {
    "fma4i_fmadd_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fma4i_fmadd_v2df,
    &operand_data[5749],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1762 */
  {
    "fma4i_fmadd_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fma4i_fmadd_v8sf,
    &operand_data[5753],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1762 */
  {
    "fma4i_fmadd_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fma4i_fmadd_v4df,
    &operand_data[5757],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1853 */
  {
    "fmaddsub_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmaddsub_v8sf,
    &operand_data[5753],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1853 */
  {
    "fmaddsub_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmaddsub_v4sf,
    &operand_data[5745],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1853 */
  {
    "fmaddsub_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmaddsub_v4df,
    &operand_data[5757],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1853 */
  {
    "fmaddsub_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmaddsub_v2df,
    &operand_data[5749],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1902 */
  {
    "fmai_vmfmadd_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmai_vmfmadd_v4sf,
    &operand_data[5745],
    4,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1902 */
  {
    "fmai_vmfmadd_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fmai_vmfmadd_v2df,
    &operand_data[5749],
    4,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1984 */
  {
    "fma4i_vmfmadd_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fma4i_vmfmadd_v4sf,
    &operand_data[5745],
    4,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:1984 */
  {
    "fma4i_vmfmadd_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fma4i_vmfmadd_v2df,
    &operand_data[5749],
    4,
    4,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2244 */
  {
    "floatunsv8siv8sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floatunsv8siv8sf2,
    &operand_data[5761],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2244 */
  {
    "floatunsv4siv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_floatunsv4siv4sf2,
    &operand_data[5763],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2307 */
  {
    "fixuns_truncv8sfv8si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fixuns_truncv8sfv8si2,
    &operand_data[5765],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2307 */
  {
    "fixuns_truncv4sfv4si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_fixuns_truncv4sfv4si2,
    &operand_data[5767],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2537 */
  {
    "avx_cvtpd2dq256_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_cvtpd2dq256_2,
    &operand_data[5769],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2558 */
  {
    "sse2_cvtpd2dq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtpd2dq,
    &operand_data[5771],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2598 */
  {
    "avx_cvttpd2dq256_2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_cvttpd2dq256_2,
    &operand_data[5769],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2617 */
  {
    "sse2_cvttpd2dq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvttpd2dq,
    &operand_data[5771],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2697 */
  {
    "sse2_cvtpd2ps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_cvtpd2ps,
    &operand_data[5773],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2766 */
  {
    "vec_unpacks_hi_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_hi_v4sf,
    &operand_data[5775],
    2,
    2,
    3,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2782 */
  {
    "vec_unpacks_hi_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_hi_v8sf,
    &operand_data[5777],
    2,
    2,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2794 */
  {
    "vec_unpacks_lo_v4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_lo_v4sf,
    &operand_data[5775],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2802 */
  {
    "vec_unpacks_lo_v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_lo_v8sf,
    &operand_data[5777],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2814 */
  {
    "vec_unpacks_float_hi_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_float_hi_v16hi,
    &operand_data[5779],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2814 */
  {
    "vec_unpacks_float_hi_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_float_hi_v8hi,
    &operand_data[5781],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2827 */
  {
    "vec_unpacks_float_lo_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_float_lo_v16hi,
    &operand_data[5779],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2827 */
  {
    "vec_unpacks_float_lo_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_float_lo_v8hi,
    &operand_data[5781],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2840 */
  {
    "vec_unpacku_float_hi_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_float_hi_v16hi,
    &operand_data[5779],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2840 */
  {
    "vec_unpacku_float_hi_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_float_hi_v8hi,
    &operand_data[5781],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2853 */
  {
    "vec_unpacku_float_lo_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_float_lo_v16hi,
    &operand_data[5779],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2853 */
  {
    "vec_unpacku_float_lo_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_float_lo_v8hi,
    &operand_data[5781],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2866 */
  {
    "vec_unpacks_float_hi_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_float_hi_v4si,
    &operand_data[5783],
    2,
    2,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2880 */
  {
    "vec_unpacks_float_lo_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_float_lo_v4si,
    &operand_data[5783],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2888 */
  {
    "vec_unpacks_float_hi_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_float_hi_v8si,
    &operand_data[5785],
    2,
    2,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2900 */
  {
    "vec_unpacks_float_lo_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_float_lo_v8si,
    &operand_data[5785],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2909 */
  {
    "vec_unpacku_float_hi_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_float_hi_v4si,
    &operand_data[5783],
    2,
    2,
    11,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2945 */
  {
    "vec_unpacku_float_lo_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_float_lo_v4si,
    &operand_data[5783],
    2,
    2,
    9,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:2974 */
  {
    "vec_unpacku_float_hi_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_float_hi_v8si,
    &operand_data[5787],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3001 */
  {
    "vec_unpacku_float_lo_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_float_lo_v8si,
    &operand_data[5785],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3026 */
  {
    "vec_pack_trunc_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_trunc_v4df,
    &operand_data[5789],
    3,
    3,
    4,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3043 */
  {
    "vec_pack_trunc_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_trunc_v2df,
    &operand_data[5792],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3071 */
  {
    "vec_pack_sfix_trunc_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_sfix_trunc_v4df,
    &operand_data[5795],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3088 */
  {
    "vec_pack_sfix_trunc_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_sfix_trunc_v2df,
    &operand_data[5798],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3122 */
  {
    "vec_pack_ufix_trunc_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_ufix_trunc_v4df,
    &operand_data[5801],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3122 */
  {
    "vec_pack_ufix_trunc_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_ufix_trunc_v2df,
    &operand_data[5804],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3152 */
  {
    "vec_pack_sfix_v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_sfix_v4df,
    &operand_data[5795],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3169 */
  {
    "vec_pack_sfix_v2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_sfix_v2df,
    &operand_data[5798],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3206 */
  {
    "sse_movhlps_exp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse_movhlps_exp,
    &operand_data[5746],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3251 */
  {
    "sse_movlhps_exp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse_movlhps_exp,
    &operand_data[5746],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3313 */
  {
    "vec_interleave_highv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_highv8sf,
    &operand_data[5807],
    3,
    3,
    6,
    1,
    0
  },
  /* ../../gcc/config/i386/sse.md:3381 */
  {
    "vec_interleave_lowv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_lowv8sf,
    &operand_data[5807],
    3,
    3,
    6,
    1,
    0
  },
  /* ../../gcc/config/i386/sse.md:3500 */
  {
    "avx_shufps256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_shufps256,
    &operand_data[5810],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3555 */
  {
    "sse_shufps",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse_shufps,
    &operand_data[5814],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3620 */
  {
    "sse_loadhps_exp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse_loadhps_exp,
    &operand_data[5818],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3673 */
  {
    "sse_loadlps_exp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse_loadlps_exp,
    &operand_data[5818],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3813 */
  {
    "vec_initv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv16qi,
    &operand_data[5821],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3813 */
  {
    "vec_initv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv8hi,
    &operand_data[5823],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3813 */
  {
    "vec_initv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv4si,
    &operand_data[5825],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3813 */
  {
    "vec_initv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv2di,
    &operand_data[5827],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3813 */
  {
    "vec_initv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv4sf,
    &operand_data[5829],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3813 */
  {
    "vec_initv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv2df,
    &operand_data[5831],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3926 */
  {
    "vec_initv2df+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5833],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3926 */
  {
    "vec_setv32qi-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5835],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3941 */
  {
    "vec_setv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv32qi,
    &operand_data[5837],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3941 */
  {
    "vec_setv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv16qi,
    &operand_data[5840],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3941 */
  {
    "vec_setv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv16hi,
    &operand_data[5843],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3941 */
  {
    "vec_setv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv8hi,
    &operand_data[5846],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3941 */
  {
    "vec_setv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv8si,
    &operand_data[5849],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3941 */
  {
    "vec_setv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv4si,
    &operand_data[5852],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3941 */
  {
    "vec_setv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv4di,
    &operand_data[5855],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3941 */
  {
    "vec_setv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv2di,
    &operand_data[5858],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3941 */
  {
    "vec_setv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv8sf,
    &operand_data[5861],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3941 */
  {
    "vec_setv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv4sf,
    &operand_data[5864],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3941 */
  {
    "vec_setv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv4df,
    &operand_data[5867],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3941 */
  {
    "vec_setv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_setv2df,
    &operand_data[5870],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3952 */
  {
    "vec_setv2df+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4171],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:3971 */
  {
    "vec_setv2df+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5873],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4011 */
  {
    "avx_vextractf128v32qi-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5876],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4027 */
  {
    "avx_vextractf128v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vextractf128v32qi,
    &operand_data[5879],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4027 */
  {
    "avx_vextractf128v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vextractf128v16hi,
    &operand_data[5882],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4027 */
  {
    "avx_vextractf128v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vextractf128v8si,
    &operand_data[5885],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4027 */
  {
    "avx_vextractf128v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vextractf128v4di,
    &operand_data[5888],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4027 */
  {
    "avx_vextractf128v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vextractf128v8sf,
    &operand_data[5891],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4027 */
  {
    "avx_vextractf128v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vextractf128v4df,
    &operand_data[5894],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4051 */
  {
    "avx_vextractf128v4df+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5897],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4051 */
  {
    "avx_vextractf128v4df+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5899],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4084 */
  {
    "avx_vextractf128v4df+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5901],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4084 */
  {
    "vec_extractv32qi-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5903],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4119 */
  {
    "vec_extractv32qi-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5905],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4158 */
  {
    "vec_extractv32qi-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5907],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4214 */
  {
    "vec_extractv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv32qi,
    &operand_data[5909],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4214 */
  {
    "vec_extractv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv16qi,
    &operand_data[5912],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4214 */
  {
    "vec_extractv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv16hi,
    &operand_data[5915],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4214 */
  {
    "vec_extractv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv8hi,
    &operand_data[5918],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4214 */
  {
    "vec_extractv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv8si,
    &operand_data[5921],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4214 */
  {
    "vec_extractv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv4si,
    &operand_data[5924],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4214 */
  {
    "vec_extractv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv4di,
    &operand_data[5927],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4214 */
  {
    "vec_extractv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv2di,
    &operand_data[5930],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4214 */
  {
    "vec_extractv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv8sf,
    &operand_data[5933],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4214 */
  {
    "vec_extractv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv4sf,
    &operand_data[5936],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4214 */
  {
    "vec_extractv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv4df,
    &operand_data[5939],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4214 */
  {
    "vec_extractv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_extractv2df,
    &operand_data[5942],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4246 */
  {
    "vec_interleave_highv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_highv4df,
    &operand_data[5945],
    3,
    3,
    6,
    1,
    0
  },
  /* ../../gcc/config/i386/sse.md:4275 */
  {
    "vec_interleave_highv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_highv2df,
    &operand_data[5571],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4312 */
  {
    "avx_movddup256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_movddup256,
    &operand_data[5567],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4322 */
  {
    "avx_unpcklpd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_unpcklpd256,
    &operand_data[5575],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4348 */
  {
    "vec_interleave_lowv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_lowv4df,
    &operand_data[5945],
    3,
    3,
    6,
    1,
    0
  },
  /* ../../gcc/config/i386/sse.md:4376 */
  {
    "vec_interleave_lowv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_lowv2df,
    &operand_data[5571],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4412 */
  {
    "vec_interleave_lowv2df+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5549],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4429 */
  {
    "avx_shufpd256-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5948],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4443 */
  {
    "avx_shufpd256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_shufpd256,
    &operand_data[5952],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4485 */
  {
    "sse2_shufpd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_shufpd,
    &operand_data[5956],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4625 */
  {
    "sse2_shufpd+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5960],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4667 */
  {
    "sse2_loadhpd_exp-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[4638],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4698 */
  {
    "sse2_loadhpd_exp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_loadhpd_exp,
    &operand_data[5962],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4745 */
  {
    "sse2_loadhpd_exp+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5965],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4754 */
  {
    "sse2_loadlpd_exp",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_loadlpd_exp,
    &operand_data[5962],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4814 */
  {
    "sse2_loadlpd_exp+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5965],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4900 */
  {
    "negv32qi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negv32qi2,
    &operand_data[5967],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4900 */
  {
    "negv16qi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negv16qi2,
    &operand_data[5969],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4900 */
  {
    "negv16hi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negv16hi2,
    &operand_data[5971],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4900 */
  {
    "negv8hi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negv8hi2,
    &operand_data[5973],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4900 */
  {
    "negv8si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negv8si2,
    &operand_data[5975],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4900 */
  {
    "negv4si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negv4si2,
    &operand_data[5977],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4900 */
  {
    "negv4di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negv4di2,
    &operand_data[5979],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4900 */
  {
    "negv2di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_negv2di2,
    &operand_data[5981],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "addv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "subv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "addv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "subv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "addv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "subv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "addv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "subv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "addv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addv8si3,
    &operand_data[5995],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "subv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subv8si3,
    &operand_data[5995],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "addv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addv4si3,
    &operand_data[5998],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "subv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subv4si3,
    &operand_data[5998],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "addv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addv4di3,
    &operand_data[6001],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "subv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subv4di3,
    &operand_data[6001],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "addv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_addv2di3,
    &operand_data[6004],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4908 */
  {
    "subv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_subv2di3,
    &operand_data[6004],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "avx2_ssaddv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_ssaddv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "avx2_usaddv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_usaddv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "avx2_sssubv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_sssubv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "avx2_ussubv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_ussubv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "sse2_ssaddv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_ssaddv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "sse2_usaddv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_usaddv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "sse2_sssubv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_sssubv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "sse2_ussubv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_ussubv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "avx2_ssaddv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_ssaddv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "avx2_usaddv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_usaddv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "avx2_sssubv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_sssubv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "avx2_ussubv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_ussubv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "sse2_ssaddv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_ssaddv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "sse2_usaddv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_usaddv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "sse2_sssubv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_sssubv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4931 */
  {
    "sse2_ussubv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_ussubv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4954 */
  {
    "sse2_ussubv8hi3+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[2368],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:4954 */
  {
    "mulv16hi3-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[2371],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5021 */
  {
    "mulv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5021 */
  {
    "mulv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5042 */
  {
    "smulv16hi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smulv16hi3_highpart,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5042 */
  {
    "umulv16hi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umulv16hi3_highpart,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5042 */
  {
    "smulv8hi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smulv8hi3_highpart,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5042 */
  {
    "umulv8hi3_highpart",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umulv8hi3_highpart,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5075 */
  {
    "avx2_umulv4siv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_umulv4siv4di3,
    &operand_data[6007],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5110 */
  {
    "sse2_umulv2siv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_umulv2siv2di3,
    &operand_data[6010],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5145 */
  {
    "avx2_mulv4siv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_mulv4siv4di3,
    &operand_data[6007],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5182 */
  {
    "sse4_1_mulv2siv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_mulv2siv2di3,
    &operand_data[6010],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5218 */
  {
    "avx2_pmaddwd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pmaddwd,
    &operand_data[6013],
    3,
    3,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5268 */
  {
    "sse2_pmaddwd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_pmaddwd,
    &operand_data[6016],
    3,
    3,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5397 */
  {
    "mulv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulv8si3,
    &operand_data[6019],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5397 */
  {
    "mulv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mulv4si3,
    &operand_data[2389],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5421 */
  {
    "mulv4si3+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[2389],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5477 */
  {
    "mulv4si3+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[2392],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5477 */
  {
    "vec_widen_smult_hi_v16hi-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[2395],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5565 */
  {
    "vec_widen_smult_hi_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_smult_hi_v16hi,
    &operand_data[6021],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5565 */
  {
    "vec_widen_umult_hi_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_umult_hi_v16hi,
    &operand_data[6021],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5565 */
  {
    "vec_widen_smult_hi_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_smult_hi_v8hi,
    &operand_data[6024],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5565 */
  {
    "vec_widen_umult_hi_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_umult_hi_v8hi,
    &operand_data[6024],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5586 */
  {
    "vec_widen_smult_lo_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_smult_lo_v16hi,
    &operand_data[6021],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5586 */
  {
    "vec_widen_umult_lo_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_umult_lo_v16hi,
    &operand_data[6021],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5586 */
  {
    "vec_widen_smult_lo_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_smult_lo_v8hi,
    &operand_data[6024],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5586 */
  {
    "vec_widen_umult_lo_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_umult_lo_v8hi,
    &operand_data[6024],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5607 */
  {
    "vec_widen_smult_hi_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_smult_hi_v8si,
    &operand_data[6007],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5607 */
  {
    "vec_widen_umult_hi_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_umult_hi_v8si,
    &operand_data[6007],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5633 */
  {
    "vec_widen_smult_lo_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_smult_lo_v8si,
    &operand_data[6007],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5633 */
  {
    "vec_widen_umult_lo_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_umult_lo_v8si,
    &operand_data[6007],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5659 */
  {
    "vec_widen_smult_hi_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_smult_hi_v4si,
    &operand_data[6027],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5692 */
  {
    "vec_widen_smult_lo_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_smult_lo_v4si,
    &operand_data[6027],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5725 */
  {
    "vec_widen_umult_hi_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_umult_hi_v4si,
    &operand_data[6027],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5744 */
  {
    "vec_widen_umult_lo_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_widen_umult_lo_v4si,
    &operand_data[6027],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5763 */
  {
    "sdot_prodv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sdot_prodv16hi,
    &operand_data[6030],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5763 */
  {
    "sdot_prodv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sdot_prodv8hi,
    &operand_data[6034],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5781 */
  {
    "sdot_prodv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sdot_prodv4si,
    &operand_data[6038],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5781 */
  {
    "udot_prodv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_udot_prodv4si,
    &operand_data[6038],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5810 */
  {
    "sdot_prodv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sdot_prodv8si,
    &operand_data[6042],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5810 */
  {
    "udot_prodv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_udot_prodv8si,
    &operand_data[6042],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5877 */
  {
    "vec_shl_v16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_shl_v16qi,
    &operand_data[6046],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5877 */
  {
    "vec_shl_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_shl_v8hi,
    &operand_data[6049],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5877 */
  {
    "vec_shl_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_shl_v4si,
    &operand_data[6052],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5877 */
  {
    "vec_shl_v2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_shl_v2di,
    &operand_data[6055],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5914 */
  {
    "vec_shr_v16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_shr_v16qi,
    &operand_data[6046],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5914 */
  {
    "vec_shr_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_shr_v8hi,
    &operand_data[6049],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5914 */
  {
    "vec_shr_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_shr_v4si,
    &operand_data[6052],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5914 */
  {
    "vec_shr_v2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_shr_v2di,
    &operand_data[6055],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5953 */
  {
    "smaxv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smaxv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5953 */
  {
    "sminv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sminv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5953 */
  {
    "umaxv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umaxv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5953 */
  {
    "uminv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_uminv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5953 */
  {
    "smaxv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smaxv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5953 */
  {
    "sminv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sminv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5953 */
  {
    "umaxv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umaxv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5953 */
  {
    "uminv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_uminv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5953 */
  {
    "smaxv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smaxv8si3,
    &operand_data[5995],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5953 */
  {
    "sminv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sminv8si3,
    &operand_data[5995],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5953 */
  {
    "umaxv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umaxv8si3,
    &operand_data[5995],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5953 */
  {
    "uminv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_uminv8si3,
    &operand_data[5995],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5973 */
  {
    "smaxv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smaxv4di3,
    &operand_data[2392],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5973 */
  {
    "sminv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sminv4di3,
    &operand_data[2392],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5973 */
  {
    "umaxv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umaxv4di3,
    &operand_data[2392],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5973 */
  {
    "uminv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_uminv4di3,
    &operand_data[2392],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5973 */
  {
    "smaxv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smaxv2di3,
    &operand_data[2395],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5973 */
  {
    "sminv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sminv2di3,
    &operand_data[2395],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5973 */
  {
    "umaxv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umaxv2di3,
    &operand_data[2395],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:5973 */
  {
    "uminv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_uminv2di3,
    &operand_data[2395],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6008 */
  {
    "smaxv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smaxv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6008 */
  {
    "sminv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sminv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6008 */
  {
    "smaxv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smaxv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6008 */
  {
    "sminv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sminv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6008 */
  {
    "smaxv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_smaxv4si3,
    &operand_data[5998],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6008 */
  {
    "sminv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sminv4si3,
    &operand_data[5998],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6078 */
  {
    "umaxv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umaxv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6078 */
  {
    "uminv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_uminv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6078 */
  {
    "umaxv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umaxv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6078 */
  {
    "uminv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_uminv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6078 */
  {
    "umaxv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_umaxv4si3,
    &operand_data[5998],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6078 */
  {
    "uminv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_uminv4si3,
    &operand_data[5998],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6165 */
  {
    "avx2_eqv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_eqv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6165 */
  {
    "avx2_eqv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_eqv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6165 */
  {
    "avx2_eqv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_eqv8si3,
    &operand_data[5995],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6165 */
  {
    "avx2_eqv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_eqv4di3,
    &operand_data[6001],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6216 */
  {
    "sse2_eqv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_eqv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6216 */
  {
    "sse2_eqv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_eqv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6216 */
  {
    "sse2_eqv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_eqv4si3,
    &operand_data[5998],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6224 */
  {
    "sse4_1_eqv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_eqv2di3,
    &operand_data[6004],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv32qiv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv32qiv32qi,
    &operand_data[6058],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv16hiv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv16hiv32qi,
    &operand_data[6064],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv8siv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8siv32qi,
    &operand_data[6070],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv4div32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4div32qi,
    &operand_data[6076],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv8sfv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8sfv32qi,
    &operand_data[6082],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv4dfv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4dfv32qi,
    &operand_data[6088],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv32qiv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv32qiv16hi,
    &operand_data[6094],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv16hiv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv16hiv16hi,
    &operand_data[6100],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv8siv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8siv16hi,
    &operand_data[6106],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv4div16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4div16hi,
    &operand_data[6112],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv8sfv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8sfv16hi,
    &operand_data[6118],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv4dfv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4dfv16hi,
    &operand_data[6124],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv32qiv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv32qiv8si,
    &operand_data[6130],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv16hiv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv16hiv8si,
    &operand_data[6136],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv8siv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8siv8si,
    &operand_data[6142],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv4div8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4div8si,
    &operand_data[6148],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv8sfv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8sfv8si,
    &operand_data[6154],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv4dfv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4dfv8si,
    &operand_data[6160],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv32qiv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv32qiv4di,
    &operand_data[6166],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv16hiv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv16hiv4di,
    &operand_data[6172],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv8siv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8siv4di,
    &operand_data[6178],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv4div4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4div4di,
    &operand_data[6184],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv8sfv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8sfv4di,
    &operand_data[6190],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6274 */
  {
    "vcondv4dfv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4dfv4di,
    &operand_data[6196],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv16qiv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv16qiv16qi,
    &operand_data[6202],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv8hiv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8hiv16qi,
    &operand_data[6208],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv4siv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4siv16qi,
    &operand_data[6214],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv2div16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv2div16qi,
    &operand_data[6220],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv4sfv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4sfv16qi,
    &operand_data[6226],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv2dfv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv2dfv16qi,
    &operand_data[6232],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv16qiv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv16qiv8hi,
    &operand_data[6238],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv8hiv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8hiv8hi,
    &operand_data[6244],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv4siv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4siv8hi,
    &operand_data[6250],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv2div8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv2div8hi,
    &operand_data[6256],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv4sfv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4sfv8hi,
    &operand_data[6262],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv2dfv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv2dfv8hi,
    &operand_data[6268],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv16qiv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv16qiv4si,
    &operand_data[6274],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv8hiv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv8hiv4si,
    &operand_data[6280],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv4siv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4siv4si,
    &operand_data[6286],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv2div4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv2div4si,
    &operand_data[6292],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv4sfv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv4sfv4si,
    &operand_data[6298],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6291 */
  {
    "vcondv2dfv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv2dfv4si,
    &operand_data[6304],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6308 */
  {
    "vcondv2div2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv2div2di,
    &operand_data[6310],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6308 */
  {
    "vcondv2dfv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcondv2dfv2di,
    &operand_data[6316],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv32qiv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv32qiv32qi,
    &operand_data[6322],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv16hiv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv16hiv32qi,
    &operand_data[6328],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv8siv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv8siv32qi,
    &operand_data[6334],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv4div32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4div32qi,
    &operand_data[6340],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv8sfv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv8sfv32qi,
    &operand_data[6346],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv4dfv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4dfv32qi,
    &operand_data[6352],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv32qiv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv32qiv16hi,
    &operand_data[6358],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv16hiv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv16hiv16hi,
    &operand_data[6364],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv8siv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv8siv16hi,
    &operand_data[6370],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv4div16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4div16hi,
    &operand_data[6376],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv8sfv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv8sfv16hi,
    &operand_data[6382],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv4dfv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4dfv16hi,
    &operand_data[6388],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv32qiv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv32qiv8si,
    &operand_data[6394],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv16hiv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv16hiv8si,
    &operand_data[6400],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv8siv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv8siv8si,
    &operand_data[6406],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv4div8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4div8si,
    &operand_data[6412],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv8sfv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv8sfv8si,
    &operand_data[6418],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv4dfv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4dfv8si,
    &operand_data[6424],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv32qiv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv32qiv4di,
    &operand_data[6430],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv16hiv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv16hiv4di,
    &operand_data[6436],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv8siv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv8siv4di,
    &operand_data[6442],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv4div4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4div4di,
    &operand_data[6448],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv8sfv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv8sfv4di,
    &operand_data[6454],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6323 */
  {
    "vconduv4dfv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4dfv4di,
    &operand_data[6460],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv16qiv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv16qiv16qi,
    &operand_data[6466],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv8hiv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv8hiv16qi,
    &operand_data[6472],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv4siv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4siv16qi,
    &operand_data[6478],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv2div16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv2div16qi,
    &operand_data[6484],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv4sfv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4sfv16qi,
    &operand_data[6490],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv2dfv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv2dfv16qi,
    &operand_data[6496],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv16qiv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv16qiv8hi,
    &operand_data[6502],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv8hiv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv8hiv8hi,
    &operand_data[6508],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv4siv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4siv8hi,
    &operand_data[6514],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv2div8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv2div8hi,
    &operand_data[6520],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv4sfv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4sfv8hi,
    &operand_data[6526],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv2dfv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv2dfv8hi,
    &operand_data[6532],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv16qiv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv16qiv4si,
    &operand_data[6538],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv8hiv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv8hiv4si,
    &operand_data[6544],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv4siv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4siv4si,
    &operand_data[6550],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv2div4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv2div4si,
    &operand_data[6556],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv4sfv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv4sfv4si,
    &operand_data[6562],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6340 */
  {
    "vconduv2dfv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv2dfv4si,
    &operand_data[6568],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6357 */
  {
    "vconduv2div2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv2div2di,
    &operand_data[6574],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6357 */
  {
    "vconduv2dfv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vconduv2dfv2di,
    &operand_data[6580],
    6,
    6,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6378 */
  {
    "vec_permv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_permv16qi,
    &operand_data[6586],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6378 */
  {
    "vec_permv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_permv8hi,
    &operand_data[6590],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6378 */
  {
    "vec_permv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_permv4si,
    &operand_data[6594],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6378 */
  {
    "vec_permv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_permv2di,
    &operand_data[6598],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6378 */
  {
    "vec_permv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_permv4sf,
    &operand_data[6602],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6378 */
  {
    "vec_permv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_permv2df,
    &operand_data[6606],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6378 */
  {
    "vec_permv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_permv32qi,
    &operand_data[6610],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6378 */
  {
    "vec_permv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_permv16hi,
    &operand_data[6614],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6378 */
  {
    "vec_permv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_permv8si,
    &operand_data[6618],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6378 */
  {
    "vec_permv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_permv4di,
    &operand_data[6622],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6378 */
  {
    "vec_permv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_permv8sf,
    &operand_data[6626],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6378 */
  {
    "vec_permv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_permv4df,
    &operand_data[6630],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6397 */
  {
    "vec_perm_constv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_perm_constv4sf,
    &operand_data[6634],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6397 */
  {
    "vec_perm_constv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_perm_constv4si,
    &operand_data[6638],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6397 */
  {
    "vec_perm_constv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_perm_constv2df,
    &operand_data[6642],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6397 */
  {
    "vec_perm_constv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_perm_constv2di,
    &operand_data[6646],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6397 */
  {
    "vec_perm_constv16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_perm_constv16qi,
    &operand_data[6650],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6397 */
  {
    "vec_perm_constv8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_perm_constv8hi,
    &operand_data[6654],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6397 */
  {
    "vec_perm_constv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_perm_constv8sf,
    &operand_data[6658],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6397 */
  {
    "vec_perm_constv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_perm_constv4df,
    &operand_data[6662],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6397 */
  {
    "vec_perm_constv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_perm_constv8si,
    &operand_data[6666],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6397 */
  {
    "vec_perm_constv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_perm_constv4di,
    &operand_data[6670],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6397 */
  {
    "vec_perm_constv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_perm_constv32qi,
    &operand_data[6674],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6397 */
  {
    "vec_perm_constv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_perm_constv16hi,
    &operand_data[6678],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6416 */
  {
    "one_cmplv32qi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_one_cmplv32qi2,
    &operand_data[5967],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6416 */
  {
    "one_cmplv16qi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_one_cmplv16qi2,
    &operand_data[5969],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6416 */
  {
    "one_cmplv16hi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_one_cmplv16hi2,
    &operand_data[5971],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6416 */
  {
    "one_cmplv8hi2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_one_cmplv8hi2,
    &operand_data[5973],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6416 */
  {
    "one_cmplv8si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_one_cmplv8si2,
    &operand_data[5975],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6416 */
  {
    "one_cmplv4si2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_one_cmplv4si2,
    &operand_data[5977],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6416 */
  {
    "one_cmplv4di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_one_cmplv4di2,
    &operand_data[5979],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6416 */
  {
    "one_cmplv2di2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_one_cmplv2di2,
    &operand_data[5981],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6431 */
  {
    "avx2_andnotv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_andnotv32qi3,
    &operand_data[6682],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6431 */
  {
    "sse2_andnotv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_andnotv16qi3,
    &operand_data[6685],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6431 */
  {
    "avx2_andnotv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_andnotv16hi3,
    &operand_data[6688],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6431 */
  {
    "sse2_andnotv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_andnotv8hi3,
    &operand_data[6691],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6431 */
  {
    "avx2_andnotv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_andnotv8si3,
    &operand_data[6694],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6431 */
  {
    "sse2_andnotv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_andnotv4si3,
    &operand_data[6697],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6431 */
  {
    "avx2_andnotv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_andnotv4di3,
    &operand_data[6700],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6431 */
  {
    "sse2_andnotv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_andnotv2di3,
    &operand_data[6703],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "andv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "iorv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "xorv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "andv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "iorv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "xorv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "andv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "iorv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "xorv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "andv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "iorv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "xorv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "andv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andv8si3,
    &operand_data[5995],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "iorv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorv8si3,
    &operand_data[5995],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "xorv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorv8si3,
    &operand_data[5995],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "andv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andv4si3,
    &operand_data[5998],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "iorv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorv4si3,
    &operand_data[5998],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "xorv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorv4si3,
    &operand_data[5998],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "andv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andv4di3,
    &operand_data[6001],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "iorv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorv4di3,
    &operand_data[6001],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "xorv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorv4di3,
    &operand_data[6001],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "andv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andv2di3,
    &operand_data[6004],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "iorv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iorv2di3,
    &operand_data[6004],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6504 */
  {
    "xorv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xorv2di3,
    &operand_data[6004],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6594 */
  {
    "andtf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_andtf3,
    &operand_data[6706],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6594 */
  {
    "iortf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_iortf3,
    &operand_data[6706],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6594 */
  {
    "xortf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xortf3,
    &operand_data[6706],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6624 */
  {
    "vec_pack_trunc_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_trunc_v16hi,
    &operand_data[6613],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6624 */
  {
    "vec_pack_trunc_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_trunc_v8hi,
    &operand_data[6589],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6624 */
  {
    "vec_pack_trunc_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_trunc_v8si,
    &operand_data[5588],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6624 */
  {
    "vec_pack_trunc_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_trunc_v4si,
    &operand_data[6593],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6624 */
  {
    "vec_pack_trunc_v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_trunc_v4di,
    &operand_data[6621],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6624 */
  {
    "vec_pack_trunc_v2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_pack_trunc_v2di,
    &operand_data[6597],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:6939 */
  {
    "vec_interleave_highv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_highv32qi,
    &operand_data[2434],
    3,
    3,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/sse.md:6939 */
  {
    "vec_interleave_highv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_highv16hi,
    &operand_data[2437],
    3,
    3,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/sse.md:6939 */
  {
    "vec_interleave_highv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_highv8si,
    &operand_data[2440],
    3,
    3,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/sse.md:6939 */
  {
    "vec_interleave_highv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_highv4di,
    &operand_data[2285],
    3,
    3,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/sse.md:6955 */
  {
    "vec_interleave_lowv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_lowv32qi,
    &operand_data[2434],
    3,
    3,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/sse.md:6955 */
  {
    "vec_interleave_lowv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_lowv16hi,
    &operand_data[2437],
    3,
    3,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/sse.md:6955 */
  {
    "vec_interleave_lowv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_lowv8si,
    &operand_data[2440],
    3,
    3,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/sse.md:6955 */
  {
    "vec_interleave_lowv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_interleave_lowv4di,
    &operand_data[2285],
    3,
    3,
    0,
    1,
    0
  },
  /* ../../gcc/config/i386/sse.md:7133 */
  {
    "avx2_pshufdv3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pshufdv3,
    &operand_data[6709],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7184 */
  {
    "sse2_pshufd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_pshufd,
    &operand_data[6712],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7224 */
  {
    "avx2_pshuflwv3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pshuflwv3,
    &operand_data[6715],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7283 */
  {
    "sse2_pshuflw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_pshuflw,
    &operand_data[6718],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7328 */
  {
    "avx2_pshufhwv3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pshufhwv3,
    &operand_data[6715],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7387 */
  {
    "sse2_pshufhw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_pshufhw,
    &operand_data[6718],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7432 */
  {
    "sse2_loadd",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_loadd,
    &operand_data[6721],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7461 */
  {
    "sse2_loadd+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[6722],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7475 */
  {
    "sse_storeq-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[6724],
    0,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7491 */
  {
    "sse_storeq",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse_storeq,
    &operand_data[6727],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7519 */
  {
    "sse_storeq+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[6727],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7728 */
  {
    "vec_unpacks_lo_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_lo_v32qi,
    &operand_data[6729],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7728 */
  {
    "vec_unpacks_lo_v16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_lo_v16qi,
    &operand_data[6731],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7728 */
  {
    "vec_unpacks_lo_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_lo_v16hi,
    &operand_data[6021],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7728 */
  {
    "vec_unpacks_lo_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_lo_v8hi,
    &operand_data[6024],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7728 */
  {
    "vec_unpacks_lo_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_lo_v8si,
    &operand_data[6042],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7728 */
  {
    "vec_unpacks_lo_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_lo_v4si,
    &operand_data[6027],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7734 */
  {
    "vec_unpacks_hi_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_hi_v32qi,
    &operand_data[6729],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7734 */
  {
    "vec_unpacks_hi_v16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_hi_v16qi,
    &operand_data[6731],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7734 */
  {
    "vec_unpacks_hi_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_hi_v16hi,
    &operand_data[6021],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7734 */
  {
    "vec_unpacks_hi_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_hi_v8hi,
    &operand_data[6024],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7734 */
  {
    "vec_unpacks_hi_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_hi_v8si,
    &operand_data[6042],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7734 */
  {
    "vec_unpacks_hi_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacks_hi_v4si,
    &operand_data[6027],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7740 */
  {
    "vec_unpacku_lo_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_lo_v32qi,
    &operand_data[6729],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7740 */
  {
    "vec_unpacku_lo_v16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_lo_v16qi,
    &operand_data[6731],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7740 */
  {
    "vec_unpacku_lo_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_lo_v16hi,
    &operand_data[6021],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7740 */
  {
    "vec_unpacku_lo_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_lo_v8hi,
    &operand_data[6024],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7740 */
  {
    "vec_unpacku_lo_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_lo_v8si,
    &operand_data[6042],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7740 */
  {
    "vec_unpacku_lo_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_lo_v4si,
    &operand_data[6027],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7746 */
  {
    "vec_unpacku_hi_v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_hi_v32qi,
    &operand_data[6729],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7746 */
  {
    "vec_unpacku_hi_v16qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_hi_v16qi,
    &operand_data[6731],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7746 */
  {
    "vec_unpacku_hi_v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_hi_v16hi,
    &operand_data[6021],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7746 */
  {
    "vec_unpacku_hi_v8hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_hi_v8hi,
    &operand_data[6024],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7746 */
  {
    "vec_unpacku_hi_v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_hi_v8si,
    &operand_data[6042],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7746 */
  {
    "vec_unpacku_hi_v4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_unpacku_hi_v4si,
    &operand_data[6027],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7758 */
  {
    "avx2_uavgv32qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_uavgv32qi3,
    &operand_data[5983],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7788 */
  {
    "sse2_uavgv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_uavgv16qi3,
    &operand_data[5986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7872 */
  {
    "avx2_uavgv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_uavgv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:7894 */
  {
    "sse2_uavgv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_uavgv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:8012 */
  {
    "sse2_maskmovdqu",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_maskmovdqu,
    &operand_data[6733],
    3,
    3,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:9101 */
  {
    "avx2_umulhrswv16hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_umulhrswv16hi3,
    &operand_data[5989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:9153 */
  {
    "ssse3_pmulhrswv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ssse3_pmulhrswv8hi3,
    &operand_data[5992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:9201 */
  {
    "ssse3_pmulhrswv4hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ssse3_pmulhrswv4hi3,
    &operand_data[5438],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:9606 */
  {
    "avx2_pblendw",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_pblendw,
    &operand_data[6736],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:9897 */
  {
    "avx_roundps_sfix256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_roundps_sfix256,
    &operand_data[6740],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:9897 */
  {
    "sse4_1_roundps_sfix",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_roundps_sfix,
    &operand_data[6743],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:9913 */
  {
    "avx_roundpd_vec_pack_sfix256",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_roundpd_vec_pack_sfix256,
    &operand_data[6746],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:9913 */
  {
    "sse4_1_roundpd_vec_pack_sfix",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse4_1_roundpd_vec_pack_sfix,
    &operand_data[6750],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:9972 */
  {
    "roundv8sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_roundv8sf2,
    &operand_data[5551],
    2,
    2,
    4,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:9972 */
  {
    "roundv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_roundv4sf2,
    &operand_data[5553],
    2,
    2,
    4,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:9972 */
  {
    "roundv4df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_roundv4df2,
    &operand_data[5555],
    2,
    2,
    4,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:9972 */
  {
    "roundv2df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_roundv2df2,
    &operand_data[5557],
    2,
    2,
    4,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:10006 */
  {
    "roundv8sf2_sfix",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_roundv8sf2_sfix,
    &operand_data[5765],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:10006 */
  {
    "roundv4sf2_sfix",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_roundv4sf2_sfix,
    &operand_data[5767],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:10020 */
  {
    "roundv4df2_vec_pack_sfix",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_roundv4df2_vec_pack_sfix,
    &operand_data[5801],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:10020 */
  {
    "roundv2df2_vec_pack_sfix",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_roundv2df2_vec_pack_sfix,
    &operand_data[5804],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:10060 */
  {
    "roundv2df2_vec_pack_sfix+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[6754],
    0,
    7,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:10199 */
  {
    "rotlv16qi3-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[6761],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11088 */
  {
    "rotlv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotlv16qi3,
    &operand_data[6766],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11088 */
  {
    "rotlv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotlv8hi3,
    &operand_data[6769],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11088 */
  {
    "rotlv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotlv4si3,
    &operand_data[6772],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11088 */
  {
    "rotlv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotlv2di3,
    &operand_data[6775],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11119 */
  {
    "rotrv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotrv16qi3,
    &operand_data[6766],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11119 */
  {
    "rotrv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotrv8hi3,
    &operand_data[6769],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11119 */
  {
    "rotrv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotrv4si3,
    &operand_data[6772],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11119 */
  {
    "rotrv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_rotrv2di3,
    &operand_data[6775],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11178 */
  {
    "vrotrv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vrotrv16qi3,
    &operand_data[2371],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11178 */
  {
    "vrotrv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vrotrv8hi3,
    &operand_data[6590],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11178 */
  {
    "vrotrv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vrotrv4si3,
    &operand_data[2389],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11178 */
  {
    "vrotrv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vrotrv2di3,
    &operand_data[2395],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11190 */
  {
    "vrotlv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vrotlv16qi3,
    &operand_data[2371],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11190 */
  {
    "vrotlv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vrotlv8hi3,
    &operand_data[6590],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11190 */
  {
    "vrotlv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vrotlv4si3,
    &operand_data[2389],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11190 */
  {
    "vrotlv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vrotlv2di3,
    &operand_data[2395],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11220 */
  {
    "vlshrv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vlshrv16qi3,
    &operand_data[6685],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11220 */
  {
    "vlshrv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vlshrv8hi3,
    &operand_data[6691],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11233 */
  {
    "vlshrv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vlshrv4si3,
    &operand_data[6697],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11233 */
  {
    "vlshrv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vlshrv2di3,
    &operand_data[6703],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11249 */
  {
    "vlshrv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vlshrv8si3,
    &operand_data[6694],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11249 */
  {
    "vlshrv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vlshrv4di3,
    &operand_data[6700],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11256 */
  {
    "vashrv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vashrv16qi3,
    &operand_data[6685],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11256 */
  {
    "vashrv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vashrv8hi3,
    &operand_data[6691],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11256 */
  {
    "vashrv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vashrv2di3,
    &operand_data[6703],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11269 */
  {
    "vashrv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vashrv4si3,
    &operand_data[6697],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11284 */
  {
    "vashrv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vashrv8si3,
    &operand_data[6694],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11290 */
  {
    "vashlv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vashlv16qi3,
    &operand_data[6685],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11290 */
  {
    "vashlv8hi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vashlv8hi3,
    &operand_data[6691],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11301 */
  {
    "vashlv4si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vashlv4si3,
    &operand_data[6697],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11301 */
  {
    "vashlv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vashlv2di3,
    &operand_data[6703],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11316 */
  {
    "vashlv8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vashlv8si3,
    &operand_data[6694],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11316 */
  {
    "vashlv4di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vashlv4di3,
    &operand_data[6700],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11362 */
  {
    "ashlv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ashlv16qi3,
    &operand_data[6778],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11382 */
  {
    "lshrv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_lshrv16qi3,
    &operand_data[6778],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11382 */
  {
    "ashrv16qi3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ashrv16qi3,
    &operand_data[6778],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11418 */
  {
    "ashrv2di3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_ashrv2di3,
    &operand_data[6781],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11459 */
  {
    "xop_vmfrczv4sf2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xop_vmfrczv4sf2,
    &operand_data[5563],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11459 */
  {
    "xop_vmfrczv2df2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_xop_vmfrczv2df2,
    &operand_data[5571],
    2,
    2,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11667 */
  {
    "avx_vzeroall",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vzeroall,
    &operand_data[0],
    0,
    0,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11767 */
  {
    "avx2_permv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_permv4di,
    &operand_data[6784],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11858 */
  {
    "avx2_permv4di+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5849],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11858 */
  {
    "avx2_permv4di+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5861],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11858 */
  {
    "avx2_permv4di+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5855],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11858 */
  {
    "avx_vpermilv4df-3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5867],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11916 */
  {
    "avx_vpermilv4df-2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[6787],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11916 */
  {
    "avx_vpermilv4df-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[6791],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11952 */
  {
    "avx_vpermilv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vpermilv4df,
    &operand_data[6795],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11952 */
  {
    "avx_vpermilv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vpermilv2df,
    &operand_data[6798],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11974 */
  {
    "avx_vpermilv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vpermilv8sf,
    &operand_data[6801],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:11974 */
  {
    "avx_vpermilv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vpermilv4sf,
    &operand_data[6804],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12032 */
  {
    "avx_vperm2f128v8si3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vperm2f128v8si3,
    &operand_data[6807],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12032 */
  {
    "avx_vperm2f128v8sf3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vperm2f128v8sf3,
    &operand_data[6811],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12032 */
  {
    "avx_vperm2f128v4df3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vperm2f128v4df3,
    &operand_data[6815],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12109 */
  {
    "avx_vinsertf128v32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vinsertf128v32qi,
    &operand_data[6819],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12109 */
  {
    "avx_vinsertf128v16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vinsertf128v16hi,
    &operand_data[6823],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12109 */
  {
    "avx_vinsertf128v8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vinsertf128v8si,
    &operand_data[6827],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12109 */
  {
    "avx_vinsertf128v4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vinsertf128v4di,
    &operand_data[6831],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12109 */
  {
    "avx_vinsertf128v8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vinsertf128v8sf,
    &operand_data[6835],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12109 */
  {
    "avx_vinsertf128v4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx_vinsertf128v4df,
    &operand_data[6839],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12333 */
  {
    "avx_vinsertf128v4df+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5500],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12333 */
  {
    "avx_vinsertf128v4df+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5512],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12333 */
  {
    "vec_initv32qi-1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[5516],
    0,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12353 */
  {
    "vec_initv32qi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv32qi,
    &operand_data[6843],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12353 */
  {
    "vec_initv16hi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv16hi,
    &operand_data[6845],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12353 */
  {
    "vec_initv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv8si,
    &operand_data[6847],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12353 */
  {
    "vec_initv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv4di,
    &operand_data[6849],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12353 */
  {
    "vec_initv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv8sf,
    &operand_data[6851],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12353 */
  {
    "vec_initv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vec_initv4df,
    &operand_data[6853],
    2,
    2,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12362 */
  {
    "avx2_extracti128",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_extracti128,
    &operand_data[5888],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12386 */
  {
    "avx2_inserti128",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_inserti128,
    &operand_data[6831],
    4,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12497 */
  {
    "vcvtps2ph",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_vcvtps2ph,
    &operand_data[6855],
    3,
    3,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12561 */
  {
    "avx2_gathersiv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gathersiv2di,
    &operand_data[6858],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12561 */
  {
    "avx2_gathersiv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gathersiv2df,
    &operand_data[6865],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12561 */
  {
    "avx2_gathersiv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gathersiv4di,
    &operand_data[6872],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12561 */
  {
    "avx2_gathersiv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gathersiv4df,
    &operand_data[6879],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12561 */
  {
    "avx2_gathersiv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gathersiv4si,
    &operand_data[6886],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12561 */
  {
    "avx2_gathersiv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gathersiv4sf,
    &operand_data[6893],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12561 */
  {
    "avx2_gathersiv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gathersiv8si,
    &operand_data[6900],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12561 */
  {
    "avx2_gathersiv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gathersiv8sf,
    &operand_data[6907],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12622 */
  {
    "avx2_gatherdiv2di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gatherdiv2di,
    &operand_data[6914],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12622 */
  {
    "avx2_gatherdiv2df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gatherdiv2df,
    &operand_data[6921],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12622 */
  {
    "avx2_gatherdiv4di",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gatherdiv4di,
    &operand_data[6928],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12622 */
  {
    "avx2_gatherdiv4df",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gatherdiv4df,
    &operand_data[6935],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12622 */
  {
    "avx2_gatherdiv4si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gatherdiv4si,
    &operand_data[6942],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12622 */
  {
    "avx2_gatherdiv4sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gatherdiv4sf,
    &operand_data[6949],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12622 */
  {
    "avx2_gatherdiv8si",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gatherdiv8si,
    &operand_data[6956],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sse.md:12622 */
  {
    "avx2_gatherdiv8sf",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_avx2_gatherdiv8sf,
    &operand_data[6963],
    6,
    7,
    1,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:36 */
  {
    "sse2_lfence",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_lfence,
    &operand_data[0],
    0,
    0,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:55 */
  {
    "sse_sfence",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse_sfence,
    &operand_data[0],
    0,
    0,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:74 */
  {
    "sse2_mfence",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_sse2_mfence,
    &operand_data[0],
    0,
    0,
    2,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:101 */
  {
    "mem_thread_fence",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_mem_thread_fence,
    &operand_data[83],
    1,
    1,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:141 */
  {
    "atomic_loadqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atomic_loadqi,
    &operand_data[6970],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:141 */
  {
    "atomic_loadhi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atomic_loadhi,
    &operand_data[6973],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:141 */
  {
    "atomic_loadsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atomic_loadsi,
    &operand_data[6976],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:141 */
  {
    "atomic_loaddi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atomic_loaddi,
    &operand_data[6979],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:158 */
  {
    "atomic_loaddi+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[6982],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:197 */
  {
    "atomic_storeqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atomic_storeqi,
    &operand_data[6986],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:197 */
  {
    "atomic_storehi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atomic_storehi,
    &operand_data[6989],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:197 */
  {
    "atomic_storesi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atomic_storesi,
    &operand_data[6992],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:197 */
  {
    "atomic_storedi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atomic_storedi,
    &operand_data[6995],
    3,
    3,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:244 */
  {
    "atomic_storedi+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[6998],
    0,
    4,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:309 */
  {
    "atomic_compare_and_swapqi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atomic_compare_and_swapqi,
    &operand_data[7002],
    8,
    8,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:309 */
  {
    "atomic_compare_and_swaphi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atomic_compare_and_swaphi,
    &operand_data[7010],
    8,
    8,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:309 */
  {
    "atomic_compare_and_swapsi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atomic_compare_and_swapsi,
    &operand_data[7018],
    8,
    8,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:333 */
  {
    "atomic_compare_and_swapdi",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atomic_compare_and_swapdi,
    &operand_data[7026],
    8,
    8,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:333 */
  {
    "atomic_compare_and_swapti",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    (insn_gen_fn) gen_atomic_compare_and_swapti,
    &operand_data[7034],
    8,
    8,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:458 */
  {
    "atomic_compare_and_swapti+1",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[7042],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:458 */
  {
    "atomic_compare_and_swapti+2",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[7047],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:458 */
  {
    "atomic_compare_and_swapti+3",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[7052],
    0,
    5,
    0,
    0,
    0
  },
  /* ../../gcc/config/i386/sync.md:458 */
  {
    "atomic_compare_and_swapti+4",
#if HAVE_DESIGNATED_UNION_INITIALIZERS
    { 0 },
#else
    { 0, 0, 0 },
#endif
    0,
    &operand_data[7057],
    0,
    5,
    0,
    0,
    0
  },
};


const char *
get_insn_name (int code)
{
  if (code == NOOP_MOVE_INSN_CODE)
    return "NOOP_MOVE";
  else
    return insn_data[code].name;
}
